Package: bit64
Title: A S3 Class for Vectors of 64bit Integers
Version: 4.8.4
Authors@R: c(
    person("Michael", "Chirico", email="michaelchirico4@gmail.com", role=c("aut", "cre")),
    person("Jens", "Oehlschlägel", role="aut"),
    person("Leonardo", "Silvestri", role="ctb"),
    person("Ofek", "Shilon", role="ctb"),
    person("Christian", "Ullerich", role="ctb")
  )
Depends: R (>= 3.5.0)
Description: 
  Package 'bit64' provides serializable S3 atomic 64bit (signed) integers.
  These are useful for handling database keys and exact counting in +-2^63.
  WARNING: do not use them as replacement for 32bit integers, integer64 are not
  supported for subscripting by R-core and they have different semantics when
  combined with double, e.g. integer64 + double => integer64.
  Class integer64 can be used in vectors, matrices, arrays and data.frames.
  Methods are available for coercion from and to logicals, integers, doubles,
  characters and factors as well as many elementwise and summary functions.
  Many fast algorithmic operations such as 'match' and 'order' support inter-
  active data exploration and manipulation and optionally leverage caching.
License: GPL-2 | GPL-3
LazyLoad: yes
ByteCompile: yes
URL: https://github.com/r-lib/bit64, https://bit64.r-lib.org
Encoding: UTF-8
Imports: bit (>= 4.0.0), graphics, methods, stats, utils
Suggests: patrick (>= 0.3.0), testthat (>= 3.3.0), withr
Config/testthat/edition: 3
Config/Needs/development: patrick, testthat
Config/Needs/website: tidyverse/tidytemplate
Config/roxygen2/version: 8.0.0
NeedsCompilation: yes
Packaged: 2026-08-19 16:52:49 UTC; chiricom
Author: Michael Chirico [aut, cre],
  Jens Oehlschlägel [aut],
  Leonardo Silvestri [ctb],
  Ofek Shilon [ctb],
  Christian Ullerich [ctb]
Maintainer: Michael Chirico <michaelchirico4@gmail.com>
Repository: CRAN
Date/Publication: 2026-08-20 05:23:42 UTC
Built: R 4.5.2; x86_64-apple-darwin20; 2026-08-20 08:33:47 UTC; unix
Archs: bit64.so.dSYM

Package: bit64
Title: A S3 Class for Vectors of 64bit Integers
Version: 4.6.0-1
Authors@R: c(
    person("Michael", "Chirico", email = "michaelchirico4@gmail.com", role = c("aut", "cre")),
    person("Jens", "Oehlschlägel", role = "aut"),
    person("Leonardo", "Silvestri", role = "ctb"),
    person("Ofek", "Shilon", role = "ctb")
  )
Depends: R (>= 3.4.0), bit (>= 4.0.0)
Description: 
  Package 'bit64' provides serializable S3 atomic 64bit (signed) integers.
  These are useful for handling database keys and exact counting in +-2^63.
  WARNING: do not use them as replacement for 32bit integers, integer64 are not
  supported for subscripting by R-core and they have different semantics when
  combined with double, e.g. integer64 + double => integer64.
  Class integer64 can be used in vectors, matrices, arrays and data.frames.
  Methods are available for coercion from and to logicals, integers, doubles,
  characters and factors as well as many elementwise and summary functions.
  Many fast algorithmic operations such as 'match' and 'order' support inter-
  active data exploration and manipulation and optionally leverage caching.
License: GPL-2 | GPL-3
LazyLoad: yes
ByteCompile: yes
URL: https://github.com/r-lib/bit64
Encoding: UTF-8
Imports: graphics, methods, stats, utils
Suggests: testthat (>= 3.0.3), withr
Config/testthat/edition: 3
Config/needs/development: testthat
RoxygenNote: 7.3.2
NeedsCompilation: yes
Packaged: 2025-01-16 14:04:20 UTC; michael
Author: Michael Chirico [aut, cre],
  Jens Oehlschlägel [aut],
  Leonardo Silvestri [ctb],
  Ofek Shilon [ctb]
Maintainer: Michael Chirico <michaelchirico4@gmail.com>
Repository: CRAN
Date/Publication: 2025-01-16 16:00:07 UTC
Built: R 4.4.0; amd64-portbld-midnightbsd4; 2026-07-18 01:04:52 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method("!",integer64)
S3method("!=",integer64)
S3method("%%",integer64)
S3method("%/%",integer64)
S3method("%in%",default)
S3method("%in%",integer64)
S3method("&",integer64)
S3method("*",integer64)
S3method("+",integer64)
S3method("-",integer64)
S3method("/",integer64)
S3method(":",default)
S3method(":",integer64)
S3method("<",integer64)
S3method("<=",integer64)
S3method("==",integer64)
S3method(">",integer64)
S3method(">=",integer64)
S3method("[",integer64)
S3method("[<-",integer64)
S3method("[[",integer64)
S3method("[[<-",integer64)
S3method("^",integer64)
S3method("length<-",integer64)
S3method("|",integer64)
S3method(abs,integer64)
S3method(all,integer64)
S3method(all.equal,integer64)
S3method(any,integer64)
S3method(aperm,integer64)
S3method(as.bitstring,integer64)
S3method(as.character,integer64)
S3method(as.data.frame,integer64)
S3method(as.double,integer64)
S3method(as.integer,integer64)
S3method(as.integer64,"NULL")
S3method(as.integer64,bitstring)
S3method(as.integer64,character)
S3method(as.integer64,double)
S3method(as.integer64,factor)
S3method(as.integer64,integer)
S3method(as.integer64,integer64)
S3method(as.integer64,logical)
S3method(as.list,integer64)
S3method(as.logical,integer64)
S3method(c,integer64)
S3method(cbind,integer64)
S3method(ceiling,integer64)
S3method(colSums,default)
S3method(colSums,integer64)
S3method(cummax,integer64)
S3method(cummin,integer64)
S3method(cumprod,integer64)
S3method(cumsum,integer64)
S3method(diff,integer64)
S3method(duplicated,integer64)
S3method(floor,integer64)
S3method(format,integer64)
S3method(hashdup,cache_integer64)
S3method(hashfin,cache_integer64)
S3method(hashfun,integer64)
S3method(hashmap,integer64)
S3method(hashmaptab,integer64)
S3method(hashmapuni,integer64)
S3method(hashmapupo,integer64)
S3method(hashpos,cache_integer64)
S3method(hashrev,cache_integer64)
S3method(hashrin,cache_integer64)
S3method(hashtab,cache_integer64)
S3method(hashuni,cache_integer64)
S3method(hashupo,cache_integer64)
S3method(identical,integer64)
S3method(is.double,default)
S3method(is.double,integer64)
S3method(is.finite,integer64)
S3method(is.infinite,integer64)
S3method(is.na,integer64)
S3method(is.nan,integer64)
S3method(is.sorted,integer64)
S3method(is.vector,integer64)
S3method(keypos,integer64)
S3method(log,integer64)
S3method(log10,integer64)
S3method(log2,integer64)
S3method(match,default)
S3method(match,integer64)
S3method(max,integer64)
S3method(mean,integer64)
S3method(median,integer64)
S3method(mergeorder,integer64)
S3method(mergesort,integer64)
S3method(mergesortorder,integer64)
S3method(min,integer64)
S3method(na.count,integer64)
S3method(nties,integer64)
S3method(nunique,integer64)
S3method(nvalid,integer64)
S3method(order,default)
S3method(order,integer64)
S3method(orderdup,integer64)
S3method(orderfin,integer64)
S3method(orderkey,integer64)
S3method(ordernut,integer64)
S3method(orderpos,integer64)
S3method(orderqtl,integer64)
S3method(orderrnk,integer64)
S3method(ordertab,integer64)
S3method(ordertie,integer64)
S3method(orderuni,integer64)
S3method(orderupo,integer64)
S3method(prank,integer64)
S3method(print,bitstring)
S3method(print,cache)
S3method(print,integer64)
S3method(prod,integer64)
S3method(qtile,integer64)
S3method(quantile,integer64)
S3method(quickorder,integer64)
S3method(quicksort,integer64)
S3method(quicksortorder,integer64)
S3method(radixorder,integer64)
S3method(radixsort,integer64)
S3method(radixsortorder,integer64)
S3method(ramorder,integer64)
S3method(ramsort,integer64)
S3method(ramsortorder,integer64)
S3method(range,integer64)
S3method(rank,default)
S3method(rank,integer64)
S3method(rbind,integer64)
S3method(rep,integer64)
S3method(round,integer64)
S3method(rowSums,default)
S3method(rowSums,integer64)
S3method(scale,integer64)
S3method(seq,integer64)
S3method(shellorder,integer64)
S3method(shellsort,integer64)
S3method(shellsortorder,integer64)
S3method(sign,integer64)
S3method(signif,integer64)
S3method(sort,integer64)
S3method(sortfin,integer64)
S3method(sortnut,integer64)
S3method(sortorderdup,integer64)
S3method(sortorderkey,integer64)
S3method(sortorderpos,integer64)
S3method(sortorderrnk,integer64)
S3method(sortordertab,integer64)
S3method(sortordertie,integer64)
S3method(sortorderuni,integer64)
S3method(sortorderupo,integer64)
S3method(sortqtl,integer64)
S3method(sorttab,integer64)
S3method(sortuni,integer64)
S3method(sqrt,integer64)
S3method(str,integer64)
S3method(sum,integer64)
S3method(summary,integer64)
S3method(tiepos,integer64)
S3method(trunc,integer64)
S3method(unipos,integer64)
S3method(unique,integer64)
S3method(xor,integer64)
export("!.integer64")
export("!=.integer64")
export("%%.integer64")
export("%/%.integer64")
export("%in%")
export("%in%.default")
export("%in%.integer64")
export("&.integer64")
export("*.integer64")
export("+.integer64")
export("-.integer64")
export("/.integer64")
export(":")
export(":.default")
export(":.integer64")
export("<.integer64")
export("<=.integer64")
export("==.integer64")
export(">.integer64")
export(">=.integer64")
export("[.integer64")
export("[<-.integer64")
export("[[.integer64")
export("[[<-.integer64")
export("^.integer64")
export("length<-.integer64")
export("|.integer64")
export(NA_integer64_)
export(abs.integer64)
export(all.equal.integer64)
export(all.integer64)
export(any.integer64)
export(as.bitstring)
export(as.bitstring.integer64)
export(as.character.integer64)
export(as.data.frame.integer64)
export(as.double.integer64)
export(as.integer.integer64)
export(as.integer64)
export(as.integer64.NULL)
export(as.integer64.bitstring)
export(as.integer64.character)
export(as.integer64.double)
export(as.integer64.factor)
export(as.integer64.integer)
export(as.integer64.integer64)
export(as.integer64.logical)
export(as.list.integer64)
export(as.logical.integer64)
export(benchmark64)
export(binattr)
export(c.integer64)
export(cache)
export(cbind.integer64)
export(ceiling.integer64)
export(colSums)
export(cummax.integer64)
export(cummin.integer64)
export(cumprod.integer64)
export(cumsum.integer64)
export(diff.integer64)
export(duplicated.integer64)
export(floor.integer64)
export(format.integer64)
export(getcache)
export(hashcache)
export(hashdup)
export(hashdup.cache_integer64)
export(hashfin)
export(hashfin.cache_integer64)
export(hashfun)
export(hashfun.integer64)
export(hashmap)
export(hashmap.integer64)
export(hashmaptab)
export(hashmaptab.integer64)
export(hashmapuni)
export(hashmapuni.integer64)
export(hashmapupo)
export(hashmapupo.integer64)
export(hashpos)
export(hashpos.cache_integer64)
export(hashrev)
export(hashrev.cache_integer64)
export(hashrin)
export(hashrin.cache_integer64)
export(hashtab)
export(hashtab.cache_integer64)
export(hashuni)
export(hashuni.cache_integer64)
export(hashupo)
export(hashupo.cache_integer64)
export(identical.integer64)
export(integer64)
export(is.double)
export(is.double.default)
export(is.double.integer64)
export(is.finite.integer64)
export(is.infinite.integer64)
export(is.integer64)
export(is.na.integer64)
export(is.nan.integer64)
export(is.sorted.integer64)
export(is.vector.integer64)
export(jamcache)
export(keypos)
export(keypos.integer64)
export(lim.integer64)
export(log.integer64)
export(log10.integer64)
export(log2.integer64)
export(match)
export(match.default)
export(match.integer64)
export(max.integer64)
export(mean.integer64)
export(median.integer64)
export(mergeorder.integer64)
export(mergesort.integer64)
export(mergesortorder.integer64)
export(min.integer64)
export(minusclass)
export(na.count.integer64)
export(newcache)
export(nties.integer64)
export(nunique.integer64)
export(nvalid.integer64)
export(optimizer64)
export(order)
export(order.default)
export(order.integer64)
export(ordercache)
export(orderdup)
export(orderdup.integer64)
export(orderfin)
export(orderfin.integer64)
export(orderkey)
export(orderkey.integer64)
export(ordernut)
export(ordernut.integer64)
export(orderpos)
export(orderpos.integer64)
export(orderqtl)
export(orderqtl.integer64)
export(orderrnk)
export(orderrnk.integer64)
export(ordertab)
export(ordertab.integer64)
export(ordertie)
export(ordertie.integer64)
export(orderuni)
export(orderuni.integer64)
export(orderupo)
export(orderupo.integer64)
export(plusclass)
export(prank)
export(prank.integer64)
export(print.bitstring)
export(print.cache)
export(print.integer64)
export(prod.integer64)
export(qtile)
export(qtile.integer64)
export(quantile.integer64)
export(quickorder.integer64)
export(quicksort.integer64)
export(quicksortorder.integer64)
export(radixorder.integer64)
export(radixsort.integer64)
export(radixsortorder.integer64)
export(ramorder.integer64)
export(ramsort.integer64)
export(ramsortorder.integer64)
export(range.integer64)
export(rank)
export(rank.default)
export(rank.integer64)
export(rbind.integer64)
export(remcache)
export(rep.integer64)
export(round.integer64)
export(rowSums)
export(runif64)
export(scale.integer64)
export(seq.integer64)
export(setcache)
export(shellorder.integer64)
export(shellsort.integer64)
export(shellsortorder.integer64)
export(sign.integer64)
export(signif.integer64)
export(sort.integer64)
export(sortcache)
export(sortfin)
export(sortfin.integer64)
export(sortnut)
export(sortnut.integer64)
export(sortordercache)
export(sortorderdup)
export(sortorderdup.integer64)
export(sortorderkey)
export(sortorderkey.integer64)
export(sortorderpos)
export(sortorderpos.integer64)
export(sortorderrnk)
export(sortorderrnk.integer64)
export(sortordertab)
export(sortordertab.integer64)
export(sortordertie)
export(sortordertie.integer64)
export(sortorderuni)
export(sortorderuni.integer64)
export(sortorderupo)
export(sortorderupo.integer64)
export(sortqtl)
export(sortqtl.integer64)
export(sorttab)
export(sorttab.integer64)
export(sortuni)
export(sortuni.integer64)
export(sqrt.integer64)
export(str.integer64)
export(sum.integer64)
export(summary.integer64)
export(table.integer64)
export(tiepos)
export(tiepos.integer64)
export(trunc.integer64)
export(unipos)
export(unipos.integer64)
export(unique.integer64)
export(xor.integer64)
if (getRversion() >= "4.2.0") S3method(mtfrm,integer64)
importFrom(bit,clone)
importFrom(bit,is.sorted)
importFrom(bit,keyorder)
importFrom(bit,keysort)
importFrom(bit,keysortorder)
importFrom(bit,mergeorder)
importFrom(bit,mergesort)
importFrom(bit,mergesortorder)
importFrom(bit,na.count)
importFrom(bit,nties)
importFrom(bit,nunique)
importFrom(bit,nvalid)
importFrom(bit,quickorder)
importFrom(bit,quicksort)
importFrom(bit,quicksortorder)
importFrom(bit,radixorder)
importFrom(bit,radixsort)
importFrom(bit,radixsortorder)
importFrom(bit,ramorder)
importFrom(bit,ramsort)
importFrom(bit,ramsortorder)
importFrom(bit,repeat.time)
importFrom(bit,setattr)
importFrom(bit,shellorder)
importFrom(bit,shellsort)
importFrom(bit,shellsortorder)
importFrom(bit,xor)
importFrom(graphics,barplot)
importFrom(graphics,par)
importFrom(graphics,title)
importFrom(methods,as)
importFrom(methods,is)
importFrom(stats,cor)
importFrom(stats,median)
importFrom(stats,quantile)
importFrom(utils,packageDescription)
importFrom(utils,strOptions)
importFrom(utils,tail)
useDynLib(bit64, .registration = TRUE, .fixes = "C_")

# /*
# R-Code
# S3 atomic 64bit integers for R
# (c) 2026 Michael Chirico
# Licence: GPL2
# Provided 'as is', use at your own risk
# Created: 2026-01-14
#*/

#' @title Set Operations
#' @description Performs set union, intersection, (asymmetric!) difference, equality and membership on two vectors. As soon as an integer64 vector is involved, the operations are performed using integer64 semantics. Otherwise the \code{base} package functions are called.
#' @inheritParams base::union
#' @return 
#' For union, a vector of a common mode or class.
#' 
#' For intersect, a vector of a common mode or class, or NULL if x or y is NULL.
#' 
#' For setdiff, a vector of the same mode or class as x.
#' 
#' A logical scalar for setequal and a logical of the same length as x for is.element.
#' @seealso [base::union]
#' @examples
#' x <- as.integer64(1:5)
#' y <- c(1L, 3L, 5L, 7L)
#' union(x, y)
#' intersect(x, y)
#' setdiff(x, y)
#' setequal(x, y)
#' is.element(x, y)
#' 
#' @export
#' @rdname sets
union = function(x, y) {
  if (!(is.integer64(x) || is.integer64(y)))
    return(base::union(x, y))
  
  target_class = target_class(list(x, y))
  # try using the benefit of integer64 caching, if possible. I.e. call unique() before as().
  x = unique(x)
  class_x = class(x)[1L]
  if (class_x != target_class) {
    # TODO(#44): remove this special coercion for factor and ordered
    if (target_class == "integer64" && class_x %in% c("factor", "ordered") && isFALSE(getOption("bit64.promoteInteger64ToCharacter", FALSE)))
      x = as.character(x)
    x = as(x, target_class)
  }
  y = unique(y)
  class_y = class(y)[1L]
  if (class_y != target_class) {
    # TODO(#44): remove this special coercion for factor and ordered
    if (target_class == "integer64" && class_y %in% c("factor", "ordered") && isFALSE(getOption("bit64.promoteInteger64ToCharacter", FALSE)))
      y = as.character(y)
    y = as(y, target_class)
  }

  unique(c(x, y))
}

#' @export
#' @rdname sets
intersect = function(x, y) {
  if (is.null(x) || is.null(y)) return(NULL)
  if (!(is.integer64(x) || is.integer64(y)))
    return(base::intersect(x, y))
  
  target_class = target_class(list(x, y))
  x = unique(x)
  class_x = class(x)[1L]
  if (class_x != target_class) {
    # TODO(#44): remove this special coercion for factor and ordered
    if (target_class == "integer64" && class_x %in% c("factor", "ordered") && isFALSE(getOption("bit64.promoteInteger64ToCharacter", FALSE)))
      x = as.character(x)
    x = as(x, target_class)
  }
  y = unique(y)
  class_y = class(y)[1L]
  if (class_y != target_class) {
    # TODO(#44): remove this special coercion for factor and ordered
    if (target_class == "integer64" && class_y %in% c("factor", "ordered") && isFALSE(getOption("bit64.promoteInteger64ToCharacter", FALSE)))
      y = as.character(y)
    y = as(y, target_class)
  }

  x[match(x, y, 0L) > 0L]
}

#' @export
#' @rdname sets
setequal = function(x, y) {
  if (!(is.integer64(x) || is.integer64(y)))
    return(base::setequal(x, y))
  
  x = unique(x)
  y = unique(y)
  length_x = length(x)
  length_y = length(y)
  if (length_x != length_y) return(FALSE)
  if (length_x + length_y == 0L) return(TRUE)
  
  target_class = target_class(list(x, y))
  if (class(x)[1L] != target_class)
    x = as(x, target_class)
  if (class(y)[1L] != target_class)
    y = as(y, target_class)
  
  !anyNA(match(x, y))
}

#' @export
#' @rdname sets
setdiff = function(x, y) {
  if (!(is.integer64(x) || is.integer64(y)))
    return(base::setdiff(x, y))
  
  class_x = class(x)[1L]
  if (class_x %in% c("POSIXct", "Date"))
    x = unclass(x)
  if (class_x %in% c("factor", "ordered"))
    x = as.character(x)
  target_class = target_class(list(x, y))
  x = unique(x)
  if (class(x)[1L] != target_class)
    x_match = as(x, target_class)
  else
    x_match = x
  y = unique(y)
  class_y = class(y)[1L]
  if (class_y != target_class) {
    # TODO(#44): remove this special coercion for factor and ordered
    if (target_class == "integer64" && class_y %in% c("factor", "ordered") && isFALSE(getOption("bit64.promoteInteger64ToCharacter", FALSE)))
      y = as.character(y)
    y = as(y, target_class)
  }

  x[match(x_match, y, 0L) == 0L]
}

#' @export
#' @rdname sets
is.element = function(el, set) {
  if (!(is.integer64(el) || is.integer64(set)))
    return(base::is.element(el, set))
  
  target_class = target_class(list(el, set))
  class_el = class(el)[1L]
  if (class_el != target_class) {
    # TODO(#44): remove this special coercion for factor and ordered
    if (target_class == "integer64" && class_el %in% c("factor", "ordered") && isFALSE(getOption("bit64.promoteInteger64ToCharacter", FALSE)))
      el = as.character(el)
    el = as(el, target_class)
  }
  set = unique(set)
  class_set = class(set)[1L]
  if (class_set != target_class) {
    # TODO(#44): remove this special coercion for factor and ordered
    if (target_class == "integer64" && class_set %in% c("factor", "ordered") && isFALSE(getOption("bit64.promoteInteger64ToCharacter", FALSE)))
      set = as.character(set)
    set = as(set, target_class)
  }
  
  match(el, set, 0L) > 0L
}

# TODO(#45): use matrix() directly
matrix64 = function(x, nrow=1L, ncol=1L, byrow=FALSE) {
  x = as.integer64(x)
  if (byrow) {
    dim(x) = c(ncol, nrow)
    t(x)
  } else {
    dim(x) = c(nrow, ncol)
    x
  }
}

array64 = function(x, dim) {
  x = as.integer64(x)
  dim(x) = dim
  x
}

# Test that 'expr' gives the same result whether
#   the input is integer or integer64, in the sense
#   of equivalence after casting between the types.
# expr gets integer names converted to integer64,
#   retaining attributes (esp. for arrays), and
#   we test that the result of evaluating expr
#   is equivalent (after converting back to integer)
# Starting with integer and casting to integer64
#   guarantees representation, where casting integer64
#   to integer might have to stipulate inputs must be
#   representable as integer.
expect_int_32_64_equivalent <- function(expr) {
  # Capture the unevaluated expression
  esub = substitute(expr)
  evar = all.vars(esub)

  # replace all integer values in expr with integer64 equivalents
  #   in a tailored environment
  parent_ = parent.frame()
  int64_env = new.env(parent = parent_)
  for (key in evar) {
    val = get(key, parent_)
    if (!is.integer(val)) next
    assign(key, as.integer64(val), envir=int64_env)
    attributes(int64_env[[key]]) = attributes(val)
  }

  int_result = eval(expr, parent_)
  int64_result = eval(expr, int64_env)
  int64_result_as_int = as(int64_result, typeof(int_result))
  # ignore class (which includes integer64)
  a64 = attributes(int64_result)
  for (a in setdiff(names(a64), "class"))
    attr(int64_result_as_int, a) = a64[[a]]

  expect_identical(int64_result_as_int, int_result)
}

skip_if_not_r_version = function(ver) {
  skip_if(getRversion() < ver, paste("R version >=", ver, "required."))
}

with_parameters_test_that(
  "union works with basic R types (except double)",
  {
    y = 5:10
    if (!is.na(type_x))
      x = eval(parse(text=paste0("as.", type_x, "(x)")))
    int64_types = c(NA, "logical", "raw", "integer", "double", "POSIXct", "Date")
    # TODO(#44): remove the condition
    if (!promoteInteger64ToCharacter) int64_types = c(int64_types, c("character", "factor", "ordered"))
    if (type_y == "integer64" && type_x %in% int64_types) {
      expected_result_x_y = as.integer64(base::union(x, y))
      expected_result_y_x = as.integer64(base::union(y, x))
    } else {
      expected_result_x_y = base::union(x, y)
      expected_result_y_x = base::union(y, x)
    }
    y = as(y, type_y)

    local({
      # TODO(#44): remove the option
      withr::local_options(list(bit64.promoteInteger64ToCharacter=promoteInteger64ToCharacter))
      expect_identical(union(x, y), expected_result_x_y)
      expect_identical(union(y, x), expected_result_y_x)
    })
  },
  .cases=expand.grid(
    x=I(list(NULL, c(1:7, 2L, 11L))),
    type_x=c(
      NA, "double", "logical", "integer", "character", "complex", "factor", "ordered",
      if (getRversion() > "3.6.0") c("POSIXct", "Date")
    ),
    type_y=c("integer", "integer64"),
    promoteInteger64ToCharacter=c(FALSE, TRUE),
    stringsAsFactors=FALSE
  )
)

test_that("union works with special logic for double", {

  expect_identical(
    union(1.5:7, 5:10),
    base::union(1.5:7, 5:10)
  )
  expect_identical(
    union(1.5:7, as.integer64(5:10)),
    as.integer64(1:10)
  )
  expect_identical(
    union(as.integer64(5:10), 1.5:7),
    as.integer64(c(5:10, 1:4))
  )

})

test_that("union works (additional cases)", {

  expect_identical(
    union(c(5, 3, 5), c(0, 2, 1, 3, 6)),
    base::union(c(5, 3, 5), c(0, 2, 1, 3, 6))
  )
  expect_identical(
    union(c(5L, 3L, 5L), as.integer64(c(0, 2, 1, 3, 6))),
    as.integer64(c(5, 3, 0, 2, 1, 6))
  )

  expect_identical(union(NA, NA_integer_), base::union(NA, NA_integer_))
  expect_identical(union(NA_integer_, NA), base::union(NA_integer_, NA))
  expect_identical(union(NA, NA_integer64_), NA_integer64_)
  expect_identical(union(NA_integer64_, NA), NA_integer64_)

})


with_parameters_test_that(
  "intersect works with basic R types (except double)",
  {
    y = 5:10
    if (!is.na(type_x))
      x = eval(parse(text=paste0("as.", type_x, "(x)")))
    int64_types = c(NA, "logical", "raw", "integer", "double", "POSIXct", "Date")
    # TODO(#44): remove the condition
    if (!promoteInteger64ToCharacter) int64_types = c(int64_types, c("character", "factor", "ordered"))
    if (type_y == "integer64" && type_x %in% int64_types && !(is.na(type_x) && is.null(x))) {
      expected_result_x_y = as.integer64(base::intersect(x, y))
      expected_result_y_x = as.integer64(base::intersect(y, x))
    } else {
      expected_result_x_y = base::intersect(x, y)
      if (getRversion() <= "3.6.0" && type_y == "integer64") {
        if (type_x %in% c("character", "factor", "ordered")) expected_result_x_y = as.character(expected_result_x_y)
        else if (type_x %in% "complex") expected_result_x_y = as.complex(expected_result_x_y)
        else if (is.na(type_x) && is.null(x)) expected_result_x_y = NULL
      }
      if (getRversion() <= "3.6.0" && type_y == "integer" && is.na(type_x) && is.null(x)) {
        expected_result_x_y = NULL
      }
      expected_result_y_x = base::intersect(y, x)
    }
    y = as(y, type_y)

    local({
      # TODO(#44): remove the option
      withr::local_options(list(bit64.promoteInteger64ToCharacter=promoteInteger64ToCharacter))
      expect_identical(intersect(x, y), expected_result_x_y)
      expect_identical(intersect(y, x), expected_result_y_x)
    })
  },
  .cases=expand.grid(
    x=I(list(NULL, c(1:7, 2L, 11L))),
    type_x=c(
      NA, "double", "logical", "integer", "character", "complex", "factor", "ordered",
      if (getRversion() > "3.6.0") c("POSIXct", "Date")
    ),
    type_y=c("integer", "integer64"),
    promoteInteger64ToCharacter=c(FALSE, TRUE),
    stringsAsFactors=FALSE
  )
)

test_that("intersect works with special logic for double", {

  expect_identical(intersect(1.5:7, 5:10), base::intersect(1.5:7, 5:10))
  expect_identical(intersect(1.5:7, as.integer64(5:10)), as.integer64(5:6))
  expect_identical(intersect(as.integer64(5:10), 1.5:7), as.integer64(5:6))

})

test_that("intersect works (additional cases)", {

  expect_identical(
    intersect(c(5, 3, 5), c(0, 2, 1, 3, 6)),
    base::intersect(c(5, 3, 5), c(0, 2, 1, 3, 6))
  )
  expect_identical(
    intersect(as.integer64(c(5, 3, 5)), as.integer64(c(0, 2, 1, 3, 6))),
    as.integer64(3L)
  )

  expect_identical(intersect(NA, NA_integer_), base::intersect(NA, NA_integer_))
  expect_identical(intersect(NA_integer_, NA), base::intersect(NA_integer_, NA))
  expect_identical(intersect(NA, NA_integer64_), NA_integer64_)
  expect_identical(intersect(NA_integer64_, NA), NA_integer64_)

})


with_parameters_test_that(
  "setdiff works with basic R types (except double)",
  {
    y = 5:10
    if (!is.na(type_x))
      x = eval(parse(text=paste0("as.", type_x, "(x)")))
    if (type_y == "integer64") {
      expected_result_x_y = base::setdiff(x, y)
      expected_result_y_x = as.integer64(base::setdiff(y, x))
    } else {
      expected_result_x_y = base::setdiff(x, y)
      expected_result_y_x = base::setdiff(y, x)
    }
    y = as(y, type_y)

    local({
      # TODO(#44): remove the option
      withr::local_options(list(bit64.promoteInteger64ToCharacter=promoteInteger64ToCharacter))
      expect_identical(setdiff(x, y), expected_result_x_y)
      expect_identical(setdiff(y, x), expected_result_y_x)
    })
  },
  .cases=expand.grid(
    x=I(list(NULL, c(1:7, 2L, 11L))),
    type_x=c(
      NA, "double", "logical", "integer", "character", "complex", "factor", "ordered",
      if (getRversion() > "3.6.0") c("POSIXct", "Date")
    ),
    type_y=c("integer", "integer64"),
    promoteInteger64ToCharacter=c(FALSE, TRUE),
    stringsAsFactors=FALSE
  )
)

test_that("setdiff works with special logic for double", {

  expect_identical(setdiff(1.5:7, 5:10), base::setdiff(1.5:7, 5:10))
  expect_identical(setdiff(1.5:7, as.integer64(5:10)), 1.5:4.5)
  expect_identical(setdiff(as.integer64(5:10), 1.5:7), as.integer64(7:10))

})

test_that("setdiff works (additional cases)", {

  expect_identical(
    setdiff(c(5, 3, 5), c(0, 2, 1, 3, 6)),
    base::setdiff(c(5, 3, 5), c(0, 2, 1, 3, 6))
  )
  expect_identical(
    setdiff(as.integer64(c(5, 3, 5)), as.integer64(c(0, 2, 1, 3, 6))),
    as.integer64(5L)
  )

  expect_identical(setdiff(NA, NA_integer_), base::setdiff(NA, NA_integer_))
  expect_identical(setdiff(NA_integer_, NA), base::setdiff(NA_integer_, NA))
  expect_identical(setdiff(NA, NA_integer64_), logical())
  expect_identical(setdiff(NA_integer64_, NA), integer64())

})

with_parameters_test_that(
  "setequal works with basic R types (except double)",
  {
    y = 5:10
    if (!is.na(type_x))
      x = eval(parse(text=paste0("as.", type_x, "(x)")))
    expected_result_x_y = base::setequal(x, y)
    expected_result_y_x = base::setequal(y, x)
    y = as(y, type_y)

    local({
      # TODO(#44): remove the option
      withr::local_options(list(bit64.promoteInteger64ToCharacter=promoteInteger64ToCharacter))
      expect_identical(setequal(x, y), expected_result_x_y)
      expect_identical(setequal(y, x), expected_result_y_x)
    })
  },
  .cases=expand.grid(
    x=I(list(NULL, c(1:7, 2L, 11L))),
    type_x=c(
      NA, "double", "logical", "integer", "character", "complex", "factor", "ordered",
      if (getRversion() > "3.6.0") c("POSIXct", "Date")
    ),
    type_y=c("integer", "integer64"),
    promoteInteger64ToCharacter=c(FALSE, TRUE),
    stringsAsFactors=FALSE
  )
)

test_that("setequal works with special logic for double", {

  expect_identical(setequal(1.5:7, 5:10), base::setequal(1.5:7, 5:10))
  expect_true(setequal(1.5:7, as.integer64(6:1)))
  expect_true(setequal(as.integer64(6:1), 1.5:7))

})

test_that("setequal works (additional cases)", {

  expect_identical(
    setequal(c(5, 3, 5), c(0, 2, 1, 3, 6)),
    base::setequal(c(5, 3, 5), c(0, 2, 1, 3, 6))
  )
  expect_false(setequal(
    as.integer64(c(5, 3, 5)),
    as.integer64(c(0, 2, 1, 3, 6))
  ))
  expect_true(setequal(
    as.integer64(c(5, 3, 5)),
    as.integer64(c(3, 3, 3, 3, 5))
  ))

  expect_true(setequal(NA, NA_integer_))
  expect_true(setequal(NA_integer_, NA))
  expect_true(setequal(NA, NA_integer64_))
  expect_true(setequal(NA_integer64_, NA))

})

with_parameters_test_that(
  "is.element works with basic R types (except double)",
  {
    y = 5:10
    if (!is.na(type_x))
      x = eval(parse(text=paste0("as.", type_x, "(x)")))
    expected_result_x_y = base::is.element(x, y)
    expected_result_y_x = base::is.element(y, x)
    y = as(y, type_y)

    local({
      # TODO(#44): remove the option
      withr::local_options(list(bit64.promoteInteger64ToCharacter=promoteInteger64ToCharacter))
      expect_identical(is.element(x, y), expected_result_x_y)
      expect_identical(is.element(y, x), expected_result_y_x)
    })
  },
  .cases=expand.grid(
    x=I(list(NULL, c(1:7, 2L, 11L))),
    type_x=c(
      NA, "double", "logical", "integer", "character", "complex", "factor", "ordered",
      if (getRversion() > "3.6.0") c("POSIXct", "Date")
    ),
    type_y=c("integer", "integer64"),
    promoteInteger64ToCharacter=c(FALSE, TRUE),
    stringsAsFactors=FALSE
  )
)

test_that("is.element works with special logic for double", {

  expect_identical(is.element(1.5:7, 5:10), base::is.element(1.5:7, 5:10))
  expect_identical(is.element(1.5:7, as.integer64(6:1)), rep(TRUE, 6))
  expect_identical(is.element(as.integer64(6:1), 1.5:7), rep(TRUE, 6))

})

test_that("is.element works (additional cases)", {

  expect_identical(
    is.element(c(5, 3, 5), c(0, 2, 1, 3, 6)),
    base::is.element(c(5, 3, 5), c(0, 2, 1, 3, 6))
  )
  expect_identical(
    is.element(as.integer64(c(5, 3, 5)), as.integer64(c(0, 2, 1, 3, 6))),
    c(FALSE, TRUE, FALSE)
  )
  expect_identical(
    is.element(as.integer64(c(5, 3, 5)), as.integer64(c(3, 3, 3, 3, 5))),
    rep(TRUE, 3)
  )

  expect_true(is.element(NA, NA_integer_))
  expect_true(is.element(NA_integer_, NA))
  expect_true(is.element(NA, NA_integer64_))
  expect_true(is.element(NA_integer64_, NA))

})

# /*
# S3 atomic 64bit integers for R
# (c) 2011-2024 Jens Oehlschägel
# (c) 2025 Michael Chirico
# Licence: GPL2
# Provided 'as is', use at your own risk
# Created: 2011-12-11
# */

# backports
# R < 3.6.0
if (getRversion() < "3.6.0") {
  errorCondition = function(message, ..., class = NULL, call = NULL) {
    obj <- list(message = as.character(message), call = call, ...)
    class(obj) = c(class, "error", "condition")
    obj
  }

  warningCondition = function(message, ..., class = NULL, call = NULL) {
    obj <- list(message = as.character(message), call = call, ...)
    class(obj) = c(class, "warning", "condition")
    obj
  }
}

# nocov start
.onUnload = function(libpath) {
  library.dynam.unload("bit64", libpath)
}
# nocov end

# The call stack is searched for a given sequence of function names. If the last element of function_names is found, 
# we try to match as many elements in function_names with the function names in the call stack. The sequence must be 
# adhered to. The complete call of the function name of the last match is returned. If no match exists, the top call
# is returned. It is also possible to change the function name of the matched return value by providing its new name
# with name_to_display.
# Examples: 
# * call stack: [A, B, C, D, E]; function_names = c("C", "D") returns C
# * call stack: [A, B, C, D, E]; function_names = c("E", "D") returns D
# * call stack: [A, B, C, D, E]; function_names = c("E", "X") returns A
choose_sys_call = function(function_names, name_to_display=NULL, callStack=NULL) {
  calls = if (is.null(callStack)) sys.calls() else callStack
  if (length(calls) == 1L || length(function_names) == 0L) return(calls[[1L]])
  # find last occurrence of last name in function_names
  function_names_rev = rev(as.character(function_names))
  for (sel in rev(seq_along(calls))) {
    el = calls[[sel]]
    if (!is.function(el[[1L]]) && rev(as.character(el[[1L]]))[1L] == function_names_rev[1L]) break
  }
  # now check further backwards to match as far as possible
  for (fn in function_names_rev[-1L]) {
    if (sel == 1L) break
    el = calls[[sel - 1L]]
    if (is.function(el[[1L]]) || rev(as.character(el[[1L]]))[[1L]] != fn) break
    sel = sel - 1L
  }
  ret = calls[[sel]]
  if (!is.null(name_to_display))
    ret[[1L]] = as.name(name_to_display)
  ret
}

withCallingHandlers_and_choose_call = function(expr, function_names, name_to_display=NULL, callStack=NULL) {
  wch = substitute(
    withCallingHandlers(expr, error=error, warning=warning),
    list(
      expr = sys.call()[[2L]],
      error = function(e) stop(errorCondition(e$message, call=choose_sys_call(function_names, name_to_display, callStack=callStack))),
      warning = function(w) {
        warning(warningCondition(w$message, call=choose_sys_call(function_names, name_to_display, callStack=callStack)))
        invokeRestart("muffleWarning")
      }
    )
  )
  eval(wch, envir=parent.frame())
}

# function to determine target class and sample value for union, intersect, setdiff, setequal, min, max, range, sum, prod, c, cbind and rbind functions
getClassesOfElements = function(x, recursive) {
  classes = vapply(x, function(el) if (inherits(el, c("list", "data.frame"))) "list" else class(el)[1L], character(1L))
  if (recursive) {
    union(classes[classes != "list"], unlist(lapply(x[classes == "list"], function(el) getClassesOfElements(el, recursive=TRUE))))
  } else {
    unique(classes)
  }
}

target_class = function(x, recursive=FALSE, POSIXltAsCharacter=FALSE) {

  classes = getClassesOfElements(x, recursive=isTRUE(recursive))
  
  if ("POSIXlt" %in% classes && isTRUE(POSIXltAsCharacter)) return("character")
  if ("complex" %in% classes) return("complex")
  if (any(c("character", "factor", "ordered") %in% classes)) {
    # TODO(#44): next Release: change default behavior; subsequent Release: change from message to warning; subsequent Release: change from warning to error; subsequent Release: remove option
    if (isTRUE(getOption("bit64.promoteInteger64ToCharacter", FALSE))) return("character")
  }
  "integer64"
}

# quote() requires >0 arguments and substitute(...) does not WAI, so mix substitute() and quote(...)
missing_or_dots = function(x) identical(x, substitute()) || identical(x, quote(...))

#' Working with integer64 arrays and matrices
#'
#' These functions and methods facilitate working with integer64
#'   objects stored in matrices. As ever, the primary motivation
#'   for having tailor-made functions here is that R's methods
#'   often receive input from bit64 and treat the vectors as doubles,
#'   leading to unexpected and/or incorrect results.
#'
#' As of now, the `colSums()` and `rowSums()` methods are implemented
#'   as wrappers around equivalent `apply()` approaches, because
#'   re-using the default routine (and then applying integer64 to the
#'   result) does not work for objects with missing elements. Ideally
#'   this would eventually get its own dedicated C routine mimicking
#'   that of `colSums()` for integers; feature requests and PRs welcome.
#'
#' `aperm()` is required for `apply()` to work, in general, otherwise
#'    `FUN` gets applied to a class-stripped version of the input.
#'
#' @param x An array of integer64 numbers.
#' @param na.rm,dims Same interpretation as in [colSums()].
#' @param ... Passed on to subsequent methods.
#' @param data,nrow,ncol,byrow,dimnames,dim Arguments for `matrix()` and `array()`.
#' @examples
#' A = matrix(as.integer64(1:6), 3)
#'
#' colSums(A)
#' rowSums(A)
#' aperm(A, 2:1)
#' @name matrix64
#' @return A matrix of integer64 values with the appropriate dimension
NULL


#' @rdname matrix64
#' @exportS3Method matrix integer64
matrix.integer64 = function(data=NA_integer64_, ...) {
  if (!length(data)) data = NA_integer64_
  ret = withCallingHandlers_and_choose_call(
      base::matrix(data=data, ...), 
      c("matrix", "matrix.integer64")
    )
  class(ret) = class(data)
  ret
}

#' @rdname matrix64
#' @exportS3Method array integer64
array.integer64 = function(data=NA_integer64_, ...) {
  if (!length(data)) data = NA_integer64_
  ret = withCallingHandlers_and_choose_call(
      base::array(data=data, ...), 
      c("array", "array.integer64")
    )
  class(ret) = class(data)
  ret
}

#' @rdname matrix64
#' @export
colSums.integer64 = function(x, na.rm=FALSE, dims=1L) {
  dn = dim(x)
  if (!is.array(x) || length(dn) < 2L) 
    stop(errorCondition(gettext("'x' must be an array of at least two dimensions", domain="R-base"), call=choose_sys_call(c("colSums", "colSums.integer64"))))
  if (length(dims) != 1L || dims < 1L || dims > length(dn) - 1L) 
    stop(errorCondition(gettext("invalid 'dims'", domain="R-base"), call=choose_sys_call(c("colSums", "colSums.integer64"))))
  
  ret = apply(x, seq_along(dn)[-seq_len(dims)], sum, na.rm=na.rm)
  class(ret) = class(x)
  ret
}

#' @rdname matrix64
#' @export
rowSums.integer64 = function(x, na.rm=FALSE, dims=1L) {
  dn = dim(x)
  if (!is.array(x) || length(dn) < 2L) 
    stop(errorCondition(gettext("'x' must be an array of at least two dimensions", domain="R-base"), call=choose_sys_call(c("rowSums", "rowSums.integer64"))))
  if (length(dims) != 1L || dims < 1L || dims > length(dn) - 1L) 
    stop(errorCondition(gettext("invalid 'dims'", domain="R-base"), call=choose_sys_call(c("rowSums", "rowSums.integer64"))))
  
  ret = apply(x, seq_len(dims), sum, na.rm=na.rm)
  class(ret) = class(x)
  ret
}

#' @rdname matrix64
#' @param a,perm Passed on to [aperm()].
#' @exportS3Method aperm integer64
aperm.integer64 = function(a, perm, ...) {
  ret = NextMethod()
  class(ret) = class(a)
  ret
}

#' @exportS3Method `%*%` integer64
`%*%.integer64` = function(x, y) {
  if (!is.integer64(x) && !is.integer64(y)) 
    return(x%*%y)

  target_class = target_class_for_Ops(x, y)
  if (target_class %in% c("character", "factor")) {
    stop(errorCondition(gettext("requires numeric/complex matrix/vector arguments", domain="R"), call=sys.call(sys.nframe() - 1L)))
  }
  if (target_class != "integer64") {
    if (is.integer64(x)) {
      for (cc in class(y)) {
        f = getS3method("%*%", cc, optional=TRUE)
        if (!is.null(f))
          return(f(.as_double_integer64(x, keep.attributes=TRUE), y))
      }
      x = .as_double_integer64(x, keep.attributes=TRUE)
    } else {
      y = .as_double_integer64(y, keep.attributes=TRUE)
    }
    return(x%*%y)
  }

  dx = dim(x)
  dy = dim(y)
  if (length(dx) > 2L || length(dy) > 2L)
    stop("non-conformable arguments", domain="R")
  if (length(dx) <= 1L && length(dy) <= 1L) {
    dx = c(1L, length(x))
    if (length(x) == length(y)) {
      dy = c(length(y), 1L)
    } else {
      dy = c(1L, length(y))
    }
  }
  if (length(dx) <= 1L)
    dx = c(1L, dy[1L])
  if (length(dy) <= 1L)
    dy = c(dx[2L], 1L)
  if (dx[2L] != dy[1L])
    stop("non-conformable arguments", domain="R")
  dim(x) = dx
  dim(y) = dy

  if (is.double(x)) {
    ret = .Call(C_matmult_double_integer64, x, structure(as.integer64(y), dim=dy), double(dx[1L]*dy[2L]))
  } else if (is.double(y)) {
    ret = .Call(C_matmult_integer64_double, structure(as.integer64(x), dim=dx), y, double(dx[1L]*dy[2L]))
  } else {
    ret = .Call(C_matmult_integer64_integer64, structure(as.integer64(x), dim=dx), structure(as.integer64(y), dim=dy), double(dx[1L]*dy[2L]))
  }
  dim(ret) = c(dx[1L], dy[2L])
  oldClass(ret) = "integer64"
  ret
}

#' @exportS3Method as.matrix integer64
as.matrix.integer64 = function(x, ...) {
  if (is.matrix(x)) {
    x
  } else {
    array(x, c(length(x), 1L), if (!is.null(names(x))) list(names(x), NULL))
  }
}

# /*
# R-Code
# S3 atomic 64bit integers for R
# (c) 2011-2024 Jens Oehlschägel
# (c) 2025-2026 Michael Chirico
# Licence: GPL2
# Provided 'as is', use at your own risk
# Created: 2011-12-11
#*/

#' Identity function for class 'integer64'
#'
#' This will discover any deviation between objects containing integer64 vectors.
#'
#' This is simply a wrapper to [identical()] with default arguments
#'   `num.eq = FALSE, single.NA = FALSE`.
#'
#' @param x,y Atomic vector of class 'integer64'
#' @param num.eq,single.NA,attrib.as.set,ignore.bytecode,ignore.environment,ignore.srcref
#'   See [identical()].
#' @param ... Passed on to `identical()`. Only `extptr.as.ref=` is available as of R 4.4.1,
#'   and then only for versions of R >= 4.2.0.
#'
#' @return A single logical value, `TRUE` or `FALSE`, never `NA` and never
#'   anything other than a single value.
#' @keywords classes manip
#' @seealso [`==.integer64`] [identical()] [integer64()]
#' @examples
#'   i64 <- as.double(NA); class(i64) <- "integer64"
#'   identical(i64-1, i64+1)
#'   identical.integer64(i64-1, i64+1)
#' @name identical.integer64
NULL

#' Coerce from integer64
#'
#' Methods to coerce integer64 to other atomic types. 'as.bitstring' coerces
#'   to a human-readable bit representation (strings of zeroes and ones).
#'   The methods [format()], [as.character()], [as.double()],
#'   [as.logical()], [as.integer()] do what you would expect.
#'
#' @param x an integer64 vector
#' @param ...,origin,tz further arguments to the [NextMethod()]
#'
#' @return `as.bitstring` returns a string of class 'bitstring'.
#'
#' The other methods return atomic vectors of the expected types
#'
#' @keywords classes manip
#' @seealso [as.integer64.character()] [integer64()]
#' @examples
#'   as.character(lim.integer64())
#'   as.bitstring(lim.integer64())
#'   as.bitstring(as.integer64(c(-2, -1, NA, 0:2)))
#' @name as.character.integer64
NULL

#' Coerce to integer64
#'
#' Methods to coerce from other atomic types to integer64.
#'
#' @param x an atomic vector
#' @param ...,units further arguments to the [NextMethod()]
#' 
#' @details
#' `as.integer64.character` is realized using C function `strtoll` which
#'   does not support scientific notation. Instead of '1e6' use '1000000'.
#'   `as.integer64.bitstring` evaluates characters '0' and ' ' as zero-bit,
#'   all other one byte characters as one-bit, multi-byte characters are not allowed,
#'   strings shorter than 64 characters are treated as if they were left-padded with '0',
#'   strings longer than 64 bytes are mapped to `NA_INTEGER64` and a warning is emitted.
#'
#' @return The other methods return atomic vectors of the expected types
#'
#' @keywords classes manip
#' @seealso [as.character.integer64()] [integer64()]
#' @examples
#' as.integer64(as.character(lim.integer64()))
#' as.integer64(
#'   structure(c("1111111111111111111111111111111111111111111111111111111111111110",
#'               "1111111111111111111111111111111111111111111111111111111111111111",
#'               "1000000000000000000000000000000000000000000000000000000000000000",
#'               "0000000000000000000000000000000000000000000000000000000000000000",
#'               "0000000000000000000000000000000000000000000000000000000000000001",
#'               "0000000000000000000000000000000000000000000000000000000000000010"
#'   ), class = "bitstring")
#' )
#' as.integer64(
#'  structure(c("............................................................... ",
#'              "................................................................",
#'              ".                                                               ",
#'              "",
#'              ".",
#'              "10"
#'   ), class = "bitstring")
#' )
#' @name as.integer64.character
NULL

#' Extract or Replace Parts of an integer64 vector
#'
#' Methods to extract and replace parts of an integer64 vector.
#'
#' @param x an atomic vector
#' @param i,j indices specifying elements to extract
#' @param drop relevant for matrices and arrays. If TRUE the result is coerced to the lowest possible dimension.
#' @param value an atomic vector with values to be assigned
#' @param ... further arguments to the [NextMethod()]
#'
#' @note
#'   You should not subscript non-existing elements and not use `NA`s as subscripts.
#'   The current implementation returns `9218868437227407266` instead of `NA`.
#' @returns A vector, matrix, array or scalar of class 'integer64'
#' @keywords classes manip
#' @seealso [`[`][base::Extract] [integer64()]
#' @examples
#'   as.integer64(1:12)[1:3]
#'   x <- matrix(as.integer64(1:12), nrow = 3L)
#'   x
#'   x[]
#'   x[, 2:3]
#' @name extract.replace.integer64
NULL

#' Unary operators and functions for integer64 vectors
#'
#' Unary operators and functions for integer64 vectors.
#'
#' @param x an atomic vector of class 'integer64'
#' @param base an atomic scalar (we save 50% log-calls by not allowing
#'   a vector base)
#' @param digits integer indicating the number of decimal places (round)
#'   or significant digits (signif) to be used. Negative values are allowed
#'   (see [round()])
#' @param justify should it be right-justified (the default), left-justified,
#'   centred or left alone.
#' @param center see [scale()]
#' @param scale see [scale()]
#' @param ... further arguments to the [NextMethod()]
#'
#' @returns
#'   [format()] returns a character vector
#'
#'   [is.na()] and [`!`] return a logical vector
#'
#'   [sqrt()], [log()], [log2()] and [log10()] return a double vector
#'
#'   [sign()], [abs()], [floor()], [ceiling()], [trunc()] and
#'   [round()] return a vector of class 'integer64'
#'
#'   [signif()] is not implemented
#'
#' @keywords classes manip
#' @seealso [ops64] [integer64()]
#' @examples
#'   sqrt(as.integer64(1:12))
#' @name format.integer64
NULL

#' Summary functions for integer64 vectors
#'
#' Summary functions for integer64 vectors. Function 'range' without arguments
#'   returns the smallest and largest value of the 'integer64' class.
#'
#' @param ... atomic vectors of class 'integer64'
#' @param na.rm logical scalar indicating whether to ignore NAs
#' @param finite logical scalar indicating whether to ignore NAs (just for
#'   compatibility with [range.default()])
#'
#' @details
#' The numerical summary methods always return `integer64`. Wherever integer methods would
#'   return `Inf` (or its negation), here the extreme 64-bit integer `9223372036854775807` is
#'   returned. See [min()] for more details about the behavior.
#'
#' `lim.integer64` returns these limits in proper order
#'   `-9223372036854775807, +9223372036854775807` and without a [warning()].
#'
#' @returns
#'   [all()] and [any()] return a logical scalar
#'
#'   [range()] returns a integer64 vector with two elements
#'
#'   [min()], [max()], [sum()] and [prod()] return a integer64 scalar
#'
#' @keywords classes manip
#' @seealso [mean.integer64()] [cumsum.integer64()] [integer64()]
#' @examples
#'   lim.integer64()
#'   range(as.integer64(1:12))
#' @name sum.integer64
NULL

#' Cumulative Sums, Products, Extremes and lagged differences
#'
#' Cumulative Sums, Products, Extremes and lagged differences
#'
#' @param x an atomic vector of class 'integer64'
#' @param lag see [diff()]
#' @param differences see [diff()]
#' @param ... ignored
#'
#' @returns
#'   [cummin()], [cummax()] , [cumsum()] and [cumprod()]
#'      return a integer64 vector of the same length as their input
#'
#'   [diff()] returns a integer64 vector shorter by `lag*differences` elements
#'
#' @keywords classes manip
#' @seealso [sum.integer64()] [integer64()]
#' @examples
#'   cumsum(rep(as.integer64(1), 12))
#'   diff(as.integer64(c(0, 1:12)))
#'   cumsum(as.integer64(c(0, 1:12)))
#'   diff(cumsum(as.integer64(c(0, 0, 1:12))), differences=2)
#' @name cumsum.integer64
NULL

#' Concatenating integer64 vectors
#'
#' The ususal functions 'c', 'cbind' and 'rbind'
#'
#' @param ... two or more arguments coerced to 'integer64' and
#'   passed to [NextMethod()]
#' @param recursive logical. If `recursive = TRUE`, the function
#'   recursively descends through lists (and pairlists) combining all
#'   their elements into a vector.
#' @param deparse.level integer controlling the construction of labels in the case of non-matrix-like arguments
#'
#' @returns
#'   [c()] returns a vector of the appropriate mode. This could be a integer64 vector or a list of objects  
#'
#'   [cbind()] and [rbind()] return a matrix, data.frame or list with dimensions
#'
#' @note
#' R currently only dispatches generic 'c' to method 'c.integer64' if the
#'   first argument is 'integer64'
#'
#' @keywords classes manip
#' @seealso [rep.integer64()] [seq.integer64()] [as.data.frame.integer64()]
#'   [integer64()]
#'
#' @examples
#'   c(as.integer64(1), 2:6)
#'   cbind(1:6, as.integer64(1:6))
#'   rbind(1:6, as.integer64(1:6))
#' @name c.integer64
NULL

#' Replicate elements of integer64 vectors
#'
#' Replicate elements of integer64 vectors
#'
#' @param x a vector of 'integer64' to be replicated
#' @param ... further arguments passed to [NextMethod()]
#'
#' @returns [rep()] returns a integer64 vector
#' @keywords classes manip
#' @seealso [c.integer64()] [rep.integer64()]
#'   [as.data.frame.integer64()] [integer64()]
#'
#' @examples
#'   rep(as.integer64(1:2), 6)
#'   rep(as.integer64(1:2), c(6, 6))
#'   rep(as.integer64(1:2), length.out=6)
#' @name rep.integer64
NULL

#' integer64: Coercing to data.frame column
#'
#' Coercing integer64 vector to data.frame.
#'
#' @param x an integer64 vector
#' @param row.names,optional,... passed to NextMethod [as.data.frame()] after removing the
#'   'integer64' class attribute
#'
#' @returns a one-column data.frame containing an integer64 vector
#' @details
#'   'as.data.frame.integer64' is rather not intended to be called directly,
#'   but it is required to allow integer64 as data.frame columns.
#' @note This is currently very slow -- any ideas for improvement?
#' @keywords classes manip
#' @seealso
#'   [cbind.integer64()] [integer64()]
#    as.vector.integer64 removed as requested by the CRAN maintainer [as.vector.integer64()]
#' @examples
#'   as.data.frame(as.integer64(1:12))
#'   data.frame(a=1:12, b=as.integer64(1:12))
#' @name as.data.frame.integer64
NULL

#' integer64: Maintaining S3 class attribute
#'
#' Maintaining integer64 S3 class attribute.
#'
#' @param class NULL or a character vector of class attributes
#' @param whichclass the (single) class name to add or remove from the class vector
#'
#' @returns NULL or a character vector of class attributes
#'
#' @keywords classes manip internal
#' @seealso [oldClass()] [integer64()]
#' @examples
#'   plusclass("inheritingclass", "integer64")
#'   minusclass(c("inheritingclass", "integer64"), "integer64")
#' @name plusclass
NULL

#' Test if two integer64 vectors are all.equal
#'
#' A utility to compare integer64 objects 'x' and 'y' testing for
#'   ‘near equality’, see [all.equal()].
#'
#' @param target a vector of 'integer64' or an object that can be coerced
#'   with [as.integer64()]
#' @param current a vector of 'integer64' or an object that can be coerced
#'   with [as.integer64()]
#' @param tolerance numeric > 0.  Differences smaller than `tolerance` are
#'   not reported.  The default value is close to `1.5e-8`.
#' @param scale `NULL` or numeric > 0, typically of length 1 or
#'   `length(target)`. See Details.
#' @param countEQ logical indicating if the `target == current` cases should
#'   be counted when computing the mean (absolute or relative) differences.
#'   The default, `FALSE` may seem misleading in cases where `target` and
#'   `current` only differ in a few places; see the extensive example.
#' @param formatFUN a [function()] of two arguments, `err`, the relative,
#'   absolute or scaled error, and `what`, a character string indicating the
#'   _kind_ of error; maybe used, e.g., to format relative and absolute errors
#'   differently.
#' @param ... further arguments are ignored
#' @param check.attributes logical indicating if the [attributes()] of `target`
#'   and `current` (other than the names) should be compared.
#'
#' @returns
#'   Either ‘TRUE’ (‘NULL’ for ‘attr.all.equal’) or a vector of ‘mode’
#'   ‘"character"’ describing the differences between ‘target’ and
#'   ‘current’.
#'
#' @details
#'  In [all.equal.numeric()] the type `integer` is treated as a proper subset
#'   of `double` i.e. does not complain about comparing `integer` with `double`.
#'   Following this logic `all.equal.integer64` treats `integer` as a proper
#'   subset of `integer64` and does not complain about comparing `integer` with
#'   `integer64`. `double` also compares without warning as long as the values
#'   are within [lim.integer64()], if `double` are bigger `all.equal.integer64`
#'   complains about the `all.equal.integer64 overflow warning`. For further
#'   details see [all.equal()].
#'
#' @note
#'    [all.equal()] only dispatches to this method if the first argument is `integer64`,
#'    calling [all.equal()] with a `non-integer64` first and a `integer64` second argument
#'    gives undefined behavior!
#'
#' @seealso [all.equal()]
#' @examples
#'   all.equal(as.integer64(1:10), as.integer64(0:9))
#'   all.equal(as.integer64(1:10), as.integer(1:10))
#'   all.equal(as.integer64(1:10), as.double(1:10))
#'   all.equal(as.integer64(1), as.double(1e300))
#' @name all.equal.integer64
NULL

#' Factors
#'
#' The function [factor] is used to encode a vector as a factor.
#' 
#' @inheritParams base::factor
#' @param nmax an upper bound on the number of levels.
#'
#' @return An object of class "factor" or "ordered".
#' @seealso [factor][base::factor]
#' @examples
#'   x <- as.integer64(c(132724613L, -2143220989L, -1L, NA, 1L))
#'   factor(x)
#'   ordered(x)
#' @name factor
NULL

methods::setOldClass("integer64")
methods::setOldClass("difftime")

# contributed by Leonardo Silvestri with modifications of JO
#' @rdname all.equal.integer64
#' @method all.equal integer64
#' @exportS3Method all.equal integer64
all.equal.integer64  <- function(target, current,
                                 tolerance = sqrt(.Machine$double.eps),
                                 scale = NULL,
                                 countEQ = FALSE,
                                 formatFUN = function(err, what) format(err),
                                 ...,
                                 check.attributes = TRUE) {
  if (!is.numeric(tolerance))
    stop("'tolerance' should be numeric")
  if (!is.numeric(scale) && !is.null(scale))
    stop("'scale' should be numeric or NULL")
  if (!is.logical(check.attributes))
    stop(gettextf("'%s' must be logical", "check.attributes"),
         domain = NA)
  # JO: BEGIN respect that integer is a proper subset of integer64 like integer is a proper subset of double
  oldwarn = getOption("warn")
  on.exit(options(warn=oldwarn))
  options(warn=2L)
  if (!is.integer64(target)) {
    cl <- oldClass(target)
    oldClass(target) <- NULL
    target <- try(as.integer64(target))
    if (inherits(target, 'try-error'))
      return(paste("while coercing 'target' to 'integer64':",  attr(target, "condition")$message))
    oldClass(target) <- c(cl, "integer64")
  }
  if (!is.integer64(current)) {
    cl <- oldClass(current)
    oldClass(current) <- NULL
    current <- try(as.integer64(current))
    if (inherits(current, 'try-error'))
      return(paste("while coercing 'current' to 'integer64':",  attr(current, "condition")$message))
    oldClass(current) <- c(cl, "integer64")
  }
  # JO: END respect that integer is a proper subset of integer64 like integer is a proper subset of double
  msg = NULL
  msg = if (check.attributes)
           attr.all.equal(target, current, tolerance = tolerance,
                          scale = scale, ...)
  if (data.class(target) != data.class(current)) {
    msg <- c(msg, paste0("target is ", data.class(target),
                         ", current is ", data.class(current)))
    return(msg)
  }
  lt = length(target)
  lc = length(current)
  if (lt != lc) {
    if (!is.null(msg))
      msg <- msg[-grep("\\bLengths\\b", msg)]
    msg <- c(msg, paste0("integer64: lengths (", lt, ", ", lc, ") differ"))
    return(msg)
  }
  out = is.na(target)
  if (any(out != is.na(current))) {
    msg <- c(msg, paste("'is.NA' value mismatch:", sum(is.na(current)),
                        "in current", sum(out), "in target"))
    return(msg)
  }
  out = out | target == current
  if (all(out))
    # TODO(R>=4.4.0): msg %||% TRUE
    return(if (is.null(msg)) TRUE else msg)
  anyO = any(out)
  sabst0 = if (countEQ && anyO) mean(abs(target[out])) else 0.0
  if (anyO) {
    keep <- which(!out)
    target  <- target [keep]
    current <- current[keep]
    if (!is.null(scale) && length(scale) > 1L) {
      # TODO(R>=4.0.0): Try removing this ocl part when rep() dispatching WAI on all versions (#100)
      ocl = class(scale)
      scale = rep_len(scale, length(out))[keep]
      class(scale) = ocl
    }
  }
  N = length(target)
  what = if (is.null(scale)) {
    scale <- sabst0 + sum(abs(target)/N)
    if (is.finite(scale) && scale > tolerance) {
      "relative"
    } else {
      scale <- 1.0
      "absolute"
    }
  } else {
    stopifnot(scale > 0.0)
    if (all(abs(scale - 1.0) < 1e-07))
      "absolute"
    else
      "scaled"
  }
  xy = sum(abs(target - current) / (N*scale))
  if (is.na(xy) || xy > tolerance)
    msg <- c(msg, paste("Mean", what, "difference:", formatFUN(xy, what)))
  # TODO(R>=4.4.0): msg %||% TRUE
  if (is.null(msg)) TRUE else msg
}

# TODO(R>=4.2.0): Consider restoring extptr.as.ref= to the signature.
#' @rdname identical.integer64
#' @exportS3Method identical integer64
#' @export
identical.integer64 = function(x, y,
                                num.eq=FALSE,
                                single.NA=FALSE,
                                attrib.as.set=TRUE,
                                ignore.bytecode=TRUE,
                                ignore.environment=FALSE,
                                ignore.srcref=TRUE,
                                ...) {
  identical(x=x, y=y,
    num.eq=num.eq,
    single.NA=single.NA,
    attrib.as.set=attrib.as.set,
    ignore.bytecode=ignore.bytecode,
    ignore.environment=ignore.environment,
    ignore.srcref=ignore.srcref,
    ...
  )
}


#' @rdname as.integer64.character
#' @export
as.integer64 = function(x, ...) UseMethod("as.integer64")

#' @rdname as.character.integer64
#' @export
as.bitstring = function(x, ...) UseMethod("as.bitstring")

#' @rdname plusclass
#' @export
minusclass = function(class, whichclass) {
  if (!length(class)) return(class)
  i = whichclass == class
  if (any(i))
    class[!i]
  else
    class
}

#' @export
plusclass = function(class, whichclass) {
  if (!length(class)) return(whichclass)
  c(class, if (!any(whichclass == class)) whichclass)
}

#' @rdname bit64-package
#' @param length length of vector using [integer()]
#' @return `integer64` returns a vector of 'integer64', i.e.,
#'   a vector of [double()] decorated with class 'integer64'.
#' @export
integer64 = function(length=0L) {
  ret = double(length)
  oldClass(ret) = "integer64"
  ret
}

#' @rdname bit64-package
#' @param x an integer64 vector
#' @export
is.integer64 = function(x) inherits(x, "integer64")

#' @rdname as.integer64.character
#' @export
as.integer64.NULL = function(x, ...) integer64()

#' @rdname as.integer64.character
#' @param keep.names Logical, default `FALSE`. If `TRUE`, the input's names are retained.
#' @export
as.integer64.integer64 = function(x, ..., keep.names=FALSE) {
  ret = unclass(x)
  attributes(ret) = NULL
  oldClass(ret) = "integer64"
  # for back-compatibility, e.g. {nanotime}
  if (isTRUE(keep.names)) names(ret) = names(x)
  ret
}

#' @rdname as.integer64.character
#' @export
as.integer64.double = function(x, ..., keep.names=FALSE) {
  ret = .Call(C_as_integer64_double, x, double(length(x)))
  oldClass(ret) = "integer64"
  if (isTRUE(keep.names)) names(ret) = names(x)
  ret
}

#' @rdname as.integer64.character
#' @exportS3Method as.integer64 complex
as.integer64.complex = function(x, ...) {
  xd = withCallingHandlers(
    as.double(x),
    # call.=FALSE to avoid confusion about where the warning arises
    warning = function(w) {
      warning(conditionMessage(w), call.=FALSE)
      invokeRestart("muffleWarning")
    }
  )
  ret = .Call(C_as_integer64_double, xd, double(length(xd)))
  oldClass(ret) = "integer64"
  ret
}

#' @rdname as.integer64.character
#' @export
as.integer64.integer = function(x, ...) {
  ret = .Call(C_as_integer64_integer, x, double(length(x)))
  oldClass(ret) = "integer64"
  ret
}

#' @rdname as.integer64.character
#' @exportS3Method as.integer64 raw
as.integer64.raw = function(x, ...) {
  ret = .Call(C_as_integer64_integer, as.integer(x), double(length(x)))
  oldClass(ret) = "integer64"
  ret
}

#' @rdname as.integer64.character
#' @export
as.integer64.logical = as.integer64.integer

#' @rdname as.integer64.character
#' @export
as.integer64.character = function(x, ...) {
  ret = .Call(C_as_integer64_character, x, rep(NA_real_, length(x)))
  oldClass(ret) = "integer64"
  ret
}

#' @rdname as.integer64.character
#' @export
as.integer64.factor = function(x, ...) 
  as.integer64(unclass(x), ...)

#' @rdname as.integer64.character
#' @exportS3Method as.integer64 Date
as.integer64.Date = function(x, ...)
  as.integer64(as.double(x))

#' @rdname as.integer64.character
#' @exportS3Method as.integer64 POSIXct
as.integer64.POSIXct = function(x, ...)
  as.integer64(as.double(x))

#' @rdname as.integer64.character
#' @exportS3Method as.integer64 POSIXlt
as.integer64.POSIXlt = function(x, ...)
  as.integer64(as.POSIXct(x))

#' @rdname as.integer64.character
#' @exportS3Method as.integer64 difftime
as.integer64.difftime = function(x, units="auto", ...)
  as.integer64(as.double(x, units=units, ...))

.as_double_integer64 = function(x, keep.attributes=FALSE, ...) {
  ret = .Call(C_as_double_integer64, x, double(length(x)))
  if (isTRUE(keep.attributes)) {
    # like dimensions for matrix operations
    a = attributes(x)
    a$class = NULL
    attributes(ret) = a
  }
  ret
}

#' @rdname as.character.integer64
#' @export
as.double.integer64 = function(x, ...) 
  .as_double_integer64(x, keep.attributes=FALSE, ...)

#' @rdname as.character.integer64
#' @exportS3Method as.numeric integer64
as.numeric.integer64 = as.double.integer64

#' @rdname as.character.integer64
#' @exportS3Method as.complex integer64
as.complex.integer64 = function(x, ...) as.complex(as.double(x), ...)

#' @rdname as.character.integer64
#' @export
as.integer.integer64 = function(x, ...) 
  .Call(C_as_integer_integer64, x, integer(length(x)))

#' @rdname as.character.integer64
#' @exportS3Method as.raw integer64
as.raw.integer64 = function(x, ...) {
  withCallingHandlers(
    as.raw(.Call(C_as_integer_integer64, x, integer(length(x)))),
    warning = function(w) {
      warning(conditionMessage(w), call.=FALSE)
      invokeRestart("muffleWarning")
    }
  )
}

#' @rdname as.character.integer64
#' @export
as.logical.integer64 = function(x, ...)
  .Call(C_as_logical_integer64, x, logical(length(x)))

#' @rdname as.character.integer64
#' @export
as.character.integer64 = function(x, ...)
  .Call(C_as_character_integer64, x, rep(NA_character_, length(x)))

#' @rdname as.character.integer64
#' @export
as.bitstring.integer64 = function(x, ...) {
  ret = .Call(C_as_bitstring_integer64, x, rep(NA_character_, length(x)))
  oldClass(ret) = 'bitstring'
  attr(ret, 'nbits') = c(1L, 63L)
  attr(ret, 'type') = "int64"
  ret
}

#' @rdname as.character.integer64
#' @exportS3Method as.Date integer64
as.Date.integer64 = function(x, origin, ...)
  as.Date(as.double(x), origin=origin, ...)

#' @rdname as.character.integer64
#' @exportS3Method as.POSIXct integer64
as.POSIXct.integer64 = function(x, tz="", origin, ...)
  as.POSIXct(as.double(x), tz=tz, origin=origin, ...)

#' @rdname as.character.integer64
#' @exportS3Method as.POSIXlt integer64
as.POSIXlt.integer64 = function(x, tz="", origin, ...)
  as.POSIXlt(as.double(x, ...), tz=tz, origin=origin, ...)

#' @rdname as.character.integer64
#' @export as.factor
as.factor = function(x) factor(x=x)

#' @rdname as.character.integer64
#' @export as.ordered
as.ordered = function(x) ordered(x=x)

#' @rdname factor
#' @export
factor = function(x=character(), levels, labels=levels, exclude=NA, ordered=is.ordered(x), nmax=NA) {
  force(x)
  if (!is.integer64(x)) {
    sys_call = match.call()
    sys_call[[1L]] = base::factor
    sys_call$x = x
    pf = parent.frame()
    return(withCallingHandlers_and_choose_call(eval(sys_call, envir=pf), "factor"))
  }
  
  nx = names(x)
  if (missing(levels)) {
    levels = sort(unique(x))
  } else if (length(x) >= 4000) {
    levels = as.integer64(levels)
  }
  # use base::factor for short vectors because it is faster
  if (length(x) < 4000) {
    force(ordered)
    x = as.character(x)
    levels = as.character(levels)
    if (missing(labels))
      return(withCallingHandlers_and_choose_call(base::factor(x=x, levels=levels, exclude=exclude, ordered=ordered, nmax=nmax), "factor"))
    else
      return(withCallingHandlers_and_choose_call(base::factor(x=x, levels=levels, labels=labels, exclude=exclude, ordered=ordered, nmax=nmax), "factor"))
  }

  # basically copied from base::factor, but using the benefit from caching
  levels = levels[is.na(match(levels, exclude))]
  ret = match(x, levels)
  if (!is.null(nx)) 
    names(ret) = nx
  if (missing(labels)) {
    levels(ret) = as.character(levels)
  } else {
    nlab = length(labels)
    if (nlab == length(levels)) {
      xlevs = as.character(labels)
      nlevs = unique(xlevs)
      at = attributes(ret)
      at$levels = nlevs
      ret = match(xlevs, nlevs)[ret]
      attributes(ret) = at
    } else if (nlab == 1L) {
      levels(ret) = paste0(labels, seq_along(levels))
    } else {
      stop(gettextf("invalid 'labels'; length %d should be 1 or %d", nlab, length(levels), domain="R-base"), domain=NA)
    }
  }
  class(ret) <- c(if (ordered) "ordered", "factor")
  ret
}

#' @rdname factor
#' @export
ordered = function(x=character(), ...) factor(x, ..., ordered=TRUE)

#' @rawNamespace if (getRversion() < "4.6.0") S3method(print,bitstring)
print.bitstring = function(x, ...) {
  reset_class = minusclass(class(x), 'bitstring')
  attributes(x) = NULL
  oldClass(x) = reset_class
  NextMethod(x)
}

#' @rdname as.integer64.character
#' @export
as.integer64.bitstring = function(x, ...) {
  ret = .Call(C_as_integer64_bitstring, x, double(length(x)))
  oldClass(ret) = "integer64"
  ret
}

# read.table expects S4 as()
methods::setAs("ANY", "integer64", function(from) as.integer64(from))
methods::setAs("integer64", "factor", function(from) as.factor(from))
methods::setAs("integer64", "ordered", function(from) as.ordered(from))
methods::setAs("integer64", "difftime", function(from) as.difftime(from, units="secs"))
methods::setAs("integer64", "POSIXct", function(from) as.POSIXct(from))
methods::setAs("integer64", "POSIXlt", function(from) as.POSIXlt(from))
methods::setAs("integer64", "Date", function(from) as.Date(from))
methods::setAs("integer64", "raw", function(from) as.raw(from))

# this is a trick to generate NA_integer64_ for namespace export before
# as.integer64() is available because dll is not loaded
#' @rdname as.integer64.character
#' @export
NA_integer64_ = unserialize(as.raw(c(
  0x58, 0x0a, 0x00, 0x00, 0x00, 0x02, 0x00, 0x03, 0x03, 0x00, 0x00, 0x02, 0x03, 0x00, 0x00, 0x00,
  0x03, 0x0e, 0x00, 0x00, 0x00, 0x01, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x04, 0x02, 0x00, 0x00, 0x00, 0x01, 0x00, 0x04, 0x00, 0x09, 0x00, 0x00, 0x00, 0x05, 0x63, 0x6c,
  0x61, 0x73, 0x73, 0x00, 0x00, 0x00, 0x10, 0x00, 0x00, 0x00, 0x01, 0x00, 0x04, 0x00, 0x09, 0x00,
  0x00, 0x00, 0x09, 0x69, 0x6e, 0x74, 0x65, 0x67, 0x65, 0x72, 0x36, 0x34, 0x00, 0x00, 0x00, 0xfe
)))

#' @rdname bit64-package
#' @param value an integer64 vector of values to be assigned
#' @export
`length<-.integer64` <- function(x, value) {
  cl = oldClass(x)
  n = length(x)
  x = NextMethod()
  oldClass(x) = cl
  if (value>n)
    x[(n+1L):value] <- 0L
  x
}

#' @rdname format.integer64
#' @export
format.integer64 = function(x, justify="right", ...) {
  a = attributes(x)
  x = as.character(x)
  ret = format(x, justify=justify, ...)
  a$class = minusclass(a$class, "integer64")
  attributes(ret) = a
  ret
}

#' @rdname bit64-package
#' @param quote logical, indicating whether or not strings should be printed with surrounding quotes.
#' @param ... further arguments to the [NextMethod()]
#' @export
print.integer64 = function(x, quote=FALSE, ...) {
  a = attributes(x)
  if (length(x)) {
    cat("integer64\n")
    ret <- as.character(x)
    a$class <- minusclass(a$class, "integer64")
    attributes(ret) <- a
    print(ret, quote=quote, ...)
  } else {
    cat("integer64(0)\n")
  }
  invisible(x)
}

#' @rdname bit64-package
#' @param object an integer64 vector
#' @param vec.len,give.head,give.length see [utils::str()]
#' @export
str.integer64 = function(object, vec.len=strO$vec.len, give.head=TRUE, give.length=give.head, ...) {
  strO = strOptions()
  vec.len = 2L*vec.len
  n = length(object)
  displayObject = object[seq_len(min(vec.len, length(object)))]

  cat(
    if (isTRUE(give.head)) {
      if (length(object) == 0L && is.null(dim(object))) {
        "integer64(0)"
      } else {
        paste0(
          "integer64 ",
          if (length(object) > 1L && is.null(dim(object))) {
            if (isTRUE(give.length)) paste0("[1:", n, "] ") else " "
          } else if (!is.null(obj_dim <- dim(object))) {
            if (prod(obj_dim) != n)
              stop(gettextf("dims [product %d] do not match the length of object [%d]", prod(obj_dim), n, domain="R"), domain=NA)
            if (length(obj_dim) == 1L) {
              paste0("[", n, "(1d)] ")
            } else {
              paste0("[", toString(vapply(obj_dim, function(el) if (el < 2L) as.character(el) else paste0("1:", el), "")), "] ")
            }
          }
        )
      }
    },
    paste(as.character(displayObject), collapse=" "),
    if (n > vec.len) " ...",
    " \n",
    sep=""
  )
  invisible()
}


position_args_with_int64_to_int_coercion = function(sys_call, eval_frame, skipLast=FALSE) {
  sc = as.list(sys_call)[-(1:2)]
  if (isTRUE(skipLast))
    sc = sc[-length(sc)]
  lapply(sc, function(el) {
    if(missing_or_dots(el)) return(el)
    el = eval(el, eval_frame)   
    if (is.integer64(el))
      el = as.integer(el)
    el
  })
}

#' @rdname extract.replace.integer64
#' @export
`[.integer64` = function(x, i, j, ..., drop=TRUE) {
  old_class = oldClass(x)

  sc = sys.call() # NB: not match.call(), which eats a missing argument in x[1, , 3]
  pf = parent.frame()
  args = position_args_with_int64_to_int_coercion(sc, pf)
  args$drop = FALSE
  if (length(args) == 1L && isFALSE(drop)) return(x)
  oldClass(x) = NULL
  ret = withCallingHandlers_and_choose_call(do.call(`[`, c(list(x=x), args)), c("[", "[.integer64"))
  NA_integer64_real = NA_integer64_
  oldClass(NA_integer64_real) = NULL
  # drop is not relevant anymore for NA handling
  args$drop = NULL

  # NA handling
  if (length(dim(ret)) <= 1L) {
    # vector mode
    if (!missing_or_dots(args[[1L]])) {
      arg1Value = args[[1L]]
      if (is.logical(arg1Value)) {
        ret[is.na(arg1Value[arg1Value])] = NA_integer64_real
      } else if (is.character(arg1Value)) {
        ret[is.na(arg1Value) | arg1Value == "" | !arg1Value %in% names(x)] = NA_integer64_real
      } else if (anyNA(arg1Value) || suppressWarnings(max(arg1Value, na.rm=TRUE)) > length(x)) {
        arg1Value = arg1Value[arg1Value != 0]
        ret[which(is.na(arg1Value) | arg1Value > length(x))] = NA_integer64_real
      }
    }
  } else {
    # array/matrix mode
    dimSelect = args[seq_along(dim(x))]
    for (ii in seq_along(dimSelect)) {
      if (missing_or_dots(dimSelect[[ii]])) next
      dsValue = dimSelect[[ii]]
      if (is.logical(dsValue) && anyNA(dsValue)) {
        naIndex = which(is.na(seq_len(dim(x)[ii])[dsValue]))
      } else {
        naIndex = which(is.na(dsValue[dsValue != 0L]))
      }
      if (length(naIndex)) {
        setArgs = rep(list(substitute()), length(dimSelect))
        setArgs[[ii]] = naIndex
        ret = do.call(`[<-`, c(list(x=ret), setArgs, list(value=NA_integer64_real)))
      }
    }
  }

  # dimension handling
  if (!isFALSE(drop) && !is.null(dim(ret))) {
    newDim = dim(ret)[dim(ret) != 1L]
    if(length(newDim) == 1L && !(length(dim(x)) == 1L && newDim != 1L))
      newDim = NULL
    dim(ret) = if (length(newDim)) newDim else NULL
  }

  oldClass(ret) = old_class
  ret
}

#' @rdname extract.replace.integer64
#' @export
`[<-.integer64` = function(x, ..., value) {
  sc = sys.call()
  pf = parent.frame()
  args = position_args_with_int64_to_int_coercion(sc, pf, skipLast=TRUE)
  
  # TODO(#44): next Release: change default behavior; subsequent Release: change from message to warning; subsequent Release: change from warning to error; subsequent Release: remove option and promote_to_char
  if ((is.character(value) && isTRUE(getOption("bit64.promoteInteger64ToCharacter", FALSE))) || is.complex(value) || (is.double(value) && class(value)[1L] != "numeric")) {
    args$value = value
    x = structure(as(x, class(value)[1L]), dim = dim(x), dimnames = dimnames(x))
    ret = withCallingHandlers_and_choose_call(do.call(`[<-`, c(list(x=x), args)), c("[<-", "[<-.integer64"))  
  } else {
    args$value = as.integer64(value)
    old_class = oldClass(x)
    oldClass(x) = NULL
    ret = withCallingHandlers_and_choose_call(do.call(`[<-`, c(list(x=x), args)), c("[<-", "[<-.integer64"))  
    oldClass(ret) = old_class
  }
  ret
}

#' @rdname extract.replace.integer64
#' @export
`[[.integer64` = function(x, ...) {
  args = lapply(list(...), function(el) {
    if (is.integer64(el))
      el = as.integer(el)
    el
  })
  old_class = oldClass(x)
  oldClass(x) = NULL
  withCallingHandlers_and_choose_call({ret = do.call(`[[`, c(list(x=x), args))}, c("[[", "[[.integer64"))  
  oldClass(ret) = old_class
  ret
}

#' @rdname extract.replace.integer64
#' @export
`[[<-.integer64` = function(x, ..., value) {
  args = lapply(list(...), function(el) {
    if (is.integer64(el))
      el = as.integer(el)
    el
  })
  # TODO(#44): next Release: change default behavior; subsequent Release: change from message to warning; subsequent Release: change from warning to error; subsequent Release: remove option and promote_to_char
  if ((is.character(value) && isTRUE(getOption("bit64.promoteInteger64ToCharacter", FALSE))) || is.complex(value) || (is.double(value) && class(value)[1L] != "numeric")) {
    args$value = value
    x = structure(as(x, class(value)[1L]), dim = dim(x), dimnames = dimnames(x))
    withCallingHandlers_and_choose_call({ret = do.call(`[[<-`, c(list(x=x), args))}, c("[[<-", "[[<-.integer64"))  
  } else {
    args$value = as.integer64(value)
    old_class = oldClass(x)
    oldClass(x) = NULL
    withCallingHandlers_and_choose_call({ret = do.call(`[[<-`, c(list(x=x), args))}, c("[[<-", "[[<-.integer64"))  
    oldClass(ret) = old_class
  }
  ret
}

#' @rdname c.integer64
#' @export
c.integer64 = function(..., recursive=FALSE) {
  # This check can be dropped in the future when c.integer64 is not exported anymore
  if (...length() == 0L) return(NULL)
  dots = list(...)
  
  if (!isTRUE(recursive) && any(vapply(dots, is.list, FALSE))) {
    return(unlist(lapply(dots, function(el) if (inherits(el, "POSIXlt")) el else as.list(el)), recursive=FALSE))
  }
  
  value_class = target_class(dots, recursive=recursive, POSIXltAsCharacter=TRUE)
  # find positions of elements to be converted
  if(value_class == "integer64") {
    # integer64 doesn't have to be converted, but `oldClass(val) = NULL` has to be applied
    checkFunc = Negate(is.null)
  } else {
    checkFunc = is.integer64
  }
  findPositionsOfItemsToConvert = function(x) {
    res = list()
    for (ii in seq_along(x)) {
      if (inherits(x[[ii]], c("list", "data.frame"))) {
        res = c(res, lapply(findPositionsOfItemsToConvert(x[[ii]]), function(el) c(ii, el)))
      } else if (checkFunc(x[[ii]])) {
        res = c(res, list(ii))
      }
    }
    res
  }
  for (idx in findPositionsOfItemsToConvert(dots)) {
    val = dots[[idx]]
    if (inherits(val, "POSIXlt")) {
      val = lapply(unclass(val), as, value_class)
    } else if (value_class == "integer64") {
      # NB: as(, "integer64") may not work, #298
      val = as.integer64(val)
      oldClass(val) = NULL
    } else {
      val = as(val, value_class)
    }
    names(val) = names(dots[[idx]])
    dots[[idx]] = val
  }

  ret = do.call(c, c(dots, list(recursive=recursive)))
  if (value_class == "integer64")
    oldClass(ret) = value_class
  ret
}

# helper function to generate names for cbind/rbind if missing; it is not intended to be exported
make_names_for_cbind = function(sys_call, dots, deparse.level=1) {
  nd = names(dots)
  if (!deparse.level %in% c(1L, 2L)) return(nd)
  if (is.null(nd)) 
    nd = character(length(dots))
  
  sys_call_dots = sys_call[-1L][seq_along(dots)]
  sel = !logical(length(sys_call_dots))
  if (deparse.level == 1L)
    sel = vapply(sys_call_dots, is.symbol, FALSE)
  sel = sel & nd == "" & !vapply(dots, function(el) length(el) == 1L && is.na(el), FALSE)
  nd[sel] = as.character(sys_call_dots[sel])
  nd
}

#' @rdname c.integer64
#' @export
cbind.integer64 = function(..., deparse.level=1) {
  dots = list(...)
  value_class = target_class(dots, recursive=FALSE)
  
  # find positions of elements to be converted
  if(value_class == "integer64") {
    # integer64 doesn't have to be converted, but `oldClass(val) = NULL` has to be applied
    checkFunc = Negate(is.null)
  } else {
    checkFunc = is.integer64
  }
  positionsOfItemsToConvert = which(vapply(dots, function(el) !is.list(el) && checkFunc(el), FALSE, USE.NAMES=FALSE))

  # set names if missing  
  names(dots) = make_names_for_cbind(sys.call(sys.nframe() - 1L), dots, deparse.level)
  # convert relevant items
  for (idx in positionsOfItemsToConvert) {
    val = dots[[idx]]
    # NB: as(, "integer64") may not work, #298
    if (value_class == "integer64") {
      val = structure(as.integer64(val), dim=dim(val), dimnames=dimnames(val), names=names(val))
      oldClass(val) = NULL
    }else {
      val = structure(as(val, value_class), dim=dim(val), dimnames=dimnames(val), names=names(val))
    }
    dots[[idx]] = val
  }
  ret = withCallingHandlers_and_choose_call(
    do.call(cbind, c(dots, list(deparse.level=deparse.level))), 
    c("cbind", "cbind"),
    callStack = sys.calls()
  )
  
  # restore integer64 class
  if (value_class == "integer64") {
    if (is.list(ret)) {
      estimatedColumnIndices = lapply(dots, function(el) {
        res = ncol(el)
        if (is.null(res)) 
          res = as.integer(length(el) > 0L)
        res
      })
      lastValue = 0L
      for (ii in seq_along(estimatedColumnIndices)) {
        estimatedColumnIndices[[ii]] = lastValue + seq_len(estimatedColumnIndices[[ii]])
        lastValue = lastValue + length(estimatedColumnIndices[[ii]])
      }
      nrow_ret = nrow(ret)
      for (idx in positionsOfItemsToConvert) {
        for (ii in estimatedColumnIndices[[idx]]) {
          if (is.data.frame(ret)) {
            oldClass(ret[[ii]]) = value_class
          } else {
            for (jj in seq_len(nrow_ret))
              oldClass(ret[[(ii - 1L)*nrow_ret + jj]]) = value_class
          }
        }
      }
    } else {
      oldClass(ret) = value_class
    }
  }
  ret
}

#' @rdname c.integer64
#' @export
rbind.integer64 = function(..., deparse.level=1) {
  dots = list(...)
  value_class = target_class(dots, recursive=TRUE)
  
  # find positions of elements to be converted
  if(value_class == "integer64") {
    # integer64 doesn't have to be converted, but `oldClass(val) = NULL` has to be applied
    checkFunc = Negate(is.null)
  } else {
    checkFunc = is.integer64
  }
  findPositionsOfItemsToConvert = function(x) {
    res = list()
    for (ii in seq_along(x)) {
      if (is.data.frame(x[[ii]])) {
        res = c(res, lapply(findPositionsOfItemsToConvert(x[[ii]]), function(el) c(ii, el)))
      } else if (checkFunc(x[[ii]]) && !is.list(x[[ii]])) {
        res = c(res, list(ii))
      }
    }
    res
  }
  positionsOfItemsToConvert = findPositionsOfItemsToConvert(dots)

  # set names if missing  
  names(dots) = make_names_for_cbind(sys.call(sys.nframe() - 1L), dots, deparse.level)
  # convert relevant items
  for (idx in positionsOfItemsToConvert) {
    val = dots[[idx]]
    # NB: as(, "integer64") may not work, #298
    if (value_class == "integer64") {
      val = structure(as.integer64(val), dim=dim(val), dimnames=dimnames(val), names=names(val))
      oldClass(val) = NULL
    } else {
      val = structure(as(val, value_class), dim=dim(val), dimnames=dimnames(val), names=names(val))
    }
    dots[[idx]] = val
  }
  ret = withCallingHandlers_and_choose_call(
    do.call(rbind, c(dots, list(deparse.level=deparse.level))), 
    c("rbind", "rbind"),
    callStack = sys.calls()
  )
  
  # restore integer64 class
  if (value_class == "integer64") {
    if (is.list(ret)) {
      if (is.data.frame(ret)) {
        for (ii in seq_along(ret))
          oldClass(ret[[ii]]) = value_class
      } else {
        # for rbind, we have to estimate row indices for each item to convert because of possible recycling
        # this is mainly for POSIXlt
        estimatedRowIndices = lapply(dots, function(el) {
          res = nrow(el)
          if (is.null(res)) 
            res = as.integer(length(el) > 0L)
          res
        })
        lastValue = 0L
        for (ii in seq_along(estimatedRowIndices)) {
          estimatedRowIndices[[ii]] = lastValue + seq_len(estimatedRowIndices[[ii]])
          lastValue = lastValue + length(estimatedRowIndices[[ii]])
        }
        col_offset = (seq_len(ncol(ret)) - 1L)*nrow(ret)
        for (idx in positionsOfItemsToConvert) {
          for (ii in estimatedRowIndices[[idx]]) {
            for (jj in col_offset)
              oldClass(ret[[ii + jj]]) = value_class
          }
        }
      }
    } else {
      oldClass(ret) = value_class
    }
  }
  ret
}

#' @rdname as.data.frame.integer64
#' @export
as.data.frame.integer64 = function(x, row.names=NULL, optional=FALSE, ...) {
  cl = oldClass(x)
  on.exit(setattr(x, "class", cl))
  # tenfold runtime if using attr() here instead of setattr()
  setattr(x, "class", minusclass(cl, "integer64"))
  ret = as.data.frame(x, row.names=row.names, optional=optional, ...)
  for (i in seq_along(ret))
    setattr(ret[[i]], "class", cl)
  ret
}

#' @export
rep.integer64 = function(x, ...) {
  cl = oldClass(x)
  ret = NextMethod()
  oldClass(ret) = cl
  ret
}

# FIXME no method dispatch for :
`:.integer64` <- function(from, to) {
  from = as.integer64(from)
  to = as.integer64(to)
  ret = .Call(C_seq_integer64, from, as.integer64(1L), double(as.integer(to-from+1L)))
  oldClass(ret) = "integer64"
  ret
}

#' Generating sequence of integer64 values
#'
#' @param from integer64 scalar (in order to dispatch the integer64 method of [seq()])
#' @param to scalar
#' @param by scalar
#' @param length.out scalar
#' @param along.with scalar
#' @param ... ignored
#' @details
#'   `seq.integer64` coerces its arguments `from`, `to`, and `by` to `integer64`. Consistency
#'   with [seq()] is typically maintained, though results may differ when mixing `integer64` and
#'   `double` inputs, for the same reason that any arithmetic with these mixed types can be
#'   ambiguous. Whereas `seq(1L, 10L, length.out=8L)` can back up to double storage to give an
#'   exact result, this not possible for generic inputs `seq(i64, dbl, length.out=n)`.
#'
#' @returns An integer64 vector with the generated sequence
#'
#' @keywords classes manip
#' @seealso [c.integer64()] [rep.integer64()]
#'   [as.data.frame.integer64()] [integer64()]
#' @examples
#' seq(as.integer64(1), 12, 2)
#' seq(as.integer64(1), by=2, length.out=6)
#'
#' # truncation rules
#' seq(as.integer64(1), 10, by=1.5)
#' seq(as.integer64(1), 10, length.out=5)
#' @export
seq.integer64 = function(from=NULL, to=NULL, by=NULL, length.out=NULL, along.with=NULL, ...) {
  if (!is.null(along.with)) return(seq.integer64(from, to, by=by, length.out=length(along.with)))

  n_args = 4L - is.null(from) - is.null(to) - is.null(by) - is.null(length.out)

  if (n_args == 4L)
    stop("too many arguments")

  if (n_args == 1L) {
    one = as.integer64(1L)
    if (!is.null(from)) return(one:from)
    if (!is.null(to)) return(one:to)
    if (!is.null(length.out)) {
      if (length.out < 0L)
        stop("'length.out' must be a non-negative number")
      if (length.out == 0L)
        return(integer64())
      return(one:length.out)
    }
    # match seq(by=integer(1))
    return(one)
  }

  if (n_args == 2L) {
    if (!is.null(length.out)) {
      if (length.out == 0L)
        return(integer64())
      if (length.out < 0L)
        stop("'length.out' must be a non-negative number")
      # do before mixing with from/to to avoid integer64/double fraction arithmetic
      if (is.double(length.out) && length.out %% 1L != 0L)
        length.out = ceiling(length.out)
      if (!is.null(from))
        return(seq.integer64(from, from+length.out-1L, by=1L))
      if (!is.null(to))
        return(seq.integer64(to-length.out+1L, to, by=1L))
      if (!is.null(by))
        return(seq.integer64(as.integer64(1L), by=by, length.out=length.out))
    }
    if (!is.null(from) && !is.null(to)) return(seq.integer64(from, to, by=sign(to - from)))
    if (!is.null(from) && !is.null(by)) return(seq.integer64(from, 1L, by=by))
    return(seq.integer64(as.integer64(1L), to, by=by))
  }

  # match base behavior for seq(1, 2, length.out=1.5)
  if (!is.null(length.out) && is.double(length.out))
    length.out = ceiling(length.out)

  if (!is.null(by))
    by = as.integer64(by) # always coerce, e.g. for nanotime classes, #297

  if (is.null(from)) {
    from = to - (length.out - 1L) * by
  } else if (is.null(by)) {
    if (length.out == 1L)
      return(as.integer64(from))
    by = as.integer64((to - from) / (length.out - 1L))
  } else if (is.null(length.out)) {
    if (to != from && by == 0L)
      stop("invalid '(to - from)/by'")
    if (to == from)
      return(as.integer64(from))
    if (sign(to - from) != sign(by))
      stop("wrong sign in 'by' argument'")
    length.out = (to - from) / by + 1L
  }
  if (length.out < 0L)
    stop("'length.out' must be a non-negative number")
  ret = .Call(C_seq_integer64, as.integer64(from), by, double(as.integer(length.out)))
  oldClass(ret) = "integer64"
  ret
}

#' @rdname format.integer64
#' @export
sign.integer64 = function(x) {
  a = attributes(x)
  ret = .Call(C_sign_integer64, x, double(length(x)))
  attributes(ret) = a
  ret
}

#' @rdname format.integer64
#' @export
abs.integer64 = function(x) {
  a = attributes(x)
  ret = .Call(C_abs_integer64, x, double(length(x)))
  attributes(ret) = a
  ret
}

#' @rdname format.integer64
#' @export
sqrt.integer64 = function(x) {
  a = attributes(x)
  ret = .Call(C_sqrt_integer64, x, double(length(x)))
  a$class = minusclass(a$class, "integer64")
  attributes(ret) = a
  ret
}

#' @rdname format.integer64
#' @export
log.integer64 = function(x, base=NULL) {
  a = attributes(x)
  l.x = length(x)
  l.base = length(base)
  l = if (l.x==0L || (!is.null(base) && l.base==0L)) 0L else max(l.base, l.x)
  ret = double(l)
  if (!l.x) {
  } else if (is.null(base)) {
    .Call(C_log_integer64, x, ret)
  } else if (length(base)==1L) {
    .Call(C_logbase_integer64, x, as.double(base), ret)
  } else {
    .Call(C_logvect_integer64, x, as.double(base), ret)
  }
  a$class = minusclass(a$class, "integer64")
  attributes(ret) = a
  ret
}

#' @rdname format.integer64
#' @export
log10.integer64 = function(x) {
  a = attributes(x)
  ret = .Call(C_log10_integer64, x, double(length(x)))
  a$class = minusclass(a$class, "integer64")
  attributes(ret) = a
  ret
}

#' @rdname format.integer64
#' @export
log2.integer64 = function(x) {
  a = attributes(x)
  ret = .Call(C_log2_integer64, x, double(length(x)))
  a$class = minusclass(a$class, "integer64")
  attributes(ret) = a
  ret
}

#' @rdname format.integer64
#' @export
trunc.integer64 = function(x, ...) x
#' @rdname format.integer64
#' @export
floor.integer64 = function(x) x
#' @rdname format.integer64
#' @export
ceiling.integer64 = function(x) x

#' @rdname format.integer64
#' @export
signif.integer64 = function(x, digits=6L) x

#' @rdname format.integer64
#' @export
scale.integer64 = function(x, center = TRUE, scale = TRUE) {
  scale(.as_double_integer64(x, keep.attributes=TRUE), center=center, scale=scale)
}

#' @rdname format.integer64
#' @export
round.integer64 = function(x, digits=0L) {
  if (digits >= 0L) return(x)
  a = attributes(x)
  b = as.integer64(10L^round(-digits))
  b2 = b%/%2L
  ret = ((x + b2 - (((x - b2)%%(2L*b)) == 0L)) %/% b)*b
  attributes(ret) = a
  ret
}

#' @rdname sum.integer64
#' @export
any.integer64 = function(..., na.rm=FALSE) {
  l = list(...)
  if (length(l) == 1L)
    return(.Call(C_any_integer64, l[[1L]], na.rm, logical(1L)))
  any_elts = vapply(l, FUN.VALUE=logical(1L), function(e) {
    if (is.integer64(e)) {
      .Call(C_any_integer64, e, na.rm, logical(1L))
    } else {
      any(e, na.rm=na.rm)
    }
  })
  any(any_elts, na.rm=na.rm)
}

#' @rdname sum.integer64
#' @export
all.integer64 = function(..., na.rm=FALSE) {
  l = list(...)
  if (length(l) == 1L)
    return(.Call(C_all_integer64, l[[1L]], na.rm, logical(1L)))
  all_elts = vapply(l, FUN.VALUE=logical(1L), function(e) {
    if (is.integer64(e)) {
      .Call(C_all_integer64, e, na.rm, logical(1L))
    } else {
      all(e, na.rm=na.rm)
    }
  })
  all(all_elts, na.rm=na.rm)
}

#' @rdname sum.integer64
#' @export
sum.integer64 = function(..., na.rm=FALSE) {
  l = list(...)
  if (length(l) == 1L) {
    ret = .Call(C_sum_integer64, l[[1L]], na.rm, double(1L))
    oldClass(ret) = "integer64"
    ret
  } else {
    ret = vapply(l, FUN.VALUE=integer64(1L), function(e) {
      if (is.integer64(e)) {
        .Call(C_sum_integer64, e, na.rm, double(1L))
      } else {
        as.integer64(sum(e, na.rm=na.rm))
      }
    })
    oldClass(ret) = "integer64"
    sum(ret, na.rm=na.rm)
  }
}

#' @rdname sum.integer64
#' @export
prod.integer64 = function(..., na.rm=FALSE) {
  l = list(...)
  if (length(l) == 1L) {
    ret = .Call(C_prod_integer64, l[[1L]], na.rm, double(1L))
    oldClass(ret) = "integer64"
    ret
  } else {
    ret <- vapply(l, FUN.VALUE=integer64(1L), function(e) {
      if (is.integer64(e)) {
        .Call(C_prod_integer64, e, na.rm, double(1L))
      } else {
        as.integer64(prod(e, na.rm=na.rm))
      }
    })
    oldClass(ret) = "integer64"
    prod(ret, na.rm=na.rm)
  }
}

# not exactly analogous to anyNA, but convenient for min/max/range
has_no_values = function(x, na.rm) {
  if (!length(x)) return(TRUE)
  if (!na.rm) return(FALSE)
  if (is.integer64(x)) {
    .Call(C_r_ram_integer64_all_na, x=x)
  } else {
    all(is.na(x))
  }
}

#' @rdname sum.integer64
#' @export
min.integer64 = function(..., na.rm=FALSE) {
  l = list(...)
  na.rm = isTRUE(na.rm)
  ret = NULL
  no_values = NULL
  
  if (length(l) == 1L) {
    if (length(l[[1]]) > 0L) {
      ret = .Call(C_min_integer64, l[[1L]], na.rm, double(1L))
      oldClass(ret) = "integer64"
    }
  } else {
    ret = vapply(Filter(length, l), FUN.VALUE=integer64(1L), function(e) {
      if (is.integer64(e)) {
        .Call(C_min_integer64, e, na.rm, double(1L))
      } else {
        suppressWarnings(as.integer64(min(e, na.rm=na.rm)))
      }
    })
    oldClass(ret) = "integer64"
    no_values = has_no_values(ret, na.rm)
    if (!no_values) {
      ret = min(ret, na.rm=na.rm)
      no_values = NULL
    }
  }
  if (is.null(no_values))
    no_values = has_no_values(ret, na.rm)
  if (no_values) {
    ret = lim.integer64()[2L]
    warning("no non-NA value, returning the highest possible integer64 value +", ret)
  }
  ret
}

#' @rdname sum.integer64
#' @export
max.integer64 = function(..., na.rm=FALSE) {
  l = list(...)
  na.rm = isTRUE(na.rm)
  ret = NULL
  no_values = NULL

  if (length(l) == 1L) {
    if (length(l[[1]]) > 0L) {
      ret = .Call(C_max_integer64, l[[1L]], na.rm, double(1L))
      oldClass(ret) = "integer64"
    }
  } else {
    ret = vapply(Filter(length, l), FUN.VALUE=integer64(1L), function(e) {
      if (is.integer64(e)) {
        .Call(C_max_integer64, e, na.rm, double(1L))
      } else {
        suppressWarnings(as.integer64(max(e, na.rm=na.rm)))
      }
    })
    oldClass(ret) = "integer64"
    no_values = has_no_values(ret, na.rm)
    if (!no_values) {
      ret = max(ret, na.rm=na.rm)
      no_values = NULL
    }
  }
  if (is.null(no_values))
    no_values = has_no_values(ret, na.rm)
  if (no_values) {
    ret = lim.integer64()[1L]
    warning("no non-NA value, returning the lowest possible integer64 value ", ret)
  }
  ret
}

#' @rdname sum.integer64
#' @export
range.integer64 = function(..., na.rm=FALSE, finite=FALSE) {
  l = list(...)
  if (isTRUE(finite)) {
    na.rm = TRUE
  } else {
    na.rm = isTRUE(na.rm)
  }
  ret = NULL
  no_values = NULL
  
  if (length(l) == 1L) {
    if (length(l[[1]]) > 0L) {
      ret = .Call(C_range_integer64, l[[1L]], na.rm, double(2L))
      oldClass(ret) = "integer64"
    }
  } else {
    ret = vapply(Filter(length, l), FUN.VALUE=integer64(2L), function(e) {
      if (is.integer64(e)) {
        .Call(C_range_integer64, e, na.rm, double(2L))
      } else {
        suppressWarnings(as.integer64(range(e, na.rm=na.rm)))
      }
    })
    oldClass(ret) = "integer64"
    no_values = has_no_values(ret, na.rm)
    if (!no_values) {
      ret = range(ret, na.rm=na.rm)
      no_values = NULL
    }
  }
  if (is.null(no_values))
    no_values = has_no_values(ret, na.rm)
  if (no_values) {
    ret = c(lim.integer64()[2L], lim.integer64()[1L])
    warning("no non-NA value, returning c(+", ret[1L], ", ", ret[2L], ")")
  }
  ret
}

#' @rdname sum.integer64
#' @export
lim.integer64 = function() {
  ret = .Call(C_lim_integer64, double(2L))
  oldClass(ret) = "integer64"
  ret
}

#' @rdname cumsum.integer64
#' @export
diff.integer64 = function(x, lag=1L, differences=1L, ...) {
  lag = as.integer(lag)
  n = length(x)
  d = differences <- as.integer(differences)
  while (d > 0L) {
    n <- n - lag
    if (n <= 0L) {
      ret <- double()
      break
    }
    # not assigning ret<-.Call in the following is intended because faster
    if (d==differences) {
      ret <- double(n)
      .Call(C_diff_integer64, x, as.integer64(lag), as.integer64(n), ret)
    } else {
      .Call(C_diff_integer64, ret, as.integer64(lag), as.integer64(n), ret)
    }
    d <- d - 1L
  }
  # length of ret is only change once here
  length(ret) = n
  oldClass(ret) = "integer64"
  ret
}

#' @rdname cumsum.integer64
#' @export
cummin.integer64 = function(x) {
  ret = .Call(C_cummin_integer64, x, double(length(x)))
  oldClass(ret) = "integer64"
  ret
}
#' @rdname cumsum.integer64
#' @export
cummax.integer64 = function(x) {
  ret = .Call(C_cummax_integer64, x, double(length(x)))
  oldClass(ret) = "integer64"
  ret
}

#' @rdname cumsum.integer64
#' @export
cumsum.integer64 = function(x) {
  ret = .Call(C_cumsum_integer64, x, double(length(x)))
  oldClass(ret) = "integer64"
  ret
}

#' @rdname cumsum.integer64
#' @export
cumprod.integer64 = function(x) {
  ret = .Call(C_cumprod_integer64, x, double(length(x)))
  oldClass(ret) = "integer64"
  ret
}

#' @rdname format.integer64
#' @export
is.na.integer64 = function(x) {
  a = attributes(x)
  ret = .Call(C_isna_integer64, x, logical(length(x)))
  a$class = minusclass(a$class, "integer64")
  attributes(ret) = a
  ret
}

#' @rdname format.integer64
#' @export
is.finite.integer64 = function(x) !is.na(x)
#' @rdname format.integer64
#' @export
is.infinite.integer64 = function(x) rep(FALSE, length(x))
#' @rdname format.integer64
#' @export
is.nan.integer64 = function(x) rep(FALSE, length(x))

# as.vector.integer64 removed as requested by the CRAN maintainer
# as.vector.integer64 <- function(x, mode="any") {
  # ret <- NextMethod()
  # if (mode=="any")
    # oldClass(ret) <- "integer64"
  # ret
# }

# bug in R does not dispatch
#' @exportS3Method is.vector integer64
is.vector.integer64 = function(x, mode="any") {
  cl = minusclass(oldClass(x), "integer64")
  a = setdiff(names(attributes(x)), c("class", "names"))
  is.na(match(mode, c("any", "integer64"))) && !length(cl) && !length(a)
}

#' @rdname as.character.integer64
#' @export
as.list.integer64 = function(x, ...) {
  ret = NextMethod("as.list", x, ...)
  .Call(C_as_list_integer64, ret)
}


#' @exportS3Method anyNA integer64
anyNA.integer64 = function(x, recursive) {
  .Call(C_r_ram_integer64_any_na, x=x)
}

# These tests were previously kept as tests under \examples{\dontshow{...}}.
#   Converted to "proper" unit tests for clarity, after making them more
#   canonical within {testthat}, e.g. better capturing expected warnings,
#   changing stopifnot(identical(...)) to expect_identical(...).

test_that("identical.integer64", {
  i64 = NA_real_
  class(i64) = "integer64"
  expect_identical(unclass(i64 - 1.0), unclass(i64 + 1.0))
  expect_identical(i64 - 1.0, i64 + 1.0)
  expect_false(identical.integer64(i64 - 1.0, i64 + 1.0))
})

test_that("dispatch of 'c' method", {
  expect_true(identical.integer64(c(integer64(0L), NA), as.integer64(NA)))
})

test_that("Dispatch on the second argument fails and we want to be notified once that changes", {
  expect_false(identical.integer64(c(NA, integer64(0L)), as.integer64(NA)))
})

test_that("Minus and plus", {
  d64 = c(
    -.Machine$double.base^.Machine$double.digits,
    -.Machine$integer.max,
    -1.0, 0.0, 1.0,
    .Machine$integer.max,
    .Machine$double.base^.Machine$double.digits
  )
  i64 = as.integer64(d64)
  expect_true(identical.integer64(i64 - 1.0 + 1.0, i64))
  expect_true(identical.integer64(i64 + 1.0 - 1.0, i64))
})

test_that("Minus and plus edge cases and 'rev'", {
  # UBSAN signed integer overflow expected for type 'long long int'
  # This is a false UBSAN alarm because overflow is detected and NA returned
  expect_warning(
    expect_true(
        identical.integer64(lim.integer64() + 1.0 - 1.0,
        c(lim.integer64()[1L], NA))
    ),
    "NAs produced by integer64 overflow",
    fixed = TRUE
  )
  expect_warning(
    expect_true(
        identical.integer64(rev(lim.integer64()) - 1.0 + 1.0,
        c(lim.integer64()[2L], NA))
    ),
    "NAs produced by integer64 overflow",
    fixed = TRUE
  )
})

test_that("'range.integer64', multiplication, integer division, sqrt, power, and log", {
  i64 = integer64(63L)
  i64[1L] = 1.0
  for (i in 2:63)
    i64[i] = 2.0 * i64[i-1L]
  expect_true(identical.integer64(i64 * rev(i64), rep(i64[63L], 63L)))
  for (i in 63:2)
    i64[i-1L] = i64[i] %/% 2.0
  expect_true(identical.integer64(i64 * rev(i64), rep(i64[63L], 63L)))
  for (i in 63:2)
    i64[i-1L] = i64[i] / 2.0
  expect_true(identical.integer64(i64 * rev(i64), rep(i64[63L], 63L)))
  expect_true(identical.integer64(
    c(
      -i64[63L] - (i64[63L] - 1.0),
      i64[63L] + (i64[63L] - 1.0)
    ),
    lim.integer64()
  ))

  expect_true(identical.integer64(i64[-1L] %/%2.0 * as.integer64(2L), i64[-1L]))
  expect_true(identical.integer64(i64[-1L] %/%2L * as.integer64(2L), i64[-1L]))
  expect_true(identical.integer64(i64[-1L] / 2.0 * as.integer64(2L), i64[-1L]))
  expect_true(identical.integer64(i64[-1L] / 2.0 * as.integer64(2L), i64[-1L]))

  expect_true(identical.integer64(i64[-63L] * 2.0 %/% 2.0, i64[-63L]))
  expect_true(identical.integer64(i64[-63L] * 2L %/% 2L, i64[-63L]))
  expect_true(identical.integer64(as.integer64(i64[-63L] * 2.0 / 2.0), i64[-63L]))
  expect_true(identical.integer64(as.integer64(i64[-63L] * 2L / 2L), i64[-63L]))

  expect_true(identical.integer64(
    as.integer64(sqrt(
      i64[-1L][c(FALSE, TRUE)]) * sqrt(i64[-1L][c(FALSE, TRUE)]
    )),
    i64[-1L][c(FALSE, TRUE)]
  ))

  expect_true(identical.integer64(as.integer64(2L) ^ (0:62), i64))
  expect_true(identical.integer64(as.integer64(0:62), as.integer64(round(log2(i64)))))
  expect_true(identical.integer64(
    as.integer64(round(log(as.integer64(2L)^(0:62), 2.0))),
    as.integer64(0:62)
  ))
  expect_true(identical.integer64(
    as.integer64(round(log(as.integer64(3L)^(0:39), 3.0))),
    as.integer64(0:39)
  ))
  expect_true(identical.integer64(
    as.integer64(round(log(as.integer64(10L)^(0:18), 10.0))),
    as.integer64(0:18)
  ))
  expect_true(identical.integer64(
    as.integer64(round(log10(as.integer64(10L)^(0:18)))),
    as.integer64(0:18)
  ))

  expect_true(identical.integer64(
    (as.integer64(2L)^(1:62))^(1.0/1:62),
    as.integer64(rep(2.0, 62L))
  ))
  expect_true(identical.integer64(
    (as.integer64(3L)^(1:39))^(1.0/1:39),
    as.integer64(rep(3.0, 39L))
  ))
  expect_true(identical.integer64(
    (as.integer64(10L)^(1:18))^(1.0/1:18),
    as.integer64(rep(10.0, 18L))
  ))
})

test_that("c and rep", {
  expect_true(identical.integer64(
    as.integer64(rep(1:3, 1:3)),
    rep(as.integer64(1:3), 1:3)
  ))
  expect_true(identical.integer64(
    as.integer64(rep(1:3, 3L)),
    rep(as.integer64(1:3), 3L)
  ))

  x = rep(NA_real_, 3L)
  class(x) = "integer64"
  x = x + -1:1
  expect_true(identical.integer64(rep(x, 3L), c(x, x, x)))
  expect_true(identical.integer64(
    c.integer64(list(x, x, x), recursive=TRUE),
    c(x, x, x)
  ))
})

test_that("seq", {
  expect_true(identical.integer64(
    seq(as.integer64(1L), 10.0, 2.0),
    as.integer64(seq(1.0, 10.0, 2.0))
  ))
  expect_true(identical.integer64(
    seq(as.integer64(1L), by=2.0, length.out=5.0),
    as.integer64(seq(1.0, by=2.0, length.out=5.0))
  ))
  expect_true(identical.integer64(
    seq(as.integer64(1L), by=2.0, length.out=6.0),
    as.integer64(seq(1.0, by=2.0, length.out=6.0))
  ))
  expect_true(identical.integer64(
    seq.integer64(along.with=3:5),
    as.integer64(seq(along.with=3:5))
  ))
  expect_true(identical.integer64(
    seq(as.integer64(1L), to=-9.0),
    as.integer64(seq(1.0, to=-9.0))
  ))
})

test_that("cbind and rbind", {
  x = rep(as.integer64(1:3), 2L)
  dim(x) = c(3L, 2L)
  expect_true(identical.integer64(cbind(as.integer64(1:3), 1:3), x))
  expect_true(identical.integer64(rbind(as.integer64(1:3), 1:3), t(x)))
})

test_that("Coercion", {
  expect_identical(
    as.double(as.integer64(c(NA, seq(0.0, 9.0, 0.25)))),
    as.double(as.integer(c(NA, seq(0.0, 9.0, 0.25))))
  )
  expect_identical(
    as.character(as.integer64(c(NA, seq(0.0, 9.0, 0.25)))),
    as.character(as.integer(c(NA, seq(0.0, 9.0, 0.25))))
  )
  expect_identical(
    as.integer(as.integer64(c(NA, seq(0.0, 9.0, 0.25)))),
    as.integer(c(NA, seq(0.0, 9.0, 0.25)))
  )
  expect_identical(
    as.logical(as.integer64(c(NA, seq(0.0, 9.0, 0.25)))),
    as.logical(as.integer(c(NA, seq(0.0, 9.0, 0.25))))
  )
  expect_identical(
    as.integer(as.integer64(c(NA, FALSE, TRUE))),
    as.integer(c(NA, FALSE, TRUE))
  )
  expect_identical(
    as.integer64(as.integer(as.integer64(-9:9))),
    as.integer64(-9:9)
  )
  expect_identical(
    as.integer64(as.double(as.integer64(-9:9))),
    as.integer64(-9:9)
  )
  expect_identical(
    as.integer64(as.character(as.integer64(-9:9))),
    as.integer64(-9:9)
  )
  expect_identical(
    as.integer64(as.character(lim.integer64())),
    lim.integer64()
  )
})

test_that("Logical operators", {
  expect_true(identical.integer64(
    !c(NA, -1:1),
    !c(as.integer64(NA), -1:1)
  ))

  xi = rep(c(NA, -1:1), 4L)
  xi64 = as.integer64(xi)
  yi = rep(c(NA, -1:1), each=4L)
  yi64 = as.integer64(yi)

  expect_true(identical.integer64(xi64 & yi64, xi & yi))
  expect_true(identical.integer64(xi64 | yi64, xi | yi))
  expect_true(identical.integer64(xor(xi64, yi64), xor(xi, yi)))
})

test_that("Comparison operators", {
  xi = rep(c(NA, -1:1), 4L)
  xi64 = as.integer64(xi)
  yi = rep(c(NA, -1:1), each=4L)
  yi64 = as.integer64(yi)

  expect_true(identical.integer64(xi64 == yi64, xi == yi))
  expect_true(identical.integer64(xi64 != yi64, xi != yi))
  expect_true(identical.integer64(xi64 > yi64, xi > yi))
  expect_true(identical.integer64(xi64 >= yi64, xi >= yi))
  expect_true(identical.integer64(xi64 < yi64, xi < yi))
  expect_true(identical.integer64(xi64 <= yi64, xi <= yi))
})

test_that("Vector functions", {
  xi = c(NA, -1:1)
  xi64 = as.integer64(xi)

  expect_true(identical.integer64(is.na(xi64), is.na(xi)))
  expect_true(identical.integer64(format(xi64), format(xi)))
  expect_true(identical.integer64(abs(xi64), as.integer64(abs(xi))))
  expect_true(identical.integer64(sign(xi64), as.integer64(sign(xi))))
  expect_true(identical.integer64(ceiling(xi64), as.integer64(ceiling(xi))))
  expect_true(identical.integer64(floor(xi64), as.integer64(floor(xi))))
  expect_true(identical.integer64(trunc(xi64), as.integer64(trunc(xi))))
  expect_true(identical.integer64(signif(xi64), xi64))
})

test_that("Summary functions", {
  expect_identical(all(as.integer64(1L)), all(1L))
  expect_identical(all(as.integer64(0L)), all(0L))
  expect_identical(all(NA_integer64_), all(NA_integer_))
  expect_identical(all(NA_integer64_, na.rm=TRUE), all(NA_integer_, na.rm=TRUE))
  expect_identical(all(as.integer64(1L), NA), all(1L, NA))
  expect_identical(all(as.integer64(0L), NA), all(0L, NA))
  expect_identical(all(as.integer64(1L), NA, na.rm=TRUE), all(1L, NA, na.rm=TRUE))
  expect_identical(all(as.integer64(0L), NA, na.rm=TRUE), all(0L, NA, na.rm=TRUE))
  expect_identical(all(as.integer64(c(1L, NA))), all(c(1L, NA_integer_)))
  expect_identical(all(as.integer64(c(0L, NA))), all(c(0L, NA_integer_)))
  expect_identical(all(as.integer64(c(1L, NA)), na.rm=TRUE), all(c(1L, NA_integer_), na.rm=TRUE))
  expect_identical(all(as.integer64(c(0L, NA)), na.rm=TRUE), all(c(0L, NA_integer_), na.rm=TRUE))

  expect_identical(any(as.integer64(1L)), any(1L))
  expect_identical(any(as.integer64(0L)), any(0L))
  expect_identical(any(NA_integer64_), any(NA_integer_))
  expect_identical(any(NA_integer64_, na.rm=TRUE), any(NA_integer_, na.rm=TRUE))
  expect_identical(any(as.integer64(1L), NA), any(1L, NA))
  expect_identical(any(as.integer64(0L), NA), any(0L, NA))
  expect_identical(any(as.integer64(1L), NA, na.rm=TRUE), any(1L, NA, na.rm=TRUE))
  expect_identical(any(as.integer64(0L), NA, na.rm=TRUE), any(0L, NA, na.rm=TRUE))
  expect_identical(any(as.integer64(c(1L, NA))), any(c(1L, NA_integer_)))
  expect_identical(any(as.integer64(c(0L, NA))), any(c(0L, NA_integer_)))
  expect_identical(any(as.integer64(c(1L, NA)), na.rm=TRUE), any(c(1L, NA_integer_), na.rm=TRUE))
  expect_identical(any(as.integer64(c(0L, NA)), na.rm=TRUE), any(c(0L, NA_integer_), na.rm=TRUE))

  xd = c(2.0, 3.0, NA)
  xi64 = as.integer64(xd)

  expect_true(identical.integer64(
    as.integer64(sum(xd)),
    sum(xi64)
  ))
  expect_true(identical.integer64(
    as.integer64(sum(xd, na.rm=TRUE)),
    sum(xi64, na.rm=TRUE)
  ))
  expect_true(identical.integer64(
    as.integer64(sum(xd)),
    sum(xi64)
  ))
  expect_true(identical.integer64(
    as.integer64(sum(xd, na.rm=TRUE)),
    sum(xi64, na.rm=TRUE)
  ))
  expect_true(identical.integer64(
    as.integer64(sum(2.0, 3.0, NA)),
    sum(as.integer64(2L), 3.0, NA)
  ))
  expect_true(identical.integer64(
    as.integer64(sum(2.0, 3.0, NA, na.rm=TRUE)),
    sum(as.integer64(2L), 3.0, NA, na.rm=TRUE)
  ))
  expect_true(identical.integer64(
    as.integer64(sum(2.0, 3.0, NA)),
    sum(as.integer64(2L), 3.0, NA)
  ))
  expect_true(identical.integer64(
    as.integer64(sum(2.0, 3.0, NA, na.rm=TRUE)),
    sum(as.integer64(2L), 3.0, NA, na.rm=TRUE)
  ))

  expect_true(identical.integer64(
    as.integer64(prod(xd)),
    prod(xi64)
  ))
  expect_true(identical.integer64(
    as.integer64(prod(xd, na.rm=TRUE)),
    prod(xi64, na.rm=TRUE)
  ))
  expect_true(identical.integer64(
    as.integer64(prod(xd)),
    prod(xi64)
  ))
  expect_true(identical.integer64(
    as.integer64(prod(xd, na.rm=TRUE)),
    prod(xi64, na.rm=TRUE)
  ))
  expect_true(identical.integer64(
    as.integer64(prod(2.0, 3.0, NA)),
    prod(as.integer64(2L), 3.0, NA)
  ))
  expect_true(identical.integer64(
    as.integer64(prod(2.0, 3.0, NA, na.rm=TRUE)),
    prod(as.integer64(2L), 3.0, NA, na.rm=TRUE)
  ))
  expect_true(identical.integer64(
    as.integer64(prod(2.0, 3.0, NA)),
    prod(as.integer64(2L), 3.0, NA)
  ))
  expect_true(identical.integer64(
    as.integer64(prod(2.0, 3.0, NA, na.rm=TRUE)),
    prod(as.integer64(2L), 3.0, NA, na.rm=TRUE)
  ))

  expect_true(identical.integer64(
    as.integer64(min(xd)),
    min(xi64)
  ))
  expect_true(identical.integer64(
    as.integer64(min(xd, na.rm=TRUE)),
    min(xi64, na.rm=TRUE)
  ))
  expect_true(identical.integer64(
    as.integer64(min(xd)),
    min(xi64)
  ))
  expect_true(identical.integer64(
    as.integer64(min(xd, na.rm=TRUE)),
    min(xi64, na.rm=TRUE)
  ))
  expect_true(identical.integer64(
    as.integer64(min(2.0, 3.0, NA)),
    min(as.integer64(2L), 3.0, NA)
  ))
  expect_warning(
    expect_warning(
      expect_true(identical.integer64(
        as.integer64(min(2.0, 3.0, NA, na.rm=TRUE)),
        min(as.integer64(2L), 3.0, NA, na.rm=TRUE)
      )),
      "no non-missing arguments to min; returning Inf",
      fixed = TRUE
    ),
    "NAs produced by integer64 overflow",
    fixed = TRUE
  )
  expect_true(identical.integer64(
    as.integer64(min(2.0, 3.0, NA)),
    min(as.integer64(2L), 3.0, NA)
  ))
  expect_warning(
    expect_warning(
      expect_true(identical.integer64(
        as.integer64(min(2.0, 3.0, NA, na.rm=TRUE)),
        min(as.integer64(2L), 3.0, NA, na.rm=TRUE)
      )),
      "no non-missing arguments to min; returning Inf",
      fixed = TRUE
    ),
    "NAs produced by integer64 overflow",
    fixed = TRUE
  )

  expect_true(identical.integer64(
    as.integer64(max(xd)),
    max(xi64)
  ))
  expect_true(identical.integer64(
    as.integer64(max(xd, na.rm=TRUE)),
    max(xi64, na.rm=TRUE)
  ))
  expect_true(identical.integer64(
    as.integer64(max(xd)),
    max(xi64)
  ))
  expect_true(identical.integer64(
    as.integer64(max(xd, na.rm=TRUE)),
    max(xi64, na.rm=TRUE)
  ))
  expect_true(identical.integer64(
    as.integer64(max(2.0, 3.0, NA)),
    max(as.integer64(2L), 3.0, NA)
  ))
  expect_warning(
    expect_warning(
      expect_true(identical.integer64(
        as.integer64(max(2.0, 3.0, NA, na.rm=TRUE)),
        max(as.integer64(2L), 3.0, NA, na.rm=TRUE)
      )),
      "no non-missing arguments to max; returning -Inf",
      fixed = TRUE
    ),
    "NAs produced by integer64 overflow",
    fixed = TRUE
  )
  expect_true(identical.integer64(
    as.integer64(max(2.0, 3.0, NA)),
    max(as.integer64(2L), 3.0, NA)
  ))
  expect_warning(
    expect_warning(
      expect_true(identical.integer64(
        as.integer64(max(2.0, 3.0, NA, na.rm=TRUE)),
        max(as.integer64(2L), 3.0, NA, na.rm=TRUE)
      )),
      "no non-missing arguments to max; returning -Inf",
      fixed = TRUE
    ),
    "NAs produced by integer64 overflow",
    fixed = TRUE
  )

  expect_true(identical.integer64(
    as.integer64(range(xd)),
    range(xi64)
  ))
  expect_true(identical.integer64(
    as.integer64(range(xd, na.rm=TRUE)),
    range(xi64, na.rm=TRUE)
  ))
  expect_true(identical.integer64(
    as.integer64(range(xd)),
    range(xi64)
  ))
  expect_true(identical.integer64(
    as.integer64(range(xd, na.rm=TRUE)),
    range(xi64, na.rm=TRUE)
  ))
  expect_true(identical.integer64(
    as.integer64(range(2.0, 3.0, NA)),
    range(as.integer64(2L), 3.0, NA)
  ))
  expect_warning(
    expect_warning(
      expect_warning(
        expect_true(identical.integer64(
          as.integer64(range(2.0, 3.0, NA, na.rm=TRUE)),
          range(as.integer64(2L), 3.0, NA, na.rm=TRUE)
        )),
        "no non-missing arguments to max; returning -Inf",
        fixed = TRUE
      ),
      "no non-missing arguments to min; returning Inf",
      fixed = TRUE
    ),
    "NAs produced by integer64 overflow",
    fixed = TRUE
  )
})

test_that("Cumulative functions", {
  xd = c(2.0, 3.0, NA, 1.0, 4.0)
  xi64 = as.integer64(xd)

  expect_true(identical.integer64(cumsum(xi64), as.integer64(cumsum(xd))))
  expect_true(identical.integer64(cumprod(xi64), as.integer64(cumprod(xd))))
  expect_true(identical.integer64(cummin(xi64), as.integer64(cummin(xd))))
  expect_true(identical.integer64(cummax(xi64), as.integer64(cummax(xd))))
})

test_that("diff", {
  d64 = diffinv(rep(.Machine$integer.max, 100L), lag=2.0, differences=2L)
  i64 = as.integer64(d64)
  expect_identical(
    diff(d64, lag=2L, differences=2L),
    as.double(diff(i64, lag=2L, differences=2L))
  )
})

test_that("integer64 coercion to/from other types work", {
  # from integer64
  expect_identical(as.logical(as.integer64(0:1)), c(FALSE, TRUE))
  expect_identical(as.integer(as.integer64(1:10)), 1:10)
  expect_identical(as.character(as.integer64(1:10)), as.character(1:10))
  expect_identical(as.double(as.integer64(1:10)), as.double(1:10))
  expect_identical(as.numeric(as.integer64(1:10)), as.numeric(1:10))

  # to integer64
  expect_identical(as.integer64(TRUE), as.integer64(1L))
  expect_identical(as.integer64(as.character(1:10)), as.integer64(1:10))
  expect_identical(as.integer64(as.double(1:10)), as.integer64(1:10))
  expect_identical(as.integer64(NULL), as.integer64())
  x = as.integer64(1:10)
  expect_identical(as.integer64(x), x)

  # S4 version
  expect_identical(methods::as(as.character(1:10), "integer64"), as.integer64(1:10))
  expect_identical(methods::as(as.integer64(1:10), "character"), as.character(1:10))

  # now for NA
  expect_identical(as.logical(NA_integer64_), NA)
  expect_identical(as.integer(NA_integer64_), NA_integer_)
  expect_identical(as.double(NA_integer64_), NA_real_)
  expect_identical(as.character(NA_integer64_), NA_character_)
  expect_identical(as.integer64(NA), NA_integer64_)
  expect_identical(as.integer64(NA_integer_), NA_integer64_)
  expect_identical(as.integer64(NA_real_), NA_integer64_)
  expect_identical(as.integer64(NA_character_), NA_integer64_)
})

test_that("S3 class basics work", {
  x = as.integer64(1:10)
  expect_s3_class(x, "integer64")
  expect_true(is.integer64(x))

  length(x) = 11L
  expect_length(x, 11L)
  expect_identical(x[11L], as.integer64(0L))
})

test_that("indexing works", {
  x = as.integer64(1:10)

  x[1.0] = 2.0
  x[2L] = 3L
  expect_identical(x, as.integer64(c(2:3, 3:10)))

  x[[1.0]] = 3.0
  x[[2L]] = 4L
  expect_identical(x, as.integer64(c(3:4, 3:10)))

  expect_identical(x[3L], as.integer64(3L))
  expect_identical(x[[4L]], as.integer64(4L))

  names(x) = letters[1:10]
  expect_identical(x[c("b", "c")], x[2:3])
  expect_identical(x[["d"]], x[[4L]])
})

test_that("arithmetic & basic math works", {
  x = as.integer64(1:10)
  y = as.integer64(10:1)

  expect_identical(x + y, as.integer64(rep(11L, 10L)))
  expect_identical(y - x, as.integer64(seq(9L, -9L, by=-2L)))
  expect_identical(x * y, as.integer64(c(10L, 18L, 24L, 28L, 30L, 30L, 28L, 24L, 18L, 10L)))
  # output is double even though it fits in integer [and integer64]
  expect_identical(x[seq(2L, 10L, by=2L)] / 2L, as.double(1:5))
  expect_identical(x ^ 2L, as.integer64((1:10)^2L))
  expect_identical(-x, as.integer64(-(1:10)))

  expect_identical(x %/% 2L, as.integer64(c(0L, 1L, 1L, 2L, 2L, 3L, 3L, 4L, 4L, 5L)))
  expect_identical(x %% 2L, as.integer64(rep_len(c(1L, 0L), 10L)))

  expect_identical(sign(x - 6L), as.integer64(rep(c(-1L, 0L, 1L), c(5L, 1L, 4L))))
  expect_identical(abs(x - 6.0), as.integer64(c(5:0, 1:4)))

  expect_identical(sqrt(as.integer64(c(0L, 1L, 4L, 9L))), as.numeric(0:3))
  expect_identical(log(x), log(as.numeric(x)))
  expect_identical(log(as.integer64(c(1L, 2L, 4L, 8L)), base=2L), as.numeric(0:3))
  expect_identical(log2(as.integer64(c(1L, 2L, 4L, 8L))), as.numeric(0:3))
  # TODO(#48): Improve the numerical precision here.
  expect_identical(log10(as.integer64(c(1L, 10L, 100L, 1000L))), as.numeric(0:3), tolerance=1e-7)

  expect_identical(trunc(x), x)
  expect_identical(floor(x), x)
  expect_identical(ceiling(x), x)
  expect_identical(signif(x), x)
  expect_identical(round(x), x)

  expect_identical(round(x, -1L), as.integer64(rep(c(0L, 10L), each=5L)))
})

test_that("basic statistics work", {
  x = as.integer64(1:10)

  expect_identical(sum(x), as.integer64(55L))
  expect_identical(sum(x, x), as.integer64(110L))
  expect_identical(prod(x), as.integer64(factorial(10L)))
  expect_identical(prod(x[1:5], x[6:10]), as.integer64(factorial(10L)))
  expect_identical(min(x), x[1L])
  expect_identical(min(x, as.integer64(0L)), as.integer64(0L))
  expect_identical(max(x), x[10L])
  expect_identical(max(x, as.integer64(11L)), as.integer64(11L))
  expect_identical(range(x), x[c(1L, 10L)])
  expect_identical(range(x, x+1L), c(x[1L], x[10L]+1L))

  expect_identical(diff(x), as.integer64(rep(1L, 9L)))

  expect_identical(cummin(x), as.integer64(rep(1L, 10L)))
  expect_identical(cummax(x), x)
  expect_identical(cumsum(x), as.integer64(choose(2:11, 2L)))
  expect_identical(cumprod(x), as.integer64(factorial(1:10)))
})

test_that("display methods work", {
  x = as.integer64(1:3)
  expect_identical(format(x), as.character(1:3))
  expect_output(print(x), "integer64.*\\s*1\\s*2\\s*3")
  expect_output(print(x[0L]), "integer64(0)", fixed=TRUE)
  expect_output(str(x), "integer64 [1:3] 1 2 3", fixed=TRUE)
})

test_that("vector builders of integer64 work", {
  x = as.integer64(1:3)
  expect_identical(c(x, FALSE), as.integer64(c(1:3, 0L)))
  expect_identical(c(x, 4:6), as.integer64(1:6))
  expect_identical(c(x, 4.0, 5.0, 6.0), as.integer64(1:6))
  expect_identical(c(x, as.integer64(4:6)), as.integer64(1:6))

  expect_identical(cbind(x, FALSE), matrix64(c(1:3, 0L, 0L, 0L), nrow=3L, ncol=2L))
  expect_identical(cbind(x, 4:6), matrix64(1:6, nrow=3L, ncol=2L))
  expect_identical(cbind(x, 0.0), matrix64(c(1:3, 0L, 0L, 0L), nrow=3L, ncol=2L))
  expect_identical(cbind(x, as.integer64(4:6)), matrix64(1:6, nrow=3L, ncol=2L))

  expect_identical(rbind(x, FALSE), matrix64(c(1:3, 0L, 0L, 0L), nrow=2L, ncol=3L, byrow=TRUE))
  expect_identical(rbind(x, 4:6), matrix64(1:6, nrow=2L, ncol=3L, byrow=TRUE))
  expect_identical(rbind(x, 0.0), matrix64(c(1:3, 0L, 0L, 0L), nrow=2L, ncol=3L, byrow=TRUE))
  expect_identical(rbind(x, as.integer64(4:6)), matrix64(1:6, nrow=2L, ncol=3L, byrow=TRUE))

  expect_identical(rep(x, 2L), c(x, x))
  expect_identical(rep(x, each=2L), as.integer64(c(1L, 1L, 2L, 2L, 3L, 3L)))

  expect_identical(x[1L]:x[3L], x)
  expect_identical(x[3L]:x[1L], x[3:1]) # rev() a separate method

  expect_identical(seq(x[1L], x[3L], by=1L), x)
  expect_identical(seq(x[1L], x[3L], by=x[1L]), x)
  expect_identical(seq(x[1L], to=10L, by=1L), as.integer64(1:10))
  expect_identical(seq(x[1L], to=11L, by=2L), as.integer64(c(1L, 3L, 5L, 7L, 9L, 11L)))
  # TODO(#47): More tests when the behavior is corrected.
})

# These tests were previously kept as tests under \examples{\dontshow{...}}.
#   Converted to "proper" unit tests for clarity, after making them more
#   canonical within {testthat}, e.g. better capturing expected warnings,
#   changing stopifnot(identical(...)) to expect_identical(...).
test_that("Old \\dontshow{} tests in ?format.integer64 continue working", {
  i <- -999:999
  for (s in -3:3){
    r <- as.integer64(round(as.integer(i), s))
    r64 <- round(as.integer64(i), s)
    expect_identical(r, r64)
  }
})

test_that("Old \\dontshow{} tests in ?extract.replace.integer64 continue working", {
  r <- c(runif64(1000L, lim.integer64()[1L], lim.integer64()[2L]), NA, -2:2)
  expect_identical(r, as.integer64(as.bitstring(r)))
})

test_that("empty inputs give empty outputs for arithmetic", {
  x = integer64(1L)
  y = integer64(0L)

  expect_identical(x+y, integer64())
  expect_identical(y+x, integer64())

  expect_identical(x-y, integer64())
  expect_identical(y-x, integer64())

  expect_identical(+y, integer64())
  expect_identical(-y, integer64())

  expect_identical(x*y, integer64())
  expect_identical(y*x, integer64())

  expect_identical(x/y, double())
  expect_identical(y/x, double())

  expect_identical(x^y, integer64())
  expect_identical(y^x, integer64())

  expect_identical(x %/% y, integer64())
  expect_identical(y %/% x, integer64())

  expect_identical(x%%y, integer64())
  expect_identical(y%%x, integer64())

  expect_identical(log(x, base=y), double())
  # TODO(#93): don't suppress this warning which is inconsistent with integer()
  expect_identical(suppressWarnings(log(y, base=x)), double())

  expect_identical(x==y, logical())
  expect_identical(y==x, logical())

  expect_identical(x!=y, logical())
  expect_identical(y!=x, logical())

  expect_identical(x>=y, logical())
  expect_identical(y>=x, logical())

  expect_identical(x<=y, logical())
  expect_identical(y<=x, logical())

  expect_identical(x>y, logical())
  expect_identical(y>x, logical())

  expect_identical(x<y, logical())
  expect_identical(y<x, logical())

  expect_identical(x&y, logical())
  expect_identical(y&x, logical())

  expect_identical(x|y, logical())
  expect_identical(y|x, logical())

  expect_identical(xor(x, y), logical())
  expect_identical(xor(y, x), logical())
})

test_that("semantics about mixed types for multiplication are respected", {
  int = 5L
  i64 = as.integer64(2L)
  dbl = 3.5

  # default: "old" semantics, to be deprecated
  expect_identical(i64 * dbl, as.integer64(7L))
  expect_identical(dbl * i64, as.integer64(6L))
  expect_identical(i64 * int, as.integer64(10L))
  expect_identical(int * i64, as.integer64(10L))
  expect_identical(i64 * i64, as.integer64(4L))

  skip_if_not_installed("withr") # only really for testing without testthat
  withr::with_options(list(integer64_semantics = "new"), {
    expect_identical(i64 * dbl, as.integer64(7L))
    expect_identical(dbl * i64, as.integer64(7L))
    expect_identical(i64 * int, as.integer64(10L))
    expect_identical(int * i64, as.integer64(10L))
    expect_identical(i64 * i64, as.integer64(4L))
  })
})

test_that("semantics about mixed types for division are respected", {
  int = 10L
  i64 = as.integer64(5L)
  dbl = 2.5

  # default: "old" semantics, to be deprecated
  expect_identical(i64 / dbl, 2.0)
  expect_identical(dbl / i64, 0.4)
  expect_identical(i64 / int, 0.5)
  expect_identical(int / i64, 2.0)
  expect_identical(i64 / i64, 1.0)

  skip_if_not_installed("withr") # only really for testing without testthat
  withr::with_options(list(integer64_semantics = "new"), {
    expect_identical(i64 / dbl, 2.0)
    expect_identical(dbl / i64, 0.5)
    expect_identical(i64 / int, 0.5)
    expect_identical(int / i64, 2.0)
    expect_identical(i64 / i64, 1.0)
  })
})

test_that("all.equal.integer64 reflects changes for vector scale= from all.equal.numeric", {
  # same test as for base R, multiplied by 1000 so the inputs are all integer64
  expect_identical(
    all.equal(
      as.integer64(c(1000L, 1000L)),
      as.integer64(c(1010L, 1010L)),
      scale = c(10.0, 10.0)
    ),
    "Mean scaled difference: 1"
  )
  # same test as for base R, multiplied by 1e9
  one_e9 = as.integer64(1000000000L)
  expect_true(all.equal(
    rep(one_e9, 5L),
    one_e9 + (-1L:3), # TODO(r-lib/lintr#): no 'L'
    scale = (1:5)*one_e9
  ))
})

test_that("all.equal works", {
  x = y = as.integer64(1L)

  expect_true(all.equal(x, x))

  class(y) = c("xx", "integer64")
  expect_match(all.equal(x, y), "target is integer64, current is xx", fixed=TRUE, all=FALSE)
  expect_match(all.equal(x[0L], x[1L]), "integer64: lengths.*differ", all=FALSE)

  class(y) = "integer64"
  attr(y, "xx") = "zz"
  expect_match(all.equal(x, y), "Attributes", fixed=TRUE)
  expect_no_match(
    expect_match(all.equal(x[0L], y), "integer64: lengths.*differ", all=FALSE),
    "Lengths:", fixed = TRUE
  )

  y = NA_integer64_
  expect_match(all.equal(x, y), "'is.NA' value mismatch", fixed=TRUE)

  x = as.integer64(1000000000L)
  expect_true(all.equal(x, x+1L))
  expect_true(all.equal(x, x+1L, tolerance=1.0e9)) # forcing scale=1
  expect_match(all.equal(x, x+100L), "Mean relative difference", fixed=TRUE)
  expect_match(all.equal(x, x+1L, scale=1.0), "Mean absolute difference", fixed=TRUE)
})

test_that("order basics work", {
  x = as.integer64(c(2L, 4L, 3L))
  expect_identical(order(x), c(1L, 3L, 2L))
  expect_identical(order(x, decreasing=TRUE), c(2L, 3L, 1L))

  x = c(x, NA_integer64_)
  expect_identical(order(x), c(1L, 3L, 2L, 4L))
  expect_identical(order(x, decreasing=TRUE), c(2L, 3L, 1L, 4L))
  expect_identical(order(x, na.last=FALSE), c(4L, 1L, 3L, 2L))
  expect_identical(order(x, na.last=FALSE, decreasing=TRUE), c(4L, 2L, 3L, 1L))
})

test_that("integer64 coercion to/from other types work for atomic vectors", {
  # from integer64
  i32 = 1:10
  i64 = as.integer64(i32)
  expect_identical(as.logical(as.integer64(0:1)), c(FALSE, TRUE))
  expect_identical(as.integer(i64), i32)
  expect_identical(as.character(i64), as.character(i32))
  expect_identical(as.double(i64), as.double(i32))
  expect_identical(as.numeric(i64), as.numeric(i32))

  # to integer64
  expect_identical(as.integer64(TRUE), as.integer64(1L))
  expect_identical(as.integer64(as.character(1:10)), i64)
  expect_identical(as.integer64(as.double(1:10)), i64)
  expect_identical(as.integer64(as.complex(1:10)), i64)
  expect_identical(as.integer64(as.raw(1:10)), i64)
  expect_identical(as.integer64(NULL), as.integer64())
  expect_identical(as.integer64(i64), i64)
})

test_that("integer64 coercion to/from other types works via S4 coercion", {
  expect_identical(methods::as(as.character(1:10), "integer64"), as.integer64(1:10))
  expect_identical(methods::as(as.factor(11:20), "integer64"), as.integer64(1:10))
  expect_identical(methods::as(as.ordered(11:20), "integer64"), as.integer64(1:10))
  expect_warning(
    expect_identical(methods::as(as.complex(1:10) + 1.0i, "integer64"), as.integer64(1:10)),
    "imaginary parts discarded in coercion"
  )
  expect_identical(methods::as(as.numeric(1:10), "integer64"), as.integer64(1:10))
  expect_identical(methods::as(as.integer(1:10), "integer64"), as.integer64(1:10))
  expect_identical(methods::as(as.raw(1:10), "integer64"), as.integer64(1:10))
  expect_identical(methods::as(as.logical(0:2), "integer64"), as.integer64(c(0L, 1L, 1L)))
  expect_identical(methods::as(as.integer64(1:10), "character"), as.character(1:10))
  expect_identical(methods::as(as.integer64(1:10), "factor"), as.factor(1:10))
  expect_identical(methods::as(as.integer64(1:10), "ordered"), as.ordered(1:10))
  expect_identical(methods::as(as.integer64(1:10), "numeric"), as.numeric(1:10))
  expect_identical(methods::as(as.integer64(1:10), "integer"), as.integer(1:10))
  expect_identical(methods::as(as.integer64(1:10), "logical"), as.logical(1:10))
})

test_that("integer64 coercion to/from other types works for NA", {
  expect_identical(as.logical(NA_integer64_), NA)
  expect_identical(as.integer(NA_integer64_), NA_integer_)
  expect_identical(as.double(NA_integer64_), NA_real_)
  expect_identical(as.character(NA_integer64_), NA_character_)
  expect_identical(as.integer64(NA), NA_integer64_)
  expect_identical(as.integer64(NA_integer_), NA_integer64_)
  expect_identical(as.integer64(NA_real_), NA_integer64_)
  expect_identical(as.integer64(NA_character_), NA_integer64_)
})

test_that("integer64 coercion to/from factor types works", {
  i32 = 1:10
  i64 = as.integer64(i32)
  expect_identical(as.factor(i64), as.factor(i32))
  expect_identical(as.ordered(i64), as.ordered(i32))
  expect_identical(as.integer64(as.factor(11:20)), as.integer64(1:10))
  expect_identical(as.integer64(as.ordered(11:20)), as.integer64(1:10))
})

test_that("integer64 coercion to/from time types works", {
  posixct = Sys.time()
  posixct = c(posixct, posixct + 100)
  posix_delta = difftime(posixct + 1000, posixct)

  expect_identical(
    as.integer64(posix_delta), 
    as.integer64(as.integer(posix_delta))
  )
  # as.integer.difftime does not work with `units`
  expect_identical(
    as.integer64(posix_delta, units="secs"), 
    as.integer64(as.numeric(posix_delta, units="secs"))
  )
  expect_identical(
    as.integer64(posix_delta, units="mins"), 
    as.integer64(as.numeric(posix_delta, units="mins"))
  )

  expect_identical(as.integer64(posixct), as.integer64(as.integer(posixct)))
  # as.integer.POSIXlt does not work properly
  expect_identical(
    as.integer64(as.POSIXlt(posixct)),
    as.integer64(as.numeric(as.POSIXlt(posixct)))
  )
  expect_identical(
    as.integer64(as.Date(posixct)),
    as.integer64(as.integer(as.Date(posixct)))
  )

  # S4 version
  expect_identical(methods::as(posix_delta, "integer64"), as.integer64(posix_delta))
  expect_identical(methods::as(posixct, "integer64"), as.integer64(posixct))
  expect_identical(methods::as(as.POSIXlt(posixct), "integer64"), as.integer64(as.POSIXlt(posixct)))
  expect_identical(methods::as(as.Date(posixct), "integer64"), as.integer64(as.Date(posixct)))
})

test_that("integer64 coercion from generic object works", {
  x = structure(
    as.integer64(1:10),
    class=c("otherClass", "integer64"),
    dim=c(2L, 5L),
    dimnames=list(LETTERS[1:2], letters[1:5]),
    otherAttr="some other attribute"
  )
  expect_identical(as.integer64(x), as.integer64(1:10))
})

test_that("integer64 coercion to/from other types work for R >=4.0.0", {
  skip_unless_r(">= 4.0.0")
  # Strange behavior due to British Standard Time, as observed e.g. on r-universe runners (#233)
  #   Possibly an R bug: https://stat.ethz.ch/pipermail/r-devel/2026-January/084326.html
  # See also https://github.com/r-lib/waldo/issues/230 to allow dropping 'else FALSE'
  ignore_attr = if ("BST" %in% attr(as.POSIXlt(0), "tzone")) "tzone" else FALSE

  # from integer64
  i32 = 1:10
  i64 = as.integer64(i32)

  expect_identical(as.complex(i64), as.complex(i32))
  expect_identical(as.raw(i64), as.raw(i32))
  expect_identical(methods::as(as.integer64(1:10), "complex"), as.complex(1:10))

  expect_identical(as.Date(i64), as.Date(as.numeric(i32)))
  expect_identical(as.Date(i64, origin=10), as.Date(as.numeric(i32), origin=10))
  expect_identical(as.POSIXct(i64), as.POSIXct(as.numeric(i32)))
  expect_identical(as.POSIXct(i64, origin=10), as.POSIXct(as.numeric(i32), origin=10))
  expect_identical(as.POSIXct(i64, tz="UTC", origin=10), as.POSIXct(as.numeric(i32), tz="UTC", origin=10))
  expect_identical(as.POSIXct(i64, tz="CET", origin=10), as.POSIXct(as.numeric(i32), tz="CET", origin=10))
  expect_identical(as.POSIXlt(i64), as.POSIXlt(i32), ignore_attr=ignore_attr)
  expect_identical(as.POSIXlt(i64, origin=10), as.POSIXlt(i32, origin=10))
  expect_identical(as.POSIXlt(i64, tz="UTC", origin=10), as.POSIXlt(i32, tz="UTC", origin=10))
  expect_identical(as.POSIXlt(i64, tz="CET", origin=10), as.POSIXlt(i32, tz="CET", origin=10))
  expect_error(as.difftime(i32), "need explicit units for numeric conversion", fixed=TRUE)
  expect_error(as.difftime(i64), "need explicit units for numeric conversion", fixed=TRUE)
  expect_identical(as.difftime(i64, units="secs"), as.difftime(i32, units="secs"))
  
  # S4 version
  expect_identical(methods::as(as.integer64(1:10), "raw"), as.raw(1:10))
  expect_identical(methods::as(as.integer64(1:10), "difftime"), as.difftime(1:10, units="secs"))
  expect_identical(methods::as(as.integer64(1:10), "POSIXct"), as.POSIXct(as.numeric(1:10)))
  expect_identical(methods::as(as.integer64(1:10), "POSIXlt"), as.POSIXlt(1:10), ignore_attr=ignore_attr)
  expect_identical(methods::as(as.integer64(1:10), "Date"), as.Date(as.numeric(1:10)))

})

test_that("integer64 coercion from character works for numbers near +/- 2^63", {
  expect_identical(
    as.character(as.integer64(c("-9223372036854775807", "-9223372036854775806", "9223372036854775806", "9223372036854775807", ""))),
    c("-9223372036854775807", "-9223372036854775806", "9223372036854775806", "9223372036854775807", NA)
  )
})

test_that("Conversion from hex is supported", {
  expect_identical(
    as.integer64(c("0x1", "0xF", "0x7FFFFFFFFFFFFFFF", "-0x1", "-0xF", "-0x7FFFFFFFFFFFFFFF", "0x0")),
    as.integer64(c(1, 15, "9223372036854775807", -1, -15, "-9223372036854775807", 0))
  )
})

test_that("as.integer64 can trim whitespace", {
  expect_identical(as.integer64("  123   "), as.integer64("123"))
  expect_identical(as.integer64("  123"), as.integer64("123"))
  expect_identical(as.integer64("123   "), as.integer64("123"))
  expect_identical(as.integer64(c("  1", "2  ", " 3 ")), as.integer64(1:3))
  expect_identical(as.integer64("  0xAAA   "), as.integer64("0xAAA"))
  expect_identical(as.integer64("  0xBBB"), as.integer64("0xBBB"))
  expect_identical(as.integer64("0xCCC   "), as.integer64("0xCCC"))
  expect_identical(as.integer64(c("  0x1", "0x2  ", " 0x3 ")), as.integer64(c("0x1", "0x2", "0x3")))
})

with_parameters_test_that(
  "base-10 edge cases return missing",
  expect_warning(
    expect_identical(as.integer64(string), rep(NA_integer64_, length(string))),
    "NAs introduced by coercion to integer64 range", fixed=TRUE
  ),
  string = list(
    strrep("9", 63L),
    "9223372036854775808",
    "-9223372036854775808",
    "999x",
    "-999x",
    c("9223372036854775809", "-9223372036854775809") # vector case from #175
  )
)

with_parameters_test_that(
  "hex edge cases return missing",
  expect_warning(
    expect_identical(as.integer64(string), NA_integer64_),
    "NAs introduced by coercion to integer64 range", fixed=TRUE
  ),
  string = c(
    "-0x8000000000000000",
    "0x8000000000000000",
    "0x",
    "-0x",
    "0xx",
    " 0x",
    "0x ",
    "0x0Z"
  )
)

test_that("as.integer64.double handles 2^63", {
  expect_warning(
    expect_identical(as.integer64(2^63), NA_integer64_),
    "NAs produced by integer64 overflow"
  )
})

test_that("S3 class basics work", {
  x = as.integer64(1:10)
  expect_s3_class(x, "integer64")
  expect_true(is.integer64(x))

  length(x) = 11L
  expect_length(x, 11L)
  expect_identical(x[11L], as.integer64(0L))
})

test_that("indexing works: [<-", {
  x = as.integer64(1:10)
  x_updated = as.integer64(c(2:3, 3:10))

  local({
    x[1.0] = 2.0
    x[2L] = 3L
    expect_identical(x, x_updated)
  })
  local({
    x[1.0] = 2L
    x[2L] = 3.0
    expect_identical(x, x_updated)
  })
  local({
    x[1.0] = 2L
    x[2L] = 3.0 + 0.0i
    expect_identical(x, as.complex(c(2:3, 3:10)))
  })
  # TODO(#44): remove this deprecated test
  local({
    x[1.0] = 2L
    x[2L] = "3"
    expect_identical(x, x_updated)
  })
  local({
    # TODO(#44): remove the option
    withr::local_options(list(bit64.promoteInteger64ToCharacter=TRUE))
    x[1.0] = 2L
    x[2L] = "3"
    expect_identical(x, as.character(c(2:3, 3:10)))
  })

  # local writes didn't edit 'x'
  expect_identical(x[1L], as.integer64(1L))
  expect_identical(x[[2L]], as.integer64(2L))
})

test_that("indexing works: [[<-", {
  x = as.integer64(1:10)
  x_updated = as.integer64(c(3:4, 3:10))

  local({
    x[[1.0]] = 3.0
    x[[2L]] = 4L
    expect_identical(x, x_updated)
  })
  local({
    x[[1.0]] = 3L
    x[[2L]] = 4.0
    expect_identical(x, x_updated)
  })
  local({
    x[[1.0]] = 3L
    x[[2L]] = 4.0 + 0.0i
    expect_identical(x, as.complex(c(3:4, 3:10)))
  })
  # TODO(#44): remove this deprecated test
  local({
    x[[1.0]] = 3L
    x[[2L]] = "4"
    expect_identical(x, x_updated)
  })
  local({
    # TODO(#44): remove `withr::local_options`
    withr::local_options(list(bit64.promoteInteger64ToCharacter=TRUE))
    x[[1.0]] = 3L
    x[[2L]] = "4"
    expect_identical(x, as.character(c(3:4, 3:10)))
  })

  # local writes didn't edit 'x'
  expect_identical(x[1L], as.integer64(1L))
  expect_identical(x[[2L]], as.integer64(2L))
})

test_that("indexing works: get", {
  x = as.integer64(1:10)

  names(x) = letters[1:10]
  expect_identical(x[c("b", "c")], x[2:3])
  expect_identical(x[["d"]], x[[4L]])
  
  expect_no_warning(expect_identical(integer64()[integer()], integer64()))
  expect_no_warning(expect_identical(structure(as.integer64(1L), dim=1L)[1L], as.integer64(1L)))

  expect_no_warning(expect_identical(as.integer64(1L)[NA_integer_], NA_integer64_))
  expect_no_warning(expect_identical(as.integer64(1L)[NA_integer64_], NA_integer64_))
})

test_that("arithmetic & basic math works", {
  x = as.integer64(1:10)
  y = as.integer64(10:1)
  expect_identical(sign(x - 6L), as.integer64(rep(c(-1L, 0L, 1L), c(5L, 1L, 4L))))
  expect_identical(abs(x - 6.0), as.integer64(c(5:0, 1:4)))

  expect_identical(sqrt(as.integer64(c(0L, 1L, 4L, 9L))), as.numeric(0:3))
  expect_identical(log(x), log(as.numeric(x)))
  expect_identical(log(as.integer64(c(1L, 2L, 4L, 8L)), base=2L), as.numeric(0:3))
  expect_identical(log2(as.integer64(c(1L, 2L, 4L, 8L))), as.numeric(0:3))
  expect_identical(log10(as.integer64(c(1L, 10L, 100L, 1000L))), as.numeric(0:3), tolerance=1e-7)

  expect_identical(trunc(x), x)
  expect_identical(floor(x), x)
  expect_identical(ceiling(x), x)
  expect_identical(signif(x), x)
  expect_identical(round(x), x)

  expect_identical(round(x, -1L), as.integer64(rep(c(0L, 10L), each=5L)))
})

test_that("basic statistics work", {
  x = as.integer64(1:10)

  expect_identical(sum(x), as.integer64(55L))
  expect_identical(sum(x, x), as.integer64(110L))
  expect_identical(prod(x), as.integer64(factorial(10L)))
  expect_identical(prod(x[1:5], x[6:10]), as.integer64(factorial(10L)))
  expect_identical(diff(x), as.integer64(rep(1L, 9L)))

  expect_identical(cummin(x), as.integer64(rep(1L, 10L)))
  expect_identical(cummax(x), x)
  expect_identical(cumsum(x), as.integer64(choose(2:11, 2L)))
  expect_identical(cumprod(x), as.integer64(factorial(1:10)))
})

test_that("min, max, and range work", {
  x = as.integer64(1:10)

  expect_identical(min(x), x[1L])
  expect_identical(max(x), x[10L])
  expect_identical(range(x), x[c(1L, 10L)])

  expect_identical(min(x, as.integer64(0L), as.integer64(11L)), as.integer64(0L))
  expect_identical(max(x, as.integer64(0L), as.integer64(11L)), as.integer64(11L))
  expect_identical(range(x, as.integer64(0L), as.integer64(11L)), as.integer64(c(0L, 11L)))

  expect_identical(range(x, NA_integer64_, finite=TRUE), x[c(1L, 10L)])

  expect_identical(min(x, integer64()), x[1L])
  expect_identical(max(x, integer64()), x[10L])
  expect_identical(range(x, integer64()), x[c(1L, 10L)])

  expect_identical(min(x, integer64(), na.rm=TRUE), x[1L])
  expect_identical(max(x, integer64(), na.rm=TRUE), x[10L])
  expect_identical(range(x, integer64(), na.rm=TRUE), x[c(1L, 10L)])
})

test_that("min, max, and range work in edge cases", {
  expect_warning(
    expect_identical(min(integer64()), lim.integer64()[2L]),
    "no non-NA value"
  )
  expect_warning(
    expect_identical(max(integer64()), lim.integer64()[1L]),
    "no non-NA value"
  )
  expect_warning(
    expect_identical(range(integer64()), lim.integer64()[2:1]),
    "no non-NA value"
  )

  expect_warning(
    expect_identical(min(integer64(), na.rm=TRUE), lim.integer64()[2L]),
    "no non-NA value"
  )
  expect_warning(
    expect_identical(max(integer64(), na.rm=TRUE), lim.integer64()[1L]),
    "no non-NA value"
  )
  expect_warning(
    expect_identical(range(integer64(), na.rm=TRUE), lim.integer64()[2:1]),
    "no non-NA value"
  )

  expect_no_warning(
    expect_identical(min(lim.integer64()[2L]), lim.integer64()[2L])
  )
  expect_no_warning(
    expect_identical(max(lim.integer64()[1L]), lim.integer64()[1L])
  )
  expect_no_warning(
    expect_identical(range(lim.integer64()), lim.integer64())
  )

  expect_no_warning(
    expect_identical(min(lim.integer64()[2L], NA_integer64_, na.rm=TRUE), lim.integer64()[2L])
  )
  expect_no_warning(
    expect_identical(max(lim.integer64()[1L], NA_integer64_, na.rm=TRUE), lim.integer64()[1L])
  )
  expect_no_warning(
    expect_identical(range(lim.integer64(), NA_integer64_, na.rm=TRUE), lim.integer64())
  )

  expect_no_warning(
    expect_identical(min(integer64(), NA_integer64_), NA_integer64_)
  )
  expect_no_warning(
    expect_identical(max(integer64(), NA_integer64_), NA_integer64_)
  )
  expect_no_warning(
    expect_identical(range(integer64(), NA_integer64_), rep(NA_integer64_, 2L))
  )

  expect_warning(
    expect_identical(min(integer64(), NA_integer64_, na.rm=TRUE), lim.integer64()[2L]),
    "no non-NA value"
  )
  expect_warning(
    expect_identical(max(integer64(), NA_integer64_, na.rm=TRUE), lim.integer64()[1L]),
    "no non-NA value"
  )
  expect_warning(
    expect_identical(range(integer64(), NA_integer64_, na.rm=TRUE), lim.integer64()[2:1]),
    "no non-NA value"
  )

  expect_no_warning(
    expect_identical(min(integer64(1), NA_integer64_), NA_integer64_)
  )
  expect_no_warning(
    expect_identical(max(integer64(1), NA_integer64_), NA_integer64_)
  )
  expect_no_warning(
    expect_identical(range(integer64(1), NA_integer64_), rep(NA_integer64_, 2L))
  )

  expect_no_warning(
    expect_identical(min(integer64(1), NA_integer64_, na.rm=TRUE), as.integer64(0L))
  )
  expect_no_warning(
    expect_identical(max(integer64(1), NA_integer64_, na.rm=TRUE), as.integer64(0L))
  )
  expect_no_warning(
    expect_identical(range(integer64(1), NA_integer64_, na.rm=TRUE), rep(as.integer64(0L), 2L))
  )

  expect_no_warning(
    expect_identical(min(integer64(1), integer64(), NA_integer64_), NA_integer64_)
  )
  expect_no_warning(
    expect_identical(max(integer64(1), integer64(), NA_integer64_), NA_integer64_)
  )
  expect_no_warning(
    expect_identical(range(integer64(1), integer64(), NA_integer64_), rep(NA_integer64_, 2L))
  )

  expect_no_warning(
    expect_identical(min(integer64(1), integer64(), NA_integer64_, na.rm=TRUE), as.integer64(0L))
  )
  expect_no_warning(
    expect_identical(max(integer64(1), integer64(), NA_integer64_, na.rm=TRUE), as.integer64(0L))
  )
  expect_no_warning(
    expect_identical(range(integer64(1), integer64(), NA_integer64_, na.rm=TRUE), rep(as.integer64(0L), 2L))
  )

  expect_no_warning(
    expect_identical(min(integer64(1), integer64()), as.integer64(0L))
  )
  expect_no_warning(
    expect_identical(max(integer64(1), integer64()), as.integer64(0L))
  )
  expect_no_warning(
    expect_identical(range(integer64(1), integer64()), as.integer64(c(0L, 0L)))
  )

  expect_no_warning(
    expect_identical(min(integer64(1), integer64(), na.rm=TRUE), as.integer64(0L))
  )
  expect_no_warning(
    expect_identical(max(integer64(1), integer64(), na.rm=TRUE), as.integer64(0L))
  )
  expect_no_warning(
    expect_identical(range(integer64(1), integer64(), na.rm=TRUE), rep(as.integer64(0L), 2L))
  )
})

test_that("display methods work", {
  x = as.integer64(1:3)
  expect_identical(format(x), as.character(1:3))
  expect_output(print(x), "integer64.*\\s*1\\s*2\\s*3")
  expect_output(print(x[0L]), "integer64(0)", fixed=TRUE)
  expect_output(str(x), "integer64 [1:3] 1 2 3", fixed=TRUE)
})

test_that("vector builders of integer64 work", {
  x = as.integer64(1:3)
  expect_identical(c(x, FALSE), as.integer64(c(1:3, 0L)))
  expect_identical(c(x, 4:6), as.integer64(1:6))
  expect_identical(c(x, 4.0, 5.0, 6.0), as.integer64(1:6))
  expect_identical(c(x, as.integer64(4:6)), as.integer64(1:6))

  expect_identical(cbind(x, FALSE), matrix64(c(1:3, 0L, 0L, 0L), nrow=3L, ncol=2L, dimnames=list(NULL, c("x", ""))))
  expect_identical(cbind(x, 4:6), matrix64(1:6, nrow=3L, ncol=2L, dimnames=list(NULL, c("x", ""))))
  expect_identical(cbind(x, 0.0), matrix64(c(1:3, 0L, 0L, 0L), nrow=3L, ncol=2L, dimnames=list(NULL, c("x", ""))))
  expect_identical(cbind(x, as.integer64(4:6)), matrix64(1:6, nrow=3L, ncol=2L, dimnames=list(NULL, c("x", ""))))

  expect_identical(rbind(x, FALSE), matrix64(c(1:3, 0L, 0L, 0L), nrow=2L, ncol=3L, byrow=TRUE, dimnames=list(c("x", ""), NULL)))
  expect_identical(rbind(x, 4:6), matrix64(1:6, nrow=2L, ncol=3L, byrow=TRUE, dimnames=list(c("x", ""), NULL)))
  expect_identical(rbind(x, 0.0), matrix64(c(1:3, 0L, 0L, 0L), nrow=2L, ncol=3L, byrow=TRUE, dimnames=list(c("x", ""), NULL)))
  expect_identical(rbind(x, as.integer64(4:6)), matrix64(1:6, nrow=2L, ncol=3L, byrow=TRUE, dimnames=list(c("x", ""), NULL)))

  expect_identical(rep(x, 2L), c(x, x))
  expect_identical(rep(x, each=2L), as.integer64(c(1L, 1L, 2L, 2L, 3L, 3L)))

  expect_identical(x[1L]:x[3L], x)
  expect_identical(x[3L]:x[1L], x[3:1]) # rev() a separate method
})

with_parameters_test_that(
  "seq method works analogously to integer: 1 argument (except along.with);",
  {
    n64 = as.integer64(n)
    expect_identical(seq(n64), as.integer64(seq(n)))
    expect_identical(seq(from=n64), as.integer64(seq(from=n)))
    expect_identical(seq(to=n64), as.integer64(seq(to=n)))
    expect_identical(seq(by=n64), as.integer64(seq(by=n)))
    if (n < 0L) {
      expect_error(seq(length.out=n64), "'length.out' must be a non-negative number")
    } else {
      expect_identical(seq(length.out=n64), as.integer64(seq(length.out=n)))
    }
  },
  n = c(0L, 5L, -1L)
)

with_parameters_test_that(
  "seq method works analogously to integer: 2 arguments (except along.with)",
  {
    # be sure to coerce (truncate) consistently across actual/expected
    n1_32 = as.integer(n1)
    n2_32 = as.integer(n2)

    n1_64 = as.integer64(n1)
    n2_64 = as.integer64(n2)

    # TODO(#211): restore parity to seq() here
    if (n2 %% 1L == 0L) expect_identical(seq(n1_64, n2), as.integer64(seq(n1_32, n2)))
    expect_identical(seq(n1_64, n2_64), as.integer64(seq(n1_32, n2_32)))


    if (n2 == 0L) {
      err_msg = "invalid '(to - from)/by'"
      expect_error(seq(n1_64, by=n2), err_msg, fixed=TRUE)
      expect_error(seq(to=n1_64, by=n2), err_msg, fixed=TRUE)
    } else if (sign(1L - n1) == sign(n2)) {
      if (n2 %% 1L == 0L) expect_identical(seq(n1_64, by=n2), as.integer64(seq(n1, by=n2)))
      expect_identical(seq(n1_64, by=n2_64), as.integer64(seq(n1, by=n2_32)))

      expect_error(seq(to=n1_64, by=n2), "wrong sign in 'by' argument", fixed=TRUE)
    } else {
      expect_error(seq(n1_64, by=n2), "wrong sign in 'by' argument", fixed=TRUE)

      # TODO(#211): restore parity to seq() here
      if (n2 %% 1L == 0L) expect_identical(seq(to=n1_64, by=n2), as.integer64(seq(to=n1, by=n2)))
      expect_identical(seq(to=n1_64, by=n2_64), as.integer64(seq(to=n1, by=n2_32)))
    }

    if (n2 >= 0L) {
      expect_identical(seq(n1_64, length.out=n2), as.integer64(seq(n1_32, length.out=n2)))
      expect_identical(seq(n1_64, length.out=n2_64), as.integer64(seq(n1_32, length.out=n2_32)))

      expect_identical(seq(to=n1_64, length.out=n2), as.integer64(seq(to=n1_32, length.out=n2)))
      expect_identical(seq(to=n1_64, length.out=n2_64), as.integer64(seq(to=n1_32, length.out=n2_32)))

      expect_identical(seq(by=n1_64, length.out=n2), as.integer64(seq(by=n1_32, length.out=n2)))
      expect_identical(seq(by=n1_64, length.out=n2_64), as.integer64(seq(by=n1_32, length.out=n2_32)))
    } else {
      err_msg = "'length.out' must be a non-negative number"
      expect_error(seq(n1_64, length.out=n2), err_msg, fixed=TRUE)
      expect_error(seq(to=n1_64, length.out=n2), err_msg, fixed=TRUE)
      expect_error(seq(by=n1_64, length.out=n2), err_msg, fixed=TRUE)
    }
  },
  .cases = expand.grid(n1=c(0L, 5L, -1L), n2=c(0.0, 5.0, -1.0, 1.5))
)

with_parameters_test_that(
  "seq method works analogously to integer: 3 arguments (except along.with)",
  {
    # be sure to coerce (truncate) consistently across actual/expected
    n1_32 = as.integer(n1)
    n2_32 = as.integer(n2)
    n3_32 = as.integer(n3)

    n1_64 = as.integer64(n1)
    n2_64 = as.integer64(n2)
    n3_64 = as.integer64(n3)

    # TODO(#211): restore parity to seq() here
    if (n2 == n1 || sign(n2 - n1) == sign(n3)) {
      # TODO(#211): restore parity to seq() here
      if (n2 %% 1L == 0L && n3 %% 1L == 0L) {
        expect_identical(seq(n1_64, n2, by=n3), as.integer64(seq(n1_32, n2, by=n3)))
        expect_identical(seq(n1_64, n2_64, by=n3), as.integer64(seq(n1_32, n2_32, by=n3)))
        expect_identical(seq(n1_64, n2, by=n3_64), as.integer64(seq(n1_32, n2, by=n3_32)))
        expect_identical(seq(n1_64, n2_64, by=n3_64), as.integer64(seq(n1_32, n2_32, by=n3_32)))
      }
    } else {
      err_msg <- if (n3 == 0L) "invalid '(to - from)/by'" else "wrong sign in 'by' argument"
      expect_error(seq(n1_64, n2, by=n3), err_msg, fixed=TRUE)
    }

    if (n3 < 0L) {
      err_msg = "'length.out' must be a non-negative"
      expect_error(seq(n1_64, n2, length.out=n3), err_msg, fixed=TRUE)
      expect_error(seq(n1_64, by=n2, length.out=n3), err_msg, fixed=TRUE)
      expect_error(seq(to=n1_64, by=n2, length.out=n3), err_msg, fixed=TRUE)
    } else {
      # TODO(#211): restore parity to seq() here
      if (((n2 - n1) / (n3 - 1L)) %% 1L == 0L) {
        expect_identical(seq(n1_64, n2, length.out=n3), as.integer64(seq(n1_32, n2, length.out=n3)))
        expect_identical(seq(n1_64, n2_64, length.out=n3), as.integer64(seq(n1_32, n2_32, length.out=n3)))
        expect_identical(seq(n1_64, n2, length.out=n3_64), as.integer64(seq(n1_32, n2, length.out=n3_32)))
        expect_identical(seq(n1_64, n2_64, length.out=n3_64), as.integer64(seq(n1_32, n2_32, length.out=n3_32)))
      }

      # TODO(#211): restore parity to seq() here
      if (n2 %% 1L == 0L) {
        expect_identical(seq(n1_64, by=n2, length.out=n3), as.integer64(seq(n1_32, by=n2, length.out=n3)))
        expect_identical(seq(n1_64, by=n2_64, length.out=n3), as.integer64(seq(n1_32, by=n2_32, length.out=n3)))
        expect_identical(seq(n1_64, by=n2, length.out=n3_64), as.integer64(seq(n1_32, by=n2, length.out=n3_32)))
        expect_identical(seq(n1_64, by=n2_64, length.out=n3_64), as.integer64(seq(n1_32, by=n2_32, length.out=n3_32)))

        expect_identical(seq(to=n1_64, by=n2, length.out=n3), as.integer64(seq(to=n1_32, by=n2, length.out=n3)))
        expect_identical(seq(to=n1_64, by=n2_64, length.out=n3), as.integer64(seq(to=n1_32, by=n2_32, length.out=n3)))
        expect_identical(seq(to=n1_64, by=n2, length.out=n3_64), as.integer64(seq(to=n1_32, by=n2, length.out=n3_32)))
        expect_identical(seq(to=n1_64, by=n2_64, length.out=n3_64), as.integer64(seq(to=n1_32, by=n2_32, length.out=n3_32)))
      }
    }
  },
  # TODO(#211): restore these cases which currently result in 'empty test'
  .cases = subset(
    expand.grid(n1=c(0L, 5L, -1L), n2=c(0.0, 5.0, -1.0, 1.5), n3=c(0.0, 5.0, -1.0, 1.5)),
    !(n1 <= 0 & n2 == 1.5 & n3 == 5)
  )
)

test_that("seq method works analogously to integer: 4 arguments", {
  n = as.integer64(5L)

  expect_error(seq(n, n, by=n, length.out=n), "too many arguments")
  expect_error(seq(n, n, by=n, along.with=n), "too many arguments")
})

test_that("seq() works back-compatibly w.r.t. mixed integer+double inputs", {
  one = as.integer64(1L)
  expect_identical(seq(one, 10L, by=1.5), as.integer64(1:10))
  expect_identical(seq(to=one, from=10L, by=-1.5), as.integer64(10:1))

  expect_identical(seq(one, 10L, by=10.0/3.0), as.integer64(c(1L, 4L, 7L, 10L)))
  expect_identical(seq(to=one, from=10L, by=-10.0/3.0), as.integer64(c(10L, 7L, 4L, 1L)))

  expect_error(seq(one, 10L, by=0.1), "invalid '(to - from)/by'", fixed=TRUE)
  expect_error(seq(to=one, from=10L, by=-0.1), "invalid '(to - from)/by'", fixed=TRUE)

  expect_identical(seq(one, 2.5), as.integer64(1:2))
  expect_identical(seq(to=one, from=2.5), as.integer64(2:1))

  expect_identical(seq(one, 5.5, by=1.5), as.integer64(1:5))
  expect_identical(seq(to=one, from=5.5, by=-1.5), as.integer64(5:1))
})

# These tests were previously kept as tests under \examples{\dontshow{...}}.
#   Converted to "proper" unit tests for clarity, after making them more
#   canonical within {testthat}, e.g. better capturing expected warnings,
#   changing stopifnot(identical(...)) to expect_identical(...).
local({
  i <- -999:999
  with_parameters_test_that(
    "Old \\dontshow{} tests in ?format.integer64 continue working",
    {
      r <- as.integer64(round(as.integer(i), s))
      r64 <- round(as.integer64(i), s)
      expect_identical(r, r64)
    },
    s = -3:3,
    .interpret_glue = FALSE
  )
})

test_that("Old \\dontshow{} tests in ?extract.replace.integer64 continue working", {
  r <- c(runif64(1000L, lim.integer64()[1L], lim.integer64()[2L]), NA, -2:2)
  expect_identical(r, as.integer64(as.bitstring(r)))
})

test_that("empty inputs give empty outputs for arithmetic", {
  x = integer64(1L)
  empty = integer64(0L)

  expect_identical(log(x, base=empty), double())
  expect_identical(log(empty, base=x), double())
  expect_identical(
    log(`attr<-`(empty, "asdf", "jkl")),
    `attr<-`(double(), "asdf", "jkl")
  )
})

test_that("all.equal.integer64 reflects changes for vector scale= from all.equal.numeric", {
  # same test as for base R, multiplied by 1000 so the inputs are all integer64
  expect_identical(
    all.equal(
      as.integer64(c(1000L, 1000L)),
      as.integer64(c(1010L, 1010L)),
      scale = c(10.0, 10.0)
    ),
    "Mean scaled difference: 1"
  )
  # same test as for base R, multiplied by 1e9
  one_e9 = as.integer64(1000000000L)
  expect_true(all.equal(
    rep(one_e9, 5L),
    one_e9 + (-1L:3), # TODO(r-lib/lintr#): no 'L'
    scale = (1:5)*one_e9
  ))
})

test_that("all.equal works", {
  x = y = as.integer64(1L)

  expect_true(all.equal(x, x))

  class(y) = c("xx", "integer64")
  expect_match(all.equal(x, y), "target is integer64, current is xx", fixed=TRUE, all=FALSE)
  expect_match(all.equal(x[0L], x[1L]), "integer64: lengths.*differ", all=FALSE)

  class(y) = "integer64"
  attr(y, "xx") = "zz"
  expect_match(all.equal(x, y), "Attributes", fixed=TRUE)
  expect_no_match(
    expect_match(all.equal(x[0L], y), "integer64: lengths.*differ", all=FALSE),
    "Lengths:", fixed = TRUE
  )

  y = NA_integer64_
  expect_match(all.equal(x, y), "'is.NA' value mismatch", fixed=TRUE)

  x = as.integer64(1000000000L)
  expect_true(all.equal(x, x+1L))
  expect_true(all.equal(x, x+1L, tolerance=1.0e9)) # forcing scale=1
  expect_match(all.equal(x, x+100L), "Mean relative difference", fixed=TRUE)
  expect_match(all.equal(x, x+1L, scale=1.0), "Mean absolute difference", fixed=TRUE)
})

test_that("anyNA method", {
  expect_identical(anyNA(as.integer64(c(1L, 1L))), anyNA(c(1L, 1L)))
  expect_identical(anyNA(as.integer64(c(1L, NA))), anyNA(c(1L, NA)))
  expect_identical(anyNA(as.integer64(c(NA, NA))), anyNA(c(NA, NA)))
  expect_identical(anyNA(integer64()), anyNA(integer()))
})

test_that("match works with zero length input", {
  x32 = 1:10
  x64 = as.integer64(1:10)
  expect_identical(match(x64, integer()), match(x32, integer()))
  expect_identical(match(x64, integer(), nomatch=NULL), match(x32, integer(), nomatch=NULL))
  expect_identical(match(x64, integer(), nomatch=integer()), match(x32, integer(), nomatch=integer()))
  expect_identical(match(x64, integer(), nomatch=10L), match(x32, integer(), nomatch=10L))
  expect_identical(match(integer(), x64), match(integer(), x32))
})

test_that("factor and order for integer64 are still necessary", {
  skip_unless_r(">= 4.1.0") # skip implementing "expect_failure" for ancient R
  skip_on_cran()
  skip_on_covr() # covr edits the function bodies

  x = c(132724613L, -2143220989L, -1L, NA, 1L)
  expect_failure(expect_identical(base::factor(as.integer64(x)), base::factor(x)))
  expect_failure(expect_identical(factor(as.integer64(x)), base::factor(as.integer64(x))))
  expect_identical(factor(as.integer64(x)), base::factor(x))

  expect_identical(formals(factor), formals(base::factor))
  expect_identical(formals(ordered), formals(base::ordered))
  expect_identical(body(ordered), body(base::ordered))
})

with_parameters_test_that("factor and order work analogously to integer:", {
    x = c(132724613L, -2143220989L, -1L, NA, 1L)
    # test factor() for integer64 with short (< 4000) and long (>= 4000) vectors, because of the different code paths for the two cases
    if (isTRUE(long_input))
      x = rep_len(x, 5000L)

    expect_identical(factor(as.integer64(x)), factor(x))

    expect_identical(
      tryCatch(factor(as.integer64(x), levels=levels, labels=labels, exclude=exclude, ordered=ordered), error=conditionMessage),
      tryCatch(factor(x, levels=levels, labels=labels, exclude=exclude, ordered=ordered), error=conditionMessage)
    )
    if (isTRUE(ordered))
      expect_identical(
        tryCatch(ordered(as.integer64(x), levels=levels, labels=labels, exclude=exclude), error=conditionMessage),
        tryCatch(ordered(x, levels=levels, labels=labels, exclude=exclude), error=conditionMessage)
      )
},
  .cases = expand.grid(
      levels=I(list(NULL, NA, 1L, c(-1L, 1L), "1")),
      labels=I(list(levels, NULL, letters[1L], letters[1:2])),
      exclude=I(list(NULL, NA, 1L, c(-1L, 1L))),
      ordered=c(TRUE, FALSE),
      long_input=c(FALSE, TRUE)
    )
)

test_that("factor() overwrite doesn't oversample (#274)", {
  expect_identical(
    withr::with_seed(1, factor(sample(10))),
    withr::with_seed(1, base::factor(sample(10)))
  )
  expected_result = withr::with_seed(1, base::factor(exclude = sample(1:10, 1L), sample(1:10)))
  expect_identical(
    withr::with_seed(1, factor(exclude = sample(1:10, 1L), sample(as.integer64(1:10)))),
    expected_result
  )
  expect_identical(
    withr::with_seed(1, factor(exclude = sample(1:10, 1L), sample(1:10))),
    expected_result
  )
  
  expected_result = withr::with_seed(1, base::factor(sample(1:10), exclude = sample(1:10, 1L)))
  expect_identical(
    withr::with_seed(1, factor(sample(as.integer64(1:10)), exclude = sample(1:10, 1L))),
    expected_result
  )
  expect_identical(
    withr::with_seed(1, factor(sample(1:10), exclude = sample(1:10, 1L))),
    expected_result
  )
})

test_that("extraction works consistent to integer: vector[", {
  x = 1:10
  names(x) = letters[seq_along(x)]
  y = as.integer64(x)
  names(y) = letters[seq_along(y)]

  sel = c(TRUE, FALSE, NA, TRUE)
  expect_identical(y[sel], setNames(as.integer64(x[sel]), names(x)[sel]))

  sel = c(1L, NA, 3L, 11L)
  expect_identical(y[sel], setNames(as.integer64(x[sel]), names(x)[sel]))

  sel = c(1, NA, 3, 11)
  expect_identical(y[sel], setNames(as.integer64(x[sel]), names(x)[sel]))
  expect_identical(y[as.integer64(sel)], setNames(as.integer64(x[sel]), names(x)[sel]))

  sel = c(-1L, -3L, 0L, -11L)
  expect_identical(y[sel], setNames(as.integer64(x[sel]), names(x)[sel]))

  sel = c(-1, -3, 0, -11)
  expect_identical(y[sel], setNames(as.integer64(x[sel]), names(x)[sel]))

  # `only 0's may be mixed with negative subscripts`
  sel = c(-1, -3, 0, -11, NA)
  expect_same_error(x[sel], y[sel])

  expect_identical(
    as.integer64(c("9218868437227407266", "1"))[c(1,NA,3,4)],
    as.integer64(c("9218868437227407266", NA, NA, NA))
  )

  sel = c("d", "", "b", NA_character_)
  expect_identical(y[sel], setNames(as.integer64(x[sel]), names(x[sel])))

  expect_identical(lapply(list(integer64()), `[`, 0L), list(integer64()))

  getter = function(x, ...) x[...]

  i64 = integer64(1)
  names(i64) = "a"
  expect_identical(getter(i64), i64)
  expect_identical(getter(i64, 1L), i64)
  expect_identical(getter(i64, "a"), i64)

  i32 = integer(1)
  names(i32) = "a"
  expect_identical(getter(i32), i32)
  expect_identical(getter(i32, 1L), i32)
  expect_identical(getter(i32, "a"), i32)

  l = list(integer(1L), integer(2L), integer(3L))
  expect_identical(
    lapply(l, getter, 1L),
    list(integer(1L), integer(1L), integer(1L))
  )
})

test_that("replacement works consistent to integer: vector[<-", {
  x = as.integer(1:10)
  names(x) = letters[seq_along(x)]
  y = as.integer64(x)
  names(y) = letters[seq_along(y)]

  sel = c("d", "", "b", NA_character_)
  x[sel] = 100L
  y[sel] = 100L
  expect_identical(y, setNames(as.integer64(x), names(x)))
})

test_that("extraction works consistent to integer: vector[[", {
  x = as.integer(1:10)
  names(x) = letters[seq_along(x)]
  y = as.integer64(x)
  names(y) = letters[seq_along(y)]

  expect_identical(y[[3]], as.integer64(x[[3]]))
  expect_identical(y[["d"]], as.integer64(x[["d"]]))
})

test_that("replacement works consistent to integer: vector[[<-", {
  x = as.integer(1:10)
  names(x) = letters[seq_along(x)]
  y = as.integer64(x)
  names(y) = letters[seq_along(y)]

  x[["e"]] = 100L
  y[["e"]] = 100L
  expect_identical(y, setNames(as.integer64(x), names(x)))
})

test_that("extraction works consistent to integer: matrix[", {
  m32 = matrix(1:10, nrow=2L)
  m64 = matrix64(as.integer64(m32), nrow=dim(m32)[1L], ncol=dim(m32)[2L])

  expect_identical(m32[integer(), 1:2, drop=TRUE], structure(integer(), dim=c(0L, 2L)))
  expect_identical(m64[integer(), 1:2, drop=TRUE], structure(integer64(), dim=c(0L, 2L)))

  expect_identical(m32[1:2, integer(), drop=TRUE], structure(integer(), dim=c(2L, 0L)))
  expect_identical(m64[1:2, integer(), drop=TRUE], structure(integer64(), dim=c(2L, 0L)))

  expect_identical(m32[integer(), 1:2, drop=FALSE], structure(integer(), dim=c(0L, 2L)))
  expect_identical(m64[integer(), 1:2, drop=FALSE], structure(integer64(), dim=c(0L, 2L)))

  expect_identical(m32[1:2, integer(), drop=FALSE], structure(integer(), dim=c(2L, 0L)))
  expect_identical(m64[1:2, integer(), drop=FALSE], structure(integer64(), dim=c(2L, 0L)))

  expect_identical(m32[1:2, 1:3, drop=TRUE], structure(1:6, dim=c(2L, 3L)))
  expect_identical(m64[1:2, 1:3, drop=TRUE], structure(as.integer64(1:6), dim=c(2L, 3L)))

  expect_identical(m32[1:2], 1:2)
  expect_identical(m64[1:2], as.integer64(1:2))

  expect_identical(m32[1:2, drop=TRUE], 1:2)
  expect_identical(m64[1:2, drop=TRUE], as.integer64(1:2))

  expect_identical(m32[j = 1:3, drop=TRUE], 1:3)
  expect_identical(m64[j = 1:3, drop=TRUE], as.integer64(1:3))

  expect_identical(m32[1:2, , drop=TRUE], structure(as.integer(1:10), dim=c(2L, 5L)))
  expect_identical(m64[1:2, , drop=TRUE], structure(as.integer64(1:10), dim=c(2L, 5L)))

  expect_identical(m32[, 1:3, drop=TRUE], structure(as.integer(1:6), dim=c(2L, 3L)))
  expect_identical(m64[, 1:3, drop=TRUE], structure(as.integer64(1:6), dim=c(2L, 3L)))

  expect_identical(m32[1, , drop=TRUE], c(1L, 3L, 5L, 7L, 9L))
  expect_identical(m64[1, , drop=TRUE], as.integer64(c(1L, 3L, 5L, 7L, 9L)))

  expect_identical(m32[1, , drop=FALSE], structure(c(1L, 3L, 5L, 7L, 9L), dim=c(1L, 5L)))
  expect_identical(m64[1, , drop=FALSE], structure(as.integer64(c(1L, 3L, 5L, 7L, 9L)), dim=c(1L, 5L)))

  expect_identical(m32[, 1, drop=TRUE], 1:2)
  expect_identical(m64[, 1, drop=TRUE], as.integer64(1:2))

  expect_identical(m32[, 1, drop=FALSE], structure(1:2, dim=2:1))
  expect_identical(m64[, 1, drop=FALSE], structure(as.integer64(1:2), dim=2:1))

  expect_identical(m32[c(9, NA, 11, 12), drop=FALSE], c(9L, NA, NA, NA))
  expect_identical(m64[c(9, NA, 11, 12), drop=FALSE], as.integer64(c(9L, NA, NA, NA)))

  expect_identical(m32[integer(), c(1:2, 0, NA), drop=TRUE], structure(integer(), dim=c(0L, 3L)))
  expect_identical(m64[integer(), c(1:2, 0, NA), drop=TRUE], structure(integer64(), dim=c(0L, 3L)))
  expect_identical(m64[integer64(), c(1:2, 0, NA), drop=TRUE], structure(integer64(), dim=c(0L, 3L)))

  expect_identical(m32[, c(1:2, 0, NA), drop=TRUE], structure(c(1:4, NA, NA), dim=c(2L, 3L)))
  expect_identical(m64[, c(1:2, 0, NA), drop=TRUE], structure(as.integer64(c(1:4, NA, NA)), dim=c(2L, 3L)))

  expect_identical(m32[c(1, NA, 2), 1:3, drop=TRUE], structure(c(1L, NA, 2L, 3L, NA, 4L, 5L, NA, 6L), dim=c(3L, 3L)))
  expect_identical(m64[c(1, NA, 2), 1:3, drop=TRUE], structure(as.integer64(c(1L, NA, 2L, 3L, NA, 4L, 5L, NA, 6L)), dim=c(3L, 3L)))

  m32 = matrix(1:10, 2L, dimnames = list(LETTERS[1:2], letters[1:5]))
  m64 = matrix64(as.integer64(1:10), nrow=2L, ncol=5L, dimnames = list(LETTERS[1:2], letters[1:5]))

  # `subscript out of bounds`
  expect_same_error(
    m32[c("B", "D", "A"), c("d", "a")],
    m64[c("B", "D", "A"), c("d", "a")]
  )

  expect_identical(m32[c("B", "D", "A")], rep(NA_integer_, 3L))
  expect_identical(m64[c("B", "D", "A")], rep(NA_integer64_, 3L))
})

test_that("replacement works consistent to integer: matrix[<-", {
  m32 = matrix(1:10, nrow=2L)
  m64 = matrix64(as.integer64(m32), nrow=dim(m32)[1L], ncol=dim(m32)[2L])

  local({
    m32[1, c(1, 3, NA)] = 100L
    m64[1, c(1, 3, NA)] = as.integer64(100L)
    expect_identical(m64, structure(as.integer64(m32), dim=dim(m32), dimnames=dimnames(m32)))
  })
  local({
    m32[1, c(1, 4, NA)] = 101L
    m64[1, c(1, 4, NA)] = 101L
    expect_identical(m64, structure(as.integer64(m32), dim=dim(m32), dimnames=dimnames(m32)))
  })
  local({
    m32[1, c(1, 5, NA)] = 102
    m64[1, c(1, 5, NA)] = 102
    expect_identical(m64, structure(as.integer64(m32), dim=dim(m32), dimnames=dimnames(m32)))
  })
  local({
    m32[1, c(1, 5, NA)] = 102.0 + 0.0i
    m64[1, c(1, 5, NA)] = 102.0 + 0.0i
    expect_identical(m64, m32)
  })
  # TODO(#44): remove this deprecated test
  local({
    m32[1, c(1, 3, NA)] = 103L
    m64[1, c(1, 3, NA)] = "103"
    expect_identical(m64, structure(as.integer64(m32), dim=dim(m32), dimnames=dimnames(m32)))
  })
  local({
    # TODO(#44): remove `withr::local_options`
    withr::local_options(list(bit64.promoteInteger64ToCharacter=TRUE))
    m32[1, c(1, 3, NA)] = "103"
    m64[1, c(1, 3, NA)] = "103"
    expect_identical(m64, m32)
  })
  local({
    m32[1, c(1, 3, NA)] = 101L
    m64[1, as.integer64(c(1, 3, NA))] = 101L
    expect_identical(m64, structure(as.integer64(m32), dim=dim(m32), dimnames=dimnames(m32)))
  })
  local({
    m32[, -(1:3)] = 102L
    m64[, -(1:3)] = 102L
    expect_identical(m64, structure(as.integer64(m32), dim=dim(m32), dimnames=dimnames(m32)))
  })
})

test_that("extraction works consistent to integer: matrix[[", {
  m32 = matrix(1:10, 2L, dimnames = list(LETTERS[1:2], letters[1:5]))
  m64 = matrix64(as.integer64(1:10), nrow=2L, ncol=5L, dimnames=list(LETTERS[1:2], letters[1:5]))

  expect_identical(m64[[1, 2]], as.integer64(m32[[1, 2]]))
  expect_identical(m64[[as.integer64(1L), as.integer64(2L)]], as.integer64(m32[[1, 2]]))
  expect_identical(m64[["A", "d"]], as.integer64(m32[["A", "d"]]))

  expect_identical(m64[[1]], as.integer64(m32[[1]]))
  expect_identical(m64[[as.integer64(1L)]], as.integer64(m32[[1]]))

  # `subscript out of bounds`
  expect_same_error(m32[[NA]], m64[[NA]])
  expect_same_error(m32[[NA]], m64[[NA_integer64_]])

  # `attempt to select less than one element in integerOneIndex`
  expect_same_error(m32[[0L]], m64[[0L]])
  expect_same_error(m32[[0L]], m64[[as.integer64(0L)]])

  # `attempt to select less than one element in get1index`
  expect_same_error(m32[[integer()]], m64[[integer()]])
  expect_same_error(m32[[integer()]], m64[[integer64()]])
})

test_that("replacement works consistent to integer: matrix[[<-", {
  m32 = matrix(1:10, 2L, dimnames = list(LETTERS[1:2], letters[1:5]))
  m64 = matrix64(as.integer64(1:10), nrow=2L, ncol=5L, dimnames=list(LETTERS[1:2], letters[1:5]))

  local({
    m32[[1, 3]] = 110L
    m64[[1, 3]] = 110L
    expect_identical(m64, structure(as.integer64(m32), dim=dim(m32), dimnames=dimnames(m32)))
  })
  local({
    m32[["A", "e"]] = 112L
    m64[["A", "e"]] = 112L
    expect_identical(m64, structure(as.integer64(m32), dim=dim(m32), dimnames=dimnames(m32)))
  })
  local({
    m32[[1, 3]] = 111
    m64[[1, 3]] = 111
    expect_identical(m64, structure(as.integer64(m32), dim=dim(m32), dimnames=dimnames(m32)))
  })
    local({
    m32[[1, 3]] = 111.0 + 0.0i
    m64[[1, 3]] = 111.0 + 0.0i
    expect_identical(m64, m32)
  })
  # TODO(#44): remove this deprecated test
  local({
    m32[[1, 4]] = 112L
    m64[[1, 4]] = "112"
    expect_identical(m64, structure(as.integer64(m32), dim=dim(m32), dimnames=dimnames(m32)))
  })
  local({
    # TODO(#44): remove `withr::local_options`
    withr::local_options(list(bit64.promoteInteger64ToCharacter=TRUE))
    m32[[1, 4]] = "112"
    m64[[1, 4]] = "112"
    expect_identical(m64, m32)
  })

})

test_that("extraction consistent to integer: array[", {
  a32 = array(1:27, c(3,3,3))
  a64 = array64(as.integer64(1:27), c(3,3,3))

  expect_identical(a32[2, , 3, drop=FALSE], structure(c(20L, 23L, 26L), dim = c(1L, 3L, 1L)))
  expect_identical(a64[2, , 3, drop=FALSE], structure(as.integer64(c(20L, 23L, 26L)), dim = c(1L, 3L, 1L)))

  expect_identical(a32[2, , 3, drop=TRUE], c(20L, 23L, 26L))
  expect_identical(a64[2, , 3, drop=TRUE], as.integer64(c(20L, 23L, 26L)))

  expect_identical(a32[1, c(1, 3, 2), 2:3, drop=TRUE], structure(c(10L, 16L, 13L, 19L, 25L, 22L), dim = 3:2))
  expect_identical(a64[1, c(1, 3, 2), 2:3, drop=TRUE], structure(as.integer64(c(10L, 16L, 13L, 19L, 25L, 22L)), dim = 3:2))

  expect_identical(a32[, c(1, 2, 0, 3, NA, 1), c(TRUE, FALSE, NA), drop=FALSE], structure(c(1L, 2L, 3L, 4L, 5L, 6L, 7L, 8L, 9L, NA, NA, NA, 1L, 2L, 3L, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA), dim = c(3L, 5L, 2L)))
  expect_identical(a64[, c(1, 2, 0, 3, NA, 1), c(TRUE, FALSE, NA), drop=FALSE], structure(as.integer64(c(1L, 2L, 3L, 4L, 5L, 6L, 7L, 8L, 9L, NA, NA, NA, 1L, 2L, 3L, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA)), dim = c(3L, 5L, 2L)))

  expect_identical(a32[, c(1, 2, 0, 3, NA, 1), c(TRUE, FALSE, NA), drop=TRUE], structure(c(1L, 2L, 3L, 4L, 5L, 6L, 7L, 8L, 9L, NA, NA, NA, 1L, 2L, 3L, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA), dim = c(3L, 5L, 2L)))
  expect_identical(a64[, c(1, 2, 0, 3, NA, 1), c(TRUE, FALSE, NA), drop=TRUE], structure(as.integer64(c(1L, 2L, 3L, 4L, 5L, 6L, 7L, 8L, 9L, NA, NA, NA, 1L, 2L, 3L, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA)), dim = c(3L, 5L, 2L)))

  expect_identical(a32[c(1, 0, 7, NA, 27, 28), drop=FALSE], c(1L, 7L, NA, 27L, NA))
  expect_identical(a64[c(1, 0, 7, NA, 27, 28), drop=FALSE], as.integer64(c(1L, 7L, NA, 27L, NA)))

  expect_identical(a32[c(TRUE, FALSE, NA, TRUE), drop=FALSE], c(1L, NA, 4L, 5L, NA, 8L, 9L, NA, 12L, 13L, NA, 16L, 17L, NA, 20L, 21L, NA, 24L, 25L, NA))
  expect_identical(a64[c(TRUE, FALSE, NA, TRUE), drop=FALSE], as.integer64(c(1L, NA, 4L, 5L, NA, 8L, 9L, NA, 12L, 13L, NA, 16L, 17L, NA, 20L, 21L, NA, 24L, 25L, NA)))

  expect_identical(a32[-1, , -c(0, 2:3), drop=FALSE], structure(c(2L, 3L, 5L, 6L, 8L, 9L), dim = c(2L, 3L, 1L)))
  expect_identical(a64[-1, , -c(0, 2:3), drop=FALSE], structure(as.integer64(c(2L, 3L, 5L, 6L, 8L, 9L)), dim = c(2L, 3L, 1L)))

  expect_identical(a32[-1, 2, -c(0, 2:3), drop=FALSE], structure(5:6, dim = c(2L, 1L, 1L)))
  expect_identical(a64[-1, 2, -c(0, 2:3), drop=FALSE], structure(as.integer64(5:6), dim = c(2L, 1L, 1L)))

  expect_identical(a32[-1, 2, -c(0, 2:3), drop=TRUE], 5:6)
  expect_identical(a64[-1, 2, -c(0, 2:3), drop=TRUE], as.integer64(5:6))
})

test_that("extraction from subclass retains inheritance structure", {
  x = as.integer64(1:2)
  class(x) = c("subclass", "integer64")

  expect_identical(x[1:2], x)

  y = x
  y[2L] = as.integer64(3L)
  expect_identical(y, structure(unclass(as.integer64(c(1L, 3L))), class=class(x)))

  x1 = x[[1L]]
  expect_identical(x1, structure(unclass(x)[1L], class=class(x)))

  y[[2L]] = as.integer64(4L)
  expect_identical(y, structure(unclass(as.integer64(c(1L, 4L))), class=class(x)))
})

test_that("c works consistent to R", {
  convert_x32_result_to_integer64 = function(x, x32=x, recursive=FALSE) {
    if (isTRUE(recursive))
      ret = as.integer64(x)
    else
      ret = lapply(seq_along(x), function(el) if (el <= length(x32)) as.integer64(x[[el]]) else x[[el]])
    names(ret) = names(x)
    ret
  }
  
  x32 = 1:10
  x64 = as.integer64(x32) 

  expect_identical(c(A=integer64(), B=as.raw(x32)), convert_x32_result_to_integer64(c(A=integer(), B=as.raw(x32)), recursive=TRUE))
  expect_identical(c(A=integer64(), B=as.logical(x32)), convert_x32_result_to_integer64(c(A=integer(), B=as.logical(x32)), recursive=TRUE))
  expect_identical(c(A=integer64(), B=as.difftime(x32, units="secs")), convert_x32_result_to_integer64(c(A=integer(), B=as.difftime(x32, units="secs")), recursive=TRUE))
  expect_identical(c(A=integer64(), B=as.POSIXct(x32, origin="2026-01-27")), convert_x32_result_to_integer64(c(A=integer(), B=as.POSIXct(x32, origin="2026-01-27")), recursive=TRUE))
  expect_identical(c(A=integer64(), B=as.Date(x32, origin="2026-01-27")), convert_x32_result_to_integer64(c(A=integer(), B=as.Date(x32, origin="2026-01-27")), recursive=TRUE))
  expect_identical(c(A=integer64(), B=as.factor(x32)), convert_x32_result_to_integer64(c(A=integer(), B=as.factor(x32)), recursive=TRUE))
  expect_identical(c(A=integer64(), B=as.ordered(x32)), convert_x32_result_to_integer64(c(A=integer(), B=as.ordered(x32)), recursive=TRUE))
  expect_identical(c(A=integer64(), B=as.double(x32)), convert_x32_result_to_integer64(c(A=integer(), B=as.double(x32)), recursive=TRUE))
  expect_identical(c(A=integer64(), B=x32), convert_x32_result_to_integer64(c(A=integer(), B=x32), recursive=TRUE))
  expect_identical(c(A=integer64(), B=as.complex(x32)), c(A=integer(), B=as.complex(x32)))
  # TODO(#44): adjust tests accordingly
  expect_identical(c(A=integer64(), B=as.character(x32)), convert_x32_result_to_integer64(c(A=integer(), B=as.character(x32)), recursive=TRUE))
  withr::with_options(list(bit64.promoteInteger64ToCharacter=TRUE), {
    expect_identical(c(A=integer64(), B=as.character(x32)), c(A=integer(), B=as.character(x32)))
  })
  
  expect_identical(c(A=x64, B=as.raw(x32)), convert_x32_result_to_integer64(c(A=x32, B=as.raw(x32)), recursive=TRUE))
  expect_identical(c(A=x64, B=as.logical(x32)), convert_x32_result_to_integer64(c(A=x32, B=as.logical(x32)), recursive=TRUE))
  expect_identical(c(A=x64, B=as.difftime(x32, units="secs")), convert_x32_result_to_integer64(c(A=x32, B=as.difftime(x32, units="secs")), recursive=TRUE))
  expect_identical(c(A=x64, B=as.POSIXct(x32, origin="2026-01-27")), convert_x32_result_to_integer64(c(A=x32, B=as.POSIXct(x32, origin="2026-01-27")), recursive=TRUE))
  expect_identical(c(A=x64, B=as.Date(x32, origin="2026-01-27")), convert_x32_result_to_integer64(c(A=x32, B=as.Date(x32, origin="2026-01-27")), recursive=TRUE))
  expect_identical(c(A=x64, B=as.factor(x32)), convert_x32_result_to_integer64(c(A=x32, B=as.factor(x32)), recursive=TRUE))
  expect_identical(c(A=x64, B=as.ordered(x32)), convert_x32_result_to_integer64(c(A=x32, B=as.ordered(x32)), recursive=TRUE))
  expect_identical(c(A=x64, B=as.double(x32)), convert_x32_result_to_integer64(c(A=x32, B=as.double(x32)), recursive=TRUE))
  expect_identical(c(A=x64, B=x32), convert_x32_result_to_integer64(c(A=x32, B=x32), recursive=TRUE))
  expect_identical(c(A=x64, B=as.complex(x32)), c(A=x32, B=as.complex(x32)))
  # TODO(#44): adjust tests accordingly
  expect_identical(c(A=x64, B=as.character(x32)), convert_x32_result_to_integer64(c(A=x32, B=as.character(x32)), recursive=TRUE))
  withr::with_options(list(bit64.promoteInteger64ToCharacter=TRUE), {
    expect_identical(c(A=x64, B=as.character(x32)), c(A=x32, B=as.character(x32)))
  })

  # regarding lists  
  expect_identical(c(A=integer64(), B=list()), convert_x32_result_to_integer64(c(A=integer(), B=list()), integer()))
  expect_identical(c(A=integer64(), B=list(), recursive=TRUE), convert_x32_result_to_integer64(c(A=integer64(), B=list(), recursive=TRUE), integer(), recursive=TRUE))
  expect_identical(c(A=integer64(), B=list(), C=1:2), convert_x32_result_to_integer64(c(A=integer(), B=list(), C=1:2), integer()))
  expect_identical(c(A=integer64(), B=list(), C=1:2, recursive=TRUE), convert_x32_result_to_integer64(c(A=integer(), B=list(), C=1:2, recursive=TRUE), integer(), recursive=TRUE))
  
  expect_identical(c(A=x64, B=list()), convert_x32_result_to_integer64(c(A=x32, B=list()), x32))
  expect_identical(c(A=x64, B=list(), recursive=TRUE), convert_x32_result_to_integer64(c(A=x32, B=list(), recursive=TRUE), x32, recursive=TRUE))
  expect_identical(c(A=x64, B=list(), C=1:2), convert_x32_result_to_integer64(c(A=x32, B=list(), C=1:2), x32))
  expect_identical(c(A=x64, B=list(), C=1:2, recursive=TRUE), convert_x32_result_to_integer64(c(A=x32, B=list(), C=1:2, recursive=TRUE), x32, recursive=TRUE))
  expect_identical(c(A=x64, B=list(a=1:2, b=3, c="4")), convert_x32_result_to_integer64(c(A=x32, B=list(a=1:2, b=3, c="4")), x32))
  # TODO(#44): adjust tests accordingly
  expect_identical(c(A=x64, B=list(a=1:2, b=3, c="4"), recursive=TRUE), convert_x32_result_to_integer64(c(A=x32, B=list(a=1:2, b=3, c="4"), recursive=TRUE), x32, recursive=TRUE))
  withr::with_options(list(bit64.promoteInteger64ToCharacter=TRUE), {
    expect_identical(c(A=x64, B=list(a=1:2, b=3, c="4"), recursive=TRUE), c(A=x32, B=list(a=1:2, b=3, c="4"), recursive=TRUE))
  })
  expect_identical(c(A=x64, B=data.frame(a=1:2, b=3, c="4", stringsAsFactors=FALSE)), convert_x32_result_to_integer64(c(A=x32, B=data.frame(a=1:2, b=3, c="4", stringsAsFactors=FALSE)), x32))
  # TODO(#44): adjust tests accordingly
  expect_identical(c(A=x64, B=data.frame(a=1:2, b=3, c="4", stringsAsFactors=FALSE), recursive=TRUE), convert_x32_result_to_integer64(c(A=x32, B=data.frame(a=1:2, b=3, c="4", stringsAsFactors=FALSE), recursive=TRUE), x32, recursive=TRUE))
  withr::with_options(list(bit64.promoteInteger64ToCharacter=TRUE), {
    expect_identical(c(A=x64, B=data.frame(a=1:2, b=3, c="4", stringsAsFactors=FALSE), recursive=TRUE), c(A=x32, B=data.frame(a=1:2, b=3, c="4", stringsAsFactors=FALSE), recursive=TRUE))
  })
  
  expect_identical(c(x64, as.POSIXlt(x32, origin="2026-01-27")), convert_x32_result_to_integer64(c(x32, as.POSIXlt(x32, origin="2026-01-27")), x32))
  if (getRversion() >= "4.0.0" ) # in my tests on R 3.5.0 this is identical
    expect_identical(c(x64, as.POSIXlt(x32, origin="2026-01-27"), recursive=TRUE), c(x32, as.POSIXlt(x32, origin="2026-01-27"), recursive=TRUE))

  expect_identical(
    c(A=x64, B=list(aa=x64, bb=list(bba=x64, bbb=x32, bbc=as.character(x32))), C=data.frame(a=x64, b=x32)),
    convert_x32_result_to_integer64(c(A=x32, B=list(aa=x64, bb=list(bba=x64, bbb=x32, bbc=as.character(x32))), C=data.frame(a=x64, b=x32)), x32)
  )
  # TODO(#44): adjust tests accordingly
  expect_identical(
    c(A=x64, B=list(aa=x64, bb=list(bba=x64, bbb=x32, bbc=as.character(x32))), C=data.frame(a=x64, b=x32), recursive=TRUE),
    convert_x32_result_to_integer64(c(A=x32, B=list(aa=x32, bb=list(bba=x32, bbb=x32, bbc=as.character(x32))), C=data.frame(a=x32, b=x32), recursive=TRUE), recursive=TRUE)
  )      
  withr::with_options(list(bit64.promoteInteger64ToCharacter=TRUE), {
    expect_identical(
      c(A=x64, B=list(aa=x64, bb=list(bba=x64, bbb=x32, bbc=as.character(x32))), C=data.frame(a=x64, b=x32), recursive=TRUE),
      c(A=x32, B=list(aa=x32, bb=list(bba=x32, bbb=x32, bbc=as.character(x32))), C=data.frame(a=x32, b=x32), recursive=TRUE)
    )      
  })
})

test_that("c works on extended integer64 objects (#298)", {
  x = y = as.integer64(1L)
  class(x) = c('foo', 'integer64')
  expect_identical(c(x, y), as.integer64(c(1L, 1L)))
  expect_identical(c(a=x, b=y), setNames(as.integer64(c(1L, 1L)), c("a", "b")))
  expect_identical(cbind(x, y), matrix64(c(1L, 1L), nrow=1L, ncol=2L, dimnames=list(NULL, c("x", "y"))))
  expect_identical(rbind(x, y), matrix64(c(1L, 1L), nrow=2L, ncol=1L, dimnames=list(c("x", "y"), NULL)))
})

replace_dimnames = function(x, old, new) {
  if (!is.null(dn <- dimnames(x))) 
    dimnames(x) = lapply(dn, function(el) {el[el == old] = new; el})
  x
}
test_that("cbind works consistent to R", {
  convert_x32_result_to_integer64 = function(x, colsToConvert=NULL) {
    if (!is.matrix(x) && !is.data.frame(x)) return(x)
    if (is.data.frame(x)) {
      for (col in colsToConvert)
        x[[col]] = as.integer64(x[[col]])
      return(x)
    }
    if (is.list(x)) {
      nrow_x = nrow(x)
      for (col in colsToConvert)
        for (ii in seq_len(nrow_x))
          x[[(col - 1L)*nrow_x + ii]] = as.integer64(x[[(col - 1L)*nrow_x + ii]])
      x = replace_dimnames(x, "x32", "x64")
    } else { # is.matrix(x)
      x = replace_dimnames(matrix(as.integer64(x), nrow=nrow(x), ncol=ncol(x), dimnames=dimnames(x)), "x32", "x64")
      oldClass(x) = "integer64"
    }
    x
  }

  x32 = 1:10
  x64 = as.integer64(x32)

  expect_identical(cbind(x64, as.raw(5L)), convert_x32_result_to_integer64(cbind(x32, as.raw(5L))))
  expect_identical(cbind(x64, FALSE), convert_x32_result_to_integer64(cbind(x32, FALSE)))
  expect_identical(cbind(x64, 0L), convert_x32_result_to_integer64(cbind(x32, 0L)))
  expect_identical(cbind(x64, 0.0), convert_x32_result_to_integer64(cbind(x32, 0.0)))
  expect_identical(cbind(x64, 0.0+0.0i), replace_dimnames(cbind(x32, 0.0+0.0i), "x32", "x64"))
  expect_identical(cbind(x64, as.difftime(0.0, units="secs")), convert_x32_result_to_integer64(cbind(x32, as.difftime(0.0, units="secs"))))
  expect_identical(cbind(x64, as.POSIXct(0.0, origin="2026-01-27")), convert_x32_result_to_integer64(cbind(x32, as.POSIXct(0.0, origin="2026-01-27"))))
  suppressWarnings(expect_identical(cbind(x64, as.POSIXlt(0.0, origin="2026-01-27")), convert_x32_result_to_integer64(cbind(x32, as.POSIXlt(0.0, origin="2026-01-27")), 1L)))
  expect_identical(cbind(x64, as.Date(0.0, origin="2026-01-27")), convert_x32_result_to_integer64(cbind(x32, as.Date(0.0, origin="2026-01-27"))))
  expect_identical(cbind(x64, as.factor(1L)), convert_x32_result_to_integer64(cbind(x32, as.factor(1L))))
  expect_identical(cbind(x64, as.ordered(1L)), convert_x32_result_to_integer64(cbind(x32, as.ordered(1L))))
  expect_identical(cbind(x64, as.integer64(0L)), replace_dimnames(cbind(x32, as.integer64(0L)), "x32", "x64"))
  expect_identical(cbind(integer64()), convert_x32_result_to_integer64(cbind(integer())))
  expect_identical(cbind(NA_integer64_), convert_x32_result_to_integer64(cbind(NA_integer_)))
  expect_identical(cbind(rep(NA_integer64_, 2)), convert_x32_result_to_integer64(cbind(rep(NA_integer_, 2))))
  expect_identical(cbind(x64), convert_x32_result_to_integer64(cbind(x32)))
  expect_identical(cbind(x64, c(TRUE, NA)), convert_x32_result_to_integer64(cbind(x32, c(TRUE, NA))))
  expect_identical(cbind(x64, c(42L, NA)), convert_x32_result_to_integer64(cbind(x32, c(42L, NA))))
  expect_identical(cbind(x64, c(42.0, NA)), convert_x32_result_to_integer64(cbind(x32, c(42.0, NA))))
  expect_identical(cbind(x64, c(42.0+42.0i, NA)), replace_dimnames(cbind(x32, c(42.0+42.0i, NA)), "x32", "x64"))
  # TODO(#44): adjust tests accordingly
  expect_identical(cbind(x64, c("42", NA)), convert_x32_result_to_integer64(cbind(x32, c("42", NA))))
  expect_identical(cbind(x64, character()), convert_x32_result_to_integer64(cbind(x32, character())))
  withr::with_options(list(bit64.promoteInteger64ToCharacter=TRUE), {
    expect_identical(cbind(x64, c("42", NA)), replace_dimnames(cbind(x32, c("42", NA)), "x32", "x64"))
    expect_identical(cbind(x64, character()), replace_dimnames(cbind(x32, character()), "x32", "x64"))
  })
  expect_identical(cbind(x64, complex()), replace_dimnames(cbind(x32, complex()), "x32", "x64"))
  
  expect_identical(
    cbind(A=x64, B=list()),
    convert_x32_result_to_integer64(cbind(A=x32, B=list()), 1L)
  )
  expect_identical(
    cbind(A=x64, B=list(a=1:10, b=1:2)),
    convert_x32_result_to_integer64(cbind(A=x32, B=list(a=1:10, b=1:2)), 1L)
  )
  expect_identical(
    cbind(A=matrix(x64, 5), B=list()),
    convert_x32_result_to_integer64(cbind(A=matrix(x32, 5), B=list()), 1:2)
  )
  expect_identical(
    cbind(A=matrix(x64, 5), B=list(), C=1:5),
    convert_x32_result_to_integer64(cbind(A=matrix(x32, 5), B=list(), C=1:5), 1:3)
  )
  suppressWarnings(expect_identical(
    cbind(A=matrix(x64, 5), B=list(a=1:10, b=1:2), C=1:5),
    convert_x32_result_to_integer64(cbind(A=matrix(x32, 5), B=list(a=1:10, b=1:2), C=1:5), c(1, 2, 4))
  ))
  skip_unless_r(">= 4.0.0") # in my tests on R 3.5.0 this is identical
  expect_same_error(
    cbind(matrix(x64, 5), list(), NULL, matrix(1:10, 2)),
    cbind(matrix(x32, 5), list(), NULL, matrix(1:10, 2))
  )
  expect_same_error(
    cbind(matrix(x64, 5), list(), NULL, data.frame(a=10:1, b=LETTERS[1:10]), stringsAsFactors=FALSE),
    cbind(matrix(x32, 5), list(), NULL, data.frame(a=10:1, b=LETTERS[1:10]), stringsAsFactors=FALSE)
  )
  expect_identical(
    cbind(matrix(x64, 5), data.frame(a=5:1, b=LETTERS[1:5], stringsAsFactors=FALSE)), 
    convert_x32_result_to_integer64(cbind(matrix(x32, 5), data.frame(a=5:1, b=LETTERS[1:5], stringsAsFactors=FALSE)), colsToConvert=1:2)
  )
  expect_same_error(
    cbind(matrix(x64, 5), data.frame(a=9:1, b=LETTERS[1:9], stringsAsFactors=FALSE)),
    cbind(matrix(x32, 5), data.frame(a=9:1, b=LETTERS[1:9], stringsAsFactors=FALSE))
  )
  expect_identical(
    cbind(matrix(x64, 5), data.frame(a=10:1, b=LETTERS[1:10], stringsAsFactors=FALSE)), 
    convert_x32_result_to_integer64(cbind(matrix(x32, 5), data.frame(a=10:1, b=LETTERS[1:10], stringsAsFactors=FALSE)), 1:2) 
  )
  expect_identical(
    cbind(matrix(x64, 5), data.frame(a=10:1, b=LETTERS[1:10], stringsAsFactors=FALSE), yy=as.integer64(-(1:10))), 
    convert_x32_result_to_integer64(cbind(matrix(x32, 5), data.frame(a=10:1, b=LETTERS[1:10], stringsAsFactors=FALSE), yy=as.integer(-(1:10))), c(1:2, 5))
  )
  expect_identical(
    cbind(matrix(x64, 5), data.frame(a=10:1, b=LETTERS[1:10], stringsAsFactors=FALSE), yy=as.integer64(-(1:2))), 
    convert_x32_result_to_integer64(cbind(matrix(x32, 5), data.frame(a=10:1, b=LETTERS[1:10], stringsAsFactors=FALSE), yy=-(1:2)), c(1:2, 5))
  )
  expect_identical(
    cbind(matrix(x64, 5), data.frame(a=as.integer64(10:1), b=LETTERS[1:10], stringsAsFactors=FALSE), yy=as.integer64(-(1:2))), 
    convert_x32_result_to_integer64(cbind(matrix(x32, 5), data.frame(a=as.integer64(10:1), b=LETTERS[1:10], stringsAsFactors=FALSE), yy=-(1:2)), c(1:2, 5))
  )
  expect_identical(
    tryCatch(cbind(as.integer64(1:2), matrix(x64, 5)), warning=conditionMessage),
    tryCatch(cbind(1:2, matrix(x32, 5)), warning=conditionMessage)
  )
  suppressWarnings(expect_identical(
    cbind(as.integer64(1:2), matrix(x64, 5)), 
    convert_x32_result_to_integer64(cbind(1:2, matrix(x32, 5)))
  ))
})

with_parameters_test_that("cbind deparse.level works consistent to R", {
  skip_unless_r(">= 4.0.0")
  x = 1L
  expected_result = base::cbind(a=x, x, x + 1L, deparse.level=deparse.level)
  expected_result = structure(as.integer64(expected_result), dim = dim(expected_result), dimnames = dimnames(expected_result))

  FUN = cbind
  x = as.integer64(x)
  expect_identical(cbind(a=x, x, x + 1L, deparse.level=deparse.level), expected_result)
  expect_identical(FUN(a=x, x, x + 1L, deparse.level=deparse.level), expected_result)
}, deparse.level = -1:3)

test_that("rbind works consistent to R", {
  convert_x32_result_to_integer64 = function(x, rowsToConvert=NULL) {
    if (!is.matrix(x) && !is.data.frame(x)) return(x)
    if (is.data.frame(x)) {
      for (row in rowsToConvert)
        x[[row]] = as.integer64(x[[row]])
      return(x)
    }
    if (is.list(x)) {
      nrow_x = nrow(x)
      for (row in rowsToConvert)
        for (ii in seq_len(ncol(x)))
          x[[(ii - 1L)*nrow_x + row]] = as.integer64(x[[(ii - 1L)*nrow_x + row]])
      x = replace_dimnames(x, "x32", "x64")
    } else { # is.matrix(x)
      x = replace_dimnames(matrix(as.integer64(x), nrow=nrow(x), ncol=ncol(x), dimnames=dimnames(x)), "x32", "x64")
      oldClass(x) = "integer64"
    }
    x
  }
  
  x32 = 1:10
  x64 = as.integer64(x32)

  expect_identical(rbind(x64, as.raw(5L)), convert_x32_result_to_integer64(rbind(x32, as.raw(5L))))
  expect_identical(rbind(x64, FALSE), convert_x32_result_to_integer64(rbind(x32, FALSE)))
  expect_identical(rbind(x64, 0L), convert_x32_result_to_integer64(rbind(x32, 0L)))
  expect_identical(rbind(x64, 0.0), convert_x32_result_to_integer64(rbind(x32, 0.0)))
  expect_identical(rbind(x64, 0.0+0.0i),  replace_dimnames(rbind(x32, 0.0+0.0i), "x32", "x64"))
  expect_identical(rbind(x64, as.difftime(0.0, units="secs")), convert_x32_result_to_integer64(rbind(x32, as.difftime(0.0, units="secs"))))
  expect_identical(rbind(x64, as.POSIXct(0.0, origin="2026-01-27")), convert_x32_result_to_integer64(rbind(x32, as.POSIXct(0.0, origin="2026-01-27"))))
  suppressWarnings(expect_identical(rbind(x64, as.POSIXlt(0.0, origin="2026-01-27")), convert_x32_result_to_integer64(rbind(x32, as.POSIXlt(0.0, origin="2026-01-27")), 1L)))
  expect_identical(rbind(x64, as.Date(0.0, origin="2026-01-27")), convert_x32_result_to_integer64(rbind(x32, as.Date(0.0, origin="2026-01-27"))))
  expect_identical(rbind(x64, as.factor(1L)), convert_x32_result_to_integer64(rbind(x32, as.factor(1L))))
  expect_identical(rbind(x64, as.ordered(1L)), convert_x32_result_to_integer64(rbind(x32, as.ordered(1L))))
  expect_identical(rbind(x64, as.integer64(0L)), replace_dimnames(rbind(x32, as.integer64(0L)), "x32", "x64"))
  expect_identical(rbind(integer64()), convert_x32_result_to_integer64(rbind(integer())))
  expect_identical(rbind(NA_integer64_), convert_x32_result_to_integer64(rbind(NA_integer_)))
  expect_identical(rbind(rep(NA_integer64_, 2)), convert_x32_result_to_integer64(rbind(rep(NA_integer_, 2))))
  expect_identical(rbind(x64), convert_x32_result_to_integer64(rbind(x32)))
  expect_identical(rbind(x64, c(TRUE, NA)), convert_x32_result_to_integer64(rbind(x32, c(TRUE, NA))))
  expect_identical(rbind(x64, c(42L, NA)), convert_x32_result_to_integer64(rbind(x32, c(42L, NA))))
  expect_identical(rbind(x64, c(42.0, NA)), convert_x32_result_to_integer64(rbind(x32, c(42.0, NA))))
  expect_identical(rbind(x64, c(42.0+42.0i, NA)), replace_dimnames(rbind(x32, c(42.0+42.0i, NA)), "x32", "x64"))
  # TODO(#44): adjust tests accordingly
  expect_identical(rbind(x64, c("42", NA)), convert_x32_result_to_integer64(rbind(x32, c("42", NA))))
  expect_identical(rbind(x64, character()), convert_x32_result_to_integer64(rbind(x32, character())))
  withr::with_options(list(bit64.promoteInteger64ToCharacter=TRUE), {
    expect_identical(rbind(x64, c("42", NA)), replace_dimnames(rbind(x32, c("42", NA)), "x32", "x64"))
    expect_identical(rbind(x64, character()), replace_dimnames(rbind(x32, character()), "x32", "x64"))
  })
  expect_identical(rbind(x64, complex()), replace_dimnames(rbind(x32, complex()), "x32", "x64"))
  
  expect_identical(
    rbind(A=x64, B=list()),
    convert_x32_result_to_integer64(rbind(A=x32, B=list()), 1L)
  )
  expect_identical(
    rbind(A=x64, B=list(a=1:10, b=1:2)),
    convert_x32_result_to_integer64(rbind(A=x32, B=list(a=1:10, b=1:2)), 1L)
  )
  expect_identical(
    rbind(A=matrix(x64, 5), B=list()),
    convert_x32_result_to_integer64(rbind(A=matrix(x32, 5), B=list()), 1:5)
  )
  suppressWarnings(expect_identical(
    rbind(A=matrix(x64, 5), B=list(), C=1:5),
    convert_x32_result_to_integer64(rbind(A=matrix(x32, 5), B=list(), C=1:5), 1:6)
  ))
  suppressWarnings(expect_identical(
    rbind(A=matrix(x64, 5), B=list(a=1:10, b=1:2), C=1:5),
    convert_x32_result_to_integer64(rbind(A=matrix(x32, 5), B=list(a=1:10, b=1:2), C=1:5), c(1:5, 7))
  ))
  skip_unless_r(">= 4.0.0") # in my tests on R 3.5.0 this is identical, except the one with as.complex() and "bit64.promoteInteger64ToCharacter=TRUE", because a factor instead of character is returned.
  expect_same_error( # mismatch column count
    rbind(matrix(x64, 5), list(), NULL, matrix(1:10, 2)),
    rbind(matrix(x32, 5), list(), NULL, matrix(1:10, 2))
  )
  # suppressWarnings(): character->integer64 coercion because of bit64.promoteInteger64ToCharacter=FALSE prior to error.
  #   TODO(#44): remove suppression.
  expect_same_error( # no column names in matrix
    suppressWarnings(rbind(matrix(x64, 5), list(), NULL, data.frame(a=10:1, b=LETTERS[1:10], stringsAsFactors=FALSE))),
    rbind(matrix(x32, 5), list(), NULL, data.frame(a=10:1, b=LETTERS[1:10], stringsAsFactors=FALSE))
  )
  expect_same_error( # ditto w/o intermediate empty inputs
    suppressWarnings(rbind(matrix(x64, 5), data.frame(a=5:1, b=LETTERS[1:5], stringsAsFactors=FALSE))),
    rbind(matrix(x32, 5), data.frame(a=5:1, b=LETTERS[1:5], stringsAsFactors=FALSE))
  )
  # TODO(#44): adjust tests accordingly
  expect_identical(
    rbind(matrix(x64, 5, dimnames=list(NULL, c("a", "b"))), data.frame(a=5:1, b=as.character(1:5), stringsAsFactors=FALSE)), 
    convert_x32_result_to_integer64(rbind(matrix(x32, 5, dimnames=list(NULL, c("a", "b"))), data.frame(a=5:1, b=1:5)), 1:2)
  )
  withr::with_options(list(bit64.promoteInteger64ToCharacter=TRUE), {
    expect_identical(
      rbind(matrix(x64, 5, dimnames=list(NULL, c("a", "b"))), data.frame(a=5:1, b=as.character(1:5), stringsAsFactors=FALSE)),
      rbind(matrix(x32, 5, dimnames=list(NULL, c("a", "b"))), data.frame(a=as.character(5:1), b=as.character(1:5), stringsAsFactors=FALSE))
      # This would be consistent with base R, but it requires additional logic to determine the columns to convert from the input in advance to have them coerced accordingly.
      # convert_x32_result_to_integer64(rbind(matrix(x32, 5, dimnames=list(NULL, c("a", "b"))), data.frame(a=5:1, b=as.character(1:5), stringsAsFactors=FALSE)), 1L)
    )
  })
  expect_identical(
    rbind(matrix(x64, 5, dimnames=list(NULL, c("a", "b"))), data.frame(a=5:1, b=as.complex(1:5))),
    rbind(matrix(x32, 5, dimnames=list(NULL, c("a", "b"))), data.frame(a=as.complex(5:1), b=as.complex(1:5)))
    # This would be consistent with base R, but it requires additional logic to determine the columns to convert from the input in advance to have them coerced accordingly.
    # convert_x32_result_to_integer64(rbind(matrix(x32, 5, dimnames=list(NULL, c("a", "b"))), data.frame(a=5:1, b=as.complex(1:5))), 1L)
  )
  # TODO(#44): remove supression (as above).
  expect_same_error( # no column names in matrix
    suppressWarnings(rbind(matrix(x64, 5), data.frame(a=10:1, b=LETTERS[1:10], stringsAsFactors=FALSE), yy=as.integer64(-(1:10)))),
    rbind(matrix(x32, 5), data.frame(a=10:1, b=LETTERS[1:10], stringsAsFactors=FALSE), yy=as.integer64(-(1:10)))
  )
  expect_identical(
    tryCatch(rbind(as.integer64(1:5), matrix(x64, 5)), warning=conditionMessage),
    tryCatch(rbind(1:5, matrix(x32, 5)), warning=conditionMessage)
  )
  suppressWarnings(expect_identical(
    rbind(as.integer64(1:5), matrix(x64, 5)), 
    convert_x32_result_to_integer64(rbind(1:5, matrix(x32, 5)))
  ))

  expect_identical(rbind(integer64(), data.frame(a=1:2, b=10:11)), convert_x32_result_to_integer64(rbind(integer(), data.frame(a=1:2, b=10:11)), 1:2))
  expect_identical(rbind(integer64(1), data.frame(a=1:2, b=10:11)), convert_x32_result_to_integer64(rbind(integer(1), data.frame(a=1:2, b=10:11)), 1:2))
  
  suppressWarnings(expect_identical(rbind(integer64(3), data.frame(a=1:2, b=10:11)), convert_x32_result_to_integer64(rbind(integer(3), data.frame(a=1:2, b=10:11)), 1:2)))
  expect_identical(
    tryCatch(rbind(integer64(3), data.frame(a=1:2, b=10:11)), warning=conditionMessage), 
    tryCatch(convert_x32_result_to_integer64(rbind(integer(3), data.frame(a=1:2, b=10:11)), 1:2), warning=conditionMessage)
  )
})

with_parameters_test_that("rbind deparse.level works consistent to R", {
  skip_unless_r(">= 4.0.0")
  x = 1L
  expected_result = base::rbind(a=x, x, x + 1L, deparse.level=deparse.level)
  expected_result = structure(as.integer64(expected_result), dim = dim(expected_result), dimnames = dimnames(expected_result))

  FUN = rbind
  x = as.integer64(x)
  expect_identical(rbind(a=x, x, x + 1L, deparse.level=deparse.level), expected_result)
  expect_identical(FUN(a=x, x, x + 1L, deparse.level=deparse.level), expected_result)
}, deparse.level = -1:3)

test_that("seq.integer64 works with S4 subclasses inheriting from integer64", {
  # Use a unique class name to avoid conflicts
  className <- "test64_seq_regression"
  if (!methods::isClass(className)) {
    methods::setClass(className, contains = "integer64")
    withr::defer(methods::removeClass(className))
  }

  from <- as.integer64(1L)
  to <- as.integer64(10L)
  by <- methods::new(className, as.integer64(2))

  expect_identical(seq(from=from, to=to, by=by), as.integer64(c(1L, 3L, 5L, 7L, 9L)))
  expect_identical(seq(to=to, by=by, length.out=5L), as.integer64(c(2L, 4L, 6L, 8L, 10L)))
  expect_identical(seq(from=from, by=by, length.out=5L), as.integer64(c(1L, 3L, 5L, 7L, 9L)))
})

test_that("back-compatible keep.names=TRUE is supported for limited input classes", {
  x = as.integer64(1L)
  names(x) = "a"
  expect_named(as.integer64(x, keep.names=TRUE), "a")

  y = c(a = 1.0)
  expect_named(as.integer64(y, keep.names=TRUE), "a")
})

test_that("bitstring class meshes with R's own (from 4.6.0)", {
  expect_output(print(as.bitstring(lim.integer64())), '0000')
})

test_that("match & %in% basics work", {
  x = as.integer64(2:5)
  y = as.integer64(3:6)
  expect_identical(match(x, y), c(NA, 1:3))
  expect_identical(match(y, x), c(2:4, NA))

  expect_identical(match(2:5, y), c(NA, 1:3))
  expect_identical(match(as.numeric(2:5), y), c(NA, 1:3))
  expect_identical(match(y, 2:5), c(2:4, NA))
  expect_identical(match(y, as.numeric(2:5)), c(2:4, NA))

  expect_identical(match(x, y, nomatch=0L), 0:3)

  expect_identical(x %in% y, c(FALSE, TRUE, TRUE, TRUE))
  expect_identical(y %in% x, c(TRUE, TRUE, TRUE, FALSE))
  expect_identical(x %in% 3:6, c(FALSE, TRUE, TRUE, TRUE))
  expect_identical(x %in% c(3.0, 4.0, 5.0, 6.0), c(FALSE, TRUE, TRUE, TRUE))

  expect_identical(match(integer64(), as.integer64(1L)), integer())
  expect_identical(match(as.integer64(1L), integer64()), NA_integer_)
  expect_identical(match(as.integer64(1L), integer64(), nomatch = 0L), 0L)

  x_nm <- as.integer64(c(1L, 3L))
  table_nm <- as.integer64(2:4)
  expect_identical(match(x_nm, table_nm, nomatch = -1L), c(-1L, 2L))

  expect_identical(integer64() %in% 1L, integer() %in% 1L)
  expect_identical(as.integer64(1L) %in% double(), 1L %in% double())
})

test_that("Different method= for match() and %in% work", {
  x = as.integer64(2:5)
  y = as.integer64(3:6)
  expected = c(NA_integer_, 1:3)

  expect_identical(match(x, y, method="hashpos"), expected)
  expect_identical(match(x, y, method="hashrev"), expected)
  expect_identical(match(x, y, method="sortorderpos"), expected)
  expect_error(match(x, y, method="_unknown_"), "'arg' should be one of", fixed=TRUE)
  expect_identical(match(x, y, method="orderpos"), expected)

  # NB: %in% is quite a bit different; while there's a public API to
  #   `%in%.integer64`, likely, there shouldn't be (it's strange to export
  #   an S3 method like is currently done). The tests are designed to tickle
  #   the different methods through the public API only; this makes them
  #   prone to winding up testing something totally different later. I think
  #   that's fine; now that we have coverage tests up, any refactor that bumps
  #   around what exactly the following tests are covering, will show up in the PR.

  # method="hashrin" used when x is "short" but table is "long"
  x = as.integer64(seq_len(10L))
  table = as.integer64(seq_len(2.0**16.0 * 2.0/3.0 + 10.0)) # invert condition for bx>=16, 10.0 arbitrary buffer
  expect_identical(x %in% table, rep(TRUE, 10L))
})

test_that("match.integer64: automatic method selection without cache", {
  # hashrev path: short x, long table
  nx <- 10L
  ny <- 2L^16L
  x <- as.integer64(1:nx)
  table <- as.integer64(1:ny)
  # As of this writing, this should invoke hashrev
  expect_identical(match(x, table), 1:nx)

  # hashpos path: long x, short table
  x_long_match <- as.integer64(1:ny)
  table_short_match <- as.integer64(1:nx)
  # As of this writing, this should use hashpos
  expect_identical(match(x_long_match[1:nx], table_short_match), 1:nx)
  expect_identical(match(x_long_match[(nx+1L):(nx+10L)], table_short_match), rep(NA_integer_, 10L))
})

test_that("%in%.integer64: automatic method selection without cache", {
  # hashrin path: short x, long table
  nx <- 10L
  ny <- 2L^16L
  x <- as.integer64(1:nx)
  table <- as.integer64(1:ny)
  # As of this writing, this should use hashrin
  expect_identical(x %in% table, rep(TRUE, nx))

  # hashfin path: long x, short table
  x_long_in <- as.integer64(1:ny)
  table_short_in <- as.integer64(1:nx)
  # As of this writing, this should use hashfin.
  expect_identical(x_long_in[1:nx] %in% table_short_in, rep(TRUE, nx))
  expect_identical(x_long_in[(nx+1L):(nx+10L)] %in% table_short_in, rep(FALSE, 10L))
})

test_that("match.integer64: cache-based method selection", {
  x <- as.integer64(c(1L, 3L, 5L))
  table <- as.integer64(c(2L, 4L, 3L, 1L))

  # hashcache
  hashcache(table)
  expect_identical(match(x, table), c(4L, 3L, NA_integer_))
  remcache(table)

  # sortordercache
  sortordercache(table)
  expect_identical(match(x, table), c(4L, 3L, NA_integer_))
  remcache(table)

  # ordercache
  ordercache(table)
  expect_identical(match(x, table), c(4L, 3L, NA_integer_))
  remcache(table)
})

test_that("%in%.integer64: cache-based method selection", {
  x <- as.integer64(c(1L, 3L, 5L))
  table <- as.integer64(c(2L, 4L, 3L, 1L))
  expected_in <- c(TRUE, TRUE, FALSE)

  # hashcache
  hashcache(table)
  expect_identical(x %in% table, expected_in)
  remcache(table)

  # sortcache
  sortcache(table)
  expect_identical(x %in% table, expected_in)
  remcache(table)

  # ordercache
  ordercache(table)
  expect_identical(x %in% table, expected_in)
  remcache(table)
})

test_that("match.integer64: nunique argument", {
  x <- as.integer64(c(1L, 3L, 5L))
  table <- as.integer64(c(2L, 4L, 3L, 1L, 3L))
  expect_identical(match(x, table, nunique = 4L), c(4L, 3L, NA_integer_))
})

test_that("%in%.integer64: nunique argument", {
  x <- as.integer64(c(1L, 3L, 5L))
  table <- as.integer64(c(2L, 4L, 3L, 1L, 3L))
  expect_identical(x %in% table, c(TRUE, TRUE, FALSE))
})

test_that("%in%.integer64: with NA values", {
  x <- as.integer64(c(1L, NA, 3L))
  table <- as.integer64(c(NA, 2L, 1L))
  expect_identical(x %in% table, c(TRUE, TRUE, FALSE))
})

test_that("duplicated, unique, table methods work", {
  x = as.integer64(1:3)
  expect_identical(duplicated(x), rep(FALSE, 3L))
  expect_identical(unique(x), x)
  expect_identical(table(x), table(x = 1:3))

  x = as.integer64(rep(1L, 3L))
  expect_identical(duplicated(x), c(FALSE, TRUE, TRUE))
  expect_identical(unique(x), x[1L])
  expect_identical(table(x), table(x = rep(1L, 3L)))

  x = as.integer64(c(1L, 2L, 1L))
  expect_identical(duplicated(x), c(FALSE, FALSE, TRUE))
  expect_identical(unique(x), x[1:2])
  expect_identical(table(x), table(x = c(1L, 2L, 1L)))

  x = as.integer64(c(1L, 1L, 2L))
  expect_identical(duplicated(x), c(FALSE, TRUE, FALSE))
  expect_identical(unique(x), x[c(1L, 3L)])
  expect_identical(table(x), table(x = c(1L, 1L, 2L)))

  x = c(132724613L, -2143220989L)
  expect_identical(table(x=as.integer64(x)), table(x))
  expect_identical(table(x, y=lim.integer64()), table(x=as.character(x), y=as.character(lim.integer64())))
  expect_identical(table(y=lim.integer64(), x), table(y=as.character(lim.integer64()), x=as.character(x)))

  x = as.integer64(c(1L, 1L, 2L))
  expect_error(duplicated(x, method="_unknown_"), "'arg' should be one of", fixed=TRUE)
  expect_error(unique(x, method="_unknown_"), "'arg' should be one of", fixed=TRUE)
})

test_that("exclude, useNA arguments work for integer64 method of table", {
  a32 = c(1L, 1L, 2L)
  a64 = as.integer64(a32)
  c = c(2, NA, 4)

  expect_identical(
    table(a=a64, b=1:3, c=c, exclude=1, useNA="no"),
    table(a=a32, b=1:3, c=c, exclude=1, useNA="no")
  )

  skip_unless_r("> 3.5.0") # unclear what's going on
  expect_identical(
    table(a=a64, b=1:3, c=c, exclude=1, useNA="ifany"),
    table(a=a32, b=1:3, c=c, exclude=1, useNA="ifany")
  )

  expect_identical(
    table(a=a64, b=1:3, c=c, exclude=1, useNA="always"),
    table(a=a32, b=1:3, c=c, exclude=1, useNA="always")
  )
})

test_that("our overwrite of table() is consistent with base::table", {
  expect_identical(table(NULL), base::table(NULL))
  expect_identical(table(a=NULL), base::table(a=NULL))
  expect_identical(table(NULL, NULL), base::table(NULL, NULL))
  expect_identical(table(a=NULL, b=NULL), base::table(a=NULL, b=NULL))
  expect_identical(table(integer(), NULL), base::table(integer(), NULL))
  expect_identical(table(a=integer(), b=NULL), base::table(a=integer(), b=NULL))
  expect_same_error(table(1L, NULL), base::table(1L, NULL))
  expect_same_error(table(a=1L, b=NULL), base::table(a=1L, b=NULL))

  skip_unless_r("> 3.5.0") # unclear what's going on
  expected_result = withr::with_seed(1L, base::table(exclude=sample(1:10, 1), useNA=sample(c("no", "ifany", "always"), 1), deparse.level=sample(1:2, 1), sample(1:10)))
  expect_identical(
    withr::with_seed(1L, table(exclude=sample(1:10, 1), useNA=sample(c("no", "ifany", "always"), 1), deparse.level=sample(1:2, 1), sample(as.integer64(1:10)))),
    expected_result
  )
  expect_identical(
    withr::with_seed(1L, table(exclude=sample(1:10, 1), useNA=sample(c("no", "ifany", "always"), 1), deparse.level=sample(1:2, 1), sample(1:10))),
    expected_result
  )

  expected_result = withr::with_seed(1L, base::table(exclude=sample(1:10, 1), deparse.level=sample(1:2, 1), sample(1:10)))
  expect_identical(
    withr::with_seed(1L, table(exclude=sample(1:10, 1), deparse.level=sample(1:2, 1), sample(as.integer64(1:10)))),
    expected_result
  )
  expect_identical(
    withr::with_seed(1L, table(exclude=sample(1:10, 1), deparse.level=sample(1:2, 1), sample(1:10))),
    expected_result
  )

  expected_result = withr::with_seed(1L, base::table(exclude=sample(1:10, 1), useNA=sample(c("no", "ifany", "always"), 1), sample(1:10)))
  expect_identical(
    withr::with_seed(1L, table(exclude=sample(1:10, 1), useNA=sample(c("no", "ifany", "always"), 1), sample(as.integer64(1:10)))),
    expected_result
  )
  expect_identical(
    withr::with_seed(1L, table(exclude=sample(1:10, 1), useNA=sample(c("no", "ifany", "always"), 1), sample(1:10))),
    expected_result
  )
  
  expected_result = withr::with_seed(1L, base::table(useNA=sample(c("no", "ifany", "always"), 1), sample(1:10)))
  expect_identical(
    withr::with_seed(1L, table(useNA=sample(c("no", "ifany", "always"), 1), sample(as.integer64(1:10)))),
    expected_result
  )
  expect_identical(
    withr::with_seed(1L, table(useNA=sample(c("no", "ifany", "always"), 1), sample(1:10))),
    expected_result
  )
})

test_that("different method= for duplicated, unique work", {
  x = as.integer64(c(1L, 2L, 1L))
  exp_dup = c(FALSE, FALSE, TRUE)
  exp_unq = x[1:2]

  expect_identical(duplicated(x, method="hashdup"), exp_dup)
  expect_identical(unique(x, method="hashmapuni"), exp_unq)
  expect_identical(unique(x, method="hashuni"), exp_unq)

  expect_identical(duplicated(x, method="sortorderdup"), exp_dup)
  expect_identical(unique(x, method="sortorderuni"), exp_unq)
  expect_identical(unique(x, method="sortuni"), exp_unq)

  expect_identical(duplicated(x, method="orderdup"), exp_dup)
  expect_identical(unique(x, method="orderuni"), exp_unq)
})

test_that("more coercion works", {
  expect_identical(as.factor(as.integer64(2:4)), factor(2:4))
  expect_identical(as.ordered(as.integer64(2:4)), as.ordered(2:4))
  expect_identical(as.integer64(factor(2:11)), as.integer64(1:10)) # NB: _not_ 2:11!
})

test_that("sorting methods work", {
  x = as.integer64(c(10L, 4L, 8L))
  x_rank = c(3.0, 1.0, 2.0)
  expect_identical(rank(x), x_rank)
  expect_identical(rank(x, method="orderrnk"), x_rank)

  x = as.integer64(1:100)
  q = as.integer64(c(1L, 26L, 50L, 75L, 100L))
  expect_identical(quantile(x, names=FALSE), q)
  expect_identical(median(x), q[3L])
  names(q) = c('0%', '25%', '50%', '75%', '100%')
  expect_identical(quantile(x), q)
  expect_identical(quantile(x, 0.2, names=FALSE), as.integer64(21L))

  expect_error(quantile(x, type=7L), "only.*qtile.*supported")
  expect_error(quantile(NA_integer64_), "missing values not allowed")

  x = as.integer64(1:100)
  q = as.integer64(c(1L, 26L, 50L, 75L, 100L))
  names(q) = c('0%', '25%', '50%', '75%', '100%')
  expect_identical(qtile(x, method="sortqtl"), q)
  expect_identical(qtile(x, method="orderqtl"), q)

  x = as.integer64(c(1L, 1L, 2L, 3L, 2L, 4L))
  x_tiepos = c(1L, 2L, 3L, 5L)
  expect_identical(tiepos(x), x_tiepos)
  expect_identical(tiepos(x, method="ordertie"), x_tiepos)

  expect_error(rank(x, method="_unknown_"), "'arg' should be one of", fixed=TRUE)
  expect_error(qtile(x, method="_unknown_"), "'arg' should be one of", fixed=TRUE)
  expect_error(tiepos(x, method="_unknown_"), "'arg' should be one of", fixed=TRUE)
})

test_that("missing and empty inputs to median() are handled correctly", {
  expect_identical(median(NA_integer64_, na.rm=FALSE), NA_integer64_)
  expect_identical(median(NA_integer64_, na.rm=TRUE), NA_integer64_)
  expect_identical(median(integer64()), NA_integer64_)

  x = as.integer64(c(NA, 1L))
  expect_identical(median(x, na.rm=FALSE), NA_integer64_)
  expect_identical(median(x, na.rm=TRUE), as.integer64(1L))

  x = as.integer64(c(NA, NA))
  expect_identical(median(x, na.rm=FALSE), NA_integer64_)
  expect_identical(median(x, na.rm=TRUE), NA_integer64_)
})

# These tests were previously kept as tests under \examples{\dontshow{...}}.
#   Converted to "proper" unit tests for clarity, after making them more
#   canonical within {testthat}, e.g. better capturing expected warnings,
#   changing stopifnot(identical(...)) to expect_identical(...).
test_that("Old \\dontshow{} tests continue working", {
  xi = c(1L, 1L, 2L)
  xi64 = as.integer64(xi)
  yi = c(3L, 4L, 4L)
  yi64 = as.integer64(yi)

  t_xi = table(x=xi)
  t_xi_yi = table(x=xi, y=yi)

  expect_identical(table(x=xi64), t_xi)
  expect_identical(table(x=xi64, y=yi64), t_xi_yi)

  expect_identical(table(x=xi64, y=yi), t_xi_yi)
  expect_identical(table(x=xi, y=yi64), t_xi_yi)

})

test_that("unipos() works as intended", {
  x = as.integer64(c(1L, 2L, 1L, 3L, 2L, 4L))
  x_unipos = c(1L, 2L, 4L, 6L)
  expect_identical(unipos(x), x_unipos)
  expect_identical(unipos(x, method="hashupo"), x_unipos)
  expect_identical(unipos(x, method="sortorderupo"), x_unipos)
  expect_identical(unipos(x, method="orderupo"), x_unipos)
  expect_error(unipos(x, method="_unknown_"), "'arg' should be one of", fixed=TRUE)
})

test_that("keypos() works as intended", {
  x = as.integer64(c(5L, 2L, 5L, 3L, 2L, 4L))
  x_keypos = c(4L, 1L, 4L, 2L, 1L, 3L)
  expect_identical(keypos(x), x_keypos)
  expect_identical(keypos(x, method="orderkey"), x_keypos)
  expect_error(keypos(x, method="_unknown_"), "'arg' should be one of", fixed=TRUE)
})

test_that("summary() works as intended", {
  x = as.integer64(c(1L, 2L, 10L, 20L, NA, 30L))
  # NB: as.integer64() strips names, so as.integer64(c(Min. = ...)) won't work
  x_summary = as.integer64(c(1L, 2L, 10L, 12L, 20L, 30L, 1L))
  names(x_summary) = c("Min.", "1st Qu.", "Median", "Mean", "3rd Qu.", "Max.", "NA's")
  expect_identical(summary(x), x_summary)
  expect_identical(summary(x[-5L]), x_summary[-7L])
})

test_that("prank() works as intended", {
  x = as.integer64(1:100)
  expect_identical(prank(x), (x-1.0)/99.0)
  expect_identical(prank(x[1L]), NA_integer64_)
})

test_that("match.integer64 with method='orderpos' works", {
  x <- as.integer64(1:5)
  table <- as.integer64(3:7)
  expect_identical(match(x, table, method="orderpos"), c(NA, NA, 1:3))
})

test_that("match.integer64 with partial cache triggers fallback", {
  x <- as.integer64(1:5)
  table <- as.integer64(3:7)

  sortcache(table) # creates 'sort' and 'nunique' in cache

  # This should fallback to hashpos/hashrev logic.
  # x is small, table is small.
  expect_identical(match(x, table), c(NA_integer_, NA_integer_, 1L, 2L, 3L))

  remcache(table)
})

test_that("match.integer64 and %in% cover various cache states", {
  x = as.integer64(c(1L, 3L, 5L))
  table = as.integer64(c(2L, 4L, 3L, 1L))

  # Cover hashpos via cache selection
  hashcache(table)
  expect_identical(match(x, table), c(4L, 3L, NA_integer_))
  expect_identical(x %in% table, c(TRUE, TRUE, FALSE))
  remcache(table)

  # Cover sortorderpos/sortfin via cache selection
  sortordercache(table)
  expect_identical(match(x, table), c(4L, 3L, NA_integer_))
  expect_identical(x %in% table, c(TRUE, TRUE, FALSE))
  remcache(table)

  # Cover orderfin via cache selection (used by %in%)
  ordercache(table)
  expect_identical(x %in% table, c(TRUE, TRUE, FALSE))
  # Cover orderpos logic if possible via automatic selection
  # Note: explicit method="orderpos" has known issues in other tests,
  # but this checks the automatic selection path based on cache existence.
  remcache(table)

  # Explicitly trigger hashrev with cache (requires cache on x)
  hashcache(x)
  expect_identical(match(x, table, method="hashrev"), c(4L, 3L, NA_integer_))
  remcache(x)
})

test_that("duplicated.integer64 covers various cache states", {
  x = as.integer64(c(1L, 2L, 1L))
  res = c(FALSE, FALSE, TRUE)

  # Cover sortorderdup with existing cache (retrieving sort/order from cache)
  sortordercache(x)
  expect_identical(duplicated(x), res)
  # Force specific method to ensure cache path is used
  expect_identical(duplicated(x, method="sortorderdup"), res)
  remcache(x)

  # Cover hashdup with existing cache
  hashcache(x)
  expect_identical(duplicated(x), res)
  remcache(x)

  # Cover orderdup with existing cache
  ordercache(x)
  expect_identical(duplicated(x), res)
  expect_identical(duplicated(x, method="orderdup"), res)
  remcache(x)
})

test_that("unique.integer64 covers various cache states and order arguments", {
  x = as.integer64(c(3L, 1L, 3L))

  # order="original" + hashcache
  hashcache(x)
  expect_identical(unique(x, order="original"), x[1:2])
  remcache(x)

  # order="original" + ordercache
  ordercache(x)
  expect_identical(unique(x, order="original"), x[1:2])
  remcache(x)

  # order="original" + sortordercache
  sortordercache(x)
  expect_identical(unique(x, order="original"), x[1:2])
  remcache(x)

  # order="values" + sortcache (triggers sortuni from cache)
  sortcache(x)
  expect_identical(unique(x, order="values"), as.integer64(c(1L, 3L)))
  remcache(x)

  # order="values" + hashcache (triggers hashuni if nunique < length/2)
  x2 = as.integer64(c(1, 1, 1, 1, 1))
  hashcache(x2)
  expect_identical(unique(x2, order="values"), as.integer64(1L))
  remcache(x2)

  # order="any" + hashcache
  hashcache(x)
  res = unique(x, order="any")
  expect_setequal(res, as.integer64(c(1L, 3L)))
  remcache(x)
})

test_that("unipos.integer64 covers various cache states", {
  x = as.integer64(c(3L, 1L, 3L))
  # positions: 1, 2

  # order="original" + hashcache
  hashcache(x)
  expect_identical(unipos(x, order="original"), c(1L, 2L))
  remcache(x)

  # order="values" + sortcache (triggers sortorderupo)
  sortcache(x)
  expect_identical(unipos(x, order="values"), c(2L, 1L))
  remcache(x)

  # order="any" + sortordercache
  sortordercache(x)
  res = unipos(x, order="any")
  expect_true(setequal(res, c(1L, 2L)))
  remcache(x)
})

test_that("table.integer64 covers inputs, cache states, and return types", {
  x = as.integer64(c(1L, 2L, 1L))

  # List input handling
  t_list = table(list(x))
  t_vec = table(x)
  expect_identical(as.vector(t_list), as.vector(t_vec))
  expect_identical(dim(t_list), dim(t_vec))

  # Error: length mismatch
  expect_error(table(x, as.integer64(1:2)), "all arguments must have the same length")

  # return="data.frame"
  df = table(x, return="data.frame")
  expect_identical(df, data.frame(x=as.integer64(c(1, 2)), Freq=as.integer(c(2, 1))))

  # return="list"
  lst = table(x, return="list")
  expect_identical(lst$values, as.integer64(c(1, 2)))
  # Fix: Counts are standard integer
  expect_identical(lst$counts, as.integer(c(2, 1)))

  # order="counts"
  tbl_cnt = table(x, order="counts")
  # 2 appears 1x, 1 appears 2x. Sorted by counts ascending: 2, 1.
  expect_identical(as.vector(tbl_cnt), c(1L, 2L))

  # Method selection: hashtab via cache
  hashcache(x)
  expect_identical(as.vector(table(x)), c(2L, 1L))
  remcache(x)

  # Method selection: sorttab via cache
  sortcache(x)
  expect_identical(as.vector(table(x)), c(2L, 1L))
  remcache(x)

  # Method selection: ordertab via cache
  ordercache(x)
  expect_identical(as.vector(table(x)), c(2L, 1L))
  remcache(x)

  # Cross-tabulation coverage
  y = as.integer64(c(1L, 2L, 1L))
  t2 = table(x, y, return="data.frame")
  # Unique pairs are (1,1) and (2,2) --> 2 rows
  expect_identical(nrow(t2), 2L)

  # Potential overflow check for combinations > 2^63
  # We construct a list of many small vectors.
  args = rep(list(as.integer64(1:2)), 65)
  # Suppress warning about overflow ("NAs produced by integer64 overflow")
  #   to verify the explicit stop error cleanly.
  expect_error(suppressWarnings(do.call(table, args)), "attempt to make a table from more than")
})

test_that("table dispatch integer64 and 'higher' types and factors", {
  expect_identical(table(as.integer64(1L), "a"), table(1L, "a"))
  expect_identical(table("a", as.integer64(1L)), table("a", 1L))

  expect_identical(table(as.integer64(1L), factor("a")), table(1L, factor("a")))
  expect_identical(table(factor("a"), as.integer64(1L)), table(factor("a"), 1L))

  expect_identical(table(as.integer64(1L), 1.0), table(1L, 1.0))
  expect_identical(table(1.0, as.integer64(1L)), table(1.0, 1L))
  expect_identical(table(as.integer64(1L), 1e100), table(1L, 1e100))
  expect_identical(table(1e100, as.integer64(1L)), table(1e100, 1L))

  expect_identical(table(as.integer64(1L), 1.0+1.0i), table(1L, 1.0+1.0i))
  expect_identical(table(1.0+1.0i, as.integer64(1L)), table(1.0+1.0i, 1L))
})

test_that("implicit tests from ?match work", {
  x = as.integer64(sample(c(rep(NA, 9), 0:9), 32, TRUE))
  table = as.integer64(sample(c(rep(NA, 9), 1:9), 32, TRUE))

  expect_identical(
    match(x, table),
    match(as.integer(x), as.integer(table))
  )
  expect_identical(
    x %in% table,
    as.integer(x) %in% as.integer(table)
  )
})

test_that("implicit tests from ?unipos and ?keypos work", {
  x = as.integer64(sample(c(rep(NA, 9), 1:9), 32, TRUE))
  expect_identical(unipos(x),  seq_along(x)[!duplicated(x)])
  expect_identical(unipos(x),  match(unique(x), x))
  expect_identical(unipos(x, order="values"),  match(unique(x, order="values"), x))
  expect_identical(unique(x),  x[unipos(x)])
  expect_identical(unique(x, order="values"),  x[unipos(x, order="values")])

  x = as.integer64(sample(c(rep(NA, 9), 1:9), 32, TRUE))
  expect_identical(keypos(x),  match(x, sort(unique(x), na.last=FALSE)))
})

test_that("basic math ops works", {
  x = as.integer64(1:10)
  y = as.integer64(10:1)

  expect_identical(x + y, as.integer64(rep(11L, 10L)))
  expect_identical(y - x, as.integer64(seq(9L, -9L, by=-2L)))
  expect_identical(x * y, as.integer64(c(10L, 18L, 24L, 28L, 30L, 30L, 28L, 24L, 18L, 10L)))
  # output is double even though it fits in integer [and integer64]
  expect_identical(x[seq(2L, 10L, by=2L)] / 2L, as.double(1:5))
  expect_identical(x^2L, as.integer64((1:10)^2L))
  expect_identical(-x, as.integer64(-(1:10)))

  expect_identical(x %/% 2L, as.integer64(c(0L, 1L, 1L, 2L, 2L, 3L, 3L, 4L, 4L, 5L)))
  expect_identical(x %% 2L, as.integer64(rep_len(c(1L, 0L), 10L)))

  x32 = c(10L, 10L, -10L, -10L, 7L, 10L, -10L)
  y32 = c(3L, -3L, 3L, -3L, -10L, 0L, 0L)
  x64 = as.integer64(x32)
  y64 = as.integer64(y32)
  expect_warning(
    expect_identical(x64 %/% y64, as.integer64(x32 %/% y32)),
    "NAs produced due to division by zero"
  )
  expect_warning(
    expect_identical(x64 %% y64, as.integer64(x32 %% y32)),
    "NAs produced due to division by zero"
  )
  expect_identical(
    suppressWarnings((x64 %/% y64) * y64 + x64 %% y64 == x64),
    c(rep(TRUE, 5L), rep(NA, 2L))
  )

  # regression snuck through, caught by #149
  expect_identical(as.integer64(1L) * 1:5, as.integer64(1:5))
  expect_identical(1:5 * as.integer64(1L), as.integer64(1:5))
})

test_that("empty inputs give empty outputs for ops", {
  x = integer64(1L)
  empty = integer64(0L)

  expect_identical(x + empty, integer64())
  expect_identical(empty + x, integer64())

  expect_identical(x - empty, integer64())
  expect_identical(empty - x, integer64())

  expect_identical(+empty, integer64())
  expect_identical(-empty, integer64())

  expect_identical(x * empty, integer64())
  expect_identical(empty * x, integer64())

  expect_identical(x / empty, double())
  expect_identical(empty / x, double())

  expect_identical(x^empty, integer64())
  expect_identical(empty^x, integer64())

  expect_identical(x %/% empty, integer64())
  expect_identical(empty %/% x, integer64())

  expect_identical(x %% empty, integer64())
  expect_identical(empty %% x, integer64())

  expect_identical(x == empty, logical())
  expect_identical(empty == x, logical())

  expect_identical(x != empty, logical())
  expect_identical(empty != x, logical())

  expect_identical(x >= empty, logical())
  expect_identical(empty >= x, logical())

  expect_identical(x <= empty, logical())
  expect_identical(empty <= x, logical())

  expect_identical(x > empty, logical())
  expect_identical(empty > x, logical())

  expect_identical(x < empty, logical())
  expect_identical(empty < x, logical())

  expect_identical(x & empty, logical())
  expect_identical(empty & x, logical())

  expect_identical(x | empty, logical())
  expect_identical(empty | x, logical())

  expect_identical(xor(x, empty), logical())
  expect_identical(xor(empty, x), logical())
})

test_that("semantics about mixed types for multiplication are respected", {
  int = 5L
  i64 = as.integer64(2L)
  dbl = 3.5

  # default: "old" semantics, to be deprecated
  expect_identical(i64 * dbl, as.integer64(7L))
  expect_identical(dbl * i64, as.integer64(6L))
  expect_identical(i64 * int, as.integer64(10L))
  expect_identical(int * i64, as.integer64(10L))
  expect_identical(i64 * i64, as.integer64(4L))

  withr::with_options(list(integer64_semantics = "new"), {
    expect_identical(i64 * dbl, as.integer64(7L))
    expect_identical(dbl * i64, as.integer64(7L))
    expect_identical(i64 * int, as.integer64(10L))
    expect_identical(int * i64, as.integer64(10L))
    expect_identical(i64 * i64, as.integer64(4L))
  })
})

test_that("semantics about mixed types for division are respected", {
  int = 10L
  i64 = as.integer64(5L)
  dbl = 2.5

  # default: "old" semantics, to be deprecated
  expect_identical(i64 / dbl, 2.0)
  expect_identical(dbl / i64, 0.4)
  expect_identical(i64 / int, 0.5)
  expect_identical(int / i64, 2.0)
  expect_identical(i64 / i64, 1.0)

  withr::with_options(list(integer64_semantics = "new"), {
    expect_identical(i64 / dbl, 2.0)
    expect_identical(dbl / i64, 0.5)
    expect_identical(i64 / int, 0.5)
    expect_identical(int / i64, 2.0)
    expect_identical(i64 / i64, 1.0)
  })
})

test_that("Minus and plus", {
  d64 = c(
    -.Machine$double.base^.Machine$double.digits,
    -.Machine$integer.max,
    -1.0, 0.0, 1.0,
    .Machine$integer.max,
    .Machine$double.base^.Machine$double.digits
  )
  i64 = as.integer64(d64)
  expect_true(identical.integer64(i64 - 1.0 + 1.0, i64))
  expect_true(identical.integer64(i64 + 1.0 - 1.0, i64))
})

test_that("Minus and plus edge cases and 'rev'", {
  # UBSAN signed integer overflow expected for type 'long long int'
  # This is a false UBSAN alarm because overflow is detected and NA returned
  expect_warning(
    expect_true(identical.integer64(
      lim.integer64() + 1.0 - 1.0,
      c(lim.integer64()[1L], NA)
    )),
    "NAs produced by integer64 overflow",
    fixed = TRUE
  )
  expect_warning(
    expect_true(identical.integer64(
      rev(lim.integer64()) - 1.0 + 1.0,
      c(lim.integer64()[2L], NA)
    )),
    "NAs produced by integer64 overflow",
    fixed = TRUE
  )
})

test_that("Logical operators", {
  expect_true(identical.integer64(
    !c(NA, -1:1),
    !c(as.integer64(NA), -1:1)
  ))

  xi = rep(c(NA, -1:1), 4L)
  xi64 = as.integer64(xi)
  yi = rep(c(NA, -1:1), each=4L)
  yi64 = as.integer64(yi)

  expect_true(identical.integer64(xi64 & yi64, xi & yi))
  expect_true(identical.integer64(xi64 | yi64, xi | yi))
  expect_true(identical.integer64(xor(xi64, yi64), xor(xi, yi)))
})

test_that("Comparison operators", {
  xi = rep(c(NA, -1:1), 4L)
  xi64 = as.integer64(xi)
  yi = rep(c(NA, -1:1), each=4L)
  yi64 = as.integer64(yi)

  expect_true(identical.integer64(xi64 == yi64, xi == yi))
  expect_true(identical.integer64(xi64 != yi64, xi != yi))
  expect_true(identical.integer64(xi64 > yi64, xi > yi))
  expect_true(identical.integer64(xi64 >= yi64, xi >= yi))
  expect_true(identical.integer64(xi64 < yi64, xi < yi))
  expect_true(identical.integer64(xi64 <= yi64, xi <= yi))
})

with_parameters_test_that("{operator} with integer64 vs {class} (returning integer64):", {
  withr::local_seed(42)

  if (getRversion() <= "3.6.0" && operator == "^")
    x32 = 1:10 # there seems to be an issue with negative values with the `^` operator in ancient
  else
    x32 = c(-10:-1, 1:10)
  x64 = as.integer64(x32)
  y = sample(x32)
  eval(parse(text=paste0("y = as.", class, "(y)")))

  op = match.fun(operator)
  maybe_cast = if (operator %in% c("+", "-", "*", "^", "%%", "%/%")) as.integer64 else identity

  expected = tryCatch(maybe_cast(op(x32, y)), error=conditionMessage)
  actual = tryCatch(op(x64, y), error=conditionMessage)
  expect_identical(actual, expected)
  
  expected = tryCatch(maybe_cast(op(y, x32)), error=conditionMessage)
  actual = tryCatch(op(y, x64), error=conditionMessage)
  expect_identical(actual, expected)
}, 
  .cases = expand.grid(
    operator=c("+", "-", "*", "/", "^", "%%", "%/%", "<", "<=", "==", ">=", ">", "!=", "&", "|", "xor"),
    class=c("integer", "double", "logical"),
    stringsAsFactors=FALSE
  )
)

with_parameters_test_that("{operator} with integer64 vs. {class} (not returning integer64):", {
  skip_unless_r(">= 4.3.0")
  withr::local_seed(42)

  x32 = c(-10:-1, 1:10)
  x64 = as.integer64(x32)
  y = sample(x32)
  eval(parse(text=paste0("y = as.", class, "(as.double(y)", if (class == "difftime") ', units = "secs"', ")")))
    
  op = match.fun(as.character(operator))
  test_e = tryCatch(op(x32, y), error=conditionMessage)
  test_a = tryCatch(op(x64, y), error=conditionMessage)
  expect_identical(test_a, test_e)
  
  test_e = tryCatch(op(y, x32), error=conditionMessage)
  test_a = tryCatch(op(y, x64), error=conditionMessage)
  expect_identical(test_a, test_e)

  }, 
  .cases = expand.grid(
    operator = c("+", "-", "*", "/", "^", "%%", "%/%", "<", "<=", "==", ">=", ">", "!=", "&", "|", "xor"),
    class = c("complex", "Date", "POSIXct", "POSIXlt", "difftime")
  )
)

with_parameters_test_that(
  "integer64 vs character/factor comparisons work",
  {
    x32 = c(1L, 2L, 10L)
    x64 = as.integer64(x32)
    y = c("1", "2", "10")
    if (type == "factor") y = as.factor(y)

    op = match.fun(operator)

    if (type == "factor" && operator %in% c("<", "<=", ">", ">=")) {
      expect_warning(
        expect_identical(op(x64, y), rep(NA, 3L)),
        "not meaningful for factors", fixed = TRUE
      )
      expect_warning(
        expect_identical(op(y, x64), rep(NA, 3L)),
        "not meaningful for factors", fixed = TRUE
      )
    } else {
      expected_xy = op(x32, y)
      actual_xy = op(x64, y)
      expect_identical(actual_xy, expected_xy)

      expected_yx = op(y, x32)
      actual_yx = op(y, x64)
      expect_identical(actual_yx, expected_yx)
    }
  },
  .cases = expand.grid(
    operator = c("==", "!=", "<", "<=", ">", ">="),
    type = c("character", if (getRversion() >= "4.3.0") "factor"),
    stringsAsFactors = FALSE
  )
)

test_that("Edge cases for character/factor comparisons work", {
  expect_true(as.integer64("999999999999999") == "999999999999999")
  skip_unless_r(">= 4.3.0")
  expect_true(as.integer64("999999999999999999") == as.factor("999999999999999999"))
})

test_that("order basics work", {
  x = as.integer64(c(2L, 4L, 3L))
  expect_identical(order(x), c(1L, 3L, 2L))
  expect_identical(order(x, decreasing=TRUE), c(2L, 3L, 1L))

  x = c(x, NA_integer64_)
  expect_identical(order(x), c(1L, 3L, 2L, 4L))
  expect_identical(order(x, decreasing=TRUE), c(2L, 3L, 1L, 4L))
  expect_identical(order(x, na.last=FALSE), c(4L, 1L, 3L, 2L))
  expect_identical(order(x, na.last=FALSE, decreasing=TRUE), c(4L, 2L, 3L, 1L))
})

# adapted from old if(FALSE) region which used 10000000L to benchmark
local({
  withr::local_seed(348594L)

  x <- as.integer64(c(sample.int(10L), NA))
  sortordercache(x)

  with_parameters_test_that(
    "ramorder and sortordercache work for na.last={na.last}, decreasing={decreasing}",
    expect_identical(
      order(x, na.last=na.last, decreasing=decreasing),
      {
        xo = seq_along(x)
        bit::ramorder(x, xo, na.last=na.last, decreasing=decreasing)
        xo
      }
    ),
    .cases = expand.grid(na.last = c(FALSE, TRUE), decreasing = c(FALSE, TRUE))
  )
})

with_parameters_test_that(
  "sorting methods for integer64 work",
  {
    withr::local_options(list(bit64.warn.exported.s3.method = FALSE))
    x = as.integer64(1:10)

    na_entries = rep(NA_integer64_, n_missing)
    y = sample(c(x, if (duplicates) x[1L], na_entries))
    expect_identical(sort_function(y, decreasing=decreasing, na.last=na.last), n_missing)
    expected_value = c(
      if (na.last) integer64() else na_entries,
      if (duplicates && !decreasing) x[1L],
      if (decreasing) rev(x) else x,
      if (duplicates && decreasing) x[1L],
      if (na.last) na_entries else integer64()
    )
    expect_identical(y, expected_value)
  },
  .cases = expand.grid(
    sort_function = list(
      bit::mergesort,
      bit::quicksort,
      bit::radixsort,
      bit::ramsort,
      bit::shellsort
    ),
    na.last = c(FALSE, TRUE),
    decreasing = c(FALSE, TRUE),
    duplicates = c(FALSE, TRUE),
    n_missing = 0:2
  )
)

with_parameters_test_that(
  "order methods for integer64 work",
  {
    withr::local_options(list(bit64.warn.exported.s3.method = FALSE))
    x = as.integer64(1:10)

    na_entries = rep(NA_integer64_, n_missing)
    y = sample(c(x, if (duplicates) x[1L], na_entries))
    i = seq_along(y)
    expect_identical(order_function(y, i, decreasing=decreasing, na.last=na.last), n_missing)
    expected_value = c(
      if (na.last) integer64() else na_entries,
      if (duplicates && !decreasing) x[1L],
      if (decreasing) rev(x) else x,
      if (duplicates && decreasing) x[1L],
      if (na.last) na_entries else integer64()
    )
    expect_identical(y[i], expected_value)
  },
  .cases = expand.grid(
    order_function = list(
      bit::mergeorder,
      bit::quickorder,
      bit::radixorder,
      bit::ramorder,
      bit::shellorder
    ),
    na.last = c(FALSE, TRUE),
    decreasing = c(FALSE, TRUE),
    duplicates = c(FALSE, TRUE),
    n_missing = 0:2
  )
)

with_parameters_test_that(
  "sortorder methods for integer64 work",
  {
    withr::local_options(list(bit64.warn.exported.s3.method = FALSE))
    x = as.integer64(1:10)

    na_entries = rep(NA_integer64_, n_missing)
    y = sample(c(x, if (duplicates) x[1L], na_entries))
    i = seq_along(y)
    expect_identical(sortorder_function(y, i, decreasing=decreasing, na.last=na.last), n_missing)
    expected_value = c(
      if (na.last) integer64() else na_entries,
      if (duplicates && !decreasing) x[1L],
      if (decreasing) rev(x) else x,
      if (duplicates && decreasing) x[1L],
      if (na.last) na_entries else integer64()
    )
    # TODO(#159): Also add expectations for the update to i
    expect_identical(y, expected_value,
      info=sprintf(
        "(na.last, decreasing, duplicates, n_missing)=(%s, %s, %s, %d)",
        na.last, decreasing, duplicates, n_missing
      )
    )
  },
  .cases = expand.grid(
    sortorder_function = list(
      bit::mergesortorder,
      bit::quicksortorder,
      bit::radixsortorder,
      bit::ramsortorder,
      bit::shellsortorder
    ),
    na.last = c(FALSE, TRUE),
    decreasing = c(FALSE, TRUE),
    duplicates = c(FALSE, TRUE),
    n_missing = 0:2
  )
)

test_that("Explicit algorithm dispatch hits C-level fallbacks and edge cases", {
  # The C code contains specific implementations for shellsort, mergesort, and
  # quicksort. ramsort() dispatches based on heuristics, so we must call the
  # exported algorithm-specific functions directly to ensure coverage of
  # specific C routines (like ram_integer64_sortmerge_desc).

  # We use a vector size likely to trigger partition loops but small enough to
  # be manageable.
  set.seed(42)
  x_base = as.integer64(sample.int(1000L, 500L))
  x_dups = as.integer64(sample(c(1:5, NA), 500L, replace=TRUE)) # High duplicate count for pivot logic

  # --- Target: ram_integer64_sortmerge_desc (Image 1) ---
  # These lines are only hit when mergesort is forced into descending mode.
  # ramsort only selects mergesort for small vectors (< 2048), but we force it here.

  x = bit::clone(x_base)
  expect_identical(
    bit::mergesort(x, decreasing=TRUE),
    0L # No NAs
  )
  expect_identical(x, as.integer64(sort(as.integer(x_base), decreasing=TRUE)))

  # Verify mergesortorder and mergeorder descending paths
  x = bit::clone(x_base)
  i = seq_along(x)
  bit::mergesortorder(x, i, decreasing=TRUE)
  expect_identical(x, as.integer64(sort(as.integer(x_base), decreasing=TRUE)))

  x = bit::clone(x_base)
  i = seq_along(x)
  bit::mergeorder(x, i, decreasing=TRUE)
  expect_identical(x[i], as.integer64(sort(as.integer(x_base), decreasing=TRUE)))

  # --- Target: ram_integer64_quicksort_..._intro (Shellsort fallback) ---
  # The C code switches from Quicksort to Shellsort if recursion depth (restlevel)
  # is exhausted. We force this by setting restlevel=0.

  x = bit::clone(x_base)
  # restlevel=0 forces immediate switch to ram_integer64_shellsort_asc/desc
  bit::quicksort(x, restlevel=0L)
  expect_identical(x, as.integer64(sort(as.integer(x_base))))

  x = bit::clone(x_base)
  bit::quicksort(x, decreasing=TRUE, restlevel=0L)
  expect_identical(x, as.integer64(sort(as.integer(x_base), decreasing=TRUE)))

  # --- Target: Quicksort Partitioning Loops (Image 2 & 3) ---
  # The 'no_sentinels' loops (lines 790+) have complex break conditions.
  # We need to test:
  # 1. Already sorted data (triggers specific pivot movements)
  # 2. Reverse sorted data
  # 3. All duplicates (triggers the "j<=i" breaks in the partition loops)

  # Case: Sorted input
  x_sorted = as.integer64(1:100)
  x = bit::clone(x_sorted)
  bit::quicksort(x, optimize="memory") # Force quicksort
  expect_identical(x, x_sorted)

  # Case: Reverse sorted input (Descending sort on Ascending data and vice versa)
  x_rev = as.integer64(100:1)
  x = bit::clone(x_rev)
  bit::quicksort(x, decreasing=FALSE, optimize="memory")
  expect_identical(x, x_sorted)

  # Case: All duplicates (High stress on partitioning equal values)
  x_all_dups = as.integer64(rep(10L, 50))
  x = bit::clone(x_all_dups)
  bit::quicksort(x, optimize="memory")
  expect_identical(x, x_all_dups)

  # Case: High duplicates with NAs (verify pivot selection with NAs present)
  x = bit::clone(x_dups)
  bit::quicksort(x, optimize="memory", has.na=TRUE)
  expect_true(bit::is.sorted(x))
})

test_that("Specific sortorder/order variants for Quicksort coverage", {
  # Covering ram_integer64_quickorderpart_... and insertion fallbacks
  x_base = as.integer64(sample(100, 50))

  # quicksortorder descending
  x = bit::clone(x_base)
  i = seq_along(x)
  bit::quicksortorder(x, i, decreasing=TRUE)
  expect_identical(x, as.integer64(sort(as.integer(x_base), decreasing=TRUE)))

  # quickorder descending (modifies i, not x)
  x = bit::clone(x_base)
  i = seq_along(x)
  bit::quickorder(x, i, decreasing=TRUE)
  expect_identical(x[i], as.integer64(sort(as.integer(x_base), decreasing=TRUE)))
})

test_that("Shellsort direct invocation", {
  # While quicksort falls back to shellsort, we can also invoke it directly
  # to ensure the 'shellsort_desc' and 'shellsortorder' paths are clean.
  x_base = as.integer64(sample(100, 50))

  x = bit::clone(x_base)
  bit::shellsort(x, decreasing=TRUE)
  expect_identical(x, as.integer64(sort(as.integer(x_base), decreasing=TRUE)))

  x = bit::clone(x_base)
  i = seq_along(x)
  bit::shellorder(x, i, decreasing=FALSE)
  expect_identical(x[i], as.integer64(sort(as.integer(x_base))))
})

test_that("Corner cases for partitioning logic", {
  # Single element and empty vectors often trip up "do { ... } while" or sentinel loops

  # Case 1: Empty
  x_empty = integer64()
  x = bit::clone(x_empty)
  # bit::quicksort returns the NA count (0L), and modifies 'x' in-place
  expect_identical(bit::quicksort(x), 0L)
  expect_identical(x, x_empty)
  expect_identical(bit::quicksort(x, decreasing = TRUE), 0L)
  expect_identical(x, x_empty)

  # Add explicit tests for shellsort variants with empty vectors
  expect_identical(bit::shellsort(x_empty), 0L)
  expect_identical(bit::shellsort(x_empty, decreasing = TRUE), 0L)
  expect_identical(bit::shellsortorder(x_empty, integer(0)), 0L)
  expect_identical(bit::shellsortorder(x_empty, integer(0), decreasing = TRUE), 0L)
  expect_identical(bit::shellorder(x_empty, integer(0)), 0L)
  expect_identical(bit::shellorder(x_empty, integer(0), decreasing = TRUE), 0L)

  # Case 2: Single Element
  x_single = as.integer64(1L)
  x = bit::clone(x_single)
  expect_identical(bit::quicksort(x), 0L)
  expect_identical(x, x_single)

  # Case 3: Pair (swap required)
  x_pair = as.integer64(c(2L, 1L))
  x = bit::clone(x_pair)
  expect_identical(bit::quicksort(x), 0L)
  expect_identical(x, as.integer64(c(1L, 2L)))
})

local({
  x_base = as.integer64(sample(c(1:100, 2^30, 2^60), 200, replace = TRUE))

  # Group 1: sort (takes x)
  with_parameters_test_that(
    "radixsort works with radixbits={radixbits}",
    {
      x = bit::clone(x_base)
      bit::radixsort(x, radixbits = radixbits)
      expect_identical(x, as.integer64(sort(x_base)))
    },
    .cases = expand.grid(radixbits = c(1L, 2L, 4L, 8L, 16L))
  )

  # Group 2: order/sortorder (takes x, i)
  with_parameters_test_that(
    "{fun_name} works with radixbits={radixbits}",
    {
      x = bit::clone(x_base)
      i = seq_along(x)
      fun(x, i, radixbits = radixbits)

      # Check x (sortorder modifies x, order does not)
      if (fun_name == "radixsortorder") {
        expect_identical(x, as.integer64(sort(x_base)))
      } else {
        expect_identical(x, x_base)
      }
      # Check i
      expect_identical(x_base[i], as.integer64(sort(x_base)))
    },
    .cases = within(
      expand.grid(
        radixbits = c(1L, 2L, 4L, 8L, 16L),
        fun_name = c("radixorder", "radixsortorder"),
        stringsAsFactors = FALSE
      ),
      fun <- lapply(fun_name, getExportedValue, ns="bit")
    )
  )
})

local({
  x_small = as.integer64(sample(15))
  x_large = as.integer64(sample(100))

  # Group 1: quicksort (takes x)
  with_parameters_test_that(
    "quicksort handles restlevel={restlevel} and decreasing={decreasing}",
    {
      target_x = if (is.na(restlevel)) x_small else x_large
      x = bit::clone(target_x)

      args = list(x = x, decreasing = decreasing)
      if (!is.na(restlevel)) args$restlevel = restlevel

      do.call(bit::quicksort, args)

      expect_identical(x, as.integer64(sort(target_x, decreasing = decreasing)))
    },
    .cases = expand.grid(
      decreasing = c(TRUE, FALSE),
      restlevel = c(NA_integer_, 0L)
    )
  )

  # Group 2: quicksortorder/quickorder (takes x, i)
  with_parameters_test_that(
    "{fun_name} handles restlevel={restlevel} and decreasing={decreasing}",
    {
      target_x = if (is.na(restlevel)) x_small else x_large
      sorted_target = as.integer64(sort(target_x, decreasing = decreasing))

      x = bit::clone(target_x)
      i = seq_along(x)

      args = list(x = x, i = i, decreasing = decreasing)
      if (!is.na(restlevel)) args$restlevel = restlevel

      do.call(fun, args)

      if (fun_name == "quicksortorder") {
        # quicksortorder: x is modified in-place to be sorted
        expect_identical(x, sorted_target)
        # i is modified to represent the order of the *original* x
        expect_identical(target_x[i], sorted_target)
      } else {
        # quickorder: x is NOT modified; x[i] yields the sorted sequence
        expect_identical(x[i], sorted_target)
      }
    },
    .cases = within(
      expand.grid(
        decreasing = c(TRUE, FALSE),
        restlevel = c(NA_integer_, 0L),
        fun_name = c("quicksortorder", "quickorder"),
        stringsAsFactors = FALSE
      ),
      fun <- lapply(fun_name, getExportedValue, ns="bit")
    )
  )
})

# We test permutations of 1:3 to ensure every branch of the ternary median selector is hit
with_parameters_test_that(
  "quickorder correctly sorts permutation {toString(perm)}",
  {
    x_base = as.integer64(perm)
    x = bit::clone(x_base)
    i = seq_along(x)
    bit::quickorder(x, i)
    expect_identical(x[i], as.integer64(1:3))
  },
  .cases = data.frame(
    # List columns are tricky in data.frame, constructing manually or via list wrapping
    perm = I(list(
      c(1, 2, 3), c(1, 3, 2), c(2, 1, 3),
      c(2, 3, 1), c(3, 1, 2), c(3, 2, 1)
    ))
  )
)

local({
  # Cases covering scanners running past bounds or extreme pivots
  cases_list = list(
    sorted = as.integer64(1:50),
    rev_sorted = as.integer64(50:1),
    all_equal = as.integer64(rep(10, 50))
  )

  with_parameters_test_that(
    "{fun_name} handles {case_name} input (decreasing={decreasing})",
    {
      val = cases_list[[case_name]]
      x = bit::clone(val)

      if (fun_name == "quicksort") {
        bit::quicksort(x, decreasing = decreasing)
        expected = as.integer64(sort(val, decreasing = decreasing))
        expect_identical(x, expected)
      } else {
        i = seq_along(x)
        bit::quicksortorder(x, i, decreasing = decreasing)
        expected = as.integer64(sort(val, decreasing = decreasing))
        expect_identical(x, expected)
      }
    },
    .cases = expand.grid(
      case_name = c("sorted", "rev_sorted", "all_equal"),
      decreasing = c(TRUE, FALSE),
      fun_name = c("quicksort", "quicksortorder"),
      stringsAsFactors = FALSE
    )
  )
})

# These tests were previously kept as tests under \examples{\dontshow{...}}.
#   Converted to "proper" unit tests for clarity, after making them more
#   canonical within {testthat}, e.g. better capturing expected warnings,
#   changing stopifnot(identical(...)) to expect_identical(...).

test_that("identical.integer64", {
  i64 = NA_real_
  class(i64) = "integer64"
  expect_identical(unclass(i64 - 1.0), unclass(i64 + 1.0))
  expect_identical(i64 - 1.0, i64 + 1.0)
  expect_false(identical.integer64(i64 - 1.0, i64 + 1.0))
})

test_that("dispatch of 'c' method", {
  expect_true(identical.integer64(c(integer64(0L), NA), as.integer64(NA)))
})

test_that("Dispatch on the second argument fails and we want to be notified once that changes", {
  expect_false(identical.integer64(c(NA, integer64(0L)), as.integer64(NA)))
})

test_that("'range.integer64', multiplication, integer division, sqrt, power, and log", {
  i64 = integer64(63L)
  i64[1L] = 1.0
  for (i in 2:63)
    i64[i] = 2.0 * i64[i-1L]
  expect_true(identical.integer64(i64 * rev(i64), rep(i64[63L], 63L)))
  for (i in 63:2)
    i64[i-1L] = i64[i] %/% 2.0
  expect_true(identical.integer64(i64 * rev(i64), rep(i64[63L], 63L)))
  for (i in 63:2)
    i64[i-1L] = i64[i] / 2.0
  expect_true(identical.integer64(i64 * rev(i64), rep(i64[63L], 63L)))
  expect_true(identical.integer64(
    c(
      -i64[63L] - (i64[63L] - 1.0),
      i64[63L] + (i64[63L] - 1.0)
    ),
    lim.integer64()
  ))

  expect_true(identical.integer64(i64[-1L] %/% 2.0 * as.integer64(2L), i64[-1L]))
  expect_true(identical.integer64(i64[-1L] %/% 2L * as.integer64(2L), i64[-1L]))
  expect_true(identical.integer64(i64[-1L] / 2.0 * as.integer64(2L), i64[-1L]))
  expect_true(identical.integer64(i64[-1L] / 2L * as.integer64(2L), i64[-1L]))

  expect_true(identical.integer64(i64[-63L] * 2.0 %/% 2.0, i64[-63L]))
  expect_true(identical.integer64(i64[-63L] * 2L %/% 2L, i64[-63L]))
  expect_true(identical.integer64(as.integer64(i64[-63L] * 2.0 / 2.0), i64[-63L]))
  expect_true(identical.integer64(as.integer64(i64[-63L] * 2L / 2L), i64[-63L]))

  expect_true(identical.integer64(
    as.integer64(sqrt(
      i64[-1L][c(FALSE, TRUE)]) * sqrt(i64[-1L][c(FALSE, TRUE)]
    )),
    i64[-1L][c(FALSE, TRUE)]
  ))

  expect_true(identical.integer64(as.integer64(2L) ^ (0:62), i64))
  expect_true(identical.integer64(as.integer64(0:62), as.integer64(round(log2(i64)))))
  expect_true(identical.integer64(
    as.integer64(round(log(as.integer64(2L)^(0:62), 2.0))),
    as.integer64(0:62)
  ))
  expect_true(identical.integer64(
    as.integer64(round(log(as.integer64(3L)^(0:39), 3.0))),
    as.integer64(0:39)
  ))
  expect_true(identical.integer64(
    as.integer64(round(log(as.integer64(10L)^(0:18), 10.0))),
    as.integer64(0:18)
  ))
  expect_true(identical.integer64(
    as.integer64(round(log10(as.integer64(10L)^(0:18)))),
    as.integer64(0:18)
  ))

  expect_true(identical.integer64(
    (as.integer64(2L)^(1:62))^(1.0/1:62),
    as.integer64(rep(2.0, 62L))
  ))
  expect_true(identical.integer64(
    (as.integer64(3L)^(1:39))^(1.0/1:39),
    as.integer64(rep(3.0, 39L))
  ))
  expect_true(identical.integer64(
    (as.integer64(10L)^(1:18))^(1.0/1:18),
    as.integer64(rep(10.0, 18L))
  ))
})

test_that("c and rep", {
  expect_true(identical.integer64(
    as.integer64(rep(1:3, 1:3)),
    rep(as.integer64(1:3), 1:3)
  ))
  expect_true(identical.integer64(
    as.integer64(rep(1:3, 3L)),
    rep(as.integer64(1:3), 3L)
  ))

  x = rep(NA_real_, 3L)
  class(x) = "integer64"
  x = x + -1:1
  expect_true(identical.integer64(rep(x, 3L), c(x, x, x)))
  expect_true(identical.integer64(
    c.integer64(list(x, x, x), recursive=TRUE),
    c(x, x, x)
  ))
})

test_that("seq", {
  expect_true(identical.integer64(
    seq(as.integer64(1L), 10.0, 2.0),
    as.integer64(seq(1.0, 10.0, 2.0))
  ))
  expect_true(identical.integer64(
    seq(as.integer64(1L), by=2.0, length.out=5.0),
    as.integer64(seq(1.0, by=2.0, length.out=5.0))
  ))
  expect_true(identical.integer64(
    seq(as.integer64(1L), by=2.0, length.out=6.0),
    as.integer64(seq(1.0, by=2.0, length.out=6.0))
  ))
  expect_true(identical.integer64(
    seq.integer64(along.with=3:5),
    as.integer64(seq(along.with=3:5))
  ))
  expect_true(identical.integer64(
    seq(as.integer64(1L), to=-9.0),
    as.integer64(seq(1.0, to=-9.0))
  ))
})

test_that("cbind and rbind", {
  x = rep(as.integer64(1:3), 2L)
  dim(x) = c(3L, 2L)
  expect_true(identical.integer64(cbind(as.integer64(1:3), 1:3), x))
  expect_true(identical.integer64(rbind(as.integer64(1:3), 1:3), t(x)))
})

test_that("Coercion from integer64 works as intended", {
  expect_identical(
    as.double(as.integer64(c(NA, seq(0.0, 9.0, 0.25)))),
    as.double(as.integer(c(NA, seq(0.0, 9.0, 0.25))))
  )
  expect_identical(
    as.character(as.integer64(c(NA, seq(0.0, 9.0, 0.25)))),
    as.character(as.integer(c(NA, seq(0.0, 9.0, 0.25))))
  )
  expect_identical(
    as.integer(as.integer64(c(NA, seq(0.0, 9.0, 0.25)))),
    as.integer(c(NA, seq(0.0, 9.0, 0.25)))
  )
  expect_identical(
    as.logical(as.integer64(c(NA, seq(0.0, 9.0, 0.25)))),
    as.logical(as.integer(c(NA, seq(0.0, 9.0, 0.25))))
  )
  expect_identical(
    as.integer(as.integer64(c(NA, FALSE, TRUE))),
    as.integer(c(NA, FALSE, TRUE))
  )
})

test_that("Coercion to integer64 with as.integer64 works as intended", {
  expect_identical(
    as.integer64(as.integer(as.integer64(-9:9))),
    as.integer64(-9:9)
  )
  expect_identical(
    as.integer64(as.double(as.integer64(-9:9))),
    as.integer64(-9:9)
  )
  expect_identical(
    as.integer64(as.complex(-9:9)),
    as.integer64(-9:9)
  )
  expect_identical(
    as.integer64(as.raw(0:9)),
    as.integer64(0:9)
  )
  expect_identical(
    as.integer64(as.character(as.integer64(-9:9))),
    as.integer64(-9:9)
  )
  expect_identical(
    as.integer64(as.character(lim.integer64())),
    lim.integer64()
  )
})

test_that("Coercion to integer64 with as() works as intended", {
  expect_identical(
    as(as.raw(1L), "integer64"),
    as.integer64(1L)
  )
  expect_identical(
    as(TRUE, "integer64"),
    as.integer64(1L)
  )
  expect_identical(
    as(111L, "integer64"),
    as.integer64(111L)
  )
  expect_identical(
    as(111.0, "integer64"),
    as.integer64(111L)
  )
  expect_identical(
    as(111.0 + 0.0i, "integer64"),
    as.integer64(111L)
  )
  expect_identical(
    as("111", "integer64"),
    as.integer64(111L)
  )
  expect_identical(
    as(as.factor(111L), "integer64"),
    as.integer64(1L)
  )
  expect_identical(
    as(as.ordered(111L), "integer64"),
    as.integer64(1L)
  )
})

test_that("Coercion from integer64 with as() works as intended", {
  expect_identical(
    as(as.integer64(1L), "logical"),
    TRUE
  )
  expect_identical(
    as(as.integer64(111L), "integer"),
    111L
  )
  expect_identical(
    as(as.integer64(111L), "numeric"),
    111.0
  )
  expect_identical(
    as(as.integer64(111L), "character"),
    "111"
  )
  expect_identical(
    as(as.integer64(111L), "factor"),
    as.factor("111")
  )
  expect_identical(
    as(as.integer64(111L), "ordered"),
    as.ordered("111")
  )
})

test_that("Coercion works for cases requiring recent R", {
  skip_unless_r(">= 4.0.0")

  expect_identical(
    as.complex(as.integer64(c(NA, seq(0.0, 9.0, 0.25)))),
    as.complex(as.integer(c(NA, seq(0.0, 9.0, 0.25))))
  )

  coercion_warning = "out-of-range values treated as 0 in coercion to raw"
  expect_warning({
    raw_from_int64 = as.raw(as.integer64(c(NA, seq(0.0, 1.25, 0.25))))
  }, coercion_warning, fixed=TRUE)
  expect_warning({
    raw_from_int32 = as.raw(as.integer(c(NA, seq(0.0, 1.25, 0.25))))
  }, coercion_warning, fixed=TRUE)

  expect_identical(raw_from_int64, raw_from_int32)

  expect_identical(
    as(as.integer64(1L), "raw"),
    as.raw(1L)
  )

  expect_identical(
    as(as.integer64(111L), "complex"),
    111.0 + 0.0i
  )
})

test_that("Vector functions", {
  xi = c(NA, -1:1)
  xi64 = as.integer64(xi)

  expect_true(identical.integer64(is.na(xi64), is.na(xi)))
  expect_true(identical.integer64(format(xi64), format(xi)))
  expect_true(identical.integer64(abs(xi64), as.integer64(abs(xi))))
  expect_true(identical.integer64(sign(xi64), as.integer64(sign(xi))))
  expect_true(identical.integer64(ceiling(xi64), as.integer64(ceiling(xi))))
  expect_true(identical.integer64(floor(xi64), as.integer64(floor(xi))))
  expect_true(identical.integer64(trunc(xi64), as.integer64(trunc(xi))))
  expect_true(identical.integer64(signif(xi64), xi64))
})

test_that("Summary functions", {
  expect_identical(all(as.integer64(1L)), all(1L))
  expect_identical(all(as.integer64(0L)), all(0L))
  expect_identical(all(NA_integer64_), all(NA_integer_))
  expect_identical(all(NA_integer64_, na.rm=TRUE), all(NA_integer_, na.rm=TRUE))
  expect_identical(all(as.integer64(1L), NA), all(1L, NA))
  expect_identical(all(as.integer64(0L), NA), all(0L, NA))
  expect_identical(all(as.integer64(1L), NA, na.rm=TRUE), all(1L, NA, na.rm=TRUE))
  expect_identical(all(as.integer64(0L), NA, na.rm=TRUE), all(0L, NA, na.rm=TRUE))
  expect_identical(all(as.integer64(c(1L, NA))), all(c(1L, NA_integer_)))
  expect_identical(all(as.integer64(c(0L, NA))), all(c(0L, NA_integer_)))
  expect_identical(all(as.integer64(c(1L, NA)), na.rm=TRUE), all(c(1L, NA_integer_), na.rm=TRUE))
  expect_identical(all(as.integer64(c(0L, NA)), na.rm=TRUE), all(c(0L, NA_integer_), na.rm=TRUE))

  expect_identical(any(as.integer64(1L)), any(1L))
  expect_identical(any(as.integer64(0L)), any(0L))
  expect_identical(any(NA_integer64_), any(NA_integer_))
  expect_identical(any(NA_integer64_, na.rm=TRUE), any(NA_integer_, na.rm=TRUE))
  expect_identical(any(as.integer64(1L), NA), any(1L, NA))
  expect_identical(any(as.integer64(0L), NA), any(0L, NA))
  expect_identical(any(as.integer64(1L), NA, na.rm=TRUE), any(1L, NA, na.rm=TRUE))
  expect_identical(any(as.integer64(0L), NA, na.rm=TRUE), any(0L, NA, na.rm=TRUE))
  expect_identical(any(as.integer64(c(1L, NA))), any(c(1L, NA_integer_)))
  expect_identical(any(as.integer64(c(0L, NA))), any(c(0L, NA_integer_)))
  expect_identical(any(as.integer64(c(1L, NA)), na.rm=TRUE), any(c(1L, NA_integer_), na.rm=TRUE))
  expect_identical(any(as.integer64(c(0L, NA)), na.rm=TRUE), any(c(0L, NA_integer_), na.rm=TRUE))

  xd = c(2.0, 3.0, NA)
  xi64 = as.integer64(xd)

  expect_true(identical.integer64(
    as.integer64(sum(xd)),
    sum(xi64)
  ))
  expect_true(identical.integer64(
    as.integer64(sum(xd, na.rm=TRUE)),
    sum(xi64, na.rm=TRUE)
  ))
  expect_true(identical.integer64(
    as.integer64(sum(xd)),
    sum(xi64)
  ))
  expect_true(identical.integer64(
    as.integer64(sum(xd, na.rm=TRUE)),
    sum(xi64, na.rm=TRUE)
  ))
  expect_true(identical.integer64(
    as.integer64(sum(2.0, 3.0, NA)),
    sum(as.integer64(2L), 3.0, NA)
  ))
  expect_true(identical.integer64(
    as.integer64(sum(2.0, 3.0, NA, na.rm=TRUE)),
    sum(as.integer64(2L), 3.0, NA, na.rm=TRUE)
  ))
  expect_true(identical.integer64(
    as.integer64(sum(2.0, 3.0, NA)),
    sum(as.integer64(2L), 3.0, NA)
  ))
  expect_true(identical.integer64(
    as.integer64(sum(2.0, 3.0, NA, na.rm=TRUE)),
    sum(as.integer64(2L), 3.0, NA, na.rm=TRUE)
  ))

  expect_true(identical.integer64(
    as.integer64(prod(xd)),
    prod(xi64)
  ))
  expect_true(identical.integer64(
    as.integer64(prod(xd, na.rm=TRUE)),
    prod(xi64, na.rm=TRUE)
  ))
  expect_true(identical.integer64(
    as.integer64(prod(xd)),
    prod(xi64)
  ))
  expect_true(identical.integer64(
    as.integer64(prod(xd, na.rm=TRUE)),
    prod(xi64, na.rm=TRUE)
  ))
  expect_true(identical.integer64(
    as.integer64(prod(2.0, 3.0, NA)),
    prod(as.integer64(2L), 3.0, NA)
  ))
  expect_true(identical.integer64(
    as.integer64(prod(2.0, 3.0, NA, na.rm=TRUE)),
    prod(as.integer64(2L), 3.0, NA, na.rm=TRUE)
  ))
  expect_true(identical.integer64(
    as.integer64(prod(2.0, 3.0, NA)),
    prod(as.integer64(2L), 3.0, NA)
  ))
  expect_true(identical.integer64(
    as.integer64(prod(2.0, 3.0, NA, na.rm=TRUE)),
    prod(as.integer64(2L), 3.0, NA, na.rm=TRUE)
  ))

  expect_true(identical.integer64(
    as.integer64(min(xd)),
    min(xi64)
  ))
  expect_true(identical.integer64(
    as.integer64(min(xd, na.rm=TRUE)),
    min(xi64, na.rm=TRUE)
  ))
  expect_true(identical.integer64(
    as.integer64(min(xd)),
    min(xi64)
  ))
  expect_true(identical.integer64(
    as.integer64(min(xd, na.rm=TRUE)),
    min(xi64, na.rm=TRUE)
  ))
  expect_true(identical.integer64(
    as.integer64(min(2.0, 3.0, NA)),
    min(as.integer64(2L), 3.0, NA)
  ))
  expect_true(identical.integer64(
    as.integer64(min(2.0, 3.0, NA, na.rm=TRUE)),
    min(as.integer64(2L), 3.0, NA, na.rm=TRUE)
  ))
  expect_true(identical.integer64(
    as.integer64(min(2.0, 3.0, NA)),
    min(as.integer64(2L), 3.0, NA)
  ))
  expect_true(identical.integer64(
    as.integer64(min(2.0, 3.0, NA, na.rm=TRUE)),
    min(as.integer64(2L), 3.0, NA, na.rm=TRUE)
  ))

  expect_true(identical.integer64(
    as.integer64(max(xd)),
    max(xi64)
  ))
  expect_true(identical.integer64(
    as.integer64(max(xd, na.rm=TRUE)),
    max(xi64, na.rm=TRUE)
  ))
  expect_true(identical.integer64(
    as.integer64(max(xd)),
    max(xi64)
  ))
  expect_true(identical.integer64(
    as.integer64(max(xd, na.rm=TRUE)),
    max(xi64, na.rm=TRUE)
  ))
  expect_true(identical.integer64(
    as.integer64(max(2.0, 3.0, NA)),
    max(as.integer64(2L), 3.0, NA)
  ))
  expect_true(identical.integer64(
    as.integer64(max(2.0, 3.0, NA, na.rm=TRUE)),
    max(as.integer64(2L), 3.0, NA, na.rm=TRUE)
  ))
  expect_true(identical.integer64(
    as.integer64(max(2.0, 3.0, NA)),
    max(as.integer64(2L), 3.0, NA)
  ))
  expect_true(identical.integer64(
    as.integer64(max(2.0, 3.0, NA, na.rm=TRUE)),
    max(as.integer64(2L), 3.0, NA, na.rm=TRUE)
  ))

  expect_true(identical.integer64(
    as.integer64(range(xd)),
    range(xi64)
  ))
  expect_true(identical.integer64(
    as.integer64(range(xd, na.rm=TRUE)),
    range(xi64, na.rm=TRUE)
  ))
  expect_true(identical.integer64(
    as.integer64(range(xd)),
    range(xi64)
  ))
  expect_true(identical.integer64(
    as.integer64(range(xd, na.rm=TRUE)),
    range(xi64, na.rm=TRUE)
  ))
  expect_true(identical.integer64(
    as.integer64(range(2.0, 3.0, NA)),
    range(as.integer64(2L), 3.0, NA)
  ))
  expect_true(identical.integer64(
    as.integer64(range(2.0, 3.0, NA, na.rm=TRUE)),
    range(as.integer64(2L), 3.0, NA, na.rm=TRUE)
  ))
})

test_that("Cumulative functions", {
  xd = c(2.0, 3.0, NA, 1.0, 4.0)
  xi64 = as.integer64(xd)

  expect_true(identical.integer64(cumsum(xi64), as.integer64(cumsum(xd))))
  expect_true(identical.integer64(cumprod(xi64), as.integer64(cumprod(xd))))
  expect_true(identical.integer64(cummin(xi64), as.integer64(cummin(xd))))
  expect_true(identical.integer64(cummax(xi64), as.integer64(cummax(xd))))
})

test_that("diff", {
  d64 = diffinv(rep(.Machine$integer.max, 100L), lag=2.0, differences=2L)
  i64 = as.integer64(d64)
  expect_identical(
    diff(d64, lag=2L, differences=2L),
    as.double(diff(i64, lag=2L, differences=2L))
  )
})

test_that("substitute() invocation of generic doesn't warn during transition", {
  x = as.integer64(1:10)
  expect_no_warning(
    expect_identical(
      eval(substitute(f(x, y), list(f=`:`, x=x[1L], y=x[10L]))),
      x
    )
  )
})

# /*
# R-Code for caching
# S3 atomic 64bit integers for R
# (c) 2011 Jens Oehlschägel
# Licence: GPL2
# Provided 'as is', use at your own risk
# Created: 2011-12-11
# Last changed:  2026-01-09
# */

#' Atomic Caching
#'
#' Functions for caching results attached to atomic objects
#'
#' @param x an integer64 vector (or a cache object in case of `print.cache`)
#'
#' @details
#' A `cache` is an [`environment`] attached to an atomic object with the
#'   [`attribute`][attr] name 'cache'. It contains at least a reference to the
#'   atomic object that carries the cache. This is used when accessing
#'   the cache to detect whether the object carrying the cache has been
#'   modified meanwhile.
#'
#' @return See details
#' @seealso
#'   [bit::still.identical()] for testing whether to symbols point to the same RAM.
#'
#' Functions that get and set small cache-content automatically when a cache is present:
#'   [bit::na.count()], [bit::nvalid()], [bit::is.sorted()], [bit::nunique()] and
#'   [bit::nties()]
#'
#' Setting big caches with a relevant memory footprint requires a conscious decision
#'   of the user: [`hashcache`], [`sortcache`], [`ordercache`], [`sortordercache`]
#'
#' Functions that use big caches: [match.integer64()], \code{\link[=match.integer64]{\%in\%.integer64}},
#'   [duplicated.integer64()], [unique.integer64()], [unipos()], [table()],
#'   [keypos()], [tiepos()], [rank.integer64()], [prank()], [qtile()],
#'   [quantile.integer64()], [median.integer64()], and [summary.integer64()]
#'
#' @examples
#'   x = as.integer64(sample(c(rep(NA, 9), 1:9), 32, TRUE))
#'   y = x
#'   bit::still.identical(x, y)
#'   y[1] = NA
#'   bit::still.identical(x, y)
#'   mycache = newcache(x)
#'   ls(mycache)
#'   mycache
#'   rm(mycache)
#'   jamcache(x)
#'   cache(x)
#'   x[1] = NA
#'   cache(x)
#'   getcache(x, "abc")
#'   setcache(x, "abc", 1)
#'   getcache(x, "abc")
#'   remcache(x)
#'   cache(x)
#' @keywords environment
#' @name cache
NULL

#' @describeIn cache creates a new cache referencing  `x`
#' @export
newcache = function(x) {
  env = new.env()
  vmode = typeof(x)
  if (vmode=="double" && is.integer64(x))
    vmode = "integer64"
  setattr(env, "class", c(paste("cache", vmode, sep="_"), "cache", "environment"))
  assign("x", x, envir=env)
  env
}

#' @describeIn cache forces `x` to have a cache
#' @export
jamcache = function(x) {
  cache = attr(x, "cache")
  if (is.null(cache)) {
    cache = newcache(x)
    setattr(x, "cache", cache)
  } else if (!still.identical(x, get("x", envir=cache, inherits=FALSE))) {
    cache = newcache(x)
    setattr(x, "cache", cache)
    warning("replaced outdated cache with empty cache")
  }
  cache
}

#' @describeIn cache returns the cache attached to `x` if it is not
#'   found to be outdated
#' @export
cache = function(x) {
  cache = attr(x, "cache")
  if (is.null(cache) || still.identical(x, get("x", envir=cache, inherits=FALSE))) {
    cache
  } else {
    remcache(x)
    warning("removed outdated cache")
    NULL
  }
}

#' @describeIn cache assigns a value into the cache of `x`
#' @param which A character naming the object to be retrieved from the cache or to be stored in the cache
#' @param value An object to be stored in the cache
#' @export
setcache = function(x, which, value) {
  env = jamcache(x)
  assign(which, value, envir=env)
  env
}

#' @describeIn cache gets cache value 'which' from `x`
#' @export
getcache = function(x, which) {
  cache = attr(x, "cache")
  if (is.null(cache))
    return(NULL)
  if (still.identical(x, get("x", envir=cache, inherits=FALSE))) {
    if (exists(which, envir=cache, inherits=FALSE))
      get(which, envir=cache, inherits=FALSE)
    else
        NULL
  } else {
    remcache(x)
    warning("removed outdated cache")
    NULL
  }
}

#' @describeIn cache removes the cache from `x`
#' @export
remcache = function(x) {
  setattr(x, "cache", NULL)
  invisible()
}

#' @rdname cache
#' @param all.names,pattern passed to [ls()] when listing the cache content
#' @param ... ignored
#' @export
print.cache = function(x, all.names=FALSE, pattern, ...) {
  l = ls(x, all.names, pattern=pattern)
  cat(class(x)[1L], ": ", paste(l, collapse=" - "), "\n", sep="")
  invisible(l)
}

#' Big caching of hashing, sorting, ordering
#'
#' Functions to create cache that accelerates many operations
#'
#' @param x an atomic vector (note that currently only integer64 is supported)
#' @param nunique giving _correct_ number of unique elements can help reducing
#'   the size of the hashmap
#' @param ... passed to [hashmap()]
#'
#' @details
#' The result of relative expensive operations [hashmap()], [bit::ramsort()],
#'   [bit::ramsortorder()], and [bit::ramorder()] can be stored in a cache in
#'   order to avoid multiple excutions. Unless in very specific situations, the
#'   recommended method is `hashsortorder` only.
#'
#' @note
#'   Note that we consider storing the big results from sorting and/or ordering
#'   as a relevant side-effect, and therefore storing them in the cache should
#'   require a conscious decision of the user.
#'
#' @return `x` with a [cache()] that contains the result of the expensive operations,
#'   possible together with small derived information (such as [nunique.integer64()])
#'   and previously cached results.
#'
#' @seealso
#'   [cache()] for caching functions and [nunique.integer64()] for methods benefiting
#'   from small caches
#'
#' @examples
#'   x = as.integer64(sample(c(rep(NA, 9), 1:9), 32, TRUE))
#'   sortordercache(x)
#'
#' @keywords environment
#' @export
hashcache = function(x, nunique=NULL, ...) {
  env = jamcache(x)
  if (is.null(nunique))
    nunique = env$nunique
  env = hashmap(x, nunique=nunique, cache=env, ...)
  if (is.null(nunique) && env$nunique<sqrt(length(x)))
    env = hashmap(x, nunique=env$nunique, cache=env, ...)
  na.count(x) # since x has cache, na.count() will update the cache, unless its already there
  # different from sortcache, ordercache and sortordercache we do not set nties: hastab is too expensive
  invisible(env)
}

#' @rdname hashcache
#' @param has.na boolean scalar defining whether the input vector might contain
#'    `NA`s. If we know we don't have `NA`s, this may speed-up. _Note_ that you
#'    risk a crash if there are unexpected `NA`s with `has.na=FALSE`.
#' @param na.last boolean scalar defining whether NA should be last.
#' @export
sortcache = function(x, has.na=NULL, na.last=FALSE) {
  if (is.null(has.na)) {
    na.count = getcache(x, "na.count")
    if (is.null(na.count))
      has.na = TRUE
    else
      has.na = na.count > 0L
  }
  s = clone(x)
  na.count = ramsort(s, has.na=has.na, na.last=na.last, decreasing=FALSE, stable=FALSE, optimize="time")
  nut = .Call(C_r_ram_integer64_sortnut, x=s)
  setcache(x, "sort", s)
  setcache(x, "na.count", na.count)
  setcache(x, "nunique", nut[[1L]])
  setcache(x, "nties", nut[[2L]])
  invisible(x)
}

#' @rdname hashcache
#' @param stable boolean scalar defining whether stable sorting is needed. Allowing
#'   non-stable may speed-up.
#' @export
sortordercache = function(x, has.na=NULL, stable=NULL, na.last=FALSE) {
  if (is.null(has.na)) {
    na.count = getcache(x, "na.count")
    if (is.null(na.count))
      has.na = TRUE
    else
      has.na = na.count > 0L
  }
  if (is.null(stable)) {
    nunique = getcache(x, "nunique")
    if (is.null(nunique))
      stable = TRUE
    else
      stable = nunique < length(x)
  }
  s = clone(x)
  o = seq_along(x)
  na.count = ramsortorder(s, o, has.na=has.na, na.last=na.last, decreasing=FALSE, stable=stable, optimize="time")
  nut = .Call(C_r_ram_integer64_sortnut, x=s)
  setcache(x, "sort", s)
  setcache(x, "order", o)
  setcache(x, "na.count", na.count)
  setcache(x, "nunique", nut[[1L]])
  setcache(x, "nties", nut[[2L]])
  invisible(x)
}

#' @rdname hashcache
#' @param optimize by default ramsort optimizes for 'time' which requires more RAM,
#'   set to 'memory' to minimize RAM requirements and sacrifice speed.
#' @export
ordercache = function(x, has.na=NULL, stable=NULL, optimize="time", na.last=FALSE) {
  if (is.null(has.na)) {
    na.count = getcache(x, "na.count")
    if (is.null(na.count))
      has.na = TRUE
    else
      has.na = na.count > 0L
  }
  if (is.null(stable)) {
    nunique = getcache(x, "nunique")
    if (is.null(nunique))
      stable = TRUE
    else
      stable = nunique < length(x)
  }
  o = seq_along(x)
  na.count = ramorder(x, o, has.na=has.na, na.last=na.last, decreasing=FALSE, stable=stable, optimize=optimize)
  nut = .Call(C_r_ram_integer64_ordernut, table=x, order=o)
  setcache(x, "order", o)
  setcache(x, "na.count", na.count)
  setcache(x, "nunique", nut[[1L]])
  setcache(x, "nties", nut[[2L]])
  invisible(x)
}

#' Small cache access methods
#'
#' These methods are packaged here for methods in packages `bit64` and `ff`.
#'
#' @param x some object
#' @param ... ignored
#'
#' @details
#' All these functions benefit from a [sortcache()], [ordercache()] or
#'   [sortordercache()]. `na.count()`, `nvalid()` and `nunique()` also
#'   benefit from a [hashcache()].
#'
#' @note
#' If a [cache()] exists but the desired value is not cached, then these
#'   functions will store their result in the cache. We do not consider this
#'   a relevant side-effect, since these small cache results do not have a
#'   relevant memory footprint.
#'
#' @return
#' `is.sorted` returns a logical scalar, the other methods return an integer scalar.
#'
#' @seealso
#' [cache()] for caching functions and [sortordercache()] for functions creating big caches
#'
#' @examples
#'  x = as.integer64(sample(c(rep(NA, 9), 1:9), 32, TRUE))
#'  length(x)
#'  bit::na.count(x)
#'  bit::nvalid(x)
#'  bit::nunique(x)
#'  bit::nties(x)
#'  table(x)
#'  x
#'
#' @keywords environment methods
#' @name is.sorted.integer64
NULL

#' @describeIn is.sorted.integer64 returns the number of `NA`s
#' @export
na.count.integer64 = function(x, ...) {
  env = cache(x)
  if (is.null(env))
    return(.Call(C_r_ram_integer64_nacount, x=x))
  if (exists("na.count", envir=env, inherits=FALSE))
    return(get("na.count", envir=env, inherits=FALSE))
  ret = .Call(C_r_ram_integer64_nacount, x=x)
  assign("na.count", ret, envir=env)
  ret
}

#' @describeIn is.sorted.integer64 returns the number of valid data points,
#'   usually [length()] minus `na.count`.
#' @export
nvalid.integer64 = function(x, ...) {
  length(x) - na.count(x)
}

#' @describeIn is.sorted.integer64 checks for sortedness of `x` (NAs sorted first)
#' @export
is.sorted.integer64 = function(x, ...) {
  env = cache(x)
  if (is.null(env))
    return(.Call(C_r_ram_integer64_issorted_asc, x=x))
  if (exists("is.sorted", envir=env, inherits=FALSE))
    return(get("is.sorted", envir=env, inherits=FALSE))
  ret = .Call(C_r_ram_integer64_issorted_asc, x=x)
  assign("is.sorted", ret, envir=env)
  ret
}

#' @describeIn is.sorted.integer64 returns the number of unique values
#' @export
nunique.integer64 = function(x, ...) {
  env = cache(x)
  if (is.null(env))
    has.cache = FALSE
  else if (exists("nunique", envir=env, inherits=FALSE))
    return(get("nunique", envir=env, inherits=FALSE))
  else # nolint: unreachable_code_linter. TODO(r-lib/lintr#2710): Re-enable.
    has.cache = TRUE
  if (is.sorted(x)) {
    ret = .Call(C_r_ram_integer64_sortnut, x=x)
    if (has.cache) {
      assign("nunique", ret[1L], envir=env)
      assign("nties", ret[2L], envir=env)
    }
    ret[1L]
  } else {
    h = hashmap(x)
    if (has.cache)
      assign("nunique", h$nunique, envir=env)
    h$nunique
  }
}

#' @describeIn is.sorted.integer64 returns the number of tied values.
#' @export
nties.integer64 = function(x, ...) {
  cv = getcache(x, "nties")
  if (is.null(cv)) {
    if (is.sorted(x)) {
      cv = .Call(C_r_ram_integer64_sortnut, x=x)[2L]
    } else {
      s = clone(x)
      # nolint next: object_usage_linter. Keep the output of in-place ramsort for debugging.
      na.count = ramsort(s, has.na=TRUE, na.last=FALSE, decreasing=FALSE, stable=FALSE, optimize="time")
      cv = .Call(C_r_ram_integer64_sortnut, x=s)[[2L]]
    }
  }
  cv
}

test_that("is.sorted works", {
  expect_true(bit::is.sorted(integer64()))
  expect_true(bit::is.sorted(as.integer64(1:10)))
  expect_false(bit::is.sorted(as.integer64(10:1)))
})

test_that("na-last works", {
  x_32 = c(1L, NA, 2L)
  x_64 = as.integer64(x_32)

  sortcache(x_64, na.last=FALSE)
  expect_identical(cache(x_64)$sort, as.integer64(sort(x_32, na.last=FALSE)))
  remcache(x_64)
  sortcache(x_64, na.last=TRUE)
  expect_identical(cache(x_64)$sort, as.integer64(sort(x_32, na.last=TRUE)))
  remcache(x_64)

  sortordercache(x_64, na.last=FALSE)
  expect_identical(cache(x_64)$sort, as.integer64(sort(x_32, na.last=FALSE)))
  expect_identical(cache(x_64)$order, order(x_32, na.last=FALSE))
  remcache(x_64)
  sortordercache(x_64, na.last=TRUE)
  expect_identical(cache(x_64)$sort, as.integer64(sort(x_32, na.last=TRUE)))
  expect_identical(cache(x_64)$order, order(x_32, na.last=TRUE))
  remcache(x_64)

  ordercache(x_64, na.last=FALSE)
  expect_identical(cache(x_64)$order, order(x_32, na.last=FALSE))
  remcache(x_64)
  ordercache(x_64, na.last=TRUE)
  expect_identical(cache(x_64)$order, order(x_32, na.last=TRUE))
  remcache(x_64)
})

# /*
# R-Code for searching and merging
# S3 atomic 64bit integers for R
# (c) 2011-2024 Jens Oehlschägel
# (c) 2024-2026 Michael Chirico
# Licence: GPL2
# Provided 'as is', use at your own risk
# Created: 2011-12-11
# Last changed:  2011-12-11
# */

#' Searching and other uses of sorting for 64bit integers
#'
#' This is roughly an implementation of hash functionality but based on sorting
#'   instead on a hashmap. Since sorting is more informative than hashing we
#'   can do some more interesting things.
#'
#' @param sorted a sorted [`integer64`] vector
#' @param ... further arguments, passed from generics, ignored in methods
#' @param method see Details
#'
#' @details
#'
# nolint start: line_length_linter.
#' | **sortfun** | **orderfun** | **sortorderfun** | **see also**       | **description** |
#' |------------:|-------------:|-----------------:|-------------------:|:----------------|
#' |   `sortnut` |   `ordernut` |                  |                    | return number of tied and of unique values |
#' |   `sortfin` |   `orderfin` |                  | \code{\link[=match.integer64]{\%in\%.integer64}} | return logical whether `x` is in `table` |
#' |             |   `orderpos` |   `sortorderpos` | [`match()`][match.integer64] | return positions of `x` in `table` |
#' |             |   `orderdup` |   `sortorderdup` | [`duplicated()`][duplicated.integer64] | return logical whether values are duplicated |
#' |   `sortuni` |   `orderuni` |   `sortorderuni` | [`unique()`][unique.integer64] | return unique values (=dimensiontable) |
#' |             |   `orderupo` |   `sortorderupo` | [`unique()`][unique.integer64] | return positions of unique values |
#' |             |   `ordertie` |   `sortordertie` |                    | return positions of tied values |
#' |             |   `orderkey` |   `sortorderkey` |                    | positions of values in vector of unique values (match in dimensiontable) |
#' |   `sorttab` |   `ordertab` |   `sortordertab` | [`table()`][table] | tabulate frequency of values  |
#' |             |   `orderrnk` |   `sortorderrnk` |                    | rank averaging ties |
#' |   `sortqtl` |   `orderqtl` |                  |                    | return quantiles given probabilities |
# nolint end: line_length_linter.
#'
#' The functions `sortfin`, `orderfin`, `orderpos` and `sortorderpos` each
#'   offer three algorithms for finding `x` in `table`.
#'
#' With `method=1L` each value of `x` is searched independently using
#'   _binary search_, this is fastest for small `table`s.
#'
#' With `method=2L` the values of `x` are first sorted and then searched using
#'   _doubly exponential search_, this is the best all-around method.
#'
#' With `method=3L` the values of `x` are first sorted and then searched using
#'   simple merging, this is the fastest method if `table` is huge and `x` has
#'   similar size and distribution of values.
#'
#' With `method=NULL` the functions use a heuristic to determine the fastest
#'   algorithm.
#'
#' The functions `orderdup` and `sortorderdup` each offer two algorithms for
#'   setting the truth values in the return vector.
#'
#' With `method=1L` the return values are set directly which causes random
#'   write access on a possibly large return vector.
#'
#' With `method=2L` the return values are first set in a smaller bit-vector --
#'   random access limited to a smaller memory region -- and finally written
#'   sequentially to the logical output  vector.
#'
#' With `method=NULL` the functions use a heuristic to determine the fastest
#'   algorithm.
#'
#' @return see details
#' @keywords programming manip
#' @seealso [`match()`][match.integer64]
#' @examples
#'  message("check the code of 'optimizer64' for examples:")
#'  print(optimizer64)
#' @export
sortnut = function(sorted, ...) UseMethod("sortnut")

#' @rdname sortnut
#' @export
sortnut.integer64 = function(sorted, ...) {
  ret = .Call(C_r_ram_integer64_sortnut, x=sorted)
  names(ret) = c("nunique", "nties")
  ret
}

#' @rdname sortnut
#' @param table the original data with original order under the sorted vector
#' @param order an [`integer`] order vector that turns 'table' into 'sorted'
#' @export
ordernut = function(table, order, ...) UseMethod("ordernut")

#' @rdname sortnut
#' @export
ordernut.integer64 = function(table, order, ...) {
  ret = .Call(C_r_ram_integer64_ordernut, table=as.integer64(table), order=as.integer(order))
  names(ret) = c("nunique", "nties")
  ret
}

#' @rdname sortnut
#' @param x an [`integer64`] vector
#' @export
sortfin = function(sorted, x, ...) UseMethod("sortfin")

#' @rdname sortnut
#' @export
sortfin.integer64 = function(sorted, x, method=NULL, ...) {
  n = length(x)
  if (is.null(method)) {
    if (n < 2048L) {
      method = 1L
    } else if (n < length(sorted) / 128L) {
      method = 2L
    } else {
      method = 3L
    }
  } else {
    method = as.integer(method)
  }
  if (method == 1L) {
    .Call(C_r_ram_integer64_sortfin_asc
    , x=as.integer64(x)
    , sorted=as.integer64(sorted)
    , method=method
    , ret=logical(n)
    )
  } else {
    sx = clone(as.integer64(x))
    o = seq_along(x)
    ramsortorder(sx, o, na.last=FALSE, ...)
    ret = logical(n)
    ret[o] = .Call(C_r_ram_integer64_sortfin_asc
      , x=sx
      , sorted=as.integer64(sorted)
      , method=method
      , ret=ret
      )
    ret
  }
}

#' @rdname sortnut
#' @export
orderfin = function(table, order, x, ...) UseMethod("orderfin")

#' @rdname sortnut
#' @export
orderfin.integer64 = function(table, order, x, method=NULL, ...) {
  n = length(x)
  if (is.null(method)) {
    if (n < 4096L) {
      method = 1L
    } else if (n < length(table) / 8L) {
      method = 2L
    } else {
      method = 3L
    }
  } else {
    method = as.integer(method)
  }
  if (method == 1L) {
      .Call(C_r_ram_integer64_orderfin_asc
      , x=as.integer64(x)
      , table=as.integer64(table)
      , order=as.integer(order)
      , method=as.integer(method)
      , ret=logical(n)
      )
  } else {
    x = as.integer64(x)
    o = seq_along(x)
    ramorder(x, o, na.last=FALSE, ...)
    ret = logical(n)
    ret[o] = .Call(C_r_ram_integer64_orderfin_asc
      , x=x[o]
      , table=as.integer64(table)
      , order=as.integer(order)
      , method=as.integer(method)
      , ret=ret
      )
      ret
  }
}

#' @rdname sortnut
#' @export
orderpos = function(table, order, x, ...) UseMethod("orderpos")

#' @rdname sortnut
#' @param nomatch the value to be returned if an element is not found in the hashmap
#' @export
orderpos.integer64 = function(table, order, x, nomatch=NA, method=NULL, ...) {
  n = length(x)
  if (is.null(method)) {
    if (n < 4096L) {
      method = 1L
    } else if (n < length(table) / 8L) {
      method = 2L
    } else {
      method = 3L
    }
  } else {
    method = as.integer(method)
  }
  if (method == 1L) {
      .Call(C_r_ram_integer64_orderpos_asc
      , x=as.integer64(x)
      , table=as.integer64(table)
      , order=as.integer(order)
      , nomatch=as.integer(nomatch)
      , method=as.integer(method)
      , ret=integer(n)
      )
  } else {
    x = as.integer64(x)
    o = seq_along(x)
    ramorder(x, o, na.last=FALSE, ...)
    ret = integer(n)
    ret[o] = .Call(C_r_ram_integer64_orderpos_asc
      , x=x[o]
      , table=as.integer64(table)
      , order=as.integer(order)
      , nomatch=as.integer(nomatch)
      , method=as.integer(method)
      , ret=ret
      )
      ret
  }
}

#' @rdname sortnut
#' @export
sortorderpos = function(sorted, order, x, ...) UseMethod("sortorderpos")

#' @rdname sortnut
#' @export
sortorderpos.integer64 = function(sorted, order, x, nomatch=NA, method=NULL, ...) {
  n = length(x)
  if (is.null(method)) {
    if (n < 2048L) {
      method = 1L
    } else if (n < length(sorted) / 128L) {
      method = 2L
    } else {
      method = 3L
    }
  } else {
    method = as.integer(method)
  }
  if (method == 1L) {
      .Call(C_r_ram_integer64_sortorderpos_asc
      , x=as.integer64(x)
      , sorted=as.integer64(sorted)
      , order=as.integer(order)
      , nomatch=as.integer(nomatch)
      , method=as.integer(method)
      , ret=integer(n)
      )
  } else {
    sx = clone(as.integer64(x))
    o = seq_along(x)
    ramsortorder(sx, o, na.last=FALSE, ...)
    ret = integer(n)
    ret[o] = .Call(C_r_ram_integer64_sortorderpos_asc
      , x=sx
      , sorted=as.integer64(sorted)
      , order=as.integer(order)
      , nomatch=as.integer(nomatch)
      , method=as.integer(method)
      , ret=ret
      )
      ret
  }
}

#' @rdname sortnut
#' @export
orderdup = function(table, order, ...) UseMethod("orderdup")

#' @rdname sortnut
#' @export
orderdup.integer64 = function(table, order, method=NULL, ...) {
  if (is.null(method)) {
    if (length(table) < 4194304L)
        method = 1L
      else
        method = 2L
  } else {
    method = as.integer(method)
  }
  .Call(C_r_ram_integer64_orderdup_asc
  , table=as.integer64(table)
  , order=as.integer(order)
  , method=method
  , ret=logical(length(table))
  )
}

#' @rdname sortnut
#' @export
sortorderdup = function(sorted, order, ...) UseMethod("sortorderdup")

#' @rdname sortnut
#' @export
sortorderdup.integer64 = function(sorted, order, method=NULL, ...) {
  if (is.null(method)) {
    if (length(sorted) < 4194304L)
        method = 1L
      else
        method = 2L
  } else {
    method = as.integer(method)
  }
  .Call(C_r_ram_integer64_sortorderdup_asc
  , sorted=as.integer64(sorted)
  , order=as.integer(order)
  , method=method
  , ret=logical(length(sorted))
  )
}

#' @rdname sortnut
#' @param nunique number of unique elements, usually we get this from cache
#'   or call `sortnut` or `ordernut`
#' @export
sortuni = function(sorted, nunique, ...) UseMethod("sortuni")

#' @rdname sortnut
#' @export
sortuni.integer64 = function(sorted, nunique, ...) {
  .Call(C_r_ram_integer64_sortuni_asc
  , sorted=as.integer64(sorted)
  , ret=integer64(nunique)
  )
}

#' @rdname sortnut
#' @export
orderuni = function(table, order, nunique, ...) UseMethod("orderuni")

#' @rdname sortnut
#' @param keep.order determines order of results and speed: `FALSE` (the default)
#'   is faster and returns in sorted order, `TRUE` returns in the order of first
#'   appearance in the original data, but this requires extra work
#' @export
orderuni.integer64 = function(table, order, nunique, keep.order=FALSE, ...) {
  .Call(C_r_ram_integer64_orderuni_asc
  , table=as.integer64(table)
  , order=as.integer(order)
  , keep.order=as.logical(keep.order)
  , ret=integer64(nunique)
  )
}

#' @rdname sortnut
#' @export
sortorderuni = function(table, sorted, order, nunique, ...) UseMethod("sortorderuni")

#' @rdname sortnut
#' @export
sortorderuni.integer64 = function(table, sorted, order, nunique, ...) {
  .Call(C_r_ram_integer64_sortorderuni_asc
  , table=as.integer64(table)
  , sorted=as.integer64(sorted)
  , order=as.integer(order)
  , ret=integer64(nunique)
  )
}

#' @rdname sortnut
#' @export
orderupo = function(table, order, nunique, ...) UseMethod("orderupo")

#' @rdname sortnut
#' @export
orderupo.integer64 = function(table, order, nunique, keep.order=FALSE, ...) {
  .Call(C_r_ram_integer64_orderupo_asc
  , table=as.integer64(table)
  , order=as.integer(order)
  , keep.order=as.logical(keep.order)
  , ret=integer(nunique)
  )
}

#' @rdname sortnut
#' @export
sortorderupo = function(sorted, order, nunique, keep.order=FALSE, ...) UseMethod("sortorderupo")

#' @rdname sortnut
#' @export
sortorderupo.integer64 = function(sorted, order, nunique, keep.order=FALSE, ...) {
  .Call(C_r_ram_integer64_sortorderupo_asc
    , sorted=as.integer64(sorted)
    , order=as.integer(order)
    , keep.order=as.logical(keep.order)
    , ret=integer(nunique)
    )
}

#' @rdname sortnut
#' @param nties number of tied values, usually we get this from cache or
#'   call `sortnut` or `ordernut`
#' @export
ordertie = function(table, order, nties, ...) UseMethod("ordertie")

#' @rdname sortnut
#' @export
ordertie.integer64 = function(table, order, nties, ...) {
  .Call(C_r_ram_integer64_ordertie_asc
  , table=as.integer64(table)
  , order=as.integer(order)
  , ret=integer(nties)
  )
}

#' @rdname sortnut
#' @export
sortordertie = function(sorted, order, nties, ...) UseMethod("sortordertie")

#' @rdname sortnut
#' @export
sortordertie.integer64 = function(sorted, order, nties, ...) {
  .Call(C_r_ram_integer64_sortordertie_asc
  , sorted=as.integer64(sorted)
  , order=as.integer(order)
  , ret=integer(nties)
  )
}

#' @rdname sortnut
#' @export
sorttab = function(sorted, nunique, ...) UseMethod("sorttab")

#' @rdname sortnut
#' @export
sorttab.integer64 = function(sorted, nunique, ...) {
  .Call(C_r_ram_integer64_sorttab_asc
  , sorted=as.integer64(sorted)
  , ret=integer(nunique)
  )
}

#' @rdname sortnut
#' @export
ordertab = function(table, order, nunique, ...) UseMethod("ordertab")

#' @rdname sortnut
#' @param denormalize FALSE returns counts of unique values, TRUE returns each
#'   value with its counts
#' @export
ordertab.integer64 = function(table, order, nunique, denormalize=FALSE, keep.order=FALSE, ...) {
  denormalize = as.logical(denormalize)
  keep.order = as.logical(keep.order)
  .Call(C_r_ram_integer64_ordertab_asc
  , table=as.integer64(table)
  , order=as.integer(order)
  , denormalize=denormalize
  , keep.order=keep.order
  , ret=integer(if (denormalize || keep.order) length(table) else nunique)
  )
}

#' @rdname sortnut
#' @export
sortordertab = function(sorted, order, ...) UseMethod("sortordertab")

#' @rdname sortnut
#' @export
sortordertab.integer64 = function(sorted, order, denormalize=FALSE, ...) {
  .Call(C_r_ram_integer64_sortordertab_asc
  , sorted=as.integer64(sorted)
  , order=as.integer(order)
  , denormalize=as.logical(denormalize)
  , ret=integer(length(sorted))
  )
}

#' @rdname sortnut
#' @param na.skip.num 0 or the number of `NA`s. With 0, `NA`s are coded with 1L,
#'   with the number of `NA`s, these are coded with `NA`
#' @export
orderkey = function(table, order, na.skip.num=0L, ...) UseMethod("orderkey")

#' @rdname sortnut
#' @export
orderkey.integer64 = function(table, order, na.skip.num=0L, ...) {
  .Call(C_r_ram_integer64_orderkey_asc
  , table=as.integer64(table)
  , order=as.integer(order)
  , na.skip.num=na.skip.num
  , ret=integer(length(table))
  )
}

#' @rdname sortnut
#' @export
sortorderkey = function(sorted, order, na.skip.num=0L, ...) UseMethod("sortorderkey")

#' @rdname sortnut
#' @export
sortorderkey.integer64 = function(sorted, order, na.skip.num=0L, ...) {
    .Call(C_r_ram_integer64_sortorderkey_asc
    , sorted=as.integer64(sorted)
    , order=as.integer(order)
    , na.skip.num=na.skip.num
    , ret=integer(length(sorted))
    )
}

#' @rdname sortnut
#' @param na.count the number of `NA`s, needed for this low-level function algorithm
#' @export
orderrnk = function(table, order, na.count, ...) UseMethod("orderrnk")

#' @rdname sortnut
#' @export
orderrnk.integer64 = function(table, order, na.count, ...) {
  .Call(C_r_ram_integer64_orderrnk_asc
  , table=as.integer64(table)
  , order=as.integer(order)
  , na.count=as.integer(na.count)
  , ret=double(length(table))
  )
}

#' @rdname sortnut
#' @export
sortorderrnk = function(sorted, order, na.count, ...) UseMethod("sortorderrnk")
#' @rdname sortnut
#' @export
sortorderrnk.integer64 = function(sorted, order, na.count, ...) {
  .Call(C_r_ram_integer64_sortorderrnk_asc
  , sorted=as.integer64(sorted)
  , order=as.integer(order)
  , na.count=as.integer(na.count)
  , ret=double(length(sorted))
  )
}

#' @rdname sortnut
#' @param probs vector of probabilities in `[0..1]` for which we seek quantiles
#' @export
sortqtl = function(sorted, na.count, probs, ...) UseMethod("sortqtl")

#' @rdname sortnut
#' @export
sortqtl.integer64 = function(sorted, na.count, probs, ...) {
  n = length(sorted) - na.count  # nvalid
  ret = sorted[na.count + round(1L + probs * (n - 1L))]
  # TODO(#31): Remove this once `[` can return NA for integer64 directly
  ret[is.na(probs)] = NA
  ret
}

#' @rdname sortnut
#' @export
orderqtl = function(table, order, na.count, probs, ...) UseMethod("orderqtl")

#' @rdname sortnut
#' @export
orderqtl.integer64 = function(table, order, na.count, probs, ...) {
  n = length(table) - na.count  # nvalid
  idx = na.count + round(1L + probs * (n - 1L))
  ret = table[order[idx]]
  # TODO(#31): Remove this once `[` can return NA for integer64 directly
  ret[is.na(probs)] = NA
  ret
}

with_parameters_test_that("sortfin works", method=1:3, {
  x = as.integer64(1:10)
  r = sample(x)
  expect_identical(sortfin(integer64(), 1:10, method=method), rep(FALSE, 10L))
  expect_identical(sortfin(x, integer64(), method=method), logical())

  expect_true(all(sortfin(x, r, method=method)))

  expect_true(sortfin(x, 1L, method=method))
  expect_true(all(sortfin(x, c(1.0, 4.0), method=method)))
  expect_identical(sortfin(x, as.integer64(0:1), method=method), c(FALSE, TRUE))
  # only 'x' need be sorted
  expect_identical(sortfin(x, as.integer64(1:0), method=method), c(TRUE, FALSE))
})

with_parameters_test_that("orderfin and orderpos work", method=1:3, {
  x = as.integer64(1:10)
  idx = seq_along(x)

  expect_identical(orderfin(x, idx, 0:1, method=method), c(FALSE, TRUE))
  expect_identical(orderfin(x, idx, as.integer64(0:1), method=method), c(FALSE, TRUE))

  expect_identical(orderpos(x, idx, 0:1, method=method), c(NA_integer_, 1L))
  expect_identical(orderpos(x, idx, as.integer64(0:1), method=method), c(NA_integer_, 1L))

  # These were segfaulting due to length(order) < length(table)
  table = as.integer64(c(10L, 20L, 30L, 5L, 15L, 25L))
  order = c(4L, 1L, 5L, 2L, 6L, 3L) # order of table is 5, 10, 15, 20, 25, 30

  partial_order = c(4L, 1L, 5L, 2L) # order of subset of table is 5, 10, 15, 20

  x_search = as.integer64(c(5L, 10L, 15L, 20L, 25L, 30L, 99L))

  # reference with full order
  full_fin = c(TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, FALSE)
  full_pos = c(4L, 1L, 5L, 2L, 6L, 3L, NA_integer_)
  expect_identical(orderfin(table, order, x_search, method=method), full_fin)
  expect_identical(orderpos(table, order, x_search, method=method), full_pos)

  # with partial order, we search in a subset
  partial_fin = c(TRUE, TRUE, TRUE, TRUE, FALSE, FALSE, FALSE)
  partial_pos = c(4L, 1L, 5L, 2L, NA_integer_, NA_integer_, NA_integer_)
  expect_identical(orderfin(table, partial_order, x_search, method=method), partial_fin)
  expect_identical(orderpos(table, partial_order, x_search, method=method), partial_pos)

  expect_error(
    orderfin(as.integer64(10:1), 1:3, 8:11, method=method),
    "'table' is not sorted by 'order'",
    fixed = TRUE
  )
  expect_error(
    orderpos(as.integer64(10:1), 1:3, 8:11, method=method),
    "'table' is not sorted by 'order'",
    fixed = TRUE
  )
})

test_that("ordertab and orderdup work", {
  x = as.integer64(1:10)
  x = c(x, x[1:8], x[1:6])
  idx = order(x)

  expect_identical(ordertab(x, idx, 10L), rep(3:1, c(6L, 2L, 2L)))
  expect_identical(orderdup(x, idx), rep(c(FALSE, TRUE), c(10L, 14L)))
  expect_identical(orderdup(x, idx, method=2L), rep(c(FALSE, TRUE), c(10L, 14L)))
})

test_that("ordertab handles nunique smaller than actual", {
  x = as.integer64(1:10)
  x = c(x, x[1:8], x[1:6])
  o = order(x)

  # makes operation quite slow, but in my test, this segfaulted on the first invocation every time.
  #   do this inside 'local' because doing so inside 'expect_identical' causes gctorture to apply
  #   to _every_ allocation induced by testthat as well as rep(). This isolated form is much faster.
  res = local({
    on.exit(gctorture(FALSE)); gctorture(TRUE)
    ordertab(x, o, 4L)
  })
  expect_identical(res, rep(3L, 4L))
})

test_that("sortorderdup works with both methods", {
  # sortorderdup requires a sorted vector and an order vector
  # Case 1: Simple duplicates
  x = as.integer64(c(1, 1, 2, 3, 3, 3))
  o = seq_along(x)

  # FALSE for the first occurrence, TRUE for subsequent
  expected = c(FALSE, TRUE, FALSE, FALSE, TRUE, TRUE)

  # Method 1 (Direct set, default for small N)
  expect_identical(sortorderdup(x, o, method=1L), expected)

  # Method 2 (Bitflags, usually for large N, but we force it here for coverage)
  expect_identical(sortorderdup(x, o, method=2L), expected)

  # Case 2: All duplicates
  x_all = as.integer64(c(1, 1, 1))
  o_all = seq_along(x_all)
  expect_identical(sortorderdup(x_all, o_all, method=2L), c(FALSE, TRUE, TRUE))

  # Case 3: No duplicates
  x_none = as.integer64(c(1, 2, 3))
  o_none = seq_along(x_none)
  expect_identical(sortorderdup(x_none, o_none, method=2L), c(FALSE, FALSE, FALSE))
})

test_that("sortordertab works with normalization options", {
  # sortordertab calculates frequencies based on a sorted vector
  x = as.integer64(c(1, 1, 2, 3, 3, 3))
  o = seq_along(x)

  # denormalize = FALSE: Returns counts for unique values
  # Unique values are 1, 2, 3. Counts are 2, 1, 3.
  expect_identical(sortordertab(x, o, denormalize=FALSE), c(2L, 1L, 3L))

  # denormalize = TRUE: Returns counts expanded to the original vector positions
  # Positions 1,2 (val 1) -> count 2
  # Position 3 (val 2) -> count 1
  # Positions 4,5,6 (val 3) -> count 3
  expect_identical(sortordertab(x, o, denormalize=TRUE), c(2L, 2L, 1L, 3L, 3L, 3L))

  # Edge case: Single element
  x_single = as.integer64(1)
  o_single = 1L
  expect_identical(sortordertab(x_single, o_single, denormalize=FALSE), 1L)
  expect_identical(sortordertab(x_single, o_single, denormalize=TRUE), 1L)
})

test_that("sortorderuni and sortorderupo work", {
  # These functions allow retrieving unique values or their positions from sorted inputs
  # They share similar bitflag logic in C that needs covering

  # Data setup: 
  # original: 3, 1, 2, 1, 3
  # sorted:   1, 1, 2, 3, 3
  # order:    2, 4, 3, 1, 5
  val = as.integer64(c(3, 1, 2, 1, 3))
  o = order(val)
  s = val[o]

  # sortorderuni: extract unique values using the table, sorted, and order
  # It should return unique(val) but usually in the order of appearance or sorted depending on method
  # The C implementation uses bitflags to filter duplicates.
  # The R wrapper allocates 'nunique' size.

  # We need 'nunique' for the call.
  nu = 3L

  # sortorderuni returns the unique values
  expect_identical(sortorderuni(val, s, o, nunique=nu), as.integer64(c(3, 1, 2)))

  # sortorderupo returns the positions (indices) of the unique values
  # indices in 'val': 1 (3), 2 (1), 3 (2) -> c(1, 2, 3) 
  # (Note: it picks the first occurrence in the original vector if keep.order=TRUE implicit logic matches)
  expect_identical(sortorderupo(s, o, nunique=nu, keep.order=TRUE), 1:3)
})

test_that("sortorderkey works", {
  # This targets r_ram_integer64_sortorderkey_asc
  x = as.integer64(c(10, 10, 20, 30, 30))
  o = seq_along(x)

  # Keys should be assigned sequentially: 1, 1, 2, 3, 3
  expect_identical(sortorderkey(x, o), c(1L, 1L, 2L, 3L, 3L))

  # With NA skipping (na.skip.num)
  x_na = as.integer64(c(NA, NA, 10, 20))
  o_na = seq_along(x_na)

  # If we skip 2 NAs: NA, NA, 1, 2
  # Note: NA_integer_ is used for NAs in the key vector
  expect_identical(sortorderkey(x_na, o_na, na.skip.num=2L), c(NA_integer_, NA_integer_, 1L, 2L))
})

# /*
# R-Code for matching and other functions based on hashing
# S3 atomic 64bit integers for R
# (c) 2012 Jens Oehlschägel
# Licence: GPL2
# Provided 'as is', use at your own risk
# Created: 2011-12-11
# Last changed:  2011-12-11
# */

#' Function for measuring algorithmic performance of high-level and low-level integer64 functions
#'
#' @param nsmall size of smaller vector
#' @param nbig size of larger bigger vector
#' @param timefun a function for timing such as [bit::repeat.time()] or [system.time()]
#' @param what a vector of names of high-level functions
#' @param uniorder one of the order parameters that are allowed in [unique.integer64()] and [unipos.integer64()]
#' @param taborder one of the order parameters that are allowed in [table()]
#' @param plot set to FALSE to suppress plotting
#'
#' @details
#' `benchmark64` compares the following scenarios for the following use cases:
#'
#' | **scenario name** | **explanation**                                 |
#' |------------------:|:------------------------------------------------|
#' |            32-bit | applying Base R function to 32-bit integer data |
#' |            64-bit | applying bit64 function to 64-bit integer data (with no cache) |
#' |         hashcache | ditto when cache contains [hashmap()], see [hashcache()] |
#' |    sortordercache | ditto when cache contains sorting and ordering, see [sortordercache()] |
#' |        ordercache | ditto when cache contains ordering only, see [ordercache()] |
#' |          allcache | ditto when cache contains sorting, ordering and hashing |
#'
#' | **use case name** | **explanation**                         |
#' |------------------:|:----------------------------------------|
#' |             cache | filling the cache according to scenario |
#' |       match(s, b) | match small in big vector               |
#' |          s %in% b | small %in% big vector                   |
#' |       match(b, s) | match big in small vector               |
#' |          b %in% s | big %in% small vector                   |
#' |       match(b, b) | match big in (different) big vector     |
#' |          b %in% b | big %in% (different) big vector         |
#' |     duplicated(b) | duplicated of big vector                |
#' |         unique(b) | unique of big vector                    |
#' |          table(b) | table of big vector                     |
#' |           sort(b) | sorting of big vector                   |
#' |          order(b) | ordering of big vector                  |
#' |           rank(b) | ranking of big vector                   |
#' |       quantile(b) | quantiles of big vector                 |
#' |        summary(b) | summary of of big vector                |
#' |         factor(b) | coercion to factor of big vector        |
#' |           SESSION | exemplary session involving multiple calls (including cache filling costs) |
#'
#' Note that the timings for the cached variants do _not_ contain the
#'   time costs of building the cache, except for the timing of the exemplary
#'   user session, where the cache costs are included in order to evaluate amortization.
#'
#' @return
#' `benchmark64` returns a matrix with elapsed seconds, different high-level tasks
#'   in rows and different scenarios to solve the task in columns. The last row
#'   named 'SESSION' contains the elapsed seconds of the exemplary sesssion.
#'
#' `optimizer64` returns a dimensioned list with one row for each high-level
#'   function timed and two columns named after the values of the `nsmall` and
#'   `nbig` sample sizes. Each list cell contains a matrix with timings,
#'   low-level-methods in rows and three measurements `c("prep", "both", "use")`
#'   in columns. If it can be measured separately, `prep` contains the timing
#'   of preparatory work such as sorting and hashing, and `use` contains the
#'   timing of using the prepared work. If the function timed does both,
#'   preparation and use, the timing is in `both`.
#'
#' @seealso [integer64()]
#' @examples
#' message("this small example using system.time does not give serious timings\n
#' this we do this only to run regression tests")
#' benchmark64(nsmall=2^7, nbig=2^13, timefun=function(expr)system.time(expr, gcFirst=FALSE))
#' optimizer64(nsmall=2^7, nbig=2^13, timefun=function(expr)system.time(expr, gcFirst=FALSE)
#' , plot=FALSE
#' )
#'\dontrun{
#' message("for real measurement of sufficiently large datasets run this on your machine")
#' benchmark64()
#' optimizer64()
#'}
#' message("let's look at the performance results on Core i7 Lenovo T410 with 8 GB RAM")
#' data(benchmark64.data)
#' print(benchmark64.data)
#'
#' matplot(log2(benchmark64.data[-1, 1]/benchmark64.data[-1, ])
#' , pch=c("3", "6", "h", "s", "o", "a")
#' , xlab="tasks [last=session]"
#' , ylab="log2(relative speed) [bigger is better]"
#' )
#' matplot(t(log2(benchmark64.data[-1, 1]/benchmark64.data[-1, ]))
#' , type="b", axes=FALSE
#' , lwd=c(rep(1, 14), 3)
#' , xlab="context"
#' , ylab="log2(relative speed) [bigger is better]"
#' )
#' axis(1
#' , labels=c("32-bit", "64-bit", "hash", "sortorder", "order", "hash+sortorder")
#' , at=1:6
#' )
#' axis(2)
#' data(optimizer64.data)
#' print(optimizer64.data)
#' oldpar <- par(no.readonly = TRUE)
#' par(mfrow=c(2, 1))
#' par(cex=0.7)
#' for (i in 1:nrow(optimizer64.data)) {
#'  for (j in 1:2) {
#'    tim <- optimizer64.data[[i, j]]
#'   barplot(t(tim))
#'   if (rownames(optimizer64.data)[i]=="match")
#'    title(paste("match", colnames(optimizer64.data)[j], "in", colnames(optimizer64.data)[3-j]))
#'   else if (rownames(optimizer64.data)[i]=="%in%")
#'    title(paste(colnames(optimizer64.data)[j], "%in%", colnames(optimizer64.data)[3-j]))
#'   else
#'    title(paste(rownames(optimizer64.data)[i], colnames(optimizer64.data)[j]))
#'  }
#' }
#' par(mfrow=c(1, 1))
#' @keywords misc
#' @name benchmark64
NULL

#' @describeIn benchmark64 compares high-level integer64 functions against the
#'   integer functions from Base R
#' @export
# nocov start
# nolint start: brace_linter, line_length_linter.
benchmark64 = function(nsmall=2L^16L, nbig=2L^25L, timefun=repeat.time) {

  message('\ncompare performance for a complete sessions of calls')
  s = sample(nbig, nsmall, TRUE)
  b = sample(nbig, nbig, TRUE)
  b2 = sample(nbig, nbig, TRUE)

  tim1 = double(6L)
  names(tim1) = c("32-bit", "64-bit", "hashcache", "sortordercache", "ordercache", "allcache")

  s = as.integer(s)
  b = as.integer(b)
  b2 = as.integer(b2)

  for (i in 1:6) {
    message("\n=== ", names(tim1)[i], " ===")

    if (i == 2L) {
      s = as.integer64(s)
      b = as.integer64(b)
      b2 = as.integer64(b2)
    }

    tim1[i] = 0L

    tim1[i] = tim1[i] + timefun({
        switch(i,
          NULL, # i=1
          NULL, # i=2
          { hashcache(s); hashcache(b); hashcache(b2) },
          { sortordercache(s); sortordercache(b); sortordercache(b2) },
          { ordercache(s); ordercache(b); ordercache(b2) },
          { hashcache(s); hashcache(b); hashcache(b2); sortordercache(s); sortordercache(b); sortordercache(b2) }
        )
      })[3L]

    message('check data range, mean etc.')
    tim1[i] = tim1[i] + timefun({
        summary(b)
      })[3L]
    message('get all percentiles for plotting distribution shape')
    tim1[i] = tim1[i] + timefun({
        quantile(b, probs=seq(0.0, 1.0, 0.01))
      })[3L]
    message('list the upper and lower permille of values')
    tim1[i] = tim1[i] + timefun({
        quantile(b, probs=c(0.001, 0.999))
        sort(b, na.last=NA)
      })[3L]
    message('OK, for some of these values I want to see the complete ROW, so I need their positions in the data.frame')
    tim1[i] = tim1[i] + timefun({
        if (i == 1L) order(b) else order.integer64(b)
      })[3L]
    message('check if any values are duplicated')
    tim1[i] = tim1[i] + timefun({
        anyDuplicated(b)
      })[3L]
    message('since not unique, then check distribution of frequencies')
    tim1[i] = tim1[i] + timefun({
        if (i == 1L) tabulate(table(b, exclude=NULL)) else tabulate(table(b, return='list')$counts)
      })[3L]
    message("OK, let's plot the percentiles of unique values versus the percentiles allowing for duplicates")
    tim1[i] = tim1[i] + timefun({
        quantile(b, probs=seq(0.0, 1.0, 0.01))
        quantile(unique(b), probs=seq(0.0, 1.0, 0.01))
      })[3L]
    message('check whether we find a match for each fact in the dimension table')
    tim1[i] = tim1[i] + timefun({
        all(if (i == 1L) b %in% s else "%in%.integer64"(b, s))
      })[3L]
    message('check whether there are any dimension table entries not in the fact table')
    tim1[i] = tim1[i] + timefun({
        all(if (i == 1L) s %in% b else "%in%.integer64"(s, b))
      })[3L]
    message('check whether we find a match for each fact in a parallel fact table')
    tim1[i] = tim1[i] + timefun({
        all(if (i == 1L) b %in% b2 else "%in%.integer64"(b, b2))
      })[3L]
    message('find positions of facts in dimension table for joining')
    tim1[i] = tim1[i] + timefun({
        if (i == 1L) match(b, s) else match.integer64(b, s)
      })[3L]
    message('find positions of facts in parallel fact table for joining')
    tim1[i] = tim1[i] + timefun({
        if (i == 1L) match(b, b2) else match.integer64(b, b2)
      })[3L]
    message('out of curiosity: how well rank-correlated are fact and parallel fact table?')
    tim1[i] = tim1[i] + timefun({
        if (i == 1L)
          cor(rank(b, na.last="keep"), rank(b2, na.last="keep"), use="na.or.complete")
        else
          cor(rank.integer64(b), rank.integer64(b2), use="na.or.complete")
      })[3L]
    message('coerce to factor')
    tim1[i] = tim1[i] + timefun({
        factor(b)
      })[3L]

    remcache(s)
    remcache(b)
    remcache(b2)

    print(round(rbind(seconds=tim1, factor=tim1[1L]/tim1), 3L))
  }

  # 32-bit         64-bit      hashcache sortordercache     ordercache       allcache
  # 196.510          8.963          8.242          5.183         12.325          6.043
  # 32-bit         64-bit      hashcache sortordercache     ordercache       allcache
  # 1.000         21.924         23.842         37.913         15.944         32.519

  message("\nnow let's look more systematically at the components involved")
  s = sample(nbig, nsmall, TRUE)
  b = sample(nbig, nbig, TRUE)
  b2 = sample(nbig, nbig, TRUE)

  tim2 = matrix(0.0, 16L, 6L)
  dimnames(tim2) = list(
    c("cache", "match(s, b)", "s %in% b", "match(b, s)", "b %in% s", "match(b, b)", "b %in% b", "duplicated(b)", "unique(b)", "table(b)", "sort(b)", "order(b)", "rank(b)", "quantile(b)", "summary(b)", "factor(b)"),
    c("32-bit", "64-bit", "hashcache", "sortordercache", "ordercache", "allcache")
  )

  s = as.integer(s)
  b = as.integer(b)
  b2 = as.integer(b2)

  i = 1L
  for (i in 1:6) {
    if (i == 2L) {
      s = as.integer64(s)
      b = as.integer64(b)
      b2 = as.integer64(b2)
    }

    if (i > 2L) message(colnames(tim2)[i], " cache")
    tim2["cache", i] = timefun({
      switch(i,
        NULL, # i=1
        NULL, # i=2
        { hashcache(s); hashcache(b); hashcache(b2) },
        { sortordercache(s); sortordercache(b); sortordercache(b2) },
        { ordercache(s); ordercache(b); ordercache(b2) },
        { hashcache(s); hashcache(b); hashcache(b2); sortordercache(s); sortordercache(b); sortordercache(b2) }
      )
    })[3L]

    message(colnames(tim2)[i], " match(s, b)")
    tim2["match(s, b)", i] = timefun({
      if (i == 1L) match(s, b) else match.integer64(s, b)
    })[3L]

    message(colnames(tim2)[i], " s %in% b")
    tim2["s %in% b", i] = timefun({
      if (i == 1L) s %in% b else "%in%.integer64"(s, b)
    })[3L]

    message(colnames(tim2)[i], " match(b, s)")
    tim2["match(b, s)", i] = timefun({
      if (i == 1L) match(b, s) else match.integer64(b, s)
    })[3L]

    message(colnames(tim2)[i], " b %in% s")
    tim2["b %in% s", i] = timefun({
      if (i == 1L) b %in% s else "%in%.integer64"(b, s)
    })[3L]

    message(colnames(tim2)[i], " match(b, b)")
    tim2["match(b, b)", i] = timefun({
      if (i == 1L) match(b, b2) else match.integer64(b, b2)
    })[3L]

    message(colnames(tim2)[i], " b %in% b")
    tim2["b %in% b", i] = timefun({
      if (i == 1L) b %in% b2 else "%in%.integer64"(b, b2)
    })[3L]

    message(colnames(tim2)[i], " duplicated(b)")
    tim2["duplicated(b)", i] = timefun({
      duplicated(b)
    })[3L]

    message(colnames(tim2)[i], " unique(b)")
    tim2["unique(b)", i] = timefun({
      unique(b)
    })[3L]

    message(colnames(tim2)[i], " table(b)")
    tim2["table(b)", i] = timefun({
      if (i == 1L) table(b) else table(b, return='list')
    })[3L]

    message(colnames(tim2)[i], " sort(b)")
    tim2["sort(b)", i] = timefun({
      sort(b)
    })[3L]

    message(colnames(tim2)[i], " order(b)")
    tim2["order(b)", i] = timefun({
      if (i == 1L) order(b) else order.integer64(b)
    })[3L]

    message(colnames(tim2)[i], " rank(b)")
    tim2["rank(b)", i] = timefun({
      if (i == 1L) rank(b) else rank.integer64(b)
    })[3L]

    message(colnames(tim2)[i], " quantile(b)")
    tim2["quantile(b)", i] = timefun({
      quantile(b)
    })[3L]

    message(colnames(tim2)[i], " summary(b)")
    tim2["summary(b)", i] = timefun({
      summary(b)
    })[3L]

    message(colnames(tim2)[i], " factor(b)")
    tim2["factor(b)", i] = timefun({
      factor(b)
    })[3L]

    remcache(s)
    remcache(b)
    remcache(b2)

    tim3 = rbind(tim2, SESSION=tim1)
    #tim2 = tim2[, 1]/tim2

    cat("seconds")
    print(round(tim3, 3L))
    cat("factor")
    print(round(tim3[, 1L]/tim3, 3L))
  }



  #                32-bit 64-bit hashcache sortordercache ordercache allcache
  # cache           0.000  0.000     0.775          1.330      6.500    2.660
  # match(s, b)     0.820  0.218     0.004          0.025      0.093    0.004
  # s %in% b        0.810  0.234     0.003          0.022      0.093    0.003
  # match(b, s)     0.450  0.228     0.232          0.224      0.224    0.226
  # b %in% s        0.510  0.226     0.224          0.222      0.218    0.222
  # match(b, b)     2.370  0.870     0.505          0.890      0.880    0.505
  # b %in% b        2.350  0.850     0.480          0.865      0.870    0.483
  # duplicated(b)   0.875  0.510     0.141          0.116      0.383    0.117
  # unique(b)       0.930  0.555     0.447          0.156      0.427    0.450
  # table(b)      110.340  0.725     0.680          0.234      0.575    0.202
  # sort(b)         2.440  0.400     0.433          0.072      0.460    0.069
  # order(b)       12.780  0.680     0.615          0.036      0.036    0.035
  # rank(b)        13.480  0.860     0.915          0.240      0.545    0.246
  # quantile(b)     0.373  0.400     0.410          0.000      0.000    0.000
  # summary(b)      0.645  0.423     0.427          0.016      0.016    0.016
  # factor(b)       6.655  8.212     7.834          7.812      7.241    8.332
  # TOTAL         149.173  7.179     6.291          4.448     11.320    5.239
  #               32-bit  64-bit hashcache sortordercache ordercache allcache
  # cache              1   1.062     0.000          0.000      0.000    0.000
  # match(s, b)        1   3.761   230.420         32.475      8.843  217.300
  # s %in% b           1   3.462   234.090         36.450      8.735  237.386
  # match(b, s)        1   1.974     1.940          2.009      2.009    1.991
  # b %in% s           1   2.257     2.277          2.297      2.339    2.297
  # match(b, b)        1   2.724     4.693          2.663      2.693    4.693
  # b %in% b           1   2.765     4.896          2.717      2.701    4.862
  # duplicated(b)      1   1.716     6.195          7.572      2.283    7.500
  # unique(b)          1   1.676     2.082          5.972      2.180    2.067
  # table(b)           1 152.193   162.265        471.538    191.896  546.238
  # sort(b)            1   6.100     5.631         33.822      5.304   35.534
  # order(b)           1  18.794    20.780        357.840    354.297  366.950
  # rank(b)            1  15.674    14.732         56.167     24.734   54.797
  # quantile(b)        1   0.933     0.911        804.907    806.027  810.133
  # summary(b)         1   1.524     1.512         39.345     39.345   39.345
  # factor(b)          1   0.812     0.851          0.848      0.920    0.767
  # TOTAL              1  20.778    23.712         33.534     13.177   28.476

  tim3
}

#' @describeIn benchmark64 compares for each high-level integer64 function the Base
#'   R integer function with several low-level integer64 functions with and
#'   without caching
#' @export
optimizer64 = function(nsmall=2L^16L,
                       nbig=2L^25L,
                       timefun=repeat.time,
                       what=c("match", "%in%", "duplicated", "unique", "unipos", "table", "rank", "quantile", "factor"),
                       uniorder=c("original", "values", "any"),
                       taborder=c("values", "counts"),
                       plot=TRUE) {
  what = match.arg(what, several.ok=TRUE)
  uniorder = match.arg(uniorder)
  taborder = match.arg(taborder)
  ret = vector("list", 2L*length(what))
  dim(ret) = c(length(what), 2L)
  dimnames(ret) = list(what, c(nsmall, nbig))

  if (plot) {
    oldpar = par(no.readonly=TRUE)
    on.exit(par(oldpar))
    par(mfrow=c(2L, 1L))
  }

  if ("match" %in% what) {
    message("match: timings of different methods")
    N1 = c(nsmall, nbig)
    N2 = c(nbig, nsmall)
    for (i in seq_along(N1)) {
      n1 = N1[i]
      n2 = N2[i]
      x1 = c(sample(n2, n1 - 1L, TRUE), NA)
      x2 = c(sample(n2, n2 - 1L, TRUE), NA)
      tim = matrix(0.0, 9L, 3L)
      dimnames(tim) = list(
        c("match", "match.64", "hashpos", "hashrev", "sortorderpos", "orderpos", "hashcache", "sortorder.cache", "order.cache"),
        c("prep", "both", "use")
      )

      tim["match", "both"] = timefun({
        p = match(x1, x2)
      })[3L]
      x1 = as.integer64(x1)
      x2 = as.integer64(x2)

      tim["match.64", "both"] = timefun({
        p2 = match.integer64(x1, x2)
      })[3L]
      stopifnot(identical(p2, p))

      tim["hashpos", "prep"] = timefun({
        h2 = hashmap(x2)
      })[3L]
      tim["hashpos", "use"] = timefun({
        p2 = hashpos(h2, x1)
      })[3L]
      stopifnot(identical(p2, p))

      tim["hashrev", "prep"] = timefun({
        h1 = hashmap(x1)
      })[3L]
      tim["hashrev", "use"] = timefun({
        p1 = hashrev(h1, x2)
      })[3L]
      stopifnot(identical(p1, p))

      tim["sortorderpos", "prep"] = system.time({
        s2 = clone(x2)
        o2 = seq_along(x2)
        ramsortorder(s2, o2, na.last=FALSE)
      })[3L]
      tim["sortorderpos", "use"] = timefun({
        p2 = sortorderpos(s2, o2, x1)
      })[3L]
      stopifnot(identical(p2, p))

      tim["orderpos", "prep"] = timefun({
        o2 = seq_along(x2)
        ramorder(x2, o2, na.last=FALSE)
      })[3L]
      tim["orderpos", "use"] = timefun({
        p2 = orderpos(x2, o2, x1, method=2L)
      })[3L]
      stopifnot(identical(p2, p))

      hashcache(x2)
      tim["hashcache", "use"] = timefun({
        p2 = match.integer64(x1, x2)
      })[3L]
      stopifnot(identical(p2, p))
      remcache(x2)

      sortordercache(x2)
      tim["sortorder.cache", "use"] = timefun({
        p2 = match.integer64(x1, x2)
      })[3L]
      stopifnot(identical(p2, p))
      remcache(x2)

      ordercache(x2)
      tim["order.cache", "use"] = timefun({
        p2 = match.integer64(x1, x2)
      })[3L]
      stopifnot(identical(p2, p))
      remcache(x2)

      if (plot) {
        barplot(t(tim))
        n = format(c(n1, n2))
        title(paste("match", n[1L], "in", n[2L]))
      }

      ret[["match", as.character(n1)]] = tim
    }
  }

  if ("%in%" %in% what) {
    message("%in%: timings of different methods")
    N1 = c(nsmall, nbig)
    N2 = c(nbig, nsmall)
    for (i in seq_along(N1)) {
      n1 = N1[i]
      n2 = N2[i]
      x1 = c(sample(n2, n1 - 1L, TRUE), NA)
      x2 = c(sample(n2, n2 - 1L, TRUE), NA)
      tim = matrix(0.0, 10L, 3L)
      dimnames(tim) = list(
        c("%in%", "match.64", "%in%.64", "hashfin", "hashrin", "sortfin", "orderfin", "hash.cache", "sortorder.cache", "order.cache"),
        c("prep", "both", "use")
      )

      tim["%in%", "both"] = timefun({
        p = x1 %in% x2
      })[3L]
      x1 = as.integer64(x1)
      x2 = as.integer64(x2)

      tim["match.64", "both"] = timefun({
        p2 = match.integer64(x1, x2, nomatch = 0L) > 0L
      })[3L]
      stopifnot(identical(p2, p))

      tim["%in%.64", "both"] = timefun({
        p2 = "%in%.integer64"(x1, x2) # this is using the custom version
      })[3L]
      stopifnot(identical(p2, p))

      tim["hashfin", "prep"] = timefun({
        h2 = hashmap(x2)
      })[3L]
      tim["hashfin", "use"] = timefun({
        p2 = hashfin(h2, x1)
      })[3L]
      stopifnot(identical(p2, p))

      tim["hashrin", "prep"] = timefun({
        h1 = hashmap(x1)
      })[3L]
      tim["hashrin", "use"] = timefun({
        p1 = hashrin(h1, x2)
      })[3L]
      stopifnot(identical(p2, p))

      tim["sortfin", "prep"] = timefun({
        s2 = clone(x2)
        ramsort(s2, na.last=FALSE)
      })[3L]
      tim["sortfin", "use"] = timefun({
        p2 = sortfin(s2, x1)
      })[3L]
      stopifnot(identical(p2, p))

      tim["orderfin", "prep"] = timefun({
        o2 = seq_along(x2)
        ramorder(x2, o2, na.last=FALSE)
      })[3L]
      tim["orderfin", "use"] = timefun({
        p2 = orderfin(x2, o2, x1)
      })[3L]
      stopifnot(identical(p2, p))

      hashcache(x2)
      tim["hash.cache", "use"] = timefun({
        p2 = "%in%.integer64"(x1, x2)
      })[3L]
      stopifnot(identical(p2, p))
      remcache(x2)

      sortordercache(x2)
      tim["sortorder.cache", "use"] = timefun({
        p2 = "%in%.integer64"(x1, x2)
      })[3L]
      stopifnot(identical(p2, p))
      remcache(x2)

      ordercache(x2)
      tim["order.cache", "use"] = timefun({
        p2 = "%in%.integer64"(x1, x2)
      })[3L]
      stopifnot(identical(p2, p))
      remcache(x2)

      if (plot) {
        barplot(t(tim))
        n = format(c(n1, n2))
        title(paste(n[1L], "%in%", n[2L]))
      }

      ret[["%in%", as.character(n1)]] = tim
    }
  }
  if ("duplicated" %in% what) {
    message("duplicated: timings of different methods")
    N = c(nsmall, nbig)
    for (i in seq_along(N)) {
      n = N[i]
      x = c(sample(n, n - 1L, TRUE), NA)
      tim = matrix(0.0, 10L, 3L)
      dimnames(tim) = list(
        c("duplicated", "duplicated.64", "hashdup", "sortorderdup1", "sortorderdup2", "orderdup1", "orderdup2", "hash.cache", "sortorder.cache", "order.cache"),
        c("prep", "both", "use")
      )

      tim["duplicated", "both"] = timefun({
        p = duplicated(x)
      })[3L]
      x = as.integer64(x)

      tim["duplicated.64", "both"] = timefun({
        p2 = duplicated(x)
      })[3L]
      stopifnot(identical(p2, p))

      tim["hashdup", "prep"] = timefun({
        h = hashmap(x)
      })[3L]
      tim["hashdup", "use"] = timefun({
        p2 = hashdup(h)
      })[3L]
      stopifnot(identical(p2, p))

      tim["sortorderdup1", "prep"] = timefun({
        s = clone(x)
        o = seq_along(x)
        ramsortorder(s, o, na.last=FALSE)
        nunique = sortnut(s)[1L]
      })[3L]
      tim["sortorderdup1", "use"] = timefun({
        p2 = sortorderdup(s, o, method=1L)
      })[3L]
      stopifnot(identical(p2, p))

      tim["sortorderdup2", "prep"] = tim["sortorderdup1", "prep"]
      tim["sortorderdup2", "use"] = timefun({
        p2 = sortorderdup(s, o, method=2L)
      })[3L]
      stopifnot(identical(p2, p))

      tim["orderdup1", "prep"] = timefun({
        o = seq_along(x)
        ramorder(x, o, na.last=FALSE)
        nunique = ordernut(x, o)[1L]
      })[3L]
      tim["orderdup1", "use"] = timefun({
        p2 = orderdup(x, o, method=1L)
      })[3L]
      stopifnot(identical(p2, p))

      tim["orderdup2", "prep"] = tim["orderdup1", "prep"]
      tim["orderdup2", "use"] = timefun({
        p2 = orderdup(x, o, method=2L)
      })[3L]
      stopifnot(identical(p2, p))

      hashcache(x)
      tim["hash.cache", "use"] = timefun({
        p2 = duplicated(x)
      })[3L]
      stopifnot(identical(p2, p))
      remcache(x)

      sortordercache(x)
      tim["sortorder.cache", "use"] = timefun({
        p2 = duplicated(x)
      })[3L]
      stopifnot(identical(p2, p))
      remcache(x)

      ordercache(x)
      tim["order.cache", "use"] = timefun({
        p2 = duplicated(x)
      })[3L]
      stopifnot(identical(p2, p))
      remcache(x)

      if (plot) {
        barplot(t(tim), cex.names=0.7)
        title(paste0("duplicated(", n, ")"))
      }

      ret[["duplicated", as.character(n)]] = tim
    }
  }
  if ("unique" %in% what) {
    message("unique: timings of different methods")
    N = c(nsmall, nbig)
    for (i in seq_along(N)) {
      n = N[i]
      x = c(sample(n, n - 1L, TRUE), NA)
      tim = matrix(0.0, 15L, 3L)
      dimnames(tim) = list(
        c("unique", "unique.64", "hashmapuni", "hashuni", "hashunikeep", "sortuni", "sortunikeep", "orderuni", "orderunikeep", "hashdup", "sortorderdup", "hash.cache", "sort.cache", "sortorder.cache", "order.cache"),
        c("prep", "both", "use")
      )

      tim["unique", "both"] = timefun({
        p = unique(x)
      })[3L]
      x = as.integer64(x)
      p = as.integer64(p)
      if (uniorder == "values")
        ramsort(p, na.last=FALSE)

      tim["unique.64", "both"] = timefun({
        p2 = unique(x, order=uniorder)
      })[3L]
      if (uniorder != "any")
        stopifnot(identical.integer64(p2, p))

      tim["hashmapuni", "both"] = timefun({
        p2 = hashmapuni(x)
      })[3L]
      if (uniorder == "original")
        stopifnot(identical.integer64(p2, p))

      tim["hashuni", "prep"] = timefun({
        h = hashmap(x)
        # for(r in 1:r)h = hashmap(x, nunique=h$nunique)
      })[3L]
      tim["hashuni", "use"] = timefun({
        p2 = hashuni(h)
      })[3L]
      if (uniorder == "values")
        stopifnot(identical.integer64(sort(p2, na.last=FALSE), p))

      tim["hashunikeep", "prep"] = tim["hashuni", "prep"]
      tim["hashunikeep", "use"] = timefun({
        p2 = hashuni(h, keep.order=TRUE)
      })[3L]
      if (uniorder == "original")
        stopifnot(identical.integer64(p2, p))

      tim["sortuni", "prep"] = timefun({
        s = clone(x)
        ramsort(s, na.last=FALSE)
        nunique = sortnut(s)[1L]
      })[3L]
      tim["sortuni", "use"] = timefun({
        p2 = sortuni(s, nunique)
      })[3L]
      if (uniorder == "values")
        stopifnot(identical.integer64(sort(p2, na.last=FALSE), p))

      tim["sortunikeep", "prep"] = timefun({
        s = clone(x)
        o = seq_along(x)
        ramsortorder(s, o, na.last=FALSE)
        nunique = sortnut(s)[1L]
      })[3L]
      tim["sortunikeep", "use"] = timefun({
        p2 = sortorderuni(x, s, o, nunique)
      })[3L]
      if (uniorder == "original")
        stopifnot(identical.integer64(p2, p))

      tim["orderuni", "prep"] = timefun({
        o = seq_along(x)
        ramorder(x, o, na.last=FALSE)
        nunique = ordernut(x, o)[1L]
      })[3L]
      tim["orderuni", "use"] = timefun({
        p2 = orderuni(x, o, nunique)
      })[3L]
      if (uniorder == "values")
        stopifnot(identical.integer64(sort(p2, na.last=FALSE), p))

      tim["orderunikeep", "prep"] = tim["orderuni", "prep"]
      tim["orderunikeep", "use"] = timefun({
        p2 = orderuni(x, o, nunique, keep.order=TRUE)
        nunique = ordernut(x, o)[1L]
      })[3L]
      if (uniorder == "original")
        stopifnot(identical.integer64(p2, p))

      tim["hashdup", "prep"] = tim["hashuni", "prep"]
      tim["hashdup", "use"] = timefun({
        p2 = x[!hashdup(h)]
      })[3L]
      if (uniorder == "original")
        stopifnot(identical.integer64(p2, p))

      tim["sortorderdup", "prep"] = tim["sortunikeep", "prep"]
      tim["sortorderdup", "use"] = timefun({
        p2 = x[!sortorderdup(s, o)]
      })[3L]
      if (uniorder == "original")
        stopifnot(identical.integer64(p2, p))

      hashcache(x)
      tim["hash.cache", "use"] = timefun({
        p2 = unique(x, order=uniorder)
      })[3L]
      if (uniorder != "any")
        stopifnot(identical.integer64(p2, p))
      remcache(x)

      sortcache(x)
      tim["sort.cache", "use"] = timefun({
        p2 = unique(x, order=uniorder)
      })[3L]
      if (uniorder != "any")
        stopifnot(identical.integer64(p2, p))
      remcache(x)

      sortordercache(x)
      tim["sortorder.cache", "use"] = timefun({
        p2 = unique(x, order=uniorder)
      })[3L]
      if (uniorder != "any")
        stopifnot(identical.integer64(p2, p))
      remcache(x)

      ordercache(x)
      tim["order.cache", "use"] = timefun({
        p2 = unique(x, order=uniorder)
      })[3L]
      if (uniorder != "any")
        stopifnot(identical.integer64(p2, p))
      remcache(x)

      if (plot) {
        barplot(t(tim), cex.names=0.7)
        title(paste0("unique(", n, ", order=", uniorder, ")"))
      }

      ret[["unique", as.character(n)]] = tim
    }
  }
  if ("unipos" %in% what) {
    message("unipos: timings of different methods")
    N = c(nsmall, nbig)
    for (i in seq_along(N)) {
      n = N[i]
      x = c(sample(n, n - 1L, TRUE), NA)
      tim = matrix(0.0, 14L, 3L)
      dimnames(tim) = list(
        c("unique", "unipos.64", "hashmapupo", "hashupo", "hashupokeep", "sortorderupo", "sortorderupokeep", "orderupo", "orderupokeep", "hashdup", "sortorderdup", "hash.cache", "sortorder.cache", "order.cache"),
        c("prep", "both", "use")
      )

      tim["unique", "both"] = timefun({
        unique(x)
      })[3L]
      x = as.integer64(x)

      tim["unipos.64", "both"] = timefun({
        p = unipos(x, order=uniorder)
      })[3L]

      tim["hashmapupo", "both"] = timefun({
        p2 = hashmapupo(x)
      })[3L]
      if (uniorder == "original")
        stopifnot(identical(p2, p))

      tim["hashupo", "prep"] = timefun({
        h = hashmap(x)
        # if nunique is small we could re-build the hashmap at a smaller size
        # h = hashmap(x, nunique=h$nunique)
      })[3L]
      tim["hashupo", "use"] = timefun({
        p2 = hashupo(h)
      })[3L]
      if (uniorder == "values")
        stopifnot(identical(sort(p2, na.last=FALSE), sort(p, na.last=FALSE)))

      tim["hashupokeep", "prep"] = tim["hashupo", "prep"]
      tim["hashupokeep", "use"] = timefun({
        p2 = hashupo(h, keep.order=TRUE)
      })[3L]
      if (uniorder == "original")
        stopifnot(identical(p2, p))

      tim["sortorderupo", "prep"] = timefun({
        s = clone(x)
        o = seq_along(x)
        ramsortorder(s, o, na.last=FALSE)
        nunique = sortnut(s)[1L]
      })[3L]
      tim["sortorderupo", "use"] = timefun({
        p2 = sortorderupo(s, o, nunique)
      })[3L]
      if (uniorder == "values")
        stopifnot(identical(p2, p))

      tim["sortorderupokeep", "prep"] = timefun({
        s = clone(x)
        o = seq_along(x)
        ramsortorder(s, o, na.last=FALSE)
        nunique = sortnut(s)[1L]
      })[3L]
      tim["sortorderupokeep", "use"] = timefun({
        p2 = sortorderupo(s, o, nunique, keep.order=TRUE)
      })[3L]
      if (uniorder == "original")
        stopifnot(identical(p2, p))

      tim["orderupo", "prep"] = timefun({
        o = seq_along(x)
        ramorder(x, o, na.last=FALSE)
        nunique = ordernut(x, o)[1L]
      })[3L]
      tim["orderupo", "use"] = timefun({
        p2 = orderupo(x, o, nunique)
      })[3L]
      if (uniorder == "values")
        stopifnot(identical(p2, p))

      tim["orderupokeep", "prep"] = tim["orderupo", "prep"]
      tim["orderupokeep", "use"] = timefun({
        p2 = orderupo(x, o, nunique, keep.order=TRUE)
        nunique = ordernut(x, o)[1L]
      })[3L]
      if (uniorder == "original")
        stopifnot(identical(p2, p))

      tim["hashdup", "prep"] = tim["hashupo", "prep"]
      tim["hashdup", "use"] = timefun({
        p2 = (1:n)[!hashdup(h)]
      })[3L]
      if (uniorder == "original")
        stopifnot(identical(p2, p))

      tim["sortorderdup", "prep"] = tim["sortorderupokeep", "prep"]
      tim["sortorderdup", "use"] = timefun({
        p2 = (1:n)[!sortorderdup(s, o)]
      })[3L]
      if (uniorder == "original")
        stopifnot(identical(p2, p))

      hashcache(x)
      tim["hash.cache", "use"] = timefun({
        p2 = unipos(x, order=uniorder)
      })[3L]
      if (uniorder != "any")
        stopifnot(identical(p2, p))
      remcache(x)

      sortordercache(x)
      tim["sortorder.cache", "use"] = timefun({
        p2 = unipos(x, order=uniorder)
      })[3L]
      if (uniorder != "any")
        stopifnot(identical(p2, p))
      remcache(x)

      ordercache(x)
      tim["order.cache", "use"] = timefun({
        p2 = unipos(x, order=uniorder)
      })[3L]
      if (uniorder != "any")
        stopifnot(identical(p2, p))
      remcache(x)

      if (plot) {
        barplot(t(tim), cex.names=0.7)
        title(paste0("unipos(", n, ", order=", uniorder, ")"))
      }

      ret[["unipos", as.character(n)]] = tim
    }
  }
  if ("table" %in% what) {
    message("table: timings of different methods")
    N = c(nsmall, nbig)
    for (i in seq_along(N)) {
      n = N[i]
      x = c(sample.int(1024L, n - 1L, replace=TRUE), NA)
      tim = matrix(0.0, 13L, 3L)
      dimnames(tim) = list(
        c("tabulate", "table", "table.64", "hashmaptab", "hashtab", "hashtab2", "sorttab", "sortordertab", "ordertab", "ordertabkeep", "hash.cache", "sort.cache", "order.cache"),
        c("prep", "both", "use")
      )

      tim["tabulate", "both"] = timefun({
        tabulate(x)
      })[3L]

      tim["table", "both"] = timefun({
        p = table(x, exclude=NULL)
      })[3L]

      x = as.integer64(x)

      tim["table.64", "both"] = timefun({
        p2 = table(x, exclude=NULL, order=taborder)
      })[3L]
      stopifnot(identical(p2, p))

      tim["hashmaptab", "both"] = timefun({
        p = hashmaptab(x)
      })[3L]

      tim["hashtab", "prep"] = timefun({
        h = hashmap(x)
      })[3L]
      tim["hashtab", "use"] = timefun({
        p2 = hashtab(h)
      })[3L]
      stopifnot(identical(p2, p))

      tim["hashtab2", "prep"] = tim["hashtab", "prep"] + timefun({
          h = hashmap(x, nunique=h$nunique)
        })[3L]
      tim["hashtab2", "use"] = timefun({
        p2 = hashtab(h)
      })[3L]

      sortp = function(p) {
        s = p$values
        o = seq_along(s)
        ramsortorder(s, o, na.last=FALSE)
        list(values=s, counts=p$counts[o])
      }
      p = sortp(p)
      p2 = sortp(p2)
      stopifnot(identical(p2, p))

      tim["sorttab", "prep"] = timefun({
        s = clone(x)
        ramsort(s, na.last=FALSE)
        nunique = sortnut(s)[1L]
      })[3L]
      tim["sorttab", "use"] = timefun({
        p2 = list(values=sortuni(s, nunique), counts=sorttab(s, nunique))
      })[3L]
      stopifnot(identical(p2, p))

      tim["sortordertab", "prep"] = timefun({
        s = clone(x)
        o = seq_along(x)
        ramsortorder(s, o, na.last=FALSE)
        nunique = sortnut(s)[1L]
      })[3L]
      tim["sortordertab", "use"] = timefun({
        p2 = list(values=sortorderuni(x, s, o, nunique), counts=sortordertab(s, o))
      })[3L]
      p2 = sortp(p2)
      stopifnot(identical(p2, p))

      tim["ordertab", "prep"] = timefun({
        o = seq_along(x)
        ramorder(x, o, na.last=FALSE)
        nunique = ordernut(x, o)[1L]
      })[3L]
      tim["ordertab", "use"] = timefun({
        p2 = list(values=orderuni(x, o, nunique), counts=ordertab(x, o, nunique))
      })[3L]
      stopifnot(identical(p2, p))

      tim["ordertabkeep", "prep"] = tim["ordertab", "prep"]
      tim["ordertabkeep", "use"] = timefun({
        p2 = list(values=orderuni(x, o, nunique, keep.order=TRUE), counts=ordertab(x, o, nunique, keep.order=TRUE))
      })[3L]
      p2 = sortp(p2)
      stopifnot(identical(p2, p))

      hashcache(x)
      tim["hash.cache", "use"] = timefun({
        p = table(x, exclude=NULL, order=taborder)
      })[3L]
      remcache(x)

      sortordercache(x, na.last=TRUE)
      tim["sort.cache", "use"] = timefun({
        p2 = table(x, exclude=NULL, order=taborder)
      })[3L]
      stopifnot(identical(p2, p))
      remcache(x)

      ordercache(x, na.last=TRUE)
      tim["order.cache", "use"] = timefun({
        p2 = table(x, exclude=NULL, order=taborder)
      })[3L]
      stopifnot(identical(p2, p))
      remcache(x)

      if (plot) {
        barplot(t(tim), cex.names=0.7)
        title(paste0("table(", n, ", order=", taborder, ")"))
      }

      ret[["table", as.character(n)]] = tim
    }
  }
  if ("rank" %in% what) {
    message("rank: timings of different methods")
    N = c(nsmall, nbig)
    for (i in seq_along(N)) {
      n = N[i]
      x = c(sample(n, n - 1L, TRUE), NA)
      tim = matrix(0.0, 7L, 3L)
      dimnames(tim) = list(
        c("rank", "rank.keep", "rank.64", "sortorderrnk", "orderrnk", "sort.cache", "order.cache"),
        c("prep", "both", "use")
      )

      tim["rank", "both"] = timefun({
        rank(x)
      })[3L]

      tim["rank.keep", "both"] = timefun({
        p = rank(x, na.last="keep")
      })[3L]

      x = as.integer64(x)

      tim["rank.64", "both"] = timefun({
        p2 = rank.integer64(x)
      })[3L]
      stopifnot(identical(p2, p))

      tim["sortorderrnk", "prep"] = timefun({
        s = clone(x)
        o = seq_along(x)
        na.count = ramsortorder(s, o, na.last=FALSE)
      })[3L]
      tim["sortorderrnk", "use"] = timefun({
        p2 = sortorderrnk(s, o, na.count)
      })[3L]
      stopifnot(identical(p2, p))

      tim["orderrnk", "prep"] = timefun({
        o = seq_along(x)
        na.count = ramorder(x, o, na.last=FALSE)
      })[3L]
      tim["orderrnk", "use"] = timefun({
        p2 = orderrnk(x, o, na.count)
      })[3L]
      stopifnot(identical(p2, p))

      sortordercache(x)
      tim["sort.cache", "use"] = timefun({
        p2 = rank.integer64(x)
      })[3L]
      stopifnot(identical(p2, p))
      remcache(x)

      ordercache(x)
      tim["order.cache", "use"] = timefun({
        p2 = rank.integer64(x)
      })[3L]
      stopifnot(identical(p2, p))
      remcache(x)

      if (plot) {
        barplot(t(tim), cex.names=0.7)
        title(paste0("rank.integer64(", n, ")"))
      }

      ret[["rank", as.character(n)]] = tim
    }
  }
  if ("quantile" %in% what) {
    message("quantile: timings of different methods")
    N = c(nsmall, nbig)
    for (i in seq_along(N)) {
      n = N[i]
      x = c(sample(n, n - 1L, TRUE), NA)
      tim = matrix(0.0, 6L, 3L)
      dimnames(tim) = list(
        c("quantile", "quantile.64", "sortqtl", "orderqtl", "sort.cache", "order.cache"),
        c("prep", "both", "use")
      )

      tim["quantile", "both"] = timefun({
        p = quantile(x, type=1L, na.rm=TRUE)
      })[3L]
      p2 = p
      p = as.integer64(p2)
      names(p) = names(p2)

      x = as.integer64(x)

      tim["quantile.64", "both"] = timefun({
        p2 = quantile(x, na.rm=TRUE)
      })[3L]
      stopifnot(identical(p2, p))

      tim["sortqtl", "prep"] = timefun({
        s = clone(x)
        na.count = ramsort(s, na.last=FALSE)
      })[3L]
      tim["sortqtl", "use"] = timefun({
        p2 = sortqtl(s, na.count, seq(0.0, 1.0, 0.25))
      })[3L]
      stopifnot(identical(unname(p2), unname(p)))

      tim["orderqtl", "prep"] = timefun({
        o = seq_along(x)
        na.count = ramorder(x, o, na.last=FALSE)
      })[3L]
      tim["orderqtl", "use"] = timefun({
        p2 = orderqtl(x, o, na.count, seq(0.0, 1.0, 0.25))
      })[3L]
      stopifnot(identical(unname(p2), unname(p)))

      sortordercache(x)
      tim["sort.cache", "use"] = timefun({
        p2 = quantile(x, na.rm=TRUE)
      })[3L]
      stopifnot(identical(p2, p))
      remcache(x)

      ordercache(x)
      tim["order.cache", "use"] = timefun({
        p2 = quantile(x, na.rm=TRUE)
      })[3L]
      stopifnot(identical(p2, p))
      remcache(x)

      if (plot) {
        barplot(t(tim), cex.names=0.7)
        title(paste0("quantile(", n, ")"))
      }

      ret[["quantile", as.character(n)]] = tim
    }
  }

  if ("factor" %in% what) {
    message("factor: timings of different methods")
    N = c(nsmall, nbig)
    for (i in seq_along(N)) {
      n = N[i]
      x = c(sample(n, n - 1L, TRUE), NA)
      tim = matrix(0.0, 5L, 3L)
      dimnames(tim) = list(
        c("factor", "factor.64", "hashcache", "sortorder.cache", "order.cache"),
        c("prep", "both", "use")
      )

      tim["factor", "both"] = timefun({
        p = base::factor(x)
      })[3L]
      x = as.integer64(x)

      tim["factor.64", "both"] = timefun({
        p2 = factor(x)
      })[3L]
      stopifnot(identical(p2, p))

      hashcache(x)
      tim["hashcache", "use"] = timefun({
        p2 = factor(x)
      })[3L]
      stopifnot(identical(p2, p))
      remcache(x)

      sortordercache(x)
      tim["sortorder.cache", "use"] = timefun({
        p2 = factor(x)
      })[3L]
      stopifnot(identical(p2, p))
      remcache(x)

      ordercache(x)
      tim["order.cache", "use"] = timefun({
        p2 = factor(x)
      })[3L]
      stopifnot(identical(p2, p))
      remcache(x)

      if (plot) {
        barplot(t(tim), cex.names=0.7)
        title(paste0("factor(", n, ")"))
      }

      ret[["factor", as.character(n)]] = tim
    }
  }

  ret
}
# nolint end: brace_linter, line_length_linter.
# nocov end

#' 64-bit integer matching
#'
#' `match` returns a vector of the positions of (first) matches of its first
#'   argument in its second.
#' `%in%` is a more intuitive interface as a binary operator, which returns a
#'   logical vector indicating if there is a match or not for its left operand.
#'
#' @param x integer64 vector: the values to be matched, optionally carrying a
#'   cache created with [hashcache()]
#' @param table integer64 vector: the values to be matched against, optionally
#'   carrying a cache created with [hashcache()] or [sortordercache()]
#' @param nomatch the value to be returned in the case when no match is found.
#'   Note that it is coerced to integer.
#' @param nunique NULL or the number of unique values of table (including NA).
#'   Providing `nunique` can speed-up matching when `table` has no cache. Note
#'   that a wrong nunique can cause undefined behaviour up to a crash.
#' @param method NULL for automatic method selection or a suitable low-level
#'   method, see details
#' @param ... ignored
#'
#' @details
#' These functions automatically choose from several low-level functions
#'   considering the size of `x` and `table` and the availability of caches.
#'
#' Suitable methods for `%in%.integer64` are
#'  - [`hashpos`] (hash table lookup)
#'  - [`hashrev`] (reverse lookup)
#'  - [`sortorderpos`] (fast ordering)
#'  - [`orderpos`] (memory saving ordering).
#'
#' Suitable methods for `match.integer64` are
#'  - [`hashfin`] (hash table lookup)
#'  - [`hashrin`] (reverse lookup)
#'  - [`sortfin`] (fast sorting)
#'  - [`orderfin`] (memory saving ordering).
#'
#' @return
#' A vector of the same length as `x`.
#'
#' `match`: An integer vector giving the position in `table` of
#'   the first match if there is a match, otherwise `nomatch`.
#'
#' If `x[i]` is found to equal `table[j]` then the value
#'   returned in the `i`-th position of the return value is `j`,
#'   for the smallest possible `j`.  If no match is found, the value
#'   is `nomatch`.
#'
#' `%in%`: A logical vector, indicating if a match was located for
#'   each element of `x`: thus the values are `TRUE` or
#'   `FALSE` and never `NA`.
#'
#' @seealso [match()]
#' @examples
#' x <- as.integer64(c(NA, 0:9), 32)
#' table <- as.integer64(c(1:9, NA))
#' match(x, table)
#' x %in% table
#'
#' x <- as.integer64(sample(c(rep(NA, 9), 0:9), 32, TRUE))
#' table <- as.integer64(sample(c(rep(NA, 9), 1:9), 32, TRUE))
#'
#' \dontrun{
#'     library(bit)
#'     message("check when reverse hash-lookup beats standard hash-lookup")
#'     e <- 4:24
#'     timx <- timy <- matrix(NA, length(e), length(e), dimnames=list(e, e))
#'     for (iy in seq_along(e))
#'     for (ix in 1:iy) {
#'         nx <- 2^e[ix]
#'         ny <- 2^e[iy]
#'         x <- as.integer64(sample(ny, nx, FALSE))
#'         y <- as.integer64(sample(ny, ny, FALSE))
#'         #hashfun(x, bits=as.integer(5))
#'         timx[ix, iy] <- repeat.time({
#'         hx <- hashmap(x)
#'         py <- hashrev(hx, y)
#'         })[3]
#'         timy[ix, iy] <- repeat.time({
#'         hy <- hashmap(y)
#'         px <- hashpos(hy, x)
#'         })[3]
#'         #identical(px, py)
#'         print(round(timx[1:iy, 1:iy]/timy[1:iy, 1:iy], 2), na.print="")
#'     }
#'
#'     message("explore best low-level method given size of x and table")
#'     B1 <- 1:27
#'     B2 <- 1:27
#'     tim <- array(NA, dim=c(length(B1), length(B2), 5)
#'  , dimnames=list(B1, B2, c("hashpos", "hashrev", "sortpos1", "sortpos2", "sortpos3")))
#'     for (i1 in B1)
#'     for (i2 in B2)
#'     {
#'       b1 <- B1[i1]
#'       b2 <- B1[i2]
#'       n1 <- 2^b1
#'       n2 <- 2^b2
#'       x1 <- as.integer64(c(sample(n2, n1-1, TRUE), NA))
#'       x2 <- as.integer64(c(sample(n2, n2-1, TRUE), NA))
#'       tim[i1, i2, 1] <- repeat.time({h <- hashmap(x2);hashpos(h, x1);rm(h)})[3]
#'       tim[i1, i2, 2] <- repeat.time({h <- hashmap(x1);hashrev(h, x2);rm(h)})[3]
#'       s <- clone(x2); o <- seq_along(s); ramsortorder(s, o)
#'       tim[i1, i2, 3] <- repeat.time(sortorderpos(s, o, x1, method=1))[3]
#'       tim[i1, i2, 4] <- repeat.time(sortorderpos(s, o, x1, method=2))[3]
#'       tim[i1, i2, 5] <- repeat.time(sortorderpos(s, o, x1, method=3))[3]
#'       rm(s, o)
#'       print(apply(tim, 1:2, function(ti)if(any(is.na(ti)))NA else which.min(ti)))
#'     }
#' }
#' @keywords manip logic
#' @export
match.integer64 = function(x, table, nomatch = NA_integer_, nunique=NULL, method=NULL, ...) {
  # trivial cases for zero length input
  if (!length(x)) return(integer())
  if (!length(table)) {
    nomatch = as.integer(c(nomatch, NA_integer_)[1L])
    return(rep(nomatch, length(x)))
  }
  stopifnot(is.integer64(x))
  table = as.integer64(table)
  cache_env = cache(table)
  if (is.null(method)) {
    if (is.null(cache_env)) {
            nx <- length(x)
            if (is.null(nunique))
                nunique <- length(table)
            btable <- as.integer(ceiling(log2(nunique*1.5)))
            bx <- as.integer(ceiling(log2(nx*1.5)))
            if (bx<=17L && btable>=16L) {
                method <- "hashrev"
            } else {
                method <- "hashpos"
            }
    } else if (!is.null(cache_env$hashmap)) {
        method <- "hashpos"
    } else if (!is.null(cache_env$sort) && !is.null(cache_env$order) && (length(table)>length(x) || length(x)<4096L)) {
        method <- "sortorderpos"
    } else if (!is.null(cache_env$order) && (length(table)>length(x) || length(x)<4096L)) {
        method <- "orderpos"
    } else {
        nx <- length(x)
        if (is.null(nunique)) {
            if (!is.null(cache_env$nunique))
                nunique <- cache_env$nunique
            else
                nunique <- length(table)
        }
        btable <- as.integer(ceiling(log2(nunique*1.5)))
        bx <- as.integer(ceiling(log2(nx*1.5)))
        if (bx<=17L && btable>=16L) {
            method <- "hashrev"
        } else {
            method <- "hashpos"
        }
    }
  }
  method = match.arg(method, c("hashpos", "hashrev", "sortorderpos", "orderpos"))
  switch(method,
    hashpos={
      if (is.null(cache_env) || is.null(cache_env$hashmap)) {
        if (exists("btable", inherits=FALSE)) {
          h <- hashmap(table, hashbits=btable)
        } else {
          if (is.null(nunique))
            nunique <- cache_env$nunique
          h <- hashmap(table, nunique=nunique)
        }
      } else {
        h <- cache_env
      }
      p = hashpos(h, x, nomatch=nomatch)
    },
    hashrev={
      cache_env = cache(x)
      if (is.null(cache_env) || is.null(cache_env$hashmap)) {
        if (exists("bx", inherits=FALSE)) {
          h <- hashmap(x, bits=bx)
        } else {
          if (is.null(nunique))
            nunique <- cache_env$nunique
          h <- hashmap(x, nunique=nunique)
        }
      } else {
        h <- cache_env
      }
      p = hashrev(h, table, nomatch=nomatch)
    },
    sortorderpos={
      if (is.null(cache_env) || !exists("sort", cache_env) || !exists("order", cache_env)) {
        s <- clone(table)
        o <- seq_along(s)
        ramsortorder(s, o, na.last=FALSE)
      } else {
        s <- get("sort", cache_env)
        o <- get("order", cache_env)
      }
      p = sortorderpos(s, o, x, nomatch=nomatch)
    },
    orderpos={
      if (is.null(cache_env) || !exists("order", cache_env)) {
        o <- seq_along(table)
        ramorder(table, o, na.last=FALSE)
      } else {
        o <- get("order", cache_env)
      }
      p = orderpos(table, o, x, nomatch=nomatch)
    }
  )
  p
}

#' @rdname match.integer64
#' @export
`%in%.integer64` = function(x, table, ...) {
  if (!length(x)) return(logical())
  if (!length(table)) return(rep(FALSE, length(x)))
  stopifnot(is.integer64(x))
  if (!is.integer64(table))
    table = as.integer64(table)
  nunique = NULL
  cache_env = cache(table)
  if (is.null(cache_env)) {
    nx = length(x)
    if (is.null(nunique))
      nunique = length(table)
    btable = as.integer(ceiling(log2(nunique*1.5)))
    bx = as.integer(ceiling(log2(nx*1.5)))
    if (bx <= 17L && btable >= 16L) {
      method = "hashrin"
    } else {
      method = "hashfin"
    }
  } else if (!is.null(cache_env$hashmap)) {
    method = "hashfin"
  } else if (!is.null(cache_env$sort) && (length(table) > length(x) || length(x) < 4096L)) {
    method = "sortfin"
  } else if (!is.null(cache_env$order) && (length(table) > length(x) || length(x) < 4096L)) {
    method = "orderfin"
  } else {
    nx = length(x)
    if (is.null(nunique)) {
      if (!is.null(cache_env$nunique))
        nunique = cache_env$nunique
      else
        nunique = length(table)
    }
    btable = as.integer(ceiling(log2(nunique*1.5)))
    bx = as.integer(ceiling(log2(nx*1.5)))
    if (bx <= 17L && btable >= 16L) {
      method = "hashrin"
    } else {
      method = "hashfin"
    }
  }
  method = match.arg(method, c("hashfin", "hashrin", "sortfin", "orderfin"))
  switch(method,
    hashfin={
      if (is.null(cache_env) || is.null(cache_env$hashmap)) {
        if (exists("btable", inherits=FALSE)) {
          h = hashmap(table, hashbits=btable)
        } else {
          if (is.null(nunique))
            nunique = cache_env$nunique
          h = hashmap(table, nunique=nunique)
        }
      } else {
        h = cache_env
      }
      p = hashfin(h, x)
    },
    hashrin={
      cache_env = cache(x)
      if (is.null(cache_env) || is.null(cache_env$hashmap)) {
        if (exists("bx", inherits=FALSE)) {
          h = hashmap(x, bits=bx)
        } else {
          if (is.null(nunique))
            nunique = cache_env$nunique
          h = hashmap(x, nunique=nunique)
        }
      } else {
        h = cache_env
      }
      p = hashrin(h, table)
    },
    sortfin={
      if (is.null(cache_env) || !exists("sort", cache_env)) {
        s = clone(table)
        ramsort(s, na.last=FALSE)
      } else {
        s = get("sort", cache_env)
      }
      p = sortfin(s, x)
    },
    orderfin={
      if (is.null(cache_env) || !exists("order", cache_env)) {
        o = seq_along(s)
        ramorder(table, o, na.last=FALSE)
      } else {
        o = get("order", cache_env)
      }
      p = orderfin(table, o, x)
    }
  )
  p
}

#' Determine Duplicate Elements of integer64
#'
#' `duplicated()` determines which elements of a vector or data frame are duplicates
#'   of elements with smaller subscripts, and returns a logical vector
#'   indicating which elements (rows) are duplicates.
#'
#' @param x a vector or a data frame or an array or `NULL`.
#' @param incomparables ignored
#' @param nunique NULL or the number of unique values (including NA). Providing
#'   `nunique` can speed-up matching when `x` has no cache. Note that a wrong
#'   `nunique` can cause undefined behaviour up to a crash.
#' @param method NULL for automatic method selection or a suitable low-level
#'   method, see details
#' @param ... ignored
#'
#' @details
#' This function automatically chooses from several low-level functions
#'   considering the size of `x` and the availability of a cache.
#'
#' Suitable methods are
#'  - [`hashdup`] (hashing)
#'  - [`sortorderdup`] (fast ordering)
#'  - [`orderdup`] (memory saving ordering).
#'
#' @return `duplicated()`: a logical vector of the same length as `x`.
#' @seealso [duplicated()], [unique.integer64()]
#' @examples
#' x <- as.integer64(sample(c(rep(NA, 9), 1:9), 32, TRUE))
#' duplicated(x)
#'
#' stopifnot(identical(duplicated(x),  duplicated(as.integer(x))))
#' @keywords logic manip
#' @export
duplicated.integer64 = function(x, incomparables = FALSE, nunique = NULL, method = NULL, ...) {
  stopifnot(identical(incomparables, FALSE))
  cache_env = cache(x)
  if (is.null(nunique) && !is.null(cache_env))
    nunique = cache_env$nunique
  if (is.null(method)) {
    if (is.null(cache_env)) {
      if (length(x)>50000000L)
        method <- "sortorderdup" # nocov. Too large for practical unit tests.
      else
        method <- "hashdup"
    } else if (!is.null(cache_env$sort) && !is.null(cache_env$order)) {
      method = "sortorderdup"
    } else if (!is.null(cache_env$hashmap)) {
      method = "hashdup"
    } else if (!is.null(cache_env$order)) {
      method = "orderdup"
    } else if (length(x) > 50000000L) {
      method = "sortorderdup"
    } else {
      method = "hashdup"
    }
  }
  method = match.arg(method, c("hashdup", "sortorderdup", "orderdup"))
  switch(method,
    hashdup={
      if (is.null(cache_env) || is.null(cache_env$hashmap))
        h <- hashmap(x, nunique=nunique)
      else
        h <- cache_env
      p = hashdup(h)
    },
    sortorderdup={
      if (is.null(cache_env) || is.null(cache_env$sort) || is.null(cache_env$order)) {
        s <- clone(x)
        o <- seq_along(s)
        ramsortorder(s, o, na.last=FALSE)
      } else {
        s <- get("sort", cache_env, inherits=FALSE)
        o <- get("order", cache_env, inherits=FALSE)
      }
      p = sortorderdup(s, o)
    },
    orderdup={
      if (is.null(cache_env) || is.null(cache_env$order)) {
        o <- seq_along(x)
        ramorder(x, o, na.last=FALSE)
      } else {
        o <- get("order", cache_env, inherits=FALSE)
      }
      p = orderdup(x, o)
    }
  )
  p
}

#' Extract Unique Elements from integer64
#'
#' `unique` returns a vector like `x` but with duplicate elements/rows removed.
#'
#' @param x a vector or a data frame or an array or `NULL`.
#' @param incomparables ignored
#' @param order The order in which unique values will be returned, see details
#' @param nunique NULL or the number of unique values (including NA). Providing
#'   `nunique` can speed-up matching when `x` has no cache. Note that a wrong
#'   `nunique`` can cause undefined behaviour up to a crash.
#' @param method NULL for automatic method selection or a suitable low-level
#'   method, see details
#' @param ... ignored
#'
#' @details
#' This function automatically chooses from several low-level functions
#'   considering the size of `x` and the availability of a cache.
#'
#' Suitable methods are
#'  - [`hashmapuni`] (simultaneously creating and using a hashmap)
#'  - [`hashuni`] (first creating a hashmap then using it)
#'  - [`sortuni`] (fast sorting for sorted order only)
#'  - [`sortorderuni`] (fast ordering for original order only)
#'  - [`orderuni`] (memory saving ordering).
#'
#' The default `order="original"` returns unique values in the order of the
#'   first appearance in `x` like in [unique()], this costs extra processing.
#'   `order="values"` returns unique values in sorted order like in [table()],
#'   this costs extra processing with the hash methods but comes for free.
#'   `order="any"` returns unique values in undefined order, possibly faster.
#'   For hash methods this will be a quasi random order, for sort methods this
#'   will be sorted order.
#'
#' @return For a vector, an object of the same type of `x`, but with only
#'   one copy of each duplicated element.  No attributes are copied (so
#'   the result has no names).
#'
#' @seealso [unique()] for the generic, [unipos()] which gives the indices
#'   of the unique elements and [table()] which gives frequencies
#'   of the unique elements.
#'
#' @examples
#' x <- as.integer64(sample(c(rep(NA, 9), 1:9), 32, TRUE))
#' unique(x)
#' unique(x, order="values")
#'
#' stopifnot(identical(unique(x),  x[!duplicated(x)]))
#' stopifnot(identical(unique(x),  as.integer64(unique(as.integer(x)))))
#' stopifnot(identical(unique(x, order="values")
#' ,  as.integer64(sort(unique(as.integer(x)), na.last=FALSE))))
#'
#' @keywords manip logic
#' @export
unique.integer64 = function(x,
                             incomparables=FALSE,
                             order=c("original", "values", "any"),
                             nunique=NULL,
                             method=NULL,
                             ...) {
  stopifnot(identical(incomparables, FALSE))
  order = match.arg(order)
  cache_env = cache(x)
  keep.order = order == "original"
  if (is.null(nunique) && !is.null(cache_env))
    nunique = cache_env$nunique
  if (is.null(method)) {
    if (is.null(cache_env)) {
        if (order=="values")
            method <- "sortuni"
        else
            method <- "hashmapuni"
    } else {
      switch(order,
        original = {
          if (!is.null(cache_env$hashmap))
            method <- "hashuni"
          else if (!is.null(cache_env$order)) {
            if (!is.null(cache_env$sort))
              method <- "sortorderuni"
            else
              method <- "orderuni"
          } else {
            method <- "hashmapuni"
          }
        },
        values = {
          if (!is.null(cache_env$sort))
            method <- "sortuni"
          else if (!is.null(cache_env$order))
            method <- "orderuni"
          else if (!is.null(cache_env$hashmap) && cache_env$nunique<length(x)/2L)
            method <- "hashuni"
          else
            method <- "sortuni"
        },
        any = {
          if (!is.null(cache_env$sort))
            method <- "sortuni"
          else if (!is.null(cache_env$hashmap))
            method <- "hashuni"
          else if (!is.null(cache_env$order))
            method <- "orderuni"
          else
            method <- "sortuni"
        }
      )
    }
  }
  method = match.arg(method, c("hashmapuni", "hashuni", "sortuni", "sortorderuni", "orderuni"))
  switch(method,
    hashmapuni={
      p = hashmapuni(x, nunique=nunique)
    },
    hashuni={
      if (is.null(cache_env) || is.null(cache_env$hashmap))
        h <- hashmap(x, nunique=nunique)
      else
        h <- cache_env
      p = hashuni(h, keep.order=keep.order)
      if (order=="values")
        ramsort(p, na.last=FALSE)
    },
    sortuni={
      if (is.null(cache_env) || is.null(cache_env$sort)) {
        s <- clone(x)
        ramsort(s, na.last=FALSE)
      } else {
        s <- get("sort", cache_env, inherits=FALSE)
      }
      if (is.null(nunique))
        nunique <- sortnut(s)[1L]
      p = sortuni(s, nunique)
    },
    sortorderuni={
      if (is.null(cache_env) || is.null(cache_env$sort) || is.null(cache_env$order)) {
        s <- clone(x)
        o <- seq_along(x)
        ramsortorder(s, o, na.last=FALSE)
      } else {
        s <- get("sort", cache_env, inherits=FALSE)
        o <- get("order", cache_env, inherits=FALSE)
      }
      if (is.null(nunique))
        nunique <- sortnut(s)[1L]
      p = sortorderuni(x, s, o, nunique)
    },
    orderuni={
      if (is.null(cache_env) || is.null(cache_env$order)) {
        o <- seq_along(x)
        ramorder(x, o, na.last=FALSE)
      } else {
        o <- get("order", cache_env, inherits=FALSE)
      }
      if (is.null(nunique))
        nunique <- ordernut(x, o)[1L]
      p = orderuni(x, o, nunique, keep.order=keep.order)
    }
  )
  p
}

#' Extract Positions of Unique Elements
#'
#' `unipos` returns the positions of those elements returned by [unique()].
#'
#' @param x a vector or a data frame or an array or `NULL`.
#' @param incomparables ignored
#' @param order The order in which positions of unique values will be returned,
#'   see details
#' @param nunique NULL or the number of unique values (including NA). Providing
#'   `nunique` can speed-up when `x` has no cache. Note that a wrong `nunique`
#'   can cause undefined behaviour up to a crash.
#' @param method NULL for automatic method selection or a suitable low-level
#'   method, see details
#' @param ... ignored
#'
#' @details
#' This function automatically chooses from several low-level functions
#'   considering the size of `x` and the availability of a cache.
#'
#' Suitable methods are
#'  - [`hashmapupo`] (simultaneously creating and using a hashmap)
#'  - [`hashupo`] (first creating a hashmap then using it)
#'  - [`sortorderupo`] (fast ordering)
#'  - [`orderupo`] (memory saving ordering).
#'
#' The default `order="original"` collects unique values in the order of
#'   the first appearance in `x` like in [unique()], this costs extra processing.
#'   `order="values"` collects unique values in sorted order like in [table()],
#'   this costs extra processing with the hash methods but comes for free.
#'   `order="any"` collects unique values in undefined order, possibly faster.
#'   For hash methods this will be a quasi random order, for sort methods this
#'   will be sorted order.
#'
#' @return an integer vector of positions
#' @seealso [unique.integer64()] for unique values and [match.integer64()]
#'   for general matching.
#' @examples
#' x <- as.integer64(sample(c(rep(NA, 9), 1:9), 32, TRUE))
#' unipos(x)
#' unipos(x, order="values")
#'
#' @keywords manip logic
#' @export
unipos = function(x, incomparables = FALSE, order = c("original", "values", "any"), ...) UseMethod("unipos")

#' @rdname unipos
#' @export
unipos.integer64 = function(x,
                             incomparables=FALSE,
                             order=c("original", "values", "any"),
                             nunique=NULL,
                             method=NULL,
                             ...) {
  stopifnot(identical(incomparables, FALSE))
  order = match.arg(order)
  cache_env = cache(x)
  keep.order = order == "original"
  if (is.null(nunique) && !is.null(cache_env))
    nunique = cache_env$nunique
  if (is.null(method)) {
    if (is.null(cache_env)) {
      if (order=="values")
        method <- "sortorderupo"
      else
        method <- "hashmapupo"
    } else {
      switch(order,
        original = {
          if (!is.null(cache_env$hashmap))
            method <- "hashupo"
          else if (!is.null(cache_env$order)) {
            if (!is.null(cache_env$sort))
              method <- "sortorderupo"
            else
              method <- "orderupo"
          } else {
            method <- "hashmapupo"
          }
        },
        values = {
          if (!is.null(cache_env$order)) {
            if (!is.null(cache_env$sort))
              method <- "sortorderupo"
            else
              method <- "orderupo"
          } else if (!is.null(cache_env$hashmap) && cache_env$nunique<length(x)/2L) {
            method <- "hashupo"
          } else {
            method <- "sortorderupo"
          }
        },
        any = {
          if (!is.null(cache_env$sort) && !is.null(cache_env$order))
            method <- "sortorderupo"
          else if (!is.null(cache_env$hashmap))
            method <- "hashupo"
          else if (!is.null(cache_env$order))
            method <- "orderupo"
          else
            method <- "sortorderupo"
        }
      )
    }
  }
  method = match.arg(method, c("hashmapupo", "hashupo", "sortorderupo", "orderupo"))
  switch(method,
    hashmapupo={
      p = hashmapupo(x, nunique=nunique)
    },
    hashupo={
      if (is.null(cache_env) || is.null(cache_env$hashmap))
        h <- hashmap(x, nunique=nunique)
      else
        h <- cache_env
      p = hashupo(h, keep.order=keep.order)
      if (order == "values") {
        s <- x[p]
        ramsortorder(s, p, na.last=FALSE)
      }
    },
    sortorderupo={
      if (is.null(cache_env) || is.null(cache_env$sort) || is.null(cache_env$order)) {
        s <- clone(x)
        o <- seq_along(x)
        ramsortorder(s, o, na.last=FALSE)
      } else {
        s <- get("sort", cache_env, inherits=FALSE)
        o <- get("order", cache_env, inherits=FALSE)
      }
      if (is.null(nunique))
        nunique <- sortnut(s)[1L]
      p = sortorderupo(s, o, nunique, keep.order=keep.order)
    },
    orderupo={
      if (is.null(cache_env) || is.null(cache_env$order)) {
        o <- seq_along(x)
        ramorder(x, o, na.last=FALSE)
      } else {
        o <- get("order", cache_env, inherits=FALSE)
      }
      if (is.null(nunique))
        nunique <- ordernut(x, o)[1L]
      p = orderupo(x, o, nunique, keep.order=keep.order)
    }
  )
  p
}

#' Cross Tabulation and Table Creation for integer64
#'
#' `table.integer64` uses the cross-classifying integer64 vectors to build a
#'   contingency table of the counts at each combination of vector values.
#'   
#' @param ... one or more objects which can be interpreted as factors
#'   (including character strings), or a list (or data frame) whose
#'   components can be so interpreted.  (For `as.table` and `as.data.frame`,
#'   arguments passed to specific methods.)
#' @param exclude levels to remove for all factors in ....
#' @param useNA whether to include NA values in the table.
#' @param dnn the names to be given to the dimensions in the result
#'   (the _dimnames names_).
#' @param deparse.level controls how the default `dnn` is constructed. See Details.
#'
#' @details
#' If at least one argument of `...` is integer64 and the remaining arguments of `...` 
#' are integer64 or integer the `table.integer64` method is used. Only this method 
#' supports the arguments `return`, `order`, `nunique`, and `method`.
#' 
#' This function automatically chooses from several low-level functions considering
#'   the size of `x` and the availability of a cache.
#'
#' Suitable methods are
#'  - [`hashmaptab`] (simultaneously creating and using a hashmap)
#'  - [`hashtab`] (first creating a hashmap then using it)
#'  - [`sortordertab`] (fast ordering)
#'  - [`ordertab`] (memory saving ordering).
#'
#' If the argument `dnn` is not supplied, the internal function
#'   `list.names` is called to compute the 'dimname names'.  If the
#'   arguments in `...` are named, those names are used.  For the
#'   remaining arguments, `deparse.level = 0` gives an empty name,
#'   `deparse.level = 1` uses the supplied argument if it is a symbol,
#'   and `deparse.level = 2` will deparse the argument.
#'
#' @return By default (with `return="table"`) [table()] returns a
#'   _contingency table_, an object of class `"table"`, an array of integer values.
#'   Note that unlike S the result is always an array, a 1D array if one factor is
#'   given. Note also that for multidimensional arrays this is a _dense_ return
#'   structure which can dramatically increase RAM requirements (for large arrays
#'   with high mutual information, i.e. many possible input combinations of which
#'   only few occur) and that [table()] is limited to `2^31` possible combinations
#'   (e.g. two input vectors with 46340 unique values only). Finally note that the
#'   tabulated values or value-combinations are represented as `dimnames` and that
#'   the implied conversion of values to strings can cause _severe_ performance
#'   problems since each string needs to be integrated into R's global string cache.
#'
#' You can use the other `return=` options to cope with these problems, the potential
#'   combination limit is increased from `2^31` to `2^63` with these options, RAM is
#'   only required for observed combinations and string conversion is avoided.
#'
#' With `return="data.frame"` you get a _dense_ representation as a [data.frame()]
#'   (like that resulting from `as.data.frame(table(...))`) where only observed
#'   combinations are listed (each as a data.frame row) with the corresponding
#'   frequency counts (the latter as component named by `responseName`). This is
#'   the inverse of [xtabs()].
#'
#' With `return="list"` you also get a _dense_ representation as a simple
#'   [list()] with components
#'  - `values` a integer64 vector of the technically tabulated values, for 1D this
#'    is the tabulated values themselves, for kD these are the values representing
#'    the potential combinations of input values
#'  - `counts` the frequency counts
#'  - `dims` only for kD: a list with the vectors of the unique values of the
#'    input dimensions
#'
#' @note Note that by using [as.integer64.factor()] we can also input
#'   factors into `table.integer64` -- only the [levels()] get lost.
#'
#' @seealso [table()] for more info on the standard version coping with Base R's
#'   data types, [tabulate()] which can faster tabulate [`integer`]s with a limited
#'   range `[1L .. nL not too big]`, [unique.integer64()] for the unique values
#'   without counting them and [unipos.integer64()] for the positions of the unique values.
#'
#' @examples
#' message("pure integer64 examples")
#' x <- as.integer64(sample(c(rep(NA, 9), 1:9), 32, TRUE))
#' y <- as.integer64(sample(c(rep(NA, 9), 1:9), 32, TRUE))
#' z <- sample(c(rep(NA, 9), letters), 32, TRUE)
#' table(x)
#' table(x, order="counts")
#' table(x, y)
#' table(x, y, return="data.frame")
#'
#' message("via as.integer64.factor we can use 'table.integer64' also for factors")
#' table(x, as.integer64(as.factor(z)))
#' @keywords category
#' @concept counts
#' @concept frequencies
#' @concept occurrences
#' @concept contingency table
#' @export
table = function(..., exclude=if (useNA == "no") c(NA, NaN), useNA=c("no", "ifany", "always"), dnn=list.names(...), deparse.level=1L) {
  # assure order of evaluation to match base::table()
  if (!missing(useNA) && !missing(exclude)) {
    force(useNA)
  } else if (!missing(exclude)) {
    force(exclude)
  } else if (!missing(useNA)) {
    force(useNA)
  }
  dots = list(...)
  if (!missing(useNA) && !missing(exclude)) force(exclude) # force after '...'

  if (is.null(names(dots)))
    sel = rep(TRUE, length(dots))
  else
    sel = !names(dots) %in% c("return", "order", "nunique", "method")
  is_int64 = vapply(dots[sel], is.integer64, logical(1L), USE.NAMES=FALSE)
  is_int = vapply(dots[sel], is.integer, logical(1L), USE.NAMES=FALSE)
  # TODO(#236): avoid this workaround to hack S3 dispatch. For now,
  #   we only use table.integer64() when we are sure there is no information loss (coercion).
  sys_call = match.call()
  sel = which(vapply(sys_call[seq_along(dots) + 1L], is.symbol, FALSE)) + 1L
  if (length(sel)) {
    n = names(sys_call)
    s = vapply(sys_call[sel], as.character, "")
    if (is.null(n)) {
      n = rep("", length(sys_call))
      n[sel] = s
    } else {
      idx = sel[n[sel] == ""]
      n[idx] = s[n[sel] == ""]
    }
    names(sys_call) = n
  }
  for (ii in which(vapply(dots, Negate(is.null), logical(1L))))
    sys_call[[ii + 1L]] = dots[[ii]]
  if (!missing(useNA)) sys_call$useNA = useNA
  if (!missing(exclude)) sys_call$exclude = exclude
  pf = parent.frame()
  # add unused function `list.names` to eliminate CMD check NOTE about missing function definition.
  list.names = function(...) {}
  if (length(dots) && any(is_int64) && all(is_int64 | is_int)) {
    sys_call[[1L]] = table.integer64
    withCallingHandlers_and_choose_call(eval(sys_call, envir=pf), c("table", "table.default"), "table.integer64")
  } else {
    sys_call[[1L]] = base::table
    withCallingHandlers_and_choose_call(eval(sys_call, envir=pf), c("table", "table.default"))
  }
}
#' @exportS3Method table default
table.default = function(..., exclude=if (useNA == "no") c(NA, NaN), useNA=c("no", "ifany", "always"), dnn=list.names(...), deparse.level=1L) {
  # avoid condition messages with `table.default`
  sys_call = sys.call()
  sys_call[[1L]] = base::table
  pf = parent.frame()
  # add unused function `list.names` to eliminate CMD check NOTE about missing function definition.
  list.names = function(...) {}
  withCallingHandlers_and_choose_call(eval(sys_call, envir=pf), c("table", "table.default"))
}

#' @method table integer64
#' @param return choose the return format, see details
#' @param order By default results are created sorted by "values", or by "counts"
#' @param nunique NULL or the number of unique values of table (including NA).
#'   Providing `nunique` can speed-up matching when `table` has no cache. Note
#'   that a wrong `nunique` can cause undefined behaviour up to a crash.
#' @param method NULL for automatic method selection or a suitable low-level
#'   method, see details
#' @aliases table.integer64
#' @exportS3Method table integer64
#' @rdname table
table.integer64 = function(...,
                            return = c("table", "data.frame", "list"),
                            order = c("values", "counts"),
                            nunique = NULL,
                            method = NULL,
                            exclude = if (useNA == "no") c(NA, NaN),
                            useNA = c("no", "ifany", "always"),
                            dnn = list.names(...),
                            deparse.level = 1L) {

  order = match.arg(order)
  return = match.arg(return)
  # this is taken from 'table'
  list.names = function(...) {
    l = as.list(substitute(list(...)))[-1L]
    nm = names(l)
    fixup = if (is.null(nm))
      seq_along(l)
    else nm == ""
    dep = vapply(
      l[fixup],
      function(x) switch(deparse.level + 1L, "", if (is.symbol(x)) as.character(x) else "", deparse(x, nlines=1L)[1L]),
      ""
    )
    if (is.null(nm)) {
      dep
    } else {
      nm[fixup] = dep
      nm
    }
  }

  # COPY ON MODIFY is broken for reading from list(...)
  # because list(...) creates a copy of all ... and this invalidates our caches
  # therefore we go this sick workaround
  argsymbols = as.list(substitute(list(...)))[-1L]
  argframe = parent.frame()
  A = function(i) eval(argsymbols[[i]], argframe)
  N = length(argsymbols)
  if (!N)
    stop("nothing to tabulate", domain="R-base")

  # table(as.integer64(1L), "a") is dispatched to table.integer64, but should be handled by table.default
  if (!all(vapply(seq_len(N), function(ii) {el = A(ii); is.integer64(el) || is.integer(el)}, logical(1L))))
    return(NextMethod())
  
  if (N == 1L && is.list(A(1L))) {
    args = A(1L) # nolint: object_overwrite_linter. This code should probably be refactored anyway.
    if (length(dnn) != length(args))
      # TODO(R>=4.4.0): names(args) %||% paste(dnn[1L], seq_along(args), sep=".")
      dnn = if (!is.null(argn <- names(args))) argn else paste(dnn[1L], seq_along(args), sep=".")
    N = length(args)
    A = function(i) args[[i]]
  }
  force(dnn)

  if (N == 1L) {
    x = A(1L)
    if (!is.integer64(x)) {
      if (!is.integer(x))
        warning("coercing first argument to integer64")
      x = as.integer64(x)
    }
  } else {
    a = A(1L)
    n = length(a)
    nu = integer(N)
    d = integer64(N + 1L)
    d[[1L]] = 1L
    dims = vector("list", N)
    names(dims) = dnn
    for (i in seq_len(N)) {
      a = A(i)
      if (length(a) != n)
        stop("all arguments must have the same length", domain="R-base")
      if (!is.integer64(a)) {
        if (!is.integer(a))
          warning("coercing argument ", i, " to integer64")
        a = as.integer64(a)
      }
      cache_env = cache(a)
      if (is.null(cache_env$order)) {
        s = clone(a)
        o = seq_along(s)
        ramsortorder(s, o, na.last=TRUE)
        nu[[i]] = sortnut(s)[["nunique"]]
      } else if (is.null(cache_env$sort)) {
        o = cache_env$order
        s = a[o]
        nu[[i]] = cache_env$nunique
      } else {
        o = cache_env$order
        s = cache_env$sort
        nu[[i]] = cache_env$nunique
      }
      d[[i+1L]] = d[[i]]*nu[[i]]
      if (is.na(d[[i + 1L]]))
        stop("attempt to make a table from more than >= 2^63 hypothetical combinations")
      dims[[i]] = sortuni(s, nu[[i]])
      if (i == 1L)
        x = sortorderkey(s, o) - 1L
      else
        x = x + d[[i]]*(sortorderkey(s, o) - 1L)
    }
  }
  cache_env = cache(x)
  if (is.null(nunique) && !is.null(cache_env))
    nunique = cache_env$nunique
  if (is.null(method)) {
    if (is.null(cache_env)) {
      if (order == "values" && (is.null(nunique) || nunique > 65536L))
        method = "sorttab"
      else
        method = "hashmaptab"
    } else {
      # nolint next: unnecessary_nesting_linter. Good parallelism.
      if (order=="values") {
        if (!is.null(cache_env$sort))
          method = "sorttab"
        else if (!is.null(cache_env$hashmap) && cache_env$nunique < sqrt(length(x)))
          method = "hashtab"
        else if (!is.null(cache_env$order))
          method = "ordertab"
        else
          method = "sorttab"
      } else { # order = "counts"
        # nolint next: unnecessary_nesting_linter. Good parallelism.
        if (!is.null(cache_env$hashmap))
          method = "hashtab"
        else if (!is.null(cache_env$sort))
          method = "sorttab"
        else if (!is.null(cache_env$order))
          method = "ordertab"
        else
          method = "hashmaptab"
      }
    }
  }
  method = match.arg(method, c("hashmaptab", "hashtab", "sorttab", "ordertab"))
  switch(method,
    hashmaptab={
      tmp = hashmaptab(x, nunique=nunique)
      cnt = tmp$counts
      val = tmp$values
      rm(tmp)
    },
    hashtab={
      if (is.null(cache_env) || is.null(cache_env$hashmap))
        h = hashmap(x, nunique=nunique)
      else
        h = cache_env
      tmp = hashtab(h, keep.order=FALSE)
      cnt = tmp$counts
      val = tmp$values
      rm(tmp)
    },
    sorttab={
      if (is.null(cache_env) || is.null(cache_env$sort)) {
        s = clone(x)
        ramsort(s, na.last=TRUE)
      } else {
        s = get("sort", cache_env, inherits=FALSE)
      }
      if (is.null(nunique))
        nunique = sortnut(s)[1L]
      val = sortuni(s, nunique)
      cnt = sorttab(s, nunique)
    },
    ordertab={
      if (is.null(cache_env) || is.null(cache_env$order)) {
        o = seq_along(x)
        ramorder(x, o, na.last=TRUE)
      } else {
        o = get("order", cache_env, inherits=FALSE)
      }
      if (is.null(nunique))
        nunique = ordernut(x, o)[1L]
      val = orderuni(x, o, nunique, keep.order=FALSE)
      cnt = ordertab(x, o, nunique, keep.order=FALSE)
      rm(o)
    }
  )
  if (order == "values") {
    if (startsWith(method, "hash")) {
      o = seq_along(val)
      ramsortorder(val, o, na.last=TRUE)
      cnt = cnt[o]
    }
  } else {
    # xx workaround until we have implemented ramsort.integer
    o = sort.list(cnt, na.last=NA, method="quick")
    cnt = cnt[o]
    # o = seq_along(cnt)
    # ramsortorder(cnt, o, na.last=FALSE)
    val = val[o]
  }

  ## attaching names is extremely expensive with many unique values, doing this only for compatibility with 'table' here
  switch(return,
    table = {
      if (N == 1L) {
        attr(cnt, "dim") = length(cnt)
        dn = list(as.character(val))
        names(dn) = dnn[1L]
        attr(cnt, "dimnames") = dn
      } else {
        a = array(0L, dim=nu, dimnames=lapply(dims, as.character))
        a[as.integer(val) + 1L] = as.integer(cnt)
        cnt = a
      }
      useNA = match.arg(useNA)
      if (length(exclude) > 0L) {
        sel = suppressWarnings(lapply(dimnames(cnt), Negate(`%in%`), table=exclude))
        cnt = do.call(`[`, c(list(x=cnt), sel, list(drop=FALSE)))
      }
      if (useNA == "always") {
        dimnames_new = lapply(dimnames(cnt), function(el) c(el, if (!anyNA(el)) NA))
        cnt_new = array(0L, as.integer(vapply(dimnames_new, length, 0L)))
        dimnames(cnt_new) = dimnames_new
        cnt = do.call(`[<-`, c(list(x=cnt_new), lapply(dim(cnt), seq_len), list(value=cnt)))
      }
      oldClass(cnt) = "table"
    },
    data.frame = {
      if (N == 1L) {
        cnt = data.frame(values=val, Freq=cnt)
        names(cnt)[[1L]] = dnn[1L]
      } else {
        for (i in N:1) {
          w = val %/% d[[i]]
          val = val - d[[i]]*w
          dims[[i]] = dims[[i]][as.integer(w) + 1L]
        }
        cnt = data.frame(dims, Freq=cnt)
      }
    },
    list = {
      if (N == 1L)
        cnt = list(values=val, counts=cnt)
      else
        cnt = list(values=val, counts=cnt, dims=dims)
    }
  )
  cnt
}

as.integer64.factor = function(x, ...) as.integer64(unclass(x))

#' Extract Positions in redundant dimension table
#'
#' `keypos` returns the positions of the (fact table) elements that participate
#'   in their sorted unique subset (dimension table)
#'
#' @param x a vector or a data frame or an array or `NULL`.
#' @param method NULL for automatic method selection or a suitable low-level
#'   method, see details
#' @param ... ignored
#'
#' @details
#' NAs are sorted first in the dimension table, see [ramorder.integer64()].
#'
#' This function automatically chooses from several low-level functions
#'   considering the size of `x` and the availability of a cache.
#'
#' Suitable methods are
#'  - [`sortorderkey`] (fast ordering)
#'  - [`orderkey`] (memory saving ordering).
#'
#' @return an integer vector of the same length as `x` containing positions
#'   relative to `sort(unique(x), na.last=FALSE)`
#' @seealso [unique.integer64()] for the unique subset and [match.integer64()]
#'   for finding positions in a different vector.
#'
#' @examples
#' x <- as.integer64(sample(c(rep(NA, 9), 1:9), 32, TRUE))
#' keypos(x)
#'
#' @keywords manip univar
#' @export
keypos = function(x, ...) UseMethod("keypos")

#' @rdname keypos
#' @export
keypos.integer64 = function(x, method = NULL, ...) {
  cache_env = cache(x)
  if (is.null(method)) {
    if (is.null(cache_env)) {
      method = "sortorderkey"
    } else if (!is.null(cache_env$order)) {
      if (!is.null(cache_env$sort))
        method <- "sortorderkey"
      else
        method <- "orderkey"
    } else {
      method = "sortorderkey"
    }
  }
  method = match.arg(method, c("sortorderkey", "orderkey"))
  switch(method,
    sortorderkey={
      if (is.null(cache_env) || is.null(cache_env$sort) || is.null(cache_env$order)) {
        s <- clone(x)
        o <- seq_along(x)
        ramsortorder(s, o, na.last=FALSE)
      } else {
        s <- get("sort", cache_env, inherits=FALSE)
        o <- get("order", cache_env, inherits=FALSE)
      }
      p = sortorderkey(s, o)
    },
    orderkey={
      if (is.null(cache_env) || is.null(cache_env$order)) {
        o <- seq_along(x)
        ramorder(x, o, na.last=FALSE)
      } else {
        o <- get("order", cache_env, inherits=FALSE)
      }
      p = orderkey(x, o)
    }
  )
  p
}

#' Extract Positions of Tied Elements
#'
#' `tiepos` returns the positions of those elements that participate in ties.
#'
#' @param x a vector or a data frame or an array or `NULL`.
#' @param nties NULL or the number of tied values (including NA). Providing
#'   `nties` can speed-up when `x` has no cache. Note that a wrong nties can
#'   cause undefined behaviour up to a crash.
#' @param method NULL for automatic method selection or a suitable low-level
#'   method, see details
#' @param ... ignored
#'
#' @details
#' This function automatically chooses from several low-level functions
#'   considering the size of `x` and the availability of a cache.
#'
#' Suitable methods are
#'  - [`sortordertie`] (fast ordering)
#'  - [`ordertie`] (memory saving ordering).
#'
#' @return an integer vector of positions
#' @seealso [rank.integer64()] for possibly tied ranks and [unipos.integer64()]
#'   for positions of unique values.
#'
#' @examples
#' x <- as.integer64(sample(c(rep(NA, 9), 1:9), 32, TRUE))
#' tiepos(x)
#'
#' stopifnot(identical(tiepos(x),  (1:length(x))[duplicated(x) | rev(duplicated(rev(x)))]))
#' @keywords manip univar
#' @export
tiepos = function(x, ...) UseMethod("tiepos")

#' @rdname tiepos
#' @export
tiepos.integer64 = function(x, nties = NULL, method = NULL, ...) {
  cache_env = cache(x)
  if (is.null(nties) && !is.null(cache_env))
    nties = cache_env$nties
  if (is.null(method)) {
    if (is.null(cache_env)) {
      method = "sortordertie"
    } else if (!is.null(cache_env$order)) {
      if (!is.null(cache_env$sort))
        method <- "sortordertie"
      else
        method <- "ordertie"
    } else {
      method = "sortordertie"
    }
  }
  method = match.arg(method, c("sortordertie", "ordertie"))
  switch(method,
    sortordertie={
      if (is.null(cache_env) || is.null(cache_env$sort) || is.null(cache_env$order)) {
        s <- clone(x)
        o <- seq_along(x)
        ramsortorder(s, o, na.last=FALSE)
      } else {
        s <- get("sort", cache_env, inherits=FALSE)
        o <- get("order", cache_env, inherits=FALSE)
      }
      if (is.null(nties))
        nties <- sortnut(s)[2L]
      p = sortordertie(s, o, nties)
    },
    ordertie={
      if (is.null(cache_env) || is.null(cache_env$order)) {
        o <- seq_along(x)
        ramorder(x, o, na.last=FALSE)
      } else {
        o <- get("order", cache_env, inherits=FALSE)
      }
      if (is.null(nties))
        nties <- ordernut(x, o)[2L]
      p = ordertie(x, o, nties)
    }
  )
  p
}

#' Sample Ranks from integer64
#'
#' Returns the sample ranks of the values in a vector.  Ties (i.e., equal
#'   values) are averaged and missing values propagated.
#'
#' @param x a integer64 vector
#' @param method NULL for automatic method selection or a suitable low-level
#'   method, see details
#' @param ... ignored
#'
#' @details
#' This function automatically chooses from several low-level functions
#'   considering the size of `x` and the availability of a cache.
#' Suitable methods are
#'  - [sortorderrnk()] (fast ordering)
#'  - [orderrnk()] (memory saving ordering).
#'
#' @return A numeric vector of the same length as `x`.
#' @seealso [order.integer64()], [rank()] and [prank()] for percent rank.
#' @examples
#' x <- as.integer64(sample(c(rep(NA, 9), 1:9), 32, TRUE))
#' rank.integer64(x)
#'
#' stopifnot(identical(rank.integer64(x),  rank(as.integer(x)
#' , na.last="keep", ties.method = "average")))
#'
#' @keywords univar
#' @export
rank.integer64 = function(x, method = NULL, ...) {
  cache_env = cache(x)
  if (is.null(method)) {
    if (is.null(cache_env)) {
      method = "sortorderrnk"
    } else if (!is.null(cache_env$order)) {
      if (!is.null(cache_env$sort))
        method <- "sortorderrnk"
      else
        method <- "orderrnk"
    } else {
      method = "sortorderrnk"
    }
  }
  method = match.arg(method, c("sortorderrnk", "orderrnk"))
  switch(method,
    sortorderrnk={
      if (is.null(cache_env) || is.null(cache_env$sort) || is.null(cache_env$order)) {
        s <- clone(x)
        o <- seq_along(x)
        na.count <- ramsortorder(s, o, na.last=FALSE)
      } else {
        s <- get("sort", cache_env, inherits=FALSE)
        o <- get("order", cache_env, inherits=FALSE)
        na.count <- get("na.count", cache_env, inherits=FALSE)
      }
      p = sortorderrnk(s, o, na.count)
    },
    orderrnk={
      if (is.null(cache_env) || is.null(cache_env$order)) {
        o <- seq_along(x)
        na.count <- ramorder(x, o, na.last=FALSE)
      } else {
        o <- get("order", cache_env, inherits=FALSE)
        na.count <- get("na.count", cache_env, inherits=FALSE)
      }
      p = orderrnk(x, o, na.count)
    }
  )
  p
}

#' (P)ercent (Rank)s
#'
#' Function `prank.integer64`  projects the values `[min..max]` via ranks
#'   `[1..n]` to `[0..1]`.
#' [qtile.integer64()] is the inverse function of 'prank.integer64' and
#'   projects `[0..1]` to `[min..max]`.
#'
#' @param x a integer64 vector
#' @param method NULL for automatic method selection or a suitable low-level
#'   method, see details
#' @param ... ignored
#'
#' @details Function `prank.integer64` is based on [rank.integer64()].
#' @return `prank` returns a numeric vector of the same length as `x`.
#' @seealso [rank.integer64()] for simple ranks and [qtile()] for the
#'   inverse function quantiles.
#' @examples
#' x <- as.integer64(sample(c(rep(NA, 9), 1:9), 32, TRUE))
#' prank(x)
#'
#' x <- x[!is.na(x)]
#' stopifnot(identical(x,  unname(qtile(x, probs=prank(x)))))
#' @keywords univar
#' @export
prank = function(x, ...) UseMethod("prank")
#' @rdname prank
#' @export
prank.integer64 = function(x, method = NULL, ...) {
  n = nvalid(x)
  if (n<2L) return(rep(as.integer64(NA), length(x)))
  (rank.integer64(x, method=method, ...)-1L) / (n-1L)
}

#' (Q)uan(Tile)s
#'
#' Function [prank.integer64()]  projects the values `[min..max]` via ranks
#'   `[1..n]` to `[0..1]`.
#'
#' `qtile.integer64` is the inverse function of 'prank.integer64' and projects
#'   `[0..1]` to `[min..max]`.
#'
#' @param x a integer64 vector
#' @param probs numeric vector of probabilities with values in `[0, 1]` - possibly containing `NA`s
#' @param ... ignored
#'
#' @details
#'
#' Functions `quantile.integer64` with `type=0` and `median.integer64` are
#'   convenience wrappers to `qtile`.
#'
#' Function `qtile` behaves very similar to `quantile.default` with `type=1`
#'   in that it only returns existing values, it is mostly symmetric but it is
#'   using 'round' rather than 'floor'.
#'
#' Note that this implies that `median.integer64` does not interpolate for even
#'   number of values (interpolation would create values that could not be
#'   represented as 64-bit integers).
#'
#' This function automatically chooses from several low-level functions
#'   considering the size of `x` and the availability of a cache.
#'
#' Suitable methods are
#'  - [`sortqtl`] (fast sorting)
#'  - [`orderqtl`] (memory saving ordering).
#'
#' @return
#' `prank` returns a numeric vector of the same length as `x`.
#'
#' `qtile` returns a vector with elements from `x`
#'   at the relative positions specified by `probs`.
#' @seealso [rank.integer64()] for simple ranks and [quantile()] for quantiles.
#' @examples
#' x <- as.integer64(sample(c(rep(NA, 9), 1:9), 32, TRUE))
#' qtile(x, probs=seq(0, 1, 0.25))
#' quantile(x, probs=seq(0, 1, 0.25), na.rm=TRUE)
#' median(x, na.rm=TRUE)
#' summary(x)
#'
#' x <- x[!is.na(x)]
#' stopifnot(identical(x,  unname(qtile(x, probs=prank(x)))))
#' @keywords univar
#' @export
qtile = function(x, probs = seq(0.0, 1.0, 0.25), ...) UseMethod("qtile")

#' @rdname qtile
#' @param names logical; if `TRUE`, the result has a `names` attribute. Set to `FALSE` for speedup with many probs.
#' @param method NULL for automatic method selection or a suitable low-level method, see details
#' @export
qtile.integer64 = function(x, probs = seq(0.0, 1.0, 0.25), names = TRUE, method = NULL, ...) {
  if (any(is.na(probs) | probs<0.0 | probs>1.0))
    stop("p outside [0, 1]")
  cache_env = cache(x)
  if (is.null(method)) {
    if (is.null(cache_env))
      method = "sortqtl"
    else if (!is.null(cache_env$sort))
      method = "sortqtl"
    else if (!is.null(cache_env$order))
      method = "orderqtl"
    else
      method = "sortqtl"
  }
  method = match.arg(method, c("sortqtl", "orderqtl"))
  switch(method,
    sortqtl={
      if (is.null(cache_env) || is.null(cache_env$sort)) {
        s <- clone(x)
        na.count <- ramsort(s, na.last=FALSE)
      } else {
        s <- get("sort", cache_env, inherits=FALSE)
        na.count <- get("na.count", cache_env, inherits=FALSE)
      }
      qs = sortqtl(s, na.count, probs)
    },
    orderqtl={
      if (is.null(cache_env) || is.null(cache_env$order)) {
        o <- seq_along(x)
        na.count <- ramorder(x, o, na.last=FALSE)
      } else {
        o <- get("order", cache_env, inherits=FALSE)
        na.count <- get("na.count", cache_env, inherits=FALSE)
      }
      qs = orderqtl(x, o, na.count, probs)
    }
  )
  if (names) {
    np = length(probs)
    dig = max(2L, getOption("digits"))
    names(qs) <- paste0(
      if (np < 100L)
        formatC(100.0 * probs, format = "fg", width = 1L, digits = dig)
      else
        format(100.0 * probs, trim = TRUE, digits = dig),
      "%"
    )
  }
  qs
}

#' @rdname qtile
#' @param type an integer selecting the quantile algorithm, currently only
#'   0 is supported, see details
#' @param na.rm logical; if `TRUE`, any `NA` and `NaN`'s are removed from
#'   `x` before the quantiles are computed.
#' @export
quantile.integer64 = function(x, probs = seq(0.0, 1.0, 0.25), na.rm = FALSE, names = TRUE, type=0L, ...) {
  if (type[[1L]]!=0L)
    stop("only type==0 ('qtile') supported")
  if (!na.rm && na.count(x)>0L)
    stop("missing values not allowed with 'na.rm='==FALSE")
  qtile(x, probs = probs, na.rm = na.rm, names = names, ...)
}

#' @rdname qtile
#' @export
median.integer64 = function(x, na.rm=FALSE, ...) {
  if (!na.rm && na.count(x)>0L) return(NA_integer64_)
  if (!length(x)) return(NA_integer64_)
  qtile(x, probs = 0.5, na.rm = na.rm, names = FALSE)
}

#' @rdname qtile
#' @export
mean.integer64 = function(x, na.rm=FALSE, ...) {
    ret = .Call(C_mean_integer64, x, as.logical(na.rm), double(1L))
    oldClass(ret) <- "integer64"
    ret
}

#' @rdname qtile
#' @param object a integer64 vector
#' @export
summary.integer64 = function(object, ...) {
    nas = na.count(object)
    qq = quantile(object, na.rm=TRUE)
    qq = c(qq[1L:3L], mean(object, na.rm=TRUE), qq[4L:5L])
    names(qq) <- c("Min.", "1st Qu.", "Median", "Mean", "3rd Qu.", "Max.")
    if (any(nas))
        c(qq, "NA's" = nas)
    else qq
}

test_that("match & %in% basics work", {
  x = as.integer64(2:5)
  y = as.integer64(3:6)
  expect_identical(match(x, y), c(NA, 1:3))
  expect_identical(match(y, x), c(2:4, NA))

  expect_identical(match(2:5, y), c(NA, 1:3))
  expect_identical(match(as.numeric(2:5), y), c(NA, 1:3))
  expect_identical(match(y, 2:5), c(2:4, NA))
  expect_identical(match(y, as.numeric(2:5)), c(2:4, NA))

  expect_identical(match(x, y, nomatch=0L), 0:3)

  expect_identical(x %in% y, c(FALSE, TRUE, TRUE, TRUE))
  expect_identical(y %in% x, c(TRUE, TRUE, TRUE, FALSE))
  expect_identical(x %in% 3:6, c(FALSE, TRUE, TRUE, TRUE))
  expect_identical(x %in% c(3.0, 4.0, 5.0, 6.0), c(FALSE, TRUE, TRUE, TRUE))
})

test_that("Different method= for match() and %in% work", {
  x = as.integer64(2:5)
  y = as.integer64(3:6)
  expected = c(NA_integer_, 1:3)

  expect_identical(match(x, y, method="hashpos"), expected)
  expect_identical(match(x, y, method="hashrev"), expected)
  expect_identical(match(x, y, method="sortorderpos"), expected)
  expect_error(match(x, y, method="_unknown_"), "unknown method _unknown_", fixed=TRUE)
  # TODO(#58): Fix this, currently fails.
  # expect_identical(match(x, y, method="orderpos"), expected)

  # NB: %in% is quite a bit different; while there's a public API to
  #   `%in%.integer64`, likely, there shouldn't be (it's strange to export
  #   an S3 method like is currently done). The tests are designed to tickle
  #   the different methods through the public API only; this makes them
  #   prone to winding up testing something totally different later. I think
  #   that's fine; now that we have coverage tests up, any refactor that bumps
  #   around what exactly the following tests are covering, will show up in the PR.

  # method="hashrin" used when x is "short" but table is "long"
  x = as.integer64(seq_len(10L))
  table = as.integer64(seq_len(2.0**16.0 * 2.0/3.0 + 10.0)) # invert condition for bx>=16, 10.0 arbitrary buffer
  expect_identical(x %in% table, rep(TRUE, 10L))
})

# TODO(#59): Don't call table.integer64() directly.
test_that("duplicated, unique, table methods work", {
  x = as.integer64(1:3)
  expect_identical(duplicated(x), rep(FALSE, 3L))
  expect_identical(unique(x), x)
  expect_identical(table.integer64(x), table(x = 1:3))

  x = as.integer64(rep(1L, 3L))
  expect_identical(duplicated(x), c(FALSE, TRUE, TRUE))
  expect_identical(unique(x), x[1L])
  expect_identical(table.integer64(x), table(x = rep(1L, 3L)))

  x = as.integer64(c(1L, 2L, 1L))
  expect_identical(duplicated(x), c(FALSE, FALSE, TRUE))
  expect_identical(unique(x), x[1:2])
  expect_identical(table.integer64(x), table(x = c(1L, 2L, 1L)))

  x = as.integer64(c(1L, 1L, 2L))
  expect_identical(duplicated(x), c(FALSE, TRUE, FALSE))
  expect_identical(unique(x), x[c(1L, 3L)])
  expect_identical(table.integer64(x), table(x = c(1L, 1L, 2L)))

  expect_error(duplicated(x, method="_unknown_"), "unknown method _unknown_", fixed=TRUE)
  expect_error(unique(x, method="_unknown_"), "unknown method _unknown_", fixed=TRUE)
})

test_that("different method= for duplicated, unique work", {
  x = as.integer64(c(1L, 2L, 1L))
  exp_dup = c(FALSE, FALSE, TRUE)
  exp_unq = x[1:2]

  expect_identical(duplicated(x, method="hashdup"), exp_dup)
  expect_identical(unique(x, method="hashmapuni"), exp_unq)
  expect_identical(unique(x, method="hashuni"), exp_unq)

  expect_identical(duplicated(x, method="sortorderdup"), exp_dup)
  expect_identical(unique(x, method="sortorderuni"), exp_unq)
  expect_identical(unique(x, method="sortuni"), exp_unq)

  # TODO(#58): Fix this, currently fails.
  # expect_identical(duplicated(x, method="orderdup"), exp_dup)
  expect_identical(unique(x, method="orderuni"), exp_unq)
})

test_that("more coercion works", {
  expect_identical(as.factor(as.integer64(2:4)), factor(2:4))
  expect_identical(as.ordered(as.integer64(2:4)), as.ordered(2:4))
  expect_identical(as.integer64(factor(2:11)), as.integer64(1:10)) # NB: _not_ 2:11!
})

test_that("sorting methods work", {
  x = as.integer64(c(10L, 4L, 8L))
  x_rank = c(3.0, 1.0, 2.0)
  expect_identical(rank(x), x_rank)
  expect_identical(rank(x, method="orderrnk"), x_rank)

  x = as.integer64(1:100)
  q = as.integer64(c(1L, 26L, 50L, 75L, 100L))
  expect_identical(quantile(x, names=FALSE), q)
  expect_identical(median(x), q[3L])
  names(q) = c('0%', '25%', '50%', '75%', '100%')
  expect_identical(quantile(x), q)
  expect_identical(quantile(x, 0.2, names=FALSE), as.integer64(21L))

  expect_error(quantile(x, type=7L), "only.*qtile.*supported")
  expect_error(median(NA_integer64_), "missing values not allowed")
  expect_error(quantile(NA_integer64_), "missing values not allowed")

  x = as.integer64(1:100)
  q = as.integer64(c(1L, 26L, 50L, 75L, 100L))
  names(q) = c('0%', '25%', '50%', '75%', '100%')
  expect_identical(qtile(x, method="sortqtl"), q)
  expect_identical(qtile(x, method="orderqtl"), q)

  x = as.integer64(c(1L, 1L, 2L, 3L, 2L, 4L))
  x_tiepos = c(1L, 2L, 3L, 5L)
  expect_identical(tiepos(x), x_tiepos)
  expect_identical(tiepos(x, method="ordertie"), x_tiepos)

  expect_error(rank(x, method="_unknown_"), "unknown method _unknown_", fixed=TRUE)
  expect_error(qtile(x, method="_unknown_"), "unknown method _unknown_", fixed=TRUE)
  expect_error(tiepos(x, method="_unknown_"), "unknown method _unknown_", fixed=TRUE)
})

# These tests were previously kept as tests under \examples{\dontshow{...}}.
#   Converted to "proper" unit tests for clarity, after making them more
#   canonical within {testthat}, e.g. better capturing expected warnings,
#   changing stopifnot(identical(...)) to expect_identical(...).
test_that("Old \\dontshow{} tests continue working", {
  xi = c(1L, 1L, 2L)
  xi64 = as.integer64(xi)
  yi = c(3L, 4L, 4L)
  yi64 = as.integer64(yi)

  t_xi = table(x=xi)
  t_xi_yi = table(x=xi, y=yi)

  expect_identical(table.integer64(x=xi64), t_xi)
  expect_identical(table.integer64(x=xi64, y=yi64), t_xi_yi)

  expect_warning(
    expect_identical(table.integer64(x=xi), t_xi),
    "coercing first argument to integer64",
    fixed = TRUE
  )
  expect_warning(
    expect_identical(table.integer64(x=xi64, y=yi), t_xi_yi),
    "coercing argument 2 to integer64",
    fixed = TRUE
  )
  expect_warning(
    expect_identical(table.integer64(x=xi, y=yi64), t_xi_yi),
    "coercing argument 1 to integer64",
    fixed = TRUE
  )

  expect_identical(table(x=xi64), t_xi)
  expect_identical(table(x=xi64, y=yi64), t_xi_yi)
  expect_identical(table(x=xi64, y=yi), t_xi_yi)
  expect_identical(table(x=xi, y=yi64), t_xi_yi)
})

test_that("unipos() works as intended", {
  x = as.integer64(c(1L, 2L, 1L, 3L, 2L, 4L))
  x_unipos = c(1L, 2L, 4L, 6L)
  expect_identical(unipos(x), x_unipos)
  expect_identical(unipos(x, method="hashupo"), x_unipos)
  expect_identical(unipos(x, method="sortorderupo"), x_unipos)
  expect_identical(unipos(x, method="orderupo"), x_unipos)
  expect_error(unipos(x, method="_unknown_"), "unknown method _unknown_", fixed=TRUE)
})

test_that("keypos() works as intended", {
  x = as.integer64(c(5L, 2L, 5L, 3L, 2L, 4L))
  x_keypos = c(4L, 1L, 4L, 2L, 1L, 3L)
  expect_identical(keypos(x), x_keypos)
  expect_identical(keypos(x, method="orderkey"), x_keypos)
  expect_error(keypos(x, method="_unknown_"), "unknown method _unknown_", fixed=TRUE)
})

test_that("summary() works as intended", {
  x = as.integer64(c(1L, 2L, 10L, 20L, NA, 30L))
  # NB: as.integer64() strips names, so as.integer64(c(Min. = ...)) won't work
  x_summary = as.integer64(c(1L, 2L, 10L, 12L, 20L, 30L, 1L))
  names(x_summary) = c("Min.", "1st Qu.", "Median", "Mean", "3rd Qu.", "Max.", "NA's")
  expect_identical(summary(x), x_summary)
  expect_identical(summary(x[-5L]), x_summary[-7L])
})

test_that("prank() works as intended", {
  x = as.integer64(1:100)
  expect_identical(prank(x), (x-1.0)/99.0)
  expect_identical(prank(x[1L]), NA_integer64_)
})

# /*
# R-Code for hashing
# S3 atomic 64bit integers for R
# (c) 2011 Jens Oehlschägel
# Licence: GPL2
# Provided 'as is', use at your own risk
# Created: 2011-12-11
# Last changed:  2011-12-11
# */


#' Hashing for 64bit integers
#'
#' This is an explicit implementation of hash functionality that underlies
#' matching and other functions in R. Explicit means that you can create,
#' store and use hash functionality directly. One advantage is that you can
#' re-use hashmaps, which avoid re-building hashmaps again and again.
#'
#' @param x an integer64 vector
#' @param minfac minimum factor by which the hasmap has more elements compared to the data `x`,
#'   ignored if `hashbits` is given directly
#' @param hashbits length of hashmap is `2^hashbits`
#' @param cache an optional [cache()] object into which to put the hashmap (by default a new cache is created
#' @param nunique giving _correct_ number of unique elements can help reducing the size of the hashmap
#' @param nomatch the value to be returned if an element is not found in the hashmap
#' @param keep.order determines order of results and speed: `FALSE` (the default) is faster and returns in the
#'   (pseudo)random order of the hash function, `TRUE` returns in the order of first appearance in the original
#'   data, but this requires extra work
#' @param ... further arguments, passed from generics, ignored in methods
#'
#' @details
# nolint start: line_length_linter.
#' | **function** | **see also**                            | **description** |
#' |-------------:|----------------------------------------:|:----------------|
#' |    `hashfun` |                                `digest` | export of the hash function used in `hashmap` |
#' |    `hashmap` |            [`match()`][match.integer64] | return hashmap  |
#' |    `hashpos` |            [`match()`][match.integer64] | return positions of `x` in `hashmap` |
#' |    `hashrev` |            [`match()`][match.integer64] | return positions of `hashmap` in `x` |
#' |    `hashfin` | \code{\link[=match.integer64]{\%in\%.integer64}} | return logical whether `x` is in `hashmap` |
#' |    `hashrin` | \code{\link[=match.integer64]{\%in\%.integer64}} | return logical whether `hashmap` is in `x` |
#' |    `hashdup` |  [`duplicated()`][duplicated.integer64] | return logical whether hashdat is duplicated using hashmap |
#' |    `hashuni` |          [`unique()`][unique.integer64] | return unique values of hashmap |
#' | `hashmapuni` |          [`unique()`][unique.integer64] | return unique values of `x` |
#' |    `hashupo` |          [`unique()`][unique.integer64] | return positions of unique values in hashdat |
#' | `hashmapupo` |          [`unique()`][unique.integer64] | return positions of unique values in `x` |
#' |    `hashtab` |            [`table()`][table] | tabulate values of hashdat using hashmap in `keep.order=FALSE` |
#' | `hashmaptab` |            [`table()`][table] | tabulate values of `x` building hasmap on the fly in `keep.order=FALSE` |
# nolint end: line_length_linter.
#'
#' @return See Details
#' @keywords programming manip
#' @seealso [`match()`][match.integer64], [runif64()]
#' @examples
#' x <- as.integer64(sample(c(NA, 0:9)))
#' y <- as.integer64(sample(c(NA, 1:9), 10, TRUE))
#' hashfun(y)
#' hx <- hashmap(x)
#' hy <- hashmap(y)
#' ls(hy)
#' hashpos(hy, x)
#' hashrev(hx, y)
#' hashfin(hy, x)
#' hashrin(hx, y)
#' hashdup(hy)
#' hashuni(hy)
#' hashuni(hy, keep.order=TRUE)
#' hashmapuni(y)
#' hashupo(hy)
#' hashupo(hy, keep.order=TRUE)
#' hashmapupo(y)
#' hashtab(hy)
#' hashmaptab(y)
#'
#'     \dontrun{
#'     message("explore speed given size of the hasmap in 2^hashbits and size of the data")
#'     message("more hashbits means more random access and less collisions")
#'     message("i.e. more data means less random access and more collisions")
#'     bits <- 24
#'     b <- seq(-1, 0, 0.1)
#'     tim <- matrix(NA, length(b), 2, dimnames=list(b, c("bits", "bits+1")))
#'     for (i in 1:length(b)) {
#'       n <- as.integer(2^(bits+b[i]))
#'       x <- as.integer64(sample(n))
#'       tim[i, 1] <- repeat.time(hashmap(x, hashbits=bits))[3]
#'       tim[i, 2] <- repeat.time(hashmap(x, hashbits=bits+1))[3]
#'       print(tim)
#'       matplot(b, tim)
#'     }
#'     message("we conclude that n*sqrt(2) is enough to avoid collisions")
#'     }
#' @name hashmap
NULL

#' @rdname hashmap
#' @export
hashfun <- function(x, ...) UseMethod("hashfun")
#' @rdname hashmap
#' @export
hashfun.integer64 <- function(x, minfac=1.41, hashbits=NULL, ...) {
  n <- length(x)
  if (is.null(hashbits)) {
    minlen <- ceiling(n*minfac)
    if (minlen > 0L)
      hashbits <- as.integer(ceiling(log2(minlen)))
    else
      hashbits <- 0L
  } else {
    hashbits <- as.integer(hashbits)
  }
  .Call(C_hashfun_integer64, x, hashbits, integer(n))
}

#' @rdname hashmap
#' @export
hashmap <- function(x, ...) UseMethod("hashmap")

#' @rdname hashmap
#' @export
hashmap.integer64 <- function(x, nunique=NULL, minfac=1.41, hashbits=NULL, cache=NULL, ...) {
  if (is.null(nunique)) {
    nunique <- integer(1L)
    n <- length(x)
  } else {
    nunique <- as.integer(nunique)
    n <- nunique
  }
  if (is.null(hashbits)) {
    minlen <- ceiling(n*minfac)
    if (minlen > 0L)
      hashbits <- as.integer(ceiling(log2(minlen)))
    else
      hashbits <- 0L
  } else {
    hashbits <- as.integer(hashbits)
  }
  nhash <- as.integer(2L^hashbits)
  hashmap <- integer(nhash)
  .Call(C_hashmap_integer64, x, hashbits, hashmap, nunique)

  if (is.null(cache))
      cache <- newcache(x)
  else
    if (!still.identical(x, get("x", envir=cache, inherits=FALSE)))
          stop("vector 'x' dissociated from cache")
  assign("hashmap", hashmap, envir=cache)
  assign("hashbits", hashbits, envir=cache)
  assign("nhash", nhash, envir=cache)
  assign("nunique", nunique, envir=cache)
  cache
}

#' @rdname hashmap
#' @export
hashpos <- function(cache, ...) UseMethod("hashpos")
#' @rdname hashmap
#' @export
hashpos.cache_integer64 <- function(cache, x, nomatch = NA_integer_, ...) {
  hashbits <- get("hashbits", envir=cache, inherits=FALSE)
  hashmap <- get("hashmap", envir=cache, inherits=FALSE)
  hashdat <- get("x", envir=cache, inherits=FALSE)
  .Call(C_hashpos_integer64, as.integer64(x), hashdat, hashbits, hashmap, as.integer(nomatch), integer(length(x)))
}

#' @rdname hashmap
#' @export
hashrev <- function(cache, ...) UseMethod("hashrev")
#' @rdname hashmap
#' @export
hashrev.cache_integer64 <- function(cache, x, nomatch = NA_integer_, ...) {
  hashbits <- get("hashbits", envir=cache, inherits=FALSE)
  hashmap <- get("hashmap", envir=cache, inherits=FALSE)
  hashdat <- get("x", envir=cache, inherits=FALSE)
  nunique <- get("nunique", envir=cache, inherits=FALSE)
  .Call(C_hashrev_integer64,
    as.integer64(x),
    hashdat, hashbits, hashmap, nunique,
    as.integer(nomatch),
    integer(length(hashdat))
  )
}

#' @rdname hashmap
#' @export
hashfin <- function(cache, ...) UseMethod("hashfin")
#' @rdname hashmap
#' @export
hashfin.cache_integer64 <- function(cache, x, ...) {
  hashbits <- get("hashbits", envir=cache, inherits=FALSE)
  hashmap <- get("hashmap", envir=cache, inherits=FALSE)
  hashdat <- get("x", envir=cache, inherits=FALSE)
  .Call(C_hashfin_integer64, as.integer64(x), hashdat, hashbits, hashmap, logical(length(x)))
}

#' @rdname hashmap
#' @export
hashrin <- function(cache, ...) UseMethod("hashrin")
#' @rdname hashmap
#' @export
hashrin.cache_integer64 <- function(cache, x, ...) {
  hashbits <- get("hashbits", envir=cache, inherits=FALSE)
  hashmap <- get("hashmap", envir=cache, inherits=FALSE)
  hashdat <- get("x", envir=cache, inherits=FALSE)
  .Call(C_hashrin_integer64, as.integer64(x), hashdat, hashbits, hashmap, nunique, logical(length(hashdat)))
}

#' @rdname hashmap
#' @export
hashdup <- function(cache, ...) UseMethod("hashdup")
#' @rdname hashmap
#' @export
hashdup.cache_integer64 <- function(cache, ...) {
  hashbits <- get("hashbits", envir=cache, inherits=FALSE)
  hashmap <- get("hashmap", envir=cache, inherits=FALSE)
  hashdat <- get("x", envir=cache, inherits=FALSE)
  nunique <- get("nunique", envir=cache, inherits=FALSE)
  .Call(C_hashdup_integer64, hashdat, hashbits, hashmap, nunique, logical(length(hashdat)))
}

#' @rdname hashmap
#' @export
hashuni <- function(cache, ...) UseMethod("hashuni")
#' @rdname hashmap
#' @export
hashuni.cache_integer64 <- function(cache, keep.order=FALSE, ...) {
  hashbits <- get("hashbits", envir=cache, inherits=FALSE)
  hashmap <- get("hashmap", envir=cache, inherits=FALSE)
  hashdat <- get("x", envir=cache, inherits=FALSE)
  nunique <- get("nunique", envir=cache, inherits=FALSE)
  ret <- .Call(C_hashuni_integer64, hashdat, hashbits, hashmap, as.logical(keep.order), double(nunique))
  oldClass(ret) <- "integer64"
  ret
}

#' @rdname hashmap
#' @export
hashupo <- function(cache, ...) UseMethod("hashupo")
#' @rdname hashmap
#' @export
hashupo.cache_integer64 <- function(cache, keep.order=FALSE, ...) {
  hashbits <- get("hashbits", envir=cache, inherits=FALSE)
  hashmap <- get("hashmap", envir=cache, inherits=FALSE)
  hashdat <- get("x", envir=cache, inherits=FALSE)
  nunique <- get("nunique", envir=cache, inherits=FALSE)
  .Call(C_hashupo_integer64, hashdat, hashbits, hashmap, as.logical(keep.order), integer(nunique))
}

# just returns a vector of length nunique of counts of the values
# at positions hashupo(, keep.order=FALSE) which are those of hashuni(, keep.order=FALSE)
#' @rdname hashmap
#' @export
hashtab <- function(cache, ...) UseMethod("hashtab")
#' @rdname hashmap
#' @export
hashtab.cache_integer64 <- function(cache, ...) {
  hashbits <- get("hashbits", envir=cache, inherits=FALSE)
  hashmap <- get("hashmap", envir=cache, inherits=FALSE)
  hashdat <- get("x", envir=cache, inherits=FALSE)
  nunique <- get("nunique", envir=cache, inherits=FALSE)
  ret <- .Call(C_hashtab_integer64, hashdat, hashbits, hashmap, nunique)
  attr(ret, "names") <- c("values", "counts")
  ret
}

#' @rdname hashmap
#' @export
hashmaptab <- function(x, ...) UseMethod("hashmaptab")
#' @rdname hashmap
#' @export
hashmaptab.integer64 <- function(x, nunique=NULL, minfac=1.5, hashbits=NULL, ...) {
  if (is.null(nunique)) {
    nunique <- integer(1L)
    n <- length(x)
  } else {
    nunique <- as.integer(nunique)
    n <- nunique
  }
  if (is.null(hashbits))
    hashbits <- as.integer(ceiling(log2(n*minfac)))
  else
    hashbits <- as.integer(hashbits)
  nhash <- as.integer(2L^hashbits)
  hashmap <- integer(nhash)
  ret <- .Call(C_hashmaptab_integer64, x, hashbits, hashmap, nunique)
  # theoretically we could use {hashmap, nunique} at this point the same way like after calling hashmap_integer64
  attr(ret, "names") <- c("values", "counts")
  ret
}

#' @rdname hashmap
#' @export
hashmapuni <- function(x, ...) UseMethod("hashmapuni")
#' @rdname hashmap
#' @export
hashmapuni.integer64 <- function(x, nunique=NULL, minfac=1.5, hashbits=NULL, ...) {
  if (is.null(nunique)) {
    nunique <- integer(1L)
    n <- length(x)
  } else {
    nunique <- as.integer(nunique)
    n <- nunique
  }
  if (is.null(hashbits)) {
    minlen <- ceiling(n*minfac)
    if (minlen > 0L)
      hashbits <- as.integer(ceiling(log2(minlen)))
    else
      hashbits <- 0L
  } else {
    hashbits <- as.integer(hashbits)
  }
  nhash <- as.integer(2L^hashbits)
  hashmap <- integer(nhash)
  ret <- .Call(C_hashmapuni_integer64, x, hashbits, hashmap, nunique)
  # theoretically we could use {hashmap, nunique} at this point the same way like after calling hashmap_integer64
  oldClass(ret) <- "integer64"
  ret
}

#' @rdname hashmap
#' @export
hashmapupo <- function(x, ...) UseMethod("hashmapupo")
#' @rdname hashmap
#' @export
hashmapupo.integer64 <- function(x, nunique=NULL, minfac=1.5, hashbits=NULL, ...) {
  if (is.null(nunique)) {
    nunique <- integer(1L)
    n <- length(x)
  } else {
    nunique <- as.integer(nunique)
    n <- nunique
  }
  if (is.null(hashbits)) {
    minlen <- ceiling(n*minfac)
    if (minlen > 0L)
      hashbits <- as.integer(ceiling(log2(minlen)))
    else
      hashbits <- 0L
  } else {
    hashbits <- as.integer(hashbits)
  }
  nhash <- as.integer(2L^hashbits)
  hashmap <- integer(nhash)
  # theoretically we could use {hashmap, nunique} at this point the same way like after calling hashmap_integer64
  .Call(C_hashmapupo_integer64, x, hashbits, hashmap, nunique)
}


#' integer64: random numbers
#'
#' Create uniform random 64-bit integers within defined range
#'
#' @param n length of return vector
#' @param min lower inclusive bound for random numbers
#' @param max upper inclusive bound for random numbers
#' @param replace set to FALSE for sampleing from a finite pool, see [sample()]
#'
#' @return a integer64 vector
#'
#' @details
#' For each random integer we call R's internal C interface `unif_rand()` twice.
#'   Each call is mapped to 2^32 unsigned integers. The two 32-bit patterns are
#'   concatenated to form the new integer64. This process is repeated until the
#'   result is not a `NA_INTEGER64_`.
#' @keywords classes distribution sysdata
#' @seealso [runif()], [hashfun()]
#'
#' @examples
#'   runif64(12)
#'   runif64(12, -16, 16)
#'   runif64(12, 0, as.integer64(2^60)-1)  # not 2^60-1 !
#'   var(runif(1e4))
#'   var(as.double(runif64(1e4, 0, 2^40))/2^40)  # ~ = 1/12 = .08333
#'
#'   table(sample(16, replace=FALSE))
#'   table(runif64(16, 1, 16, replace=FALSE))
#'   table(sample(16, replace=TRUE))
#'   table(runif64(16, 1, 16, replace=TRUE))
#'
#' @export
runif64 <- function(n, min=lim.integer64()[1L], max=lim.integer64()[2L], replace = TRUE) {
  n <- as.integer(n)
  min <- as.integer64(min)
  max <- as.integer64(max)
  if (replace) {
    ret <- .Call(C_runif_integer64, n, min, max)
    oldClass(ret) <- "integer64"
  } else {
    N <- n
    d <- max - min + 1L
    if (!is.na(d) && N > d)
      stop("cannot take a sample larger than the population when 'replace = FALSE'")
    if (!is.na(d) && n > d  / (2.0*log(n, 64.0))) {
      ret <- .Call(C_runif_integer64, as.integer(d), as.integer64(min), as.integer64(max))
      oldClass(ret) <- "integer64"
      ret <- sample(ret, n, FALSE)
    } else {
      ret <- integer64()
      while (N > 0L) {
        ret <- unique(c(ret, Recall(
          if (N*1.05 < .Machine$integer.max) N*1.05 else N
        , min
        , max
        , replace=TRUE
        )))
        N <- n - length(ret)
      }
      if (N != 0L)
        ret <- ret[1:n]
    }
  }
  ret
}

test_that("runif64 behaves as expected", {
  withr::local_seed(3478)

  expect_identical(
    runif64(10L),
    as.integer64(c(
      "6312937654860439830", "5047107428523623805", "7829389831893364707",
      "-3641910282010306573", "4600438248413496767", "4871064969903669683",
      "2693636032523872093", "4503042760826424596", "-8860474785465525016", "-4614238549190155011"
    ))
  )

  expect_identical(
    runif64(5L, 10L, 20L),
    as.integer64(c(16L, 19L, 16L, 15L, 20L))
  )

  # large enough number to be confident the test isn't "randomly" succeeding,
  #   but not so large as to noticeably slow down the suite.
  x = runif64(100000L, -5L, 5L)
  expect_true(all(x >= -5L & x <= 5L))
})

test_that("implicit tests from ?hashmap continue working", {
  x = as.integer64(sample(c(NA, 0:9)))
  y = as.integer64(sample(c(NA, 1:9), 10, TRUE))
  hx = hashmap(x)
  hy = hashmap(y)

  expect_identical(match(as.integer(x), as.integer(y)), hashpos(hy, x))
  expect_identical(match(as.integer(x), as.integer(y)), hashrev(hx, y))
  expect_identical(as.integer(x) %in% as.integer(y), hashfin(hy, x))
  expect_identical(as.integer(x) %in% as.integer(y), hashrin(hx, y))
  expect_identical(duplicated(as.integer(y)), hashdup(hy))
  expect_identical(as.integer64(unique(as.integer(y))), hashuni(hy, keep.order=TRUE))
  expect_identical(sort(hashuni(hy, keep.order=FALSE)), sort(hashuni(hy, keep.order=TRUE)))
  expect_identical(y[hashupo(hy, keep.order=FALSE)], hashuni(hy, keep.order=FALSE))
  expect_identical(y[hashupo(hy, keep.order=TRUE)], hashuni(hy, keep.order=TRUE))
  expect_identical(hashpos(hy, hashuni(hy, keep.order=TRUE)), hashupo(hy, keep.order=TRUE))
  expect_identical(hashpos(hy, hashuni(hy, keep.order=FALSE)), hashupo(hy, keep.order=FALSE))
  expect_identical(hashuni(hy, keep.order=FALSE), hashtab(hy)$values)
  expect_identical(
    as.vector(table(as.integer(y), useNA="ifany")),
    hashtab(hy)$counts[order(hashtab(hy)$values)]
  )
  expect_identical(hashuni(hy, keep.order=TRUE), hashmapuni(y))
  expect_identical(hashupo(hy, keep.order=TRUE), hashmapupo(y))
  expect_identical(hashtab(hy), hashmaptab(y))
})

  #   require(bit64)
  #   require(microbenchmark)
  #   n <- 1000000L
  #   print(microbenchmark(runif64(n, 1.0, n), times=20L))
  #   for (m in c(1.0, 2.0, 4.0, 8.0, 16.0)) {
  #     print(microbenchmark(runif64(n, 1.0, n*m, replace=FALSE), times=20L))
  #     print(microbenchmark(sample(n*m, n, replace=FALSE), times=20L))
  #   }
  #   print(microbenchmark(runif64(n, 1.0, replace=FALSE), times=20L))


  # library(bit64)
  # n <- 10000000L
  # x <- as.integer64(sample(n, n, TRUE))
  # t1 <- system.time({h <- hashmap(x)})[3L]
  # t2 <- system.time({value <- hashuni(h)})[3L]
  # t3 <- system.time({count <- hashtab(h)})[3L]
  # t4 <- system.time({ret1 <- list(values=value, counts=count)})[3L]
  # t1+t2+t3+t4
  # system.time({ret2 <- hashmaptab(x)})[3L]
  # identical(ret1,ret2)


  # x <- as.integer64(sample(n, n, TRUE))

  # system.time({
  #   ret2 <- hashmaptab(x)
  #   cv2 <- sum(ret2$counts[ret2$counts > 1.0])
  # })[3L]

  # system.time({
  #   s <- clone(x)
  #   na.count <- ramsort(s, has.na = TRUE, na.last = FALSE, decreasing = FALSE, stable = FALSE, optimize = "time")
  #   cv <- .Call(C_r_ram_integer64_sortnut, x = s)[[2L]]
  #   })

  # cv
  # cv2


  # nunique(x)
  # length(value)
  # length(count)
  # length(t1$value)
  # length(t1$count)
  # value
  # t1
  # count

  # s <- clone(x); o <- seq_along(x); ramsortorder(s, o)
  # t2 <- sortordertab(s,o)
  # length(s)
  # length(t2)




  # library(bit64)
  # n <- 1000000L
  # r <- runif64(n, lim.integer64()[1L], lim.integer64()[2L])
  # identical(r, as.integer64(as.bitstring(r)))
  # cbind(r,as.integer64(as.bitstring(r)))
  # cbind(as.bitstring(r),as.bitstring(as.integer64(as.bitstring(r))))

  # #sum(duplicated(r))
  # #table(r)
  # #range(r)
  # log2(abs(range(r)))

  # x <- seq(0.0, 1.0, 0.1)
  # y <- quantile.integer64(r, x)
  # z <- diff(y)
  # plot(log2(z), type="b",ylim=c(0.0, max(log2(z))))


  # n <- 10000000L
  # system.time(runif(n))
  # system.time(runif64(n))

#' Results of performance measurement on a Core i7 Lenovo T410 8 GB RAM under Windows 7 64bit
#'
#' These are the results of calling [benchmark64()]
#'
#' @format
#' The format is:
#'
#' ```
#' num [1:16, 1:6] 2.55e-05 2.37 2.39 1.28 1.39 ...
#' - attr(*, "dimnames")=List of 2
#' ..$ : chr [1:16] "cache" "match(s,b)" "s %in% b" "match(b,s)" ...
#' ..$ : chr [1:6] "32-bit" "64-bit" "hashcache" "sortordercache" ...
#' ```
#' @usage
#' data(benchmark64.data)
#'
#' @examples
#' data(benchmark64.data)
#' print(benchmark64.data)
#' matplot(log2(benchmark64.data[-1,1]/benchmark64.data[-1,])
#' , pch=c("3", "6", "h", "s", "o", "a")
#' , xlab="tasks [last=session]"
#' , ylab="log2(relative speed) [bigger is better]"
#' )
#' matplot(t(log2(benchmark64.data[-1,1]/benchmark64.data[-1,]))
#' , axes=FALSE
#' , type="b"
#' , lwd=c(rep(1, 14), 3)
#' , xlab="context"
#' , ylab="log2(relative speed) [bigger is better]"
#' )
#' axis(1
#' , labels=c("32-bit", "64-bit", "hash", "sortorder", "order", "hash+sortorder")
#' , at=1:6
#' )
#' axis(2)
#' @keywords datasets
"benchmark64.data"

#' Results of performance measurement on a Core i7 Lenovo T410 8 GB RAM under Windows 7 64bit
#'
#' These are the results of calling [optimizer64()]
#'
#' @format
#' The format is:
#'
#' ```
#' List of 16
#'  $ : num [1:9, 1:3] 0 0 1.63 0.00114 2.44 ...
#'   ..- attr(*, "dimnames")=List of 2
#'   .. ..$ : chr [1:9] "match" "match.64" "hashpos" "hashrev" ...
#'   .. ..$ : chr [1:3] "prep" "both" "use"
#'  $ : num [1:10, 1:3] 0 0 0 1.62 0.00114 ...
#'   ..- attr(*, "dimnames")=List of 2
#'   .. ..$ : chr [1:10] "%in%" "match.64" "%in%.64" "hashfin" ...
#'   .. ..$ : chr [1:3] "prep" "both" "use"
#'  $ : num [1:10, 1:3] 0 0 0.00105 0.00313 0.00313 ...
#'   ..- attr(*, "dimnames")=List of 2
#'   .. ..$ : chr [1:10] "duplicated" "duplicated.64" "hashdup" "sortorderdup1" ...
#'   .. ..$ : chr [1:3] "prep" "both" "use"
#'  $ : num [1:15, 1:3] 0 0 0 0.00104 0.00104 ...
#'   ..- attr(*, "dimnames")=List of 2
#'   .. ..$ : chr [1:15] "unique" "unique.64" "hashmapuni" "hashuni" ...
#'   .. ..$ : chr [1:3] "prep" "both" "use"
#'  $ : num [1:14, 1:3] 0 0 0 0.000992 0.000992 ...
#'   ..- attr(*, "dimnames")=List of 2
#'   .. ..$ : chr [1:14] "unique" "unipos.64" "hashmapupo" "hashupo" ...
#'   .. ..$ : chr [1:3] "prep" "both" "use"
#'  $ : num [1:13, 1:3] 0 0 0 0 0.000419 ...
#'   ..- attr(*, "dimnames")=List of 2
#'   .. ..$ : chr [1:13] "tabulate" "table" "table.64" "hashmaptab" ...
#'   .. ..$ : chr [1:3] "prep" "both" "use"
#'  $ : num [1:7, 1:3] 0 0 0 0.00236 0.00714 ...
#'   ..- attr(*, "dimnames")=List of 2
#'   .. ..$ : chr [1:7] "rank" "rank.keep" "rank.64" "sortorderrnk" ...
#'   .. ..$ : chr [1:3] "prep" "both" "use"
#'  $ : num [1:6, 1:3] 0 0 0.00189 0.00714 0 ...
#'   ..- attr(*, "dimnames")=List of 2
#'   .. ..$ : chr [1:6] "quantile" "quantile.64" "sortqtl" "orderqtl" ...
#'   .. ..$ : chr [1:3] "prep" "both" "use"
#'  $ : num [1:9, 1:3] 0 0 0.00105 1.17 0 ...
#'   ..- attr(*, "dimnames")=List of 2
#'   .. ..$ : chr [1:9] "match" "match.64" "hashpos" "hashrev" ...
#'   .. ..$ : chr [1:3] "prep" "both" "use"
#'  $ : num [1:10, 1:3] 0 0 0 0.00104 1.18 ...
#'   ..- attr(*, "dimnames")=List of 2
#'   .. ..$ : chr [1:10] "%in%" "match.64" "%in%.64" "hashfin" ...
#'   .. ..$ : chr [1:3] "prep" "both" "use"
#'  $ : num [1:10, 1:3] 0 0 1.64 2.48 2.48 ...
#'   ..- attr(*, "dimnames")=List of 2
#'   .. ..$ : chr [1:10] "duplicated" "duplicated.64" "hashdup" "sortorderdup1" ...
#'   .. ..$ : chr [1:3] "prep" "both" "use"
#'  $ : num [1:15, 1:3] 0 0 0 1.64 1.64 ...
#'   ..- attr(*, "dimnames")=List of 2
#'   .. ..$ : chr [1:15] "unique" "unique.64" "hashmapuni" "hashuni" ...
#'   .. ..$ : chr [1:3] "prep" "both" "use"
#'  $ : num [1:14, 1:3] 0 0 0 1.62 1.62 ...
#'   ..- attr(*, "dimnames")=List of 2
#'   .. ..$ : chr [1:14] "unique" "unipos.64" "hashmapupo" "hashupo" ...
#'   .. ..$ : chr [1:3] "prep" "both" "use"
#'  $ : num [1:13, 1:3] 0 0 0 0 0.32 ...
#'   ..- attr(*, "dimnames")=List of 2
#'   .. ..$ : chr [1:13] "tabulate" "table" "table.64" "hashmaptab" ...
#'   .. ..$ : chr [1:3] "prep" "both" "use"
#'  $ : num [1:7, 1:3] 0 0 0 2.96 10.69 ...
#'   ..- attr(*, "dimnames")=List of 2
#'   .. ..$ : chr [1:7] "rank" "rank.keep" "rank.64" "sortorderrnk" ...
#'   .. ..$ : chr [1:3] "prep" "both" "use"
#'  $ : num [1:6, 1:3] 0 0 1.62 10.61 0 ...
#'   ..- attr(*, "dimnames")=List of 2
#'   .. ..$ : chr [1:6] "quantile" "quantile.64" "sortqtl" "orderqtl" ...
#'   .. ..$ : chr [1:3] "prep" "both" "use"
#'  - attr(*, "dim")= int [1:2] 8 2
#'  - attr(*, "dimnames")=List of 2
#'   ..$ : chr [1:8] "match" "%in%" "duplicated" "unique" ...
#'   ..$ : chr [1:2] "65536" "33554432"
#' ```
#'
#' @usage
#' data(optimizer64.data)
#'
#' @examples
#' data(optimizer64.data)
#' print(optimizer64.data)
#' oldpar <- par(no.readonly = TRUE)
#' par(mfrow=c(2,1))
#' par(cex=0.7)
#' for (i in 1:nrow(optimizer64.data)) {
#'  for (j in 1:2) {
#'    tim <- optimizer64.data[[i,j]]
#'   barplot(t(tim))
#'   if (rownames(optimizer64.data)[i]=="match")
#'    title(paste("match", colnames(optimizer64.data)[j], "in", colnames(optimizer64.data)[3-j]))
#'   else if (rownames(optimizer64.data)[i]=="%in%")
#'    title(paste(colnames(optimizer64.data)[j], "%in%", colnames(optimizer64.data)[3-j]))
#'   else
#'    title(paste(rownames(optimizer64.data)[i], colnames(optimizer64.data)[j]))
#'  }
#' }
#' par(mfrow=c(1,1))
#'
#' @keywords datasets
"optimizer64.data"

# /*
# R-Code
# S3 atomic 64bit integers for R
# (c) 2011-2024 Jens Oehlschägel
# (c) 2025-2026 Michael Chirico
# Licence: GPL2
# Provided 'as is', use at your own risk
# Created: 2026-01-16
#*/

#' Binary operators for integer64 vectors
#'
#' Binary operators for integer64 vectors.
#'
#' @param e1,e2,x numeric or complex vectors or objects which can be coerced to such, or other objects for which methods have been written for - especially 'integer64' vectors.
#'
#' @returns
#'   [`&`], [`|`], [`!`], [`!=`], [`==`], [`<`], [`<=`], [`>`], [`>=`] return a logical vector
#'
#'   [`/`] returns a double vector
#'
#'   [`+`], [`-`], [`*`], [`%/%`][Arithmetic], [`%%`][Arithmetic], [`^`] return a vector of class 'integer64' or different class depending on the operands
#'
#' @keywords classes manip
#' @seealso [integer64()]
#' @examples
#'   as.integer64(1:12) - 1
#'   options(integer64_semantics="new")
#'   d <- 2.5
#'   i <- as.integer64(5)
#'   d/i  # new 0.5
#'   d*i  # new 13
#'   i*d  # new 13
#'   options(integer64_semantics="old")
#'   d/i  # old: 0.4
#'   d*i  # old: 10
#'   i*d  # old: 13
#' @name ops64
NULL

# Version of Leonardo Silvestri
#' @rdname ops64
#' @export
binattr = function(e1, e2) {
  d1 = dim(e1)
  d2 = dim(e2)
  n1 = length(e1)
  n2 = length(e2)

  ## this first part takes care of erroring out when the dimensions
  ## are not compatible or warning if needed:
  if (length(d1)) {
    if (length(d2)) {
      if (!identical(dim(e1), dim(e2)))
        stop(gettext("non-conformable arrays", domain="R"))
    } else {
      if (n2 > n1 && n1)
        stop("length(e2) does not match dim(e1)")
      if (n2 && n1 %% n2)
        warning("length(e1) not a multiple length(e2)")
    }
  } else if (length(d2)) {
    if (n1 > n2 && n2)
      stop("length(e1) does not match dim(n2)")
    if (n1 && n2 %% n1)
      warning("length(e2) not a multiple length(e1)")
  } else {
    # nolint next: unnecessary_nesting_linter. Good parallelism.
    if (n1 < n2 && n1) {
      if (n1 && n2 %% n1)
        warning("length(e2) not a multiple length(e1)")
    } else {
      # nolint next: unnecessary_nesting_linter. Good parallelism.
      if (n2 && n1 %% n2)
        warning("length(e1) not a multiple length(e2)")
    }
  }

  ## in this part we mimic R's algo for selecting attributes:
  if (n1 == n2) {
    ## if same size take attribute from e1 if it exists, else from e2
    if (n1 == 0L) {
      ae1 = attributes(e1)[c("class", "dim", "dimnames")]
      ae2 = attributes(e2)[c("class", "dim", "dimnames")]
    }
    ae1 = attributes(e1)
    ae2 = attributes(e2)
    nae1 = names(attributes(e1))
    nae2 = names(attributes(e2))
    if (n1 == 0L) {
      ae1 = ae1[nae1 %in% c("class", "dim", "dimnames")]
      ae2 = ae1[nae1 %in% c("class", "dim", "dimnames")]
    }
    allattr = list()
    for (a in union(nae1, nae2))
      if (a %in% nae1)
        allattr[[a]] = ae1[[a]]
    else
      allattr[[a]] = ae2[[a]]
    allattr
  } else if (n1 == 0L || n1 > n2) {
    attributes(e1)
  } else {
    attributes(e2)
  }
}

.validate_binary_operator_argument = function(x) {
  if (is.numeric(unclass(x))) return(invisible())
  if (is.logical(x)) return(invisible())
  if (is.complex(x)) return(invisible())
  if (inherits(x, "POSIXt")) return(invisible())
  if (is.character(x)) return(invisible())
  stop(errorCondition(gettext("non-numeric argument to binary operator", domain="R"), call=sys.call(sys.nframe() - 2L)))
}

# helper for determining the target class for Ops methods
target_class_for_Ops = function(e1, e2) {
  convert_to_integer64 = function(el) is.numeric(el) || is.logical(el)
  if(missing(e2)) {
    if (!is.numeric(unclass(e1)) && !is.logical(e1) && !is.complex(e1) && !inherits(e1, "POSIXt"))
      stop(errorCondition(gettext("non-numeric argument to mathematical function", domain="R"), call=sys.call(sys.nframe() - 1L)))

    if (convert_to_integer64(e1)) {
      "integer64"
    } else {
      class(e1)[1L]
    }
  } else {
    .validate_binary_operator_argument(e1)
    .validate_binary_operator_argument(e2)

    if (is.factor(e1) || is.factor(e2)) {
      "factor"
    } else if (is.character(e1) || is.character(e2)) {
      "character"
    } else {
      conv_to_int1 = convert_to_integer64(e1)
      if (conv_to_int1 && convert_to_integer64(e2)) {
        "integer64"
      } else if (conv_to_int1) {
        class(e2)[1L]
      } else {
        class(e1)[1L]
      }
    }
  }
}

factor_op_warning_na = function(op, e1, e2) {
  warning(gettextf("%s not meaningful for factors", sQuote(op), domain = "R"), domain = NA)
  l1 = length(e1)
  l2 = length(e2)
  l = if (l1 == 0L || l2 == 0L) 0L else max(l1, l2)
  rep(NA, l)
}

#' @rawNamespace if (getRversion() >= "4.3.0") S3method(chooseOpsMethod,integer64)
chooseOpsMethod.integer64 = function(x, y, mx, my, cl, reverse) {
  TRUE
}

#' @rdname ops64
#' @export
`+.integer64` = function(e1, e2) {
  if (missing(e2))
    return(e1)
  
  target_class = target_class_for_Ops(e1, e2)
  if (target_class != "integer64") {
    if (is.integer64(e1))
      e1 = .as_double_integer64(e1, keep.attributes=TRUE)
    else
      e2 = .as_double_integer64(e2, keep.attributes=TRUE)
    return(e1 + e2)
  }

  a = binattr(e1, e2)
  l1 = length(e1)
  l2 = length(e2)
  l = if (l1 == 0L || l2 == 0L) 0L else max(l1, l2)
  ret = .Call(C_plus_integer64, as.integer64(e1), as.integer64(e2), double(l))
  a$class = plusclass(a$class, "integer64")
  attributes(ret) = a
  ret
}

#' @rdname ops64
#' @export
`-.integer64` = function(e1, e2) {
  if (missing(e2)) {
    if (!is.integer64(e1)) 
      return(-e1)
    e2 = e1
    e1 = as.integer64(0L)
  } else {
    target_class = target_class_for_Ops(e1, e2)
    if (target_class != "integer64") {
      if (is.integer64(e1))
        e1 = .as_double_integer64(e1, keep.attributes=TRUE)
      else
        e2 = .as_double_integer64(e2, keep.attributes=TRUE)
      return(e1 - e2)
    }
  }

  a = binattr(e1, e2)
  l1 = length(e1)
  l2 = length(e2)
  l = if (l1 == 0L || l2 == 0L) 0L else max(l1, l2)
  ret = .Call(C_minus_integer64, as.integer64(e1), as.integer64(e2), double(l))
  a$class = plusclass(a$class, "integer64")
  attributes(ret) = a
  ret
}

#' @rdname ops64
#' @export
`%/%.integer64` = function(e1, e2) {
  target_class = target_class_for_Ops(e1, e2)
  if (target_class != "integer64") {
    if (is.integer64(e1))
      e1 = .as_double_integer64(e1, keep.attributes=TRUE)
    else
      e2 = .as_double_integer64(e2, keep.attributes=TRUE)
    return(e1 %/% e2)
  }
  
  a = binattr(e1, e2)
  l1 = length(e1)
  l2 = length(e2)
  l = if (l1 == 0L || l2 == 0L) 0L else max(l1, l2)
  ret = .Call(C_intdiv_integer64, as.integer64(e1), as.integer64(e2), double(l))
  a$class = plusclass(a$class, "integer64")
  attributes(ret) = a
  ret
}

#' @rdname ops64
#' @export
`%%.integer64` = function(e1, e2) {
  target_class = target_class_for_Ops(e1, e2)
  if (target_class != "integer64") {
    if (is.integer64(e1))
      e1 = .as_double_integer64(e1, keep.attributes=TRUE)
    else
      e2 = .as_double_integer64(e2, keep.attributes=TRUE)
    return(e1 %% e2)
  }
  
  a = binattr(e1, e2)
  l1 = length(e1)
  l2 = length(e2)
  l = if (l1 == 0L || l2 == 0L) 0L else max(l1, l2)
  ret = .Call(C_mod_integer64, as.integer64(e1), as.integer64(e2), double(l))
  a$class = plusclass(a$class, "integer64")
  attributes(ret) = a
  ret
}

#' @rdname ops64
#' @export
`*.integer64` = function(e1, e2) {
  target_class = target_class_for_Ops(e1, e2)
  if (target_class != "integer64") {
    if (is.integer64(e1))
      e1 = .as_double_integer64(e1, keep.attributes=TRUE)
    else
      e2 = .as_double_integer64(e2, keep.attributes=TRUE)
    return(e1 * e2)
  }

  a = binattr(e1, e2)
  l1 = length(e1)
  l2 = length(e2)
  l = if (l1 == 0L || l2 == 0L) 0L else max(l1, l2)
  if (getOption("integer64_semantics", "old") == "old") {
    if (is.double(e2))  # implies !is.integer64(e2)
      ret = .Call(C_times_integer64_double, as.integer64(e1), e2, double(l))
    else
      ret = .Call(C_times_integer64_integer64, as.integer64(e1), as.integer64(e2), double(l))
  } else {
    # nolint next: unnecessary_nesting_linter. Good parallelism, and on a to-be-deprecated code path.
    if (is.double(e2))  # implies !is.integer64(e2)
      ret = .Call(C_times_integer64_double, as.integer64(e1), e2, double(l))
    else if (is.double(e1))
      ret = .Call(C_times_integer64_double, as.integer64(e2), e1, double(l))
    else
      ret = .Call(C_times_integer64_integer64, as.integer64(e1), as.integer64(e2), double(l))
  }
  a$class = plusclass(a$class, "integer64")
  attributes(ret) = a
  ret
}

#' @rdname ops64
#' @export
`^.integer64` = function(e1, e2) {
  target_class = target_class_for_Ops(e1, e2)
  if (target_class != "integer64") {
    if (is.integer64(e1))
      e1 = .as_double_integer64(e1, keep.attributes=TRUE)
    else
      e2 = .as_double_integer64(e2, keep.attributes=TRUE)
    return(e1 ^ e2)
  }
  
  a = binattr(e1, e2)
  l1 = length(e1)
  l2 = length(e2)
  l = if (l1 == 0L || l2 == 0L) 0L else max(l1, l2)
  if (is.double(e2)) {
    ret = .Call(C_power_integer64_double, e1, e2, double(l))
  } else {
    ret = .Call(C_power_integer64_integer64, as.integer64(e1), as.integer64(e2), double(l))
  }
  a$class = plusclass(a$class, "integer64")
  attributes(ret) = a
  ret
}

#' @rdname ops64
#' @export
`/.integer64` = function(e1, e2) {
  target_class = target_class_for_Ops(e1, e2)
  if (target_class != "integer64") {
    if (is.integer64(e1))
      e1 = .as_double_integer64(e1, keep.attributes=TRUE)
    else
      e2 = .as_double_integer64(e2, keep.attributes=TRUE)
    return(e1 / e2)
  }
  
  a = binattr(e1, e2)
  l1 = length(e1)
  l2 = length(e2)
  l = if (l1 == 0L || l2 == 0L) 0L else max(l1, l2)
  if (getOption("integer64_semantics", "old") == "old") {
    if (is.double(e2))  # implies !is.integer64(e2)
      ret = .Call(C_divide_integer64_double, as.integer64(e1), e2, double(l))
    else
      ret = .Call(C_divide_integer64_integer64, as.integer64(e1), as.integer64(e2), double(l))
  } else {
    # nolint next: unnecessary_nesting_linter. Good parallelism, and on a to-be-deprecated code path.
    if (is.double(e2))  # implies !is.integer64(e2)
      ret = .Call(C_divide_integer64_double, as.integer64(e1), e2, double(l))
    else if (is.double(e1))
      ret = .Call(C_divide_double_integer64, e1, e2, double(l))
    else
      ret = .Call(C_divide_integer64_integer64, as.integer64(e1), as.integer64(e2), double(l))
  }
  a$class = minusclass(a$class, "integer64")
  attributes(ret) = a
  ret
}

#' @rdname ops64
#' @export
`==.integer64` = function(e1, e2) {
  target_class = target_class_for_Ops(e1, e2)
  if (target_class == "character" || target_class == "factor") {
    return(as.character(e1) == as.character(e2))
  }
  if (target_class != "integer64") {
    if (is.integer64(e1))
      e1 = .as_double_integer64(e1, keep.attributes=TRUE)
    else
      e2 = .as_double_integer64(e2, keep.attributes=TRUE)
    return(e1 == e2)
  }

  a = binattr(e1, e2)
  l1 = length(e1)
  l2 = length(e2)
  l = if (l1 == 0L || l2 == 0L) 0L else max(l1, l2)
  ret = .Call(C_EQ_integer64, as.integer64(e1), as.integer64(e2), logical(l))
  names(ret) = a$names
  ret
}

#' @rdname ops64
#' @export
`!=.integer64` = function(e1, e2) {
  target_class = target_class_for_Ops(e1, e2)
  if (target_class == "character" || target_class == "factor") {
    return(as.character(e1) != as.character(e2))
  }
  if (target_class != "integer64") {
    if (is.integer64(e1))
      e1 = .as_double_integer64(e1, keep.attributes=TRUE)
    else
      e2 = .as_double_integer64(e2, keep.attributes=TRUE)
    return(e1 != e2)
  }
  
  a = binattr(e1, e2)
  l1 = length(e1)
  l2 = length(e2)
  l = if (l1 == 0L || l2 == 0L) 0L else max(l1, l2)
  ret = .Call(C_NE_integer64, as.integer64(e1), as.integer64(e2), logical(l))
  names(ret) = a$names
  ret
}

#' @rdname ops64
#' @export
`<.integer64` = function(e1, e2) {
  target_class = target_class_for_Ops(e1, e2)
  if (target_class == "character") {
    return(as.character(e1) < as.character(e2))
  } else if (target_class == "factor") {
    return(factor_op_warning_na("<", e1, e2))
  }
  if (target_class != "integer64") {
    if (is.integer64(e1))
      e1 = .as_double_integer64(e1, keep.attributes=TRUE)
    else
      e2 = .as_double_integer64(e2, keep.attributes=TRUE)
    return(e1 < e2)
  }

  a = binattr(e1, e2)
  l1 = length(e1)
  l2 = length(e2)
  l = if (l1 == 0L || l2 == 0L) 0L else max(l1, l2)
  ret = .Call(C_LT_integer64, as.integer64(e1), as.integer64(e2), logical(l))
  names(ret) = a$names
  ret
}

#' @rdname ops64
#' @export
`<=.integer64` = function(e1, e2) {
  target_class = target_class_for_Ops(e1, e2)
  if (target_class == "character") {
    return(as.character(e1) <= as.character(e2))
  } else if (target_class == "factor") {
    return(factor_op_warning_na("<=", e1, e2))
  }
  if (target_class != "integer64") {
    if (is.integer64(e1))
      e1 = .as_double_integer64(e1, keep.attributes=TRUE)
    else
      e2 = .as_double_integer64(e2, keep.attributes=TRUE)
    return(e1 <= e2)
  }

  a = binattr(e1, e2)
  l1 = length(e1)
  l2 = length(e2)
  l = if (l1 == 0L || l2 == 0L) 0L else max(l1, l2)
  ret = .Call(C_LE_integer64, as.integer64(e1), as.integer64(e2), logical(l))
  names(ret) = a$names
  ret
}

#' @rdname ops64
#' @export
`>.integer64` = function(e1, e2) {
  target_class = target_class_for_Ops(e1, e2)
  if (target_class == "character") {
    return(as.character(e1) > as.character(e2))
  } else if (target_class == "factor") {
    return(factor_op_warning_na(">", e1, e2))
  }
  if (target_class != "integer64") {
    if (is.integer64(e1))
      e1 = .as_double_integer64(e1, keep.attributes=TRUE)
    else
      e2 = .as_double_integer64(e2, keep.attributes=TRUE)
    return(e1 > e2)
  }

  a = binattr(e1, e2)
  l1 = length(e1)
  l2 = length(e2)
  l = if (l1 == 0L || l2 == 0L) 0L else max(l1, l2)
  ret = .Call(C_GT_integer64, as.integer64(e1), as.integer64(e2), logical(l))
  names(ret) = a$names
  ret
}

#' @rdname ops64
#' @export
`>=.integer64` = function(e1, e2) {
  target_class = target_class_for_Ops(e1, e2)
  if (target_class == "character") {
    return(as.character(e1) >= as.character(e2))
  } else if (target_class == "factor") {
    return(factor_op_warning_na(">=", e1, e2))
  }
  if (target_class != "integer64") {
    if (is.integer64(e1))
      e1 = .as_double_integer64(e1, keep.attributes=TRUE)
    else
      e2 = .as_double_integer64(e2, keep.attributes=TRUE)
    return(e1 >= e2)
  }
  
  a = binattr(e1, e2)
  l1 = length(e1)
  l2 = length(e2)
  l = if (l1 == 0L || l2 == 0L) 0L else max(l1, l2)
  ret = .Call(C_GE_integer64, as.integer64(e1), as.integer64(e2), logical(l))
  names(ret) = a$names
  ret
}

#' @rdname ops64
#' @export
`&.integer64` = function(e1, e2) {
  target_class = target_class_for_Ops(e1, e2)
  if (target_class != "integer64") {
    if (is.integer64(e1))
      e1 = .as_double_integer64(e1, keep.attributes=TRUE)
    else
      e2 = .as_double_integer64(e2, keep.attributes=TRUE)
    return(e1 & e2)
  }
  
  a = binattr(e1, e2)
  ret = as.logical(e1) & as.logical(e2)
  names(ret) = a$names
  ret
}

#' @rdname ops64
#' @export
`|.integer64` = function(e1, e2) {
  target_class = target_class_for_Ops(e1, e2)
  if (target_class != "integer64") {
    if (is.integer64(e1))
      e1 = .as_double_integer64(e1, keep.attributes=TRUE)
    else
      e2 = .as_double_integer64(e2, keep.attributes=TRUE)
    return(e1 | e2)
  }
  
  a = binattr(e1, e2)
  ret = as.logical(e1) | as.logical(e2)
  names(ret) = a$names
  ret
}

#' @rdname ops64
#' @export
`!.integer64` = function(x) {
  ret = !as.logical(x)
  names(ret) = names(x)
  ret
}

#' A S3 class for vectors of 64bit integers
#'
#' @description
#' Package 'bit64' provides fast serializable S3 atomic 64bit (signed) integers
#' that can be used in vectors, matrices, arrays and data.frames. Methods are
#' available for coercion from and to logicals, integers, doubles, characters
#' and factors as well as many elementwise and summary functions.
#'
#' ### Version 0.8
#' With 'integer64' vectors you can store very large integers at the expense
#' of 64 bits, which is by factor 7 better than 'int64' from package 'int64'.
#' Due to the smaller memory footprint, the atomic vector architecture and
#' using only S3 instead of S4 classes, most operations are one to three orders
#' of magnitude faster: Example speedups are 4x for serialization, 250x for
#' adding, 900x for coercion and 2000x for object creation. Also 'integer64'
#' avoids an ongoing (potentially infinite) penalty for garbage collection
#' observed during existence of 'int64' objects (see code in example section).
#'
#' ### Version 0.9
#' Package 'bit64' - which extends R with fast 64-bit integers - now has fast
#' (single-threaded) implementations the most important univariate algorithmic
#' operations (those based on hashing and sorting). We now have methods for
#' 'match', '%in%', 'duplicated', 'unique', 'table', 'sort', 'order', 'rank',
#' 'quantile', 'median' and 'summary'. Regarding data management we also have
#' novel generics 'unipos' (positions of the unique values), 'tiepos' (
#' positions of ties), 'keypos' (positions of foreign keys in a sorted
#' dimension table) and derived methods 'as.factor' and 'as.ordered'. This 64-
#' bit functionality is implemented carefully to be not slower than the
#' respective 32-bit operations in Base R and also to avoid outlying waiting
#' times observed with 'order', 'rank' and 'table' (speedup factors 20/16/200
#' respective). This increases the dataset size with wich we can work truly
#' interactive. The speed is achieved by simple heuristic optimizers in high-
#' level functions choosing the best from multiple low-level algorithms and
#' further taking advantage of a novel caching if activated. In an example R
#' session using a couple of these operations the 64-bit integers performed 22x
#'  faster than base 32-bit integers, hash-caching improved this to 24x,
#' sortorder-caching was most efficient with 38x (caching hashing and sorting
#' is not worth it with 32x at duplicated RAM consumption).
#'
#' # Design considerations
#'
#' 64 bit integers are related to big data: we need them to overcome address space
#' limitations. Therefore performance of the 64 bit integer type is critical. In the
#' S language -- designed in 1975 -- atomic objects were defined to be vectors for a
#' couple of good reasons: simplicity, option for implicit parallelization, good
#' cache locality. In recent years many analytical databases have learnt that lesson:
#' column based data bases provide superior performance for many applications, the
#' result are products such as MonetDB, Sybase IQ, Vertica, Exasol, Ingres Vectorwise.
#' If we introduce 64 bit integers not natively in Base R but as an external package,
#' we should at least strive to make them as 'basic' as possible. Therefore the design
#' choice of bit64 not only differs from package int64, it is obvious: Like the other
#' atomic types in Base R, we model data type 'integer64' as a contiguous [`atomic`]
#' vector in memory, and we use the more basic [S3] class system, not [S4]. Like
#' package int64 we want our 'integer64' to be [`serialize`]able, therefore we also
#' use an existing data type as the basis. Again the choice is obvious: R has only one
#' 64 bit data type: doubles. By using [`double`]s, `integer64` [`inherits`] some
#' functionality such as [is.atomic()], [length()], [`length<-`], [names()],
#' [`names<-`], [dim()], [`dim<-`], [dimnames()], [`dimnames<-`].
#'
#' Our R level functions strictly follow the functional programming paradigm:
#' no modification of arguments or other side-effects. Before version 0.93  we
#' internally deviated from the strict paradigm in order to boost performance. Our C
#' functions do not create new return values, instead we pass-in the memory to be
#' returned as an argument. This gives us the freedom to apply the C-function to new
#' or old vectors, which helps to avoid unnecessary memory allocation, unnecessary
#' copying and unnecessary garbage collection. Prior to 0.93 _within_ our R functions
#' we also deviated from conventional R programming by not using [`attr<-`] and
#' [`attributes<-`] because they always did new memory allocation and copying in older
#' R versions. If we wanted to set attributes of return values that we have freshly
#' created, we instead used functions [bit::setattr()] and [bit::setattributes()].
#' From version 0.93 `bit::setattr()` is only used for manipulating [`cache`] objects,
#' in [ramsort.integer64()], [sort.integer64()], and [as.data.frame.integer64()].
#'
#' # Arithmetic precision and coercion
#'
#' The fact that we introduce 64 bit long long integers -- without introducing 128-bit
#' long doubles -- creates some subtle challenges: Unlike 32 bit [`integer`]s, the
#' `integer64` are no longer a proper subset of [`double`]. If a binary arithmetic
#' operation does involve a `double` and a `integer`, it is a no-brainer to return
#' `double` without loss of information. If an `integer64` meets a `double`, it is not
#' trivial what type to return. Switching to `integer64` limits our ability to
#' represent very large numbers, switching to `double` limits our ability to
#' distinguish `x` from `x+1`. Since the latter is the purpose of introducing 64 bit
#' integers, we usually return `integer64` from functions involving `integer64`, for
#' example in [`c()`][c.integer64], [`cbind()`][cbind.integer64], and
#' [`rbind()`][rbind.integer64]
#'
#' Our operators [`+`][ops64], [`-`][ops64], [`%/%`][ops64], and [`%%`][ops64] coerce 
#' their arguments to `integer64` and return `integer64` if they are `integer`, 
#' `double` or `logical`. Otherwise the `integer64` argument is coerced to double and 
#' the R base method for the other class is called.
#'
#' Our operators [`*`][ops64] and [`^`][ops64] coerce their first argument to 
#' `integer64` and possibly the second to `integer64` if it is not `double` and 
#' return `integer64` if they are `integer`, `double` or `logical`. Otherwise the 
#' `integer64` argument is coerced to double and the R base method for the other 
#' class is called.
#' 
#' The division [`/`][ops64] operator also coerces its first argument to `integer64` 
#' and possibly the second to `integer64` if it is not `double` and returns 
#' `integer64` if they are `integer`, `double` or `logical`. Otherwise the `integer64` 
#' argument is coerced to double and the R base method for the other class is called.
#' 
#' [`sqrt()`][sqrt.integer64], [`log()`][log.integer64],
#' [`log2()`][log2.integer64], and [`log10()`][log10.integer64] coerce their first 
#' argument to `integer64` and coerce internally their second argument to 
#' 'long double', they return as `double`
#'
#' | **argument1** | **op** | **argument2** | **->** | **coerced1** | **op** | **coerced2** | **->** | **result** |
#' |:-------------:|:------:|:-------------:|:------:|:------------:|:------:|:------------:|:------:|:----------:|
#' | integer64     | +      | double        | ->     | integer64    | +      | integer64    | ->     | integer64  |
#' | double        | +      | integer64     | ->     | integer64    | +      | integer64    | ->     | integer64  |
#' | integer64     | +      | complex       | ->     | double       | +      | complex      | ->     | complex    |
#' | complex       | +      | integer64     | ->     | complex      | +      | double       | ->     | complex    |
#' | integer64     | -      | double        | ->     | integer64    | -      | integer64    | ->     | integer64  |
#' | double        | -      | integer64     | ->     | integer64    | -      | integer64    | ->     | integer64  |
#' | integer64     | -      | complex       | ->     | double       | -      | complex      | ->     | complex    |
#' | complex       | -      | integer64     | ->     | complex      | -      | double       | ->     | complex    |
#' | integer64     | %/%    | double        | ->     | integer64    | %/%    | integer64    | ->     | integer64  |
#' | double        | %/%    | integer64     | ->     | integer64    | %/%    | integer64    | ->     | integer64  |
#' | integer64     | %/%    | complex       | ->     | double       | %/%    | complex      | ->     | complex    |
#' | complex       | %/%    | integer64     | ->     | complex      | %/%    | double       | ->     | complex    |
#' | integer64     | %%     | double        | ->     | integer64    | %%     | integer64    | ->     | integer64  |
#' | double        | %%     | integer64     | ->     | integer64    | %%     | integer64    | ->     | integer64  |
#' | integer64     | %%     | complex       | ->     | double       | %%     | complex      | ->     | complex    |
#' | complex       | %%     | integer64     | ->     | complex      | %%     | double       | ->     | complex    |
#' | integer64     | *      | double        | ->     | integer64    | *      | long double  | ->     | integer64  |
#' | double        | *      | integer64     | ->     | integer64    | *      | integer64    | ->     | integer64  |
#' | integer64     | *      | complex       | ->     | double       | *      | complex      | ->     | complex    |
#' | complex       | *      | integer64     | ->     | complex      | *      | double       | ->     | complex    |
#' | integer64     | /      | double        | ->     | integer64    | /      | long double  | ->     | double     |
#' | double        | /      | integer64     | ->     | integer64    | /      | long double  | ->     | double     |
#' | integer64     | /      | complex       | ->     | double       | /      | complex      | ->     | complex    |
#' | complex       | /      | integer64     | ->     | complex      | /      | double       | ->     | complex    |
#' | integer64     | ^      | double        | ->     | integer64    | ^      | long double  | ->     | double     |
#' | double        | ^      | integer64     | ->     | integer64    | ^      | long double  | ->     | double     |
#' | integer64     | ^      | complex       | ->     | double       | ^      | complex      | ->     | complex    |
#' | complex       | ^      | integer64     | ->     | complex      | ^      | double       | ->     | complex    |
#' | integer64     | %*%    | double        | ->     | integer64    | %*%    | integer64    | ->     | integer64  |
#' | double        | %*%    | integer64     | ->     | integer64    | %*%    | integer64    | ->     | integer64  |
#' | integer64     | %*%    | complex       | ->     | double       | %*%    | complex      | ->     | complex    |
#' | complex       | %*%    | integer64     | ->     | complex      | %*%    | double       | ->     | complex    |
#'
#' # Creating and testing S3 class 'integer64'
#'
#' Our creator function `integer64` takes an argument `length`, creates an atomic
#' double vector of this length, attaches an S3 class attribute 'integer64' to it,
#' and that's it. We simply rely on S3 method dispatch and interpret those 64-bit
#' elements as 'long long int'.
#'
#' [is.double()] currently returns TRUE for `integer64` and might return `FALSE` in
#' a later release. Consider `is.double()` to have undefined behavior and do query
#' [is.integer64()] _before_ querying `is.double()`.
#'
# As a second line of defense against misinterpretation we make `is.double()` return
# FALSE by making it S3 generic and adding a method `as.double.integer64()`.
#
#' The methods [is.integer64()] and [is.vector()] both return `TRUE` for `integer64`.
#' Note that we did not patch [storage.mode()] and [typeof()], which both continue
#' returning 'double'. Like for 32 bit [`integer`], [mode()] returns 'numeric' and
#' [as.double()] tries coercing to [`double`]. It is possible that 'integer64' becomes
#' a `vmode` in package ff.
#'
#' Further methods for creating `integer64` are [`range()`][range.integer64] which
#' returns the range of the data type if calles without arguments,
#' [`rep()`][rep.integer64], [`seq()`][seq.integer64].
#'
#' For all available methods on `integer64` vectors see the index below and the examples.
#'
#' # Index of implemented methods
#'
#' | **creating, testing, printing** | **see also**    | **description**           |
#' |--------------------------------:|----------------:|:--------------------------|
#' |                 `NA_integer64_` | [`NA_integer_`] | NA constant               |
#' |                     `integer64` |     [`integer`] | create zero atomic vector |
#' |                     [runif64()] |       [runif()] | create random vector      |
#' |               [rep.integer64()] |         [rep()] |                           |
#' |               [seq.integer64()] |         [seq()] |                           |
#' |                [is.integer64()] |          [is()] |                           |
#' |                                 |  [is.integer()] | inherited from Base R     |
#  |         [is.double.integer64()] |   [is.double()] |                           |
#' |         [is.vector.integer64()] |   [is.vector()] |                           |
#' |         [identical.integer64()] |   [identical()] |                           |
#' |          [`length<-.integer64`] |    [`length<-`] |                           |
#' |                                 |      [length()] | inherited from Base R     |
#' |                                       [`names<-`] | inherited from Base R     |
#' |                                 |       [names()] | inherited from Base R     |
#' |                                 |       [`dim<-`] | inherited from Base R     |
#' |                                 |         [dim()] | inherited from Base R     |
#' |                                 |  [`dimnames<-`] | inherited from Base R     |
#' |                                 |    [dimnames()] | inherited from Base R     |
#' |                                 |         [str()] | inherited from Base R, does not print values correctly |
#' |             [print.integer64()] |       [print()] |                           |
#' |               [str.integer64()] |         [str()] |                           |
#'
#' | **coercing to integer64**  | **see also**     | **description** |
#' |---------------------------:|-----------------:|:----------------|
#' |           [as.integer64()] |                  | generic         |
#' | [as.integer64.bitstring()] | [as.bitstring()] |                 |
#' | [as.integer64.character()] |    [character()] |                 |
#' |    [as.integer64.double()] |       [double()] |                 |
#' |   [as.integer64.integer()] |      [integer()] |                 |
#' | [as.integer64.integer64()] |      `integer64` |                 |
#' |   [as.integer64.logical()] |      [logical()] |                 |
#' |      [as.integer64.NULL()] |         [NULL()] |                 |
#'
#' | **coercing from integer64** | **see also**     | **description** |
#' |----------------------------:|-----------------:|:----------------|
#' |       [as.list.integer64()] |      [as.list()] | generic         |
#' |            [as.bitstring()] | [as.bitstring()] | generic         |
#' |  [as.bitstring.integer64()] |                  |                 |
#' |  [as.character.integer64()] | [as.character()] |                 |
#' |     [as.double.integer64()] |    [as.double()] |                 |
#' |    [as.integer.integer64()] |   [as.integer()] |                 |
#' |    [as.logical.integer64()] |   [as.logical()] |                 |
# removed as requested by the CRAN maintainer
#  |     [as.vector.integer64()] |    [as.vector()] |                 |
#'
#' | **data structures**         | **see also**      | **description**                    |
#' |----------------------------:|------------------:|:-----------------------------------|
#' |             [c.integer64()] |             [c()] | vector concatenate                 |
#' |         [cbind.integer64()] |         [cbind()] | column bind                        |
#' |         [rbind.integer64()] |         [rbind()] | row bind                           |
#' | [as.data.frame.integer64()] | [as.data.frame()] | coerce atomic object to data.frame |
#' |                             |    [data.frame()] | inherited from Base R since we have coercion |
#'
#' | **subscripting**                              | **see also**            | **description**          |
#' |----------------------------------------------:|------------------------:|:-------------------------|
#' |    [`[.integer64`][extract.replace.integer64] |    [`[`][base::Extract] | vector and array extract |
#' |  [`[<-.integer64`][extract.replace.integer64] |  [`[<-`][base::Extract] | vector and array assign  |
#' |   [`[[.integer64`][extract.replace.integer64] |   [`[[`][base::Extract] | scalar extract           |
#' | [`[[<-.integer64`][extract.replace.integer64] | [`[[<-`][base::Extract] | scalar assign            |
#'
#' | **binary operators** | **see also** | **description**   |
#' |---------------------:|-------------:|:------------------|
#' |      [`+.integer64`] |        [`+`] | returns integer64 |
#' |      [`-.integer64`] |        [`-`] | returns integer64 |
#' |      [`*.integer64`] |        [`*`] | returns integer64 |
#' |      [`^.integer64`] |        [`^`] | returns double    |
#' |      [`/.integer64`] |        [`/`] | returns double    |
#' | \code{\link[=ops64]{\%/\%}} | \code{\link[=Arithmetic]{\%/\%}} | returns integer64 |
#' | \code{\link[=ops64]{\%\%}} | \code{\link[=Arithmetic]{\%\%}} | returns integer64 |
#'
#' | **comparison operators** | **see also** | **description** |
#' |-------------------------:|-------------:|:----------------|
#' |         [`==.integer64`] |       [`==`] | |
#' |         [`!=.integer64`] |       [`!=`] | |
#' |          [`<.integer64`] |        [`<`] | |
#' |         [`<=.integer64`] |       [`<=`] | |
#' |          [`>.integer64`] |        [`>`] | |
#' |         [`>=.integer64`] |       [`>=`] | |
#'
#' \tabular{rrl}{
#'    \strong{logical operators} \tab \strong{see also} \tab \strong{description} \cr
#'    \code{\link{!.integer64}} \tab \code{\link{!}} \tab  \cr
#'    \code{\link{&.integer64}} \tab \code{\link{&}} \tab  \cr
#'    \code{\link{|.integer64}} \tab \code{\link[base:Logic]{|}} \tab  \cr
#' }
# TODO(r-lib/roxygen2#1668): Restore the markdown representation of the table.
# | **logical operators** | **see also** | **description** |
# |----------------------:|-------------:|:----------------|
# |       [`!.integer64`] |        [`!`] | |
# |       [`&.integer64`] |        [`&`] | |
# | [`\|.integer64`] | [`\|`][base::Logic] | |
#'
#' | **math functions**    | **see also** | **description**              |
#' |----------------------:|-------------:|:-----------------------------|
#' |   [is.na.integer64()] |    [is.na()] | returns logical              |
#' |  [format.integer64()] |   [format()] | returns character            |
#' |     [abs.integer64()] |      [abs()] | returns integer64            |
#' |    [sign.integer64()] |     [sign()] | returns integer64            |
#' |     [log.integer64()] |      [log()] | returns double               |
#' |   [log10.integer64()] |    [log10()] | returns double               |
#' |    [log2.integer64()] |     [log2()] | returns double               |
#' |    [sqrt.integer64()] |     [sqrt()] | returns double               |
#' | [ceiling.integer64()] |  [ceiling()] | dummy returning its argument |
#' |   [floor.integer64()] |    [floor()] | dummy returning its argument |
#' |   [trunc.integer64()] |    [trunc()] | dummy returning its argument |
#' |   [round.integer64()] |    [round()] | dummy returning its argument |
#' |  [signif.integer64()] |   [signif()] | dummy returning its argument |
#'
#' | **cumulative functions** | **see also** | **description** |
#' |-------------------------:|-------------:|:----------------|
#' |     [cummin.integer64()] |   [cummin()] | |
#' |     [cummax.integer64()] |   [cummax()] | |
#' |     [cumsum.integer64()] |   [cumsum()] | |
#' |    [cumprod.integer64()] |  [cumprod()] | |
#' |       [diff.integer64()] |     [diff()] | |
#'
#' | **summary functions** | **see also** | **description** |
#' |----------------------:|-------------:|:----------------|
#' |   [range.integer64()] |    [range()] | |
#' |     [min.integer64()] |      [min()] | |
#' |     [max.integer64()] |      [max()] | |
#' |     [sum.integer64()] |      [sum()] | |
#' |    [mean.integer64()] |     [mean()] | |
#' |    [prod.integer64()] |     [prod()] | |
#' |     [all.integer64()] |      [all()] | |
#' |     [any.integer64()] |      [any()] | |
#'
#' | **algorithmically complex functions** | **see also** | **description (caching)**  |
#' |--------------------------------------:|-------------:|:---------------------------|
#' |      [match.integer64()] |      [match()] | position of x in table (h//o/so) |
#' | \code{\link[=match.integer64]{\%in\%.integer64}} | \code{\link[=match]{\%in\%}} | is x in table? (h//o/so) |
#' | [duplicated.integer64()] | [duplicated()] | is current element duplicate of previous one? (h//o/so) |
#' |     [unique.integer64()] |     [unique()] | (shorter) vector of unique values only (h/s/o/so) |
#' |     [unipos.integer64()] |     [unipos()] | positions corresponding to unique values (h/s/o/so) |
#' |     [tiepos.integer64()] |     [tiepos()] | positions of values that are tied (//o/so) |
#' |     [keypos.integer64()] |     [keypos()] | position of current value in sorted list of unique values (//o/so) |
#' |      [table.integer64()] |      [table()] | unique values and their frequencies (h/s/o/so) |
#' |       [sort.integer64()] |       [sort()] | sorted vector (/s/o/so) |
#' |      [order.integer64()] |      [order()] | positions of elements that would create sorted vector (//o/so) |
#' |       [rank.integer64()] |       [rank()] | (average) ranks of non-NAs, NAs kept in place (/s/o/so) |
#' |   [quantile.integer64()] |   [quantile()] | (existing) values at specified percentiles (/s/o/so) |
#' |     [median.integer64()] |     [median()] | (existing) value at percentile 0.5 (/s/o/so) |
#' |    [summary.integer64()] |    [summary()] | (/s/o/so) |
#' |  [all.equal.integer64()] |  [all.equal()] | test if two objects are (nearly) equal (/s/o/so) |
#'
#' | **helper functions** | **see also**   | **description**            |
#' |-----------------:|---------------:|:---------------------------|
#' |   [minusclass()] | [minusclass()] |  removing class attritbute |
#' |    [plusclass()] |  [plusclass()] |  inserting class attribute |
#' |      [binattr()] |    [binattr()] | define binary op behaviour |
#'
#' | **tested I/O functions** | **see also**    | **description**       |
#' |-------------------------:|----------------:|:----------------------|
#' |                          |  [read.table()] | inherited from Base R |
#' |                          | [write.table()] | inherited from Base R |
#' |                          |   [serialize()] | inherited from Base R |
#' |                          | [unserialize()] | inherited from Base R |
#' |                          |        [save()] | inherited from Base R |
#' |                          |        [load()] | inherited from Base R |
#' |                          |        [dput()] | inherited from Base R |
#' |                          |        [dget()] | inherited from Base R |
#'
#' # Limitations inherited from implementing 64 bit integers via an external package
#'
#'  - **vector size** of atomic vectors is still limited to
#'    [`.Machine$integer.max`][.Machine]. However, external memory extending packages
#'    such as ff or bigmemory can extend their address space now with `integer64`.
#'    Having 64 bit integers also help with those not so obvious address issues that
#'    arise once we exchange data with SQL databases and datawarehouses, which use big
#'    integers as surrogate keys, e.g. on indexed primary key columns. This puts R into
#'    a relatively strong position compared to certain commercial statistical softwares,
#'    which sell database connectivity but neither have the range of 64 bit integers,
#'    nor have integers at all, nor have a single numeric data type in their
#'    macro-glue-language.
#'  - **literals** such as `123LL` would require changes to Base R, up to then we need
#'    to write (and call) `as.integer64(123L)` or `as.integer64(123)` or
#'    `as.integer64('123')`. Only the latter allows to specify numbers beyond Base R's
#'    numeric data types and therefore is the recommended way to use -- using only one
#'    way may facilitate migrating code to literals at a later stage.
#'
#' # Limitations inherited from Base R, Core team, can you change this?
#'
#'  -  **[identical()]** with default parameters does not distinguish all bit-patterns of
#'     doubles. For testing purposes we provide a wrapper [identical.integer64()] that
#'     will distinguish all bit-patterns. It would be desireable to have a single call
#'     of `identical()` handle both, [`double`] and `integer64`.
#'
#'  - the **colon** operator [:] officially does not dispatch S3 methods, however, we
#'    have made it generic:
#'
#'    ```r
#'    from <- lim.integer64()[1]
#'    to <- from+99
#'    from:to
#'    ```
#'
#'    As a limitation remains: it will only dispatch at its first argument `from` but
#'    not at its second `to`.
#'
#'  - **[is.double()]** does not dispatch S3 methods, However, we have made it generic
#'    and it will return `FALSE` on `integer64`.
#'
#'  - **[c()]** only dispatches [c.integer64()] if the first argument is `integer64`
#'    and it does not recursively dispatch the proper method when called with argument
#'    `recursive=TRUE`. Therefore `c(list(integer64, integer64))` does not work and
#'    for now you can only call `c.integer64(list(x, x))`.
#'
#'  - **[unlist()]** is not generic and if it were, we would face similar problems as
#'    with [c()]
#'  - **[vector()]** with argument `mode='integer64'` cannot work without adjustment
#'    of Base R
#'  - **[as.vector()]** with argument `mode='integer64'` cannot work without adjustment
#'    of Base R
#'  - **[is.vector()]** does not dispatch its method [is.vector.integer64()]
#'  - **[mode<-()]** drops the class 'integer64' which is returned from
#'    `as.integer64()`. Also it does not remove an existing class 'integer64' when
#'    assigning mode 'integer'.
#'  - **[storage.mode<-()]** does not support external data types such as `integer64`
#'  - **[matrix()]** does drop the 'integer64' class attribute.
#'  - **[array()]**  does drop the 'integer64' class attribute.
#'    + In current R versions (1.15.1) this can be circumvented by activating the
#'      function `as.vector.integer64()`. However, the CRAN maintainer has requested
#'      to remove `as.vector.integer64()`, even at the price of breaking previously
#'      working functionality of the package.
#'
#'  - **[str()]** does not print the values of `integer64` correctly
#'
#' # Further limitations
#'
#'  - **subscripting** non-existing elements and subscripting with `NA`s is currently
#'    not supported. Such subscripting currently returns `9218868437227407266` instead
#'    of `NA` (the `NA` value of the underlying double code). Following the full R
#'    behaviour here would either destroy performance or require extensive C-coding.
#'
#' @note `integer64` are useful for handling database keys and exact counting in +-2^63.
#'   Do not use them as replacement for 32bit integers, integer64 are not supported for
#'   subscripting by R-core and they have different semantics when combined with double.
#'   Do understand that `integer64` can only be useful over `double` if we do not coerce
#'   it to `double`.
#'
#'   While
#'
#'   integer + double -> double + double -> double
#'
#'   or
#'
#'   1L + 0.5 -> 1.5
#'
#'   for additive operations we coerce to `integer64`
#'
#'   integer64 + double ->  integer64 + integer64 -> integer64
#'
#'   hence
#'
#'   as.integer64(1) + 0.5 -> 1LL + 0LL -> 1LL
#'
#'   see section "Arithmetic precision and coercion" above
#'
#' @seealso  [integer()] in base R
#' @examples
#' message("Using integer64 in vector")
#' x <- integer64(8)    # create 64 bit vector
#' x
#' is.atomic(x)         # TRUE
#' is.integer64(x)      # TRUE
#' is.numeric(x)        # TRUE
#' is.integer(x)        # FALSE - debatable
#' is.double(x)         # FALSE - might change
#' x[] <- 1:2           # assigned value is recycled as usual
#' x[1:6]               # subscripting as usual
#' length(x) <- 13      # changing length as usual
#' x
#' rep(x, 2)            # replicate as usual
#' seq(as.integer64(1), 10)     # seq.integer64 is dispatched on first given argument
#' seq(to=as.integer64(10), 1)  # seq.integer64 is dispatched on first given argument
#' seq.integer64(along.with=x)  # or call seq.integer64 directly
#' # c.integer64 is dispatched only if *first* argument is integer64 ...
#' x <- c(x,runif(length(x), max=100))
#' # ... and coerces everything to integer64 - including double
#' x
#' names(x) <- letters  # use names as usual
#' x
#'
#' message("Using integer64 in array - note that 'matrix' currently does not work")
#' message("as.vector.integer64 removed as requested by the CRAN maintainer")
#' message("as consequence 'array' also does not work anymore")
#  y <- array(as.integer64(NA), dim=c(3,4), dimnames=list(letters[1:3], LETTERS[1:4]))
#' message("we still can create a matrix or array by assigning 'dim'")
#' y <- rep(as.integer64(NA), 12)
#' dim(y) <- c(3,4)
#' dimnames(y) <- list(letters[1:3], LETTERS[1:4])
#' y["a",] <- 1:2       # assigning as usual
#' y
#' y[1:2,-4]            # subscripting as usual
#' # cbind.integer64 dispatched on any argument and coerces everything to integer64
#' cbind(E=1:3, F=runif(3, 0, 100), G=c("-1","0","1"), y)
#'
#' message("Using integer64 in data.frame")
#' str(as.data.frame(x))
#' str(as.data.frame(y))
#' str(data.frame(y))
#' str(data.frame(I(y)))
#' d <- data.frame(x=x, y=runif(length(x), 0, 100))
#' d
#' d$x
#'
#' message("Using integer64 with csv files")
#' fi64 <- tempfile()
#' write.csv(d, file=fi64, row.names=FALSE)
#' e <- read.csv(fi64, colClasses=c("integer64", NA))
#' unlink(fi64)
#' str(e)
#' identical.integer64(d$x,e$x)
#'
#' message("Serializing and unserializing integer64")
#' dput(d, fi64)
#' e <- dget(fi64)
#' identical.integer64(d$x,e$x)
#' e <- d[,]
#' save(e, file=fi64)
#' rm(e)
#' load(file=fi64)
#' identical.integer64(d,e)
#'
#'   \dontrun{
#' message("== Differences between integer64 and int64 ==")
#' require(bit64)
#' require(int64)
#'
#' message("-- integer64 is atomic --")
#' is.atomic(integer64())
#' #is.atomic(int64())
#' str(integer64(3))
#' #str(int64(3))
#'
#' message("-- The following performance numbers are measured under RWin64  --")
#' message("-- under RWin32 the advantage of integer64 over int64 is smaller --")
#'
#' message("-- integer64 needs 7x/5x less RAM than int64 under 64/32 bit OS
#' (and twice the RAM of integer as it should be) --")
#' #as.vector(object.size(int64(1e6))/object.size(integer64(1e6)))
#' as.vector(object.size(integer64(1e6))/object.size(integer(1e6)))
#'
#' message("-- integer64 creates 2000x/1300x faster than int64 under 64/32 bit OS
#' (and 3x the time of integer) --")
#' t32 <- system.time(integer(1e8))
#' t64 <- system.time(integer64(1e8))
#' #T64 <- system.time(int64(1e7))*10  # using 1e8 as above stalls our R on an i7 8 GB RAM Thinkpad
#' #T64/t64
#' t64/t32
#'
#' i32 <- sample(1e6)
#' d64 <- as.double(i32)
#'
#' message("-- the following timings are rather conservative since timings
#'  of integer64 include garbage collection -- due to looped calls")
#' message("-- integer64 coerces 900x/100x faster than int64
#'  under 64/32 bit OS (and 2x the time of coercing to integer) --")
#' t32 <- system.time(for(i in 1:1000)as.integer(d64))
#' t64 <- system.time(for(i in 1:1000)as.integer64(d64))
#' #T64 <- system.time(as.int64(d64))*1000
#' #T64/t64
#' t64/t32
#' td64 <- system.time(for(i in 1:1000)as.double(i32))
#' t64 <- system.time(for(i in 1:1000)as.integer64(i32))
#' #T64 <- system.time(for(i in 1:10)as.int64(i32))*100
#' #T64/t64
#' t64/td64
#'
#' message("-- integer64 serializes 4x/0.8x faster than int64
#'  under 64/32 bit OS (and less than 2x/6x the time of integer or double) --")
#' t32 <- system.time(for(i in 1:10)serialize(i32, NULL))
#' td64 <- system.time(for(i in 1:10)serialize(d64, NULL))
#' i64 <- as.integer64(i32);
#' t64 <- system.time(for(i in 1:10)serialize(i64, NULL))
#' rm(i64); gc()
#' #I64 <- as.int64(i32);
#' #T64 <- system.time(for(i in 1:10)serialize(I64, NULL))
#' #rm(I64); gc()
#' #T64/t64
#' t64/t32
#' t64/td64
#'
#'
#' message("-- integer64 adds 250x/60x faster than int64
#'  under 64/32 bit OS (and less than 6x the time of integer or double) --")
#' td64 <- system.time(for(i in 1:100)d64+d64)
#' t32 <- system.time(for(i in 1:100)i32+i32)
#' i64 <- as.integer64(i32);
#' t64 <- system.time(for(i in 1:100)i64+i64)
#' rm(i64); gc()
#' #I64 <- as.int64(i32);
#' #T64 <- system.time(for(i in 1:10)I64+I64)*10
#' #rm(I64); gc()
#' #T64/t64
#' t64/t32
#' t64/td64
#'
#' message("-- integer64 sums 3x/0.2x faster than int64
#' (and at about 5x/60X the time of integer and double) --")
#' td64 <- system.time(for(i in 1:100)sum(d64))
#' t32 <- system.time(for(i in 1:100)sum(i32))
#' i64 <- as.integer64(i32);
#' t64 <- system.time(for(i in 1:100)sum(i64))
#' rm(i64); gc()
#' #I64 <- as.int64(i32);
#' #T64 <- system.time(for(i in 1:100)sum(I64))
#' #rm(I64); gc()
#' #T64/t64
#' t64/t32
#' t64/td64
#'
#' message("-- integer64 diffs 5x/0.85x faster than integer and double
#' (int64 version 1.0 does not support diff) --")
#' td64 <- system.time(for(i in 1:10)diff(d64, lag=2L, differences=2L))
#' t32 <- system.time(for(i in 1:10)diff(i32, lag=2L, differences=2L))
#' i64 <- as.integer64(i32);
#' t64 <- system.time(for(i in 1:10)diff(i64, lag=2L, differences=2L))
#' rm(i64); gc()
#' t64/t32
#' t64/td64
#'
#'
#' message("-- integer64 subscripts 1000x/340x faster than int64
#' (and at the same speed / 10x slower as integer) --")
#' ts32 <- system.time(for(i in 1:1000)sample(1e6, 1e3))
#' t32<- system.time(for(i in 1:1000)i32[sample(1e6, 1e3)])
#' i64 <- as.integer64(i32);
#' t64 <- system.time(for(i in 1:1000)i64[sample(1e6, 1e3)])
#' rm(i64); gc()
#' #I64 <- as.int64(i32);
#' #T64 <- system.time(for(i in 1:100)I64[sample(1e6, 1e3)])*10
#' #rm(I64); gc()
#' #(T64-ts32)/(t64-ts32)
#' (t64-ts32)/(t32-ts32)
#'
#' message("-- integer64 assigns 200x/90x faster than int64
#' (and 50x/160x slower than integer) --")
#' ts32 <- system.time(for(i in 1:100)sample(1e6, 1e3))
#' t32 <- system.time(for(i in 1:100)i32[sample(1e6, 1e3)] <- 1:1e3)
#' i64 <- as.integer64(i32);
#' i64 <- system.time(for(i in 1:100)i64[sample(1e6, 1e3)] <- 1:1e3)
#' rm(i64); gc()
#' #I64 <- as.int64(i32);
#' #I64 <- system.time(for(i in 1:10)I64[sample(1e6, 1e3)] <- 1:1e3)*10
#' #rm(I64); gc()
#' #(T64-ts32)/(t64-ts32)
#' (t64-ts32)/(t32-ts32)
#'
#'
#' tdfi32 <- system.time(dfi32 <- data.frame(a=i32, b=i32, c=i32))
#' tdfsi32 <- system.time(dfi32[1e6:1,])
#' fi32 <- tempfile()
#' tdfwi32 <- system.time(write.csv(dfi32, file=fi32, row.names=FALSE))
#' tdfri32 <- system.time(read.csv(fi32, colClasses=rep("integer", 3)))
#' unlink(fi32)
#' rm(dfi32); gc()
#'
#' i64 <- as.integer64(i32);
#' tdfi64 <- system.time(dfi64 <- data.frame(a=i64, b=i64, c=i64))
#' tdfsi64 <- system.time(dfi64[1e6:1,])
#' fi64 <- tempfile()
#' tdfwi64 <- system.time(write.csv(dfi64, file=fi64, row.names=FALSE))
#' tdfri64 <- system.time(read.csv(fi64, colClasses=rep("integer64", 3)))
#' unlink(fi64)
#' rm(i64, dfi64); gc()
#'
#' #I64 <- as.int64(i32);
#' #tdfI64 <- system.time(dfI64<-data.frame(a=I64, b=I64, c=I64))
#' #tdfsI64 <- system.time(dfI64[1e6:1,])
#' #fI64 <- tempfile()
#' #tdfwI64 <- system.time(write.csv(dfI64, file=fI64, row.names=FALSE))
#' #tdfrI64 <- system.time(read.csv(fI64, colClasses=rep("int64", 3)))
#' #unlink(fI64)
#' #rm(I64, dfI64); gc()
#'
#' message("-- integer64 coerces 40x/6x faster to data.frame than int64
#' (and factor 1/9 slower than integer) --")
#' #tdfI64/tdfi64
#' tdfi64/tdfi32
#' message("-- integer64 subscripts from data.frame 20x/2.5x faster than int64
#'  (and 3x/13x slower than integer) --")
#' #tdfsI64/tdfsi64
#' tdfsi64/tdfsi32
#' message("-- integer64 csv writes about 2x/0.5x faster than int64
#' (and about 1.5x/5x slower than integer) --")
#' #tdfwI64/tdfwi64
#' tdfwi64/tdfwi32
#' message("-- integer64 csv reads about 3x/1.5 faster than int64
#' (and about 2x slower than integer) --")
#' #tdfrI64/tdfri64
#' tdfri64/tdfri32
#'
#' rm(i32, d64); gc()
#'
#'
#' message("-- investigating the impact on garbage collection: --")
#' message("-- the fragmented structure of int64 messes up R's RAM --")
#' message("-- and slows down R's gargbage collection just by existing --")
#'
#' td32 <- double(21)
#' td32[1] <- system.time(d64 <- double(1e7))[3]
#' for (i in 2:11)td32[i] <- system.time(gc(), gcFirst=FALSE)[3]
#' rm(d64)
#' for (i in 12:21)td32[i] <- system.time(gc(), gcFirst=FALSE)[3]
#'
#' t64 <- double(21)
#' t64[1] <- system.time(i64 <- integer64(1e7))[3]
#' for (i in 2:11)t64[i] <- system.time(gc(), gcFirst=FALSE)[3]
#' rm(i64)
#' for (i in 12:21)t64[i] <- system.time(gc(), gcFirst=FALSE)[3]
#'
#' #T64 <- double(21)
#' #T64[1] <- system.time(I64 <- int64(1e7))[3]
#' #for (i in 2:11)T64[i] <- system.time(gc(), gcFirst=FALSE)[3]
#' #rm(I64)
#' #for (i in 12:21)T64[i] <- system.time(gc(), gcFirst=FALSE)[3]
#'
#' #matplot(1:21, cbind(td32, t64, T64), pch=c("d","i","I"), log="y")
#' matplot(1:21, cbind(td32, t64), pch=c("d","i"), log="y")
#'   }
#' @aliases bit64 is.integer.integer64 is.vector.integer64
#' @keywords internal package classes manip
#' @useDynLib bit64, .registration = TRUE, .fixes = "C_"
"_PACKAGE"

## usethis namespace: start
#' @importFrom bit clone is.sorted keyorder keysort keysortorder mergeorder
#'   mergesort mergesortorder na.count nties nunique nvalid quickorder
#'   quicksort quicksortorder radixorder radixsort radixsortorder ramorder
#'   ramsort ramsortorder repeat.time setattr shellorder shellsort
#'   shellsortorder still.identical
#' @importFrom graphics barplot par title
#' @importFrom methods as is
#' @importFrom stats cor median quantile
#' @importFrom utils head packageDescription strOptions tail getS3method
#' @export : %in% is.double match order print.cache rank
# TODO(>=4.9.0): un-export these
#' @export [<-.integer64 abs.integer64 all.integer64 any.integer64
#' @export as.character.integer64 as.data.frame.integer64 as.double.integer64
#' @export as.integer.integer64 as.integer64.character as.integer64.double
#' @export as.integer64.integer as.integer64.logical c.integer64
#' @export format.integer64 is.na.integer64 log.integer64 max.integer64
#' @export min.integer64 print.integer64 rank.integer64 rep.integer64
#' @export seq.integer64 str.integer64 sum.integer64 unique.integer64
## usethis namespace: end
NULL

# /*
# R-Code for patching S3 generics
# S3 atomic 64bit integers for R
# (c) 2011 Jens Oehlschägel
# Licence: GPL2
# Provided 'as is', use at your own risk
# Created: 2011-12-11
# Last changed:  2011-12-11
# */

#' Turning base R functions into S3 generics for bit64
#'
#' Turn those base functions S3 generic which are used in bit64
#'
#' @usage
#' from:to
#' is.double(x)
#' match(x, table, ...)
#' x \%in\% table
#' rank(x, ...)
#' order(...)
#' @aliases bit64S3 : :.default :.integer64 is.double is.double.default
#'   is.double.integer64 match match.default %in% %in%.default rank
#'   rank.default order order.default
#  @aliases table table.default
#' @param x integer64 vector: the values to be matched, optionally carrying a
#'   cache created with [hashcache()]
#' @param table integer64 vector: the values to be matched against, optionally
#'   carrying a cache created with [hashcache()] or [sortordercache()]
#' @param from scalar denoting first element of sequence
#' @param to scalar denoting last element of sequence
#' @param ... ignored
#'
#' @details
#' The following functions are turned into S3 generics in order to dispatch
#'   methods for [integer64()]:
#'
#'  - [`:`]
#'  - [is.double()]
#'  - [match()]
#'  - \code{\link[=match]{\%in\%}}
#   - [table()]
#'  - [rank()]
#'  - [order()]
#'
#' @return [invisible()]
#' @note
#'  - [is.double()] returns `FALSE` for [`integer64`]
#'  - [`:`] currently only dispatches at its first argument, thus
#'    `as.integer64(1):9` works but `1:as.integer64(9)` doesn't
#'  - [match()] currently only dispatches at its first argument and expects
#'    its second argument also to be integer64, otherwise throws an error.
#'    Beware of something like `match(2, as.integer64(0:3))`
#'  - \code{\link[=match]{\%in\%}} currently only dispatches at its first argument and expects
#'    its second argument also to be integer64, otherwise throws an error.
#'    Beware of something like `2 %in% as.integer64(0:3)`
#'  - [order()] currently only orders a single argument, trying more than
#'    one raises an error
#'
#' @seealso [bit64()], [S3]
#'
#' @examples
#' is.double(as.integer64(1))
#' as.integer64(1):9
#' match(as.integer64(2), as.integer64(0:3))
#' as.integer64(2) %in% as.integer64(0:3)
#'
#' unique(as.integer64(c(1,1,2)))
#' rank(as.integer64(c(1,1,2)))
#'
#   %table(as.integer64(c(1,1,2)))
#   %table(as.integer64(c(1,1,2)),as.integer64(c(3,4,4)))
#   %table(as.integer64(c(1,1,2)),c(3,4,4))
#   %table(c(1,1,2),as.integer64(c(3,4,4)))
#'
#' order(as.integer64(c(1,NA,2)))
#' @keywords methods
#' @name bit64S3
NULL

# OCT 2013: bit64S3() at wish of CRAN maintainers replaced by direct conversion to S3 generics
# in order to avoid assigning to globalenv

`:` <- function(from, to) UseMethod(":")
#' @export
`:.default` <- function(from, to) base::`:`(from, to)

#' @export
`:.integer64` <- function(from, to) seq.integer64(from=from, to=to)

is.double <- function(x) UseMethod("is.double")
#' @rdname bit64S3
#' @export
is.double.default <- function(x) base::is.double(x)

#' @rdname bit64S3
#' @export
is.double.integer64 <- function(x) FALSE

# TODO(R>=4.2.0): Remove workarounds for match(). Needed for #85 and #111.
#' @rdname bit64S3
#' @rawNamespace if (getRversion() >= "4.2.0") S3method(mtfrm,integer64)
mtfrm.integer64 = function(x) as.character(x)

match <- function(x, table, ...) UseMethod("match")
#' @rdname bit64S3
#' @export
match.default <- function(x, table, ...) {
  if (!exists("mtfrm", baseenv()) && is.integer64(table)) base::match(as.character(x), as.character(table), ...) # nocov
  else base::match(x, table, ...)
}

`%in%` <- function(x, table) UseMethod("%in%")
#' @rdname bit64S3
#' @export
`%in%.default` <- function(x, table) base::`%in%`(x, table)

rank <- function(x, ...) UseMethod("rank")
#' @rdname bit64S3
#' @export
rank.default <- function(x, ...) base::rank(x, ...)

# not yet able to combinewith other column types - better leave table() as is and hope for as.factor.integer64
#if (!exists("table.default")) {
#    "table" <- function(...) UseMethod("table")
#    "table.default" <- function(...) base::"table"(...)
#}

order <- function(...) UseMethod("order")
#' @rdname bit64S3
#' @export
order.default <- function(...) base::order(...)

#' @rdname matrix64
#' @export matrix
matrix = function(data=NA, nrow=1L, ncol=1L, byrow=FALSE, dimnames=NULL) UseMethod("matrix")
#' @exportS3Method matrix default
matrix.default = function(...) {
  withCallingHandlers_and_choose_call(
    base::matrix(...), 
    c("matrix", "matrix.default")
  )
}

#' @rdname matrix64
#' @export array
array = function(data=NA, dim=length(data), dimnames=NULL) UseMethod("array")
#' @exportS3Method array default
array.default = function(...) {
  withCallingHandlers_and_choose_call(
    base::array(...), 
    c("array", "array.default")
  )
}

#' @rdname matrix64
#' @export
colSums = function(x, na.rm=FALSE, dims=1L) UseMethod("colSums")
#' @rdname matrix64
#' @export
colSums.default = function(x, na.rm=FALSE, dims=1L) 
  withCallingHandlers_and_choose_call(
    base::colSums(x=x, na.rm=na.rm, dims=dims), 
    c("colSums", "colSums.default")
  )

#' @rdname matrix64
#' @export
rowSums = function(x, na.rm=FALSE, dims=1L) UseMethod("rowSums")
#' @rdname matrix64
#' @export
rowSums.default = function(x, na.rm=FALSE, dims=1L) 
  withCallingHandlers_and_choose_call(
    base::rowSums(x=x, na.rm=na.rm, dims=dims), 
    c("rowSums", "rowSums.default")
  )

test_that("colSums and rowSums work on simple integer64 input", {
  A = array(seq_len(120L), dim = 2:5)
  A64 = array64(A, dim=dim(A))

  # matches the behavior of sum.integer64 to not become numeric
  expect_s3_class(rowSums(A64), "integer64")
  expect_s3_class(colSums(A64), "integer64")

  expect_int_32_64_equivalent(rowSums(A))
  expect_int_32_64_equivalent(rowSums(A, dims=2L))
  expect_int_32_64_equivalent(rowSums(A, dims=3L))

  expect_int_32_64_equivalent(colSums(A))
  expect_int_32_64_equivalent(colSums(A, dims=2L))
  expect_int_32_64_equivalent(colSums(A, dims=3L))

  skip_if_not_r_version("4.0.0") # named args in stopifnot() unsupported -> different error
  expect_error(
    rowSums(A64, dims=4L),
    "dims= should be a length-1 integer",
    fixed = TRUE
  )
  expect_error(
    colSums(A64, dims=4L),
    "dims= should be a length-1 integer",
    fixed = TRUE
  )
})

test_that("colSums and rowSums work in presence of missing", {
  A = array(seq_len(120L), dim = 2:5)
  A[1L, 1L, 1L, 1L] = NA_integer_

  expect_int_32_64_equivalent(rowSums(A))
  expect_int_32_64_equivalent(rowSums(A, dims=2L))
  expect_int_32_64_equivalent(rowSums(A, dims=3L))

  expect_int_32_64_equivalent(colSums(A))
  expect_int_32_64_equivalent(colSums(A, dims=2L))
  expect_int_32_64_equivalent(colSums(A, dims=3L))

  expect_int_32_64_equivalent(rowSums(A, na.rm=TRUE))
  expect_int_32_64_equivalent(rowSums(A, na.rm=TRUE, dims=2L))
  expect_int_32_64_equivalent(rowSums(A, na.rm=TRUE, dims=3L))

  expect_int_32_64_equivalent(colSums(A, na.rm=TRUE))
  expect_int_32_64_equivalent(colSums(A, na.rm=TRUE, dims=2L))
  expect_int_32_64_equivalent(colSums(A, na.rm=TRUE, dims=3L))
})

test_that("All-missing inputs are handled correctly by colSums and rowSums", {
  A64 = matrix64(rep(NA_integer64_, 6L), nrow=3L, ncol=2L)

  expect_identical(rowSums(A64), rep(NA_integer64_, 3L))
  expect_identical(colSums(A64), rep(NA_integer64_, 2L))
})

test_that("out-of-integer-range inputs are handled correctly", {
  A64 = matrix64(2.0^(30:35), nrow=3L, ncol=2L)

  expect_identical(rowSums(A64), as.integer64(2L^30L*c(1L+8L, 2L+16L, 4L+32L)))
  expect_identical(colSums(A64), as.integer64(2L^30L*c(1L+2L+4L, 8L+16L+32L)))
})

test_that("aperm works in simple cases", {
  # example from ?aperm
  A = array64(1:24, 2:4)
  B = aperm(A, c(2L, 1L, 3L))
  # ignore class: t() gives 'array', not easy to delete it/add it to A[...]
  expect_identical(t(B[,, 2L]), A[,, 2L], ignore_attr="class")
  expect_identical(t(B[,, 3L]), A[,, 3L], ignore_attr="class")
  expect_identical(t(B[,, 4L]), A[,, 4L], ignore_attr="class")
})

test_that("base generic overwrites work", {
  x = c(2L, 4L, 3L)
  expect_identical(rank(x), c(1.0, 3.0, 2.0))
  expect_identical(order(x), c(1L, 3L, 2L))
})

# These tests were previously kept as tests under \examples{\dontshow{...}}.
#   Converted to "proper" unit tests for clarity, after making them more
#   canonical within {testthat}, e.g. better capturing expected warnings,
#   changing stopifnot(identical(...)) to expect_identical(...).
test_that("Old \\dontshow{} tests continue working", {
  expect_identical(match(as.integer64(2L), as.integer64(0:3)), match(2L, 0:3))
  expect_identical(as.integer64(2L) %in% as.integer64(0:3), 2L %in% 0:3)

  xi = c(1L, 1L, 2L)
  xi64 = as.integer64(xi)
  yi = c(3L, 4L, 4L)
  yi64 = as.integer64(yi)
  zi = c(1L, NA_integer_, 2L)
  zi64 = as.integer64(zi)

  expect_identical(unique(xi64), as.integer64(unique(xi)))
  expect_identical(rank(xi64), rank(xi))

  expect_identical(table(x=xi64), table(x=xi))
  expect_identical(table(x=xi64, y=yi64), table(x=xi, y=yi))
  expect_identical(table(x=xi64, y=yi), table(x=xi, y=yi))
  expect_identical(table(x=xi, y=yi64), table(x=xi, y=yi))

  expect_identical(order(zi64), order(zi))
  expect_identical(order(zi64, decreasing=TRUE), order(zi, decreasing=TRUE))
})

with_parameters_test_that(
  "matrix() wrapper works for {type}",
  {
    M = matrix(x)
  
    expect_true(is.matrix(M))
    if (type == "integer64") expect_s3_class(M, "integer64")
    expect_identical(M[seq_along(x)], x)
    expect_identical(dim(M), c(10L, 1L))
    expect_identical(matrix(x, byrow=TRUE)[seq_along(x)], x)
    expect_identical(dim(matrix(x, byrow=TRUE)), c(10L, 1L))
  
    expect_identical(matrix(x, nrow=2)[seq_along(x)], x)
    expect_identical(dim(matrix(x, nrow=2)), c(2L, 5L))
    expect_identical(matrix(x, nrow=2, byrow=TRUE)[seq_along(x)], x[c(1,6, 2,7, 3,8, 4,9, 5,10)])
    expect_identical(dim(matrix(x, nrow=2, byrow=TRUE)), c(2L, 5L))
  
    if (type == "integer64") {
      missing = NA_integer64_
      empty = integer64()
    } else {
      missing = NA_integer_
      empty = integer()
    }

    expect_identical(matrix(missing, nrow=2, ncol=1)[1:2], c(missing, missing))
    expect_identical(dim(matrix(missing, nrow=2, ncol=1)), c(2L, 1L))
    expect_identical(matrix(empty, nrow=2, ncol=1)[1:2], c(missing, missing))
    expect_identical(dim(matrix(empty, nrow=2, ncol=1)), c(2L, 1L))
  
    expect_identical(
      dimnames(matrix(x, 2, dimnames=list(NULL, letters[1:5]))),
      list(NULL, letters[1:5])
    )
    expect_identical(
      dimnames(matrix(x, 2, dimnames=list(LETTERS[1:2]))),
      list(LETTERS[1:2], NULL)
    )
    expect_identical(
      dimnames(matrix(x, 2, dimnames=list(LETTERS[1:2], letters[1:5]))),
      list(LETTERS[1:2], letters[1:5])
    )

    warning_stub = "data length [10] is not a sub-multiple or multiple of the number of"
    expect_warning(
      expect_identical(matrix(x, nrow=2, ncol=6)[seq_len(2*6)], c(x, x[1:2])),
      paste(warning_stub, "columns [6]"), fixed=TRUE
    )
    expect_warning(
      expect_identical(dim(matrix(x, nrow=2, ncol=6)), c(2L, 6L)),
      paste(warning_stub, "columns [6]"), fixed=TRUE
    )
    expect_warning(
      expect_identical(matrix(x, nrow=2, ncol=3)[seq_len(2*3)], x[seq_len(2*3)]),
      paste(warning_stub, "columns [3]"), fixed=TRUE
    )
    expect_warning(
      expect_identical(dim(matrix(x, nrow=2, ncol=3)), c(2L, 3L)),
      paste(warning_stub, "columns [3]"), fixed=TRUE
    )
    expect_warning(
      expect_identical(matrix(x, nrow=3, ncol=2)[seq_len(3*2)], x[seq_len(3*2)]),
      paste(warning_stub, "rows [3]"), fixed=TRUE
    )
    expect_error(
      matrix(x, nrow=-1),
      "invalid 'nrow' value", fixed=TRUE
    )
    expect_error(
      matrix(x, ncol=-1),
      "invalid 'ncol' value", fixed=TRUE
    )
  },
  .cases=data.frame(x=I(list(1:10, as.integer64(1:10))), type=c("integer", "integer64"))
)

with_parameters_test_that(
  "array() wrapper works for {type}",
  {
    A = array(x)

    expect_true(is.array(A))
    if (type == "integer64") expect_s3_class(A, "integer64")
    expect_identical(A[seq_along(x)], structure(x, dim = length(x)))
    expect_identical(dim(A), c(10L))
    expect_identical(array(x, c(2,5))[seq_along(x)], x)
    expect_identical(dim(array(x, c(2,5))), c(2L,5L))
    # TODO(R>=4.0.0): use rep_len(x, 1*2*3) over the ugly versions with seq_len()
    expect_identical(array(x, c(1,2,3))[seq_len(1*2*3)], x[seq_len(1*2*3)])
    expect_identical(dim(array(x, c(1,2,3))), c(1L,2L,3L))
    expect_identical(array(x, c(3,2,3))[seq_len(3*2*3)], x[(seq_len(3*2*3)-1L) %% length(x) + 1L])

    if (type == "integer64") {
      missing = NA_integer64_
      empty = integer64()
    } else {
      missing = NA_integer_
      empty = integer()
    }

    expect_identical(array(missing, c(2,1))[1:2], c(missing, missing))
    expect_identical(array(empty, c(2,1))[1:2], c(missing, missing))

    expect_identical(
      dimnames(array(x, c(2,5), dimnames=list(NULL, letters[1:5]))),
      list(NULL, letters[1:5])
    )
    expect_identical(
      dimnames(array(x, c(2,5), dimnames=list(LETTERS[1:2]))),
      list(LETTERS[1:2], NULL)
    )
    expect_identical(
      dimnames(array(x, c(2,5), dimnames=list(LETTERS[1:2], letters[1:5]))),
      list(LETTERS[1:2], letters[1:5])
    )
  
    expect_error(
      array(x, dim=NULL), 
      "'dims?' cannot be of length 0"
    )
    expect_error(
      array(x, dim=-1),
      "negative length vectors are not allowed"
    )

    expect_identical(array(x, dim=0),  structure(empty, dim = 0L))
  },
  .cases=data.frame(x=I(list(1:10, as.integer64(1:10))), type=c("integer", "integer64"))
)

test_that("colSums and rowSums work on simple integer64 input", {
  A = array(seq_len(120L), dim = 2:5)
  A64 = array64(A, dim=dim(A))

  # matches the behavior of sum.integer64 to not become numeric
  expect_s3_class(rowSums(A64), "integer64")
  expect_s3_class(colSums(A64), "integer64")

  expect_int_32_64_equivalent(rowSums(A))
  expect_int_32_64_equivalent(rowSums(A, dims=2L))
  expect_int_32_64_equivalent(rowSums(A, dims=3L))

  expect_int_32_64_equivalent(colSums(A))
  expect_int_32_64_equivalent(colSums(A, dims=2L))
  expect_int_32_64_equivalent(colSums(A, dims=3L))

  skip_unless_r(">= 4.0.0") # named args in stopifnot() unsupported -> different error
  expect_error(
    rowSums(A64, dims=4L),
    "invalid 'dims'",
    fixed = TRUE
  )
  expect_error(
    colSums(A64, dims=4L),
    "invalid 'dims'",
    fixed = TRUE
  )
})

test_that("colSums and rowSums work in presence of missing", {
  A = array(seq_len(120L), dim = 2:5)
  A[1L, 1L, 1L, 1L] = NA_integer_

  expect_int_32_64_equivalent(rowSums(A))
  expect_int_32_64_equivalent(rowSums(A, dims=2L))
  expect_int_32_64_equivalent(rowSums(A, dims=3L))

  expect_int_32_64_equivalent(colSums(A))
  expect_int_32_64_equivalent(colSums(A, dims=2L))
  expect_int_32_64_equivalent(colSums(A, dims=3L))

  expect_int_32_64_equivalent(rowSums(A, na.rm=TRUE))
  expect_int_32_64_equivalent(rowSums(A, na.rm=TRUE, dims=2L))
  expect_int_32_64_equivalent(rowSums(A, na.rm=TRUE, dims=3L))

  expect_int_32_64_equivalent(colSums(A, na.rm=TRUE))
  expect_int_32_64_equivalent(colSums(A, na.rm=TRUE, dims=2L))
  expect_int_32_64_equivalent(colSums(A, na.rm=TRUE, dims=3L))
})

test_that("All-missing inputs are handled correctly by colSums and rowSums", {
  A64 = matrix64(rep(NA_integer64_, 6L), nrow=3L, ncol=2L)

  expect_identical(rowSums(A64), rep(NA_integer64_, 3L))
  expect_identical(colSums(A64), rep(NA_integer64_, 2L))
})

test_that("out-of-integer-range inputs are handled correctly", {
  A64 = matrix64(2.0^(30:35), nrow=3L, ncol=2L)

  expect_identical(rowSums(A64), as.integer64(2L^30L*c(1L+8L, 2L+16L, 4L+32L)))
  expect_identical(colSums(A64), as.integer64(2L^30L*c(1L+2L+4L, 8L+16L+32L)))
})

test_that("dimnames with colSums and rowSums", {
  M32 = matrix(1:(3*2), nrow=3L, ncol=2L, dimnames=list(LETTERS[1:3], letters[1:2]))
  A32 = array(1:(2*5*3), dim=c(2, 5, 3), dimnames=list(LETTERS[1:2], letters[1:5], rev(LETTERS)[1:3]))
  M64 = matrix(as.integer64(1:(3*2)), nrow=3L, ncol=2L, dimnames=list(LETTERS[1:3], letters[1:2]))
  A64 = array(as.integer64(1:(2*5*3)), dim=c(2, 5, 3), dimnames=list(LETTERS[1:2], letters[1:5], rev(LETTERS)[1:3]))

  expect_identical(names(colSums(M32)), names(colSums(M64)))
  expect_identical(dimnames(colSums(M32)), dimnames(colSums(M64)))
  expect_identical(names(colSums(A32)), names(colSums(A64)))
  expect_identical(dimnames(colSums(A32)), dimnames(colSums(A64)))
  expect_identical(names(colSums(A32, dims=2L)), names(colSums(A64, dims=2L)))
  expect_identical(dimnames(colSums(A32, dims=2L)), dimnames(colSums(A64, dims=2L)))
  expect_identical(names(rowSums(M32)), names(rowSums(M64)))
  expect_identical(dimnames(rowSums(M32)), dimnames(rowSums(M64)))
  expect_identical(names(rowSums(A32)), names(rowSums(A64)))
  expect_identical(dimnames(rowSums(A32)), dimnames(rowSums(A64)))
  expect_identical(names(rowSums(A32, dims=2L)), names(rowSums(A64, dims=2L)))
  expect_identical(dimnames(rowSums(A32, dims=2L)), dimnames(rowSums(A64, dims=2L)))
})

test_that("aperm works in simple cases", {
  # example from ?aperm
  A = array64(1:24, 2:4)
  B = aperm(A, c(2L, 1L, 3L))
  # ignore class: t() gives 'array', not easy to delete it/add it to A[...]
  expect_identical(t(B[, , 2L]), A[, , 2L], ignore_attr="class")
  expect_identical(t(B[, , 3L]), A[, , 3L], ignore_attr="class")
  expect_identical(t(B[, , 4L]), A[, , 4L], ignore_attr="class")
})

test_that("matrix multiplication", {
  skip_unless_r(">= 4.0.0") # it does not work with ubuntu-latest (3.6), because a double vector is returned
  m32 = matrix(1:10, 2)
  m64 = matrix(as.integer64(m32), nrow(m32))
  mDo = matrix(as.numeric(m32), nrow(m32))
  mCo = matrix(as.complex(m32), nrow(m32))
  expect_error(m32%*%m32, "non-conformable arguments")
  expect_error(m64%*%m64, "non-conformable arguments")
  expect_identical(m64%*%t(m64), matrix(as.integer64(m32%*%t(m32)), nrow=nrow(m32)))
  expect_identical(t(m64)%*%m64, matrix(as.integer64(t(m32)%*%m32), ncol=ncol(m32)))
  expect_identical((1:2)%*%m64, matrix(as.integer64((1:2)%*%m32), ncol=ncol(m32)))
  expect_identical(m64%*%(1:5), matrix(as.integer64(m32%*%(1:5)), nrow=nrow(m32)))
  expect_error((1:2)%*%3L, "non-conformable arguments")
  expect_error(as.integer64(1:2)%*%as.integer64(3L), "non-conformable arguments")
  expect_identical(as.integer64(1L)%*%(3:4), matrix(as.integer64(3:4), nrow=1L))
  expect_identical(as.integer64(1:2)%*%(3:4), matrix(as.integer64(11L), nrow=1L))

  expect_identical(m64%*%t(m32), matrix(as.integer64(m32%*%t(m32)), nrow=nrow(m32)))
  expect_identical(m32%*%t(m64), matrix(as.integer64(m32%*%t(m32)), nrow=nrow(m32)))
  expect_identical(m64%*%t(mDo), matrix(as.integer64(m32%*%t(m32)), nrow=nrow(m32)))
  expect_identical(mDo%*%t(m64), matrix(as.integer64(m32%*%t(m32)), nrow=nrow(m32)))
  expect_identical(m64%*%t(mCo), matrix(as.complex(m32%*%t(m32)), nrow=nrow(m32)))
  expect_identical(mCo%*%t(m64), matrix(as.complex(m32%*%t(m32)), nrow=nrow(m32)))

  expect_same_error(m64 %*% LETTERS[1:5], m32 %*% LETTERS[1:5])
  expect_same_error(m64 %*% as.factor(LETTERS[1:5]), m32 %*% as.factor(LETTERS[1:5]))
  expect_error(m64%*%as.raw(1:5), "non-numeric argument to binary operator", fixed=TRUE)
  
  # warning in multiplication part
  x = as.integer64("4000000000") # x**2 > 2^63
  expect_warning(expect_identical(matrix(x, 1)%*%matrix(x, ncol=1), matrix(NA_integer64_, 1, 1)), "NAs produced by integer64 overflow")
  
  # warning in summation part
  x = rep_len(as.integer64("3000000000"), 2) # x**2 < 2^63, but 2 * x**2 > 2^63
  expect_warning(expect_identical(matrix(x, 1)%*%matrix(x, ncol=1), matrix(NA_integer64_, 1, 1)), "NAs produced by integer64 overflow")
  
})


test_that("coercion to matrix and array", {
  
  i32 = 1:10
  i64 = as.integer64(i32)
  m64 = matrix(as.integer64(i32), 2L)

  expect_identical(as.matrix(i64), structure(i64, dim = c(length(i64), 1L)), ignore_attr=if (getRversion() < "4.0.0") "class" else FALSE)
  expect_identical(as.matrix(m64), m64)

  expect_identical(as.array(i64), structure(i64, dim = c(length(i64))), ignore_attr=if (getRversion() < "4.0.0") "class" else FALSE)
  expect_identical(as.array(m64), m64)
})

matrix64 = function(x, nrow=1L, ncol=1L, byrow=FALSE, dimnames=NULL) {
  matrix(as.integer64(x), nrow=nrow, ncol=ncol, byrow=byrow, dimnames=dimnames)
}

array64 = function(x, dim) {
  array(as.integer64(x), dim=dim)
}

# Test that 'expr' gives the same result whether
#   the input is integer or integer64, in the sense
#   of equivalence after casting between the types.
# expr gets integer names converted to integer64,
#   retaining attributes (esp. for arrays), and
#   we test that the result of evaluating expr
#   is equivalent (after converting back to integer)
# Starting with integer and casting to integer64
#   guarantees representation, where casting integer64
#   to integer might have to stipulate inputs must be
#   representable as integer.
expect_int_32_64_equivalent = function(expr) {
  # Capture the unevaluated expression
  esub = substitute(expr)
  evar = all.vars(esub)

  # replace all integer values in expr with integer64 equivalents
  #   in a tailored environment
  parent_ = parent.frame()
  int64_env = new.env(parent = parent_)
  for (key in evar) {
    val = get(key, parent_)
    if (!is.integer(val)) next
    assign(key, as.integer64(val), envir=int64_env)
    attributes(int64_env[[key]]) = attributes(val)
  }

  int_result = eval(expr, parent_)
  int64_result = eval(expr, int64_env)
  int64_result_as_int = methods::as(int64_result, typeof(int_result))
  # ignore class (which includes integer64)
  a64 = attributes(int64_result)
  for (a in setdiff(names(a64), "class"))
    attr(int64_result_as_int, a) = a64[[a]]

  expect_identical(int64_result_as_int, int_result)
}

expect_same_error = function(expr1, expr2) {
  err1 = tryCatch(expr1, error=identity)
  if (!inherits(err1, "error")) return(fail("'expr1' did not produce an error"))

  err2 = tryCatch(expr2, error=identity)
  if (!inherits(err2, "error")) return(fail("'expr2' did not produce an error"))

  msg1 = conditionMessage(err1)
  msg2 = conditionMessage(err2)

  if (msg1 == msg2) return(pass())
  fail(sprintf("Error messages differ.\n'expr1' failed as %s\n'expr2' failed as %s", msg1, msg2))
}

# /*
# R-Code for sorting and ordering
# S3 atomic 64bit integers for R
# (c) 2011 Jens Oehlschägel
# Licence: GPL2
# Provided 'as is', use at your own risk
# Created: 2011-12-11
# Last changed:  2011-12-11
# */

#' Low-level intger64 methods for in-RAM sorting and ordering
#'
#' Fast low-level methods for sorting and ordering. The `..sortorder`
#'   methods do sorting and ordering at once, which requires more RAM
#'   than ordering but is (almost) as fast as as sorting.
#'
#' @note
#' Note that these methods purposely violate the functional programming
#'   paradigm: they are called for the side-effect of changing some of
#'   their arguments. The `sort`-methods change `x`, the `order`-methods
#'   change `i`, and the `sortoder`-methods change both `x` and `i`
#'
#' @param x a vector to be sorted by [ramsort.integer64()] and
#'   [ramsortorder.integer64()], i.e. the output of  [sort.integer64()]
#' @param i integer positions to be modified by [ramorder.integer64()] and
#'   [ramsortorder.integer64()], default is 1:n, in this case the output is
#'   similar to [order.integer64()]
#' @param has.na boolean scalar defining whether the input vector might contain
#'   `NA`s. If we know we don't have NAs, this may speed-up. _Note_ that you
#'   risk a crash if there are unexpected `NA`s with `has.na=FALSE`
#' @param na.last boolean scalar telling ramsort whether to sort `NA`s last
#'   or first. _Note_ that 'boolean' means that there is no third option `NA`
#'   as in [sort()]
#' @param decreasing boolean scalar telling ramsort whether to sort increasing
#'   or decreasing
#' @param stable boolean scalar defining whether stable sorting is needed.
#'   Allowing non-stable may speed-up.
#' @param optimize by default ramsort optimizes for 'time' which requires more
#'   RAM, set to 'memory' to minimize RAM requirements and sacrifice speed
#' @param restlevel number of remaining recursionlevels before `quicksort`
#'   switches from recursing to `shellsort`
#' @param radixbits size of radix in bits
#' @param VERBOSE cat some info about chosen method
#' @param ... further arguments, passed from generics, ignored in methods
#'
#' @details See [bit::ramsort()]
#' @return These functions return the number of `NAs` found or assumed
#'   during sorting
#' @keywords programming manip
#' @seealso [bit::ramsort()] for the generic, `ramsort.default` for the methods
#'   provided by package ff, [sort.integer64()] for the sort interface and
#'   [sortcache()] for caching the work of sorting
#' @examples
#'   x <- as.integer64(sample(c(rep(NA, 9), 1:9), 32, TRUE))
#'   x
#'   message("ramsort example")
#'   s <- bit::clone(x)
#'   bit::ramsort(s)
#'   message("s has been changed in-place - whether or not ramsort uses an in-place algorithm")
#'   s
#'   message("ramorder example")
#'   s <- bit::clone(x)
#'   o <- seq_along(s)
#'   bit::ramorder(s, o)
#'   message("o has been changed in-place - s remains unchanged")
#'   s
#'   o
#'   s[o]
#'   message("ramsortorder example")
#'   o <- seq_along(s)
#'   bit::ramsortorder(s, o)
#'   message("s and o have both been changed in-place - this is much faster")
#'   s
#'   o
#' @name ramsort.integer64
NULL

#' @rdname ramsort.integer64
#' @export
shellsort.integer64 <- function(x, has.na=TRUE, na.last=FALSE, decreasing=FALSE, ...) {
  force(x)
  .Call(C_r_ram_integer64_shellsort
  , x = x
  , has_na     = as.logical(has.na)
  , na_last    = as.logical(na.last)
  , decreasing = as.logical(decreasing)
  )
}

#' @rdname ramsort.integer64
#' @export
shellsortorder.integer64 <- function(x, i, has.na=TRUE, na.last=FALSE, decreasing=FALSE, ...) {
  force(x)
  force(i)
  if (!is.integer(i))
    stop("i must be integer")
  if (length(i) != length(x))
    stop("lengths of x and i don't match")
  .Call(C_r_ram_integer64_shellsortorder
  , x = x
  , i = i
  , has_na     = as.logical(has.na)
  , na_last    = as.logical(na.last)
  , decreasing = as.logical(decreasing)
  )
}

#' @rdname ramsort.integer64
#' @export
shellorder.integer64 <- function(x, i, has.na=TRUE, na.last=FALSE, decreasing=FALSE, ...) {
  force(x)
  force(i)
  if (!is.integer(i))
    stop("i must be integer")
  if (length(i) != length(x))
    stop("lengths of x and i don't match")
  .Call(C_r_ram_integer64_shellorder
  , x = x
  , i = i
  , has_na     = as.logical(has.na)
  , na_last    = as.logical(na.last)
  , decreasing = as.logical(decreasing)
  )
}

#' @rdname ramsort.integer64
#' @export
mergesort.integer64 <- function(x, has.na=TRUE, na.last=FALSE, decreasing=FALSE, ...) {
  force(x)
  .Call(C_r_ram_integer64_mergesort
  , x = x
  , has_na     = as.logical(has.na)
  , na_last    = as.logical(na.last)
  , decreasing = as.logical(decreasing)
  )
}

#' @rdname ramsort.integer64
#' @export
mergeorder.integer64 <- function(x, i, has.na=TRUE, na.last=FALSE, decreasing=FALSE, ...) {
  force(x)
  force(i)
  if (!is.integer(i))
    stop("i must be integer")
  if (length(i) != length(x))
    stop("lengths of x and i don't match")
  .Call(C_r_ram_integer64_mergeorder
  , x = x
  , i = i
  , has_na     = as.logical(has.na)
  , na_last    = as.logical(na.last)
  , decreasing = as.logical(decreasing)
  )
}

#' @rdname ramsort.integer64
#' @export
mergesortorder.integer64 <- function(x, i, has.na=TRUE, na.last=FALSE, decreasing=FALSE, ...) {
  force(x)
  force(i)
  if (!is.integer(i))
    stop("i must be integer")
  if (length(i) != length(x))
    stop("lengths of x and i don't match")
  .Call(C_r_ram_integer64_mergesortorder
  , x = x
  , i = i
  , has_na     = as.logical(has.na)
  , na_last    = as.logical(na.last)
  , decreasing = as.logical(decreasing)
  )
}

#' @rdname ramsort.integer64
#' @export
quicksort.integer64 <- function(x,
                                has.na=TRUE,
                                na.last=FALSE,
                                decreasing=FALSE,
                                restlevel=floor(1.5 * log2(length(x))),
                                ...) {
  force(x)
  if (restlevel<0L)
    restlevel = 0L
  .Call(C_r_ram_integer64_quicksort
  , x = x
  , has_na     = as.logical(has.na)
  , na_last    = as.logical(na.last)
  , decreasing = as.logical(decreasing)
  , restlevel = as.integer(restlevel)
  )
}

#' @rdname ramsort.integer64
#' @export
quicksortorder.integer64 <- function(x, i,
                                     has.na=TRUE,
                                     na.last=FALSE,
                                     decreasing=FALSE,
                                     restlevel=floor(1.5 * log2(length(x))),
                                     ...) {
  force(x)
  force(i)
  if (!is.integer(i))
    stop("i must be integer")
  if (length(i) != length(x))
    stop("lengths of x and i don't match")
  if (restlevel<0L)
    restlevel = 0L
  .Call(C_r_ram_integer64_quicksortorder
  , x = x
  , i = i
  , has_na     = as.logical(has.na)
  , na_last    = as.logical(na.last)
  , decreasing = as.logical(decreasing)
  , restlevel = as.integer(restlevel)
  )
}

#' @rdname ramsort.integer64
#' @export
quickorder.integer64 <- function(x, i,
                                 has.na=TRUE,
                                 na.last=FALSE,
                                 decreasing=FALSE,
                                 restlevel=floor(1.5 * log2(length(x))),
                                 ...) {
  force(x)
  force(i)
  if (!is.integer(i))
    stop("i must be integer")
  if (length(i) != length(x))
    stop("lengths of x and i don't match")
  if (restlevel<0L)
    restlevel = 0L
  .Call(C_r_ram_integer64_quickorder
  , x = x
  , i = i
  , has_na     = as.logical(has.na)
  , na_last    = as.logical(na.last)
  , decreasing = as.logical(decreasing)
  , restlevel = as.integer(restlevel)
  )
}

#' @rdname ramsort.integer64
#' @export
radixsort.integer64 <- function(x, has.na=TRUE, na.last=FALSE, decreasing=FALSE, radixbits=8L, ...) {
  stopifnot(radixbits %in% c(1L, 2L, 4L, 8L, 16L))
  force(x)
  .Call(C_r_ram_integer64_radixsort
  , x = x
  , has_na     = as.logical(has.na)
  , na_last    = as.logical(na.last)
  , decreasing = as.logical(decreasing)
  , radixbits = as.integer(radixbits)
  )
}

#' @rdname ramsort.integer64
#' @export
radixsortorder.integer64 <- function(x, i, has.na=TRUE, na.last=FALSE, decreasing=FALSE, radixbits=8L, ...) {
  stopifnot(radixbits %in% c(1L, 2L, 4L, 8L, 16L))
  force(x)
  force(i)
  if (!is.integer(i))
    stop("i must be integer")
  if (length(i) != length(x))
    stop("lengths of x and i don't match")
  .Call(C_r_ram_integer64_radixsortorder
  , x = x
  , i = i
  , has_na     = as.logical(has.na)
  , na_last    = as.logical(na.last)
  , decreasing = as.logical(decreasing)
  , radixbits = as.integer(radixbits)
  )
}

#' @rdname ramsort.integer64
#' @export
radixorder.integer64 <- function(x, i, has.na=TRUE, na.last=FALSE, decreasing=FALSE, radixbits=8L, ...) {
  stopifnot(radixbits %in% c(1L, 2L, 4L, 8L, 16L))
  force(x)
  force(i)
  if (!is.integer(i))
    stop("i must be integer")
  if (length(i) != length(x))
    stop("lengths of x and i don't match")
  .Call(C_r_ram_integer64_radixorder
  , x = x
  , i = i
  , has_na     = as.logical(has.na)
  , na_last    = as.logical(na.last)
  , decreasing = as.logical(decreasing)
  , radixbits = as.integer(radixbits)
  )
}

#' @rdname ramsort.integer64
#' @export
ramsort.integer64 <- function(x,
                              has.na=TRUE,
                              na.last=FALSE,
                              decreasing=FALSE,
                              stable=TRUE,
                              optimize=c("time", "memory"),
                              VERBOSE=FALSE,
                              ...) {
  optimize <- match.arg(optimize)
  if (is.null(names(x))) {
    if (optimize == "time") {
      if (length(x) < 2048L) {
        if (VERBOSE) cat("ramsort selected mergesort\n")
        mergesort(x, has.na=has.na, na.last=na.last, decreasing=decreasing)
      } else if (length(x) < 16777216L) {
        if (VERBOSE) cat("ramsort selected radix8sort\n")
        radixsort(x, radixbits=8L, has.na=has.na, na.last=na.last, decreasing=decreasing)
      } else {
        if (VERBOSE) cat("ramsort selected radix4sort\n")
        radixsort(x, radixbits=4L, has.na=has.na, na.last=na.last, decreasing=decreasing)
      }
    } else {
      if (VERBOSE) cat("ramsort selected quicksort\n")
      quicksort(x, has.na=has.na, na.last=na.last, decreasing=decreasing)
    }
  } else {
    if (stable || optimize == "time") {
      i <- seq_along(x)
      if (length(x) < 2048L) {
        if (VERBOSE) cat("ramsortorder selected mergesortorder\n")
        ret <- mergesortorder(x, i, has.na=has.na, na.last=na.last, decreasing=decreasing)
      } else if (length(x) < 2097152L) {
        if (VERBOSE) cat("ramsortorder selected radix8sortorder\n")
        ret <- radixsortorder(x, i, radixbits=8L, has.na=has.na, na.last=na.last, decreasing=decreasing)
      } else {
        if (VERBOSE) cat("ramsortorder selected radix4sortorder\n")
        ret <- radixsortorder(x, i, radixbits=4L, has.na=has.na, na.last=na.last, decreasing=decreasing)
      }
    } else {
      if (VERBOSE) cat("ramsort selected quicksortorder\n")
      i <- seq_along(x)
      ret <- quicksortorder(x, i, has.na=has.na, na.last=na.last, decreasing=decreasing)
    }
    setattr(x, "names", names(x)[i])
    ret
  }
}

#' @rdname ramsort.integer64
#' @export
ramsortorder.integer64 <- function(x, i,
                                   has.na=TRUE,
                                   na.last=FALSE,
                                   decreasing=FALSE,
                                   stable=TRUE,
                                   optimize=c("time", "memory"),
                                   VERBOSE=FALSE,
                                   ...) {
  optimize <- match.arg(optimize)
  if (!is.null(names(x)) && !is.null(names(i))) stop("names not supported")
  if (stable || optimize == "time") {
    if (length(x) < 2048L) {
      if (VERBOSE) cat("ramsortorder selected mergesortorder\n")
      mergesortorder(x, i, has.na=has.na, na.last=na.last, decreasing=decreasing)
    } else if (length(x) < 16777216L) {
      if (VERBOSE) cat("ramsortorder selected radix8sortorder\n")
      radixsortorder(x, i, radixbits=8L, has.na=has.na, na.last=na.last, decreasing=decreasing)
    } else {
      if (VERBOSE) cat("ramsortorder selected radix4sortorder\n")
      radixsortorder(x, i, radixbits=4L, has.na=has.na, na.last=na.last, decreasing=decreasing)
    }
  } else {
    if (VERBOSE) cat("ramsortorder selected quicksortorder\n")
    quicksortorder(x, i, has.na=has.na, na.last=na.last, decreasing=decreasing)
  }
}

#' @rdname ramsort.integer64
#' @export
ramorder.integer64 <- function(x, i,
                               has.na=TRUE,
                               na.last=FALSE,
                               decreasing=FALSE,
                               stable=TRUE,
                               optimize=c("time", "memory"),
                               VERBOSE=FALSE,
                               ...) {
  optimize <- match.arg(optimize)
  if (!is.null(names(x)) || !is.null(names(i))) stop("names not supported")
  if (stable) {
    if (VERBOSE) cat("ramorder selected mergeorder\n")
    mergeorder(x, i, has.na=has.na, na.last=na.last, decreasing=decreasing)
  } else {
    if (VERBOSE) cat("ramorder selected quickorder\n")
    quickorder(x, i, has.na=has.na, na.last=na.last, decreasing=decreasing)
  }
}

#' High-level intger64 methods for sorting and ordering
#'
#' Fast high-level methods for sorting and ordering. These are wrappers to
#'   [ramsort.integer64()] and friends and do not modify their arguments.
#'
#' @param x a vector to be sorted by [ramsort.integer64()] and
#'   [ramsortorder.integer64()], i.e. the output of  [sort.integer64()]
#' @param has.na boolean scalar defining whether the input vector might
#'   contain `NA`s. If we know we don't have NAs, this may speed-up. _Note_
#'   that you risk a crash if there are unexpected `NA`s with `has.na=FALSE`
#' @param na.last boolean scalar telling ramsort whether to sort `NA`s last
#'   or first. _Note_ that 'boolean' means that there is no third option
#'   `NA` as in [sort()]
#' @param decreasing boolean scalar telling ramsort whether to sort
#'   increasing or decreasing
#' @param stable boolean scalar defining whether stable sorting is needed.
#'   Allowing non-stable may speed-up.
#' @param optimize by default ramsort optimizes for 'time' which requires
#'   more RAM, set to 'memory' to minimize RAM requirements and sacrifice speed
#' @param VERBOSE cat some info about chosen method
#' @param ... further arguments, passed from generics, ignored in methods
#'
#' @details see [sort()] and [order()]
#' @return `sort` returns the sorted vector and `vector` returns the order positions.
#' @keywords programming manip
#' @seealso [`sort()`][sort.integer64], [sortcache()]
#' @examples
#'   x <- as.integer64(sample(c(rep(NA, 9), 1:9), 32, TRUE))
#'   x
#'   sort(x)
#'   message(
#'     "the following has default optimize='time' which is faster ",
#'     "but requires more RAM, this calls 'ramorder'"
#'   )
#'   order(x)
#'   message("slower with less RAM, this calls 'ramsortorder'")
#'   order(x, optimize="memory")
#' @name sort.integer64
NULL

#' @rdname sort.integer64
#' @export
sort.integer64 <- function(x,
                           decreasing=FALSE,
                           has.na=TRUE,
                           na.last=TRUE,
                           stable=TRUE,
                           optimize=c("time", "memory"),
                           VERBOSE=FALSE,
                           ...) {
  do.na.last <- is.na(na.last) || na.last
  cache_env <- cache(x)
  if (!is.null(cache_env$sort)) {
    if (do.na.last || decreasing) {
      na.count <- cache_env$na.count
      s <- double(length(x))
      .Call(C_r_ram_integer64_sortsrt,
        x = cache_env$sort,
        na_count = as.integer(na.count),
        na_last = as.logical(do.na.last),
        decreasing = as.logical(decreasing),
        s = s
      )
      setattr(s, "class", "integer64")
    } else {
      s <- cache_env$sort  # here we save copying at all
    }
  } else if (!is.null(cache_env$order)) {
    if (do.na.last || decreasing) {
      na.count <- cache_env$na.count
      s <- double(length(x))
      .Call(C_r_ram_integer64_sortsrt,
        x = x[cache_env$order],
        na_count = as.integer(na.count),
        na_last = as.logical(do.na.last),
        decreasing = as.logical(decreasing),
        s = s
      )
      setattr(s, "class", "integer64")
    } else {
      s <- x[cache_env$order]
    }
  } else {
    if (identical(cache_env$na.count, 0L))
      has.na <- FALSE
    s <- clone(x)
    na.count <- ramsort(
        s
    , has.na=has.na
    , na.last=do.na.last
    , decreasing=decreasing
    , stable=stable
    , optimize = optimize
    , VERBOSE = FALSE
    )
  }
  if (is.na(na.last) && na.count)
    length(s) <- length(s) - na.count
  s
}

#' @rdname sort.integer64
#' @export
order.integer64 <- function(...,
                            na.last=TRUE,
                            decreasing=FALSE,
                            has.na=TRUE,
                            stable=TRUE,
                            optimize=c("time", "memory"),
                            VERBOSE=FALSE) {
  do.na.last <- is.na(na.last) || na.last
    # COPY ON MODIFY is broken for reading from list(...)
    # because list(...) creates a copy of all ... and this invalidates our caches
    # therefore we go this sick workaround
    argsymbols <- as.list(substitute(list(...)))[-1L]
    argframe <- parent.frame()
    A <- function(i) eval(argsymbols[[i]], argframe)
    N <- length(argsymbols)
  if (N!=1L)
    stop("can only order one vector at the moment")
  x <- A(1L)
  cache_env <- cache(x)
  if (!is.null(cache_env$order)) {
    if (do.na.last || decreasing) {
        o <- integer(length(x))
        if (is.null(cache_env$sort)) {
          na.count <- cache_env$na.count
          .Call(C_r_ram_integer64_orderord,
            x = x,
            i = cache_env$order,
            na_count = as.integer(na.count),
            na_last = as.logical(do.na.last),
            decreasing = as.logical(decreasing),
            o = o
          )
        } else {
          na.count <- cache_env$na.count
          .Call(C_r_ram_integer64_sortorderord,
            x = cache_env$sort,
            i = cache_env$order,
            na_count = as.integer(na.count),
            na_last = as.logical(do.na.last),
            decreasing = as.logical(decreasing),
            o = o
          )
        }
      } else {
        o <- cache_env$order  # here we save copying at all
      }
  } else {
    if (identical(cache_env$na.count, 0L))
      has.na <- FALSE
    optimize <- match.arg(optimize)
    o <- seq_along(x)
    if (optimize == "time") {
        s <- clone(x)
        na.count <- ramsortorder(s, o
        , has.na=has.na
        , na.last=do.na.last
        , decreasing=decreasing
        , stable=stable
        , optimize = optimize
        , VERBOSE = FALSE
        )
    } else {
        na.count <- ramorder(x, o
        , has.na=has.na
        , na.last=do.na.last
        , decreasing=decreasing
        , stable=stable
        , optimize = optimize
        , VERBOSE = FALSE
        )
    }
  }
  if (is.na(na.last) && na.count)
    length(o) <- length(o) - na.count
  o
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/integer64.R
\name{as.data.frame.integer64}
\alias{as.data.frame.integer64}
\title{integer64: Coercing to data.frame column}
\usage{
\method{as.data.frame}{integer64}(x, row.names = NULL, optional = FALSE, ...)
}
\arguments{
\item{x}{an integer64 vector}

\item{row.names, optional, ...}{passed to NextMethod \code{\link[=as.data.frame]{as.data.frame()}} after removing the
'integer64' class attribute}
}
\value{
a one-column data.frame containing an integer64 vector
}
\description{
Coercing integer64 vector to data.frame.
}
\details{
'as.data.frame.integer64' is rather not intended to be called directly,
but it is required to allow integer64 as data.frame columns.
}
\note{
This is currently very slow -- any ideas for improvement?
}
\examples{
  as.data.frame(as.integer64(1:12))
  data.frame(a=1:12, b=as.integer64(1:12))
}
\seealso{
\code{\link[=cbind.integer64]{cbind.integer64()}} \code{\link[=integer64]{integer64()}}
}
\keyword{classes}
\keyword{manip}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sort64.R
\name{sort.integer64}
\alias{sort.integer64}
\alias{order.integer64}
\title{High-level intger64 methods for sorting and ordering}
\usage{
\method{sort}{integer64}(
  x,
  decreasing = FALSE,
  has.na = TRUE,
  na.last = TRUE,
  stable = TRUE,
  optimize = c("time", "memory"),
  VERBOSE = FALSE,
  ...
)

\method{order}{integer64}(
  ...,
  na.last = TRUE,
  decreasing = FALSE,
  has.na = TRUE,
  stable = TRUE,
  optimize = c("time", "memory"),
  VERBOSE = FALSE
)
}
\arguments{
\item{x}{a vector to be sorted by \code{\link[=ramsort.integer64]{ramsort.integer64()}} and
\code{\link[=ramsortorder.integer64]{ramsortorder.integer64()}}, i.e. the output of  \code{\link[=sort.integer64]{sort.integer64()}}}

\item{decreasing}{boolean scalar telling ramsort whether to sort
increasing or decreasing}

\item{has.na}{boolean scalar defining whether the input vector might
contain \code{NA}s. If we know we don't have NAs, this may speed-up. \emph{Note}
that you risk a crash if there are unexpected \code{NA}s with \code{has.na=FALSE}}

\item{na.last}{boolean scalar telling ramsort whether to sort \code{NA}s last
or first. \emph{Note} that 'boolean' means that there is no third option
\code{NA} as in \code{\link[=sort]{sort()}}}

\item{stable}{boolean scalar defining whether stable sorting is needed.
Allowing non-stable may speed-up.}

\item{optimize}{by default ramsort optimizes for 'time' which requires
more RAM, set to 'memory' to minimize RAM requirements and sacrifice speed}

\item{VERBOSE}{cat some info about chosen method}

\item{...}{further arguments, passed from generics, ignored in methods}
}
\value{
\code{sort} returns the sorted vector and \code{vector} returns the order positions.
}
\description{
Fast high-level methods for sorting and ordering. These are wrappers to
\code{\link[=ramsort.integer64]{ramsort.integer64()}} and friends and do not modify their arguments.
}
\details{
see \code{\link[=sort]{sort()}} and \code{\link[=order]{order()}}
}
\examples{
  x <- as.integer64(sample(c(rep(NA, 9), 1:9), 32, TRUE))
  x
  sort(x)
  message(
    "the following has default optimize='time' which is faster ",
    "but requires more RAM, this calls 'ramorder'"
  )
  order(x)
  message("slower with less RAM, this calls 'ramsortorder'")
  order(x, optimize="memory")
}
\seealso{
\code{\link[=sort.integer64]{sort()}}, \code{\link[=sortcache]{sortcache()}}
}
\keyword{manip}
\keyword{programming}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/highlevel64.R
\name{unique.integer64}
\alias{unique.integer64}
\title{Extract Unique Elements from integer64}
\usage{
\method{unique}{integer64}(
  x,
  incomparables = FALSE,
  order = c("original", "values", "any"),
  nunique = NULL,
  method = NULL,
  ...
)
}
\arguments{
\item{x}{a vector or a data frame or an array or \code{NULL}.}

\item{incomparables}{ignored}

\item{order}{The order in which unique values will be returned, see details}

\item{nunique}{NULL or the number of unique values (including NA). Providing
\code{nunique} can speed-up matching when \code{x} has no cache. Note that a wrong
`nunique`` can cause undefined behaviour up to a crash.}

\item{method}{NULL for automatic method selection or a suitable low-level
method, see details}

\item{...}{ignored}
}
\value{
For a vector, an object of the same type of \code{x}, but with only
one copy of each duplicated element.  No attributes are copied (so
the result has no names).
}
\description{
\code{unique} returns a vector like \code{x} but with duplicate elements/rows removed.
}
\details{
This function automatically chooses from several low-level functions
considering the size of \code{x} and the availability of a cache.

Suitable methods are
\itemize{
\item \code{\link{hashmapuni}} (simultaneously creating and using a hashmap)
\item \code{\link{hashuni}} (first creating a hashmap then using it)
\item \code{\link{sortuni}} (fast sorting for sorted order only)
\item \code{\link{sortorderuni}} (fast ordering for original order only)
\item \code{\link{orderuni}} (memory saving ordering).
}

The default \code{order="original"} returns unique values in the order of the
first appearance in \code{x} like in \code{\link[=unique]{unique()}}, this costs extra processing.
\code{order="values"} returns unique values in sorted order like in \code{\link[=table]{table()}},
this costs extra processing with the hash methods but comes for free.
\code{order="any"} returns unique values in undefined order, possibly faster.
For hash methods this will be a quasi random order, for sort methods this
will be sorted order.
}
\examples{
x <- as.integer64(sample(c(rep(NA, 9), 1:9), 32, TRUE))
unique(x)
unique(x, order="values")

stopifnot(identical(unique(x),  x[!duplicated(x)]))
stopifnot(identical(unique(x),  as.integer64(unique(as.integer(x)))))
stopifnot(identical(unique(x, order="values")
,  as.integer64(sort(unique(as.integer(x)), na.last=FALSE))))

}
\seealso{
\code{\link[=unique]{unique()}} for the generic, \code{\link[=unipos]{unipos()}} which gives the indices
of the unique elements and \code{\link[=table]{table()}} which gives frequencies
of the unique elements.
}
\keyword{logic}
\keyword{manip}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/integer64.R
\name{factor}
\alias{factor}
\alias{ordered}
\title{Factors}
\usage{
factor(
  x = character(),
  levels,
  labels = levels,
  exclude = NA,
  ordered = is.ordered(x),
  nmax = NA
)

ordered(x = character(), ...)
}
\arguments{
\item{x}{a vector of data, usually taking a small number of distinct
    values.}

\item{levels}{an optional vector of the unique values (as character strings)
    that \code{x} might have taken.  The default is the unique set of
    values taken by \code{\link[base]{as.character}(x)}, sorted into
    increasing order \emph{of \code{x}}.  Note that this set can be
    specified as smaller than \code{sort(unique(x))}.}

\item{labels}{\emph{either} an optional character vector of
    labels for the levels (in the same order as \code{levels} after
    removing those in \code{exclude}), \emph{or} a character string of
    length 1.  Duplicated values in \code{labels} can be used to map
    different values of \code{x} to the same factor level.}

\item{exclude}{a vector of values to be excluded when forming the
    set of levels.  This may be factor with the same level set as \code{x}
    or should be a \code{character}.}

\item{ordered}{logical flag to determine if the levels should be regarded
    as ordered (in the order given).}

\item{nmax}{an upper bound on the number of levels.}

\item{...}{(in \code{ordered(.)}): any of the above, apart from
    \code{ordered} itself.}
}
\value{
An object of class "factor" or "ordered".
}
\description{
The function \link{factor} is used to encode a vector as a factor.
}
\examples{
  x <- as.integer64(c(132724613L, -2143220989L, -1L, NA, 1L))
  factor(x)
  ordered(x)
}
\seealso{
\link[base:factor]{factor}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/highlevel64.R
\name{unipos}
\alias{unipos}
\alias{unipos.integer64}
\title{Extract Positions of Unique Elements}
\usage{
unipos(x, incomparables = FALSE, order = c("original", "values", "any"), ...)

\method{unipos}{integer64}(
  x,
  incomparables = FALSE,
  order = c("original", "values", "any"),
  nunique = NULL,
  method = NULL,
  ...
)
}
\arguments{
\item{x}{a vector or a data frame or an array or \code{NULL}.}

\item{incomparables}{ignored}

\item{order}{The order in which positions of unique values will be returned,
see details}

\item{...}{ignored}

\item{nunique}{NULL or the number of unique values (including NA). Providing
\code{nunique} can speed-up when \code{x} has no cache. Note that a wrong \code{nunique}
can cause undefined behaviour up to a crash.}

\item{method}{NULL for automatic method selection or a suitable low-level
method, see details}
}
\value{
an integer vector of positions
}
\description{
\code{unipos} returns the positions of those elements returned by \code{\link[=unique]{unique()}}.
}
\details{
This function automatically chooses from several low-level functions
considering the size of \code{x} and the availability of a cache.

Suitable methods are
\itemize{
\item \code{\link{hashmapupo}} (simultaneously creating and using a hashmap)
\item \code{\link{hashupo}} (first creating a hashmap then using it)
\item \code{\link{sortorderupo}} (fast ordering)
\item \code{\link{orderupo}} (memory saving ordering).
}

The default \code{order="original"} collects unique values in the order of
the first appearance in \code{x} like in \code{\link[=unique]{unique()}}, this costs extra processing.
\code{order="values"} collects unique values in sorted order like in \code{\link[=table]{table()}},
this costs extra processing with the hash methods but comes for free.
\code{order="any"} collects unique values in undefined order, possibly faster.
For hash methods this will be a quasi random order, for sort methods this
will be sorted order.
}
\examples{
x <- as.integer64(sample(c(rep(NA, 9), 1:9), 32, TRUE))
unipos(x)
unipos(x, order="values")

}
\seealso{
\code{\link[=unique.integer64]{unique.integer64()}} for unique values and \code{\link[=match.integer64]{match.integer64()}}
for general matching.
}
\keyword{logic}
\keyword{manip}

\name{as.data.frame.integer64}
\alias{as.data.frame.integer64}
\title{
   integer64: Coercing to data.frame column
}
\description{
  Coercing integer64 vector to data.frame.
}
\usage{
  \method{as.data.frame}{integer64}(x, \dots)
}
\arguments{
  \item{x}{ an integer64 vector }
  \item{\dots}{ passed to NextMethod \code{\link{as.data.frame}} after removing the 'integer64' class attribute }
}
\value{
  a one-column data.frame containing an integer64 vector
}
\details{
  'as.data.frame.integer64' is rather not intended to be called directly,
  but it is required to allow integer64 as data.frame columns.
}
\note{
  This is currently very slow -- any ideas for improvement?
}
\author{
Jens Oehlschlägel <Jens.Oehlschlaegel@truecluster.com>
}
\keyword{ classes }
\keyword{ manip }
\seealso{ 
  \code{\link{cbind.integer64}} \code{\link{integer64}}  %as.vector.integer64 removed as requested by the CRAN maintainer \code{\link{as.vector.integer64}} 
}
\examples{
  as.data.frame.integer64(as.integer64(1:12))
  data.frame(a=1:12, b=as.integer64(1:12))
}

\name{sort.integer64}
\alias{sort.integer64}
\alias{order.integer64}
\title{
   High-level intger64 methods for sorting and ordering
}
\description{
  Fast high-level methods for sorting and ordering. 
  These are wrappers to \code{\link{ramsort}} and friends and do not modify their arguments.
}
\usage{
\method{sort}{integer64}(x, decreasing = FALSE, has.na = TRUE, na.last = TRUE, stable = TRUE
, optimize = c("time", "memory"), VERBOSE = FALSE, \dots)
\method{order}{integer64}(\dots, na.last = TRUE, decreasing = FALSE, has.na = TRUE, stable = TRUE
, optimize = c("time", "memory"), VERBOSE = FALSE)
}
\arguments{
  \item{x}{ a vector to be sorted by \code{\link{ramsort}} and \code{\link{ramsortorder}}, i.e. the output of  \code{\link{sort}} }
  \item{has.na}{
boolean scalar defining whether the input vector might contain \code{NA}s. If we know we don't have NAs, this may speed-up.
\emph{Note} that you risk a crash if there are unexpected \code{NA}s with \code{has.na=FALSE}
}
  \item{na.last}{
boolean scalar telling ramsort whether to sort \code{NA}s last or first.
\emph{Note} that 'boolean' means that there is no third option \code{NA} as in \code{\link{sort}}
}
  \item{decreasing}{
boolean scalar telling ramsort whether to sort increasing or decreasing
}
  \item{stable}{
boolean scalar defining whether stable sorting is needed. Allowing non-stable may speed-up.
}
  \item{optimize}{
by default ramsort optimizes for 'time' which requires more RAM,
set to 'memory' to minimize RAM requirements and sacrifice speed
}
  \item{VERBOSE}{
  cat some info about chosen method
}
  \item{\dots}{ further arguments, passed from generics, ignored in methods }
}
\details{
 see \code{\link{sort}} and \code{\link{order}}
}
\value{
  \code{sort} returns the sorted vector and \code{vector} returns the order positions. 
}
\author{
Jens Oehlschlägel <Jens.Oehlschlaegel@truecluster.com>
}
\keyword{ programming }
\keyword{ manip }
\seealso{ \code{\link[=sort.integer64]{sort}}, \code{\link{sortcache}} }
\examples{
  x <- as.integer64(sample(c(rep(NA, 9), 1:9), 32, TRUE))
  x
  sort(x)
  message("the following has default optimize='time' which is faster but requires more RAM
, this calls 'ramorder'")
  order.integer64(x)
  message("slower with less RAM, this calls 'ramsortorder'")
  order.integer64(x, optimize="memory")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cache.R
\name{cache}
\alias{cache}
\alias{newcache}
\alias{jamcache}
\alias{setcache}
\alias{getcache}
\alias{remcache}
\alias{print.cache}
\title{Atomic Caching}
\usage{
newcache(x)

jamcache(x)

cache(x)

setcache(x, which, value)

getcache(x, which)

remcache(x)

\method{print}{cache}(x, all.names = FALSE, pattern, ...)
}
\arguments{
\item{x}{an integer64 vector (or a cache object in case of \code{print.cache})}

\item{which}{A character naming the object to be retrieved from the cache or to be stored in the cache}

\item{value}{An object to be stored in the cache}

\item{all.names, pattern}{passed to \code{\link[=ls]{ls()}} when listing the cache content}

\item{...}{ignored}
}
\value{
See details
}
\description{
Functions for caching results attached to atomic objects
}
\details{
A \code{cache} is an \code{\link{environment}} attached to an atomic object with the
\code{\link[=attr]{attribute}} name 'cache'. It contains at least a reference to the
atomic object that carries the cache. This is used when accessing
the cache to detect whether the object carrying the cache has been
modified meanwhile.
}
\section{Functions}{
\itemize{
\item \code{newcache()}: creates a new cache referencing  \code{x}

\item \code{jamcache()}: forces \code{x} to have a cache

\item \code{cache()}: returns the cache attached to \code{x} if it is not
found to be outdated

\item \code{setcache()}: assigns a value into the cache of \code{x}

\item \code{getcache()}: gets cache value 'which' from \code{x}

\item \code{remcache()}: removes the cache from \code{x}

}}
\examples{
  x = as.integer64(sample(c(rep(NA, 9), 1:9), 32, TRUE))
  y = x
  bit::still.identical(x, y)
  y[1] = NA
  bit::still.identical(x, y)
  mycache = newcache(x)
  ls(mycache)
  mycache
  rm(mycache)
  jamcache(x)
  cache(x)
  x[1] = NA
  cache(x)
  getcache(x, "abc")
  setcache(x, "abc", 1)
  getcache(x, "abc")
  remcache(x)
  cache(x)
}
\seealso{
\code{\link[bit:still.identical]{bit::still.identical()}} for testing whether to symbols point to the same RAM.

Functions that get and set small cache-content automatically when a cache is present:
\code{\link[bit:Metadata]{bit::na.count()}}, \code{\link[bit:Metadata]{bit::nvalid()}}, \code{\link[bit:Metadata]{bit::is.sorted()}}, \code{\link[bit:Metadata]{bit::nunique()}} and
\code{\link[bit:Metadata]{bit::nties()}}

Setting big caches with a relevant memory footprint requires a conscious decision
of the user: \code{\link{hashcache}}, \code{\link{sortcache}}, \code{\link{ordercache}}, \code{\link{sortordercache}}

Functions that use big caches: \code{\link[=match.integer64]{match.integer64()}}, \code{\link[=match.integer64]{\%in\%.integer64}},
\code{\link[=duplicated.integer64]{duplicated.integer64()}}, \code{\link[=unique.integer64]{unique.integer64()}}, \code{\link[=unipos]{unipos()}}, \code{\link[=table]{table()}},
\code{\link[=keypos]{keypos()}}, \code{\link[=tiepos]{tiepos()}}, \code{\link[=rank.integer64]{rank.integer64()}}, \code{\link[=prank]{prank()}}, \code{\link[=qtile]{qtile()}},
\code{\link[=quantile.integer64]{quantile.integer64()}}, \code{\link[=median.integer64]{median.integer64()}}, and \code{\link[=summary.integer64]{summary.integer64()}}
}
\keyword{environment}

\name{unique.integer64}
\alias{unique.integer64}
\title{Extract Unique Elements from integer64}
\description{
  \code{unique} returns a vector like \code{x} but with duplicate elements/rows removed.
}
\usage{
\method{unique}{integer64}(x, incomparables = FALSE, order = c("original","values","any")
, nunique = NULL, method = NULL, \dots)
}
\arguments{
  \item{x}{a vector or a data frame or an array or \code{NULL}.}
  \item{incomparables}{ignored}
  \item{order}{The order in which unique values will be returned, see details}
  \item{nunique}{
	NULL or the number of unique values (including NA). Providing \code{nunique} can speed-up matching when \code{x} has no cache. Note that a wrong nunique can cause undefined behaviour up to a crash.
}
  \item{method}{
	NULL for automatic method selection or a suitable low-level method, see details
}
  \item{\dots}{ignored}
}
\details{
  This function automatically chooses from several low-level functions considering the size of \code{x} and the availability of a cache. 
  Suitable methods are \code{\link{hashmapuni}} (simultaneously creating and using a hashmap)
, \code{\link{hashuni}} (first creating a hashmap then using it)
, \code{\link{sortuni}} (fast sorting for sorted order only)
, \code{\link{sortorderuni}} (fast ordering for original order only) 
and \code{\link{orderuni}} (memory saving ordering).
\cr
The default \code{order="original"} returns unique values in the order of the first appearance in \code{x} like in \code{\link{unique}}, this costs extra processing. 
\code{order="values"} returns unique values in sorted order like in \code{\link{table}}, this costs extra processing with the hash methods but comes for free. 
\code{order="any"} returns unique values in undefined order, possibly faster. For hash methods this will be a quasi random order, for sort methods this will be sorted order.
}
\value{
  For a vector, an object of the same type of \code{x}, but with only
  one copy of each duplicated element.  No attributes are copied (so
  the result has no names).
}
\author{
	Jens Oehlschlägel <Jens.Oehlschlaegel@truecluster.com>
}
\seealso{
  \code{\link{unique}} for the generic, \code{\link{unipos}} which gives the indices of the unique
  elements and \code{\link{table.integer64}} which gives frequencies of the unique elements.
}
\examples{
x <- as.integer64(sample(c(rep(NA, 9), 1:9), 32, TRUE))
unique(x)
unique(x, order="values")

stopifnot(identical(unique(x),  x[!duplicated(x)]))
stopifnot(identical(unique(x),  as.integer64(unique(as.integer(x)))))
stopifnot(identical(unique(x, order="values")
,  as.integer64(sort(unique(as.integer(x)), na.last=FALSE))))
}
\keyword{manip}
\keyword{logic}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/setops64.R
\name{union}
\alias{union}
\alias{intersect}
\alias{setequal}
\alias{setdiff}
\alias{is.element}
\title{Set Operations}
\usage{
union(x, y)

intersect(x, y)

setequal(x, y)

setdiff(x, y)

is.element(el, set)
}
\arguments{
\item{x, y, el, set}{vectors (of the same mode) containing a sequence
    of items (conceptually) with no duplicated values.}
}
\value{
For union, a vector of a common mode or class.

For intersect, a vector of a common mode or class, or NULL if x or y is NULL.

For setdiff, a vector of the same mode or class as x.

A logical scalar for setequal and a logical of the same length as x for is.element.
}
\description{
Performs set union, intersection, (asymmetric!) difference, equality and membership on two vectors. As soon as an integer64 vector is involved, the operations are performed using integer64 semantics. Otherwise the \code{base} package functions are called.
}
\examples{
x <- as.integer64(1:5)
y <- c(1L, 3L, 5L, 7L)
union(x, y)
intersect(x, y)
setdiff(x, y)
setequal(x, y)
is.element(x, y)

}
\seealso{
\link[base:sets]{base::union}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/integer64.R
\name{cumsum.integer64}
\alias{cumsum.integer64}
\alias{diff.integer64}
\alias{cummin.integer64}
\alias{cummax.integer64}
\alias{cumprod.integer64}
\title{Cumulative Sums, Products, Extremes and lagged differences}
\usage{
\method{diff}{integer64}(x, lag = 1L, differences = 1L, ...)

\method{cummin}{integer64}(x)

\method{cummax}{integer64}(x)

\method{cumsum}{integer64}(x)

\method{cumprod}{integer64}(x)
}
\arguments{
\item{x}{an atomic vector of class 'integer64'}

\item{lag}{see \code{\link[=diff]{diff()}}}

\item{differences}{see \code{\link[=diff]{diff()}}}

\item{...}{ignored}
}
\value{
\code{\link[=cummin]{cummin()}}, \code{\link[=cummax]{cummax()}} , \code{\link[=cumsum]{cumsum()}} and \code{\link[=cumprod]{cumprod()}}
return a integer64 vector of the same length as their input

\code{\link[=diff]{diff()}} returns a integer64 vector shorter by \code{lag*differences} elements
}
\description{
Cumulative Sums, Products, Extremes and lagged differences
}
\examples{
  cumsum(rep(as.integer64(1), 12))
  diff(as.integer64(c(0, 1:12)))
  cumsum(as.integer64(c(0, 1:12)))
  diff(cumsum(as.integer64(c(0, 0, 1:12))), differences=2)
}
\seealso{
\code{\link[=sum.integer64]{sum.integer64()}} \code{\link[=integer64]{integer64()}}
}
\keyword{classes}
\keyword{manip}

\name{unipos}
\alias{unipos}
\alias{unipos.integer64}
\title{Extract Positions of Unique Elements}
\description{
  \code{unipos} returns the positions of those elements returned by \code{\link{unique}}.
}
\usage{
unipos(x, incomparables = FALSE, order = c("original","values","any"), \dots)
\method{unipos}{integer64}(x, incomparables = FALSE, order = c("original","values","any")
, nunique = NULL, method = NULL, \dots)
}
\arguments{
  \item{x}{a vector or a data frame or an array or \code{NULL}.}
  \item{incomparables}{ignored}
  \item{order}{The order in which positions of unique values will be returned, see details}
  \item{nunique}{
	NULL or the number of unique values (including NA). Providing \code{nunique} can speed-up when \code{x} has no cache. Note that a wrong nunique can cause undefined behaviour up to a crash.
}
  \item{method}{
	NULL for automatic method selection or a suitable low-level method, see details
}
  \item{\dots}{ignored}
}
\details{
  This function automatically chooses from several low-level functions considering the size of \code{x} and the availability of a cache. 
  Suitable methods are \code{\link{hashmapupo}} (simultaneously creating and using a hashmap)
, \code{\link{hashupo}} (first creating a hashmap then using it)
, \code{\link{sortorderupo}} (fast ordering) 
and \code{\link{orderupo}} (memory saving ordering).
\cr
The default \code{order="original"} collects unique values in the order of the first appearance in \code{x} like in \code{\link{unique}}, this costs extra processing. 
\code{order="values"} collects unique values in sorted order like in \code{\link{table}}, this costs extra processing with the hash methods but comes for free. 
\code{order="any"} collects unique values in undefined order, possibly faster. For hash methods this will be a quasi random order, for sort methods this will be sorted order.
}
\value{
  an integer vector of positions
}
\author{
	Jens Oehlschlägel <Jens.Oehlschlaegel@truecluster.com>
}
\seealso{
  \code{\link{unique.integer64}} for unique values and \code{\link{match.integer64}} for general matching.
}
\examples{
x <- as.integer64(sample(c(rep(NA, 9), 1:9), 32, TRUE))
unipos(x)
unipos(x, order="values")

stopifnot(identical(unipos(x),  (1:length(x))[!duplicated(x)]))
stopifnot(identical(unipos(x),  match.integer64(unique(x), x)))
stopifnot(identical(unipos(x, order="values"),  match.integer64(unique(x, order="values"), x)))
stopifnot(identical(unique(x),  x[unipos(x)]))
stopifnot(identical(unique(x, order="values"),  x[unipos(x, order="values")]))
}
\keyword{manip}
\keyword{logic}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{optimizer64.data}
\alias{optimizer64.data}
\title{Results of performance measurement on a Core i7 Lenovo T410 8 GB RAM under Windows 7 64bit}
\format{
The format is:

\if{html}{\out{<div class="sourceCode">}}\preformatted{List of 16
 $ : num [1:9, 1:3] 0 0 1.63 0.00114 2.44 ...
  ..- attr(*, "dimnames")=List of 2
  .. ..$ : chr [1:9] "match" "match.64" "hashpos" "hashrev" ...
  .. ..$ : chr [1:3] "prep" "both" "use"
 $ : num [1:10, 1:3] 0 0 0 1.62 0.00114 ...
  ..- attr(*, "dimnames")=List of 2
  .. ..$ : chr [1:10] "\%in\%" "match.64" "\%in\%.64" "hashfin" ...
  .. ..$ : chr [1:3] "prep" "both" "use"
 $ : num [1:10, 1:3] 0 0 0.00105 0.00313 0.00313 ...
  ..- attr(*, "dimnames")=List of 2
  .. ..$ : chr [1:10] "duplicated" "duplicated.64" "hashdup" "sortorderdup1" ...
  .. ..$ : chr [1:3] "prep" "both" "use"
 $ : num [1:15, 1:3] 0 0 0 0.00104 0.00104 ...
  ..- attr(*, "dimnames")=List of 2
  .. ..$ : chr [1:15] "unique" "unique.64" "hashmapuni" "hashuni" ...
  .. ..$ : chr [1:3] "prep" "both" "use"
 $ : num [1:14, 1:3] 0 0 0 0.000992 0.000992 ...
  ..- attr(*, "dimnames")=List of 2
  .. ..$ : chr [1:14] "unique" "unipos.64" "hashmapupo" "hashupo" ...
  .. ..$ : chr [1:3] "prep" "both" "use"
 $ : num [1:13, 1:3] 0 0 0 0 0.000419 ...
  ..- attr(*, "dimnames")=List of 2
  .. ..$ : chr [1:13] "tabulate" "table" "table.64" "hashmaptab" ...
  .. ..$ : chr [1:3] "prep" "both" "use"
 $ : num [1:7, 1:3] 0 0 0 0.00236 0.00714 ...
  ..- attr(*, "dimnames")=List of 2
  .. ..$ : chr [1:7] "rank" "rank.keep" "rank.64" "sortorderrnk" ...
  .. ..$ : chr [1:3] "prep" "both" "use"
 $ : num [1:6, 1:3] 0 0 0.00189 0.00714 0 ...
  ..- attr(*, "dimnames")=List of 2
  .. ..$ : chr [1:6] "quantile" "quantile.64" "sortqtl" "orderqtl" ...
  .. ..$ : chr [1:3] "prep" "both" "use"
 $ : num [1:9, 1:3] 0 0 0.00105 1.17 0 ...
  ..- attr(*, "dimnames")=List of 2
  .. ..$ : chr [1:9] "match" "match.64" "hashpos" "hashrev" ...
  .. ..$ : chr [1:3] "prep" "both" "use"
 $ : num [1:10, 1:3] 0 0 0 0.00104 1.18 ...
  ..- attr(*, "dimnames")=List of 2
  .. ..$ : chr [1:10] "\%in\%" "match.64" "\%in\%.64" "hashfin" ...
  .. ..$ : chr [1:3] "prep" "both" "use"
 $ : num [1:10, 1:3] 0 0 1.64 2.48 2.48 ...
  ..- attr(*, "dimnames")=List of 2
  .. ..$ : chr [1:10] "duplicated" "duplicated.64" "hashdup" "sortorderdup1" ...
  .. ..$ : chr [1:3] "prep" "both" "use"
 $ : num [1:15, 1:3] 0 0 0 1.64 1.64 ...
  ..- attr(*, "dimnames")=List of 2
  .. ..$ : chr [1:15] "unique" "unique.64" "hashmapuni" "hashuni" ...
  .. ..$ : chr [1:3] "prep" "both" "use"
 $ : num [1:14, 1:3] 0 0 0 1.62 1.62 ...
  ..- attr(*, "dimnames")=List of 2
  .. ..$ : chr [1:14] "unique" "unipos.64" "hashmapupo" "hashupo" ...
  .. ..$ : chr [1:3] "prep" "both" "use"
 $ : num [1:13, 1:3] 0 0 0 0 0.32 ...
  ..- attr(*, "dimnames")=List of 2
  .. ..$ : chr [1:13] "tabulate" "table" "table.64" "hashmaptab" ...
  .. ..$ : chr [1:3] "prep" "both" "use"
 $ : num [1:7, 1:3] 0 0 0 2.96 10.69 ...
  ..- attr(*, "dimnames")=List of 2
  .. ..$ : chr [1:7] "rank" "rank.keep" "rank.64" "sortorderrnk" ...
  .. ..$ : chr [1:3] "prep" "both" "use"
 $ : num [1:6, 1:3] 0 0 1.62 10.61 0 ...
  ..- attr(*, "dimnames")=List of 2
  .. ..$ : chr [1:6] "quantile" "quantile.64" "sortqtl" "orderqtl" ...
  .. ..$ : chr [1:3] "prep" "both" "use"
 - attr(*, "dim")= int [1:2] 8 2
 - attr(*, "dimnames")=List of 2
  ..$ : chr [1:8] "match" "\%in\%" "duplicated" "unique" ...
  ..$ : chr [1:2] "65536" "33554432"
}\if{html}{\out{</div>}}
}
\usage{
data(optimizer64.data)
}
\description{
These are the results of calling \code{\link[=optimizer64]{optimizer64()}}
}
\examples{
data(optimizer64.data)
print(optimizer64.data)
oldpar <- par(no.readonly = TRUE)
par(mfrow=c(2,1))
par(cex=0.7)
for (i in 1:nrow(optimizer64.data)) {
 for (j in 1:2) {
   tim <- optimizer64.data[[i,j]]
  barplot(t(tim))
  if (rownames(optimizer64.data)[i]=="match")
   title(paste("match", colnames(optimizer64.data)[j], "in", colnames(optimizer64.data)[3-j]))
  else if (rownames(optimizer64.data)[i]=="\%in\%")
   title(paste(colnames(optimizer64.data)[j], "\%in\%", colnames(optimizer64.data)[3-j]))
  else
   title(paste(rownames(optimizer64.data)[i], colnames(optimizer64.data)[j]))
 }
}
par(mfrow=c(1,1))

}
\keyword{datasets}

\name{cache}
\alias{cache}
\alias{newcache}
\alias{jamcache}
\alias{setcache}
\alias{getcache}
\alias{remcache}
\alias{print.cache}
\title{
	Atomic Caching
}
\description{
	Functions for caching results attached to atomic objects
}
\usage{
newcache(x)
jamcache(x)
cache(x)
setcache(x, which, value)
getcache(x, which)
remcache(x)
\method{print}{cache}(x, all.names = FALSE, pattern, \dots)
}
\arguments{
  \item{x}{
  an integer64 vector (or a cache object in case of \code{print.cache})
}
  \item{which}{
  A character naming the object to be retrieved from the cache or to be stored in the cache
}
  \item{value}{
  An object to be stored in the cache 
}
  \item{all.names}{
  passed to \code{\link{ls}} when listing the cache content
}
  \item{pattern}{
  passed to \code{\link{ls}} when listing the cache content
}
  \item{\dots}{
	ignored
}
}
\details{
	A \code{cache} is an \code{link{environment}} attached to an atomic object with the \code{link{attrib}} name 'cache'. 
	It contains at least a reference to the atomic object that carries the cache. 
	This is used when accessing the cache to detect whether the object carrying the cache has been modified meanwhile.
	Function \code{newcache(x)} creates a new cache referencing  \code{x} \cr
	Function \code{jamcache(x)} forces \code{x} to have a cache \cr
	Function \code{cache(x)} returns the cache attached to \code{x} if it is not found to be outdated \cr
	Function \code{setcache(x, which, value)} assigns a value into the cache of \code{x} \cr
	Function \code{getcache(x, which)} gets cache value 'which' from \code{x} \cr
	Function \code{remcache} removes the cache from \code{x} \cr
}
\value{
	see details
}
\author{
Jens Oehlschlägel <Jens.Oehlschlaegel@truecluster.com>
}
\seealso{
 \code{\link{still.identical}} for testing whether to symbols point to the same RAM. \cr
	Functions that get and set small cache-content automatically when a cache is present: \code{\link{na.count}}, \code{\link{nvalid}}, \code{\link{is.sorted}}, \code{\link{nunique}} and \code{\link{nties}} \cr
	Setting big caches with a relevant memory footprint requires a conscious decision of the user: \code{\link{hashcache}}, \code{\link{sortcache}}, \code{\link{ordercache}} and \code{\link{sortordercache}} \cr
	Functions that use big caches: \code{\link{match.integer64}}, \code{\link{\%in\%.integer64}}, \code{\link{duplicated.integer64}}, \code{\link{unique.integer64}}, \code{\link{unipos}}, \code{\link{table.integer64}}, \code{\link{as.factor.integer64}}, \code{\link{as.ordered.integer64}}, \code{\link{keypos}}, \code{\link{tiepos}}, \code{\link{rank.integer64}}, \code{\link{prank}}, \code{\link{qtile}}, \code{\link{quantile.integer64}}, \code{\link{median.integer64}} and \code{\link{summary.integer64}} \cr
}
\examples{
	x <- as.integer64(sample(c(rep(NA, 9), 1:9), 32, TRUE))
	y <- x
	still.identical(x,y)
	y[1] <- NA
	still.identical(x,y)
	mycache <- newcache(x)
	ls(mycache)
	mycache
	rm(mycache)
	jamcache(x)
	cache(x)
	x[1] <- NA
	cache(x)
	getcache(x, "abc")
	setcache(x, "abc", 1)
	getcache(x, "abc")
	remcache(x)
	cache(x)
}
\keyword{ environment }

\name{cumsum.integer64}
\alias{cummin.integer64}
\alias{cummax.integer64}
\alias{cumsum.integer64}
\alias{cumprod.integer64}
\alias{diff.integer64}
\title{
   Cumulative Sums, Products, Extremes and lagged differences
}
\description{
  Cumulative Sums, Products, Extremes and lagged differences
}
\usage{
\method{cummin}{integer64}(x)
\method{cummax}{integer64}(x)
\method{cumsum}{integer64}(x)
\method{cumprod}{integer64}(x)
\method{diff}{integer64}(x, lag = 1L, differences = 1L, \dots)
}
\arguments{
  \item{x}{ an atomic vector of class 'integer64'}
  \item{lag}{ see \code{\link{diff}} }
  \item{differences}{ see \code{\link{diff}} }
  \item{\dots}{ ignored }
}
\value{
  \code{\link{cummin}}, \code{\link{cummax}} , \code{\link{cumsum}} and \code{\link{cumprod}} 
     return a integer64 vector of the same length as their input\cr
  \code{\link{diff}} returns a integer64 vector shorter by \code{lag*differences} elements \cr
}
\author{
Jens Oehlschlägel <Jens.Oehlschlaegel@truecluster.com>
}
\keyword{ classes }
\keyword{ manip }
\seealso{ \code{\link{sum.integer64}} \code{\link{integer64}}  }
\examples{
  cumsum(rep(as.integer64(1), 12))
  diff(as.integer64(c(0,1:12)))
  cumsum(as.integer64(c(0, 1:12)))
  diff(cumsum(as.integer64(c(0,0,1:12))), differences=2)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cache.R
\name{hashcache}
\alias{hashcache}
\alias{sortcache}
\alias{sortordercache}
\alias{ordercache}
\title{Big caching of hashing, sorting, ordering}
\usage{
hashcache(x, nunique = NULL, ...)

sortcache(x, has.na = NULL, na.last = FALSE)

sortordercache(x, has.na = NULL, stable = NULL, na.last = FALSE)

ordercache(x, has.na = NULL, stable = NULL, optimize = "time", na.last = FALSE)
}
\arguments{
\item{x}{an atomic vector (note that currently only integer64 is supported)}

\item{nunique}{giving \emph{correct} number of unique elements can help reducing
the size of the hashmap}

\item{...}{passed to \code{\link[=hashmap]{hashmap()}}}

\item{has.na}{boolean scalar defining whether the input vector might contain
\code{NA}s. If we know we don't have \code{NA}s, this may speed-up. \emph{Note} that you
risk a crash if there are unexpected \code{NA}s with \code{has.na=FALSE}.}

\item{na.last}{boolean scalar defining whether NA should be last.}

\item{stable}{boolean scalar defining whether stable sorting is needed. Allowing
non-stable may speed-up.}

\item{optimize}{by default ramsort optimizes for 'time' which requires more RAM,
set to 'memory' to minimize RAM requirements and sacrifice speed.}
}
\value{
\code{x} with a \code{\link[=cache]{cache()}} that contains the result of the expensive operations,
possible together with small derived information (such as \code{\link[=nunique.integer64]{nunique.integer64()}})
and previously cached results.
}
\description{
Functions to create cache that accelerates many operations
}
\details{
The result of relative expensive operations \code{\link[=hashmap]{hashmap()}}, \code{\link[bit:Sorting]{bit::ramsort()}},
\code{\link[bit:Sorting]{bit::ramsortorder()}}, and \code{\link[bit:Sorting]{bit::ramorder()}} can be stored in a cache in
order to avoid multiple excutions. Unless in very specific situations, the
recommended method is \code{hashsortorder} only.
}
\note{
Note that we consider storing the big results from sorting and/or ordering
as a relevant side-effect, and therefore storing them in the cache should
require a conscious decision of the user.
}
\examples{
  x = as.integer64(sample(c(rep(NA, 9), 1:9), 32, TRUE))
  sortordercache(x)

}
\seealso{
\code{\link[=cache]{cache()}} for caching functions and \code{\link[=nunique.integer64]{nunique.integer64()}} for methods benefiting
from small caches
}
\keyword{environment}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/integer64.R
\name{identical.integer64}
\alias{identical.integer64}
\title{Identity function for class 'integer64'}
\usage{
identical.integer64(
  x,
  y,
  num.eq = FALSE,
  single.NA = FALSE,
  attrib.as.set = TRUE,
  ignore.bytecode = TRUE,
  ignore.environment = FALSE,
  ignore.srcref = TRUE,
  ...
)
}
\arguments{
\item{x, y}{Atomic vector of class 'integer64'}

\item{num.eq, single.NA, attrib.as.set, ignore.bytecode, ignore.environment, ignore.srcref}{See \code{\link[=identical]{identical()}}.}

\item{...}{Passed on to \code{identical()}. Only \verb{extptr.as.ref=} is available as of R 4.4.1,
and then only for versions of R >= 4.2.0.}
}
\value{
A single logical value, \code{TRUE} or \code{FALSE}, never \code{NA} and never
anything other than a single value.
}
\description{
This will discover any deviation between objects containing integer64 vectors.
}
\details{
This is simply a wrapper to \code{\link[=identical]{identical()}} with default arguments
\verb{num.eq = FALSE, single.NA = FALSE}.
}
\examples{
  i64 <- as.double(NA); class(i64) <- "integer64"
  identical(i64-1, i64+1)
  identical.integer64(i64-1, i64+1)
}
\seealso{
\code{\link{==.integer64}} \code{\link[=identical]{identical()}} \code{\link[=integer64]{integer64()}}
}
\keyword{classes}
\keyword{manip}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/integer64.R
\name{extract.replace.integer64}
\alias{extract.replace.integer64}
\alias{[.integer64}
\alias{[<-.integer64}
\alias{[[.integer64}
\alias{[[<-.integer64}
\title{Extract or Replace Parts of an integer64 vector}
\usage{
\method{[}{integer64}(x, i, j, ..., drop = TRUE)

\method{[}{integer64}(x, ...) <- value

\method{[[}{integer64}(x, ...)

\method{[[}{integer64}(x, ...) <- value
}
\arguments{
\item{x}{an atomic vector}

\item{i, j}{indices specifying elements to extract}

\item{...}{further arguments to the \code{\link[=NextMethod]{NextMethod()}}}

\item{drop}{relevant for matrices and arrays. If TRUE the result is coerced to the lowest possible dimension.}

\item{value}{an atomic vector with values to be assigned}
}
\value{
A vector, matrix, array or scalar of class 'integer64'
}
\description{
Methods to extract and replace parts of an integer64 vector.
}
\note{
You should not subscript non-existing elements and not use \code{NA}s as subscripts.
The current implementation returns \code{9218868437227407266} instead of \code{NA}.
}
\examples{
  as.integer64(1:12)[1:3]
  x <- matrix(as.integer64(1:12), nrow = 3L)
  x
  x[]
  x[, 2:3]
}
\seealso{
\code{\link[base:Extract]{[}} \code{\link[=integer64]{integer64()}}
}
\keyword{classes}
\keyword{manip}

\name{hashcache}
\alias{hashcache}
\alias{sortcache}
\alias{sortordercache}
\alias{ordercache}
\title{
		Big caching of hashing, sorting, ordering
}
\description{
	Functions to create cache that accelerates many operations
}
\usage{
hashcache(x, nunique=NULL, \dots)
sortcache(x, has.na = NULL)
sortordercache(x, has.na = NULL, stable = NULL)
ordercache(x, has.na = NULL, stable = NULL, optimize = "time")
}
\arguments{
  \item{x}{
		an atomic vector (note that currently only integer64 is supported)
}
  \item{nunique}{ giving \emph{correct} number of unique elements can help reducing the size of the hashmap }
  \item{has.na}{
boolean scalar defining whether the input vector might contain \code{NA}s. If we know we don't have NAs, this may speed-up.
\emph{Note} that you risk a crash if there are unexpected \code{NA}s with \code{has.na=FALSE}
}
  \item{stable}{
boolean scalar defining whether stable sorting is needed. Allowing non-stable may speed-up.
}
  \item{optimize}{
by default ramsort optimizes for 'time' which requires more RAM,
set to 'memory' to minimize RAM requirements and sacrifice speed
}
  \item{\dots}{
		passed to \code{\link{hashmap}}
}
}
\details{
	The result of relative expensive operations \code{\link{hashmap}}, \code{\link{ramsort}}, \code{\link{ramsortorder}} and \code{\link{ramorder}} can be stored in a cache in order to avoid multiple excutions. Unless in very specific situations, the recommended method is \code{hashsortorder} only.
}
\note{
  Note that we consider storing the big results from sorting and/or ordering as a relevant side-effect, 
and therefore storing them in the cache should require a conscious decision of the user.
}
\value{
	\code{x} with a \code{\link{cache}} that contains the result of the expensive operations, possible together with small derived information (such as \code{\link{nunique.integer64}}) and previously cached results.
}
\author{
Jens Oehlschlägel <Jens.Oehlschlaegel@truecluster.com>
}
\seealso{
	\code{\link{cache}} for caching functions and \code{\link{nunique}} for methods bennefitting from small caches
}
\examples{
	x <- as.integer64(sample(c(rep(NA, 9), 1:9), 32, TRUE))
 sortordercache(x)
}
\keyword{ environment }

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cache.R
\name{is.sorted.integer64}
\alias{is.sorted.integer64}
\alias{na.count.integer64}
\alias{nvalid.integer64}
\alias{nunique.integer64}
\alias{nties.integer64}
\title{Small cache access methods}
\usage{
\method{na.count}{integer64}(x, ...)

\method{nvalid}{integer64}(x, ...)

\method{is.sorted}{integer64}(x, ...)

\method{nunique}{integer64}(x, ...)

\method{nties}{integer64}(x, ...)
}
\arguments{
\item{x}{some object}

\item{...}{ignored}
}
\value{
\code{is.sorted} returns a logical scalar, the other methods return an integer scalar.
}
\description{
These methods are packaged here for methods in packages \code{bit64} and \code{ff}.
}
\details{
All these functions benefit from a \code{\link[=sortcache]{sortcache()}}, \code{\link[=ordercache]{ordercache()}} or
\code{\link[=sortordercache]{sortordercache()}}. \code{na.count()}, \code{nvalid()} and \code{nunique()} also
benefit from a \code{\link[=hashcache]{hashcache()}}.
}
\section{Functions}{
\itemize{
\item \code{na.count(integer64)}: returns the number of \code{NA}s

\item \code{nvalid(integer64)}: returns the number of valid data points,
usually \code{\link[=length]{length()}} minus \code{na.count}.

\item \code{is.sorted(integer64)}: checks for sortedness of \code{x} (NAs sorted first)

\item \code{nunique(integer64)}: returns the number of unique values

\item \code{nties(integer64)}: returns the number of tied values.

}}
\note{
If a \code{\link[=cache]{cache()}} exists but the desired value is not cached, then these
functions will store their result in the cache. We do not consider this
a relevant side-effect, since these small cache results do not have a
relevant memory footprint.
}
\examples{
 x = as.integer64(sample(c(rep(NA, 9), 1:9), 32, TRUE))
 length(x)
 bit::na.count(x)
 bit::nvalid(x)
 bit::nunique(x)
 bit::nties(x)
 table(x)
 x

}
\seealso{
\code{\link[=cache]{cache()}} for caching functions and \code{\link[=sortordercache]{sortordercache()}} for functions creating big caches
}
\keyword{environment}
\keyword{methods}

\name{identical.integer64}
\alias{identical.integer64}
\title{
   Identity function for class 'integer64'
}
\description{
  This will discover any deviation between objects containing integer64 vectors. 
}
\usage{
 identical.integer64(x, y, num.eq = FALSE, single.NA = FALSE
, attrib.as.set = TRUE, ignore.bytecode = TRUE)
}
\arguments{
  \item{x}{ atomic vector of class 'integer64' }
  \item{y}{ atomic vector of class 'integer64' }
  \item{num.eq}{ see \code{\link{identical}} }
  \item{single.NA}{ see \code{\link{identical}} }
  \item{attrib.as.set}{ see \code{\link{identical}} }
  \item{ignore.bytecode}{ see \code{\link{identical}} }
}
\details{
  This is simply a wrapper to \code{\link{identical}} with default arguments \code{num.eq = FALSE, single.NA = FALSE}.
}
\value{
  A single logical value, \code{TRUE} or \code{FALSE}, never \code{NA} and never anything other than a single value. 
}
\author{
Jens Oehlschlägel <Jens.Oehlschlaegel@truecluster.com>
}
\keyword{ classes }
\keyword{ manip }
\seealso{ \code{\link{==.integer64}} \code{\link{identical}} \code{\link{integer64}}  }
\examples{
  i64 <- as.double(NA); class(i64) <- "integer64"
  identical(i64-1, i64+1)
  identical.integer64(i64-1, i64+1)
}

\name{extract.replace.integer64}
\alias{[.integer64}
\alias{[[.integer64}
\alias{[[<-.integer64}
\alias{[<-.integer64}
\title{
   Extract or Replace Parts of an integer64 vector
}
\description{
  Methods to extract and replace parts of an integer64 vector.
}
\usage{
 \method{[}{integer64}(x, i, \dots)
 \method{[}{integer64}(x, \dots) <- value 
 \method{[[}{integer64}(x, \dots)
 \method{[[}{integer64}(x, \dots) <- value
}
\arguments{
  \item{x}{ an atomic vector }
  \item{i}{ indices specifying elements to extract }
  \item{value}{ an atomic vector with values to be assigned }
  \item{\dots}{ further arguments to the \code{\link{NextMethod}} }
}
\note{
  You should not subscript non-existing elements and not use \code{NA}s as subscripts.
  The current implementation returns \code{9218868437227407266} instead of \code{NA}.
}
\value{
  A vector or scalar of class 'integer64'
}
\author{
Jens Oehlschlägel <Jens.Oehlschlaegel@truecluster.com>
}
\keyword{ classes }
\keyword{ manip }
\seealso{ \code{\link{[}} \code{\link{integer64}}  }
\examples{
  as.integer64(1:12)[1:3]
  x <- as.integer64(1:12)
  dim(x) <- c(3,4)
  x
  x[]
  x[,2:3]
  \dontshow{
    r <- c(runif64(1e3, lim.integer64()[1], lim.integer64()[2]), NA, -2:2)
    stopifnot(identical(r, as.integer64(as.bitstring(r))))
  }
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hash64.R
\name{runif64}
\alias{runif64}
\title{integer64: random numbers}
\usage{
runif64(
  n,
  min = lim.integer64()[1L],
  max = lim.integer64()[2L],
  replace = TRUE
)
}
\arguments{
\item{n}{length of return vector}

\item{min}{lower inclusive bound for random numbers}

\item{max}{upper inclusive bound for random numbers}

\item{replace}{set to FALSE for sampleing from a finite pool, see \code{\link[=sample]{sample()}}}
}
\value{
a integer64 vector
}
\description{
Create uniform random 64-bit integers within defined range
}
\details{
For each random integer we call R's internal C interface \code{unif_rand()} twice.
Each call is mapped to 2^32 unsigned integers. The two 32-bit patterns are
concatenated to form the new integer64. This process is repeated until the
result is not a \code{NA_INTEGER64_}.
}
\examples{
  runif64(12)
  runif64(12, -16, 16)
  runif64(12, 0, as.integer64(2^60)-1)  # not 2^60-1 !
  var(runif(1e4))
  var(as.double(runif64(1e4, 0, 2^40))/2^40)  # ~ = 1/12 = .08333

  table(sample(16, replace=FALSE))
  table(runif64(16, 1, 16, replace=FALSE))
  table(sample(16, replace=TRUE))
  table(runif64(16, 1, 16, replace=TRUE))

}
\seealso{
\code{\link[=runif]{runif()}}, \code{\link[=hashfun]{hashfun()}}
}
\keyword{classes}
\keyword{distribution}
\keyword{sysdata}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/highlevel64.R
\name{duplicated.integer64}
\alias{duplicated.integer64}
\title{Determine Duplicate Elements of integer64}
\usage{
\method{duplicated}{integer64}(x, incomparables = FALSE, nunique = NULL, method = NULL, ...)
}
\arguments{
\item{x}{a vector or a data frame or an array or \code{NULL}.}

\item{incomparables}{ignored}

\item{nunique}{NULL or the number of unique values (including NA). Providing
\code{nunique} can speed-up matching when \code{x} has no cache. Note that a wrong
\code{nunique} can cause undefined behaviour up to a crash.}

\item{method}{NULL for automatic method selection or a suitable low-level
method, see details}

\item{...}{ignored}
}
\value{
\code{duplicated()}: a logical vector of the same length as \code{x}.
}
\description{
\code{duplicated()} determines which elements of a vector or data frame are duplicates
of elements with smaller subscripts, and returns a logical vector
indicating which elements (rows) are duplicates.
}
\details{
This function automatically chooses from several low-level functions
considering the size of \code{x} and the availability of a cache.

Suitable methods are
\itemize{
\item \code{\link{hashdup}} (hashing)
\item \code{\link{sortorderdup}} (fast ordering)
\item \code{\link{orderdup}} (memory saving ordering).
}
}
\examples{
x <- as.integer64(sample(c(rep(NA, 9), 1:9), 32, TRUE))
duplicated(x)

stopifnot(identical(duplicated(x),  duplicated(as.integer(x))))
}
\seealso{
\code{\link[=duplicated]{duplicated()}}, \code{\link[=unique.integer64]{unique.integer64()}}
}
\keyword{logic}
\keyword{manip}

\name{is.sorted.integer64}
\alias{is.sorted.integer64}
\alias{na.count.integer64}
\alias{nvalid.integer64}
\alias{nunique.integer64}
\alias{nties.integer64}
\title{
	Small cache access methods
}
\description{
	These methods are packaged here for methods in packages \code{bit64} and \code{ff}.
}
\usage{
	\method{is.sorted}{integer64}(x, \dots)
	\method{na.count}{integer64}(x, \dots)
	\method{nvalid}{integer64}(x, \dots)
	\method{nunique}{integer64}(x, \dots)
	\method{nties}{integer64}(x, \dots)
}
\arguments{
  \item{x}{
	some object
	}
  \item{\dots}{
	ignored
	}
}
\details{
  All these functions benefit from a \code{\link{sortcache}}, \code{\link{ordercache}} or \code{\link{sortordercache}}.  
  \code{na.count}, \code{nvalid} and \code{nunique} also benefit from a \code{\link{hashcache}}.
	\cr
	\code{is.sorted} checks for sortedness of \code{x} (NAs sorted first) \cr
 \code{na.count} returns the number of \code{NA}s \cr 
 \code{nvalid} returns the number of valid data points, usually \code{\link{length}} minus \code{na.count}. \cr
 \code{nunique} returns the number of unique values \cr
 \code{nties} returns the number of tied values. 
}
\note{
	If a \code{\link{cache}} exists but the desired value is not cached, 
 then these functions will store their result in the cache. 
 We do not consider this a relevant side-effect, 
 since these small cache results do not have a relevant memory footprint.
}
\value{
	\code{is.sorted} returns a logical scalar, the other methods return an integer scalar.
}
\author{
Jens Oehlschlägel <Jens.Oehlschlaegel@truecluster.com>
}
\seealso{
	\code{\link{cache}} for caching functions and \code{\link{sortordercache}} for functions creating big caches
}
\examples{
	x <- as.integer64(sample(c(rep(NA, 9), 1:9), 32, TRUE))
 length(x)
 na.count(x)
 nvalid(x)
 nunique(x)
 nties(x)
 table.integer64(x)
 x
}
\keyword{ environment }
\keyword{ methods }

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ops64.R
\name{ops64}
\alias{ops64}
\alias{binattr}
\alias{+.integer64}
\alias{-.integer64}
\alias{\%/\%.integer64}
\alias{\%\%.integer64}
\alias{*.integer64}
\alias{^.integer64}
\alias{/.integer64}
\alias{==.integer64}
\alias{!=.integer64}
\alias{<.integer64}
\alias{<=.integer64}
\alias{>.integer64}
\alias{>=.integer64}
\alias{&.integer64}
\alias{|.integer64}
\alias{!.integer64}
\title{Binary operators for integer64 vectors}
\usage{
binattr(e1, e2)

\method{+}{integer64}(e1, e2)

\method{-}{integer64}(e1, e2)

\method{\%/\%}{integer64}(e1, e2)

\method{\%\%}{integer64}(e1, e2)

\method{*}{integer64}(e1, e2)

\method{^}{integer64}(e1, e2)

\method{/}{integer64}(e1, e2)

\method{==}{integer64}(e1, e2)

\method{!=}{integer64}(e1, e2)

\method{<}{integer64}(e1, e2)

\method{<=}{integer64}(e1, e2)

\method{>}{integer64}(e1, e2)

\method{>=}{integer64}(e1, e2)

\method{&}{integer64}(e1, e2)

\method{|}{integer64}(e1, e2)

\method{!}{integer64}(x)
}
\arguments{
\item{e1, e2, x}{numeric or complex vectors or objects which can be coerced to such, or other objects for which methods have been written for - especially 'integer64' vectors.}
}
\value{
\code{\link{&}}, \code{\link{|}}, \code{\link{!}}, \code{\link{!=}}, \code{\link{==}}, \code{\link{<}}, \code{\link{<=}}, \code{\link{>}}, \code{\link{>=}} return a logical vector

\code{\link{/}} returns a double vector

\code{\link{+}}, \code{\link{-}}, \code{\link{*}}, \code{\link[=Arithmetic]{\%/\%}}, \code{\link[=Arithmetic]{\%\%}}, \code{\link{^}} return a vector of class 'integer64' or different class depending on the operands
}
\description{
Binary operators for integer64 vectors.
}
\examples{
  as.integer64(1:12) - 1
  options(integer64_semantics="new")
  d <- 2.5
  i <- as.integer64(5)
  d/i  # new 0.5
  d*i  # new 13
  i*d  # new 13
  options(integer64_semantics="old")
  d/i  # old: 0.4
  d*i  # old: 10
  i*d  # old: 13
}
\seealso{
\code{\link[=integer64]{integer64()}}
}
\keyword{classes}
\keyword{manip}

\name{runif64}
\alias{runif64}
\title{
   integer64: random numbers
}
\description{
  Create uniform random 64-bit integers within defined range
}
\usage{
  runif64(n, min = lim.integer64()[1], max = lim.integer64()[2], replace=TRUE)
}
\arguments{
  \item{n}{ length of return vector }
  \item{min}{ lower inclusive bound for random numbers }
  \item{max}{ upper inclusive bound for random numbers }
  \item{replace}{ set to FALSE for sampleing from a finite pool, see \code{\link{sample}} }
}
\value{
  a integer64 vector
}
\details{
  For each random integer we call R's internal C interface \code{unif_rand()} twice.
  Each call is mapped to 2^32 unsigned integers. The two 32-bit patterns are concatenated
  to form the new integer64. This process is repeated until the result is not a \code{NA_INTEGER64}. 
}
\author{
Jens Oehlschlägel <Jens.Oehlschlaegel@truecluster.com>
}
\keyword{ classes }
\keyword{distribution}
\keyword{sysdata}
\seealso{ 
  \code{\link{runif}}, \code{\link{hashfun}}
}
\examples{
  runif64(12)
  runif64(12, -16, 16)
  runif64(12, 0, as.integer64(2^60)-1)  # not 2^60-1 !
  var(runif(1e4))
  var(as.double(runif64(1e4, 0, 2^40))/2^40)  # ~ = 1/12 = .08333

  table(sample(16, replace=FALSE))
  table(runif64(16, 1, 16, replace=FALSE))
  table(sample(16, replace=TRUE))
  table(runif64(16, 1, 16, replace=TRUE))
}

\name{duplicated.integer64}
\alias{duplicated.integer64}
\title{Determine Duplicate Elements of integer64}
\description{
  \code{duplicated()} determines which elements of a vector or data frame are duplicates
  of elements with smaller subscripts, and returns a logical vector
  indicating which elements (rows) are duplicates.
}
\usage{
\method{duplicated}{integer64}(x, incomparables = FALSE, nunique = NULL, method = NULL, \dots)
}
\arguments{
  \item{x}{a vector or a data frame or an array or \code{NULL}.}
  \item{incomparables}{ignored}
  \item{nunique}{
	NULL or the number of unique values (including NA). Providing \code{nunique} can speed-up matching when \code{x} has no cache. Note that a wrong nunique can cause undefined behaviour up to a crash.
}
  \item{method}{
	NULL for automatic method selection or a suitable low-level method, see details
}
  \item{\dots}{ignored}
}
\details{
  This function automatically chooses from several low-level functions considering the size of \code{x} and the availability of a cache. 

  Suitable methods are \code{\link{hashdup}} (hashing), \code{\link{sortorderdup}} (fast ordering) and \code{\link{orderdup}} (memory saving ordering).
}
\value{
    \code{duplicated()}: a logical vector of the same length as \code{x}.  
}
\author{
	Jens Oehlschlägel <Jens.Oehlschlaegel@truecluster.com>
}
\seealso{ \code{\link{duplicated}}, \code{\link{unique.integer64}}  }
\examples{
x <- as.integer64(sample(c(rep(NA, 9), 1:9), 32, TRUE))
duplicated(x)

stopifnot(identical(duplicated(x),  duplicated(as.integer(x))))
}
\keyword{logic}
\keyword{manip}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/highlevel64.R
\name{prank}
\alias{prank}
\alias{prank.integer64}
\title{(P)ercent (Rank)s}
\usage{
prank(x, ...)

\method{prank}{integer64}(x, method = NULL, ...)
}
\arguments{
\item{x}{a integer64 vector}

\item{...}{ignored}

\item{method}{NULL for automatic method selection or a suitable low-level
method, see details}
}
\value{
\code{prank} returns a numeric vector of the same length as \code{x}.
}
\description{
Function \code{prank.integer64}  projects the values \verb{[min..max]} via ranks
\verb{[1..n]} to \verb{[0..1]}.
\code{\link[=qtile.integer64]{qtile.integer64()}} is the inverse function of 'prank.integer64' and
projects \verb{[0..1]} to \verb{[min..max]}.
}
\details{
Function \code{prank.integer64} is based on \code{\link[=rank.integer64]{rank.integer64()}}.
}
\examples{
x <- as.integer64(sample(c(rep(NA, 9), 1:9), 32, TRUE))
prank(x)

x <- x[!is.na(x)]
stopifnot(identical(x,  unname(qtile(x, probs=prank(x)))))
}
\seealso{
\code{\link[=rank.integer64]{rank.integer64()}} for simple ranks and \code{\link[=qtile]{qtile()}} for the
inverse function quantiles.
}
\keyword{univar}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/integer64.R
\name{all.equal.integer64}
\alias{all.equal.integer64}
\title{Test if two integer64 vectors are all.equal}
\usage{
\method{all.equal}{integer64}(
  target,
  current,
  tolerance = sqrt(.Machine$double.eps),
  scale = NULL,
  countEQ = FALSE,
  formatFUN = function(err, what) format(err),
  ...,
  check.attributes = TRUE
)
}
\arguments{
\item{target}{a vector of 'integer64' or an object that can be coerced
with \code{\link[=as.integer64]{as.integer64()}}}

\item{current}{a vector of 'integer64' or an object that can be coerced
with \code{\link[=as.integer64]{as.integer64()}}}

\item{tolerance}{numeric > 0.  Differences smaller than \code{tolerance} are
not reported.  The default value is close to \code{1.5e-8}.}

\item{scale}{\code{NULL} or numeric > 0, typically of length 1 or
\code{length(target)}. See Details.}

\item{countEQ}{logical indicating if the \code{target == current} cases should
be counted when computing the mean (absolute or relative) differences.
The default, \code{FALSE} may seem misleading in cases where \code{target} and
\code{current} only differ in a few places; see the extensive example.}

\item{formatFUN}{a \code{\link[=function]{function()}} of two arguments, \code{err}, the relative,
absolute or scaled error, and \code{what}, a character string indicating the
\emph{kind} of error; maybe used, e.g., to format relative and absolute errors
differently.}

\item{...}{further arguments are ignored}

\item{check.attributes}{logical indicating if the \code{\link[=attributes]{attributes()}} of \code{target}
and \code{current} (other than the names) should be compared.}
}
\value{
Either ‘TRUE’ (‘NULL’ for ‘attr.all.equal’) or a vector of ‘mode’
‘"character"’ describing the differences between ‘target’ and
‘current’.
}
\description{
A utility to compare integer64 objects 'x' and 'y' testing for
‘near equality’, see \code{\link[=all.equal]{all.equal()}}.
}
\details{
In \code{\link[=all.equal.numeric]{all.equal.numeric()}} the type \code{integer} is treated as a proper subset
of \code{double} i.e. does not complain about comparing \code{integer} with \code{double}.
Following this logic \code{all.equal.integer64} treats \code{integer} as a proper
subset of \code{integer64} and does not complain about comparing \code{integer} with
\code{integer64}. \code{double} also compares without warning as long as the values
are within \code{\link[=lim.integer64]{lim.integer64()}}, if \code{double} are bigger \code{all.equal.integer64}
complains about the \verb{all.equal.integer64 overflow warning}. For further
details see \code{\link[=all.equal]{all.equal()}}.
}
\note{
\code{\link[=all.equal]{all.equal()}} only dispatches to this method if the first argument is \code{integer64},
calling \code{\link[=all.equal]{all.equal()}} with a \code{non-integer64} first and a \code{integer64} second argument
gives undefined behavior!
}
\examples{
  all.equal(as.integer64(1:10), as.integer64(0:9))
  all.equal(as.integer64(1:10), as.integer(1:10))
  all.equal(as.integer64(1:10), as.double(1:10))
  all.equal(as.integer64(1), as.double(1e300))
}
\seealso{
\code{\link[=all.equal]{all.equal()}}
}

\name{table.integer64}
\title{Cross Tabulation and Table Creation for integer64}
\alias{table.integer64}

\concept{counts}
\concept{frequencies}
\concept{occurrences}
\concept{contingency table}

\description{
  \code{table.integer64} uses the cross-classifying integer64 vectors to build a contingency
  table of the counts at each combination of vector values.
}
\usage{
table.integer64(\dots
, return = c("table","data.frame","list")
, order = c("values","counts")
, nunique = NULL
, method = NULL
, dnn = list.names(...), deparse.level = 1
) 
}
\arguments{
  \item{\dots}{one or more objects which can be interpreted as factors
    (including character strings), or a list (or data frame) whose
    components can be so interpreted.  (For \code{as.table} and
    \code{as.data.frame}, arguments passed to specific methods.)}
  \item{nunique}{
	NULL or the number of unique values of table (including NA). Providing \code{nunique} can speed-up matching when \code{table} has no cache. Note that a wrong nunique can cause undefined behaviour up to a crash.
}
  \item{order}{
	By default results are created sorted by "values", or by "counts"
}
  \item{method}{
	NULL for automatic method selection or a suitable low-level method, see details
}
  \item{return}{
     choose the return format, see details
}
  \item{dnn}{the names to be given to the dimensions in the result (the
    \emph{dimnames names}).}
  \item{deparse.level}{controls how the default \code{dnn} is
    constructed.  See \sQuote{Details}.}
}
\details{
  This function automatically chooses from several low-level functions considering the size of \code{x} and the availability of a cache. 
  Suitable methods are \code{\link{hashmaptab}} (simultaneously creating and using a hashmap)
, \code{\link{hashtab}} (first creating a hashmap then using it)
, \code{\link{sortordertab}} (fast ordering) 
and \code{\link{ordertab}} (memory saving ordering).
\cr
  If the argument \code{dnn} is not supplied, the internal function
  \code{list.names} is called to compute the \sQuote{dimname names}.  If the
  arguments in \code{\dots} are named, those names are used.  For the
  remaining arguments, \code{deparse.level = 0} gives an empty name,
  \code{deparse.level = 1} uses the supplied argument if it is a symbol,
  and \code{deparse.level = 2} will deparse the argument.

  Arguments \code{exclude}, \code{useNA}, are not supported, i.e. \code{NA}s are always tabulated, and, different from \code{\link{table}} they are sorted first if \code{order="values"}. 
}
\value{
  By default (with \code{return="table"}) \code{\link{table}} returns a \emph{contingency table}, an object of
  class \code{"table"}, an array of integer values. Note that unlike S the result is always an array, a 1D array if one factor is given. Note also that for multidimensional arrays this is a \emph{dense} return structure which can dramatically increase RAM requirements (for large arrays with high mutual information, i.e. many possible input combinations of which only few occur) and that \code{\link{table}} is limited to \code{2^31} possible combinations (e.g. two input vectors with 46340 unique values only). Finally note that the tabulated values or value-combinations are represented as \code{dimnames} and that the implied conversion of values to strings can cause \emph{severe} performance problems since each string needs to be integrated into R's global string cache. 
  \cr
  You can use the other \code{return=} options to cope with these problems, the potential combination limit is increased from \code{2^31} to \code{2^63} with these options, RAM is only rewquired for observed combinations and string conversion is avoided. 
  \cr
  With \code{return="data.frame"} you get a \emph{dense} representation as a \code{\link{data.frame}} (like that resulting from \code{as.data.frame(table(...))}) where only observed combinations are listed (each as a data.frame row) with the corresponding frequency counts (the latter as component
  named by \code{responseName}).  This is the inverse of \code{\link{xtabs}}..
  \cr
  With \code{return="list"} you also get a \emph{dense} representation as a simple \code{\link{list}} with components 
  \item{values }{a integer64 vector of the technically tabulated values, for 1D this is the tabulated values themselves, for kD these are the values representing the potential combinations of input values}
  \item{counts}{the frequency counts}
  \item{dims}{only for kD: a list with the vectors of the unique values of the input dimensions}
}
\note{
  Note that by using \code{\link{as.integer64.factor}} we can also input 
  factors into \code{table.integer64} -- only the \code{\link{levels}} get lost.
 \cr
  Note that because of the existence of \code{\link{as.factor.integer64}} 
the standard \code{\link{table}} function -- within its limits -- can also be used 
for \code{\link{integer64}}, and especially for combining \code{\link{integer64}} input 
with other data types.
}
\seealso{
  \code{\link{table}} for more info on the standard version coping with Base R's data types, \code{\link{tabulate}} which can faster tabulate \code{\link{integer}s} with a limited range \code{[1L .. nL not too big]}, \code{\link{unique.integer64}} for the unique values without counting them and \code{\link{unipos.integer64}} for the positions of the unique values. 
}
\examples{
message("pure integer64 examples")
x <- as.integer64(sample(c(rep(NA, 9), 1:9), 32, TRUE))
y <- as.integer64(sample(c(rep(NA, 9), 1:9), 32, TRUE))
z <- sample(c(rep(NA, 9), letters), 32, TRUE)
table.integer64(x)
table.integer64(x, order="counts")
table.integer64(x, y)
table.integer64(x, y, return="data.frame")

message("via as.integer64.factor we can use 'table.integer64' also for factors")
table.integer64(x, as.integer64(as.factor(z)))

message("via as.factor.integer64 we can also use 'table' for integer64")
table(x)
table(x, exclude=NULL)
table(x, z, exclude=NULL)

\dontshow{
 stopifnot(identical(table.integer64(as.integer64(c(1,1,2))), table(c(1,1,2))))
 stopifnot(identical(table.integer64(as.integer64(c(1,1,2)),as.integer64(c(3,4,4))), table(c(1,1,2),c(3,4,4))))
 message("the following works with three warnings due to coercion")
 stopifnot(identical(table.integer64(c(1,1,2)), table(c(1,1,2))))
 stopifnot(identical(table.integer64(as.integer64(c(1,1,2)),c(3,4,4)), table(c(1,1,2),c(3,4,4))))
 stopifnot(identical(table.integer64(c(1,1,2),as.integer64(c(3,4,4))), table(c(1,1,2),c(3,4,4))))
 message("the following works because of as.factor.integer64")
 stopifnot(identical(table(as.integer64(c(1,1,2))), table(c(1,1,2))))  
 stopifnot(identical(table(as.integer64(c(1,1,2)),as.integer64(c(3,4,4))), table(c(1,1,2),c(3,4,4))))
 stopifnot(identical(table(as.integer64(c(1,1,2)),c(3,4,4)), table(c(1,1,2),c(3,4,4))))
 stopifnot(identical(table(c(1,1,2),as.integer64(c(3,4,4))), table(c(1,1,2),c(3,4,4))))
}

}
\keyword{category}

\name{prank}
\alias{prank}
\alias{prank.integer64}
\title{(P)ercent (Rank)s}
\description{
	Function \code{prank.integer64}  projects the values [min..max] via ranks [1..n] to [0..1]. 
	\code{\link{qtile.integer64}} is the inverse function of 'prank.integer64' and projects [0..1] to [min..max].
}
\usage{
	prank(x, \dots)
	\method{prank}{integer64}(x, method = NULL, \dots)
}
\arguments{
  \item{x}{a integer64 vector}
  \item{method}{
	NULL for automatic method selection or a suitable low-level method, see details
}
  \item{\dots}{ignored}
}
\details{
	Function \code{prank.integer64} is based on \code{\link{rank.integer64}}.
}
\value{
  \code{prank} returns a numeric vector of the same length as \code{x}.
}
\author{
	Jens Oehlschlägel <Jens.Oehlschlaegel@truecluster.com>
}
\seealso{
  \code{\link{rank.integer64}} for simple ranks and \code{\link{qtile}} for the inverse function quantiles.
}
\examples{
x <- as.integer64(sample(c(rep(NA, 9), 1:9), 32, TRUE))
prank(x)

x <- x[!is.na(x)]
stopifnot(identical(x,  unname(qtile(x, probs=prank(x)))))
}
\keyword{univar}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sort64.R
\name{ramsort.integer64}
\alias{ramsort.integer64}
\alias{shellsort.integer64}
\alias{shellsortorder.integer64}
\alias{shellorder.integer64}
\alias{mergesort.integer64}
\alias{mergeorder.integer64}
\alias{mergesortorder.integer64}
\alias{quicksort.integer64}
\alias{quicksortorder.integer64}
\alias{quickorder.integer64}
\alias{radixsort.integer64}
\alias{radixsortorder.integer64}
\alias{radixorder.integer64}
\alias{ramsortorder.integer64}
\alias{ramorder.integer64}
\title{Low-level intger64 methods for in-RAM sorting and ordering}
\usage{
\method{shellsort}{integer64}(x, has.na = TRUE, na.last = FALSE, decreasing = FALSE, ...)

\method{shellsortorder}{integer64}(x, i, has.na = TRUE, na.last = FALSE, decreasing = FALSE, ...)

\method{shellorder}{integer64}(x, i, has.na = TRUE, na.last = FALSE, decreasing = FALSE, ...)

\method{mergesort}{integer64}(x, has.na = TRUE, na.last = FALSE, decreasing = FALSE, ...)

\method{mergeorder}{integer64}(x, i, has.na = TRUE, na.last = FALSE, decreasing = FALSE, ...)

\method{mergesortorder}{integer64}(x, i, has.na = TRUE, na.last = FALSE, decreasing = FALSE, ...)

\method{quicksort}{integer64}(
  x,
  has.na = TRUE,
  na.last = FALSE,
  decreasing = FALSE,
  restlevel = floor(1.5 * log2(length(x))),
  ...
)

\method{quicksortorder}{integer64}(
  x,
  i,
  has.na = TRUE,
  na.last = FALSE,
  decreasing = FALSE,
  restlevel = floor(1.5 * log2(length(x))),
  ...
)

\method{quickorder}{integer64}(
  x,
  i,
  has.na = TRUE,
  na.last = FALSE,
  decreasing = FALSE,
  restlevel = floor(1.5 * log2(length(x))),
  ...
)

\method{radixsort}{integer64}(
  x,
  has.na = TRUE,
  na.last = FALSE,
  decreasing = FALSE,
  radixbits = 8L,
  ...
)

\method{radixsortorder}{integer64}(
  x,
  i,
  has.na = TRUE,
  na.last = FALSE,
  decreasing = FALSE,
  radixbits = 8L,
  ...
)

\method{radixorder}{integer64}(
  x,
  i,
  has.na = TRUE,
  na.last = FALSE,
  decreasing = FALSE,
  radixbits = 8L,
  ...
)

\method{ramsort}{integer64}(
  x,
  has.na = TRUE,
  na.last = FALSE,
  decreasing = FALSE,
  stable = TRUE,
  optimize = c("time", "memory"),
  VERBOSE = FALSE,
  ...
)

\method{ramsortorder}{integer64}(
  x,
  i,
  has.na = TRUE,
  na.last = FALSE,
  decreasing = FALSE,
  stable = TRUE,
  optimize = c("time", "memory"),
  VERBOSE = FALSE,
  ...
)

\method{ramorder}{integer64}(
  x,
  i,
  has.na = TRUE,
  na.last = FALSE,
  decreasing = FALSE,
  stable = TRUE,
  optimize = c("time", "memory"),
  VERBOSE = FALSE,
  ...
)
}
\arguments{
\item{x}{a vector to be sorted by \code{\link[=ramsort.integer64]{ramsort.integer64()}} and
\code{\link[=ramsortorder.integer64]{ramsortorder.integer64()}}, i.e. the output of  \code{\link[=sort.integer64]{sort.integer64()}}}

\item{has.na}{boolean scalar defining whether the input vector might contain
\code{NA}s. If we know we don't have NAs, this may speed-up. \emph{Note} that you
risk a crash if there are unexpected \code{NA}s with \code{has.na=FALSE}}

\item{na.last}{boolean scalar telling ramsort whether to sort \code{NA}s last
or first. \emph{Note} that 'boolean' means that there is no third option \code{NA}
as in \code{\link[=sort]{sort()}}}

\item{decreasing}{boolean scalar telling ramsort whether to sort increasing
or decreasing}

\item{...}{further arguments, passed from generics, ignored in methods}

\item{i}{integer positions to be modified by \code{\link[=ramorder.integer64]{ramorder.integer64()}} and
\code{\link[=ramsortorder.integer64]{ramsortorder.integer64()}}, default is 1:n, in this case the output is
similar to \code{\link[=order.integer64]{order.integer64()}}}

\item{restlevel}{number of remaining recursionlevels before \code{quicksort}
switches from recursing to \code{shellsort}}

\item{radixbits}{size of radix in bits}

\item{stable}{boolean scalar defining whether stable sorting is needed.
Allowing non-stable may speed-up.}

\item{optimize}{by default ramsort optimizes for 'time' which requires more
RAM, set to 'memory' to minimize RAM requirements and sacrifice speed}

\item{VERBOSE}{cat some info about chosen method}
}
\value{
These functions return the number of \code{NAs} found or assumed
during sorting
}
\description{
Fast low-level methods for sorting and ordering. The \code{..sortorder}
methods do sorting and ordering at once, which requires more RAM
than ordering but is (almost) as fast as as sorting.
}
\details{
See \code{\link[bit:Sorting]{bit::ramsort()}}
}
\note{
Note that these methods purposely violate the functional programming
paradigm: they are called for the side-effect of changing some of
their arguments. The \code{sort}-methods change \code{x}, the \code{order}-methods
change \code{i}, and the \code{sortoder}-methods change both \code{x} and \code{i}
}
\examples{
  x <- as.integer64(sample(c(rep(NA, 9), 1:9), 32, TRUE))
  x
  message("ramsort example")
  s <- bit::clone(x)
  bit::ramsort(s)
  message("s has been changed in-place - whether or not ramsort uses an in-place algorithm")
  s
  message("ramorder example")
  s <- bit::clone(x)
  o <- seq_along(s)
  bit::ramorder(s, o)
  message("o has been changed in-place - s remains unchanged")
  s
  o
  s[o]
  message("ramsortorder example")
  o <- seq_along(s)
  bit::ramsortorder(s, o)
  message("s and o have both been changed in-place - this is much faster")
  s
  o
}
\seealso{
\code{\link[bit:Sorting]{bit::ramsort()}} for the generic, \code{ramsort.default} for the methods
provided by package ff, \code{\link[=sort.integer64]{sort.integer64()}} for the sort interface and
\code{\link[=sortcache]{sortcache()}} for caching the work of sorting
}
\keyword{manip}
\keyword{programming}

\name{all.equal.integer64}
\alias{all.equal.integer64}
\title{
   Test if two integer64 vectors are all.equal
}
\description{
   A utility to compare integer64 objects 'x' and 'y' testing for ‘near equality’, see \code{\link{all.equal}}.
}
\usage{
  \method{all.equal}{integer64}(
  target
, current
, tolerance = sqrt(.Machine$double.eps)
, scale = NULL
, countEQ = FALSE
, formatFUN = function(err, what) format(err)
, ...
, check.attributes = TRUE
)
}
\arguments{
  \item{target}{ a vector of 'integer64' or an object that can be coerced with \code{\link{as.integer64}} }
  \item{current}{ a vector of 'integer64' or an object that can be coerced with \code{\link{as.integer64}} }
  \item{tolerance}{numeric \eqn{\ge} 0.  Differences smaller than
    \code{tolerance} are not reported.  The default value is close to
    \code{1.5e-8}.}
  \item{scale}{\code{NULL} or numeric > 0, typically of length 1 or
    \code{length(target)}.  See \sQuote{Details}.}
  \item{countEQ}{logical indicating if the \code{target == current}
    cases should be counted when computing the mean (absolute or
    relative) differences.  The default, \code{FALSE} may seem
    misleading in cases where \code{target} and \code{current} only
    differ in a few places; see the extensive example.}
  \item{formatFUN}{a \code{\link{function}} of two arguments,
    \code{err}, the relative, absolute or scaled error, and
    \code{what}, a character string indicating the \emph{kind} of error;
    maybe used, e.g., to format relative and absolute errors differently.}
  \item{\dots}{further arguments are ignored}
  \item{check.attributes}{logical indicating if the
    \code{\link{attributes}} of \code{target} and \code{current}
    (other than the names) should be compared.}
}
\value{
  Either ‘TRUE’ (‘NULL’ for ‘attr.all.equal’) or a vector of ‘mode’
  ‘"character"’ describing the differences between ‘target’ and
  ‘current’.
}
\details{
   In \code{\link{all.equal.numeric}} the type \code{integer} is treated as a proper subset of \code{double} 
   i.e. does not complain about comparing \code{integer} with \code{double}. 
   Following this logic \code{all.equal.integer64} treats \code{integer} as a proper subset of \code{integer64} 
   and does not complain about comparing \code{integer} with \code{integer64}. \code{double} also compares without warning 
   as long as the values are within \code{\link{lim.integer64}}, if \code{double} are bigger \code{all.equal.integer64}
   complains about the \code{all.equal.integer64 overflow warning}. For further details see \code{\link{all.equal}}.
}
\note{
   \code{\link{all.equal}} only dispatches to this method if the first argument is \code{integer64}, 
   calling \code{\link{all.equal}} with a \code{non-integer64} first and a \code{integer64} second argument 
   gives undefined behavior!
}
\author{
  Leonardo Silvestri (for package nanotime)
}
\seealso{
  \code{\link{all.equal}}
}
\examples{
  all.equal(as.integer64(1:10), as.integer64(0:9))
  all.equal(as.integer64(1:10), as.integer(1:10))
  all.equal(as.integer64(1:10), as.double(1:10))
  all.equal(as.integer64(1), as.double(1e300))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/highlevel64.R
\name{benchmark64}
\alias{benchmark64}
\alias{optimizer64}
\title{Function for measuring algorithmic performance of high-level and low-level integer64 functions}
\usage{
benchmark64(nsmall = 2L^16L, nbig = 2L^25L, timefun = repeat.time)

optimizer64(
  nsmall = 2L^16L,
  nbig = 2L^25L,
  timefun = repeat.time,
  what = c("match", "\%in\%", "duplicated", "unique", "unipos", "table", "rank",
    "quantile", "factor"),
  uniorder = c("original", "values", "any"),
  taborder = c("values", "counts"),
  plot = TRUE
)
}
\arguments{
\item{nsmall}{size of smaller vector}

\item{nbig}{size of larger bigger vector}

\item{timefun}{a function for timing such as \code{\link[bit:repeat.time]{bit::repeat.time()}} or \code{\link[=system.time]{system.time()}}}

\item{what}{a vector of names of high-level functions}

\item{uniorder}{one of the order parameters that are allowed in \code{\link[=unique.integer64]{unique.integer64()}} and \code{\link[=unipos.integer64]{unipos.integer64()}}}

\item{taborder}{one of the order parameters that are allowed in \code{\link[=table]{table()}}}

\item{plot}{set to FALSE to suppress plotting}
}
\value{
\code{benchmark64} returns a matrix with elapsed seconds, different high-level tasks
in rows and different scenarios to solve the task in columns. The last row
named 'SESSION' contains the elapsed seconds of the exemplary sesssion.

\code{optimizer64} returns a dimensioned list with one row for each high-level
function timed and two columns named after the values of the \code{nsmall} and
\code{nbig} sample sizes. Each list cell contains a matrix with timings,
low-level-methods in rows and three measurements \code{c("prep", "both", "use")}
in columns. If it can be measured separately, \code{prep} contains the timing
of preparatory work such as sorting and hashing, and \code{use} contains the
timing of using the prepared work. If the function timed does both,
preparation and use, the timing is in \code{both}.
}
\description{
Function for measuring algorithmic performance of high-level and low-level integer64 functions
}
\details{
\code{benchmark64} compares the following scenarios for the following use cases:\tabular{rl}{
   \strong{scenario name} \tab \strong{explanation} \cr
   32-bit \tab applying Base R function to 32-bit integer data \cr
   64-bit \tab applying bit64 function to 64-bit integer data (with no cache) \cr
   hashcache \tab ditto when cache contains \code{\link[=hashmap]{hashmap()}}, see \code{\link[=hashcache]{hashcache()}} \cr
   sortordercache \tab ditto when cache contains sorting and ordering, see \code{\link[=sortordercache]{sortordercache()}} \cr
   ordercache \tab ditto when cache contains ordering only, see \code{\link[=ordercache]{ordercache()}} \cr
   allcache \tab ditto when cache contains sorting, ordering and hashing \cr
}
\tabular{rl}{
   \strong{use case name} \tab \strong{explanation} \cr
   cache \tab filling the cache according to scenario \cr
   match(s, b) \tab match small in big vector \cr
   s \%in\% b \tab small \%in\% big vector \cr
   match(b, s) \tab match big in small vector \cr
   b \%in\% s \tab big \%in\% small vector \cr
   match(b, b) \tab match big in (different) big vector \cr
   b \%in\% b \tab big \%in\% (different) big vector \cr
   duplicated(b) \tab duplicated of big vector \cr
   unique(b) \tab unique of big vector \cr
   table(b) \tab table of big vector \cr
   sort(b) \tab sorting of big vector \cr
   order(b) \tab ordering of big vector \cr
   rank(b) \tab ranking of big vector \cr
   quantile(b) \tab quantiles of big vector \cr
   summary(b) \tab summary of of big vector \cr
   factor(b) \tab coercion to factor of big vector \cr
   SESSION \tab exemplary session involving multiple calls (including cache filling costs) \cr
}


Note that the timings for the cached variants do \emph{not} contain the
time costs of building the cache, except for the timing of the exemplary
user session, where the cache costs are included in order to evaluate amortization.
}
\section{Functions}{
\itemize{
\item \code{benchmark64()}: compares high-level integer64 functions against the
integer functions from Base R

\item \code{optimizer64()}: compares for each high-level integer64 function the Base
R integer function with several low-level integer64 functions with and
without caching

}}
\examples{
message("this small example using system.time does not give serious timings\n
this we do this only to run regression tests")
benchmark64(nsmall=2^7, nbig=2^13, timefun=function(expr)system.time(expr, gcFirst=FALSE))
optimizer64(nsmall=2^7, nbig=2^13, timefun=function(expr)system.time(expr, gcFirst=FALSE)
, plot=FALSE
)
\dontrun{
message("for real measurement of sufficiently large datasets run this on your machine")
benchmark64()
optimizer64()
}
message("let's look at the performance results on Core i7 Lenovo T410 with 8 GB RAM")
data(benchmark64.data)
print(benchmark64.data)

matplot(log2(benchmark64.data[-1, 1]/benchmark64.data[-1, ])
, pch=c("3", "6", "h", "s", "o", "a")
, xlab="tasks [last=session]"
, ylab="log2(relative speed) [bigger is better]"
)
matplot(t(log2(benchmark64.data[-1, 1]/benchmark64.data[-1, ]))
, type="b", axes=FALSE
, lwd=c(rep(1, 14), 3)
, xlab="context"
, ylab="log2(relative speed) [bigger is better]"
)
axis(1
, labels=c("32-bit", "64-bit", "hash", "sortorder", "order", "hash+sortorder")
, at=1:6
)
axis(2)
data(optimizer64.data)
print(optimizer64.data)
oldpar <- par(no.readonly = TRUE)
par(mfrow=c(2, 1))
par(cex=0.7)
for (i in 1:nrow(optimizer64.data)) {
 for (j in 1:2) {
   tim <- optimizer64.data[[i, j]]
  barplot(t(tim))
  if (rownames(optimizer64.data)[i]=="match")
   title(paste("match", colnames(optimizer64.data)[j], "in", colnames(optimizer64.data)[3-j]))
  else if (rownames(optimizer64.data)[i]=="\%in\%")
   title(paste(colnames(optimizer64.data)[j], "\%in\%", colnames(optimizer64.data)[3-j]))
  else
   title(paste(rownames(optimizer64.data)[i], colnames(optimizer64.data)[j]))
 }
}
par(mfrow=c(1, 1))
}
\seealso{
\code{\link[=integer64]{integer64()}}
}
\keyword{misc}

\name{ramsort.integer64}
\alias{ramsort.integer64}
\alias{shellsort.integer64}
\alias{quicksort.integer64}
\alias{mergesort.integer64}
\alias{radixsort.integer64}
\alias{ramorder.integer64}
\alias{shellorder.integer64}
\alias{quickorder.integer64}
\alias{mergeorder.integer64}
\alias{radixorder.integer64}
\alias{ramsortorder.integer64}
\alias{shellsortorder.integer64}
\alias{quicksortorder.integer64}
\alias{mergesortorder.integer64}
\alias{radixsortorder.integer64}
\title{
   Low-level intger64 methods for in-RAM sorting and ordering
}
\description{
  Fast low-level methods for sorting and ordering. 
  The \code{..sortorder} methods do sorting and ordering at once, which requires more RAM than ordering but is (almost) as fast as as sorting.
}
\note{
 Note that these methods purposely violate the functional programming paradigm: they are called for the side-effect of changing some of their arguments.
 The \code{sort}-methods change \code{x}, the \code{order}-methods change \code{i}, and the \code{sortoder}-methods change both \code{x} and \code{i}
}
\usage{
\method{shellsort}{integer64}(x, has.na=TRUE, na.last=FALSE, decreasing=FALSE, \dots)
\method{shellsortorder}{integer64}(x, i, has.na=TRUE, na.last=FALSE, decreasing=FALSE, \dots)
\method{shellorder}{integer64}(x, i, has.na=TRUE, na.last=FALSE, decreasing=FALSE, \dots)
\method{mergesort}{integer64}(x, has.na=TRUE, na.last=FALSE, decreasing=FALSE, \dots)
\method{mergeorder}{integer64}(x, i, has.na=TRUE, na.last=FALSE, decreasing=FALSE, \dots)
\method{mergesortorder}{integer64}(x, i, has.na=TRUE, na.last=FALSE, decreasing=FALSE, \dots)
\method{quicksort}{integer64}(x, has.na=TRUE, na.last=FALSE, decreasing=FALSE
, restlevel=floor(1.5*log2(length(x))), \dots)
\method{quicksortorder}{integer64}(x, i, has.na=TRUE, na.last=FALSE, decreasing=FALSE
, restlevel=floor(1.5*log2(length(x))), \dots)
\method{quickorder}{integer64}(x, i, has.na=TRUE, na.last=FALSE, decreasing=FALSE
, restlevel=floor(1.5*log2(length(x))), \dots)
\method{radixsort}{integer64}(x, has.na=TRUE, na.last=FALSE, decreasing=FALSE, radixbits=8L, \dots)
\method{radixsortorder}{integer64}(x, i, has.na=TRUE, na.last=FALSE, decreasing=FALSE, radixbits=8L, \dots)
\method{radixorder}{integer64}(x, i, has.na=TRUE, na.last=FALSE, decreasing=FALSE, radixbits=8L, \dots)
\method{ramsort}{integer64}(x, has.na = TRUE, na.last=FALSE, decreasing = FALSE, stable = TRUE
, optimize = c("time", "memory"), VERBOSE = FALSE, \dots)
\method{ramsortorder}{integer64}(x, i, has.na = TRUE, na.last=FALSE, decreasing = FALSE, stable = TRUE
, optimize = c("time", "memory"), VERBOSE = FALSE, \dots)
\method{ramorder}{integer64}(x, i, has.na = TRUE, na.last=FALSE, decreasing = FALSE, stable = TRUE
, optimize = c("time", "memory"), VERBOSE = FALSE, \dots)
}
\arguments{
  \item{x}{ a vector to be sorted by \code{\link{ramsort}} and \code{\link{ramsortorder}}, i.e. the output of  \code{\link{sort}} }
  \item{i}{ integer positions to be modified by \code{\link{ramorder}} and \code{\link{ramsortorder}}, default is 1:n, in this case the output is similar to \code{\link{order}} }
  \item{has.na}{
boolean scalar defining whether the input vector might contain \code{NA}s. If we know we don't have NAs, this may speed-up.
\emph{Note} that you risk a crash if there are unexpected \code{NA}s with \code{has.na=FALSE}
}
  \item{na.last}{
boolean scalar telling ramsort whether to sort \code{NA}s last or first.
\emph{Note} that 'boolean' means that there is no third option \code{NA} as in \code{\link{sort}}
}
  \item{decreasing}{
boolean scalar telling ramsort whether to sort increasing or decreasing
}
  \item{stable}{
boolean scalar defining whether stable sorting is needed. Allowing non-stable may speed-up.
}
  \item{optimize}{
by default ramsort optimizes for 'time' which requires more RAM,
set to 'memory' to minimize RAM requirements and sacrifice speed
}
  \item{restlevel}{
number of remaining recursionlevels before \code{quicksort} switches from recursing to \code{shellsort}
}
  \item{radixbits}{
	size of radix in bits
}
  \item{VERBOSE}{
  cat some info about chosen method
}
  \item{\dots}{ further arguments, passed from generics, ignored in methods }
}
\details{
 see \code{\link{ramsort}}
}
\value{
  These functions return the number of \code{NAs} found or assumed during sorting
}
\author{
Jens Oehlschlägel <Jens.Oehlschlaegel@truecluster.com>
}
\keyword{ programming }
\keyword{ manip }
\seealso{ \code{\link{ramsort}} for the generic, \code{ramsort.default} for the methods provided by package \code{ff}, \code{\link{sort.integer64}} for the sort interface and \code{\link{sortcache}} for caching the work of sorting}
\examples{
  x <- as.integer64(sample(c(rep(NA, 9), 1:9), 32, TRUE))
  x
  message("ramsort example")
  s <- clone(x)
  ramsort(s)
  message("s has been changed in-place - whether or not ramsort uses an in-place algorithm")
  s
  message("ramorder example")
  s <- clone(x)
  o <- seq_along(s)
  ramorder(s, o)
  message("o has been changed in-place - s remains unchanged")
  s
  o
  s[o]
  message("ramsortorder example")
  o <- seq_along(s)
  ramsortorder(s, o)
  message("s and o have both been changed in-place - this is much faster")
  s
  o
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/matrix64.R, R/patch64.R
\name{matrix64}
\alias{matrix64}
\alias{matrix.integer64}
\alias{array.integer64}
\alias{colSums.integer64}
\alias{rowSums.integer64}
\alias{aperm.integer64}
\alias{matrix}
\alias{array}
\alias{colSums}
\alias{colSums.default}
\alias{rowSums}
\alias{rowSums.default}
\title{Working with integer64 arrays and matrices}
\usage{
\method{matrix}{integer64}(data = NA_integer64_, ...)

\method{array}{integer64}(data = NA_integer64_, ...)

\method{colSums}{integer64}(x, na.rm = FALSE, dims = 1L)

\method{rowSums}{integer64}(x, na.rm = FALSE, dims = 1L)

\method{aperm}{integer64}(a, perm, ...)

matrix(data = NA, nrow = 1L, ncol = 1L, byrow = FALSE, dimnames = NULL)

array(data = NA, dim = length(data), dimnames = NULL)

colSums(x, na.rm = FALSE, dims = 1L)

\method{colSums}{default}(x, na.rm = FALSE, dims = 1L)

rowSums(x, na.rm = FALSE, dims = 1L)

\method{rowSums}{default}(x, na.rm = FALSE, dims = 1L)
}
\arguments{
\item{data, nrow, ncol, byrow, dimnames, dim}{Arguments for \code{matrix()} and \code{array()}.}

\item{...}{Passed on to subsequent methods.}

\item{x}{An array of integer64 numbers.}

\item{na.rm, dims}{Same interpretation as in \code{\link[=colSums]{colSums()}}.}

\item{a, perm}{Passed on to \code{\link[=aperm]{aperm()}}.}
}
\value{
A matrix of integer64 values with the appropriate dimension
}
\description{
These functions and methods facilitate working with integer64
objects stored in matrices. As ever, the primary motivation
for having tailor-made functions here is that R's methods
often receive input from bit64 and treat the vectors as doubles,
leading to unexpected and/or incorrect results.
}
\details{
As of now, the \code{colSums()} and \code{rowSums()} methods are implemented
as wrappers around equivalent \code{apply()} approaches, because
re-using the default routine (and then applying integer64 to the
result) does not work for objects with missing elements. Ideally
this would eventually get its own dedicated C routine mimicking
that of \code{colSums()} for integers; feature requests and PRs welcome.

\code{aperm()} is required for \code{apply()} to work, in general, otherwise
\code{FUN} gets applied to a class-stripped version of the input.
}
\examples{
A = matrix(as.integer64(1:6), 3)

colSums(A)
rowSums(A)
aperm(A, 2:1)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/highlevel64.R
\name{keypos}
\alias{keypos}
\alias{keypos.integer64}
\title{Extract Positions in redundant dimension table}
\usage{
keypos(x, ...)

\method{keypos}{integer64}(x, method = NULL, ...)
}
\arguments{
\item{x}{a vector or a data frame or an array or \code{NULL}.}

\item{...}{ignored}

\item{method}{NULL for automatic method selection or a suitable low-level
method, see details}
}
\value{
an integer vector of the same length as \code{x} containing positions
relative to \code{sort(unique(x), na.last=FALSE)}
}
\description{
\code{keypos} returns the positions of the (fact table) elements that participate
in their sorted unique subset (dimension table)
}
\details{
NAs are sorted first in the dimension table, see \code{\link[=ramorder.integer64]{ramorder.integer64()}}.

This function automatically chooses from several low-level functions
considering the size of \code{x} and the availability of a cache.

Suitable methods are
\itemize{
\item \code{\link{sortorderkey}} (fast ordering)
\item \code{\link{orderkey}} (memory saving ordering).
}
}
\examples{
x <- as.integer64(sample(c(rep(NA, 9), 1:9), 32, TRUE))
keypos(x)

}
\seealso{
\code{\link[=unique.integer64]{unique.integer64()}} for the unique subset and \code{\link[=match.integer64]{match.integer64()}}
for finding positions in a different vector.
}
\keyword{manip}
\keyword{univar}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/highlevel64.R
\name{table}
\alias{table}
\alias{table.integer64}
\title{Cross Tabulation and Table Creation for integer64}
\usage{
table(
  ...,
  exclude = if (useNA == "no") c(NA, NaN),
  useNA = c("no", "ifany", "always"),
  dnn = list.names(...),
  deparse.level = 1L
)

\method{table}{integer64}(
  ...,
  return = c("table", "data.frame", "list"),
  order = c("values", "counts"),
  nunique = NULL,
  method = NULL,
  exclude = if (useNA == "no") c(NA, NaN),
  useNA = c("no", "ifany", "always"),
  dnn = list.names(...),
  deparse.level = 1L
)
}
\arguments{
\item{...}{one or more objects which can be interpreted as factors
(including character strings), or a list (or data frame) whose
components can be so interpreted.  (For \code{as.table} and \code{as.data.frame},
arguments passed to specific methods.)}

\item{exclude}{levels to remove for all factors in ....}

\item{useNA}{whether to include NA values in the table.}

\item{dnn}{the names to be given to the dimensions in the result
(the \emph{dimnames names}).}

\item{deparse.level}{controls how the default \code{dnn} is constructed. See Details.}

\item{return}{choose the return format, see details}

\item{order}{By default results are created sorted by "values", or by "counts"}

\item{nunique}{NULL or the number of unique values of table (including NA).
Providing \code{nunique} can speed-up matching when \code{table} has no cache. Note
that a wrong \code{nunique} can cause undefined behaviour up to a crash.}

\item{method}{NULL for automatic method selection or a suitable low-level
method, see details}
}
\value{
By default (with \code{return="table"}) \code{\link[=table]{table()}} returns a
\emph{contingency table}, an object of class \code{"table"}, an array of integer values.
Note that unlike S the result is always an array, a 1D array if one factor is
given. Note also that for multidimensional arrays this is a \emph{dense} return
structure which can dramatically increase RAM requirements (for large arrays
with high mutual information, i.e. many possible input combinations of which
only few occur) and that \code{\link[=table]{table()}} is limited to \code{2^31} possible combinations
(e.g. two input vectors with 46340 unique values only). Finally note that the
tabulated values or value-combinations are represented as \code{dimnames} and that
the implied conversion of values to strings can cause \emph{severe} performance
problems since each string needs to be integrated into R's global string cache.

You can use the other \verb{return=} options to cope with these problems, the potential
combination limit is increased from \code{2^31} to \code{2^63} with these options, RAM is
only required for observed combinations and string conversion is avoided.

With \code{return="data.frame"} you get a \emph{dense} representation as a \code{\link[=data.frame]{data.frame()}}
(like that resulting from \code{as.data.frame(table(...))}) where only observed
combinations are listed (each as a data.frame row) with the corresponding
frequency counts (the latter as component named by \code{responseName}). This is
the inverse of \code{\link[=xtabs]{xtabs()}}.

With \code{return="list"} you also get a \emph{dense} representation as a simple
\code{\link[=list]{list()}} with components
\itemize{
\item \code{values} a integer64 vector of the technically tabulated values, for 1D this
is the tabulated values themselves, for kD these are the values representing
the potential combinations of input values
\item \code{counts} the frequency counts
\item \code{dims} only for kD: a list with the vectors of the unique values of the
input dimensions
}
}
\description{
\code{table.integer64} uses the cross-classifying integer64 vectors to build a
contingency table of the counts at each combination of vector values.
}
\details{
If at least one argument of \code{...} is integer64 and the remaining arguments of \code{...}
are integer64 or integer the \code{table.integer64} method is used. Only this method
supports the arguments \code{return}, \code{order}, \code{nunique}, and \code{method}.

This function automatically chooses from several low-level functions considering
the size of \code{x} and the availability of a cache.

Suitable methods are
\itemize{
\item \code{\link{hashmaptab}} (simultaneously creating and using a hashmap)
\item \code{\link{hashtab}} (first creating a hashmap then using it)
\item \code{\link{sortordertab}} (fast ordering)
\item \code{\link{ordertab}} (memory saving ordering).
}

If the argument \code{dnn} is not supplied, the internal function
\code{list.names} is called to compute the 'dimname names'.  If the
arguments in \code{...} are named, those names are used.  For the
remaining arguments, \code{deparse.level = 0} gives an empty name,
\code{deparse.level = 1} uses the supplied argument if it is a symbol,
and \code{deparse.level = 2} will deparse the argument.
}
\note{
Note that by using \code{\link[=as.integer64.factor]{as.integer64.factor()}} we can also input
factors into \code{table.integer64} -- only the \code{\link[=levels]{levels()}} get lost.
}
\examples{
message("pure integer64 examples")
x <- as.integer64(sample(c(rep(NA, 9), 1:9), 32, TRUE))
y <- as.integer64(sample(c(rep(NA, 9), 1:9), 32, TRUE))
z <- sample(c(rep(NA, 9), letters), 32, TRUE)
table(x)
table(x, order="counts")
table(x, y)
table(x, y, return="data.frame")

message("via as.integer64.factor we can use 'table.integer64' also for factors")
table(x, as.integer64(as.factor(z)))
}
\seealso{
\code{\link[=table]{table()}} for more info on the standard version coping with Base R's
data types, \code{\link[=tabulate]{tabulate()}} which can faster tabulate \code{\link{integer}}s with a limited
range \verb{[1L .. nL not too big]}, \code{\link[=unique.integer64]{unique.integer64()}} for the unique values
without counting them and \code{\link[=unipos.integer64]{unipos.integer64()}} for the positions of the unique values.
}
\concept{contingency table}
\concept{counts}
\concept{frequencies}
\concept{occurrences}
\keyword{category}

\name{keypos}
\alias{keypos}
\alias{keypos.integer64}
\title{Extract Positions in redundant dimension table}
\description{
  \code{keypos} returns the positions of the (fact table) elements that participate in their sorted unique subset (dimension table)
}
\usage{
keypos(x, \dots)
\method{keypos}{integer64}(x, method = NULL, \dots)
}
\arguments{
  \item{x}{a vector or a data frame or an array or \code{NULL}.}
  \item{method}{
	NULL for automatic method selection or a suitable low-level method, see details
}
  \item{\dots}{ignored}
}
\details{
  NAs are sorted first in the dimension table, see \code{\link{ramorder.integer64}}.
  \cr
  This function automatically chooses from several low-level functions considering the size of \code{x} and the availability of a cache. 
  Suitable methods are \code{\link{sortorderkey}} (fast ordering) 
and \code{\link{orderkey}} (memory saving ordering).
}
\value{
  an integer vector of the same length as code{x} containing positions relativ to code{sort(unique(x), na.last=FALSE)}
}
\author{
	Jens Oehlschlägel <Jens.Oehlschlaegel@truecluster.com>
}
\seealso{
  \code{\link{unique.integer64}} for the unique subset and \code{\link{match.integer64}} for finding positions in a different vector.
}
\examples{
x <- as.integer64(sample(c(rep(NA, 9), 1:9), 32, TRUE))
keypos(x)

stopifnot(identical(keypos(x),  match.integer64(x, sort(unique(x), na.last=FALSE))))
}
\keyword{manip}
\keyword{univar}

\name{sortnut}
\alias{sortnut}
\alias{sortnut.integer64}
\alias{ordernut}
\alias{ordernut.integer64}
\alias{sortfin}
\alias{sortfin.integer64}
\alias{orderpos}
\alias{orderpos.integer64}
\alias{orderfin}
\alias{orderfin.integer64}
\alias{sortorderpos}
\alias{sortorderpos.integer64}
\alias{orderdup}
\alias{orderdup.integer64}
\alias{sortorderdup}
\alias{sortorderdup.integer64}
\alias{sortuni}
\alias{sortuni.integer64}
\alias{orderuni}
\alias{orderuni.integer64}
\alias{sortorderuni}
\alias{sortorderuni.integer64}
\alias{orderupo}
\alias{orderupo.integer64}
\alias{sortorderupo}
\alias{sortorderupo.integer64}
\alias{ordertie}
\alias{ordertie.integer64}
\alias{sortordertie}
\alias{sortordertie.integer64}
\alias{sorttab}
\alias{sorttab.integer64}
\alias{ordertab}
\alias{ordertab.integer64}
\alias{sortordertab}
\alias{sortordertab.integer64}
\alias{orderkey}
\alias{orderkey.integer64}
\alias{sortorderkey}
\alias{sortorderkey.integer64}
\alias{orderrnk}
\alias{orderrnk.integer64}
\alias{sortorderrnk}
\alias{sortorderrnk.integer64}
\alias{sortqtl}
\alias{sortqtl.integer64}
\alias{orderqtl}
\alias{orderqtl.integer64}
\title{
   Searching and other uses of sorting for 64bit integers
}
\description{
  This is roughly an implementation of hash functionality but based on sorting instead on a hasmap.
  Since sorting is more informative than hashingwe can do some more interesting things.
}
\usage{
sortnut(sorted, \dots)
ordernut(table, order, \dots)
sortfin(sorted, x, \dots)
orderfin(table, order, x, \dots)
orderpos(table, order, x, \dots)
sortorderpos(sorted, order, x, \dots)
orderdup(table, order, \dots)
sortorderdup(sorted, order, \dots)
sortuni(sorted, nunique, \dots)
orderuni(table, order, nunique, \dots)
sortorderuni(table, sorted, order, nunique, \dots)
orderupo(table, order, nunique, \dots)
sortorderupo(sorted, order, nunique, keep.order = FALSE, \dots)
ordertie(table, order, nties, \dots)
sortordertie(sorted, order, nties, \dots)
sorttab(sorted, nunique, \dots)
ordertab(table, order, nunique, \dots)
sortordertab(sorted, order, \dots)
orderkey(table, order, na.skip.num = 0L, \dots)
sortorderkey(sorted, order, na.skip.num = 0L, \dots)
orderrnk(table, order, na.count, \dots)
sortorderrnk(sorted, order, na.count, \dots)
\method{sortnut}{integer64}(sorted, \dots)
\method{ordernut}{integer64}(table, order, \dots)
\method{sortfin}{integer64}(sorted, x, method=NULL, \dots)
\method{orderfin}{integer64}(table, order, x, method=NULL, \dots)
\method{orderpos}{integer64}(table, order, x, nomatch=NA, method=NULL, \dots)
\method{sortorderpos}{integer64}(sorted, order, x, nomatch=NA, method=NULL, \dots)
\method{orderdup}{integer64}(table, order, method=NULL, \dots)
\method{sortorderdup}{integer64}(sorted, order, method=NULL, \dots)
\method{sortuni}{integer64}(sorted, nunique, \dots)
\method{orderuni}{integer64}(table, order, nunique, keep.order=FALSE, \dots)
\method{sortorderuni}{integer64}(table, sorted, order, nunique, \dots)
\method{orderupo}{integer64}(table, order, nunique, keep.order=FALSE, \dots)
\method{sortorderupo}{integer64}(sorted, order, nunique, keep.order = FALSE, \dots)
\method{ordertie}{integer64}(table, order, nties, \dots)
\method{sortordertie}{integer64}(sorted, order, nties, \dots)
\method{sorttab}{integer64}(sorted, nunique, \dots)
\method{ordertab}{integer64}(table, order, nunique, denormalize=FALSE, keep.order=FALSE, \dots)
\method{sortordertab}{integer64}(sorted, order, denormalize=FALSE, \dots)
\method{orderkey}{integer64}(table, order, na.skip.num = 0L, \dots)
\method{sortorderkey}{integer64}(sorted, order, na.skip.num = 0L, \dots)
\method{orderrnk}{integer64}(table, order, na.count, \dots)
\method{sortorderrnk}{integer64}(sorted, order, na.count, \dots)
\method{sortqtl}{integer64}(sorted, na.count, probs, \dots)
\method{orderqtl}{integer64}(table, order, na.count, probs, \dots)
}
\arguments{
  \item{x}{ an \code{\link{integer64}} vector }
  \item{sorted}{ a sorted \code{\link{integer64}} vector }
  \item{table}{ the original data with original order under the sorted vector }
  \item{order}{ an \code{\link{integer}} order vector that turns 'table' into 'sorted' }
  \item{nunique}{ number of unique elements, usually we get this from cache or call \code{sortnut} or \code{ordernut} }
  \item{nties}{ number of tied values, usually we get this from cache or call \code{sortnut} or \code{ordernut} }
  \item{denormalize}{ FALSE returns counts of unique values, TRUE returns each value with its counts }
  \item{nomatch}{ the value to be returned if an element is not found in the hashmap }
  \item{keep.order}{ determines order of results and speed: \code{FALSE} (the default) is faster and returns in sorted order, \code{TRUE} returns in the order of first appearance in the original data, but this requires extra work } 
  \item{probs}{ vector of probabilities in [0..1] for which we seek quantiles }
  \item{na.skip.num}{ 0 or the number of \code{NA}s. With 0, \code{NA}s are coded with 1L, with the number of \code{NA}s, these are coded with \code{NA}, the latter needed for \code{\link{as.factor.integer64}} }
  \item{na.count}{ the number of \code{NA}s, needed for this low-level function algorithm }
  \item{method}{ see details }
  \item{\dots}{ further arguments, passed from generics, ignored in methods }
}
\details{
\tabular{rrrrl}{
   \bold{sortfun} \tab \bold{orderfun} \tab \bold{sortorderfun} \tab \bold{see also}          \tab \bold{description} \cr
   \code{sortnut} \tab \code{ordernut} \tab                     \tab  \tab return number of tied and of unique values \cr
   \code{sortfin} \tab \code{orderfin} \tab                     \tab \code{\link{\%in\%.integer64}} \tab return logical whether \code{x} is in \code{table} \cr
                  \tab \code{orderpos} \tab \code{sortorderpos} \tab \code{\link[=match.integer64]{match}} \tab return positions of \code{x} in \code{table} \cr
                  \tab \code{orderdup} \tab \code{sortorderdup} \tab \code{\link[=duplicated.integer64]{duplicated}} \tab return logical whether values are duplicated \cr
   \code{sortuni} \tab \code{orderuni} \tab \code{sortorderuni} \tab \code{\link[=unique.integer64]{unique}} \tab return unique values (=dimensiontable) \cr
                  \tab \code{orderupo} \tab \code{sortorderupo} \tab \code{\link[=unique.integer64]{unique}} \tab return positions of unique values \cr
                  \tab \code{ordertie} \tab \code{sortordertie} \tab  \tab return positions of tied values \cr
                  \tab \code{orderkey} \tab \code{sortorderkey} \tab  \tab positions of values in vector of unique values (match in dimensiontable) \cr
   \code{sorttab} \tab \code{ordertab} \tab \code{sortordertab} \tab \code{\link[=table.integer64]{table}} \tab tabulate frequency of values  \cr
                  \tab \code{orderrnk} \tab \code{sortorderrnk} \tab  \tab rank averaging ties \cr
   \code{sortqtl} \tab \code{orderqtl} \tab                     \tab  \tab return quantiles given probabilities \cr
}
The functions \code{sortfin}, \code{orderfin}, \code{orderpos} and \code{sortorderpos} each offer three algorithms for finding \code{x} in \code{table}.  \cr
With \code{method=1L} each value of \code{x} is searched independently using \emph{binary search}, this is fastest for small \code{table}s. \cr
With \code{method=2L} the values of \code{x} are first sorted and then searched using \emph{doubly exponential search}, this is the best allround method. \cr
With \code{method=3L} the values of \code{x} are first sorted and then searched using simple merging, this is the fastest method if \code{table} is huge and \code{x} has similar size and distribution of values. \cr
With \code{method=NULL} the functions use a heuristic to determine the fastest algorithm. \cr

The functions \code{orderdup} and \code{sortorderdup} each offer two algorithms for setting the truth values in the return vector.  \cr
With \code{method=1L} the return values are set directly which causes random write access on a possibly large return vector. \cr
With \code{method=2L} the return values are first set in a smaller bit-vector -- random access limited to a smaller memory region -- and finally written sequentially to the logical output  vector. \cr
With \code{method=NULL} the functions use a heuristic to determine the fastest algorithm. \cr
}
\value{
  see details
}
\author{
Jens Oehlschlägel <Jens.Oehlschlaegel@truecluster.com>
}
\keyword{ programming }
\keyword{ manip }
\seealso{ \code{\link[=match.integer64]{match}} }
\examples{
 message("check the code of 'optimizer64' for examples:")
 print(optimizer64)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sortuse64.R
\name{sortnut}
\alias{sortnut}
\alias{sortnut.integer64}
\alias{ordernut}
\alias{ordernut.integer64}
\alias{sortfin}
\alias{sortfin.integer64}
\alias{orderfin}
\alias{orderfin.integer64}
\alias{orderpos}
\alias{orderpos.integer64}
\alias{sortorderpos}
\alias{sortorderpos.integer64}
\alias{orderdup}
\alias{orderdup.integer64}
\alias{sortorderdup}
\alias{sortorderdup.integer64}
\alias{sortuni}
\alias{sortuni.integer64}
\alias{orderuni}
\alias{orderuni.integer64}
\alias{sortorderuni}
\alias{sortorderuni.integer64}
\alias{orderupo}
\alias{orderupo.integer64}
\alias{sortorderupo}
\alias{sortorderupo.integer64}
\alias{ordertie}
\alias{ordertie.integer64}
\alias{sortordertie}
\alias{sortordertie.integer64}
\alias{sorttab}
\alias{sorttab.integer64}
\alias{ordertab}
\alias{ordertab.integer64}
\alias{sortordertab}
\alias{sortordertab.integer64}
\alias{orderkey}
\alias{orderkey.integer64}
\alias{sortorderkey}
\alias{sortorderkey.integer64}
\alias{orderrnk}
\alias{orderrnk.integer64}
\alias{sortorderrnk}
\alias{sortorderrnk.integer64}
\alias{sortqtl}
\alias{sortqtl.integer64}
\alias{orderqtl}
\alias{orderqtl.integer64}
\title{Searching and other uses of sorting for 64bit integers}
\usage{
sortnut(sorted, ...)

\method{sortnut}{integer64}(sorted, ...)

ordernut(table, order, ...)

\method{ordernut}{integer64}(table, order, ...)

sortfin(sorted, x, ...)

\method{sortfin}{integer64}(sorted, x, method = NULL, ...)

orderfin(table, order, x, ...)

\method{orderfin}{integer64}(table, order, x, method = NULL, ...)

orderpos(table, order, x, ...)

\method{orderpos}{integer64}(table, order, x, nomatch = NA, method = NULL, ...)

sortorderpos(sorted, order, x, ...)

\method{sortorderpos}{integer64}(sorted, order, x, nomatch = NA, method = NULL, ...)

orderdup(table, order, ...)

\method{orderdup}{integer64}(table, order, method = NULL, ...)

sortorderdup(sorted, order, ...)

\method{sortorderdup}{integer64}(sorted, order, method = NULL, ...)

sortuni(sorted, nunique, ...)

\method{sortuni}{integer64}(sorted, nunique, ...)

orderuni(table, order, nunique, ...)

\method{orderuni}{integer64}(table, order, nunique, keep.order = FALSE, ...)

sortorderuni(table, sorted, order, nunique, ...)

\method{sortorderuni}{integer64}(table, sorted, order, nunique, ...)

orderupo(table, order, nunique, ...)

\method{orderupo}{integer64}(table, order, nunique, keep.order = FALSE, ...)

sortorderupo(sorted, order, nunique, keep.order = FALSE, ...)

\method{sortorderupo}{integer64}(sorted, order, nunique, keep.order = FALSE, ...)

ordertie(table, order, nties, ...)

\method{ordertie}{integer64}(table, order, nties, ...)

sortordertie(sorted, order, nties, ...)

\method{sortordertie}{integer64}(sorted, order, nties, ...)

sorttab(sorted, nunique, ...)

\method{sorttab}{integer64}(sorted, nunique, ...)

ordertab(table, order, nunique, ...)

\method{ordertab}{integer64}(table, order, nunique, denormalize = FALSE, keep.order = FALSE, ...)

sortordertab(sorted, order, ...)

\method{sortordertab}{integer64}(sorted, order, denormalize = FALSE, ...)

orderkey(table, order, na.skip.num = 0L, ...)

\method{orderkey}{integer64}(table, order, na.skip.num = 0L, ...)

sortorderkey(sorted, order, na.skip.num = 0L, ...)

\method{sortorderkey}{integer64}(sorted, order, na.skip.num = 0L, ...)

orderrnk(table, order, na.count, ...)

\method{orderrnk}{integer64}(table, order, na.count, ...)

sortorderrnk(sorted, order, na.count, ...)

\method{sortorderrnk}{integer64}(sorted, order, na.count, ...)

sortqtl(sorted, na.count, probs, ...)

\method{sortqtl}{integer64}(sorted, na.count, probs, ...)

orderqtl(table, order, na.count, probs, ...)

\method{orderqtl}{integer64}(table, order, na.count, probs, ...)
}
\arguments{
\item{sorted}{a sorted \code{\link{integer64}} vector}

\item{...}{further arguments, passed from generics, ignored in methods}

\item{table}{the original data with original order under the sorted vector}

\item{order}{an \code{\link{integer}} order vector that turns 'table' into 'sorted'}

\item{x}{an \code{\link{integer64}} vector}

\item{method}{see Details}

\item{nomatch}{the value to be returned if an element is not found in the hashmap}

\item{nunique}{number of unique elements, usually we get this from cache
or call \code{sortnut} or \code{ordernut}}

\item{keep.order}{determines order of results and speed: \code{FALSE} (the default)
is faster and returns in sorted order, \code{TRUE} returns in the order of first
appearance in the original data, but this requires extra work}

\item{nties}{number of tied values, usually we get this from cache or
call \code{sortnut} or \code{ordernut}}

\item{denormalize}{FALSE returns counts of unique values, TRUE returns each
value with its counts}

\item{na.skip.num}{0 or the number of \code{NA}s. With 0, \code{NA}s are coded with 1L,
with the number of \code{NA}s, these are coded with \code{NA}}

\item{na.count}{the number of \code{NA}s, needed for this low-level function algorithm}

\item{probs}{vector of probabilities in \verb{[0..1]} for which we seek quantiles}
}
\value{
see details
}
\description{
This is roughly an implementation of hash functionality but based on sorting
instead on a hashmap. Since sorting is more informative than hashing we
can do some more interesting things.
}
\details{
\tabular{rrrrl}{
   \strong{sortfun} \tab \strong{orderfun} \tab \strong{sortorderfun} \tab \strong{see also} \tab \strong{description} \cr
   \code{sortnut} \tab \code{ordernut} \tab  \tab  \tab return number of tied and of unique values \cr
   \code{sortfin} \tab \code{orderfin} \tab  \tab \code{\link[=match.integer64]{\%in\%.integer64}} \tab return logical whether \code{x} is in \code{table} \cr
    \tab \code{orderpos} \tab \code{sortorderpos} \tab \code{\link[=match.integer64]{match()}} \tab return positions of \code{x} in \code{table} \cr
    \tab \code{orderdup} \tab \code{sortorderdup} \tab \code{\link[=duplicated.integer64]{duplicated()}} \tab return logical whether values are duplicated \cr
   \code{sortuni} \tab \code{orderuni} \tab \code{sortorderuni} \tab \code{\link[=unique.integer64]{unique()}} \tab return unique values (=dimensiontable) \cr
    \tab \code{orderupo} \tab \code{sortorderupo} \tab \code{\link[=unique.integer64]{unique()}} \tab return positions of unique values \cr
    \tab \code{ordertie} \tab \code{sortordertie} \tab  \tab return positions of tied values \cr
    \tab \code{orderkey} \tab \code{sortorderkey} \tab  \tab positions of values in vector of unique values (match in dimensiontable) \cr
   \code{sorttab} \tab \code{ordertab} \tab \code{sortordertab} \tab \code{\link[=table]{table()}} \tab tabulate frequency of values \cr
    \tab \code{orderrnk} \tab \code{sortorderrnk} \tab  \tab rank averaging ties \cr
   \code{sortqtl} \tab \code{orderqtl} \tab  \tab  \tab return quantiles given probabilities \cr
}


The functions \code{sortfin}, \code{orderfin}, \code{orderpos} and \code{sortorderpos} each
offer three algorithms for finding \code{x} in \code{table}.

With \code{method=1L} each value of \code{x} is searched independently using
\emph{binary search}, this is fastest for small \code{table}s.

With \code{method=2L} the values of \code{x} are first sorted and then searched using
\emph{doubly exponential search}, this is the best all-around method.

With \code{method=3L} the values of \code{x} are first sorted and then searched using
simple merging, this is the fastest method if \code{table} is huge and \code{x} has
similar size and distribution of values.

With \code{method=NULL} the functions use a heuristic to determine the fastest
algorithm.

The functions \code{orderdup} and \code{sortorderdup} each offer two algorithms for
setting the truth values in the return vector.

With \code{method=1L} the return values are set directly which causes random
write access on a possibly large return vector.

With \code{method=2L} the return values are first set in a smaller bit-vector --
random access limited to a smaller memory region -- and finally written
sequentially to the logical output  vector.

With \code{method=NULL} the functions use a heuristic to determine the fastest
algorithm.
}
\examples{
 message("check the code of 'optimizer64' for examples:")
 print(optimizer64)
}
\seealso{
\code{\link[=match.integer64]{match()}}
}
\keyword{manip}
\keyword{programming}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hash64.R
\name{hashmap}
\alias{hashmap}
\alias{hashfun}
\alias{hashfun.integer64}
\alias{hashmap.integer64}
\alias{hashpos}
\alias{hashpos.cache_integer64}
\alias{hashrev}
\alias{hashrev.cache_integer64}
\alias{hashfin}
\alias{hashfin.cache_integer64}
\alias{hashrin}
\alias{hashrin.cache_integer64}
\alias{hashdup}
\alias{hashdup.cache_integer64}
\alias{hashuni}
\alias{hashuni.cache_integer64}
\alias{hashupo}
\alias{hashupo.cache_integer64}
\alias{hashtab}
\alias{hashtab.cache_integer64}
\alias{hashmaptab}
\alias{hashmaptab.integer64}
\alias{hashmapuni}
\alias{hashmapuni.integer64}
\alias{hashmapupo}
\alias{hashmapupo.integer64}
\title{Hashing for 64bit integers}
\usage{
hashfun(x, ...)

\method{hashfun}{integer64}(x, minfac = 1.41, hashbits = NULL, ...)

hashmap(x, ...)

\method{hashmap}{integer64}(x, nunique = NULL, minfac = 1.41, hashbits = NULL, cache = NULL, ...)

hashpos(cache, ...)

\method{hashpos}{cache_integer64}(cache, x, nomatch = NA_integer_, ...)

hashrev(cache, ...)

\method{hashrev}{cache_integer64}(cache, x, nomatch = NA_integer_, ...)

hashfin(cache, ...)

\method{hashfin}{cache_integer64}(cache, x, ...)

hashrin(cache, ...)

\method{hashrin}{cache_integer64}(cache, x, ...)

hashdup(cache, ...)

\method{hashdup}{cache_integer64}(cache, ...)

hashuni(cache, ...)

\method{hashuni}{cache_integer64}(cache, keep.order = FALSE, ...)

hashupo(cache, ...)

\method{hashupo}{cache_integer64}(cache, keep.order = FALSE, ...)

hashtab(cache, ...)

\method{hashtab}{cache_integer64}(cache, ...)

hashmaptab(x, ...)

\method{hashmaptab}{integer64}(x, nunique = NULL, minfac = 1.5, hashbits = NULL, ...)

hashmapuni(x, ...)

\method{hashmapuni}{integer64}(x, nunique = NULL, minfac = 1.5, hashbits = NULL, ...)

hashmapupo(x, ...)

\method{hashmapupo}{integer64}(x, nunique = NULL, minfac = 1.5, hashbits = NULL, ...)
}
\arguments{
\item{x}{an integer64 vector}

\item{...}{further arguments, passed from generics, ignored in methods}

\item{minfac}{minimum factor by which the hasmap has more elements compared to the data \code{x},
ignored if \code{hashbits} is given directly}

\item{hashbits}{length of hashmap is \code{2^hashbits}}

\item{nunique}{giving \emph{correct} number of unique elements can help reducing the size of the hashmap}

\item{cache}{an optional \code{\link[=cache]{cache()}} object into which to put the hashmap (by default a new cache is created}

\item{nomatch}{the value to be returned if an element is not found in the hashmap}

\item{keep.order}{determines order of results and speed: \code{FALSE} (the default) is faster and returns in the
(pseudo)random order of the hash function, \code{TRUE} returns in the order of first appearance in the original
data, but this requires extra work}
}
\value{
See Details
}
\description{
This is an explicit implementation of hash functionality that underlies
matching and other functions in R. Explicit means that you can create,
store and use hash functionality directly. One advantage is that you can
re-use hashmaps, which avoid re-building hashmaps again and again.
}
\details{
\tabular{rrl}{
   \strong{function} \tab \strong{see also} \tab \strong{description} \cr
   \code{hashfun} \tab \code{digest} \tab export of the hash function used in \code{hashmap} \cr
   \code{hashmap} \tab \code{\link[=match.integer64]{match()}} \tab return hashmap \cr
   \code{hashpos} \tab \code{\link[=match.integer64]{match()}} \tab return positions of \code{x} in \code{hashmap} \cr
   \code{hashrev} \tab \code{\link[=match.integer64]{match()}} \tab return positions of \code{hashmap} in \code{x} \cr
   \code{hashfin} \tab \code{\link[=match.integer64]{\%in\%.integer64}} \tab return logical whether \code{x} is in \code{hashmap} \cr
   \code{hashrin} \tab \code{\link[=match.integer64]{\%in\%.integer64}} \tab return logical whether \code{hashmap} is in \code{x} \cr
   \code{hashdup} \tab \code{\link[=duplicated.integer64]{duplicated()}} \tab return logical whether hashdat is duplicated using hashmap \cr
   \code{hashuni} \tab \code{\link[=unique.integer64]{unique()}} \tab return unique values of hashmap \cr
   \code{hashmapuni} \tab \code{\link[=unique.integer64]{unique()}} \tab return unique values of \code{x} \cr
   \code{hashupo} \tab \code{\link[=unique.integer64]{unique()}} \tab return positions of unique values in hashdat \cr
   \code{hashmapupo} \tab \code{\link[=unique.integer64]{unique()}} \tab return positions of unique values in \code{x} \cr
   \code{hashtab} \tab \code{\link[=table]{table()}} \tab tabulate values of hashdat using hashmap in \code{keep.order=FALSE} \cr
   \code{hashmaptab} \tab \code{\link[=table]{table()}} \tab tabulate values of \code{x} building hasmap on the fly in \code{keep.order=FALSE} \cr
}
}
\examples{
x <- as.integer64(sample(c(NA, 0:9)))
y <- as.integer64(sample(c(NA, 1:9), 10, TRUE))
hashfun(y)
hx <- hashmap(x)
hy <- hashmap(y)
ls(hy)
hashpos(hy, x)
hashrev(hx, y)
hashfin(hy, x)
hashrin(hx, y)
hashdup(hy)
hashuni(hy)
hashuni(hy, keep.order=TRUE)
hashmapuni(y)
hashupo(hy)
hashupo(hy, keep.order=TRUE)
hashmapupo(y)
hashtab(hy)
hashmaptab(y)

    \dontrun{
    message("explore speed given size of the hasmap in 2^hashbits and size of the data")
    message("more hashbits means more random access and less collisions")
    message("i.e. more data means less random access and more collisions")
    bits <- 24
    b <- seq(-1, 0, 0.1)
    tim <- matrix(NA, length(b), 2, dimnames=list(b, c("bits", "bits+1")))
    for (i in 1:length(b)) {
      n <- as.integer(2^(bits+b[i]))
      x <- as.integer64(sample(n))
      tim[i, 1] <- repeat.time(hashmap(x, hashbits=bits))[3]
      tim[i, 2] <- repeat.time(hashmap(x, hashbits=bits+1))[3]
      print(tim)
      matplot(b, tim)
    }
    message("we conclude that n*sqrt(2) is enough to avoid collisions")
    }
}
\seealso{
\code{\link[=match.integer64]{match()}}, \code{\link[=runif64]{runif64()}}
}
\keyword{manip}
\keyword{programming}

\name{sum.integer64}
\alias{all.integer64}
\alias{any.integer64}
\alias{min.integer64}
\alias{max.integer64}
\alias{range.integer64}
\alias{lim.integer64}
\alias{sum.integer64}
\alias{prod.integer64}
\title{
   Summary functions for integer64 vectors
}
\description{
  Summary functions for integer64 vectors. 
  Function 'range' without arguments returns the smallest and largest value of the 'integer64' class.
}
\usage{
\method{all}{integer64}(\dots, na.rm = FALSE)
\method{any}{integer64}(\dots, na.rm = FALSE)
\method{min}{integer64}(\dots, na.rm = FALSE)
\method{max}{integer64}(\dots, na.rm = FALSE)
\method{range}{integer64}(\dots, na.rm = FALSE, finite = FALSE)
lim.integer64()
\method{sum}{integer64}(\dots, na.rm = FALSE)
\method{prod}{integer64}(\dots, na.rm = FALSE)
}
\arguments{
  \item{\dots}{ atomic vectors of class 'integer64'}
  \item{na.rm}{ logical scalar indicating whether to ignore NAs }
  \item{finite}{ logical scalar indicating whether to ignore NAs (just for compatibility with \code{\link{range.default}}) }
}
\details{
  The numerical summary methods always return \code{integer64}. 
  Therefor the methods for \code{min},\code{max} and \code{range} do not return \code{+Inf,-Inf}
  on empty arguments, but \code{+9223372036854775807, -9223372036854775807} (in this sequence).
  The same is true if only  \code{NA}s are submitted with argument \code{na.rm=TRUE}. 
 \cr
  \code{lim.integer64} returns these limits in proper order \code{-9223372036854775807, +9223372036854775807} and without a \code{\link{warning}}.
}
\value{
  \code{\link{all}} and \code{\link{any}} return a logical scalar\cr
  \code{\link{range}} returns a integer64 vector with two elements\cr
  \code{\link{min}}, \code{\link{max}}, \code{\link{sum}} and \code{\link{prod}} return a integer64 scalar
}
\author{
Jens Oehlschlägel <Jens.Oehlschlaegel@truecluster.com>
}
\keyword{ classes }
\keyword{ manip }
\seealso{ \code{\link{mean.integer64}} \code{\link{cumsum.integer64}} \code{\link{integer64}}  }
\examples{
  lim.integer64()
  range(as.integer64(1:12))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/integer64.R
\name{sum.integer64}
\alias{sum.integer64}
\alias{any.integer64}
\alias{all.integer64}
\alias{prod.integer64}
\alias{min.integer64}
\alias{max.integer64}
\alias{range.integer64}
\alias{lim.integer64}
\title{Summary functions for integer64 vectors}
\usage{
\method{any}{integer64}(..., na.rm = FALSE)

\method{all}{integer64}(..., na.rm = FALSE)

\method{sum}{integer64}(..., na.rm = FALSE)

\method{prod}{integer64}(..., na.rm = FALSE)

\method{min}{integer64}(..., na.rm = FALSE)

\method{max}{integer64}(..., na.rm = FALSE)

\method{range}{integer64}(..., na.rm = FALSE, finite = FALSE)

lim.integer64()
}
\arguments{
\item{...}{atomic vectors of class 'integer64'}

\item{na.rm}{logical scalar indicating whether to ignore NAs}

\item{finite}{logical scalar indicating whether to ignore NAs (just for
compatibility with \code{\link[=range.default]{range.default()}})}
}
\value{
\code{\link[=all]{all()}} and \code{\link[=any]{any()}} return a logical scalar

\code{\link[=range]{range()}} returns a integer64 vector with two elements

\code{\link[=min]{min()}}, \code{\link[=max]{max()}}, \code{\link[=sum]{sum()}} and \code{\link[=prod]{prod()}} return a integer64 scalar
}
\description{
Summary functions for integer64 vectors. Function 'range' without arguments
returns the smallest and largest value of the 'integer64' class.
}
\details{
The numerical summary methods always return \code{integer64}. Wherever integer methods would
return \code{Inf} (or its negation), here the extreme 64-bit integer \code{9223372036854775807} is
returned. See \code{\link[=min]{min()}} for more details about the behavior.

\code{lim.integer64} returns these limits in proper order
\verb{-9223372036854775807, +9223372036854775807} and without a \code{\link[=warning]{warning()}}.
}
\examples{
  lim.integer64()
  range(as.integer64(1:12))
}
\seealso{
\code{\link[=mean.integer64]{mean.integer64()}} \code{\link[=cumsum.integer64]{cumsum.integer64()}} \code{\link[=integer64]{integer64()}}
}
\keyword{classes}
\keyword{manip}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/integer64.R
\name{as.character.integer64}
\alias{as.character.integer64}
\alias{as.bitstring}
\alias{as.double.integer64}
\alias{as.numeric.integer64}
\alias{as.complex.integer64}
\alias{as.integer.integer64}
\alias{as.raw.integer64}
\alias{as.logical.integer64}
\alias{as.bitstring.integer64}
\alias{as.Date.integer64}
\alias{as.POSIXct.integer64}
\alias{as.POSIXlt.integer64}
\alias{as.factor}
\alias{as.ordered}
\alias{as.list.integer64}
\title{Coerce from integer64}
\usage{
as.bitstring(x, ...)

\method{as.double}{integer64}(x, ...)

\method{as.numeric}{integer64}(x, ...)

\method{as.complex}{integer64}(x, ...)

\method{as.integer}{integer64}(x, ...)

\method{as.raw}{integer64}(x, ...)

\method{as.logical}{integer64}(x, ...)

\method{as.character}{integer64}(x, ...)

\method{as.bitstring}{integer64}(x, ...)

\method{as.Date}{integer64}(x, origin, ...)

\method{as.POSIXct}{integer64}(x, tz = "", origin, ...)

\method{as.POSIXlt}{integer64}(x, tz = "", origin, ...)

as.factor(x)

as.ordered(x)

\method{as.list}{integer64}(x, ...)
}
\arguments{
\item{x}{an integer64 vector}

\item{..., origin, tz}{further arguments to the \code{\link[=NextMethod]{NextMethod()}}}
}
\value{
\code{as.bitstring} returns a string of class 'bitstring'.

The other methods return atomic vectors of the expected types
}
\description{
Methods to coerce integer64 to other atomic types. 'as.bitstring' coerces
to a human-readable bit representation (strings of zeroes and ones).
The methods \code{\link[=format]{format()}}, \code{\link[=as.character]{as.character()}}, \code{\link[=as.double]{as.double()}},
\code{\link[=as.logical]{as.logical()}}, \code{\link[=as.integer]{as.integer()}} do what you would expect.
}
\examples{
  as.character(lim.integer64())
  as.bitstring(lim.integer64())
  as.bitstring(as.integer64(c(-2, -1, NA, 0:2)))
}
\seealso{
\code{\link[=as.integer64.character]{as.integer64.character()}} \code{\link[=integer64]{integer64()}}
}
\keyword{classes}
\keyword{manip}

\name{as.character.integer64}
\alias{as.character.integer64}
\alias{as.double.integer64}
\alias{as.integer.integer64}
\alias{as.logical.integer64}
\alias{as.bitstring}
\alias{print.bitstring}
\alias{as.bitstring.integer64}
\alias{as.factor.integer64}
\alias{as.ordered.integer64}
\title{
   Coerce from integer64
}
\description{
  Methods to coerce integer64 to other atomic types. 
  'as.bitstring' coerces to a human-readable bit representation (strings of zeroes and ones). 
  The methods \code{\link{format}}, \code{\link{as.character}}, \code{\link{as.double}},
  \code{\link{as.logical}}, \code{\link{as.integer}} do what you would expect.
}
\usage{
 as.bitstring(x, \dots)
 \method{as.bitstring}{integer64}(x, \dots)
 \method{print}{bitstring}(x, \dots)
 \method{as.character}{integer64}(x, \dots)
 \method{as.double}{integer64}(x, keep.names = FALSE, \dots)
 \method{as.integer}{integer64}(x, \dots)
 \method{as.logical}{integer64}(x, \dots)
 \method{as.factor}{integer64}(x)
 \method{as.ordered}{integer64}(x)
}
\arguments{
  \item{x}{ an integer64 vector }
  \item{keep.names}{ FALSE, set to TRUE to keep a names vector }
  \item{\dots}{ further arguments to the \code{\link{NextMethod}} }
}
\value{
  \code{as.bitstring} returns a string of class 'bitstring'. \cr
  The other methods return atomic vectors of the expected types
}
\author{
Jens Oehlschlägel <Jens.Oehlschlaegel@truecluster.com>
}
\keyword{ classes }
\keyword{ manip }
\seealso{ \code{\link{as.integer64.character}} \code{\link{integer64}}  }
\examples{
  as.character(lim.integer64())
  as.bitstring(lim.integer64())
  as.bitstring(as.integer64(c(
   -2,-1,NA,0:2
  )))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{benchmark64.data}
\alias{benchmark64.data}
\title{Results of performance measurement on a Core i7 Lenovo T410 8 GB RAM under Windows 7 64bit}
\format{
The format is:

\if{html}{\out{<div class="sourceCode">}}\preformatted{num [1:16, 1:6] 2.55e-05 2.37 2.39 1.28 1.39 ...
- attr(*, "dimnames")=List of 2
..$ : chr [1:16] "cache" "match(s,b)" "s \%in\% b" "match(b,s)" ...
..$ : chr [1:6] "32-bit" "64-bit" "hashcache" "sortordercache" ...
}\if{html}{\out{</div>}}
}
\usage{
data(benchmark64.data)
}
\description{
These are the results of calling \code{\link[=benchmark64]{benchmark64()}}
}
\examples{
data(benchmark64.data)
print(benchmark64.data)
matplot(log2(benchmark64.data[-1,1]/benchmark64.data[-1,])
, pch=c("3", "6", "h", "s", "o", "a")
, xlab="tasks [last=session]"
, ylab="log2(relative speed) [bigger is better]"
)
matplot(t(log2(benchmark64.data[-1,1]/benchmark64.data[-1,]))
, axes=FALSE
, type="b"
, lwd=c(rep(1, 14), 3)
, xlab="context"
, ylab="log2(relative speed) [bigger is better]"
)
axis(1
, labels=c("32-bit", "64-bit", "hash", "sortorder", "order", "hash+sortorder")
, at=1:6
)
axis(2)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/integer64.R
\name{format.integer64}
\alias{format.integer64}
\alias{sign.integer64}
\alias{abs.integer64}
\alias{sqrt.integer64}
\alias{log.integer64}
\alias{log10.integer64}
\alias{log2.integer64}
\alias{trunc.integer64}
\alias{floor.integer64}
\alias{ceiling.integer64}
\alias{signif.integer64}
\alias{scale.integer64}
\alias{round.integer64}
\alias{is.na.integer64}
\alias{is.finite.integer64}
\alias{is.infinite.integer64}
\alias{is.nan.integer64}
\title{Unary operators and functions for integer64 vectors}
\usage{
\method{format}{integer64}(x, justify = "right", ...)

\method{sign}{integer64}(x)

\method{abs}{integer64}(x)

\method{sqrt}{integer64}(x)

\method{log}{integer64}(x, base = NULL)

\method{log10}{integer64}(x)

\method{log2}{integer64}(x)

\method{trunc}{integer64}(x, ...)

\method{floor}{integer64}(x)

\method{ceiling}{integer64}(x)

\method{signif}{integer64}(x, digits = 6L)

\method{scale}{integer64}(x, center = TRUE, scale = TRUE)

\method{round}{integer64}(x, digits = 0L)

\method{is.na}{integer64}(x)

\method{is.finite}{integer64}(x)

\method{is.infinite}{integer64}(x)

\method{is.nan}{integer64}(x)
}
\arguments{
\item{x}{an atomic vector of class 'integer64'}

\item{justify}{should it be right-justified (the default), left-justified,
centred or left alone.}

\item{...}{further arguments to the \code{\link[=NextMethod]{NextMethod()}}}

\item{base}{an atomic scalar (we save 50\% log-calls by not allowing
a vector base)}

\item{digits}{integer indicating the number of decimal places (round)
or significant digits (signif) to be used. Negative values are allowed
(see \code{\link[=round]{round()}})}

\item{center}{see \code{\link[=scale]{scale()}}}

\item{scale}{see \code{\link[=scale]{scale()}}}
}
\value{
\code{\link[=format]{format()}} returns a character vector

\code{\link[=is.na]{is.na()}} and \code{\link{!}} return a logical vector

\code{\link[=sqrt]{sqrt()}}, \code{\link[=log]{log()}}, \code{\link[=log2]{log2()}} and \code{\link[=log10]{log10()}} return a double vector

\code{\link[=sign]{sign()}}, \code{\link[=abs]{abs()}}, \code{\link[=floor]{floor()}}, \code{\link[=ceiling]{ceiling()}}, \code{\link[=trunc]{trunc()}} and
\code{\link[=round]{round()}} return a vector of class 'integer64'

\code{\link[=signif]{signif()}} is not implemented
}
\description{
Unary operators and functions for integer64 vectors.
}
\examples{
  sqrt(as.integer64(1:12))
}
\seealso{
\link{ops64} \code{\link[=integer64]{integer64()}}
}
\keyword{classes}
\keyword{manip}

\name{format.integer64}
\alias{format.integer64}
\alias{is.na.integer64}
\alias{is.nan.integer64}
\alias{is.finite.integer64}
\alias{is.infinite.integer64}
\alias{!.integer64}
\alias{sign.integer64}
\alias{abs.integer64}
\alias{sqrt.integer64}
\alias{log.integer64}
\alias{log2.integer64}
\alias{log10.integer64}
\alias{floor.integer64}
\alias{ceiling.integer64}
\alias{trunc.integer64}
\alias{round.integer64}
\alias{signif.integer64}
\alias{scale.integer64}
\title{
   Unary operators and functions for integer64 vectors
}
\description{
  Unary operators and functions for integer64 vectors.
}
\usage{
\method{format}{integer64}(x, justify="right", \dots)
\method{is.na}{integer64}(x)
\method{is.nan}{integer64}(x)
\method{is.finite}{integer64}(x)
\method{is.infinite}{integer64}(x)
\method{!}{integer64}(x)
\method{sign}{integer64}(x)
\method{abs}{integer64}(x)
\method{sqrt}{integer64}(x)
\method{log}{integer64}(x, base)
\method{log2}{integer64}(x)
\method{log10}{integer64}(x)
\method{floor}{integer64}(x)
\method{ceiling}{integer64}(x)
\method{trunc}{integer64}(x, \dots)
\method{round}{integer64}(x, digits=0)
\method{signif}{integer64}(x, digits=6)
\method{scale}{integer64}(x, center = TRUE, scale = TRUE)
}
\arguments{
  \item{x}{ an atomic vector of class 'integer64'}
  \item{base}{ an atomic scalar (we save 50\% log-calls by not allowing a vector base) }
  \item{digits}{ integer indicating the number of decimal places (round) or significant digits (signif) to be used. 
                 Negative values are allowed (see \code{\link{round}}) }
  \item{justify}{ should it be right-justified (the default), left-justified, centred or left alone. }
  \item{center}{ see \code{\link{scale}} }
  \item{scale}{  see \code{\link{scale}} }
  \item{\dots}{ further arguments to the \code{\link{NextMethod}} }
}
\value{
  \code{\link{format}} returns a character vector \cr
  \code{\link{is.na}} and \code{\link{!}} return a logical vector \cr
  \code{\link{sqrt}}, \code{\link{log}}, \code{\link{log2}} and \code{\link{log10}} return a double vector \cr
  \code{\link{sign}}, \code{\link{abs}}, \code{\link{floor}}, \code{\link{ceiling}}, \code{\link{trunc}} and 
  \code{\link{round}} return a vector of class 'integer64' \cr
  \code{\link{signif}} is not implemented 
}
\author{
Jens Oehlschlägel <Jens.Oehlschlaegel@truecluster.com>
}
\keyword{ classes }
\keyword{ manip }
\seealso{ \code{\link{xor.integer64}} \code{\link{integer64}}  }
\examples{
  sqrt(as.integer64(1:12))
\dontshow{
i <- -999:999
for (s in -3:3){
r <- as.integer64(round(as.integer(i), s))
  r64 <- round(as.integer64(i), s)
  stopifnot(identical(r,r64))
}
}
}

\name{xor.integer64}
\alias{&.integer64}
\alias{|.integer64}
\alias{xor.integer64}
\alias{!=.integer64}
\alias{==.integer64}
\alias{<.integer64}
\alias{<=.integer64}
\alias{>.integer64}
\alias{>=.integer64}
\alias{+.integer64}
\alias{-.integer64}
\alias{*.integer64}
\alias{^.integer64}
\alias{/.integer64}
\alias{\%/\%.integer64}
\alias{\%\%.integer64}
\alias{binattr}
\title{
   Binary operators for integer64 vectors
}
\description{
  Binary operators for integer64 vectors.
}
\usage{
\method{&}{integer64}(e1,e2)
\method{|}{integer64}(e1,e2)
\method{xor}{integer64}(x,y)
\method{!=}{integer64}(e1,e2)
\method{==}{integer64}(e1,e2)
\method{<}{integer64}(e1,e2)
\method{<=}{integer64}(e1,e2)
\method{>}{integer64}(e1,e2)
\method{>=}{integer64}(e1,e2)
\method{+}{integer64}(e1,e2)
\method{-}{integer64}(e1,e2)
\method{*}{integer64}(e1,e2)
\method{^}{integer64}(e1,e2)
\method{/}{integer64}(e1,e2)
\method{\%/\%}{integer64}(e1,e2)
\method{\%\%}{integer64}(e1,e2)
binattr(e1,e2) # for internal use only
}
\arguments{
  \item{e1}{ an atomic vector of class 'integer64'}
  \item{e2}{ an atomic vector of class 'integer64'}
  \item{x}{ an atomic vector of class 'integer64'}
  \item{y}{ an atomic vector of class 'integer64'}
}
\value{
  \code{\link{&}}, \code{\link{|}}, \code{\link{xor}}, \code{\link{!=}}, \code{\link{==}}, 
  \code{\link{<}}, \code{\link{<=}}, \code{\link{>}}, \code{\link{>=}} return a logical vector \cr
  \code{\link{^}} and \code{\link{/}} return a double vector\cr
  \code{\link{+}}, \code{\link{-}}, \code{\link{*}}, \code{\link{\%/\%}}, \code{\link{\%\%}}
   return a vector of class 'integer64'
}
\author{
Jens Oehlschlägel <Jens.Oehlschlaegel@truecluster.com>
}
\keyword{ classes }
\keyword{ manip }
\seealso{ \code{\link{format.integer64}} \code{\link{integer64}}  }
\examples{
  as.integer64(1:12) - 1
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/integer64.R
\docType{data}
\name{as.integer64.character}
\alias{as.integer64.character}
\alias{as.integer64}
\alias{as.integer64.NULL}
\alias{as.integer64.integer64}
\alias{as.integer64.double}
\alias{as.integer64.complex}
\alias{as.integer64.integer}
\alias{as.integer64.raw}
\alias{as.integer64.logical}
\alias{as.integer64.factor}
\alias{as.integer64.Date}
\alias{as.integer64.POSIXct}
\alias{as.integer64.POSIXlt}
\alias{as.integer64.difftime}
\alias{as.integer64.bitstring}
\alias{NA_integer64_}
\title{Coerce to integer64}
\format{
An object of class \code{integer64} of length 1.
}
\usage{
as.integer64(x, ...)

\method{as.integer64}{`NULL`}(x, ...)

\method{as.integer64}{integer64}(x, ..., keep.names = FALSE)

\method{as.integer64}{double}(x, ..., keep.names = FALSE)

\method{as.integer64}{complex}(x, ...)

\method{as.integer64}{integer}(x, ...)

\method{as.integer64}{raw}(x, ...)

\method{as.integer64}{logical}(x, ...)

\method{as.integer64}{character}(x, ...)

\method{as.integer64}{factor}(x, ...)

\method{as.integer64}{Date}(x, ...)

\method{as.integer64}{POSIXct}(x, ...)

\method{as.integer64}{POSIXlt}(x, ...)

\method{as.integer64}{difftime}(x, units = "auto", ...)

\method{as.integer64}{bitstring}(x, ...)

NA_integer64_
}
\arguments{
\item{x}{an atomic vector}

\item{..., units}{further arguments to the \code{\link[=NextMethod]{NextMethod()}}}

\item{keep.names}{Logical, default \code{FALSE}. If \code{TRUE}, the input's names are retained.}
}
\value{
The other methods return atomic vectors of the expected types
}
\description{
Methods to coerce from other atomic types to integer64.
}
\details{
\code{as.integer64.character} is realized using C function \code{strtoll} which
does not support scientific notation. Instead of '1e6' use '1000000'.
\code{as.integer64.bitstring} evaluates characters '0' and ' ' as zero-bit,
all other one byte characters as one-bit, multi-byte characters are not allowed,
strings shorter than 64 characters are treated as if they were left-padded with '0',
strings longer than 64 bytes are mapped to \code{NA_INTEGER64} and a warning is emitted.
}
\examples{
as.integer64(as.character(lim.integer64()))
as.integer64(
  structure(c("1111111111111111111111111111111111111111111111111111111111111110",
              "1111111111111111111111111111111111111111111111111111111111111111",
              "1000000000000000000000000000000000000000000000000000000000000000",
              "0000000000000000000000000000000000000000000000000000000000000000",
              "0000000000000000000000000000000000000000000000000000000000000001",
              "0000000000000000000000000000000000000000000000000000000000000010"
  ), class = "bitstring")
)
as.integer64(
 structure(c("............................................................... ",
             "................................................................",
             ".                                                               ",
             "",
             ".",
             "10"
  ), class = "bitstring")
)
}
\seealso{
\code{\link[=as.character.integer64]{as.character.integer64()}} \code{\link[=integer64]{integer64()}}
}
\keyword{classes}
\keyword{datasets}
\keyword{manip}

\name{as.integer64.character}
\alias{as.integer64}
\alias{as.integer64.integer64}
\alias{as.integer64.NULL}
\alias{as.integer64.bitstring}
\alias{as.integer64.character}
\alias{as.integer64.double}
\alias{as.integer64.integer}
\alias{as.integer64.logical}
\alias{as.integer64.factor}
\alias{NA_integer64_}
\title{
   Coerce to integer64
}
\description{
  Methods to coerce from other atomic types to integer64. 
}
\usage{
 NA_integer64_
 as.integer64(x, \dots)
 \method{as.integer64}{integer64}(x, \dots)
 \method{as.integer64}{NULL}(x, \dots)
 \method{as.integer64}{character}(x, \dots)
 \method{as.integer64}{bitstring}(x, \dots)
 \method{as.integer64}{double}(x, keep.names = FALSE, \dots)
 \method{as.integer64}{integer}(x, \dots)
 \method{as.integer64}{logical}(x, \dots)
 \method{as.integer64}{factor}(x, \dots)
}
\arguments{
  \item{x}{ an atomic vector }
  \item{keep.names}{ FALSE, set to TRUE to keep a names vector }
  \item{\dots}{ further arguments to the \code{\link{NextMethod}} }
}
\details{
  \code{as.integer64.character} is realized using C function \code{strtoll} which does not support scientific notation. 
  Instead of '1e6' use '1000000'. 
  \code{as.integer64.bitstring} evaluates characters '0' anbd ' ' as zero-bit,
  all other one byte characters as one-bit,
  multi-byte characters are not allowed,
  strings shorter than 64 characters are treated as if they were left-padded with '0',
  strings longer than 64 bytes are mapped to \code{NA_INTEGER64} and a warning is emitted.
}
\value{
  The other methods return atomic vectors of the expected types
}
\author{
Jens Oehlschlägel <Jens.Oehlschlaegel@truecluster.com>
}
\keyword{ classes }
\keyword{ manip }
\seealso{ \code{\link{as.character.integer64}} \code{\link{integer64}}  }
\examples{
as.integer64(as.character(lim.integer64()))
as.integer64(
  structure(c("1111111111111111111111111111111111111111111111111111111111111110", 
              "1111111111111111111111111111111111111111111111111111111111111111", 
              "1000000000000000000000000000000000000000000000000000000000000000",
              "0000000000000000000000000000000000000000000000000000000000000000", 
              "0000000000000000000000000000000000000000000000000000000000000001", 
              "0000000000000000000000000000000000000000000000000000000000000010" 
  ), class = "bitstring")
)
as.integer64(
 structure(c("............................................................... ", 
             "................................................................", 
             ".                                                               ",
             "", 
             ".", 
             "10"
  ), class = "bitstring")
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/highlevel64.R
\name{rank.integer64}
\alias{rank.integer64}
\title{Sample Ranks from integer64}
\usage{
\method{rank}{integer64}(x, method = NULL, ...)
}
\arguments{
\item{x}{a integer64 vector}

\item{method}{NULL for automatic method selection or a suitable low-level
method, see details}

\item{...}{ignored}
}
\value{
A numeric vector of the same length as \code{x}.
}
\description{
Returns the sample ranks of the values in a vector.  Ties (i.e., equal
values) are averaged and missing values propagated.
}
\details{
This function automatically chooses from several low-level functions
considering the size of \code{x} and the availability of a cache.
Suitable methods are
\itemize{
\item \code{\link[=sortorderrnk]{sortorderrnk()}} (fast ordering)
\item \code{\link[=orderrnk]{orderrnk()}} (memory saving ordering).
}
}
\examples{
x <- as.integer64(sample(c(rep(NA, 9), 1:9), 32, TRUE))
rank.integer64(x)

stopifnot(identical(rank.integer64(x),  rank(as.integer(x)
, na.last="keep", ties.method = "average")))

}
\seealso{
\code{\link[=order.integer64]{order.integer64()}}, \code{\link[=rank]{rank()}} and \code{\link[=prank]{prank()}} for percent rank.
}
\keyword{univar}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bit64-package.R, R/integer64.R
\docType{package}
\name{bit64-package}
\alias{bit64-package}
\alias{bit64}
\alias{is.integer.integer64}
\alias{is.vector.integer64}
\alias{integer64}
\alias{is.integer64}
\alias{length<-.integer64}
\alias{print.integer64}
\alias{str.integer64}
\title{A S3 class for vectors of 64bit integers}
\usage{
integer64(length = 0L)

is.integer64(x)

\method{length}{integer64}(x) <- value

\method{print}{integer64}(x, quote = FALSE, ...)

\method{str}{integer64}(
  object,
  vec.len = strO$vec.len,
  give.head = TRUE,
  give.length = give.head,
  ...
)
}
\arguments{
\item{length}{length of vector using \code{\link[=integer]{integer()}}}

\item{x}{an integer64 vector}

\item{value}{an integer64 vector of values to be assigned}

\item{quote}{logical, indicating whether or not strings should be printed with surrounding quotes.}

\item{...}{further arguments to the \code{\link[=NextMethod]{NextMethod()}}}

\item{object}{an integer64 vector}

\item{vec.len, give.head, give.length}{see \code{\link[utils:str]{utils::str()}}}
}
\value{
\code{integer64} returns a vector of 'integer64', i.e.,
a vector of \code{\link[=double]{double()}} decorated with class 'integer64'.
}
\description{
Package 'bit64' provides fast serializable S3 atomic 64bit (signed) integers
that can be used in vectors, matrices, arrays and data.frames. Methods are
available for coercion from and to logicals, integers, doubles, characters
and factors as well as many elementwise and summary functions.
\subsection{Version 0.8}{

With 'integer64' vectors you can store very large integers at the expense
of 64 bits, which is by factor 7 better than 'int64' from package 'int64'.
Due to the smaller memory footprint, the atomic vector architecture and
using only S3 instead of S4 classes, most operations are one to three orders
of magnitude faster: Example speedups are 4x for serialization, 250x for
adding, 900x for coercion and 2000x for object creation. Also 'integer64'
avoids an ongoing (potentially infinite) penalty for garbage collection
observed during existence of 'int64' objects (see code in example section).
}

\subsection{Version 0.9}{

Package 'bit64' - which extends R with fast 64-bit integers - now has fast
(single-threaded) implementations the most important univariate algorithmic
operations (those based on hashing and sorting). We now have methods for
'match', '\%in\%', 'duplicated', 'unique', 'table', 'sort', 'order', 'rank',
'quantile', 'median' and 'summary'. Regarding data management we also have
novel generics 'unipos' (positions of the unique values), 'tiepos' (
positions of ties), 'keypos' (positions of foreign keys in a sorted
dimension table) and derived methods 'as.factor' and 'as.ordered'. This 64-
bit functionality is implemented carefully to be not slower than the
respective 32-bit operations in Base R and also to avoid outlying waiting
times observed with 'order', 'rank' and 'table' (speedup factors 20/16/200
respective). This increases the dataset size with wich we can work truly
interactive. The speed is achieved by simple heuristic optimizers in high-
level functions choosing the best from multiple low-level algorithms and
further taking advantage of a novel caching if activated. In an example R
session using a couple of these operations the 64-bit integers performed 22x
faster than base 32-bit integers, hash-caching improved this to 24x,
sortorder-caching was most efficient with 38x (caching hashing and sorting
is not worth it with 32x at duplicated RAM consumption).
}
}
\note{
\code{integer64} are useful for handling database keys and exact counting in +-2^63.
Do not use them as replacement for 32bit integers, integer64 are not supported for
subscripting by R-core and they have different semantics when combined with double.
Do understand that \code{integer64} can only be useful over \code{double} if we do not coerce
it to \code{double}.

While

integer + double -> double + double -> double

or

1L + 0.5 -> 1.5

for additive operations we coerce to \code{integer64}

integer64 + double ->  integer64 + integer64 -> integer64

hence

as.integer64(1) + 0.5 -> 1LL + 0LL -> 1LL

see section "Arithmetic precision and coercion" above
}
\section{Design considerations}{
64 bit integers are related to big data: we need them to overcome address space
limitations. Therefore performance of the 64 bit integer type is critical. In the
S language -- designed in 1975 -- atomic objects were defined to be vectors for a
couple of good reasons: simplicity, option for implicit parallelization, good
cache locality. In recent years many analytical databases have learnt that lesson:
column based data bases provide superior performance for many applications, the
result are products such as MonetDB, Sybase IQ, Vertica, Exasol, Ingres Vectorwise.
If we introduce 64 bit integers not natively in Base R but as an external package,
we should at least strive to make them as 'basic' as possible. Therefore the design
choice of bit64 not only differs from package int64, it is obvious: Like the other
atomic types in Base R, we model data type 'integer64' as a contiguous \code{\link{atomic}}
vector in memory, and we use the more basic \link{S3} class system, not \link{S4}. Like
package int64 we want our 'integer64' to be \code{\link{serialize}}able, therefore we also
use an existing data type as the basis. Again the choice is obvious: R has only one
64 bit data type: doubles. By using \code{\link{double}}s, \code{integer64} \code{\link{inherits}} some
functionality such as \code{\link[=is.atomic]{is.atomic()}}, \code{\link[=length]{length()}}, \code{\link{length<-}}, \code{\link[=names]{names()}},
\code{\link{names<-}}, \code{\link[=dim]{dim()}}, \code{\link{dim<-}}, \code{\link[=dimnames]{dimnames()}}, \code{\link{dimnames<-}}.

Our R level functions strictly follow the functional programming paradigm:
no modification of arguments or other side-effects. Before version 0.93  we
internally deviated from the strict paradigm in order to boost performance. Our C
functions do not create new return values, instead we pass-in the memory to be
returned as an argument. This gives us the freedom to apply the C-function to new
or old vectors, which helps to avoid unnecessary memory allocation, unnecessary
copying and unnecessary garbage collection. Prior to 0.93 \emph{within} our R functions
we also deviated from conventional R programming by not using \code{\link{attr<-}} and
\code{\link{attributes<-}} because they always did new memory allocation and copying in older
R versions. If we wanted to set attributes of return values that we have freshly
created, we instead used functions \code{\link[bit:getsetattr]{bit::setattr()}} and \code{\link[bit:getsetattr]{bit::setattributes()}}.
From version 0.93 \code{bit::setattr()} is only used for manipulating \code{\link{cache}} objects,
in \code{\link[=ramsort.integer64]{ramsort.integer64()}}, \code{\link[=sort.integer64]{sort.integer64()}}, and \code{\link[=as.data.frame.integer64]{as.data.frame.integer64()}}.
}

\section{Arithmetic precision and coercion}{
The fact that we introduce 64 bit long long integers -- without introducing 128-bit
long doubles -- creates some subtle challenges: Unlike 32 bit \code{\link{integer}}s, the
\code{integer64} are no longer a proper subset of \code{\link{double}}. If a binary arithmetic
operation does involve a \code{double} and a \code{integer}, it is a no-brainer to return
\code{double} without loss of information. If an \code{integer64} meets a \code{double}, it is not
trivial what type to return. Switching to \code{integer64} limits our ability to
represent very large numbers, switching to \code{double} limits our ability to
distinguish \code{x} from \code{x+1}. Since the latter is the purpose of introducing 64 bit
integers, we usually return \code{integer64} from functions involving \code{integer64}, for
example in \code{\link[=c.integer64]{c()}}, \code{\link[=cbind.integer64]{cbind()}}, and
\code{\link[=rbind.integer64]{rbind()}}

Our operators \code{\link[=ops64]{+}}, \code{\link[=ops64]{-}}, \code{\link[=ops64]{\%/\%}}, and \code{\link[=ops64]{\%\%}} coerce
their arguments to \code{integer64} and return \code{integer64} if they are \code{integer},
\code{double} or \code{logical}. Otherwise the \code{integer64} argument is coerced to double and
the R base method for the other class is called.

Our operators \code{\link[=ops64]{*}} and \code{\link[=ops64]{^}} coerce their first argument to
\code{integer64} and possibly the second to \code{integer64} if it is not \code{double} and
return \code{integer64} if they are \code{integer}, \code{double} or \code{logical}. Otherwise the
\code{integer64} argument is coerced to double and the R base method for the other
class is called.

The division \code{\link[=ops64]{/}} operator also coerces its first argument to \code{integer64}
and possibly the second to \code{integer64} if it is not \code{double} and returns
\code{integer64} if they are \code{integer}, \code{double} or \code{logical}. Otherwise the \code{integer64}
argument is coerced to double and the R base method for the other class is called.

\code{\link[=sqrt.integer64]{sqrt()}}, \code{\link[=log.integer64]{log()}},
\code{\link[=log2.integer64]{log2()}}, and \code{\link[=log10.integer64]{log10()}} coerce their first
argument to \code{integer64} and coerce internally their second argument to
'long double', they return as \code{double}\tabular{ccccccccc}{
   \strong{argument1} \tab \strong{op} \tab \strong{argument2} \tab \strong{->} \tab \strong{coerced1} \tab \strong{op} \tab \strong{coerced2} \tab \strong{->} \tab \strong{result} \cr
   integer64 \tab + \tab double \tab -> \tab integer64 \tab + \tab integer64 \tab -> \tab integer64 \cr
   double \tab + \tab integer64 \tab -> \tab integer64 \tab + \tab integer64 \tab -> \tab integer64 \cr
   integer64 \tab + \tab complex \tab -> \tab double \tab + \tab complex \tab -> \tab complex \cr
   complex \tab + \tab integer64 \tab -> \tab complex \tab + \tab double \tab -> \tab complex \cr
   integer64 \tab - \tab double \tab -> \tab integer64 \tab - \tab integer64 \tab -> \tab integer64 \cr
   double \tab - \tab integer64 \tab -> \tab integer64 \tab - \tab integer64 \tab -> \tab integer64 \cr
   integer64 \tab - \tab complex \tab -> \tab double \tab - \tab complex \tab -> \tab complex \cr
   complex \tab - \tab integer64 \tab -> \tab complex \tab - \tab double \tab -> \tab complex \cr
   integer64 \tab \%/\% \tab double \tab -> \tab integer64 \tab \%/\% \tab integer64 \tab -> \tab integer64 \cr
   double \tab \%/\% \tab integer64 \tab -> \tab integer64 \tab \%/\% \tab integer64 \tab -> \tab integer64 \cr
   integer64 \tab \%/\% \tab complex \tab -> \tab double \tab \%/\% \tab complex \tab -> \tab complex \cr
   complex \tab \%/\% \tab integer64 \tab -> \tab complex \tab \%/\% \tab double \tab -> \tab complex \cr
   integer64 \tab \%\% \tab double \tab -> \tab integer64 \tab \%\% \tab integer64 \tab -> \tab integer64 \cr
   double \tab \%\% \tab integer64 \tab -> \tab integer64 \tab \%\% \tab integer64 \tab -> \tab integer64 \cr
   integer64 \tab \%\% \tab complex \tab -> \tab double \tab \%\% \tab complex \tab -> \tab complex \cr
   complex \tab \%\% \tab integer64 \tab -> \tab complex \tab \%\% \tab double \tab -> \tab complex \cr
   integer64 \tab * \tab double \tab -> \tab integer64 \tab * \tab long double \tab -> \tab integer64 \cr
   double \tab * \tab integer64 \tab -> \tab integer64 \tab * \tab integer64 \tab -> \tab integer64 \cr
   integer64 \tab * \tab complex \tab -> \tab double \tab * \tab complex \tab -> \tab complex \cr
   complex \tab * \tab integer64 \tab -> \tab complex \tab * \tab double \tab -> \tab complex \cr
   integer64 \tab / \tab double \tab -> \tab integer64 \tab / \tab long double \tab -> \tab double \cr
   double \tab / \tab integer64 \tab -> \tab integer64 \tab / \tab long double \tab -> \tab double \cr
   integer64 \tab / \tab complex \tab -> \tab double \tab / \tab complex \tab -> \tab complex \cr
   complex \tab / \tab integer64 \tab -> \tab complex \tab / \tab double \tab -> \tab complex \cr
   integer64 \tab ^ \tab double \tab -> \tab integer64 \tab ^ \tab long double \tab -> \tab double \cr
   double \tab ^ \tab integer64 \tab -> \tab integer64 \tab ^ \tab long double \tab -> \tab double \cr
   integer64 \tab ^ \tab complex \tab -> \tab double \tab ^ \tab complex \tab -> \tab complex \cr
   complex \tab ^ \tab integer64 \tab -> \tab complex \tab ^ \tab double \tab -> \tab complex \cr
   integer64 \tab \%*\% \tab double \tab -> \tab integer64 \tab \%*\% \tab integer64 \tab -> \tab integer64 \cr
   double \tab \%*\% \tab integer64 \tab -> \tab integer64 \tab \%*\% \tab integer64 \tab -> \tab integer64 \cr
   integer64 \tab \%*\% \tab complex \tab -> \tab double \tab \%*\% \tab complex \tab -> \tab complex \cr
   complex \tab \%*\% \tab integer64 \tab -> \tab complex \tab \%*\% \tab double \tab -> \tab complex \cr
}
}

\section{Creating and testing S3 class 'integer64'}{
Our creator function \code{integer64} takes an argument \code{length}, creates an atomic
double vector of this length, attaches an S3 class attribute 'integer64' to it,
and that's it. We simply rely on S3 method dispatch and interpret those 64-bit
elements as 'long long int'.

\code{\link[=is.double]{is.double()}} currently returns TRUE for \code{integer64} and might return \code{FALSE} in
a later release. Consider \code{is.double()} to have undefined behavior and do query
\code{\link[=is.integer64]{is.integer64()}} \emph{before} querying \code{is.double()}.

The methods \code{\link[=is.integer64]{is.integer64()}} and \code{\link[=is.vector]{is.vector()}} both return \code{TRUE} for \code{integer64}.
Note that we did not patch \code{\link[=storage.mode]{storage.mode()}} and \code{\link[=typeof]{typeof()}}, which both continue
returning 'double'. Like for 32 bit \code{\link{integer}}, \code{\link[=mode]{mode()}} returns 'numeric' and
\code{\link[=as.double]{as.double()}} tries coercing to \code{\link{double}}. It is possible that 'integer64' becomes
a \code{vmode} in package ff.

Further methods for creating \code{integer64} are \code{\link[=range.integer64]{range()}} which
returns the range of the data type if calles without arguments,
\code{\link[=rep.integer64]{rep()}}, \code{\link[=seq.integer64]{seq()}}.

For all available methods on \code{integer64} vectors see the index below and the examples.
}

\section{Index of implemented methods}{
\tabular{rrl}{
   \strong{creating, testing, printing} \tab \strong{see also} \tab \strong{description} \cr
   \code{NA_integer64_} \tab \code{\link{NA_integer_}} \tab NA constant \cr
   \code{integer64} \tab \code{\link{integer}} \tab create zero atomic vector \cr
   \code{\link[=runif64]{runif64()}} \tab \code{\link[=runif]{runif()}} \tab create random vector \cr
   \code{\link[=rep.integer64]{rep.integer64()}} \tab \code{\link[=rep]{rep()}} \tab  \cr
   \code{\link[=seq.integer64]{seq.integer64()}} \tab \code{\link[=seq]{seq()}} \tab  \cr
   \code{\link[=is.integer64]{is.integer64()}} \tab \code{\link[=is]{is()}} \tab  \cr
    \tab \code{\link[=is.integer]{is.integer()}} \tab inherited from Base R \cr
   \code{\link[=is.vector.integer64]{is.vector.integer64()}} \tab \code{\link[=is.vector]{is.vector()}} \tab  \cr
   \code{\link[=identical.integer64]{identical.integer64()}} \tab \code{\link[=identical]{identical()}} \tab  \cr
   \code{\link{length<-.integer64}} \tab \code{\link{length<-}} \tab  \cr
    \tab \code{\link[=length]{length()}} \tab inherited from Base R \cr
   \code{\link{names<-}} \tab inherited from Base R \tab  \cr
    \tab \code{\link[=names]{names()}} \tab inherited from Base R \cr
    \tab \code{\link{dim<-}} \tab inherited from Base R \cr
    \tab \code{\link[=dim]{dim()}} \tab inherited from Base R \cr
    \tab \code{\link{dimnames<-}} \tab inherited from Base R \cr
    \tab \code{\link[=dimnames]{dimnames()}} \tab inherited from Base R \cr
    \tab \code{\link[=str]{str()}} \tab inherited from Base R, does not print values correctly \cr
   \code{\link[=print.integer64]{print.integer64()}} \tab \code{\link[=print]{print()}} \tab  \cr
   \code{\link[=str.integer64]{str.integer64()}} \tab \code{\link[=str]{str()}} \tab  \cr
}
\tabular{rrl}{
   \strong{coercing to integer64} \tab \strong{see also} \tab \strong{description} \cr
   \code{\link[=as.integer64]{as.integer64()}} \tab  \tab generic \cr
   \code{\link[=as.integer64.bitstring]{as.integer64.bitstring()}} \tab \code{\link[=as.bitstring]{as.bitstring()}} \tab  \cr
   \code{\link[=as.integer64.character]{as.integer64.character()}} \tab \code{\link[=character]{character()}} \tab  \cr
   \code{\link[=as.integer64.double]{as.integer64.double()}} \tab \code{\link[=double]{double()}} \tab  \cr
   \code{\link[=as.integer64.integer]{as.integer64.integer()}} \tab \code{\link[=integer]{integer()}} \tab  \cr
   \code{\link[=as.integer64.integer64]{as.integer64.integer64()}} \tab \code{integer64} \tab  \cr
   \code{\link[=as.integer64.logical]{as.integer64.logical()}} \tab \code{\link[=logical]{logical()}} \tab  \cr
   \code{\link[=as.integer64.NULL]{as.integer64.NULL()}} \tab \code{\link[=NULL]{NULL()}} \tab  \cr
}
\tabular{rrl}{
   \strong{coercing from integer64} \tab \strong{see also} \tab \strong{description} \cr
   \code{\link[=as.list.integer64]{as.list.integer64()}} \tab \code{\link[=as.list]{as.list()}} \tab generic \cr
   \code{\link[=as.bitstring]{as.bitstring()}} \tab \code{\link[=as.bitstring]{as.bitstring()}} \tab generic \cr
   \code{\link[=as.bitstring.integer64]{as.bitstring.integer64()}} \tab  \tab  \cr
   \code{\link[=as.character.integer64]{as.character.integer64()}} \tab \code{\link[=as.character]{as.character()}} \tab  \cr
   \code{\link[=as.double.integer64]{as.double.integer64()}} \tab \code{\link[=as.double]{as.double()}} \tab  \cr
   \code{\link[=as.integer.integer64]{as.integer.integer64()}} \tab \code{\link[=as.integer]{as.integer()}} \tab  \cr
   \code{\link[=as.logical.integer64]{as.logical.integer64()}} \tab \code{\link[=as.logical]{as.logical()}} \tab  \cr
}
\tabular{rrl}{
   \strong{data structures} \tab \strong{see also} \tab \strong{description} \cr
   \code{\link[=c.integer64]{c.integer64()}} \tab \code{\link[=c]{c()}} \tab vector concatenate \cr
   \code{\link[=cbind.integer64]{cbind.integer64()}} \tab \code{\link[=cbind]{cbind()}} \tab column bind \cr
   \code{\link[=rbind.integer64]{rbind.integer64()}} \tab \code{\link[=rbind]{rbind()}} \tab row bind \cr
   \code{\link[=as.data.frame.integer64]{as.data.frame.integer64()}} \tab \code{\link[=as.data.frame]{as.data.frame()}} \tab coerce atomic object to data.frame \cr
    \tab \code{\link[=data.frame]{data.frame()}} \tab inherited from Base R since we have coercion \cr
}
\tabular{rrl}{
   \strong{subscripting} \tab \strong{see also} \tab \strong{description} \cr
   \code{\link[=extract.replace.integer64]{[.integer64}} \tab \code{\link[base:Extract]{[}} \tab vector and array extract \cr
   \code{\link[=extract.replace.integer64]{[<-.integer64}} \tab \code{\link[base:Extract]{[<-}} \tab vector and array assign \cr
   \code{\link[=extract.replace.integer64]{[[.integer64}} \tab \code{\link[base:Extract]{[[}} \tab scalar extract \cr
   \code{\link[=extract.replace.integer64]{[[<-.integer64}} \tab \code{\link[base:Extract]{[[<-}} \tab scalar assign \cr
}
\tabular{rrl}{
   \strong{binary operators} \tab \strong{see also} \tab \strong{description} \cr
   \code{\link{+.integer64}} \tab \code{\link{+}} \tab returns integer64 \cr
   \code{\link{-.integer64}} \tab \code{\link{-}} \tab returns integer64 \cr
   \code{\link{*.integer64}} \tab \code{\link{*}} \tab returns integer64 \cr
   \code{\link{^.integer64}} \tab \code{\link{^}} \tab returns double \cr
   \code{\link{/.integer64}} \tab \code{\link{/}} \tab returns double \cr
   \code{\link[=ops64]{\%/\%}} \tab \code{\link[=Arithmetic]{\%/\%}} \tab returns integer64 \cr
   \code{\link[=ops64]{\%\%}} \tab \code{\link[=Arithmetic]{\%\%}} \tab returns integer64 \cr
}
\tabular{rrl}{
   \strong{comparison operators} \tab \strong{see also} \tab \strong{description} \cr
   \code{\link{==.integer64}} \tab \code{\link{==}} \tab  \cr
   \code{\link{!=.integer64}} \tab \code{\link{!=}} \tab  \cr
   \code{\link{<.integer64}} \tab \code{\link{<}} \tab  \cr
   \code{\link{<=.integer64}} \tab \code{\link{<=}} \tab  \cr
   \code{\link{>.integer64}} \tab \code{\link{>}} \tab  \cr
   \code{\link{>=.integer64}} \tab \code{\link{>=}} \tab  \cr
}


\tabular{rrl}{
\strong{logical operators} \tab \strong{see also} \tab \strong{description} \cr
\code{\link{!.integer64}} \tab \code{\link{!}} \tab  \cr
\code{\link{&.integer64}} \tab \code{\link{&}} \tab  \cr
\code{\link{|.integer64}} \tab \code{\link[base:Logic]{|}} \tab  \cr
}\tabular{rrl}{
   \strong{math functions} \tab \strong{see also} \tab \strong{description} \cr
   \code{\link[=is.na.integer64]{is.na.integer64()}} \tab \code{\link[=is.na]{is.na()}} \tab returns logical \cr
   \code{\link[=format.integer64]{format.integer64()}} \tab \code{\link[=format]{format()}} \tab returns character \cr
   \code{\link[=abs.integer64]{abs.integer64()}} \tab \code{\link[=abs]{abs()}} \tab returns integer64 \cr
   \code{\link[=sign.integer64]{sign.integer64()}} \tab \code{\link[=sign]{sign()}} \tab returns integer64 \cr
   \code{\link[=log.integer64]{log.integer64()}} \tab \code{\link[=log]{log()}} \tab returns double \cr
   \code{\link[=log10.integer64]{log10.integer64()}} \tab \code{\link[=log10]{log10()}} \tab returns double \cr
   \code{\link[=log2.integer64]{log2.integer64()}} \tab \code{\link[=log2]{log2()}} \tab returns double \cr
   \code{\link[=sqrt.integer64]{sqrt.integer64()}} \tab \code{\link[=sqrt]{sqrt()}} \tab returns double \cr
   \code{\link[=ceiling.integer64]{ceiling.integer64()}} \tab \code{\link[=ceiling]{ceiling()}} \tab dummy returning its argument \cr
   \code{\link[=floor.integer64]{floor.integer64()}} \tab \code{\link[=floor]{floor()}} \tab dummy returning its argument \cr
   \code{\link[=trunc.integer64]{trunc.integer64()}} \tab \code{\link[=trunc]{trunc()}} \tab dummy returning its argument \cr
   \code{\link[=round.integer64]{round.integer64()}} \tab \code{\link[=round]{round()}} \tab dummy returning its argument \cr
   \code{\link[=signif.integer64]{signif.integer64()}} \tab \code{\link[=signif]{signif()}} \tab dummy returning its argument \cr
}
\tabular{rrl}{
   \strong{cumulative functions} \tab \strong{see also} \tab \strong{description} \cr
   \code{\link[=cummin.integer64]{cummin.integer64()}} \tab \code{\link[=cummin]{cummin()}} \tab  \cr
   \code{\link[=cummax.integer64]{cummax.integer64()}} \tab \code{\link[=cummax]{cummax()}} \tab  \cr
   \code{\link[=cumsum.integer64]{cumsum.integer64()}} \tab \code{\link[=cumsum]{cumsum()}} \tab  \cr
   \code{\link[=cumprod.integer64]{cumprod.integer64()}} \tab \code{\link[=cumprod]{cumprod()}} \tab  \cr
   \code{\link[=diff.integer64]{diff.integer64()}} \tab \code{\link[=diff]{diff()}} \tab  \cr
}
\tabular{rrl}{
   \strong{summary functions} \tab \strong{see also} \tab \strong{description} \cr
   \code{\link[=range.integer64]{range.integer64()}} \tab \code{\link[=range]{range()}} \tab  \cr
   \code{\link[=min.integer64]{min.integer64()}} \tab \code{\link[=min]{min()}} \tab  \cr
   \code{\link[=max.integer64]{max.integer64()}} \tab \code{\link[=max]{max()}} \tab  \cr
   \code{\link[=sum.integer64]{sum.integer64()}} \tab \code{\link[=sum]{sum()}} \tab  \cr
   \code{\link[=mean.integer64]{mean.integer64()}} \tab \code{\link[=mean]{mean()}} \tab  \cr
   \code{\link[=prod.integer64]{prod.integer64()}} \tab \code{\link[=prod]{prod()}} \tab  \cr
   \code{\link[=all.integer64]{all.integer64()}} \tab \code{\link[=all]{all()}} \tab  \cr
   \code{\link[=any.integer64]{any.integer64()}} \tab \code{\link[=any]{any()}} \tab  \cr
}
\tabular{rrl}{
   \strong{algorithmically complex functions} \tab \strong{see also} \tab \strong{description (caching)} \cr
   \code{\link[=match.integer64]{match.integer64()}} \tab \code{\link[=match]{match()}} \tab position of x in table (h//o/so) \cr
   \code{\link[=match.integer64]{\%in\%.integer64}} \tab \code{\link[=match]{\%in\%}} \tab is x in table? (h//o/so) \cr
   \code{\link[=duplicated.integer64]{duplicated.integer64()}} \tab \code{\link[=duplicated]{duplicated()}} \tab is current element duplicate of previous one? (h//o/so) \cr
   \code{\link[=unique.integer64]{unique.integer64()}} \tab \code{\link[=unique]{unique()}} \tab (shorter) vector of unique values only (h/s/o/so) \cr
   \code{\link[=unipos.integer64]{unipos.integer64()}} \tab \code{\link[=unipos]{unipos()}} \tab positions corresponding to unique values (h/s/o/so) \cr
   \code{\link[=tiepos.integer64]{tiepos.integer64()}} \tab \code{\link[=tiepos]{tiepos()}} \tab positions of values that are tied (//o/so) \cr
   \code{\link[=keypos.integer64]{keypos.integer64()}} \tab \code{\link[=keypos]{keypos()}} \tab position of current value in sorted list of unique values (//o/so) \cr
   \code{\link[=table.integer64]{table.integer64()}} \tab \code{\link[=table]{table()}} \tab unique values and their frequencies (h/s/o/so) \cr
   \code{\link[=sort.integer64]{sort.integer64()}} \tab \code{\link[=sort]{sort()}} \tab sorted vector (/s/o/so) \cr
   \code{\link[=order.integer64]{order.integer64()}} \tab \code{\link[=order]{order()}} \tab positions of elements that would create sorted vector (//o/so) \cr
   \code{\link[=rank.integer64]{rank.integer64()}} \tab \code{\link[=rank]{rank()}} \tab (average) ranks of non-NAs, NAs kept in place (/s/o/so) \cr
   \code{\link[=quantile.integer64]{quantile.integer64()}} \tab \code{\link[=quantile]{quantile()}} \tab (existing) values at specified percentiles (/s/o/so) \cr
   \code{\link[=median.integer64]{median.integer64()}} \tab \code{\link[=median]{median()}} \tab (existing) value at percentile 0.5 (/s/o/so) \cr
   \code{\link[=summary.integer64]{summary.integer64()}} \tab \code{\link[=summary]{summary()}} \tab (/s/o/so) \cr
   \code{\link[=all.equal.integer64]{all.equal.integer64()}} \tab \code{\link[=all.equal]{all.equal()}} \tab test if two objects are (nearly) equal (/s/o/so) \cr
}
\tabular{rrl}{
   \strong{helper functions} \tab \strong{see also} \tab \strong{description} \cr
   \code{\link[=minusclass]{minusclass()}} \tab \code{\link[=minusclass]{minusclass()}} \tab removing class attritbute \cr
   \code{\link[=plusclass]{plusclass()}} \tab \code{\link[=plusclass]{plusclass()}} \tab inserting class attribute \cr
   \code{\link[=binattr]{binattr()}} \tab \code{\link[=binattr]{binattr()}} \tab define binary op behaviour \cr
}
\tabular{rrl}{
   \strong{tested I/O functions} \tab \strong{see also} \tab \strong{description} \cr
    \tab \code{\link[=read.table]{read.table()}} \tab inherited from Base R \cr
    \tab \code{\link[=write.table]{write.table()}} \tab inherited from Base R \cr
    \tab \code{\link[=serialize]{serialize()}} \tab inherited from Base R \cr
    \tab \code{\link[=unserialize]{unserialize()}} \tab inherited from Base R \cr
    \tab \code{\link[=save]{save()}} \tab inherited from Base R \cr
    \tab \code{\link[=load]{load()}} \tab inherited from Base R \cr
    \tab \code{\link[=dput]{dput()}} \tab inherited from Base R \cr
    \tab \code{\link[=dget]{dget()}} \tab inherited from Base R \cr
}
}

\section{Limitations inherited from implementing 64 bit integers via an external package}{
\itemize{
\item \strong{vector size} of atomic vectors is still limited to
\code{\link[=.Machine]{.Machine$integer.max}}. However, external memory extending packages
such as ff or bigmemory can extend their address space now with \code{integer64}.
Having 64 bit integers also help with those not so obvious address issues that
arise once we exchange data with SQL databases and datawarehouses, which use big
integers as surrogate keys, e.g. on indexed primary key columns. This puts R into
a relatively strong position compared to certain commercial statistical softwares,
which sell database connectivity but neither have the range of 64 bit integers,
nor have integers at all, nor have a single numeric data type in their
macro-glue-language.
\item \strong{literals} such as \verb{123LL} would require changes to Base R, up to then we need
to write (and call) \code{as.integer64(123L)} or \code{as.integer64(123)} or
\code{as.integer64('123')}. Only the latter allows to specify numbers beyond Base R's
numeric data types and therefore is the recommended way to use -- using only one
way may facilitate migrating code to literals at a later stage.
}
}

\section{Limitations inherited from Base R, Core team, can you change this?}{
\itemize{
\item \strong{\code{\link[=identical]{identical()}}} with default parameters does not distinguish all bit-patterns of
doubles. For testing purposes we provide a wrapper \code{\link[=identical.integer64]{identical.integer64()}} that
will distinguish all bit-patterns. It would be desireable to have a single call
of \code{identical()} handle both, \code{\link{double}} and \code{integer64}.
\item the \strong{colon} operator \link{:} officially does not dispatch S3 methods, however, we
have made it generic:

\if{html}{\out{<div class="sourceCode r">}}\preformatted{from <- lim.integer64()[1]
to <- from+99
from:to
}\if{html}{\out{</div>}}

As a limitation remains: it will only dispatch at its first argument \code{from} but
not at its second \code{to}.
\item \strong{\code{\link[=is.double]{is.double()}}} does not dispatch S3 methods, However, we have made it generic
and it will return \code{FALSE} on \code{integer64}.
\item \strong{\code{\link[=c]{c()}}} only dispatches \code{\link[=c.integer64]{c.integer64()}} if the first argument is \code{integer64}
and it does not recursively dispatch the proper method when called with argument
\code{recursive=TRUE}. Therefore \code{c(list(integer64, integer64))} does not work and
for now you can only call \code{c.integer64(list(x, x))}.
\item \strong{\code{\link[=unlist]{unlist()}}} is not generic and if it were, we would face similar problems as
with \code{\link[=c]{c()}}
\item \strong{\code{\link[=vector]{vector()}}} with argument \code{mode='integer64'} cannot work without adjustment
of Base R
\item \strong{\code{\link[=as.vector]{as.vector()}}} with argument \code{mode='integer64'} cannot work without adjustment
of Base R
\item \strong{\code{\link[=is.vector]{is.vector()}}} does not dispatch its method \code{\link[=is.vector.integer64]{is.vector.integer64()}}
\item \strong{\code{\link[=mode<-]{mode<-()}}} drops the class 'integer64' which is returned from
\code{as.integer64()}. Also it does not remove an existing class 'integer64' when
assigning mode 'integer'.
\item \strong{\code{\link[=storage.mode<-]{storage.mode<-()}}} does not support external data types such as \code{integer64}
\item \strong{\code{\link[=matrix]{matrix()}}} does drop the 'integer64' class attribute.
\item \strong{\code{\link[=array]{array()}}}  does drop the 'integer64' class attribute.
\itemize{
\item In current R versions (1.15.1) this can be circumvented by activating the
function \code{as.vector.integer64()}. However, the CRAN maintainer has requested
to remove \code{as.vector.integer64()}, even at the price of breaking previously
working functionality of the package.
}
\item \strong{\code{\link[=str]{str()}}} does not print the values of \code{integer64} correctly
}
}

\section{Further limitations}{
\itemize{
\item \strong{subscripting} non-existing elements and subscripting with \code{NA}s is currently
not supported. Such subscripting currently returns \code{9218868437227407266} instead
of \code{NA} (the \code{NA} value of the underlying double code). Following the full R
behaviour here would either destroy performance or require extensive C-coding.
}
}

\examples{
message("Using integer64 in vector")
x <- integer64(8)    # create 64 bit vector
x
is.atomic(x)         # TRUE
is.integer64(x)      # TRUE
is.numeric(x)        # TRUE
is.integer(x)        # FALSE - debatable
is.double(x)         # FALSE - might change
x[] <- 1:2           # assigned value is recycled as usual
x[1:6]               # subscripting as usual
length(x) <- 13      # changing length as usual
x
rep(x, 2)            # replicate as usual
seq(as.integer64(1), 10)     # seq.integer64 is dispatched on first given argument
seq(to=as.integer64(10), 1)  # seq.integer64 is dispatched on first given argument
seq.integer64(along.with=x)  # or call seq.integer64 directly
# c.integer64 is dispatched only if *first* argument is integer64 ...
x <- c(x,runif(length(x), max=100))
# ... and coerces everything to integer64 - including double
x
names(x) <- letters  # use names as usual
x

message("Using integer64 in array - note that 'matrix' currently does not work")
message("as.vector.integer64 removed as requested by the CRAN maintainer")
message("as consequence 'array' also does not work anymore")
message("we still can create a matrix or array by assigning 'dim'")
y <- rep(as.integer64(NA), 12)
dim(y) <- c(3,4)
dimnames(y) <- list(letters[1:3], LETTERS[1:4])
y["a",] <- 1:2       # assigning as usual
y
y[1:2,-4]            # subscripting as usual
# cbind.integer64 dispatched on any argument and coerces everything to integer64
cbind(E=1:3, F=runif(3, 0, 100), G=c("-1","0","1"), y)

message("Using integer64 in data.frame")
str(as.data.frame(x))
str(as.data.frame(y))
str(data.frame(y))
str(data.frame(I(y)))
d <- data.frame(x=x, y=runif(length(x), 0, 100))
d
d$x

message("Using integer64 with csv files")
fi64 <- tempfile()
write.csv(d, file=fi64, row.names=FALSE)
e <- read.csv(fi64, colClasses=c("integer64", NA))
unlink(fi64)
str(e)
identical.integer64(d$x,e$x)

message("Serializing and unserializing integer64")
dput(d, fi64)
e <- dget(fi64)
identical.integer64(d$x,e$x)
e <- d[,]
save(e, file=fi64)
rm(e)
load(file=fi64)
identical.integer64(d,e)

  \dontrun{
message("== Differences between integer64 and int64 ==")
require(bit64)
require(int64)

message("-- integer64 is atomic --")
is.atomic(integer64())
#is.atomic(int64())
str(integer64(3))
#str(int64(3))

message("-- The following performance numbers are measured under RWin64  --")
message("-- under RWin32 the advantage of integer64 over int64 is smaller --")

message("-- integer64 needs 7x/5x less RAM than int64 under 64/32 bit OS
(and twice the RAM of integer as it should be) --")
#as.vector(object.size(int64(1e6))/object.size(integer64(1e6)))
as.vector(object.size(integer64(1e6))/object.size(integer(1e6)))

message("-- integer64 creates 2000x/1300x faster than int64 under 64/32 bit OS
(and 3x the time of integer) --")
t32 <- system.time(integer(1e8))
t64 <- system.time(integer64(1e8))
#T64 <- system.time(int64(1e7))*10  # using 1e8 as above stalls our R on an i7 8 GB RAM Thinkpad
#T64/t64
t64/t32

i32 <- sample(1e6)
d64 <- as.double(i32)

message("-- the following timings are rather conservative since timings
 of integer64 include garbage collection -- due to looped calls")
message("-- integer64 coerces 900x/100x faster than int64
 under 64/32 bit OS (and 2x the time of coercing to integer) --")
t32 <- system.time(for(i in 1:1000)as.integer(d64))
t64 <- system.time(for(i in 1:1000)as.integer64(d64))
#T64 <- system.time(as.int64(d64))*1000
#T64/t64
t64/t32
td64 <- system.time(for(i in 1:1000)as.double(i32))
t64 <- system.time(for(i in 1:1000)as.integer64(i32))
#T64 <- system.time(for(i in 1:10)as.int64(i32))*100
#T64/t64
t64/td64

message("-- integer64 serializes 4x/0.8x faster than int64
 under 64/32 bit OS (and less than 2x/6x the time of integer or double) --")
t32 <- system.time(for(i in 1:10)serialize(i32, NULL))
td64 <- system.time(for(i in 1:10)serialize(d64, NULL))
i64 <- as.integer64(i32);
t64 <- system.time(for(i in 1:10)serialize(i64, NULL))
rm(i64); gc()
#I64 <- as.int64(i32);
#T64 <- system.time(for(i in 1:10)serialize(I64, NULL))
#rm(I64); gc()
#T64/t64
t64/t32
t64/td64


message("-- integer64 adds 250x/60x faster than int64
 under 64/32 bit OS (and less than 6x the time of integer or double) --")
td64 <- system.time(for(i in 1:100)d64+d64)
t32 <- system.time(for(i in 1:100)i32+i32)
i64 <- as.integer64(i32);
t64 <- system.time(for(i in 1:100)i64+i64)
rm(i64); gc()
#I64 <- as.int64(i32);
#T64 <- system.time(for(i in 1:10)I64+I64)*10
#rm(I64); gc()
#T64/t64
t64/t32
t64/td64

message("-- integer64 sums 3x/0.2x faster than int64
(and at about 5x/60X the time of integer and double) --")
td64 <- system.time(for(i in 1:100)sum(d64))
t32 <- system.time(for(i in 1:100)sum(i32))
i64 <- as.integer64(i32);
t64 <- system.time(for(i in 1:100)sum(i64))
rm(i64); gc()
#I64 <- as.int64(i32);
#T64 <- system.time(for(i in 1:100)sum(I64))
#rm(I64); gc()
#T64/t64
t64/t32
t64/td64

message("-- integer64 diffs 5x/0.85x faster than integer and double
(int64 version 1.0 does not support diff) --")
td64 <- system.time(for(i in 1:10)diff(d64, lag=2L, differences=2L))
t32 <- system.time(for(i in 1:10)diff(i32, lag=2L, differences=2L))
i64 <- as.integer64(i32);
t64 <- system.time(for(i in 1:10)diff(i64, lag=2L, differences=2L))
rm(i64); gc()
t64/t32
t64/td64


message("-- integer64 subscripts 1000x/340x faster than int64
(and at the same speed / 10x slower as integer) --")
ts32 <- system.time(for(i in 1:1000)sample(1e6, 1e3))
t32<- system.time(for(i in 1:1000)i32[sample(1e6, 1e3)])
i64 <- as.integer64(i32);
t64 <- system.time(for(i in 1:1000)i64[sample(1e6, 1e3)])
rm(i64); gc()
#I64 <- as.int64(i32);
#T64 <- system.time(for(i in 1:100)I64[sample(1e6, 1e3)])*10
#rm(I64); gc()
#(T64-ts32)/(t64-ts32)
(t64-ts32)/(t32-ts32)

message("-- integer64 assigns 200x/90x faster than int64
(and 50x/160x slower than integer) --")
ts32 <- system.time(for(i in 1:100)sample(1e6, 1e3))
t32 <- system.time(for(i in 1:100)i32[sample(1e6, 1e3)] <- 1:1e3)
i64 <- as.integer64(i32);
i64 <- system.time(for(i in 1:100)i64[sample(1e6, 1e3)] <- 1:1e3)
rm(i64); gc()
#I64 <- as.int64(i32);
#I64 <- system.time(for(i in 1:10)I64[sample(1e6, 1e3)] <- 1:1e3)*10
#rm(I64); gc()
#(T64-ts32)/(t64-ts32)
(t64-ts32)/(t32-ts32)


tdfi32 <- system.time(dfi32 <- data.frame(a=i32, b=i32, c=i32))
tdfsi32 <- system.time(dfi32[1e6:1,])
fi32 <- tempfile()
tdfwi32 <- system.time(write.csv(dfi32, file=fi32, row.names=FALSE))
tdfri32 <- system.time(read.csv(fi32, colClasses=rep("integer", 3)))
unlink(fi32)
rm(dfi32); gc()

i64 <- as.integer64(i32);
tdfi64 <- system.time(dfi64 <- data.frame(a=i64, b=i64, c=i64))
tdfsi64 <- system.time(dfi64[1e6:1,])
fi64 <- tempfile()
tdfwi64 <- system.time(write.csv(dfi64, file=fi64, row.names=FALSE))
tdfri64 <- system.time(read.csv(fi64, colClasses=rep("integer64", 3)))
unlink(fi64)
rm(i64, dfi64); gc()

#I64 <- as.int64(i32);
#tdfI64 <- system.time(dfI64<-data.frame(a=I64, b=I64, c=I64))
#tdfsI64 <- system.time(dfI64[1e6:1,])
#fI64 <- tempfile()
#tdfwI64 <- system.time(write.csv(dfI64, file=fI64, row.names=FALSE))
#tdfrI64 <- system.time(read.csv(fI64, colClasses=rep("int64", 3)))
#unlink(fI64)
#rm(I64, dfI64); gc()

message("-- integer64 coerces 40x/6x faster to data.frame than int64
(and factor 1/9 slower than integer) --")
#tdfI64/tdfi64
tdfi64/tdfi32
message("-- integer64 subscripts from data.frame 20x/2.5x faster than int64
 (and 3x/13x slower than integer) --")
#tdfsI64/tdfsi64
tdfsi64/tdfsi32
message("-- integer64 csv writes about 2x/0.5x faster than int64
(and about 1.5x/5x slower than integer) --")
#tdfwI64/tdfwi64
tdfwi64/tdfwi32
message("-- integer64 csv reads about 3x/1.5 faster than int64
(and about 2x slower than integer) --")
#tdfrI64/tdfri64
tdfri64/tdfri32

rm(i32, d64); gc()


message("-- investigating the impact on garbage collection: --")
message("-- the fragmented structure of int64 messes up R's RAM --")
message("-- and slows down R's gargbage collection just by existing --")

td32 <- double(21)
td32[1] <- system.time(d64 <- double(1e7))[3]
for (i in 2:11)td32[i] <- system.time(gc(), gcFirst=FALSE)[3]
rm(d64)
for (i in 12:21)td32[i] <- system.time(gc(), gcFirst=FALSE)[3]

t64 <- double(21)
t64[1] <- system.time(i64 <- integer64(1e7))[3]
for (i in 2:11)t64[i] <- system.time(gc(), gcFirst=FALSE)[3]
rm(i64)
for (i in 12:21)t64[i] <- system.time(gc(), gcFirst=FALSE)[3]

#T64 <- double(21)
#T64[1] <- system.time(I64 <- int64(1e7))[3]
#for (i in 2:11)T64[i] <- system.time(gc(), gcFirst=FALSE)[3]
#rm(I64)
#for (i in 12:21)T64[i] <- system.time(gc(), gcFirst=FALSE)[3]

#matplot(1:21, cbind(td32, t64, T64), pch=c("d","i","I"), log="y")
matplot(1:21, cbind(td32, t64), pch=c("d","i"), log="y")
  }
}
\seealso{
\code{\link[=integer]{integer()}} in base R
}
\author{
\strong{Maintainer}: Michael Chirico \email{michaelchirico4@gmail.com}

Authors:
\itemize{
  \item Jens Oehlschlägel
}

Other contributors:
\itemize{
  \item Leonardo Silvestri [contributor]
  \item Ofek Shilon [contributor]
  \item Christian Ullerich [contributor]
}

}
\keyword{classes}
\keyword{internal}
\keyword{manip}
\keyword{package}

\name{rank.integer64}
\alias{rank.integer64}
\title{Sample Ranks from integer64}
\description{
  Returns the sample ranks of the values in a vector.  Ties (i.e., equal
  values) are averaged and missing values propagated.
}
\usage{
	\method{rank}{integer64}(x, method = NULL, \dots)
}
\arguments{
  \item{x}{a integer64 vector}
  \item{method}{
	NULL for automatic method selection or a suitable low-level method, see details
}
  \item{\dots}{ignored}
}
\details{
  This function automatically chooses from several low-level functions considering the size of \code{x} and the availability of a cache. 
  Suitable methods are \code{\link{sortorderrnk}} (fast ordering) 
and \code{\link{orderrnk}} (memory saving ordering).
}
\value{
  A numeric vector of the same length as \code{x}.
}
\author{
	Jens Oehlschlägel <Jens.Oehlschlaegel@truecluster.com>
}
\seealso{
  \code{\link{order.integer64}}, \code{\link{rank}} and \code{\link{prank}} for percent rank.
}
\examples{
x <- as.integer64(sample(c(rep(NA, 9), 1:9), 32, TRUE))
rank.integer64(x)

stopifnot(identical(rank.integer64(x),  rank(as.integer(x)
, na.last="keep", ties.method = "average")))
}
\keyword{univar}

\name{bit64-package}
\alias{bit64-package}
\alias{bit64}
\alias{integer64}
\alias{is.integer64}
\alias{is.integer.integer64}
\alias{is.vector.integer64}
%as.vector.integer64 removed as requested by the CRAN maintainer \alias{as.vector.integer64}
\alias{length<-.integer64}
\alias{print.integer64}
\alias{str.integer64}
\docType{package}
\title{
   A S3 class for vectors of 64bit integers
}
\description{
Package 'bit64' provides fast serializable S3 atomic 64bit (signed) integers 
that can be used in vectors, matrices, arrays and data.frames. Methods are 
available for coercion from and to logicals, integers, doubles, characters  
and factors as well as many elementwise and summary functions. 
\cr
\bold{Version 0.8}
With 'integer64' vectors you can store very large integers at the expense
of 64 bits, which is by factor 7 better than 'int64' from package 'int64'.
Due to the smaller memory footprint, the atomic vector architecture and  
using only S3 instead of S4 classes, most operations are one to three orders 
of magnitude faster: Example speedups are 4x for serialization, 250x for 
adding, 900x for coercion and 2000x for object creation. Also 'integer64' 
avoids an ongoing (potentially infinite) penalty for garbage collection
observed during existence of 'int64' objects (see code in example section). 
\cr
\bold{Version 0.9}
Package 'bit64' - which extends R with fast 64-bit integers - now has fast
(single-threaded) implementations the most important univariate algorithmic 
operations (those based on hashing and sorting). We now have methods for 
'match', '%in%', 'duplicated', 'unique', 'table', 'sort', 'order', 'rank', 
'quantile', 'median' and 'summary'. Regarding data management we also have 
novel generics 'unipos' (positions of the unique values), 'tiepos' (
positions of ties), 'keypos' (positions of foreign keys in a sorted 
dimension table) and derived methods 'as.factor' and 'as.ordered'. This 64-
bit functionality is implemented carefully to be not slower than the 
respective 32-bit operations in Base R and also to avoid outlying waiting 
times observed with 'order', 'rank' and 'table' (speedup factors 20/16/200 
respective). This increases the dataset size with wich we can work truly 
interactive. The speed is achieved by simple heuristic optimizers in high-
level functions choosing the best from multiple low-level algorithms and 
further taking advantage of a novel caching if activated. In an example R 
session using a couple of these operations the 64-bit integers performed 22x
 faster than base 32-bit integers, hash-caching improved this to 24x, 
sortorder-caching was most efficient with 38x (caching hashing and sorting 
is not worth it with 32x at duplicated RAM consumption).
}
\usage{
 integer64(length)
 \method{is}{integer64}(x)
 \method{length}{integer64}(x) <- value
 \method{print}{integer64}(x, quote=FALSE, \dots)
 \method{str}{integer64}(object, vec.len  = strO$vec.len, give.head = TRUE, give.length = give.head, \dots)
}
\arguments{
  \item{length}{ length of vector using \code{\link{integer}} }
  \item{x}{ an integer64 vector }
  \item{object}{ an integer64 vector }
  \item{value}{ an integer64 vector of values to be assigned }
  \item{quote}{ logical, indicating whether or not strings should be printed with surrounding quotes. }
  \item{vec.len}{ see \code{\link[utils]{str}} }
  \item{give.head}{ see \code{\link[utils]{str}} }
  \item{give.length}{ see \code{\link[utils]{str}} }
  \item{\dots}{ further arguments to the \code{\link{NextMethod}} }
}
\details{
\tabular{ll}{
   Package: \tab bit64\cr
   Type: \tab Package\cr
   Version: \tab 0.5.0\cr
   Date: \tab 2011-12-12\cr
   License: \tab GPL-2\cr
   LazyLoad: \tab yes\cr
   Encoding: \tab latin1\cr
}
}
\section{Design considerations}{
  64 bit integers are related to big data: we need them to overcome address space limitations. 
  Therefore performance of the 64 bit integer type is critical. 
  In the S language -- designed in 1975 -- atomic objects were defined to be vectors for a couple of good reasons:
  simplicity, option for implicit parallelization, good cache locality. 
  In recent years many analytical databases have learnt that lesson: column based data bases provide superior performance
  for many applications, the result are products such as MonetDB, Sybase IQ, Vertica, Exasol, Ingres Vectorwise.
  If we introduce 64 bit integers not natively in Base R but as an external package, we should at least strive to 
  make them as 'basic' as possible. Therefore the design choice of bit64 not only differs from \code{int64}, it is obvious: 
  Like the other atomic types in Base R, we model data type 'integer64' as a contiguous \code{\link{atomic}} vector in memory, 
  and we use the more basic \code{\link{S3}} class system, not \code{\link{S4}}. Like package \code{int64} we want our 'integer64' to be \code{\link{serialize}able}, 
  therefore we also use an existing data type as the basis. Again the choice is obvious: R has only one 64 bit data type: doubles.
  By using \code{\link{double}s}, \code{integer64} \code{\link{inherits}} some functionality such as \code{\link{is.atomic}}, \code{\link{length}}, 
  \code{\link{length<-}}, \code{\link{names}}, \code{\link{names<-}}, \code{\link{dim}}, \code{\link{dim<-}}, \code{\link{dimnames}}, \code{\link{dimnames}}.
  \cr
  Our R level functions strictly follow the functional programming paragdim: 
  no modification of arguments or other sideffects. Before version 0.93  we internally deviated from the strict paradigm
  in order to boost performance. Our C functions do not create new return values, 
  instead we pass-in the memory to be returned as an argument. This gives us the freedom to apply the C-function 
  to new or old vectors, which helps to avoid unnecessary memory allocation, unnecessary copying and unnessary garbage collection.
  Prior to 0.93 \emph{within} our R functions we also deviated from conventional R programming by not using \code{\link{attr<-}} and \code{\link{attributes<-}} 
  because they always did new memory allocation and copying in older R versions. If we wanted to set attributes of return values that we have freshly created,
  we instead used functions \code{\link[bit:getsetattr]{setattr}} and \code{\link[bit:getsetattr]{setattributes}} from package \code{\link[bit]{bit}}. 
  From version 0.93 \code{\link[bit:getsetattr]{setattr}} is only used for manipulating \code{\link{cache}} objects, in \code{\link{ramsort.integer64}} and \code{\link{sort.integer64}} and in \code{\link{as.data.frame.integer64}}.
}
\section{Arithmetic precision and coercion}{
  The fact that we introduce 64 bit long long integers -- without introducing 128-bit long doubles -- creates some subtle challenges:
  Unlike 32 bit \code{\link{integer}s}, the \code{integer64} are no longer a proper subset of \code{\link{double}}. 
  If a binary arithmetic operation does involve a \code{double} and a \code{integer}, it is a no-brainer to return \code{double} 
  without loss of information. If an \code{integer64} meets a \code{double}, it is not trivial what type to return. 
  Switching to \code{integer64} limits our ability to represent very large numbers, switching to \code{double} limits our ability 
  to distinguish \code{x} from \code{x+1}. Since the latter is the purpose of introducing 64 bit integers, we usually return \code{integer64} 
  from functions involving \code{integer64}, for example in \code{\link[=c.integer64]{c}}, \code{\link[=cbind.integer64]{cbind}} 
  and \code{\link[=rbind.integer64]{rbind}}. 
  \cr
  Different from Base R, our operators \code{\link[=+.integer64]{+}}, 
  \code{\link[=-.integer64]{-}}, \code{\link[=\%/\%.integer64]{\%/\%}} and \code{\link[=\%\%.integer64]{\%\%}} coerce their arguments to 
  \code{integer64} and always return \code{integer64}. 
  \cr
  The multiplication operator \code{\link[=*.integer64]{*}} coerces its first argument to \code{integer64} 
  but allows its second argument to be also \code{double}: the second argument is internaly coerced to 'long double' 
  and the result of the multiplication is returned as \code{integer64}. 
  \cr
  The division \code{\link[=/.integer64]{/}} and power \code{\link[=^.integer64]{^}} operators also coerce their first argument to \code{integer64} 
  and coerce internally their second argument to 'long double', they return as \code{double}, like \code{\link[=sqrt.integer64]{sqrt}}, 
  \code{\link[=log.integer64]{log}}, \code{\link[=log2.integer64]{log2}} and \code{\link[=log10.integer64]{log10}} do. 

 \tabular{ccccccccc}{
  \bold{argument1} \tab \bold{op} \tab \bold{argument2} \tab \bold{->} \tab \bold{coerced1} \tab \bold{op} \tab \bold{coerced2} \tab \bold{->} \tab \bold{result} \cr
  integer64 \tab + \tab double \tab -> \tab integer64 \tab + \tab integer64 \tab -> \tab integer64 \cr
  double \tab + \tab integer64 \tab -> \tab integer64 \tab + \tab integer64 \tab -> \tab integer64 \cr
  integer64 \tab - \tab double \tab -> \tab integer64 \tab - \tab integer64 \tab -> \tab integer64 \cr
  double \tab - \tab integer64 \tab -> \tab integer64 \tab - \tab integer64 \tab -> \tab integer64 \cr
  integer64 \tab \%/\% \tab double \tab -> \tab integer64 \tab \%/\% \tab integer64 \tab -> \tab integer64 \cr
  double \tab \%/\% \tab integer64 \tab -> \tab integer64 \tab \%/\% \tab integer64 \tab -> \tab integer64 \cr
  integer64 \tab \%\% \tab double \tab -> \tab integer64 \tab \%\% \tab integer64 \tab -> \tab integer64 \cr
  double \tab \%\% \tab integer64 \tab -> \tab integer64 \tab \%\% \tab integer64 \tab -> \tab integer64 \cr
  integer64 \tab * \tab double \tab -> \tab integer64 \tab * \tab long double \tab -> \tab integer64 \cr
  double \tab * \tab integer64 \tab -> \tab integer64 \tab * \tab integer64 \tab -> \tab integer64 \cr
  integer64 \tab / \tab double \tab -> \tab integer64 \tab / \tab long double \tab -> \tab double \cr
  double \tab / \tab integer64 \tab -> \tab integer64 \tab / \tab long double \tab -> \tab double \cr
  integer64 \tab ^ \tab double \tab -> \tab integer64 \tab / \tab long double \tab -> \tab double \cr
  double \tab ^ \tab integer64 \tab -> \tab integer64 \tab / \tab long double \tab -> \tab double \cr
 }
}
\section{Creating and testing S3 class 'integer64'}{
  Our creator function \code{integer64} takes an argument \code{length}, creates an atomic double vector of this length,
  attaches an S3 class attribute 'integer64' to it, and that's it. We simply rely on S3 method dispatch and interpret those 
  64bit elements as 'long long int'. 
  \cr
 \code{\link{is.double}} currently returns TRUE for \code{integer64} and might return FALSE in a later release.
 Consider \code{is.double} to have undefined behaviour and do query \code{is.integer64} \emph{before} querying \code{is.double}.
%As a second line of defense against misinterpretation we make \code{\link{is.double}}
%return \code{FALSE} by making it S3 generic and adding a method \code{\link{as.double.integer64}}. 
  The methods \code{\link{is.integer64}} and \code{\link{is.vector}} both return \code{TRUE} for \code{integer64}. 
 Note that we did not patch \code{\link{storage.mode}} and \code{\link{typeof}}, which both continue returning 'double' 
 Like for 32 bit \code{\link{integer}}, \code{\link{mode}} returns 'numeric' and \code{\link{as.double}}) tries coercing to \code{\link{double}}).
 It is possible that 'integer64' becomes a \code{vmode} in package \code{ff}. 
 \cr
 Further methods for creating \code{integer64} are \code{\link[=range.integer64]{range}} which returns the range of the data type if calles without arguments,
 \code{\link[=rep.integer64]{rep}}, \code{\link[=seq.integer64]{seq}}. 
 \cr
 For all available methods on \code{integer64} vectors see the index below and the examples.
}
\section{Index of implemented methods}{
\tabular{rrl}{
   \bold{creating,testing,printing} \tab \bold{see also}          \tab \bold{description} \cr
   \code{NA_integer64_} \tab \code{\link{NA_integer_}} \tab NA constant \cr
   \code{integer64} \tab \code{\link{integer}} \tab create zero atomic vector \cr
   \code{\link{runif64}} \tab \code{\link{runif}} \tab create random vector \cr
   \code{\link{rep.integer64}} \tab \code{\link{rep}} \tab  \cr
   \code{\link{seq.integer64}} \tab \code{\link{seq}} \tab  \cr
   \code{\link{is.integer64}} \tab \code{\link{is}} \tab  \cr
                                     \tab \code{\link{is.integer}} \tab inherited from Base R \cr
   %\code{\link{is.double.integer64}} \tab \code{\link{is.double}} \tab  \cr
   \code{\link{is.vector.integer64}} \tab \code{\link{is.vector}} \tab  \cr
   \code{\link{identical.integer64}} \tab \code{\link{identical}} \tab  \cr
   \code{\link{length<-.integer64}} \tab \code{\link{length<-}} \tab  \cr
                                     \tab \code{\link{length}} \tab inherited from Base R \cr
                                     \tab \code{\link{names<-}} \tab inherited from Base R \cr
                                     \tab \code{\link{names}} \tab inherited from Base R \cr
                                     \tab \code{\link{dim<-}} \tab inherited from Base R \cr
                                     \tab \code{\link{dim}} \tab inherited from Base R \cr
                                     \tab \code{\link{dimnames<-}} \tab inherited from Base R \cr
                                     \tab \code{\link{dimnames}} \tab inherited from Base R \cr
                                    \tab \code{\link{str}} \tab inherited from Base R, does not print values correctly \cr
   \code{\link{print.integer64}} \tab \code{\link{print}} \tab  \cr
   \code{\link{str.integer64}} \tab \code{\link{str}} \tab  \cr
 \cr
   \bold{coercing to integer64} \tab \bold{see also}          \tab \bold{description} \cr
   \code{\link{as.integer64}} \tab   \tab generic \cr
   \code{\link{as.integer64.bitstring}} \tab \code{\link{as.bitstring}} \tab  \cr
   \code{\link{as.integer64.character}} \tab \code{\link{character}} \tab  \cr
   \code{\link{as.integer64.double}} \tab \code{\link{double}} \tab  \cr
   \code{\link{as.integer64.integer}} \tab \code{\link{integer}} \tab  \cr
   \code{\link{as.integer64.integer64}} \tab \code{integer64} \tab  \cr
   \code{\link{as.integer64.logical}} \tab \code{\link{logical}} \tab  \cr
   \code{\link{as.integer64.NULL}} \tab \code{\link{NULL}} \tab  \cr
 \cr
   \bold{coercing from integer64} \tab \bold{see also}          \tab \bold{description} \cr
   \code{\link{as.bitstring}} \tab \code{\link{as.bitstring}} \tab generic \cr
   \code{\link{as.bitstring.integer64}} \tab  \tab  \cr
   \code{\link{as.character.integer64}} \tab \code{\link{as.character}} \tab  \cr
   \code{\link{as.double.integer64}} \tab \code{\link{as.double}} \tab  \cr
   \code{\link{as.integer.integer64}} \tab \code{\link{as.integer}} \tab  \cr
   \code{\link{as.logical.integer64}} \tab \code{\link{as.logical}} \tab  \cr
   %as.vector.integer64 removed as requested by the CRAN maintainer \code{\link{as.vector.integer64}} \tab \code{\link{as.vector}} \tab  \cr
 \cr
   \bold{data structures} \tab \bold{see also}          \tab \bold{description} \cr
   \code{\link{c.integer64}} \tab \code{\link{c}} \tab vector concatenate \cr
   \code{\link{cbind.integer64}} \tab \code{\link{cbind}} \tab column bind \cr
   \code{\link{rbind.integer64}} \tab \code{\link{rbind}} \tab row bind \cr
   \code{\link{as.data.frame.integer64}} \tab \code{\link{as.data.frame}} \tab coerce atomic object to data.frame \cr
                                         \tab \code{\link{data.frame}} \tab inherited from Base R since we have coercion \cr
 \cr
   \bold{subscripting} \tab \bold{see also}          \tab \bold{description} \cr
   \code{\link{[.integer64}} \tab \code{\link{[}} \tab vector and array extract \cr
   \code{\link{[<-.integer64}} \tab \code{\link{[<-}} \tab vector and array assign \cr
   \code{\link{[[.integer64}} \tab \code{\link{[[}} \tab scalar extract \cr
   \code{\link{[[<-.integer64}} \tab \code{\link{[[<-}} \tab scalar assign \cr
 \cr
   \bold{binary operators} \tab \bold{see also}          \tab \bold{description} \cr
   \code{\link{+.integer64}} \tab \code{\link{+}} \tab returns integer64 \cr
   \code{\link{-.integer64}} \tab \code{\link{-}} \tab returns integer64 \cr
   \code{\link{*.integer64}} \tab \code{\link{*}} \tab returns integer64 \cr
   \code{\link{^.integer64}} \tab \code{\link{^}} \tab returns double \cr
   \code{\link{/.integer64}} \tab \code{\link{/}} \tab returns double \cr
   \code{\link{\%/\%.integer64}} \tab \code{\link{\%/\%}} \tab returns integer64 \cr
   \code{\link{\%\%.integer64}} \tab \code{\link{\%\%}} \tab returns integer64 \cr
 \cr
   \bold{comparison operators} \tab \bold{see also}          \tab \bold{description} \cr
   \code{\link{==.integer64}} \tab \code{\link{==}} \tab  \cr
   \code{\link{!=.integer64}} \tab \code{\link{!=}} \tab  \cr
   \code{\link{<.integer64}} \tab \code{\link{<}} \tab  \cr
   \code{\link{<=.integer64}} \tab \code{\link{<=}} \tab  \cr
   \code{\link{>.integer64}} \tab \code{\link{>}} \tab  \cr
   \code{\link{>=.integer64}} \tab \code{\link{>=}} \tab  \cr
 \cr
   \bold{logical operators} \tab \bold{see also}          \tab \bold{description} \cr
   \code{\link{!.integer64}} \tab \code{\link{!}} \tab  \cr
   \code{\link{&.integer64}} \tab \code{\link{&}} \tab  \cr
   \code{\link{|.integer64}} \tab \code{\link{|}} \tab  \cr
   \code{\link{xor.integer64}} \tab \code{\link{xor}} \tab  \cr
 \cr
   \bold{math functions} \tab \bold{see also}          \tab \bold{description} \cr
   \code{\link{is.na.integer64}} \tab \code{\link{is.na}} \tab returns logical \cr
   \code{\link{format.integer64}} \tab \code{\link{format}} \tab returns character \cr
   \code{\link{abs.integer64}} \tab \code{\link{abs}} \tab returns integer64 \cr
   \code{\link{sign.integer64}} \tab \code{\link{sign}} \tab returns integer64 \cr
   \code{\link{log.integer64}} \tab \code{\link{log}} \tab returns double \cr
   \code{\link{log10.integer64}} \tab \code{\link{log10}} \tab  returns double \cr
   \code{\link{log2.integer64}} \tab \code{\link{log2}} \tab  returns double \cr
   \code{\link{sqrt.integer64}} \tab \code{\link{sqrt}} \tab  returns double \cr
   \code{\link{ceiling.integer64}} \tab \code{\link{ceiling}} \tab dummy returning its argument \cr
   \code{\link{floor.integer64}} \tab \code{\link{floor}} \tab dummy returning its argument \cr
   \code{\link{trunc.integer64}} \tab \code{\link{trunc}} \tab dummy returning its argument \cr
   \code{\link{round.integer64}} \tab \code{\link{round}} \tab dummy returning its argument \cr
   \code{\link{signif.integer64}} \tab \code{\link{signif}} \tab dummy returning its argument \cr
 \cr
   \bold{cumulative functions} \tab \bold{see also}          \tab \bold{description} \cr
   \code{\link{cummin.integer64}} \tab \code{\link{cummin}} \tab \cr
   \code{\link{cummax.integer64}} \tab \code{\link{cummax}} \tab \cr
   \code{\link{cumsum.integer64}} \tab \code{\link{cumsum}} \tab \cr
   \code{\link{cumprod.integer64}} \tab \code{\link{cumprod}} \tab \cr
   \code{\link{diff.integer64}} \tab \code{\link{diff}} \tab \cr
 \cr
   \bold{summary functions} \tab \bold{see also}          \tab \bold{description} \cr
   \code{\link{range.integer64}} \tab \code{\link{range}} \tab \cr
   \code{\link{min.integer64}} \tab \code{\link{min}} \tab  \cr
   \code{\link{max.integer64}} \tab \code{\link{max}} \tab  \cr
   \code{\link{sum.integer64}} \tab \code{\link{sum}} \tab  \cr
   \code{\link{mean.integer64}} \tab \code{\link{mean}} \tab  \cr
   \code{\link{prod.integer64}} \tab \code{\link{prod}} \tab  \cr
   \code{\link{all.integer64}} \tab \code{\link{all}} \tab  \cr
   \code{\link{any.integer64}} \tab \code{\link{any}} \tab  \cr
 \cr
   \bold{algorithmically complex functions} \tab \bold{see also}          \tab \bold{description (caching)}  \cr
   \code{\link{match.integer64}} \tab \code{\link{match}} \tab position of x in table (h//o/so) \cr
   \code{\link{\%in\%.integer64}} \tab \code{\link{\%in\%}} \tab is x in table? (h//o/so) \cr
   \code{\link{duplicated.integer64}} \tab \code{\link{duplicated}} \tab is current element duplicate of previous one? (h//o/so) \cr
   \code{\link{unique.integer64}} \tab \code{\link{unique}} \tab (shorter) vector of unique values only (h/s/o/so) \cr
   \code{\link{unipos.integer64}} \tab \code{\link{unipos}} \tab positions corresponding to unique values (h/s/o/so) \cr
   \code{\link{tiepos.integer64}} \tab \code{\link{tiepos}} \tab positions of values that are tied (//o/so) \cr
   \code{\link{keypos.integer64}} \tab \code{\link{keypos}} \tab position of current value in sorted list of unique values (//o/so) \cr
   \code{\link{as.factor.integer64}} \tab \code{\link{as.factor}} \tab convert to (unordered) factor with sorted levels of previous values (//o/so) \cr
   \code{\link{as.ordered.integer64}} \tab \code{\link{as.ordered}} \tab convert to ordered factor with sorted levels of previous values (//o/so) \cr
   \code{\link{table.integer64}} \tab \code{\link{table}} \tab unique values and their frequencies (h/s/o/so) \cr
   \code{\link{sort.integer64}} \tab \code{\link{sort}} \tab sorted vector (/s/o/so) \cr
   \code{\link{order.integer64}} \tab \code{\link{order}} \tab positions of elements that would create sorted vector (//o/so) \cr
   \code{\link{rank.integer64}} \tab \code{\link{rank}} \tab (average) ranks of non-NAs, NAs kept in place (/s/o/so) \cr
   \code{\link{quantile.integer64}} \tab \code{\link{quantile}} \tab (existing) values at specified percentiles (/s/o/so) \cr
   \code{\link{median.integer64}} \tab \code{\link{median}} \tab (existing) value at percentile 0.5 (/s/o/so) \cr
   \code{\link{summary.integer64}} \tab \code{\link{summary}} \tab  (/s/o/so) \cr
   \code{\link{all.equal.integer64}} \tab \code{\link{all.equal}} \tab test if two objects are (nearly) equal (/s/o/so) \cr
 \cr
   \bold{helper functions} \tab \bold{see also}          \tab \bold{description} \cr
   \code{\link{minusclass}} \tab \code{\link{minusclass}} \tab removing class attritbute \cr
   \code{\link{plusclass}} \tab \code{\link{plusclass}} \tab inserting class attribute \cr
   \code{\link{binattr}} \tab \code{\link{binattr}} \tab define binary op behaviour \cr
 \cr
   \bold{tested I/O functions} \tab \bold{see also}          \tab \bold{description} \cr
                               \tab \code{\link{read.table}} \tab inherited from Base R \cr
                               \tab \code{\link{write.table}} \tab inherited from Base R \cr
                               \tab \code{\link{serialize}} \tab inherited from Base R \cr
                               \tab \code{\link{unserialize}} \tab inherited from Base R \cr
                               \tab \code{\link{save}} \tab inherited from Base R \cr
                               \tab \code{\link{load}} \tab inherited from Base R \cr
                               \tab \code{\link{dput}} \tab inherited from Base R \cr
                               \tab \code{\link{dget}} \tab inherited from Base R \cr
}
}
\section{Limitations inherited from implementing 64 bit integers via an external package}{
  \itemize{
    \item \bold{vector size} of atomic vectors is still limited to \code{\link{.Machine}$integer.max}. 
    However, external memory extending packages such as \code{ff} or \code{bigmemory} 
    can extend their address space now with \code{integer64}. Having 64 bit integers also help 
    with those not so obvious address issues that arise once we exchange data with SQL databases 
    and datawarehouses, which use big integers as surrogate keys, e.g. on indexed primary key columns.
    This puts R into a relatively strong position compared to certain commercial statistical 
    softwares, which sell database connectivity but neither have the range of 64 bit integers, 
    nor have integers at all, nor have a single numeric data type in their macro-glue-language.

    \item \bold{literals} such as \code{123LL} would require changes to Base R, up to then we need to write (and call) 
    \code{as.integer64(123L)} or \code{as.integer64(123)} or \code{as.integer64('123')}. 
    Only the latter allows to specify numbers beyond Base R's numeric data types and therefore is the recommended
    way to use -- using only one way may facilitate migrating code to literals at a later stage.

  }
}
\section{Limitations inherited from Base R, Core team, can you change this?}{
  \itemize{
    \item \bold{\code{\link{identical}}} with default parameters does not distinguish all bit-patterns of doubles. 
    For testing purposes we provide a wrapper \code{\link{identical.integer64}} that will distinguish all bit-patterns.
    It would be desireable to have a single call of \code{\link{identical}} handle both, \code{\link{double}} and \code{integer64}.

    \item the \bold{colon} operator \code{\link{:}} officially does not dispatches S3 methods, however, we have made it generic
     \preformatted{
     from <- lim.integer64()[1]
     to <- from+99
     from:to
   }
   As a limitation remains: it will only dispatch at its first argument \code{from} but not at its second \code{to}.

    \item \bold{\code{\link{is.double}}} does not dispatches S3 methods, However, we have made it generic 
		and it will return \code{FALSE} on \code{integer64}.

    \item \bold{\code{\link{c}}} only dispatches \code{\link{c.integer64}} if the first argument is \code{integer64}
    and it does not recursively dispatch the proper method when called with argument \code{recursive=TRUE}
    Therefore \preformatted{
      c(list(integer64,integer64))
    }
     does not work and for now you can only call \preformatted{
       c.integer64(list(x,x))
     }

    \item \bold{generic binary operators} fail to dispatch *any* user-defined S3 method 
    if the two arguments have two different S3 classes. For example we have two classes 
    \code{\link{bit}} and \code{\link{bitwhich}} sparsely representing boolean vectors 
    and we have methods \code{\link{&.bit}} and \code{\link{&.bitwhich}}. For an expression
    involving both as in \code{ bit & bitwhich}, none of the two methods is dispatched. 
    Instead a standard method is dispatched, which neither handles \code{\link{bit}} 
    nor \code{\link{bitwhich}}. Although it lacks symmetry, the better choice would be to 
    dispatch simply the method of the class of the first argument in case of class conflict. 
    This choice would allow authors of extension packages providing coherent behaviour 
    at least within their contributed classes. But as long as none of the package authors 
    methods is dispatched, he cannot handle the conflicting classes at all.

    \item \bold{\code{\link{unlist}}} is not generic and if it were, we would face similar problems as with \code{c()}

    \item \bold{\code{\link{vector}}} with argument \code{mode='integer64'} cannot work without adjustment of Base R
    \item \bold{\code{\link{as.vector}}} with argument \code{mode='integer64'} cannot work without adjustment of Base R

    \item \bold{\code{\link{is.vector}}} does not dispatch its method \code{\link{is.vector.integer64}}

    \item \bold{\code{\link{mode<-}}} drops the class 'integer64' which is returned from \code{as.integer64}.
       Also it does not remove an existing class 'integer64' when assigning mode 'integer'. 

    \item \bold{\code{\link{storage.mode<-}}} does not support external data types such as \code{as.integer64}

    \item \bold{\code{\link{matrix}}} does drop the 'integer64' class attribute.

    \item \bold{\code{\link{array}}}  does drop the 'integer64' class attribute. 
           In current R versions (1.15.1) this can be circumvented by activating the function 
						\code{as.vector.integer64} further down this file.
						However, the CRAN maintainer has requested to remove \code{as.vector.integer64}, 
						even at the price of breaking previously working functionality of the package. 

    \item \bold{\code{\link{str}}} does not print the values of \code{integer64} correctly

  }
}
\section{further limitations}{
  \itemize{
    \item \bold{subscripting} non-existing elements and subscripting with \code{NA}s is currently not supported. 
    Such subscripting currently returns \code{9218868437227407266} instead of \code{NA} (the \code{NA} value of the underlying double code).
    Following the full R behaviour here would either destroy performance or require extensive C-coding. 
  }
}
\note{
   \code{integer64} are useful for handling database keys and exact counting in +-2^63.
   Do not use them as replacement for 32bit integers, integer64 are not
   supported for subscripting by R-core and they have different semantics 
   when combined with double. Do understand that \code{integer64} can only be
   useful over \code{double} if we do not coerce it to \code{double}. \cr
  \cr
  While \cr
  integer + double -> double + double -> double \cr
  or \cr
  1L + 0.5 -> 1.5 \cr 
  for additive operations we coerce to \code{integer64} \cr
  integer64 + double ->  integer64 + integer64 -> integer64 \cr
  hence \cr
  as.integer64(1) + 0.5 -> 1LL + 0LL -> 1LL \cr
  \cr
  see section "Arithmetic precision and coercion" above
}
\value{
  \code{integer64} returns a vector of 'integer64', 
   i.e. a vector of \code{\link{double}} decorated with class 'integer64'.
}
\author{
Jens Oehlschlägel <Jens.Oehlschlaegel@truecluster.com>
Maintainer: Jens Oehlschlägel <Jens.Oehlschlaegel@truecluster.com>
}
\keyword{ package }
\keyword{ classes }
\keyword{ manip }
\seealso{ \code{\link{integer}} in base R }
\examples{
message("Using integer64 in vector")
x <- integer64(8)    # create 64 bit vector
x
is.atomic(x)         # TRUE
is.integer64(x)      # TRUE
is.numeric(x)        # TRUE
is.integer(x)        # FALSE - debatable
is.double(x)         # FALSE - might change
x[] <- 1:2           # assigned value is recycled as usual
x[1:6]               # subscripting as usual
length(x) <- 13      # changing length as usual
x
rep(x, 2)            # replicate as usual
seq(as.integer64(1), 10)     # seq.integer64 is dispatched on first given argument
seq(to=as.integer64(10), 1)  # seq.integer64 is dispatched on first given argument
seq.integer64(along.with=x)  # or call seq.integer64 directly
# c.integer64 is dispatched only if *first* argument is integer64 ...
x <- c(x,runif(length(x), max=100)) 
# ... and coerces everything to integer64 - including double
x                                   
names(x) <- letters  # use names as usual
x

message("Using integer64 in array - note that 'matrix' currently does not work")
message("as.vector.integer64 removed as requested by the CRAN maintainer")
message("as consequence 'array' also does not work anymore")
%y <- array(as.integer64(NA), dim=c(3,4), dimnames=list(letters[1:3], LETTERS[1:4]))
message("we still can create a matrix or array by assigning 'dim'")
y <- rep(as.integer64(NA), 12)
dim(y) <- c(3,4)
dimnames(y) <- list(letters[1:3], LETTERS[1:4])
y["a",] <- 1:2       # assigning as usual
y
y[1:2,-4]            # subscripting as usual
# cbind.integer64 dispatched on any argument and coerces everything to integer64
cbind(E=1:3, F=runif(3, 0, 100), G=c("-1","0","1"), y)

message("Using integer64 in data.frame")
str(as.data.frame(x))
str(as.data.frame(y))
str(data.frame(y))
str(data.frame(I(y)))
d <- data.frame(x=x, y=runif(length(x), 0, 100))
d
d$x

message("Using integer64 with csv files")
fi64 <- tempfile()
write.csv(d, file=fi64, row.names=FALSE)
e <- read.csv(fi64, colClasses=c("integer64", NA))
unlink(fi64)
str(e)
identical.integer64(d$x,e$x)

message("Serializing and unserializing integer64")
dput(d, fi64)
e <- dget(fi64)
identical.integer64(d$x,e$x)
e <- d[,]
save(e, file=fi64)
rm(e)
load(file=fi64)
identical.integer64(d,e)

### A couple of unit tests follow hidden in a dontshow{} directive ###
  \dontshow{
message("Testing identical.integer64")
i64 <- as.double(NA); class(i64) <- "integer64"
stopifnot(identical(unclass(i64-1), unclass(i64+1)))
stopifnot(identical(i64-1, i64+1))
stopifnot(!identical.integer64(i64-1, i64+1))

message("Testing dispatch of 'c' method")
stopifnot(identical.integer64(c(integer64(0), NA), as.integer64(NA)))
message("Dispatch on the second argument fails and we want to be notified once that changes")
stopifnot(!identical.integer64(c(NA, integer64(0)), as.integer64(NA)))

message("Testing minus and plus")
d64 <- c(-.Machine$double.base^.Machine$double.digits, -.Machine$integer.max, -1, 0, 1, .Machine$integer.max, .Machine$double.base^.Machine$double.digits)
i64 <- as.integer64(d64)
stopifnot(identical.integer64(i64-1+1,i64))
stopifnot(identical.integer64(i64+1-1,i64))

message("Testing minus and plus edge cases and 'rev'\n")
stopifnot(identical.integer64(lim.integer64()+1-1, c(lim.integer64()[1], NA)))
stopifnot(identical.integer64(rev(lim.integer64())-1+1, c(lim.integer64()[2], NA)))

message("Testing 'range.integer64', multiplication and integer division")
i64 <- integer64(63)
i64[1] <- 1
for (i in 2:63)
	i64[i] <- 2*i64[i-1]
stopifnot(identical.integer64(i64 * rev(i64), rep(i64[63], 63)))
for (i in 63:2)
	i64[i-1] <- i64[i]\%/\%2
stopifnot(identical.integer64(i64 * rev(i64), rep(i64[63], 63)))
for (i in 63:2)
	i64[i-1] <- i64[i]/2
stopifnot(identical.integer64(i64 * rev(i64), rep(i64[63], 63)))
stopifnot(identical.integer64(c( -i64[63] - (i64[63]-1), i64[63]+(i64[63]-1) ), lim.integer64()))

stopifnot(identical.integer64(i64[-1]\%/\%2*as.integer64(2), i64[-1]))
stopifnot(identical.integer64(i64[-1]\%/\%2L*as.integer64(2), i64[-1]))
stopifnot(identical.integer64(i64[-1]/2*as.integer64(2), i64[-1]))
stopifnot(identical.integer64(i64[-1]/2*as.integer64(2), i64[-1]))

stopifnot(identical.integer64(i64[-63]*2\%/\%2, i64[-63]))
stopifnot(identical.integer64(i64[-63]*2L\%/\%2L, i64[-63]))
stopifnot(identical.integer64(as.integer64(i64[-63]*2/2), i64[-63]))
stopifnot(identical.integer64(as.integer64(i64[-63]*2L/2L), i64[-63]))

message("Testing sqrt, power and log")
stopifnot(identical.integer64( as.integer64(sqrt(i64[-1][c(FALSE, TRUE)])*sqrt(i64[-1][c(FALSE, TRUE)])), i64[-1][c(FALSE, TRUE)] ))

stopifnot(identical.integer64(as.integer64(2)^(0:62), i64))
stopifnot(identical.integer64(as.integer64(0:62), as.integer64(round(log2(i64)))))
stopifnot(identical.integer64(as.integer64(round(log(as.integer64(2)^(0:62), 2))), as.integer64(0:62)))
stopifnot(identical.integer64(as.integer64(round(log(as.integer64(3)^(0:39), 3))), as.integer64(0:39)))
stopifnot(identical.integer64(as.integer64(round(log(as.integer64(10)^(0:18), 10))), as.integer64(0:18)))
stopifnot(identical.integer64(as.integer64(round(log10(as.integer64(10)^(0:18)))), as.integer64(0:18)))

stopifnot(identical.integer64((as.integer64(2)^(1:62))^(1/1:62), as.integer64(rep(2, 62))))
stopifnot(identical.integer64((as.integer64(3)^(1:39))^(1/1:39), as.integer64(rep(3, 39))))
stopifnot(identical.integer64((as.integer64(10)^(1:18))^(1/1:18), as.integer64(rep(10, 18))))

message("Testing c and rep")
stopifnot(identical.integer64( as.integer64(rep(1:3, 1:3)), rep(as.integer64(1:3), 1:3)))
stopifnot(identical.integer64( as.integer64(rep(1:3, 3)), rep(as.integer64(1:3), 3)))
 
x <- as.double(c(NA,NA,NA))
class(x) <- "integer64"
x <- x + -1:1
stopifnot(identical.integer64(rep(x, 3), c(x,x,x) ))
stopifnot(identical.integer64(c.integer64(list(x,x,x), recursive=TRUE), c(x,x,x) ))

message("Testing seq")
stopifnot(identical.integer64(seq(as.integer64(1), 10, 2), as.integer64(seq(1, 10, 2)) ))
stopifnot(identical.integer64(seq(as.integer64(1), by=2, length.out=5), as.integer64(seq(1, by=2, length.out=5)) ))
stopifnot(identical.integer64(seq(as.integer64(1), by=2, length.out=6), as.integer64(seq(1, by=2, length.out=6)) ))
stopifnot(identical.integer64(seq.integer64(along.with=3:5), as.integer64(seq(along.with=3:5)) ))
stopifnot(identical.integer64(seq(as.integer64(1), to=-9), as.integer64(seq(1, to=-9)) ))

message("Testing cbind and rbind")
stopifnot(identical.integer64( cbind(as.integer64(1:3), 1:3), {x <- rep(as.integer64(1:3), 2); dim(x)<-c(3,2);x}))
stopifnot(identical.integer64( rbind(as.integer64(1:3), 1:3), t({x <- rep(as.integer64(1:3), 2); dim(x)<-c(3,2);x})))

message("Testing coercion")
stopifnot(identical( as.double(as.integer64(c(NA, seq(0, 9, 0.25)))), as.double(as.integer(c(NA, seq(0, 9, 0.25))))))
stopifnot(identical( as.character(as.integer64(c(NA, seq(0, 9, 0.25)))), as.character(as.integer(c(NA, seq(0, 9, 0.25))))))
stopifnot(identical( as.integer(as.integer64(c(NA, seq(0, 9, 0.25)))), as.integer(c(NA, seq(0, 9, 0.25)))))
stopifnot(identical( as.logical(as.integer64(c(NA, seq(0, 9, 0.25)))), as.logical(as.integer(c(NA, seq(0, 9, 0.25))))))
stopifnot(identical( as.integer(as.integer64(c(NA, FALSE, TRUE))), as.integer(c(NA, FALSE, TRUE))))
stopifnot(identical( as.integer64(as.integer(as.integer64(-9:9))), as.integer64(-9:9)))
stopifnot(identical( as.integer64(as.double(as.integer64(-9:9))), as.integer64(-9:9)))
stopifnot(identical( as.integer64(as.character(as.integer64(-9:9))), as.integer64(-9:9)))
stopifnot(identical( as.integer64(as.character(lim.integer64())), lim.integer64()))

message("-- testing logical operators --")
stopifnot(identical.integer64(!c(NA, -1:1), !c(as.integer64(NA), -1:1)))
stopifnot(identical.integer64(rep(c(NA, -1:1), 4)&rep(c(NA, -1:1), rep(4, 4)), as.integer64(rep(c(NA, -1:1), 4))&as.integer64(rep(c(NA, -1:1), rep(4, 4)))))
stopifnot(identical.integer64(rep(c(NA, -1:1), 4)|rep(c(NA, -1:1), rep(4, 4)), as.integer64(rep(c(NA, -1:1), 4))|as.integer64(rep(c(NA, -1:1), rep(4, 4)))))
stopifnot(identical.integer64(xor(rep(c(NA, -1:1), 4),rep(c(NA, -1:1), rep(4, 4))), xor(as.integer64(rep(c(NA, -1:1), 4)),as.integer64(rep(c(NA, -1:1), rep(4, 4))))))

message("-- testing comparison operators --")
stopifnot(identical.integer64(rep(c(NA, -1:1), 4)==rep(c(NA, -1:1), rep(4, 4)), as.integer64(rep(c(NA, -1:1), 4))==as.integer64(rep(c(NA, -1:1), rep(4, 4)))))
stopifnot(identical.integer64(rep(c(NA, -1:1), 4)!=rep(c(NA, -1:1), rep(4, 4)), as.integer64(rep(c(NA, -1:1), 4))!=as.integer64(rep(c(NA, -1:1), rep(4, 4)))))
stopifnot(identical.integer64(rep(c(NA, -1:1), 4)>rep(c(NA, -1:1), rep(4, 4)), as.integer64(rep(c(NA, -1:1), 4))>as.integer64(rep(c(NA, -1:1), rep(4, 4)))))
stopifnot(identical.integer64(rep(c(NA, -1:1), 4)>=rep(c(NA, -1:1), rep(4, 4)), as.integer64(rep(c(NA, -1:1), 4))>=as.integer64(rep(c(NA, -1:1), rep(4, 4)))))
stopifnot(identical.integer64(rep(c(NA, -1:1), 4)<rep(c(NA, -1:1), rep(4, 4)), as.integer64(rep(c(NA, -1:1), 4))<as.integer64(rep(c(NA, -1:1), rep(4, 4)))))
stopifnot(identical.integer64(rep(c(NA, -1:1), 4)<=rep(c(NA, -1:1), rep(4, 4)), as.integer64(rep(c(NA, -1:1), 4))<=as.integer64(rep(c(NA, -1:1), rep(4, 4)))))

message("-- testing vector functions --")
stopifnot(identical.integer64( is.na(as.integer64(c(NA, -1:1))), is.na(c(NA, -1:1)) ))
stopifnot(identical.integer64( format(as.integer64(c(NA, -1:1))), format(c(NA, -1:1)) ))
stopifnot(identical.integer64( abs(as.integer64(c(NA, -1:1))), as.integer64(abs(c(NA, -1:1))) ))
stopifnot(identical.integer64( sign(as.integer64(c(NA, -1:1))), as.integer64(sign(c(NA, -1:1))) ))
stopifnot(identical.integer64( ceiling(as.integer64(c(NA, -1:1))), as.integer64(ceiling(c(NA, -1:1))) ))
stopifnot(identical.integer64( floor(as.integer64(c(NA, -1:1))), as.integer64(floor(c(NA, -1:1))) ))
stopifnot(identical.integer64( trunc(as.integer64(c(NA, -1:1))), as.integer64(trunc(c(NA, -1:1))) ))
stopifnot(identical.integer64( signif(as.integer64(c(NA, -1:1))), as.integer64(c(NA, -1:1)) ))

message("Testing summary functions")
stopifnot(identical(all(as.integer(1)), all(as.integer64(1))))
stopifnot(identical(all(as.integer(0)), all(as.integer64(0))))
stopifnot(identical(all(as.integer(NA)), all(as.integer64(NA))))
stopifnot(identical(all(as.integer(NA), na.rm=TRUE), all(as.integer64(NA), na.rm=TRUE)))
stopifnot(identical(all(as.integer(1), NA), all(as.integer64(1), NA)))
stopifnot(identical(all(as.integer(0), NA), all(as.integer64(0), NA)))
stopifnot(identical(all(as.integer(1), NA, na.rm=TRUE), all(as.integer64(1), NA, na.rm=TRUE)))
stopifnot(identical(all(as.integer(0), NA, na.rm=TRUE), all(as.integer64(0), NA, na.rm=TRUE)))
stopifnot(identical(all(as.integer(c(1, NA))), all(as.integer64(c(1, NA)))))
stopifnot(identical(all(as.integer(c(0, NA))), all(as.integer64(c(0, NA)))))
stopifnot(identical(all(as.integer(c(1, NA)), na.rm=TRUE), all(as.integer64(c(1, NA)), na.rm=TRUE)))
stopifnot(identical(all(as.integer(c(0, NA)), na.rm=TRUE), all(as.integer64(c(0, NA)), na.rm=TRUE)))

stopifnot(identical(any(as.integer(1)), any(as.integer64(1))))
stopifnot(identical(any(as.integer(0)), any(as.integer64(0))))
stopifnot(identical(any(as.integer(NA)), any(as.integer64(NA))))
stopifnot(identical(any(as.integer(NA), na.rm=TRUE), any(as.integer64(NA), na.rm=TRUE)))
stopifnot(identical(any(as.integer(1), NA), any(as.integer64(1), NA)))
stopifnot(identical(any(as.integer(0), NA), any(as.integer64(0), NA)))
stopifnot(identical(any(as.integer(1), NA, na.rm=TRUE), any(as.integer64(1), NA, na.rm=TRUE)))
stopifnot(identical(any(as.integer(0), NA, na.rm=TRUE), any(as.integer64(0), NA, na.rm=TRUE)))
stopifnot(identical(any(as.integer(c(1, NA))), any(as.integer64(c(1, NA)))))
stopifnot(identical(any(as.integer(c(0, NA))), any(as.integer64(c(0, NA)))))
stopifnot(identical(any(as.integer(c(1, NA)), na.rm=TRUE), any(as.integer64(c(1, NA)), na.rm=TRUE)))
stopifnot(identical(any(as.integer(c(0, NA)), na.rm=TRUE), any(as.integer64(c(0, NA)), na.rm=TRUE)))

stopifnot(identical.integer64(as.integer64(sum(c(2, 3, NA))), sum(as.integer64(c(2, 3, NA)))))
stopifnot(identical.integer64(as.integer64(sum(c(2, 3, NA), na.rm=TRUE)), sum(as.integer64(c(2, 3, NA)), na.rm=TRUE)))
stopifnot(identical.integer64(as.integer64(sum(c(2, 3, NA))), sum(as.integer64(c(2, 3, NA)))))
stopifnot(identical.integer64(as.integer64(sum(c(2, 3, NA), na.rm=TRUE)), sum(as.integer64(c(2, 3, NA)), na.rm=TRUE)))
stopifnot(identical.integer64(as.integer64(sum(2, 3, NA)), sum(as.integer64(2), 3, NA)))
stopifnot(identical.integer64(as.integer64(sum(2, 3, NA, na.rm=TRUE)), sum(as.integer64(2), 3, NA, na.rm=TRUE)))
stopifnot(identical.integer64(as.integer64(sum(2, 3, NA)), sum(as.integer64(2), 3, NA)))
stopifnot(identical.integer64(as.integer64(sum(2, 3, NA, na.rm=TRUE)), sum(as.integer64(2), 3, NA, na.rm=TRUE)))

stopifnot(identical.integer64(as.integer64(prod(c(2, 3, NA))), prod(as.integer64(c(2, 3, NA)))))
stopifnot(identical.integer64(as.integer64(prod(c(2, 3, NA), na.rm=TRUE)), prod(as.integer64(c(2, 3, NA)), na.rm=TRUE)))
stopifnot(identical.integer64(as.integer64(prod(c(2, 3, NA))), prod(as.integer64(c(2, 3, NA)))))
stopifnot(identical.integer64(as.integer64(prod(c(2, 3, NA), na.rm=TRUE)), prod(as.integer64(c(2, 3, NA)), na.rm=TRUE)))
stopifnot(identical.integer64(as.integer64(prod(2, 3, NA)), prod(as.integer64(2), 3, NA)))
stopifnot(identical.integer64(as.integer64(prod(2, 3, NA, na.rm=TRUE)), prod(as.integer64(2), 3, NA, na.rm=TRUE)))
stopifnot(identical.integer64(as.integer64(prod(2, 3, NA)), prod(as.integer64(2), 3, NA)))
stopifnot(identical.integer64(as.integer64(prod(2, 3, NA, na.rm=TRUE)), prod(as.integer64(2), 3, NA, na.rm=TRUE)))

stopifnot(identical.integer64(as.integer64(min(c(2, 3, NA))), min(as.integer64(c(2, 3, NA)))))
stopifnot(identical.integer64(as.integer64(min(c(2, 3, NA), na.rm=TRUE)), min(as.integer64(c(2, 3, NA)), na.rm=TRUE)))
stopifnot(identical.integer64(as.integer64(min(c(2, 3, NA))), min(as.integer64(c(2, 3, NA)))))
stopifnot(identical.integer64(as.integer64(min(c(2, 3, NA), na.rm=TRUE)), min(as.integer64(c(2, 3, NA)), na.rm=TRUE)))
stopifnot(identical.integer64(as.integer64(min(2, 3, NA)), min(as.integer64(2), 3, NA)))
stopifnot(identical.integer64(as.integer64(min(2, 3, NA, na.rm=TRUE)), min(as.integer64(2), 3, NA, na.rm=TRUE)))
stopifnot(identical.integer64(as.integer64(min(2, 3, NA)), min(as.integer64(2), 3, NA)))
stopifnot(identical.integer64(as.integer64(min(2, 3, NA, na.rm=TRUE)), min(as.integer64(2), 3, NA, na.rm=TRUE)))

stopifnot(identical.integer64(as.integer64(max(c(2, 3, NA))), max(as.integer64(c(2, 3, NA)))))
stopifnot(identical.integer64(as.integer64(max(c(2, 3, NA), na.rm=TRUE)), max(as.integer64(c(2, 3, NA)), na.rm=TRUE)))
stopifnot(identical.integer64(as.integer64(max(c(2, 3, NA))), max(as.integer64(c(2, 3, NA)))))
stopifnot(identical.integer64(as.integer64(max(c(2, 3, NA), na.rm=TRUE)), max(as.integer64(c(2, 3, NA)), na.rm=TRUE)))
stopifnot(identical.integer64(as.integer64(max(2, 3, NA)), max(as.integer64(2), 3, NA)))
stopifnot(identical.integer64(as.integer64(max(2, 3, NA, na.rm=TRUE)), max(as.integer64(2), 3, NA, na.rm=TRUE)))
stopifnot(identical.integer64(as.integer64(max(2, 3, NA)), max(as.integer64(2), 3, NA)))
stopifnot(identical.integer64(as.integer64(max(2, 3, NA, na.rm=TRUE)), max(as.integer64(2), 3, NA, na.rm=TRUE)))

stopifnot(identical.integer64(as.integer64(range(c(2, 3, NA))), range(as.integer64(c(2, 3, NA)))))
stopifnot(identical.integer64(as.integer64(range(c(2, 3, NA), na.rm=TRUE)), range(as.integer64(c(2, 3, NA)), na.rm=TRUE)))
stopifnot(identical.integer64(as.integer64(range(c(2, 3, NA))), range(as.integer64(c(2, 3, NA)))))
stopifnot(identical.integer64(as.integer64(range(c(2, 3, NA), na.rm=TRUE)), range(as.integer64(c(2, 3, NA)), na.rm=TRUE)))
stopifnot(identical.integer64(as.integer64(range(2, 3, NA)), range(as.integer64(2), 3, NA)))
stopifnot(identical.integer64(as.integer64(range(2, 3, NA, na.rm=TRUE)), range(as.integer64(2), 3, NA, na.rm=TRUE)))
stopifnot(identical.integer64(as.integer64(range(2, 3, NA)), range(as.integer64(2), 3, NA)))
stopifnot(identical.integer64(as.integer64(range(2, 3, NA, na.rm=TRUE)), range(as.integer64(2), 3, NA, na.rm=TRUE)))

message("-- testing cummulative functions --")
stopifnot(identical.integer64(as.integer64(cumsum(c(2, 3, NA, 1, 4))), cumsum(as.integer64(c(2, 3, NA, 1, 4)))))
stopifnot(identical.integer64(as.integer64(cumprod(c(2, 3, NA, 1, 4))), cumprod(as.integer64(c(2, 3, NA, 1, 4)))))
stopifnot(identical.integer64(as.integer64(cummin(c(2, 3, NA, 1, 4))), cummin(as.integer64(c(2, 3, NA, 1, 4)))))
stopifnot(identical.integer64(as.integer64(cummax(c(2, 3, NA, 1, 4))), cummax(as.integer64(c(2, 3, NA, 1, 4)))))

message("testing diff")
d64 <- diffinv(rep(.Machine$integer.max, 100), lag=2, differences=2)
i64 <- as.integer64(d64)
identical(diff(d64, lag=2, differences=2), as.double(diff(i64, lag=2, differences=2)))

  }

  \dontrun{
message("== Differences between integer64 and int64 ==")
require(bit64)
require(int64)

message("-- integer64 is atomic --")
is.atomic(integer64())
#is.atomic(int64())
str(integer64(3))
#str(int64(3))

message("-- The following performance numbers are measured under RWin64  --")
message("-- under RWin32 the advantage of integer64 over int64 is smaller --")

message("-- integer64 needs 7x/5x less RAM than int64 under 64/32 bit OS 
(and twice the RAM of integer as it should be) --")
#as.vector(object.size(int64(1e6))/object.size(integer64(1e6)))
as.vector(object.size(integer64(1e6))/object.size(integer(1e6)))

message("-- integer64 creates 2000x/1300x faster than int64 under 64/32 bit OS
(and 3x the time of integer) --")
t32 <- system.time(integer(1e8))
t64 <- system.time(integer64(1e8))
#T64 <- system.time(int64(1e7))*10  # using 1e8 as above stalls our R on an i7 8 GB RAM Thinkpad
#T64/t64
t64/t32

i32 <- sample(1e6)
d64 <- as.double(i32)

message("-- the following timings are rather conservative since timings
 of integer64 include garbage collection -- due to looped calls")
message("-- integer64 coerces 900x/100x faster than int64 
 under 64/32 bit OS (and 2x the time of coercing to integer) --")
t32 <- system.time(for(i in 1:1000)as.integer(d64))
t64 <- system.time(for(i in 1:1000)as.integer64(d64))
#T64 <- system.time(as.int64(d64))*1000
#T64/t64
t64/t32
td64 <- system.time(for(i in 1:1000)as.double(i32))
t64 <- system.time(for(i in 1:1000)as.integer64(i32))
#T64 <- system.time(for(i in 1:10)as.int64(i32))*100
#T64/t64
t64/td64

message("-- integer64 serializes 4x/0.8x faster than int64 
 under 64/32 bit OS (and less than 2x/6x the time of integer or double) --")
t32 <- system.time(for(i in 1:10)serialize(i32, NULL))
td64 <- system.time(for(i in 1:10)serialize(d64, NULL))
i64 <- as.integer64(i32); 
t64 <- system.time(for(i in 1:10)serialize(i64, NULL))
rm(i64); gc()
#I64 <- as.int64(i32); 
#T64 <- system.time(for(i in 1:10)serialize(I64, NULL))
#rm(I64); gc()
#T64/t64
t64/t32
t64/td64


message("-- integer64 adds 250x/60x faster than int64
 under 64/32 bit OS (and less than 6x the time of integer or double) --")
td64 <- system.time(for(i in 1:100)d64+d64)
t32 <- system.time(for(i in 1:100)i32+i32)
i64 <- as.integer64(i32); 
t64 <- system.time(for(i in 1:100)i64+i64)
rm(i64); gc()
#I64 <- as.int64(i32); 
#T64 <- system.time(for(i in 1:10)I64+I64)*10
#rm(I64); gc()
#T64/t64
t64/t32
t64/td64

message("-- integer64 sums 3x/0.2x faster than int64 
(and at about 5x/60X the time of integer and double) --")
td64 <- system.time(for(i in 1:100)sum(d64))
t32 <- system.time(for(i in 1:100)sum(i32))
i64 <- as.integer64(i32); 
t64 <- system.time(for(i in 1:100)sum(i64))
rm(i64); gc()
#I64 <- as.int64(i32); 
#T64 <- system.time(for(i in 1:100)sum(I64))
#rm(I64); gc()
#T64/t64
t64/t32
t64/td64

message("-- integer64 diffs 5x/0.85x faster than integer and double
(int64 version 1.0 does not support diff) --")
td64 <- system.time(for(i in 1:10)diff(d64, lag=2L, differences=2L))
t32 <- system.time(for(i in 1:10)diff(i32, lag=2L, differences=2L))
i64 <- as.integer64(i32); 
t64 <- system.time(for(i in 1:10)diff(i64, lag=2L, differences=2L))
rm(i64); gc()
t64/t32
t64/td64


message("-- integer64 subscripts 1000x/340x faster than int64
(and at the same speed / 10x slower as integer) --")
ts32 <- system.time(for(i in 1:1000)sample(1e6, 1e3))
t32<- system.time(for(i in 1:1000)i32[sample(1e6, 1e3)])
i64 <- as.integer64(i32); 
t64 <- system.time(for(i in 1:1000)i64[sample(1e6, 1e3)])
rm(i64); gc()
#I64 <- as.int64(i32); 
#T64 <- system.time(for(i in 1:100)I64[sample(1e6, 1e3)])*10
#rm(I64); gc()
#(T64-ts32)/(t64-ts32)
(t64-ts32)/(t32-ts32)

message("-- integer64 assigns 200x/90x faster than int64
(and 50x/160x slower than integer) --")
ts32 <- system.time(for(i in 1:100)sample(1e6, 1e3))
t32 <- system.time(for(i in 1:100)i32[sample(1e6, 1e3)] <- 1:1e3)
i64 <- as.integer64(i32); 
i64 <- system.time(for(i in 1:100)i64[sample(1e6, 1e3)] <- 1:1e3)
rm(i64); gc()
#I64 <- as.int64(i32); 
#I64 <- system.time(for(i in 1:10)I64[sample(1e6, 1e3)] <- 1:1e3)*10
#rm(I64); gc()
#(T64-ts32)/(t64-ts32)
(t64-ts32)/(t32-ts32)


tdfi32 <- system.time(dfi32 <- data.frame(a=i32, b=i32, c=i32))
tdfsi32 <- system.time(dfi32[1e6:1,])
fi32 <- tempfile()
tdfwi32 <- system.time(write.csv(dfi32, file=fi32, row.names=FALSE))
tdfri32 <- system.time(read.csv(fi32, colClasses=rep("integer", 3)))
unlink(fi32)
rm(dfi32); gc()

i64 <- as.integer64(i32); 
tdfi64 <- system.time(dfi64 <- data.frame(a=i64, b=i64, c=i64))
tdfsi64 <- system.time(dfi64[1e6:1,])
fi64 <- tempfile()
tdfwi64 <- system.time(write.csv(dfi64, file=fi64, row.names=FALSE))
tdfri64 <- system.time(read.csv(fi64, colClasses=rep("integer64", 3)))
unlink(fi64)
rm(i64, dfi64); gc()

#I64 <- as.int64(i32); 
#tdfI64 <- system.time(dfI64<-data.frame(a=I64, b=I64, c=I64))
#tdfsI64 <- system.time(dfI64[1e6:1,])
#fI64 <- tempfile()
#tdfwI64 <- system.time(write.csv(dfI64, file=fI64, row.names=FALSE))
#tdfrI64 <- system.time(read.csv(fI64, colClasses=rep("int64", 3)))
#unlink(fI64)
#rm(I64, dfI64); gc()

message("-- integer64 coerces 40x/6x faster to data.frame than int64
(and factor 1/9 slower than integer) --")
#tdfI64/tdfi64
tdfi64/tdfi32
message("-- integer64 subscripts from data.frame 20x/2.5x faster than int64
 (and 3x/13x slower than integer) --")
#tdfsI64/tdfsi64
tdfsi64/tdfsi32
message("-- integer64 csv writes about 2x/0.5x faster than int64
(and about 1.5x/5x slower than integer) --")
#tdfwI64/tdfwi64
tdfwi64/tdfwi32
message("-- integer64 csv reads about 3x/1.5 faster than int64
(and about 2x slower than integer) --")
#tdfrI64/tdfri64
tdfri64/tdfri32

rm(i32, d64); gc()


message("-- investigating the impact on garbage collection: --")
message("-- the fragmented structure of int64 messes up R's RAM --")
message("-- and slows down R's gargbage collection just by existing --")

td32 <- double(21)
td32[1] <- system.time(d64 <- double(1e7))[3]
for (i in 2:11)td32[i] <- system.time(gc(), gcFirst=FALSE)[3]
rm(d64)
for (i in 12:21)td32[i] <- system.time(gc(), gcFirst=FALSE)[3]

t64 <- double(21)
t64[1] <- system.time(i64 <- integer64(1e7))[3]
for (i in 2:11)t64[i] <- system.time(gc(), gcFirst=FALSE)[3]
rm(i64)
for (i in 12:21)t64[i] <- system.time(gc(), gcFirst=FALSE)[3]

#T64 <- double(21)
#T64[1] <- system.time(I64 <- int64(1e7))[3]
#for (i in 2:11)T64[i] <- system.time(gc(), gcFirst=FALSE)[3]
#rm(I64)
#for (i in 12:21)T64[i] <- system.time(gc(), gcFirst=FALSE)[3]

#matplot(1:21, cbind(td32, t64, T64), pch=c("d","i","I"), log="y")
matplot(1:21, cbind(td32, t64), pch=c("d","i"), log="y")
  }

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/integer64.R
\name{c.integer64}
\alias{c.integer64}
\alias{cbind.integer64}
\alias{rbind.integer64}
\title{Concatenating integer64 vectors}
\usage{
\method{c}{integer64}(..., recursive = FALSE)

\method{cbind}{integer64}(..., deparse.level = 1)

\method{rbind}{integer64}(..., deparse.level = 1)
}
\arguments{
\item{...}{two or more arguments coerced to 'integer64' and
passed to \code{\link[=NextMethod]{NextMethod()}}}

\item{recursive}{logical. If \code{recursive = TRUE}, the function
recursively descends through lists (and pairlists) combining all
their elements into a vector.}

\item{deparse.level}{integer controlling the construction of labels in the case of non-matrix-like arguments}
}
\value{
\code{\link[=c]{c()}} returns a vector of the appropriate mode. This could be a integer64 vector or a list of objects

\code{\link[=cbind]{cbind()}} and \code{\link[=rbind]{rbind()}} return a matrix, data.frame or list with dimensions
}
\description{
The ususal functions 'c', 'cbind' and 'rbind'
}
\note{
R currently only dispatches generic 'c' to method 'c.integer64' if the
first argument is 'integer64'
}
\examples{
  c(as.integer64(1), 2:6)
  cbind(1:6, as.integer64(1:6))
  rbind(1:6, as.integer64(1:6))
}
\seealso{
\code{\link[=rep.integer64]{rep.integer64()}} \code{\link[=seq.integer64]{seq.integer64()}} \code{\link[=as.data.frame.integer64]{as.data.frame.integer64()}}
\code{\link[=integer64]{integer64()}}
}
\keyword{classes}
\keyword{manip}

\name{c.integer64}
\alias{c.integer64}
\alias{cbind.integer64}
\alias{rbind.integer64}
\title{
   Concatenating integer64 vectors
}
\description{
  The ususal functions 'c', 'cbind' and 'rbind'
}
\usage{
\method{c}{integer64}(\dots, recursive = FALSE)
\method{cbind}{integer64}(\dots)
\method{rbind}{integer64}(\dots)
}
\arguments{
  \item{\dots}{ two or more arguments coerced to 'integer64' and passed to \code{\link{NextMethod}} }
  \item{recursive}{ logical. If \code{recursive = TRUE}, the function recursively descends through lists (and pairlists) combining all their elements into a vector. }
}
\value{
  \code{\link{c}} returns a integer64 vector of the total length of the input \cr
  \code{\link{cbind}} and \code{\link{rbind}} return a integer64 matrix
}
\note{
  R currently only dispatches generic 'c' to method 'c.integer64' if the first argument is 'integer64'
}
\author{
Jens Oehlschlägel <Jens.Oehlschlaegel@truecluster.com>
}
\keyword{ classes }
\keyword{ manip }
\seealso{ \code{\link{rep.integer64}} \code{\link{seq.integer64}} 
          \code{\link{as.data.frame.integer64}} \code{\link{integer64}}  
}
\examples{
  c(as.integer64(1), 2:6)
  cbind(1:6, as.integer(1:6))
  rbind(1:6, as.integer(1:6))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/highlevel64.R
\name{match.integer64}
\alias{match.integer64}
\alias{\%in\%.integer64}
\title{64-bit integer matching}
\usage{
\method{match}{integer64}(x, table, nomatch = NA_integer_, nunique = NULL, method = NULL, ...)

\method{\%in\%}{integer64}(x, table, ...)
}
\arguments{
\item{x}{integer64 vector: the values to be matched, optionally carrying a
cache created with \code{\link[=hashcache]{hashcache()}}}

\item{table}{integer64 vector: the values to be matched against, optionally
carrying a cache created with \code{\link[=hashcache]{hashcache()}} or \code{\link[=sortordercache]{sortordercache()}}}

\item{nomatch}{the value to be returned in the case when no match is found.
Note that it is coerced to integer.}

\item{nunique}{NULL or the number of unique values of table (including NA).
Providing \code{nunique} can speed-up matching when \code{table} has no cache. Note
that a wrong nunique can cause undefined behaviour up to a crash.}

\item{method}{NULL for automatic method selection or a suitable low-level
method, see details}

\item{...}{ignored}
}
\value{
A vector of the same length as \code{x}.

\code{match}: An integer vector giving the position in \code{table} of
the first match if there is a match, otherwise \code{nomatch}.

If \code{x[i]} is found to equal \code{table[j]} then the value
returned in the \code{i}-th position of the return value is \code{j},
for the smallest possible \code{j}.  If no match is found, the value
is \code{nomatch}.

\code{\%in\%}: A logical vector, indicating if a match was located for
each element of \code{x}: thus the values are \code{TRUE} or
\code{FALSE} and never \code{NA}.
}
\description{
\code{match} returns a vector of the positions of (first) matches of its first
argument in its second.
\code{\%in\%} is a more intuitive interface as a binary operator, which returns a
logical vector indicating if there is a match or not for its left operand.
}
\details{
These functions automatically choose from several low-level functions
considering the size of \code{x} and \code{table} and the availability of caches.

Suitable methods for \verb{\%in\%.integer64} are
\itemize{
\item \code{\link{hashpos}} (hash table lookup)
\item \code{\link{hashrev}} (reverse lookup)
\item \code{\link{sortorderpos}} (fast ordering)
\item \code{\link{orderpos}} (memory saving ordering).
}

Suitable methods for \code{match.integer64} are
\itemize{
\item \code{\link{hashfin}} (hash table lookup)
\item \code{\link{hashrin}} (reverse lookup)
\item \code{\link{sortfin}} (fast sorting)
\item \code{\link{orderfin}} (memory saving ordering).
}
}
\examples{
x <- as.integer64(c(NA, 0:9), 32)
table <- as.integer64(c(1:9, NA))
match(x, table)
x \%in\% table

x <- as.integer64(sample(c(rep(NA, 9), 0:9), 32, TRUE))
table <- as.integer64(sample(c(rep(NA, 9), 1:9), 32, TRUE))

\dontrun{
    library(bit)
    message("check when reverse hash-lookup beats standard hash-lookup")
    e <- 4:24
    timx <- timy <- matrix(NA, length(e), length(e), dimnames=list(e, e))
    for (iy in seq_along(e))
    for (ix in 1:iy) {
        nx <- 2^e[ix]
        ny <- 2^e[iy]
        x <- as.integer64(sample(ny, nx, FALSE))
        y <- as.integer64(sample(ny, ny, FALSE))
        #hashfun(x, bits=as.integer(5))
        timx[ix, iy] <- repeat.time({
        hx <- hashmap(x)
        py <- hashrev(hx, y)
        })[3]
        timy[ix, iy] <- repeat.time({
        hy <- hashmap(y)
        px <- hashpos(hy, x)
        })[3]
        #identical(px, py)
        print(round(timx[1:iy, 1:iy]/timy[1:iy, 1:iy], 2), na.print="")
    }

    message("explore best low-level method given size of x and table")
    B1 <- 1:27
    B2 <- 1:27
    tim <- array(NA, dim=c(length(B1), length(B2), 5)
 , dimnames=list(B1, B2, c("hashpos", "hashrev", "sortpos1", "sortpos2", "sortpos3")))
    for (i1 in B1)
    for (i2 in B2)
    {
      b1 <- B1[i1]
      b2 <- B1[i2]
      n1 <- 2^b1
      n2 <- 2^b2
      x1 <- as.integer64(c(sample(n2, n1-1, TRUE), NA))
      x2 <- as.integer64(c(sample(n2, n2-1, TRUE), NA))
      tim[i1, i2, 1] <- repeat.time({h <- hashmap(x2);hashpos(h, x1);rm(h)})[3]
      tim[i1, i2, 2] <- repeat.time({h <- hashmap(x1);hashrev(h, x2);rm(h)})[3]
      s <- clone(x2); o <- seq_along(s); ramsortorder(s, o)
      tim[i1, i2, 3] <- repeat.time(sortorderpos(s, o, x1, method=1))[3]
      tim[i1, i2, 4] <- repeat.time(sortorderpos(s, o, x1, method=2))[3]
      tim[i1, i2, 5] <- repeat.time(sortorderpos(s, o, x1, method=3))[3]
      rm(s, o)
      print(apply(tim, 1:2, function(ti)if(any(is.na(ti)))NA else which.min(ti)))
    }
}
}
\seealso{
\code{\link[=match]{match()}}
}
\keyword{logic}
\keyword{manip}

\name{match.integer64}
\alias{match.integer64}
\alias{\%in\%.integer64}
\title{
64-bit integer matching
}
\description{
\code{match} returns a vector of the positions of (first) matches of its first argument in its second. 

\code{\%in\%} is a more intuitive interface as a binary operator, which returns a logical vector indicating if there is a match or not for its left operand. 

}
\usage{
\method{match}{integer64}(x, table, nomatch = NA_integer_, nunique = NULL, method = NULL, ...)
\method{\%in\%}{integer64}(x, table, ...)
}
\arguments{
  \item{x}{
	integer64 vector: the values to be matched, optionally carrying a cache created with \code{\link{hashcache}}
}
  \item{table}{
	integer64 vector: the values to be matched against, optionally carrying a cache created with \code{\link{hashcache}} or \code{\link{sortordercache}}
}
  \item{nomatch}{
  the value to be returned in the case when no match is found. Note that it is coerced to integer.
}
  \item{nunique}{
	NULL or the number of unique values of table (including NA). Providing \code{nunique} can speed-up matching when \code{table} has no cache. Note that a wrong nunique can cause undefined behaviour up to a crash.
}
  \item{method}{
	NULL for automatic method selection or a suitable low-level method, see details
}
  \item{\dots}{
ignored
}
}
\details{
  These functions automatically choose from several low-level functions considering the size of \code{x} and \code{table} and the availability of caches. 


  Suitable methods for \code{\%in\%.integer64} are \code{\link{hashpos}} (hash table lookup), \code{\link{hashrev}} (reverse lookup), \code{\link{sortorderpos}} (fast ordering) and \code{\link{orderpos}} (memory saving ordering).
  Suitable methods for \code{match.integer64} are \code{\link{hashfin}} (hash table lookup), \code{\link{hashrin}} (reverse lookup), \code{\link{sortfin}} (fast sorting) and \code{\link{orderfin}} (memory saving ordering).
}
\value{
  A vector of the same length as \code{x}.

  \code{match}: An integer vector giving the position in \code{table} of
  the first match if there is a match, otherwise \code{nomatch}.

  If \code{x[i]} is found to equal \code{table[j]} then the value
  returned in the \code{i}-th position of the return value is \code{j},
  for the smallest possible \code{j}.  If no match is found, the value
  is \code{nomatch}.

  \code{\%in\%}: A logical vector, indicating if a match was located for
  each element of \code{x}: thus the values are \code{TRUE} or
  \code{FALSE} and never \code{NA}.
}
\author{
	Jens Oehlschlägel <Jens.Oehlschlaegel@truecluster.com>
}
\seealso{
	\code{\link{match}}
}
\examples{
x <- as.integer64(c(NA, 0:9), 32)
table <- as.integer64(c(1:9, NA))
match.integer64(x, table)
"\%in\%.integer64"(x, table)

x <- as.integer64(sample(c(rep(NA, 9), 0:9), 32, TRUE))
table <- as.integer64(sample(c(rep(NA, 9), 1:9), 32, TRUE))
stopifnot(identical(match.integer64(x, table), match(as.integer(x), as.integer(table))))
stopifnot(identical("\%in\%.integer64"(x, table), as.integer(x) \%in\% as.integer(table)))

\dontrun{
	message("check when reverse hash-lookup beats standard hash-lookup")
	e <- 4:24
	timx <- timy <- matrix(NA, length(e), length(e), dimnames=list(e,e))
	for (iy in seq_along(e))
	for (ix in 1:iy){
		nx <- 2^e[ix]
		ny <- 2^e[iy]
		x <- as.integer64(sample(ny, nx, FALSE))
		y <- as.integer64(sample(ny, ny, FALSE))
		#hashfun(x, bits=as.integer(5))
		timx[ix,iy] <- repeat.time({
		hx <- hashmap(x)
		py <- hashrev(hx, y)
		})[3]
		timy[ix,iy] <- repeat.time({
		hy <- hashmap(y)
		px <- hashpos(hy, x)
		})[3]
		#identical(px, py)
		print(round(timx[1:iy,1:iy]/timy[1:iy,1:iy], 2), na.print="")
	}

	message("explore best low-level method given size of x and table")
	B1 <- 1:27
	B2 <- 1:27
	tim <- array(NA, dim=c(length(B1), length(B2), 5)
 , dimnames=list(B1, B2, c("hashpos","hashrev","sortpos1","sortpos2","sortpos3")))
	for (i1 in B1)
	for (i2 in B2)
	{
	  b1 <- B1[i1]
	  b2 <- B1[i2]
	  n1 <- 2^b1
	  n2 <- 2^b2
	  x1 <- as.integer64(c(sample(n2, n1-1, TRUE), NA))
	  x2 <- as.integer64(c(sample(n2, n2-1, TRUE), NA))
	  tim[i1,i2,1] <- repeat.time({h <- hashmap(x2);hashpos(h, x1);rm(h)})[3]
	  tim[i1,i2,2] <- repeat.time({h <- hashmap(x1);hashrev(h, x2);rm(h)})[3]
	  s <- clone(x2); o <- seq_along(s); ramsortorder(s, o)
	  tim[i1,i2,3] <- repeat.time(sortorderpos(s, o, x1, method=1))[3]
	  tim[i1,i2,4] <- repeat.time(sortorderpos(s, o, x1, method=2))[3]
	  tim[i1,i2,5] <- repeat.time(sortorderpos(s, o, x1, method=3))[3]
	  rm(s,o)
	  print(apply(tim, 1:2, function(ti)if(any(is.na(ti)))NA else which.min(ti)))
	}
}
}
\keyword{manip}
\keyword{logic}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/highlevel64.R
\name{qtile}
\alias{qtile}
\alias{qtile.integer64}
\alias{quantile.integer64}
\alias{median.integer64}
\alias{mean.integer64}
\alias{summary.integer64}
\title{(Q)uan(Tile)s}
\usage{
qtile(x, probs = seq(0, 1, 0.25), ...)

\method{qtile}{integer64}(x, probs = seq(0, 1, 0.25), names = TRUE, method = NULL, ...)

\method{quantile}{integer64}(
  x,
  probs = seq(0, 1, 0.25),
  na.rm = FALSE,
  names = TRUE,
  type = 0L,
  ...
)

\method{median}{integer64}(x, na.rm = FALSE, ...)

\method{mean}{integer64}(x, na.rm = FALSE, ...)

\method{summary}{integer64}(object, ...)
}
\arguments{
\item{x}{a integer64 vector}

\item{probs}{numeric vector of probabilities with values in \verb{[0, 1]} - possibly containing \code{NA}s}

\item{...}{ignored}

\item{names}{logical; if \code{TRUE}, the result has a \code{names} attribute. Set to \code{FALSE} for speedup with many probs.}

\item{method}{NULL for automatic method selection or a suitable low-level method, see details}

\item{na.rm}{logical; if \code{TRUE}, any \code{NA} and \code{NaN}'s are removed from
\code{x} before the quantiles are computed.}

\item{type}{an integer selecting the quantile algorithm, currently only
0 is supported, see details}

\item{object}{a integer64 vector}
}
\value{
\code{prank} returns a numeric vector of the same length as \code{x}.

\code{qtile} returns a vector with elements from \code{x}
at the relative positions specified by \code{probs}.
}
\description{
Function \code{\link[=prank.integer64]{prank.integer64()}}  projects the values \verb{[min..max]} via ranks
\verb{[1..n]} to \verb{[0..1]}.
}
\details{
\code{qtile.integer64} is the inverse function of 'prank.integer64' and projects
\verb{[0..1]} to \verb{[min..max]}.

Functions \code{quantile.integer64} with \code{type=0} and \code{median.integer64} are
convenience wrappers to \code{qtile}.

Function \code{qtile} behaves very similar to \code{quantile.default} with \code{type=1}
in that it only returns existing values, it is mostly symmetric but it is
using 'round' rather than 'floor'.

Note that this implies that \code{median.integer64} does not interpolate for even
number of values (interpolation would create values that could not be
represented as 64-bit integers).

This function automatically chooses from several low-level functions
considering the size of \code{x} and the availability of a cache.

Suitable methods are
\itemize{
\item \code{\link{sortqtl}} (fast sorting)
\item \code{\link{orderqtl}} (memory saving ordering).
}
}
\examples{
x <- as.integer64(sample(c(rep(NA, 9), 1:9), 32, TRUE))
qtile(x, probs=seq(0, 1, 0.25))
quantile(x, probs=seq(0, 1, 0.25), na.rm=TRUE)
median(x, na.rm=TRUE)
summary(x)

x <- x[!is.na(x)]
stopifnot(identical(x,  unname(qtile(x, probs=prank(x)))))
}
\seealso{
\code{\link[=rank.integer64]{rank.integer64()}} for simple ranks and \code{\link[=quantile]{quantile()}} for quantiles.
}
\keyword{univar}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/integer64.R
\name{rep.integer64}
\alias{rep.integer64}
\title{Replicate elements of integer64 vectors}
\arguments{
\item{x}{a vector of 'integer64' to be replicated}

\item{...}{further arguments passed to \code{\link[=NextMethod]{NextMethod()}}}
}
\value{
\code{\link[=rep]{rep()}} returns a integer64 vector
}
\description{
Replicate elements of integer64 vectors
}
\examples{
  rep(as.integer64(1:2), 6)
  rep(as.integer64(1:2), c(6, 6))
  rep(as.integer64(1:2), length.out=6)
}
\seealso{
\code{\link[=c.integer64]{c.integer64()}} \code{\link[=rep.integer64]{rep.integer64()}}
\code{\link[=as.data.frame.integer64]{as.data.frame.integer64()}} \code{\link[=integer64]{integer64()}}
}
\keyword{classes}
\keyword{manip}

\name{qtile}
\alias{qtile}
\alias{qtile.integer64}
\alias{quantile.integer64}
\alias{median.integer64}
\alias{mean.integer64}
\alias{summary.integer64}
\title{(Q)uan(Tile)s }
\description{
	Function \code{\link{prank.integer64}}  projects the values [min..max] via ranks [1..n] to [0..1]. 
	\code{qtile.ineger64} is the inverse function of 'prank.integer64' and projects [0..1] to [min..max].
}
\usage{
	qtile(x, probs=seq(0, 1, 0.25), \dots)
	\method{qtile}{integer64}(x, probs = seq(0, 1, 0.25), names = TRUE, method = NULL, \dots)
	\method{quantile}{integer64}(x, probs = seq(0, 1, 0.25), na.rm = FALSE, names = TRUE, type=0L, \dots)
	\method{median}{integer64}(x, na.rm = FALSE, \dots)
 \method{mean}{integer64}(x, na.rm = FALSE, \dots)
	\method{summary}{integer64}(object, \dots)
 ## mean(x, na.rm = FALSE, ...)
 ## or
 ## mean(x, na.rm = FALSE)
}
\arguments{
  \item{x}{a integer64 vector}
  \item{object}{a integer64 vector}
  \item{probs}{
		numeric vector of probabilities with values in [0,1] - possibly containing \code{NA}s
}
  \item{names}{
	logical; if \code{TRUE}, the result has a \code{names} attribute. Set to \code{FALSE} for speedup with many probs.
}
  \item{type}{
	an integer selecting the quantile algorithm, currently only 0 is supported, see details
}
  \item{method}{
	NULL for automatic method selection or a suitable low-level method, see details
}
  \item{na.rm}{
	logical; if \code{TRUE}, any \code{NA} and \code{NaN}'s are removed from \code{x} before the quantiles are computed.
}
  \item{\dots}{ignored}
}
\details{
 Functions \code{quantile.integer64} with \code{type=0} and \code{median.integer64} are convenience wrappers to \code{qtile}.
 \cr
	Function \code{qtile} behaves very similar to \code{quantile.default} with \code{type=1} 
 in that it only returns existing values, it is mostly symetric 
 but it is using 'round' rather than 'floor'. 
 \cr
 Note that this implies that \code{median.integer64} does not interpolate for even number of values 
(interpolation would create values that could not be represented as 64-bit integers).
 \cr
  This function automatically chooses from several low-level functions considering the size of \code{x} and the availability of a cache. 
  Suitable methods are \code{\link{sortqtl}} (fast sorting) 
and \code{\link{orderqtl}} (memory saving ordering).
}
\value{
  \code{prank} returns a numeric vector of the same length as \code{x}.
  \cr
  \code{qtile} returns a vector with elements from \code{x} 
  at the relative positions specified by \code{probs}.
}
\author{
	Jens Oehlschlägel <Jens.Oehlschlaegel@truecluster.com>
}
\seealso{
  \code{\link{rank.integer64}} for simple ranks and \code{\link{quantile}} for quantiles.
}
\examples{
x <- as.integer64(sample(c(rep(NA, 9), 1:9), 32, TRUE))
qtile(x, probs=seq(0, 1, 0.25))
quantile(x, probs=seq(0, 1, 0.25), na.rm=TRUE)
median(x, na.rm=TRUE)
summary(x)

x <- x[!is.na(x)]
stopifnot(identical(x,  unname(qtile(x, probs=prank(x)))))
}
\keyword{univar}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/highlevel64.R
\name{tiepos}
\alias{tiepos}
\alias{tiepos.integer64}
\title{Extract Positions of Tied Elements}
\usage{
tiepos(x, ...)

\method{tiepos}{integer64}(x, nties = NULL, method = NULL, ...)
}
\arguments{
\item{x}{a vector or a data frame or an array or \code{NULL}.}

\item{...}{ignored}

\item{nties}{NULL or the number of tied values (including NA). Providing
\code{nties} can speed-up when \code{x} has no cache. Note that a wrong nties can
cause undefined behaviour up to a crash.}

\item{method}{NULL for automatic method selection or a suitable low-level
method, see details}
}
\value{
an integer vector of positions
}
\description{
\code{tiepos} returns the positions of those elements that participate in ties.
}
\details{
This function automatically chooses from several low-level functions
considering the size of \code{x} and the availability of a cache.

Suitable methods are
\itemize{
\item \code{\link{sortordertie}} (fast ordering)
\item \code{\link{ordertie}} (memory saving ordering).
}
}
\examples{
x <- as.integer64(sample(c(rep(NA, 9), 1:9), 32, TRUE))
tiepos(x)

stopifnot(identical(tiepos(x),  (1:length(x))[duplicated(x) | rev(duplicated(rev(x)))]))
}
\seealso{
\code{\link[=rank.integer64]{rank.integer64()}} for possibly tied ranks and \code{\link[=unipos.integer64]{unipos.integer64()}}
for positions of unique values.
}
\keyword{manip}
\keyword{univar}

\name{rep.integer64}
\alias{rep.integer64}
\title{
   Replicate elements of integer64 vectors
}
\description{
  Replicate elements of integer64 vectors
}
\usage{
\method{rep}{integer64}(x, \dots)
}
\arguments{
  \item{x}{ a vector of 'integer64' to be replicated }
  \item{\dots}{ further arguments passed to \code{\link{NextMethod}} }
}
\value{
  \code{\link{rep}} returns a integer64 vector
}
\author{
Jens Oehlschlägel <Jens.Oehlschlaegel@truecluster.com>
}
\keyword{ classes }
\keyword{ manip }
\seealso{ \code{\link{c.integer64}} \code{\link{rep.integer64}} 
          \code{\link{as.data.frame.integer64}} \code{\link{integer64}}  
}
\examples{
  rep(as.integer64(1:2), 6)
  rep(as.integer64(1:2), c(6,6))
  rep(as.integer64(1:2), length.out=6)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/patch64.R
\name{bit64S3}
\alias{bit64S3}
\alias{:}
\alias{:.default}
\alias{:.integer64}
\alias{is.double}
\alias{is.double.default}
\alias{is.double.integer64}
\alias{match}
\alias{match.default}
\alias{\%in\%}
\alias{\%in\%.default}
\alias{rank}
\alias{rank.default}
\alias{order}
\alias{order.default}
\alias{mtfrm.integer64}
\title{Turning base R functions into S3 generics for bit64}
\usage{
from:to
is.double(x)
match(x, table, ...)
x \%in\% table
rank(x, ...)
order(...)

\method{is.double}{default}(x)

\method{is.double}{integer64}(x)

\method{mtfrm}{integer64}(x)

\method{match}{default}(x, table, ...)

\method{\%in\%}{default}(x, table)

\method{rank}{default}(x, ...)

\method{order}{default}(...)
}
\arguments{
\item{x}{integer64 vector: the values to be matched, optionally carrying a
cache created with \code{\link[=hashcache]{hashcache()}}}

\item{table}{integer64 vector: the values to be matched against, optionally
carrying a cache created with \code{\link[=hashcache]{hashcache()}} or \code{\link[=sortordercache]{sortordercache()}}}

\item{...}{ignored}

\item{from}{scalar denoting first element of sequence}

\item{to}{scalar denoting last element of sequence}
}
\value{
\code{\link[=invisible]{invisible()}}
}
\description{
Turn those base functions S3 generic which are used in bit64
}
\details{
The following functions are turned into S3 generics in order to dispatch
methods for \code{\link[=integer64]{integer64()}}:
\itemize{
\item \code{\link{:}}
\item \code{\link[=is.double]{is.double()}}
\item \code{\link[=match]{match()}}
\item \code{\link[=match]{\%in\%}}
\item \code{\link[=rank]{rank()}}
\item \code{\link[=order]{order()}}
}
}
\note{
\itemize{
\item \code{\link[=is.double]{is.double()}} returns \code{FALSE} for \code{\link{integer64}}
\item \code{\link{:}} currently only dispatches at its first argument, thus
\code{as.integer64(1):9} works but \code{1:as.integer64(9)} doesn't
\item \code{\link[=match]{match()}} currently only dispatches at its first argument and expects
its second argument also to be integer64, otherwise throws an error.
Beware of something like \code{match(2, as.integer64(0:3))}
\item \code{\link[=match]{\%in\%}} currently only dispatches at its first argument and expects
its second argument also to be integer64, otherwise throws an error.
Beware of something like \code{2 \%in\% as.integer64(0:3)}
\item \code{\link[=order]{order()}} currently only orders a single argument, trying more than
one raises an error
}
}
\examples{
is.double(as.integer64(1))
as.integer64(1):9
match(as.integer64(2), as.integer64(0:3))
as.integer64(2) \%in\% as.integer64(0:3)

unique(as.integer64(c(1,1,2)))
rank(as.integer64(c(1,1,2)))


order(as.integer64(c(1,NA,2)))
}
\seealso{
\code{\link[=bit64]{bit64()}}, \link{S3}
}
\keyword{methods}

\name{tiepos}
\alias{tiepos}
\alias{tiepos.integer64}
\title{Extract Positions of Tied Elements}
\description{
  \code{tiepos} returns the positions of those elements that participate in ties.
}
\usage{
tiepos(x, \dots)
\method{tiepos}{integer64}(x, nties = NULL, method = NULL, \dots)
}
\arguments{
  \item{x}{a vector or a data frame or an array or \code{NULL}.}
  \item{nties}{
	NULL or the number of tied values (including NA). Providing \code{nties} can speed-up when \code{x} has no cache. Note that a wrong nties can cause undefined behaviour up to a crash.
}
  \item{method}{
	NULL for automatic method selection or a suitable low-level method, see details
}
  \item{\dots}{ignored}
}
\details{
  This function automatically chooses from several low-level functions considering the size of \code{x} and the availability of a cache. 
  Suitable methods are \code{\link{sortordertie}} (fast ordering) 
and \code{\link{ordertie}} (memory saving ordering).
}
\value{
  an integer vector of positions
}
\author{
	Jens Oehlschlägel <Jens.Oehlschlaegel@truecluster.com>
}
\seealso{
  \code{\link{rank.integer64}} for possibly tied ranks and \code{\link{unipos.integer64}} for positions of unique values.
}
\examples{
x <- as.integer64(sample(c(rep(NA, 9), 1:9), 32, TRUE))
tiepos(x)

stopifnot(identical(tiepos(x),  (1:length(x))[duplicated(x) | rev(duplicated(rev(x)))]))
}
\keyword{manip}
\keyword{univar}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/integer64.R
\name{seq.integer64}
\alias{seq.integer64}
\title{Generating sequence of integer64 values}
\usage{
\method{seq}{integer64}(
  from = NULL,
  to = NULL,
  by = NULL,
  length.out = NULL,
  along.with = NULL,
  ...
)
}
\arguments{
\item{from}{integer64 scalar (in order to dispatch the integer64 method of \code{\link[=seq]{seq()}})}

\item{to}{scalar}

\item{by}{scalar}

\item{length.out}{scalar}

\item{along.with}{scalar}

\item{...}{ignored}
}
\value{
An integer64 vector with the generated sequence
}
\description{
Generating sequence of integer64 values
}
\details{
\code{seq.integer64} coerces its arguments \code{from}, \code{to}, and \code{by} to \code{integer64}. Consistency
with \code{\link[=seq]{seq()}} is typically maintained, though results may differ when mixing \code{integer64} and
\code{double} inputs, for the same reason that any arithmetic with these mixed types can be
ambiguous. Whereas \code{seq(1L, 10L, length.out=8L)} can back up to double storage to give an
exact result, this not possible for generic inputs \code{seq(i64, dbl, length.out=n)}.
}
\examples{
seq(as.integer64(1), 12, 2)
seq(as.integer64(1), by=2, length.out=6)

# truncation rules
seq(as.integer64(1), 10, by=1.5)
seq(as.integer64(1), 10, length.out=5)
}
\seealso{
\code{\link[=c.integer64]{c.integer64()}} \code{\link[=rep.integer64]{rep.integer64()}}
\code{\link[=as.data.frame.integer64]{as.data.frame.integer64()}} \code{\link[=integer64]{integer64()}}
}
\keyword{classes}
\keyword{manip}

\name{bit64S3}
\alias{bit64S3}
\alias{:}
\alias{:.default}
\alias{:.integer64}
\alias{is.double}
\alias{is.double.default}
\alias{is.double.integer64}
\alias{match}
\alias{match.default}
\alias{\%in\%}
\alias{\%in\%.default}
\alias{rank}
\alias{rank.default}
%\alias{table}
%\alias{table.default}
\alias{order}
\alias{order.default}
\title{
  Turning base R functions into S3 generics for bit64 
}
\description{
	Turn those base functions S3 generic which are used in bit64
}
\usage{
	from:to
 #--as-cran complains about \method{:}{default}(from, to)
 #--as-cran complains about \method{:}{integer64}(from, to)
	is.double(x)
 \method{is.double}{default}(x)
 \method{is.double}{integer64}(x)
	match(x, table, ...)
 \method{match}{default}(x, table, ...)
	x \%in\% table
 \method{\%in\%}{default}(x, table)
	rank(x, ...)
 \method{rank}{default}(x, ...)
	%table(...)
 %\method{table}{default}(...)
	order(...)
 \method{order}{default}(...)
}
\arguments{
  \item{x}{
	integer64 vector: the values to be matched, optionally carrying a cache created with \code{\link{hashcache}}
}
  \item{table}{
	integer64 vector: the values to be matched against, optionally carrying a cache created with \code{\link{hashcache}} or \code{\link{sortordercache}}
}
  \item{from}{ scalar denoting first element of sequence }
  \item{to}{ scalar denoting last element of sequence }
  \item{\dots}{ ignored }
}
\details{
   The following functions are turned into S3 gernerics in order to dispatch methods for \code{\link{integer64}}: 
   \preformatted{
	   \code{\link{:}}
	   \code{\link{is.double}}
	   \code{\link{match}}
	   \code{\link{\%in\%}}
	   %\code{\link{table}}
	   \code{\link{rank}}
	   \code{\link{order}}
   }
}
\value{
	\code{\link{invisible}}
}
\author{
Jens Oehlschlägel <Jens.Oehlschlaegel@truecluster.com>
}
\note{
	\code{\link{is.double}} returns \code{FALSE} for \code{\link{integer64}} \cr
	\code{\link{:}} currently only dispatches at its first argument, thus \code{as.integer64(1):9} works but \code{1:as.integer64(9)} doesn't
	\code{\link{match}} currently only dispatches at its first argument and expects its second argument also to be integer64, otherwise throws an error. Beware of something like \code{match(2, as.integer64(0:3))}
	\code{\link{\%in\%}} currently only dispatches at its first argument and expects its second argument also to be integer64, otherwise throws an error. Beware of something like \code{2 \%in\% as.integer64(0:3)}
	\code{\link{order}} currently only orders a single argument, trying more than one raises an error
}
\seealso{
	\code{\link{bit64}}, \code{\link{S3}}
}
\examples{
 is.double(as.integer64(1))
	as.integer64(1):9
 match(as.integer64(2), as.integer64(0:3))
 as.integer64(2) \%in\% as.integer64(0:3)
 
 unique(as.integer64(c(1,1,2)))
 rank(as.integer64(c(1,1,2)))
 
 %table(as.integer64(c(1,1,2)))
 %table(as.integer64(c(1,1,2)),as.integer64(c(3,4,4)))
 %table(as.integer64(c(1,1,2)),c(3,4,4))
 %table(c(1,1,2),as.integer64(c(3,4,4)))
 
 order(as.integer64(c(1,NA,2)))
 
 \dontshow{
 stopifnot(identical(match(as.integer64(2), as.integer64(0:3)), match(2, 0:3)))
 stopifnot(identical(as.integer64(2) \%in\% as.integer64(0:3), 2 \%in\% 0:3))
 
 stopifnot(identical(unique(as.integer64(c(1,1,2))), as.integer64(unique(c(1,1,2)))))
 stopifnot(identical(rank(as.integer64(c(1,1,2))), rank(c(1,1,2))))
 
 %stopifnot(identical(table(as.integer64(c(1,1,2))), table(c(1,1,2))))
 %stopifnot(identical(table(as.integer64(c(1,1,2)),as.integer64(c(3,4,4))), table(c(1,1,2),c(3,4,4))))
 %stopifnot(identical(table(as.integer64(c(1,1,2)),c(3,4,4)), table(c(1,1,2),c(3,4,4))))
 %stopifnot(identical(table(c(1,1,2),as.integer64(c(3,4,4))), table(c(1,1,2),c(3,4,4))))
 
 stopifnot(identical(order(as.integer64(c(1,NA,2))), order(c(1,NA,2))))
 stopifnot(identical(order(as.integer64(c(1,NA,2)), decreasing=TRUE), order(c(1,NA,2), decreasing=TRUE)))
 }
}
\keyword{ methods }

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/integer64.R
\name{plusclass}
\alias{plusclass}
\alias{minusclass}
\title{integer64: Maintaining S3 class attribute}
\usage{
minusclass(class, whichclass)
}
\arguments{
\item{class}{NULL or a character vector of class attributes}

\item{whichclass}{the (single) class name to add or remove from the class vector}
}
\value{
NULL or a character vector of class attributes
}
\description{
Maintaining integer64 S3 class attribute.
}
\examples{
  plusclass("inheritingclass", "integer64")
  minusclass(c("inheritingclass", "integer64"), "integer64")
}
\seealso{
\code{\link[=oldClass]{oldClass()}} \code{\link[=integer64]{integer64()}}
}
\keyword{classes}
\keyword{internal}
\keyword{manip}

\name{seq.integer64}
\alias{seq.integer64}
\title{
   integer64: Sequence Generation
}
\description{
  Generating sequence of integer64 values
}
\usage{
\method{seq}{integer64}(from = NULL, to = NULL, by = NULL, length.out = NULL, along.with = NULL, \dots)
}
\arguments{
  \item{from}{ integer64 scalar (in order to dispatch the integer64 method of \code{\link{seq}} }
  \item{to}{ scalar }
  \item{by}{ scalar }
  \item{length.out}{ scalar }
  \item{along.with}{ scalar }
  \item{\dots}{ ignored }
}
\details{
  \code{seq.integer64} does coerce its arguments 'from', 'to' and 'by' to \code{integer64}.
  If not provided, the argument 'by' is automatically determined as \code{+1} or \code{-1},
  but the size of 'by' is not calculated as in \code{\link{seq}} (because this might result in a non-integer value).
}
\value{
  an integer64 vector with the generated sequence
}
\note{
  In base R \code{\link{:}} currently is not generic and does not dispatch, see section "Limitations inherited from Base R" in \code{\link{integer64}}
}
\author{
Jens Oehlschlägel <Jens.Oehlschlaegel@truecluster.com>
}
\keyword{ classes }
\keyword{ manip }
\seealso{ \code{\link{c.integer64}} \code{\link{rep.integer64}} 
          \code{\link{as.data.frame.integer64}} \code{\link{integer64}}  
}
\examples{
  # colon not activated: as.integer64(1):12
  seq(as.integer64(1), 12, 2)
  seq(as.integer64(1), by=2, length.out=6)
}

\name{plusclass}
\alias{plusclass}
\alias{minusclass}
\title{
   integer64: Maintaining S3 class attribute
}
\description{
  Maintaining integer64 S3 class attribute.
}
\usage{
  plusclass(class, whichclass)
  minusclass(class, whichclass)
}
\arguments{
  \item{class}{ NULL or a character vector of class attributes }
  \item{whichclass}{ the (single) class name to add or remove from the class vector  }
}
\value{
  NULL or a character vector of class attributes
}
\author{
Jens Oehlschlägel <Jens.Oehlschlaegel@truecluster.com>
}
\keyword{ classes }
\keyword{ manip }
\keyword{ internal }
\seealso{ 
  \code{\link{oldClass}} \code{\link{integer64}}  
}
\examples{
  plusclass("inheritingclass","integer64")
  minusclass(c("inheritingclass","integer64"), "integer64")
}
